@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: PrismDisplayProxyEditor.h,v $
+   Module:    PrismDisplayProxyEditor.h
 
 =========================================================================*/
 #ifndef _PrismDisplayProxyEditor_h
@@ -28,11 +28,10 @@ public:
   PrismDisplayProxyEditor(pqPipelineRepresentation* display, QWidget* p = NULL);
   /// destructor
   ~PrismDisplayProxyEditor();
+  protected slots:
+      virtual void editCubeAxes();
+      virtual void cubeAxesVisibilityChanged();
 
-
-protected slots:
-  void editCubeAxes();
-  void cubeAxesVisibilityChanged();
 protected:
   vtkSMPrismCubeAxesRepresentationProxy* CubeAxesActor;
   QPointer<pqPipelineRepresentation> Representation;
