@@ -1,7 +1,7 @@
 /*=========================================================================
 
 Program:   Visualization Toolkit
-Module:    $RCSfile: vtkPrismFilter.h,v $
+Module:    vtkPrismFilter.h
 
 =========================================================================*/
 // .NAME vtkPrismFilter - 
@@ -10,27 +10,26 @@ Module:    $RCSfile: vtkPrismFilter.h,v $
 #ifndef __vtkPrismFilter
 #define __vtkPrismFilter
 
-#include "vtkAlgorithm.h"
+
 
 #include "vtkCell.h" // Needed for VTK_CELL_SIZE
-
+#include "vtkStringArray.h"
+#include "vtkMultiBlockDataSetAlgorithm.h"
 class vtkIntArray;
 class vtkDoubleArray;
 
-class VTK_EXPORT vtkPrismFilter : public vtkAlgorithm 
+class VTK_EXPORT vtkPrismFilter : public vtkMultiBlockDataSetAlgorithm 
 {
 public:
-  vtkTypeRevisionMacro(vtkPrismFilter,vtkAlgorithm);
+  vtkTypeMacro(vtkPrismFilter,vtkMultiBlockDataSetAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   static vtkPrismFilter *New();
 
+  unsigned long GetMTime();
 
-  // Description:
-  // see vtkAlgorithm for details
-  virtual int ProcessRequest(vtkInformation*,
-    vtkInformationVector**,
-    vtkInformationVector*);
+ 
+  
   // Description:
   // Set the filename to read
   void SetFileName(const char* file);
@@ -95,6 +94,70 @@ public:
  vtkDoubleArray* GetRanges();
  
 
+
+  void SetSESAMEXAxisVarName( const char *name );
+  void SetSESAMEYAxisVarName( const char *name );
+  void SetSESAMEZAxisVarName( const char *name );
+  const char *GetSESAMEXAxisVarName();
+  const char *GetSESAMEYAxisVarName(); 
+  const char *GetSESAMEZAxisVarName(); 
+
+  void SetSESAMEXLogScaling(bool);
+  void SetSESAMEYLogScaling(bool);
+  void SetSESAMEZLogScaling(bool);
+  bool GetSESAMEXLogScaling();
+  bool GetSESAMEYLogScaling();
+  bool GetSESAMEZLogScaling();
+
+  void SetSESAMEConversions(double,double,double, double);
+  virtual double *GetSESAMEConversions();
+  virtual void GetSESAMEConversions (double &_arg1, double &_arg2,double &_arg3,double &_arg4);
+  virtual void GetSESAMEConversions (double _arg[4]);
+
+
+
+  vtkDoubleArray* GetSESAMEXRange();
+  vtkDoubleArray* GetSESAMEYRange();
+
+  void SetThresholdSESAMEXBetween(double lower, double upper);
+  void SetThresholdSESAMEYBetween(double lower, double upper);
+
+ virtual double *GetSESAMEXThresholdBetween();
+ virtual void GetSESAMEXThresholdBetween (double &_arg1, double &_arg2);
+ virtual void GetSESAMEXThresholdBetween (double _arg[2]);
+
+
+ virtual double *GetSESAMEYThresholdBetween();
+ virtual void GetSESAMEYThresholdBetween (double &_arg1, double &_arg2);
+ virtual void GetSESAMEYThresholdBetween (double _arg[2]);
+
+
+
+ 
+
+
+
+  void SetWarpSESAMESurface(bool);
+  void SetDisplaySESAMEContours(bool);
+  void SetSESAMEContourVarName( const char *name );
+  const char *GetSESAMEContourVarName(); 
+  vtkDoubleArray* GetSESAMEContourVarRange();
+
+
+  void SetSESAMEContourValue(int i, double value);
+  double GetSESAMEContourValue(int i);
+  double *GetSESAMEContourValues();
+  void GetSESAMEContourValues(double *contourValues);
+
+  void SetNumberOfSESAMEContours(int);
+
+
+  vtkStringArray* GetSESAMEAxisVarNames();
+
+
+
+
+
 protected:
   vtkPrismFilter();
   ~vtkPrismFilter() {}
@@ -112,22 +175,12 @@ protected:
     vtkInformationVector** inputVector,
     vtkInformationVector* outputVector);
 
-  // Description:
-  // This is called by the superclass.
-  // This is the method you should override.
-  virtual int RequestUpdateExtent(vtkInformation*,
-    vtkInformationVector**,
-    vtkInformationVector*);
 
-  int RequestInformation(vtkInformation *, vtkInformationVector **, vtkInformationVector *);
 
   // see algorithm for more info
   virtual int FillOutputPortInformation(int port, vtkInformation* info);
-  virtual int FillInputPortInformation(int port, vtkInformation* info);
-
-  // virtual void ExecuteData(vtkDataObject *output);
 private:
-  int CalculateValues( double *x, double *f );
+ // int CalculateValues( double *x, double *f );
   vtkPrismFilter(const vtkPrismFilter&);  // Not implemented.
   void operator=(const vtkPrismFilter&);  // Not implemented.
   int RequestSESAMEData(
@@ -136,7 +189,7 @@ private:
     vtkInformationVector *outputVector);
   int RequestGeometryData(
     vtkInformation *vtkNotUsed(request),
-    vtkInformationVector **vtkNotUsed(inputVector),
+    vtkInformationVector **inputVector,
     vtkInformationVector *outputVector);
 };
 
