@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pq3DWidget.cxx,v $
+   Module:    pq3DWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -32,6 +32,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pq3DWidget.h"
 
 // ParaView Server Manager includes.
+#include "vtkBoundingBox.h"
 #include "vtkEventQtSlotConnect.h"
 #include "vtkMemberFunctionCommand.h"
 #include "vtkPVDataInformation.h"
@@ -51,6 +52,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QShortcut>
 
 // ParaView GUI includes.
+#include "pq3DWidgetInterface.h"
 #include "pqApplicationCore.h"
 #include "pqBoxWidget.h"
 #include "pqDistanceWidget.h"
@@ -59,12 +61,65 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqPickHelper.h"
 #include "pqPipelineFilter.h"
 #include "pqPipelineSource.h"
+#include "pqPluginManager.h"
 #include "pqPointSourceWidget.h"
 #include "pqProxy.h"
-#include "pqRenderView.h"
+#include "pqRenderViewBase.h"
+#include "pqServerManagerSelectionModel.h"
 #include "pqSMAdaptor.h"
 #include "pqSphereWidget.h"
+#include "pqSplineWidget.h"
 
+//-----------------------------------------------------------------------------
+class pq3DWidget::pqStandardWidgets : public pq3DWidgetInterface
+{
+public:
+  pq3DWidget* newWidget(const QString& name,
+    vtkSMProxy* referenceProxy,
+    vtkSMProxy* controlledProxy)
+    {
+    pq3DWidget *widget = 0;
+    if (name == "Plane")
+      {
+      widget = new pqImplicitPlaneWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Box")
+      {
+      widget = new pqBoxWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Handle")
+      {
+      widget = new pqHandleWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "PointSource")
+      {
+      widget = new pqPointSourceWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "LineSource")
+      {
+      widget = new pqLineSourceWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Line")
+      {
+      widget = new pqLineWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Distance")
+      {
+      widget = new pqDistanceWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Sphere")
+      {
+      widget = new pqSphereWidget(referenceProxy, controlledProxy, 0);
+      }
+    else if (name == "Spline")
+      {
+      widget = new pqSplineWidget(referenceProxy, controlledProxy, 0);
+      }
+    return widget;
+    }
+};
+
+//-----------------------------------------------------------------------------
 class pq3DWidgetInternal
 {
 public:
@@ -101,6 +156,7 @@ pq3DWidget::pq3DWidget(vtkSMProxy* refProxy, vtkSMProxy* pxy, QWidget* _p) :
   pqProxyPanel(pxy, _p),
   Internal(new pq3DWidgetInternal())
 {
+  this->UseSelectionDataBounds = false;
   this->Internal->ReferenceProxy = refProxy;
 
   this->Internal->ControlledPropertiesObserver.TakeReference(
@@ -128,6 +184,9 @@ QList<pq3DWidget*> pq3DWidget::createWidgets(vtkSMProxy* refProxy, vtkSMProxy* p
 {
   QList<pq3DWidget*> widgets;
 
+  QList<pq3DWidgetInterface*> interfaces =
+    pqApplicationCore::instance()->getPluginManager()->findInterfaces<pq3DWidgetInterface*>();
+
   vtkPVXMLElement* hints = pxy->GetHints();
   unsigned int max = hints->GetNumberOfNestedElements();
   for (unsigned int cc=0; cc < max; cc++)
@@ -137,39 +196,22 @@ QList<pq3DWidget*> pq3DWidget::createWidgets(vtkSMProxy* refProxy, vtkSMProxy* p
       {
       QString widgetType = element->GetAttribute("type");
       pq3DWidget *widget = 0;
-      if (widgetType == "Plane")
-        {
-        widget = new pqImplicitPlaneWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "Box")
-        {
-        widget = new pqBoxWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "Handle")
-        {
-        widget = new pqHandleWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "PointSource")
-        {
-        widget = new pqPointSourceWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "LineSource")
-        {
-        widget = new pqLineSourceWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "Line")
-        {
-        widget = new pqLineWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "Distance")
-        {
-        widget = new pqDistanceWidget(refProxy, pxy, 0);
-        }
-      else if (widgetType == "Sphere")
-        {
-        widget = new pqSphereWidget(refProxy, pxy, 0);
-        }
 
+      // Create the widget from plugins.
+      foreach (pq3DWidgetInterface* iface, interfaces)
+        {
+        widget = iface->newWidget(widgetType, refProxy, pxy);
+        if (widget)
+          {
+          break;
+          }
+        }
+      if (!widget)
+        {
+        // try to create the standard widget if the plugins fail.
+        pqStandardWidgets standardWidgets;
+        widget = standardWidgets.newWidget(widgetType, refProxy, pxy);
+        }
       if (widget)
         {
         widget->setHints(element);
@@ -181,9 +223,9 @@ QList<pq3DWidget*> pq3DWidget::createWidgets(vtkSMProxy* refProxy, vtkSMProxy* p
 }
 
 //-----------------------------------------------------------------------------
-pqRenderView* pq3DWidget::renderView() const
+pqRenderViewBase* pq3DWidget::renderView() const
 {
-  return qobject_cast<pqRenderView*>(this->view());
+  return qobject_cast<pqRenderViewBase*>(this->view());
 }
 
 //-----------------------------------------------------------------------------
@@ -195,7 +237,7 @@ void pq3DWidget::pickingSupported(const QKeySequence& key)
 //-----------------------------------------------------------------------------
 void pq3DWidget::setView(pqView* pqview)
 { 
-  pqRenderView* rview = this->renderView();
+  pqRenderViewBase* rview = this->renderView();
   if (pqview == rview)
     {
     this->Superclass::setView(pqview);
@@ -214,7 +256,7 @@ void pq3DWidget::setView(pqView* pqview)
     // To add/remove the 3D widget display from the view module.
     // we don't use the property. This is so since the 3D widget add/remove 
     // should not get saved in state or undo-redo. 
-    rview->getRenderViewProxy()->RemoveRepresentation(widget);
+    rview->getViewProxy()->RemoveRepresentation(widget);
     }
 
   this->Superclass::setView(pqview);
@@ -235,7 +277,7 @@ void pq3DWidget::setView(pqView* pqview)
     // we don't use the property. This is so since the 3D widget add/remove 
     // should not get saved in state or undo-redo. 
     this->updateWidgetVisibility();
-    rview->getRenderViewProxy()->AddRepresentation(widget);
+    rview->getViewProxy()->AddRepresentation(widget);
     }
 
   if (cur_visbility)
@@ -248,7 +290,7 @@ void pq3DWidget::setView(pqView* pqview)
 //-----------------------------------------------------------------------------
 void pq3DWidget::render()
 {
-  if (pqRenderView* rview = this->renderView())
+  if (pqRenderViewBase* rview = this->renderView())
     {
     rview->render();
     }
@@ -272,13 +314,13 @@ void pq3DWidget::setWidgetProxy(vtkSMNewWidgetRepresentationProxy* pxy)
   this->Internal->VTKConnect->Disconnect();
 
     vtkSMNewWidgetRepresentationProxy* widget = this->getWidgetProxy();
-  pqRenderView* rview = this->renderView();
+  pqRenderViewBase* rview = this->renderView();
   if (rview && widget)
     {
     // To add/remove the 3D widget display from the view module.
     // we don't use the property. This is so since the 3D widget add/remove 
     // should not get saved in state or undo-redo. 
-    rview->getRenderViewProxy()->RemoveRepresentation(widget);
+    rview->getViewProxy()->RemoveRepresentation(widget);
     rview->render();
     }
 
@@ -302,7 +344,7 @@ void pq3DWidget::setWidgetProxy(vtkSMNewWidgetRepresentationProxy* pxy)
     // we don't use the property. This is so since the 3D widget add/remove 
     // should not get saved in state or undo-redo. 
     this->updateWidgetVisibility();
-    rview->getRenderViewProxy()->AddRepresentation(widget);
+    rview->getViewProxy()->AddRepresentation(widget);
     rview->render();
     }
 }
@@ -451,6 +493,12 @@ bool pq3DWidget::widgetVisible() const
   return this->Internal->WidgetVisible;
 }
 
+//-----------------------------------------------------------------------------
+bool pq3DWidget::widgetSelected() const
+{
+  return this->Internal->Selected;
+}
+
 //-----------------------------------------------------------------------------
 void pq3DWidget::select()
 {
@@ -498,24 +546,35 @@ void pq3DWidget::hideWidget()
 //-----------------------------------------------------------------------------
 int pq3DWidget::getReferenceInputBounds(double bounds[6]) const
 {
-  if (!this->getReferenceProxy())
+  vtkSMProxy* refProxy = this->getReferenceProxy();
+  if (!refProxy)
     {
     return 0;
     }
   
   vtkSMSourceProxy* input = NULL;
   vtkSMInputProperty* ivp = vtkSMInputProperty::SafeDownCast(
-    this->getReferenceProxy()->GetProperty("Input"));
-  if(ivp && ivp->GetNumberOfProxies())
+    refProxy->GetProperty("Input"));
+  int output_port = 0;
+  if (ivp && ivp->GetNumberOfProxies())
     {
     vtkSMProxy* pxy = ivp->GetProxy(0);
     input = vtkSMSourceProxy::SafeDownCast(pxy);
+    output_port =ivp->GetOutputPortForConnection(0);
+    }
+  else
+    {
+    // reference proxy has no input. This generally happens when the widget is
+    // controlling properties of a source. In that case, if the source has been
+    // "created", simply use the source's bounds.
+    input = vtkSMSourceProxy::SafeDownCast(refProxy);
     }
 
   if(input)
     {
-    input->GetDataInformation()->GetBounds(bounds);
-    return 1;
+    input->GetDataInformation(output_port)->GetBounds(bounds);
+    return (bounds[1] >= bounds[0] && bounds[3] >= bounds[2] && bounds[5] >=
+      bounds[4]) ? 1 : 0;
     }
   return 0;
 }
@@ -528,6 +587,12 @@ void pq3DWidget::updateWidgetVisibility()
     
   const bool widget_enabled = widget_visible;
   
+  this->updateWidgetState(widget_visible, widget_enabled);
+}
+
+//-----------------------------------------------------------------------------
+void pq3DWidget::updateWidgetState(bool widget_visible, bool widget_enabled)
+{
   if (this->Internal->WidgetProxy && this->renderView())
     {
     if(vtkSMIntVectorProperty* const visibility =
@@ -559,9 +624,41 @@ void pq3DWidget::updatePickShortcut()
     && this->Internal->WidgetVisible &&
     this->Internal->WidgetProxy && this->renderView());
 
+  this->updatePickShortcut(pickable);
+}
+
+//-----------------------------------------------------------------------------
+void pq3DWidget::updatePickShortcut(bool pickable)
+{
   if (this->Internal->PickShortcut)
     {
     this->Internal->PickShortcut->setEnabled(pickable);
     }
 }
 
+//-----------------------------------------------------------------------------
+void pq3DWidget::resetBounds()
+{
+  vtkSMNewWidgetRepresentationProxy* widget = this->getWidgetProxy();
+  if (!widget)
+    {
+    return;
+    }
+
+  double input_bounds[6];
+  if (this->UseSelectionDataBounds)
+    {
+    if (!pqApplicationCore::instance()->getSelectionModel()->
+      getSelectionDataBounds(input_bounds))
+      {
+      return;
+      }
+    }
+  else if (!this->getReferenceInputBounds(input_bounds))
+    {
+    return;
+    }
+  this->resetBounds(input_bounds);
+  this->setModified();
+  this->render();
+}
