@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqActiveChartOptions.h,v $
+   Module:    pqActiveChartOptions.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -40,8 +40,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqComponentsExport.h"
 #include "pqActiveViewOptions.h"
 
-class pqActiveChartOptionsInternal;
+class pqBarChartOptionsHandler;
+class pqBoxChartOptionsHandler;
+class pqChartOptionsHandler;
+class pqLineChartOptionsHandler;
 class pqOptionsDialog;
+class pqStackedChartOptionsHandler;
 
 
 /// \class pqActiveChartOptions
@@ -83,6 +87,9 @@ private slots:
   /// Cleans up the options dialog data in case it is deleted.
   void cleanupDialog();
 
+  void openUndoSet();
+  void closeUndoSet();
+
   void setTitleModified();
   void setTitleFontModified();
   void setTitleColorModified();
@@ -110,9 +117,31 @@ private slots:
   void setAxisTitleColorModified();
   void setAxisTitleAlignmentModified();
 
+  void setBarHelpFormatModified();
+  void setBarOutlineStyleModified();
+  void setBarGroupFractionModified();
+  void setBarWidthFractionModified();
+
+  void setLineHelpFormatModified();
+
+  void setStackedHelpFormatModified();
+  void setStackedNormalizationModified();
+  void setStackedGradientModified();
+
+  void setBoxHelpFormatModified();
+  void setBoxOutlierFormatModified();
+  void setBoxOutlineStyleModified();
+  void setBoxWidthFractionModified();
+
 private:
-  pqActiveChartOptionsInternal *Internal; ///< Handles the modified data.
-  pqOptionsDialog *Dialog;                ///< Stores the dialog.
+  pqOptionsDialog *Dialog;              ///< Stores the dialog.
+  pqChartOptionsHandler *Chart;         ///< Handles the base chart options.
+  pqBarChartOptionsHandler *BarChart;   ///< Handles the bar chart options.
+  pqLineChartOptionsHandler *LineChart; ///< Handles the line chart options.
+
+  /// Handles the stacked chart options.
+  pqStackedChartOptionsHandler *StackedChart;
+  pqBoxChartOptionsHandler *BoxChart;   ///< Handles the box chart options.
 };
 
 #endif
