@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationViewWidget.h,v $
+   Module:    pqAnimationViewWidget.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -42,7 +42,7 @@ class pqView;
 class pqServerManagerModelItem;
 class vtkSMProxy;
 
-/// This is the Animation panel widget. It controls the behaviour
+/// This is the Animation panel widget. It controls the behavior
 /// of the Animation panel which includes adding of key frames,
 /// changing of keyframes etc etc.
 class PQCOMPONENTS_EXPORT pqAnimationViewWidget : public QWidget
@@ -50,7 +50,7 @@ class PQCOMPONENTS_EXPORT pqAnimationViewWidget : public QWidget
   Q_OBJECT
   typedef QWidget Superclass;
 public:
-  pqAnimationViewWidget(QWidget* parent);
+  pqAnimationViewWidget(QWidget* parent=0);
   virtual ~pqAnimationViewWidget();
 
 public slots:
@@ -81,6 +81,9 @@ protected slots:
   // called when play mode changes
   void updatePlayMode();
 
+  /// Called to toggle a track's enabled state.
+  void toggleTrackEnabled(pqAnimationTrack* track);
+
   // called when deleting a track
   void deleteTrack(pqAnimationTrack* track);
   // called when creating a track
