@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqApplicationOptions.h,v $
+   Module:    pqApplicationOptions.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -58,6 +58,17 @@ public:
   // tell pqOptionsDialog that we want an apply button
   virtual bool isApplyUsed() const { return true; }
 
+public slots:
+  /// load a palette with the given name.
+  void loadPalette(const QString& name);
+
+protected slots:
+  void resetColorsToDefault();
+  void onPalette(QAction*);
+  void onChartNewHiddenSeries();
+  void onChartDeleteHiddenSeries();
+  void onChartResetHiddenSeries();
+
 private:
   class pqInternal;
   pqInternal* Internal;
