@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqCloseViewUndoElement.cxx,v $
+   Module:    pqCloseViewUndoElement.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -38,7 +38,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqViewManager.h"
 
 vtkStandardNewMacro(pqCloseViewUndoElement);
-vtkCxxRevisionMacro(pqCloseViewUndoElement, "$Revision: 1.2 $");
 //----------------------------------------------------------------------------
 pqCloseViewUndoElement::pqCloseViewUndoElement()
 {
@@ -80,7 +79,7 @@ int pqCloseViewUndoElement::Undo()
       << "MULTIVIEW_MANAGER must be registered with application core.");
     return 0;
     }
-  manager->loadState(state, this->StateLoader);
+  manager->loadState(state, this->GetProxyLocator());
   return 1;
 }
 
