@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqColorScaleEditor.cxx,v $
+   Module:    pqColorScaleEditor.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -37,16 +37,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "ui_pqColorScaleDialog.h"
 
 #include "pqApplicationCore.h"
-#include "pqBarChartRepresentation.h"
 #include "pqChartValue.h"
-#include "pqColorMapColorChanger.h"
 #include "pqColorMapModel.h"
-#include "pqColorMapWidget.h"
 #include "pqColorPresetManager.h"
 #include "pqColorPresetModel.h"
+#include "pqCoreUtilities.h"
 #include "pqDataRepresentation.h"
 #include "pqLookupTableManager.h"
-#include "pqObjectBuilder.h"
+#include "pqOutputPort.h"
 #include "pqPipelineRepresentation.h"
 #include "pqPropertyLinks.h"
 #include "pqRenderViewBase.h"
@@ -54,48 +52,54 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqScalarBarRepresentation.h"
 #include "pqScalarOpacityFunction.h"
 #include "pqScalarsToColors.h"
+#include "pqScatterPlotRepresentation.h"
 #include "pqSignalAdaptors.h"
 #include "pqSMAdaptor.h"
+#include "pqStandardColorLinkAdaptor.h"
+#include "vtkColorTransferFunction.h"
+#include "vtkEventQtSlotConnect.h"
+#include "vtkPiecewiseFunction.h"
+#include "vtkPVTemporalDataInformation.h"
+#include "vtkSMProperty.h"
+#include "vtkSMProxy.h"
+#include "vtkSMPVRepresentationProxy.h"
+#include "vtkTransferFunctionViewer.h"
+#include "vtkType.h"
 
 #include <QCloseEvent>
 #include <QColor>
 #include <QColorDialog>
+#include <QDoubleValidator>
 #include <QGridLayout>
+#include <QIntValidator>
 #include <QItemSelectionModel>
 #include <QList>
 #include <QMenu>
+#include <QMessageBox>
 #include <QPointer>
 #include <QSpacerItem>
 #include <QString>
 #include <QtDebug>
 #include <QVariant>
-#include <QIntValidator>
-#include <QDoubleValidator>
 
-#include "vtkColorTransferFunction.h"
-#include "vtkEventQtSlotConnect.h"
-#include "vtkPiecewiseFunction.h"
-#include "vtkSMPVRepresentationProxy.h"
-#include "vtkSMProperty.h"
-#include "vtkSMProxy.h"
-#include "vtkTransferFunctionViewer.h"
-#include "vtkType.h"
+
 
 
 class pqColorScaleEditorForm : public Ui::pqColorScaleDialog
 {
 public:
   pqColorScaleEditorForm();
-  ~pqColorScaleEditorForm() {}
+  ~pqColorScaleEditorForm() { }
 
   pqPropertyLinks Links; // used to link properties on the legend
   pqPropertyLinks ReprLinks; // used to link properties on the representation.
   pqSignalAdaptorColor *TitleColorAdaptor;
   pqSignalAdaptorColor *LabelColorAdaptor;
+  pqStandardColorLinkAdaptor* TitleColorLink;
+  pqStandardColorLinkAdaptor* LabelColorLink;
   pqSignalAdaptorComboBox *TitleFontAdaptor;
   pqSignalAdaptorComboBox *LabelFontAdaptor;
   vtkEventQtSlotConnect *Listener;
-  pqColorMapWidget *Gradient;
   pqColorPresetManager *Presets;
   QPointer<pqDataRepresentation> CurrentDisplay;
   int CurrentIndex;
@@ -112,10 +116,11 @@ pqColorScaleEditorForm::pqColorScaleEditorForm()
 {
   this->TitleColorAdaptor = 0;
   this->LabelColorAdaptor = 0;
+  this->TitleColorLink = 0;
+  this->LabelColorLink = 0;
   this->TitleFontAdaptor = 0;
   this->LabelFontAdaptor = 0;
   this->Listener = 0;
-  this->Gradient = 0;
   this->Presets = 0;
   this->CurrentIndex = -1;
   this->InSetColors = false;
@@ -124,7 +129,6 @@ pqColorScaleEditorForm::pqColorScaleEditorForm()
   this->MakingLegend = false;
 }
 
-
 //----------------------------------------------------------------------------
 pqColorScaleEditor::pqColorScaleEditor(QWidget *widgetParent)
   : QDialog(widgetParent)
@@ -141,7 +145,8 @@ pqColorScaleEditor::pqColorScaleEditor(QWidget *widgetParent)
   this->Form->Listener = vtkEventQtSlotConnect::New();
   this->Form->Presets = new pqColorPresetManager(this);
   this->Form->Presets->restoreSettings();
-
+  this->Form->ColorScale->setToolTip("Note: Use Tab or Shift+Tab to navigate among points.");
+  
   vtkRenderWindow *win = this->Form->ColorScale->GetRenderWindow();
   vtkRenderWindowInteractor *iren = this->Form->ColorScale->GetInteractor();
   this->Viewer->SetInteractor(iren);
@@ -185,12 +190,11 @@ pqColorScaleEditor::pqColorScaleEditor(QWidget *widgetParent)
   // Add the color scale presets menu.
   this->loadBuiltinColorPresets();
 
-  // TODO would be nice to give them their own validators with their own ranges
   // Make sure the line edits only allow number inputs.
-  QDoubleValidator *doubleValidator = new QDoubleValidator(this);
-  this->Form->ScalarValue->setValidator(doubleValidator);
-  this->Form->Opacity->setValidator(doubleValidator);
-  this->Form->ScalarOpacityUnitDistance->setValidator(doubleValidator);
+  this->Form->ScalarValue->setValidator(new QDoubleValidator(this));
+  this->Form->Opacity->setValidator(new QDoubleValidator(this));
+  this->Form->ScalarOpacityUnitDistance->setValidator(
+    new QDoubleValidator(this));
 
   QIntValidator *intValidator = new QIntValidator(this);
   this->Form->TableSizeText->setValidator(intValidator);
@@ -218,6 +222,8 @@ pqColorScaleEditor::pqColorScaleEditor(QWidget *widgetParent)
       this, SLOT(rescaleToNewRange()));
   this->connect(this->Form->RescaleToDataButton, SIGNAL(clicked()),
       this, SLOT(rescaleToDataRange()));
+  this->connect(this->Form->RescaleToDataOverTimeButton, SIGNAL(clicked()),
+      this, SLOT(rescaleToDataRangeOverTime()));
 
   this->connect(this->Form->UseDiscreteColors, SIGNAL(toggled(bool)),
       this, SLOT(setUseDiscreteColors(bool)));
@@ -530,15 +536,19 @@ void pqColorScaleEditor::setValueFromText()
     return;
     }
 
+
+
   // Make sure the value is greater than the previous point and less
   // than the next point.
   bool endpoint = this->Form->CurrentIndex == 0;
   if(this->Form->CurrentIndex > 0)
     {
     double prev = this->Viewer->GetElementScalar(this->Form->CurrentIndex - 1);
-    if(value < prev)
+    if(value <= prev)
       {
-      value = prev;
+      // value not acceptable.
+      this->updatePointValues();
+      return;
       }
     }
 
@@ -547,12 +557,16 @@ void pqColorScaleEditor::setValueFromText()
   if(this->Form->CurrentIndex < colors->GetSize() - 1)
     {
     double next = this->Viewer->GetElementScalar(this->Form->CurrentIndex + 1);
-    if(value > next)
+    if(value >= next)
       {
-      value = next;
+      // value not acceptable.
+      this->updatePointValues();
+      return;
       }
     }
 
+  
+
   // Set the new value on the point in the editor.
   this->Form->IgnoreEditor = true;
   this->Viewer->SetElementScalar(this->Form->CurrentIndex, value);
@@ -775,30 +789,13 @@ void pqColorScaleEditor::setLogScale(bool on)
   pqSMAdaptor::setElementProperty(
       lookupTable->GetProperty("UseLogScale"), on ? 1 : 0);
 
-  // If the range is invalid, modify the range for log scale.
-  QPair<double, double> range = this->ColorMap->getScalarRange();
-  if(on && range.first <= 0.0)
-    {
-    // Use a minimum value to make the range valid.
-    range.first = 1.0;
-    if(range.second < range.first)
-      {
-      range.second = 10.0;
-      }
+  // Set the log scale flag on the editor.
+  this->Viewer->GetColorFunction()->SetScale(
+                                           on ? VTK_CTF_LOG10 : VTK_CTF_LINEAR);
+  this->Viewer->Render();
 
-    this->Form->UseAutoRescale->setChecked(false);
-    this->setScalarRange(range.first, range.second);
-    }
-  else
-    {
-    // Set the log scale flag on the editor.
-    this->Viewer->GetColorFunction()->SetScale(
-        on ? VTK_CTF_LOG10 : VTK_CTF_LINEAR);
-    this->Viewer->Render();
-
-    lookupTable->UpdateVTKObjects();
-    this->Display->renderViewEventually();
-    }
+  lookupTable->UpdateVTKObjects();
+  this->Display->renderViewEventually();
 }
 
 void pqColorScaleEditor::setAutoRescale(bool on)
@@ -826,24 +823,55 @@ void pqColorScaleEditor::rescaleToNewRange()
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqColorScaleEditor::rescaleToDataRangeOverTime()
+{
+  if (QMessageBox::warning(
+      pqCoreUtilities::mainWidget(),
+      "Potentially slow operation",
+      "This can potentially take a long time to complete. \n"
+      "Are you sure you want to continue?",
+      QMessageBox::Yes |QMessageBox::No, QMessageBox::No) ==
+    QMessageBox::Yes)
+    {
+    pqPipelineRepresentation *pipeline =
+      qobject_cast<pqPipelineRepresentation *>(this->Display);
+    if(pipeline)
+      {
+      pipeline->resetLookupTableScalarRangeOverTime();
+      pipeline->renderViewEventually();
+      }
+    }
+  // TODO: Handle all the other representation types!
+}
+
+//-----------------------------------------------------------------------------
 void pqColorScaleEditor::rescaleToDataRange()
 {
   pqPipelineRepresentation *pipeline =
       qobject_cast<pqPipelineRepresentation *>(this->Display);
-  pqBarChartRepresentation *histogram =
-      qobject_cast<pqBarChartRepresentation *>(this->Display);
+  //pqBarChartRepresentation *histogram =
+  //    qobject_cast<pqBarChartRepresentation *>(this->Display);
+  pqScatterPlotRepresentation* scatterPlot =
+    qobject_cast<pqScatterPlotRepresentation *>(this->Display);
   if(pipeline)
     {
     pipeline->resetLookupTableScalarRange();
     pipeline->renderViewEventually();
     }
-  else if(histogram)
+  //else if(histogram)
+  //  {
+  //  histogram->resetLookupTableScalarRange();
+  //  histogram->renderViewEventually();
+  //  }
+  else if(scatterPlot)
     {
-    histogram->resetLookupTableScalarRange();
-    histogram->renderViewEventually();
+    scatterPlot->resetLookupTableScalarRange();
+    scatterPlot->renderViewEventually();
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqColorScaleEditor::setUseDiscreteColors(bool on)
 {
   // Update the color scale widget and gui controls.
@@ -930,14 +958,10 @@ void pqColorScaleEditor::setLegendVisibility(bool visible)
       // Create a scalar bar in the current view. Use the display to
       // set up the title.
       this->Form->MakingLegend = true;
-      pqObjectBuilder *builder =
-          pqApplicationCore::instance()->getObjectBuilder();
-      pqRenderViewBase *renderModule = qobject_cast<pqRenderViewBase *>(
-          this->Display->getView());
-      pqScalarBarRepresentation *legend = builder->createScalarBarDisplay(
-          this->ColorMap, renderModule);
-      legend->makeTitle(qobject_cast<pqPipelineRepresentation *>(
-          this->Display));
+      pqLookupTableManager* lutManager =
+        pqApplicationCore::instance()->getLookupTableManager();
+      pqScalarBarRepresentation* legend = lutManager->setScalarBarVisibility(
+        this->Display->getView(), this->ColorMap, visible);
       this->setLegend(legend);
       this->Form->MakingLegend = false;
       }
@@ -1019,8 +1043,8 @@ void pqColorScaleEditor::loadBuiltinColorPresets()
   pqColorMapModel colorMap;
   pqColorPresetModel *model = this->Form->Presets->getModel();
   colorMap.setColorSpace(pqColorMapModel::DivergingSpace);
-  colorMap.addPoint(pqChartValue((double)0.0), QColor( 35, 61, 181), 0.0);
-  colorMap.addPoint(pqChartValue((double)1.0), QColor(172, 36,  32), 1.0);
+  colorMap.addPoint(pqChartValue((double)0.0), QColor( 59, 76, 192), 0.0);
+  colorMap.addPoint(pqChartValue((double)1.0), QColor(180,  4,  38), 1.0);
   model->addBuiltinColorMap(colorMap, "Cool to Warm");
 
   colorMap.removeAllPoints();
@@ -1041,6 +1065,12 @@ void pqColorScaleEditor::loadBuiltinColorPresets()
   colorMap.addPoint(pqChartValue((double)1.0), QColor(255, 255, 255), (double)1.0);
   model->addBuiltinColorMap(colorMap, "Grayscale");
 
+  colorMap.removeAllPoints();
+  colorMap.setColorSpace(pqColorMapModel::RgbSpace);
+  colorMap.addPoint(pqChartValue((double)0.0), QColor( 10,  10, 242), (double)0.0);
+  colorMap.addPoint(pqChartValue((double)1.0), QColor(242, 242,  10), (double)1.0);
+  model->addBuiltinColorMap(colorMap, "Blue to Yellow");
+
   colorMap.removeAllPoints();
   colorMap.setColorSpace(pqColorMapModel::RgbSpace);
   colorMap.addPoint(pqChartValue((double)0.0), QColor(0,   0,   0  ), (double)0.0);
@@ -1286,13 +1316,6 @@ void pqColorScaleEditor::updateScalarRange(double min, double max)
   // Update the editor scalar range.
   this->Viewer->SetWholeScalarRange(min, max);
   this->Viewer->SetVisibleScalarRange(min, max);
-
-  // Check the scalar range when log scale is on.
-  if(this->Form->UseLogScale->isChecked() && min <= 0.0)
-    {
-    // Clear the log scale flag on the editor and proxy.
-    this->Form->UseLogScale->setChecked(false);
-    }
 }
 
 void pqColorScaleEditor::setLegend(pqScalarBarRepresentation *legend)
@@ -1307,6 +1330,10 @@ void pqColorScaleEditor::setLegend(pqScalarBarRepresentation *legend)
     // Clean up the current connections.
     this->disconnect(this->Legend, 0, this, 0);
     this->Form->Links.removeAllPropertyLinks();
+    delete this->Form->TitleColorLink;
+    this->Form->TitleColorLink = 0;
+    delete this->Form->LabelColorLink;
+    this->Form->LabelColorLink = 0;
     }
 
   this->Legend = legend;
@@ -1379,6 +1406,12 @@ void pqColorScaleEditor::setLegend(pqScalarBarRepresentation *legend)
                                       "value", SIGNAL(valueChanged(double)),
                                       proxy, proxy->GetProperty("AspectRatio"));
 
+    // this manages the linking between the global properties and the color
+    // properties.
+    this->Form->TitleColorLink = new pqStandardColorLinkAdaptor(
+      this->Form->TitleColorButton, proxy, "TitleColor");
+    this->Form->LabelColorLink = new pqStandardColorLinkAdaptor(
+      this->Form->LabelColorButton, proxy, "LabelColor");
     // Update the legend title gui.
     this->updateLegendTitle();
     }
@@ -1413,6 +1446,10 @@ void pqColorScaleEditor::makeDefault()
   pqLookupTableManager* lut_mgr = core->getLookupTableManager();
   if (lut_mgr)
     {
-    lut_mgr->saveAsDefault(this->ColorMap);
+    lut_mgr->saveLUTAsDefault(this->ColorMap);
+    if(this->OpacityFunction)
+      {
+      lut_mgr->saveOpacityFunctionAsDefault(this->OpacityFunction);
+      }
     }
 }
