@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqColorScaleEditor.h,v $
+   Module:    pqColorScaleEditor.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -92,6 +92,7 @@ private slots:
   void setAutoRescale(bool on);
   void rescaleToNewRange();
   void rescaleToDataRange();
+  void rescaleToDataRangeOverTime();
 
   void setUseDiscreteColors(bool on);
   void setSizeFromText();
