@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqComboBoxDomain.cxx,v $
+   Module:    pqComboBoxDomain.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -128,12 +128,48 @@ void pqComboBoxDomain::addString(const QString& str)
   this->domainChanged();
 }
 
+void pqComboBoxDomain::insertString(int index, const QString& str)
+{
+  this->Internal->UserStrings.insert(index, str);
+  this->domainChanged();
+}
+
+void pqComboBoxDomain::removeString(const QString& str)
+{
+  int index = this->Internal->UserStrings.indexOf(str);
+  if (index >= 0)
+    {
+    this->Internal->UserStrings.removeAt(index);
+    this->domainChanged();
+    }
+}
+
 void pqComboBoxDomain::removeAllStrings()
 {
   this->Internal->UserStrings.clear();
   this->domainChanged();
 }
 
+vtkSMProperty* pqComboBoxDomain::getProperty()const
+{
+  return this->Internal->Property;
+}
+
+vtkSMDomain* pqComboBoxDomain::getDomain()const
+{
+  return this->Internal->Domain;  
+}
+
+const QString& pqComboBoxDomain::getDomainName()const
+{
+  return this->Internal->DomainName;
+}
+
+const QStringList& pqComboBoxDomain::getUserStrings()const
+{
+  return this->Internal->UserStrings;
+}
+
 void pqComboBoxDomain::domainChanged()
 {
   if(this->Internal->MarkedForUpdate)
@@ -141,8 +177,8 @@ void pqComboBoxDomain::domainChanged()
     return;
     }
 
-  this->Internal->MarkedForUpdate = true;
-  QTimer::singleShot(0, this, SLOT(internalDomainChanged()));
+  this->markForUpdate(true);
+  this->internalDomainChanged();
 }
 
 void pqComboBoxDomain::internalDomainChanged()
@@ -252,7 +288,10 @@ void pqComboBoxDomain::internalDomainChanged()
       combo->setCurrentIndex(0);
       }
     }
-  this->Internal->MarkedForUpdate = false;
+  this->markForUpdate(false);
 }
 
-
+void pqComboBoxDomain::markForUpdate(bool mark)
+{
+  this->Internal->MarkedForUpdate = mark;
+}
