@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayColorWidget.cxx,v $
+   Module:    pqDisplayColorWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -67,7 +67,7 @@ pqDisplayColorWidget::pqDisplayColorWidget( QWidget *p ) :
   this->Variables->setMaxVisibleItems(60);
   this->Variables->setObjectName("Variables");
   this->Variables->setMinimumSize( QSize( 150, 0 ) );
-  this->Variables->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLength);
+  this->Variables->setSizeAdjustPolicy(QComboBox::AdjustToContents);
   
   this->Components = new QComboBox( this );
   this->Components->setObjectName("Components");
@@ -85,15 +85,6 @@ pqDisplayColorWidget::pqDisplayColorWidget( QWidget *p ) :
     SLOT(onVariableChanged(pqVariableType, const QString&)));
 
   this->VTKConnect = vtkEventQtSlotConnect::New();
-
-  pqUndoStack* stack = pqApplicationCore::instance()->getUndoStack();
-  if (stack)
-    {
-    QObject::connect(this, SIGNAL(begin(const QString&)),
-      stack, SLOT(beginUndoSet(const QString&)));
-    QObject::connect(this, SIGNAL(end()),
-      stack, SLOT(endUndoSet()));
-    }
 }
 
 //-----------------------------------------------------------------------------
@@ -178,7 +169,7 @@ void pqDisplayColorWidget::onComponentActivated(int row)
   pqPipelineRepresentation* display = this->getRepresentation();
   if(display)
     {
-    emit this->begin("Color Component Change");
+    BEGIN_UNDO_SET("Color Component Change");
     pqScalarsToColors* lut = display->getLookupTable();
     if(row == 0)
       {
@@ -190,7 +181,7 @@ void pqDisplayColorWidget::onComponentActivated(int row)
       }
     lut->updateScalarBarTitles(this->Components->itemText(row));
     display->resetLookupTableScalarRange();
-    emit this->end();
+    END_UNDO_SET();
 
     display->renderViewEventually();
     }
@@ -204,7 +195,7 @@ void pqDisplayColorWidget::onVariableActivated(int row)
     return;
     }
 
-  const QStringList d = this->Variables->itemData(row).toString().split("|");
+  const QStringList d = this->Variables->itemData(row).toStringList();
   if(d.size() != 2)
     return;
     
@@ -225,20 +216,29 @@ void pqDisplayColorWidget::onVariableActivated(int row)
 }
 
 //-----------------------------------------------------------------------------
-const QString pqDisplayColorWidget::variableData(pqVariableType type, 
-  const QString& name)
+const QStringList pqDisplayColorWidget::variableData(pqVariableType type, 
+                                                     const QString& name)
 {
+  QStringList result;
+  result << name;
+
   switch(type)
     {
     case VARIABLE_TYPE_NONE:
-      return name + "|none";
+      result << "none";
+      break;
     case VARIABLE_TYPE_NODE:
-      return name + "|point";
+      result << "point";
+      break;
     case VARIABLE_TYPE_CELL:
-      return name + "|cell";
+      result << "cell";
+      break;
+    default:
+      // Return empty list.
+      return QStringList();
     }
     
-  return QString();
+  return result;
 }
 
 //-----------------------------------------------------------------------------
@@ -248,7 +248,7 @@ void pqDisplayColorWidget::onVariableChanged(pqVariableType type,
   pqPipelineRepresentation* display = this->getRepresentation();
   if (display)
     {
-    emit this->begin("Color Change");
+    BEGIN_UNDO_SET("Color Change");
     switch(type)
       {
     case VARIABLE_TYPE_NONE:
@@ -263,7 +263,7 @@ void pqDisplayColorWidget::onVariableChanged(pqVariableType type,
         vtkSMDataRepresentationProxy::CELL_DATA);
       break;
       }
-    emit this->end();
+    END_UNDO_SET();
     display->renderViewEventually();
     }
 }
