@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayColorWidget.h,v $
+   Module:    pqDisplayColorWidget.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -98,15 +98,6 @@ signals:
   /// changes.
   void modified();
 
-  /// Fired just before the color is changed on the underlying proxy.
-  /// This must be hooked to an undo stack to record the
-  /// changes in a undo set.
-  void begin(const QString&);
-
-  /// Fired just after the color is changed on the underlying proxy.
-  /// This must be hooked to an undo stack to record the
-  /// changes in a undo set.
-  void end();
 private slots:
   /// Called to emit the variableChanged() signal in response to user input 
   /// or the chooseVariable() method.
@@ -122,7 +113,7 @@ private slots:
 private:
   /// Converts a variable type and name into a packed string representation 
   /// that can be used with a combo box.
-  static const QString variableData(pqVariableType, const QString& name);
+  static const QStringList variableData(pqVariableType, const QString& name);
  
   QIcon* CellDataIcon;
   QIcon* PointDataIcon;
