@@ -1,13 +1,13 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayProxyEditor.cxx,v $
+   Module:    pqDisplayProxyEditor.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.2. 
+   under the terms of the ParaView license version 1.2.
 
    See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
@@ -66,7 +66,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // ParaView client includes
 #include "pqApplicationCore.h"
-#include "pqColorScaleToolbar.h"
+#include "pqColorScaleEditor.h"
+#include "pqCoreUtilities.h"
 #include "pqCubeAxesEditorDialog.h"
 #include "pqFileDialog.h"
 #include "pqOutputPort.h"
@@ -77,6 +78,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqScalarsToColors.h"
 #include "pqSignalAdaptorCompositeTreeWidget.h"
 #include "pqSMAdaptor.h"
+#include "pqStandardColorLinkAdaptor.h"
 #include "pqUndoStack.h"
 #include "pqWidgetRangeDomain.h"
 
@@ -88,7 +90,9 @@ public:
     this->Links = new pqPropertyLinks;
     this->InterpolationAdaptor = 0;
     this->EdgeColorAdaptor = 0;
+    this->AmbientColorAdaptor = 0;
     this->SliceDirectionAdaptor = 0;
+    this->BackfaceRepresentationAdaptor = 0;
     this->SliceDomain = 0;
     this->SelectedMapperAdaptor = 0;
     this->CompositeTreeAdaptor = 0;
@@ -99,7 +103,10 @@ public:
     delete this->Links;
     delete this->InterpolationAdaptor;
     delete this->SliceDirectionAdaptor;
+    delete this->BackfaceRepresentationAdaptor;
     delete this->SliceDomain;
+    delete this->AmbientColorAdaptor;
+    delete this->EdgeColorAdaptor;
     }
 
   pqPropertyLinks* Links;
@@ -108,8 +115,10 @@ public:
   QPointer<pqPipelineRepresentation> Representation;
   pqSignalAdaptorComboBox* InterpolationAdaptor;
   pqSignalAdaptorColor*    EdgeColorAdaptor;
+  pqSignalAdaptorColor*    AmbientColorAdaptor;
   pqSignalAdaptorComboBox* SliceDirectionAdaptor;
   pqSignalAdaptorComboBox* SelectedMapperAdaptor;
+  pqSignalAdaptorComboBox* BackfaceRepresentationAdaptor;
   pqWidgetRangeDomain* SliceDomain;
   pqSignalAdaptorCompositeTreeWidget* CompositeTreeAdaptor;
 
@@ -153,7 +162,7 @@ pqDisplayProxyEditor::~pqDisplayProxyEditor()
 
 //-----------------------------------------------------------------------------
 /// set the proxy to repr display properties for
-void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr) 
+void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
 {
   if(this->Internal->Representation == repr)
     {
@@ -180,7 +189,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     }
 
   this->setEnabled(true);
-  
+
   // The slots are already connected but we do not want them to execute
   // while we are initializing the GUI
   this->DisableSlots = 1;
@@ -190,6 +199,10 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     "checked", SIGNAL(stateChanged(int)),
     reprProxy, reprProxy->GetProperty("Visibility"));
 
+  this->Internal->Links->addPropertyLink(this->Internal->Selectable,
+    "checked", SIGNAL(stateChanged(int)),
+    reprProxy, reprProxy->GetProperty("Pickable"));
+
   vtkSMProperty* prop = 0;
 
   // setup cube axes visibility.
@@ -204,6 +217,13 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     this->Internal->AnnotationGroup->hide();
     }
 
+  //if ((prop = reprProxy->GetProperty("PieceBoundsVisibility")) != 0) //DDM TODO
+  //  {
+  //  this->Internal->Links->addPropertyLink(this->Internal->ShowPieceBounds,
+  //    "checked", SIGNAL(stateChanged(int)),
+  //    reprProxy, prop);
+  //  }
+
   // setup for choosing color
   if (reprProxy->GetProperty("DiffuseColor"))
     {
@@ -236,7 +256,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     QObject::connect(this->Internal->SpecularPower, SIGNAL(editingFinished()),
       this, SLOT(updateAllViews()));
     }
-  
+
   // setup for interpolation
   this->Internal->StyleInterpolation->clear();
   if ((prop = reprProxy->GetProperty("Interpolation")) != 0)
@@ -295,7 +315,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
     reprProxy, reprProxy->GetProperty("Position"), 1);
   validator = new QDoubleValidator(this);
   this->Internal->TranslateY->setValidator(validator);
-  
+
   this->Internal->Links->addPropertyLink(this->Internal->TranslateZ,
     "text", SIGNAL(editingFinished()),
     reprProxy, reprProxy->GetProperty("Position"), 2);
@@ -374,7 +394,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
   if (reprProxy->GetProperty("InterpolateScalarsBeforeMapping"))
     {
     this->Internal->Links->addPropertyLink(
-      this->Internal->ColorInterpolateColors, "checked", SIGNAL(stateChanged(int)),
+      this->Internal->ColorInterpolateScalars, "checked", SIGNAL(stateChanged(int)),
       reprProxy, reprProxy->GetProperty("InterpolateScalarsBeforeMapping"));
     }
 
@@ -401,6 +421,13 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
       reprProxy, prop);
     }
 
+  if ( (prop = reprProxy->GetProperty("AmbientColor")) != 0)
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->AmbientColorAdaptor,
+      "color", SIGNAL(colorChanged(const QVariant&)),
+      reprProxy, prop);
+    }
+
   if (reprProxy->GetProperty("Slice"))
     {
     this->Internal->SliceDomain = new pqWidgetRangeDomain(
@@ -411,7 +438,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
       "value", SIGNAL(valueChanged(int)),
       reprProxy, reprProxy->GetProperty("Slice"));
 
-    QList<QVariant> sliceModes = 
+    QList<QVariant> sliceModes =
       pqSMAdaptor::getEnumerationPropertyDomain(
         reprProxy->GetProperty("SliceMode"));
     foreach(QVariant item, sliceModes)
@@ -426,7 +453,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
 
   if (reprProxy->GetProperty("ExtractedBlockIndex"))
     {
-    this->Internal->CompositeTreeAdaptor = 
+    this->Internal->CompositeTreeAdaptor =
       new pqSignalAdaptorCompositeTreeWidget(
         this->Internal->compositeTree,
         this->Internal->Representation->getOutputPortFromInput()->getOutputPortProxy(),
@@ -436,7 +463,7 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
 
   if (reprProxy->GetProperty("SelectedMapperIndex"))
     {
-    QList<QVariant> mapperNames = 
+    QList<QVariant> mapperNames =
       pqSMAdaptor::getEnumerationPropertyDomain(
         reprProxy->GetProperty("SelectedMapperIndex"));
     foreach(QVariant item, mapperNames)
@@ -448,9 +475,58 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
       "currentText", SIGNAL(currentTextChanged(const QString&)),
       reprProxy, reprProxy->GetProperty("SelectedMapperIndex"));
     }
-    
 
-#if 0                                       //FIXME 
+  this->Internal->BackfaceStyleRepresentation->clear();
+  if ((prop = reprProxy->GetProperty("BackfaceRepresentation")) != NULL)
+    {
+    prop->UpdateDependentDomains();
+    QList<QVariant> items = pqSMAdaptor::getEnumerationPropertyDomain(prop);
+    foreach (QVariant item, items)
+      {
+      this->Internal->BackfaceStyleRepresentation->addItem(item.toString());
+      }
+    this->Internal->Links->addPropertyLink(
+                      this->Internal->BackfaceRepresentationAdaptor,
+                      "currentText", SIGNAL(currentTextChanged(const QString&)),
+                      reprProxy, prop);
+    this->Internal->BackfaceStyleGroup->setEnabled(true);
+    }
+  else
+    {
+    this->Internal->BackfaceStyleGroup->setEnabled(false);
+    }
+
+  QObject::connect(this->Internal->BackfaceStyleRepresentation,
+                   SIGNAL(currentIndexChanged(const QString&)),
+                   this, SLOT(updateEnableState()), Qt::QueuedConnection);
+
+  // setup for choosing backface color
+  if (reprProxy->GetProperty("BackfaceDiffuseColor"))
+    {
+    QList<QVariant> curColor = pqSMAdaptor::getMultipleElementProperty(
+                                reprProxy->GetProperty("BackfaceDiffuseColor"));
+
+    bool prev = this->Internal->BackfaceActorColor->blockSignals(true);
+    this->Internal->BackfaceActorColor->setChosenColor(
+                               QColor(qRound(curColor[0].toDouble()*255),
+                                      qRound(curColor[1].toDouble()*255),
+                                      qRound(curColor[2].toDouble()*255), 255));
+    this->Internal->BackfaceActorColor->blockSignals(prev);
+
+    new pqStandardColorLinkAdaptor(this->Internal->BackfaceActorColor,
+                                   reprProxy, "BackfaceDiffuseColor");
+    }
+
+  // setup for backface opacity
+  if (reprProxy->GetProperty("BackfaceOpacity"))
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->BackfaceOpacity,
+                                         "value", SIGNAL(editingFinished()),
+                                         reprProxy,
+                                         reprProxy->GetProperty("BackfaceOpacity"));
+    }
+
+#if 0                                       //FIXME
   // material
   this->Internal->StyleMaterial->blockSignals(true);
   this->Internal->StyleMaterial->clear();
@@ -485,16 +561,47 @@ void pqDisplayProxyEditor::setRepresentation(pqPipelineRepresentation* repr)
   this->Internal->StyleMaterial->blockSignals(false);
 #endif
 
+
+  new pqStandardColorLinkAdaptor(this->Internal->ColorActorColor,
+    reprProxy, "DiffuseColor");
+  if (reprProxy->GetProperty("EdgeColor"))
+    {
+    new pqStandardColorLinkAdaptor(this->Internal->EdgeColor,
+      reprProxy, "EdgeColor");
+    }
+  if (reprProxy->GetProperty("AmbientColor"))
+    {
+    new pqStandardColorLinkAdaptor(this->Internal->AmbientColor,
+      reprProxy, "AmbientColor");
+    }
+
   this->DisableSlots = 0;
-  
   QTimer::singleShot(0, this, SLOT(updateEnableState()));
+
+
+  //
+  if(reprProxy->GetProperty("SampleDistance"))
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->SampleDistanceValue,
+      "text", SIGNAL(editingFinished()),
+      reprProxy, reprProxy->GetProperty("SampleDistance"));
+    validator = new QDoubleValidator(this);
+    this->Internal->SampleDistanceValue->setValidator(validator);
+    }
+
+  if(reprProxy->GetProperty("AutoAdjustSampleDistances"))
+    {
+    this->Internal->Links->addPropertyLink(this->Internal->AutoAdjustSampleDistances,
+      "checked", SIGNAL(toggled(bool)),
+      reprProxy, reprProxy->GetProperty("AutoAdjustSampleDistances"));
+    }
 }
 
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::setupGUIConnections()
 {
   QObject::connect(
-    this->Internal->ViewZoomToData, SIGNAL(clicked(bool)), 
+    this->Internal->ViewZoomToData, SIGNAL(clicked(bool)),
     this, SLOT(zoomToData()));
   QObject::connect(
     this->Internal->EditColorMapButton, SIGNAL(clicked()),
@@ -513,14 +620,52 @@ void pqDisplayProxyEditor::setupGUIConnections()
     this->Internal->StyleInterpolation);
   this->Internal->InterpolationAdaptor->setObjectName(
     "StyleInterpolationAdapator");
- 
+
   QObject::connect(this->Internal->ColorActorColor,
     SIGNAL(chosenColorChanged(const QColor&)),
     this, SLOT(setSolidColor(const QColor&)));
 
+  /// Set up signal-slot connections to create a single undo-set for all the
+  /// changes that happen when the solid color is changed.
+  /// We need to do this for both solid and edge color since we want to make
+  /// sure that the undo-element for setting up of the "global property" link
+  /// gets added in the same set in which the solid/edge color is changed.
+  this->Internal->ColorActorColor->setUndoLabel("Change Solid Color");
+  pqUndoStack* stack = pqApplicationCore::instance()->getUndoStack();
+  if (stack)
+    {
+    QObject::connect(this->Internal->ColorActorColor,
+      SIGNAL(beginUndo(const QString&)),
+      stack, SLOT(beginUndoSet(const QString&)));
+    QObject::connect(this->Internal->ColorActorColor,
+      SIGNAL(endUndo()), stack, SLOT(endUndoSet()));
+    }
+
   this->Internal->EdgeColorAdaptor = new pqSignalAdaptorColor(
     this->Internal->EdgeColor, "chosenColor",
     SIGNAL(chosenColorChanged(const QColor&)), false);
+  this->Internal->EdgeColor->setUndoLabel("Change Edge Color");
+  if (stack)
+    {
+    QObject::connect(this->Internal->EdgeColor,
+      SIGNAL(beginUndo(const QString&)),
+      stack, SLOT(beginUndoSet(const QString&)));
+    QObject::connect(this->Internal->EdgeColor,
+      SIGNAL(endUndo()), stack, SLOT(endUndoSet()));
+    }
+
+  this->Internal->AmbientColorAdaptor = new pqSignalAdaptorColor(
+    this->Internal->AmbientColor, "chosenColor",
+    SIGNAL(chosenColorChanged(const QColor&)), false);
+  this->Internal->AmbientColor->setUndoLabel("Change Ambient Color");
+  if (stack)
+    {
+    QObject::connect(this->Internal->AmbientColor,
+      SIGNAL(beginUndo(const QString&)),
+      stack, SLOT(beginUndoSet(const QString&)));
+    QObject::connect(this->Internal->AmbientColor,
+      SIGNAL(endUndo()), stack, SLOT(endUndoSet()));
+    }
 
   QObject::connect(this->Internal->StyleMaterial, SIGNAL(currentIndexChanged(int)),
                    this, SLOT(updateMaterial(int)));
@@ -536,17 +681,57 @@ void pqDisplayProxyEditor::setupGUIConnections()
   QObject::connect(this->Internal->SelectedMapperAdaptor,
     SIGNAL(currentTextChanged(const QString&)),
     this, SLOT(selectedMapperChanged()), Qt::QueuedConnection);
+
+  this->Internal->BackfaceRepresentationAdaptor = new pqSignalAdaptorComboBox(
+                                   this->Internal->BackfaceStyleRepresentation);
+  this->Internal->BackfaceRepresentationAdaptor->setObjectName(
+                                         "BackfaceStyleRepresentationAdapator");
+
+  QObject::connect(this->Internal->BackfaceActorColor,
+                   SIGNAL(chosenColorChanged(const QColor&)),
+                   this, SLOT(setBackfaceSolidColor(const QColor&)));
+
+  this->Internal->BackfaceActorColor->setUndoLabel("Change Backface Solid Color");
+  stack = pqApplicationCore::instance()->getUndoStack();
+  if (stack)
+    {
+    QObject::connect(this->Internal->BackfaceActorColor,
+                     SIGNAL(beginUndo(const QString&)),
+                     stack, SLOT(beginUndoSet(const QString&)));
+    QObject::connect(this->Internal->BackfaceActorColor,
+                     SIGNAL(endUndo()), stack, SLOT(endUndoSet()));
+    }
+
+
+  QObject::connect(this->Internal->AutoAdjustSampleDistances,
+                   SIGNAL(toggled(bool)),
+                   this,
+                   SLOT(setAutoAdjustSampleDistances(bool)));
 }
 
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::updateEnableState()
 {
+  int reprType = this->Internal->Representation->getRepresentationType();
+
   if (this->Internal->ColorBy->getCurrentText() == "Solid Color")
     {
-    this->Internal->ColorInterpolateColors->setEnabled(false);
-    this->Internal->ColorButtonStack->setCurrentWidget(
+    this->Internal->ColorInterpolateScalars->setEnabled(false);
+    if (reprType == vtkSMPVRepresentationProxy::WIREFRAME ||
+      reprType == vtkSMPVRepresentationProxy::POINTS ||
+      reprType == vtkSMPVRepresentationProxy::OUTLINE)
+      {
+      this->Internal->ColorButtonStack->setCurrentWidget(
+        this->Internal->AmbientColorPage);
+      this->Internal->LightingGroup->setEnabled(false);
+      }
+    else
+      {
+      this->Internal->ColorButtonStack->setCurrentWidget(
         this->Internal->SolidColorPage);
-    this->Internal->LightingGroup->setEnabled(true);
+      this->Internal->LightingGroup->setEnabled(true);
+      }
+    this->Internal->BackfaceActorColor->setEnabled(true);
     }
   else
     {
@@ -554,13 +739,13 @@ void pqDisplayProxyEditor::updateEnableState()
       {
       this->Internal->LightingGroup->setEnabled(false);
       }
-    this->Internal->ColorInterpolateColors->setEnabled(true);
+    this->Internal->ColorInterpolateScalars->setEnabled(true);
     this->Internal->ColorButtonStack->setCurrentWidget(
         this->Internal->ColorMapPage);
+    this->Internal->BackfaceActorColor->setEnabled(false);
     }
 
-  int reprType = this->Internal->Representation->getRepresentationType();
-  
+
   this->Internal->EdgeStyleGroup->setEnabled(
     reprType == vtkSMPVRepresentationProxy::SURFACE_WITH_EDGES);
 
@@ -582,14 +767,42 @@ void pqDisplayProxyEditor::updateEnableState()
     reprType == vtkSMPVRepresentationProxy::VOLUME
     && this->Internal->Representation->getProxy()->GetProperty("SelectedMapperIndex"));
 
-  vtkSMDataRepresentationProxy* display = 
+  vtkSMProperty *backfaceRepProperty = this->Internal->Representation
+    ->getRepresentationProxy()->GetProperty("BackfaceRepresentation");
+  if (   !backfaceRepProperty
+      || (   (reprType != vtkSMPVRepresentationProxy::POINTS)
+          && (reprType != vtkSMPVRepresentationProxy::WIREFRAME)
+          && (reprType != vtkSMPVRepresentationProxy::SURFACE)
+          && (reprType != vtkSMPVRepresentationProxy::SURFACE_WITH_EDGES) ) )
+    {
+    this->Internal->BackfaceStyleGroup->setEnabled(false);
+    }
+  else
+    {
+    this->Internal->BackfaceStyleGroup->setEnabled(true);
+    int backRepType
+      = pqSMAdaptor::getElementProperty(backfaceRepProperty).toInt();
+
+    bool backFollowsFront
+      = (   (backRepType == vtkSMPVRepresentationProxy::FOLLOW_FRONTFACE)
+         || (backRepType == vtkSMPVRepresentationProxy::CULL_BACKFACE)
+         || (backRepType == vtkSMPVRepresentationProxy::CULL_FRONTFACE) );
+
+    this->Internal->BackfaceStyleGroupOptions->setEnabled(!backFollowsFront);
+    }
+
+  vtkSMDataRepresentationProxy* display =
     this->Internal->Representation->getRepresentationProxy();
   if (display)
     {
     QVariant scalarMode = pqSMAdaptor::getEnumerationProperty(
       display->GetProperty("ColorAttributeType"));
-    vtkPVDataInformation* geomInfo = 
+    vtkPVDataInformation* geomInfo =
       display->GetRepresentedDataInformation(/*update=*/false);
+    if (!geomInfo)
+      {
+      return;
+      }
     vtkPVDataSetAttributesInformation* attrInfo;
     if (scalarMode == "POINT_DATA")
       {
@@ -622,13 +835,10 @@ void pqDisplayProxyEditor::updateEnableState()
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::openColorMapEditor()
 {
-  // Get the color scale editor from the application core's registry.
-  pqColorScaleToolbar *colorScale = qobject_cast<pqColorScaleToolbar *>(
-      pqApplicationCore::instance()->manager("COLOR_SCALE_EDITOR"));
-  if(colorScale)
-    {
-    colorScale->editColorMap(this->Internal->Representation);
-    }
+  pqColorScaleEditor editor(pqCoreUtilities::mainWidget());
+  editor.setObjectName("pqColorScaleDialog");
+  editor.setRepresentation(this->Internal->Representation);
+  editor.exec();
 }
 
 //-----------------------------------------------------------------------------
@@ -686,7 +896,7 @@ QVariant pqDisplayProxyEditor::specularColor() const
     ret.append(1.0);
     return ret;
     }
-  
+
   vtkSMProxy* proxy = this->Internal->Representation->getProxy();
   return pqSMAdaptor::getMultipleElementProperty(
        proxy->GetProperty("DiffuseColor"));
@@ -716,7 +926,7 @@ void pqDisplayProxyEditor::setSpecularColor(QVariant specColor)
 void pqDisplayProxyEditor::updateMaterial(int vtkNotUsed(idx))
 {
   // FIXME: when we enable materials.
-#if 0   
+#if 0
   if(idx == 0)
     {
     this->Internal->Representation->getDisplayProxy()->SetMaterialCM(0);
@@ -724,7 +934,7 @@ void pqDisplayProxyEditor::updateMaterial(int vtkNotUsed(idx))
     }
   else if(idx == 1)
     {
-    pqFileDialog diag(NULL, this, "Open Material File", QString(), 
+    pqFileDialog diag(NULL, this, "Open Material File", QString(),
                       "Material Files (*.xml)");
     diag.setFileMode(pqFileDialog::ExistingFile);
     if(diag.exec() == QDialog::Accepted)
@@ -807,9 +1017,9 @@ void pqDisplayProxyEditor::volumeBlockSelected()
       && this->Internal->Representation)
     {
     bool valid = false;
-    unsigned int selectedIndex = 
+    unsigned int selectedIndex =
       this->Internal->CompositeTreeAdaptor->getCurrentFlatIndex(&valid);
-    if (valid && selectedIndex > 0) 
+    if (valid && selectedIndex > 0)
       {
       vtkSMDataRepresentationProxy* repr =
         this->Internal->Representation->getRepresentationProxy();
@@ -830,28 +1040,59 @@ void pqDisplayProxyEditor::setSolidColor(const QColor& color)
   val.push_back(color.red()/255.0);
   val.push_back(color.green()/255.0);
   val.push_back(color.blue()/255.0);
-
-  pqUndoStack* stack = pqApplicationCore::instance()->getUndoStack();
-  if (stack)
-    {
-    stack->beginUndoSet("Change Solid Color");
-    }
-  pqSMAdaptor::setMultipleElementProperty(
-    this->Internal->Representation->getProxy()->GetProperty("AmbientColor"), val);
   pqSMAdaptor::setMultipleElementProperty(
     this->Internal->Representation->getProxy()->GetProperty("DiffuseColor"), val);
 
-  // If specular while if off, then we want to update the specular color as
+  // If specular white is off, then we want to update the specular color as
   // well.
   emit this->specularColorChanged();
-  if (stack)
-    {
-    stack->endUndoSet();
-    }
+}
+
+//-----------------------------------------------------------------------------
+// Called when the GUI selection for the backface solid color changes.
+void pqDisplayProxyEditor::setBackfaceSolidColor(const QColor& color)
+{
+  QList<QVariant> val;
+  val.push_back(color.red()/255.0);
+  val.push_back(color.green()/255.0);
+  val.push_back(color.blue()/255.0);
+
+  pqSMAdaptor::setMultipleElementProperty(
+    this->Internal->Representation->getProxy()->GetProperty("BackfaceAmbientColor"), val);
+  pqSMAdaptor::setMultipleElementProperty(
+    this->Internal->Representation->getProxy()->GetProperty("BackfaceDiffuseColor"), val);
+
+  // If specular white is off, then we want to update the specular color as
+  // well.
+  emit this->specularColorChanged();
+}
+
+//-----------------------------------------------------------------------------
+void pqDisplayProxyEditor::setAutoAdjustSampleDistances(bool flag)
+{
+  this->Internal->SampleDistanceValue->setEnabled(!flag);
 }
 
 //-----------------------------------------------------------------------------
 void pqDisplayProxyEditor::selectedMapperChanged()
 {
-
+  if(!this->Internal->SelectedMapperIndex->currentText().compare(
+      QString("Fixed Point"), Qt::CaseInsensitive))
+    {
+    // Fixed point does not uses sample distance.
+    this->Internal->SampleDistance->setEnabled(0);
+    this->Internal->SampleDistanceValue->setEnabled(0);
+    this->Internal->AutoAdjustSampleDistances->setEnabled(0);
+    }
+  else if(!this->Internal->SelectedMapperIndex->currentText().compare(
+      QString("GPU"), Qt::CaseInsensitive))
+    {
+    this->Internal->SampleDistance->setEnabled(1);
+    this->Internal->SampleDistanceValue->setEnabled(0);
+    this->Internal->AutoAdjustSampleDistances->setEnabled(1);
+    }
+  else
+    {
+      // Do nothing.
+    }
 }
