@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayProxyEditorWidget.h,v $
+   Module:    pqDisplayProxyEditorWidget.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -58,15 +58,17 @@ public:
   void setView(pqView* view);
   void setOutputPort(pqOutputPort* port);
 
-  /// Set the representation to edit. If NULL, source and view must be set so
-  /// that the widget can show a default GUI which allows the user to
-  /// turn visibility on which entails creating a new representation.
-  void setRepresentation(pqRepresentation*);
+
   pqRepresentation* getRepresentation() const;
 
 public slots:
   void reloadGUI();
 
+  /// Set the representation to edit. If NULL, source and view must be set so
+  /// that the widget can show a default GUI which allows the user to
+  /// turn visibility on which entails creating a new representation.
+  void setRepresentation(pqRepresentation*);
+
 signals:
   /// Fired when the browser begins performing an undoable change.
   void beginUndo(const QString& label);
