@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqExodusIIPanel.cxx,v $
+   Module:    pqExodusIIPanel.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -36,13 +36,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QAction>
 #include <QCheckBox>
 #include <QDoubleSpinBox>
+#include <QHeaderView>
 #include <QLabel>
+#include <QMap>
+#include <QtDebug>
 #include <QTimer>
 #include <QTreeWidget>
 #include <QVariant>
 #include <QVector>
-#include <QMap>
-#include <QHeaderView>
 
 // VTK includes
 
@@ -51,28 +52,37 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkGraph.h"
 #include "vtkProcessModule.h"
 #include "vtkPVArrayInformation.h"
+#include "vtkPVCompositeDataInformation.h"
+#include "vtkPVCompositeDataInformationIterator.h"
 #include "vtkPVDataInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
 #include "vtkPVSILInformation.h"
 #include "vtkSmartPointer.h"
+#include "vtkSMDoubleVectorProperty.h"
 #include "vtkSMProperty.h"
-#include "vtkSMProxyManager.h"
-#include "vtkSMSourceProxy.h"
 #include "vtkSMPropertyHelper.h"
+#include "vtkSMSourceProxy.h"
 
 // ParaView includes
+#include "pqApplicationCore.h"
+#include "pqOutputPort.h"
+#include "pqPipelineSource.h"
 #include "pqPropertyManager.h"
 #include "pqProxy.h"
-#include "pqServer.h"
-#include "pqTimeKeeper.h"
-#include "pqSMAdaptor.h"
-#include "pqTreeWidgetSelectionHelper.h"
-#include "pqTreeWidgetItemObject.h"
-#include "vtkSMDoubleVectorProperty.h"
-#include "pqSILModel.h"
 #include "pqProxySILModel.h"
+#include "pqSelectionManager.h"
+#include "pqServer.h"
+#include "pqSILModel.h"
+#include "pqSMAdaptor.h"
+#include "pqTimeKeeper.h"
 #include "pqTreeViewSelectionHelper.h"
+#include "pqTreeWidgetItemObject.h"
+#include "pqTreeWidgetSelectionHelper.h"
 
+#include <vtkstd/vector>
+#include <vtkstd/algorithm>
+
+//-----------------------------------------------------------------------------
 class pqExodusIIPanel::pqUI : public QObject, public Ui::ExodusIIPanel 
 {
 public:
@@ -112,6 +122,7 @@ public:
   int SILUpdateStamp;
 };
 
+//-----------------------------------------------------------------------------
 pqExodusIIPanel::pqExodusIIPanel(pqProxy* object_proxy, QWidget* p) :
   Superclass(object_proxy, p)
 {
@@ -168,12 +179,26 @@ pqExodusIIPanel::pqExodusIIPanel(pqProxy* object_proxy, QWidget* p) :
     {
     new pqTreeViewSelectionHelper(tree);
     }
+
+  pqSelectionManager* selMan = qobject_cast<pqSelectionManager*>(
+    pqApplicationCore::instance()->manager("SelectionManager"));
+  if (selMan)
+    {
+    QObject::connect(selMan, SIGNAL(selectionChanged(pqOutputPort*)),
+      this, SLOT(onSelectionChanged(pqOutputPort*)));
+    }
+  QObject::connect(this->UI->checkSelected, SIGNAL(pressed()),
+    this, SLOT(setSelectedBlocksCheckState()), Qt::QueuedConnection);
+  QObject::connect(this->UI->uncheckSelected, SIGNAL(pressed()),
+    this, SLOT(uncheckSelectedBlocks()), Qt::QueuedConnection);
 }
 
+//-----------------------------------------------------------------------------
 pqExodusIIPanel::~pqExodusIIPanel()
 {
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::updateSIL()
 {
   vtkSMProxy* reader = this->referenceProxy()->getProxy();
@@ -197,6 +222,7 @@ void pqExodusIIPanel::updateSIL()
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::addSelectionsToTreeWidget(const QString& prop, 
                                       QTreeWidget* tree,
                                       PixmapType pix)
@@ -212,6 +238,7 @@ void pqExodusIIPanel::addSelectionsToTreeWidget(const QString& prop,
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::addSelectionToTreeWidget(const QString& name,
                                                const QString& realName,
                                                QTreeWidget* tree,
@@ -263,13 +290,14 @@ void pqExodusIIPanel::addSelectionToTreeWidget(const QString& name,
   this->UI->TreeItemToPropMap[item] = prop;
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::linkServerManagerProperties()
 {
 
   this->propertyManager()->registerLink(
     this->UI->Blocks->model(), "values", SIGNAL(valuesChanged()),
     this->proxy(),
-    this->proxy()->GetProperty("ElementBlockArrayStatus"));
+    this->proxy()->GetProperty("ElementBlocks"));
 
   // parent class hooks up some of our widgets in the ui
   this->Superclass::linkServerManagerProperties();
@@ -285,20 +313,23 @@ void pqExodusIIPanel::linkServerManagerProperties()
   this->addSelectionToTreeWidget("Global Element Ids", "GlobalElementId", this->UI->Variables,
                    PM_ELEMBLK, "GenerateGlobalElementIdArray");
 
+  this->addSelectionToTreeWidget("Implicit Element Ids", "ImplicitElementId", this->UI->Variables,
+                   PM_ELEMBLK, "GenerateImplicitElementIdArray");
+
   // integer array indicating file id (number in file name or position in sequence)
   this->addSelectionToTreeWidget("File Ids", "FileId", this->UI->Variables,
                    PM_ELEMBLK, "GenerateFileIdArray");
   
   // do the cell variables
-  this->addSelectionsToTreeWidget("ElementResultArrayStatus",
+  this->addSelectionsToTreeWidget("ElementVariables",
                                   this->UI->Variables, PM_ELEMBLK);
   
   // do the face variables
-  this->addSelectionsToTreeWidget("FaceResultArrayStatus",
+  this->addSelectionsToTreeWidget("FaceVariables",
                                   this->UI->Variables, PM_FACEBLK);
   
   // do the edge variables
-  this->addSelectionsToTreeWidget("EdgeResultArrayStatus",
+  this->addSelectionsToTreeWidget("EdgeVariables",
                                   this->UI->Variables, PM_EDGEBLK);
 
   // do the set results variables
@@ -317,10 +348,13 @@ void pqExodusIIPanel::linkServerManagerProperties()
   this->addSelectionToTreeWidget("Global Node Ids", "GlobalNodeId", this->UI->Variables,
                    PM_NODE, "GenerateGlobalNodeIdArray");
 
+  this->addSelectionToTreeWidget("Implicit Node Ids", "ImplicitNodeId", this->UI->Variables,
+                   PM_NODE, "GenerateImplicitNodeIdArray");
+
   int numBef = this->UI->Variables->topLevelItemCount();
   
   // do the node variables
-  this->addSelectionsToTreeWidget("PointResultArrayStatus",
+  this->addSelectionsToTreeWidget("PointVariables",
                                   this->UI->Variables, PM_NODE);
   
   int numAft = this->UI->Variables->topLevelItemCount();
@@ -356,7 +390,7 @@ void pqExodusIIPanel::linkServerManagerProperties()
     }
 
   // do the global variables
-  this->addSelectionsToTreeWidget("GlobalResultArrayStatus",
+  this->addSelectionsToTreeWidget("GlobalVariables",
                                   this->UI->Variables, PM_GLOBAL);
 
   // we hook up the sideset/nodeset 
@@ -364,9 +398,9 @@ void pqExodusIIPanel::linkServerManagerProperties()
 
 
   // blocks
-  this->addSelectionsToTreeWidget("EdgeBlockArrayStatus",
+  this->addSelectionsToTreeWidget("EdgeBlocks",
                                   this->UI->EdgeBlockArrays, PM_EDGEBLK);
-  this->addSelectionsToTreeWidget("FaceBlockArrayStatus",
+  this->addSelectionsToTreeWidget("FaceBlocks",
                                   this->UI->FaceBlockArrays, PM_FACEBLK);
 
   // sets
@@ -408,8 +442,10 @@ void pqExodusIIPanel::linkServerManagerProperties()
                                         this->proxy(),
                                         this->proxy()->
                                         GetProperty("HasModeShapes"));
-  this->UI->ModeSelectSlider->setMaximum(this->UI->TimestepValues.size()-1);
-  this->UI->ModeSelectSlider->setMaximum(this->UI->TimestepValues.size()-1);
+  this->UI->ModeSelectSlider->setMinimum(1);
+  this->UI->ModeSelectSlider->setMaximum(this->UI->TimestepValues.size());
+  this->UI->ModeSelectSpinBox->setMinimum(1);
+  this->UI->ModeSelectSpinBox->setMaximum(this->UI->TimestepValues.size());
   if (this->UI->TimestepValues.size() > 0)
     {
     this->UI->ModeLabel->setText(
@@ -425,8 +461,7 @@ void pqExodusIIPanel::linkServerManagerProperties()
                                         "value",
                                         SIGNAL(valueChanged(int)),
                                         this->proxy(),
-                                        this->proxy()
-                                        ->GetProperty("ModeShape"));
+                                        this->proxy()->GetProperty("ModeShape"));
   QObject::connect(this->UI->HasModeShapes, SIGNAL(toggled(bool)),
                    this->UI->ModeShapeOptions, SLOT(setEnabled(bool)));
   QObject::connect(this->UI->ModeSelectSlider, SIGNAL(sliderMoved(int)),
@@ -439,6 +474,7 @@ void pqExodusIIPanel::linkServerManagerProperties()
 
 }
   
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::applyDisplacements(int state)
 {
   if(state == Qt::Checked && this->DisplItem)
@@ -449,15 +485,26 @@ void pqExodusIIPanel::applyDisplacements(int state)
                                                   true : false);
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::displChanged(bool state)
 {
-  if(!state)
+  QCheckBox* ApplyDisp = this->UI->ApplyDisplacements;
+  if (state)
+    {
+    // BUG #9843. When displ array is enabled, set the value of
+    // ApplyDisplacements to the last accepted value.
+    ApplyDisp->setCheckState(
+      pqSMAdaptor::getElementProperty(
+        this->proxy()->GetProperty("ApplyDisplacements")).toBool()?
+      Qt::Checked : Qt::Unchecked);
+    }
+  else
     {
-    QCheckBox* ApplyDisp = this->UI->ApplyDisplacements;
     ApplyDisp->setCheckState(Qt::Unchecked);
     }
 }
 
+//-----------------------------------------------------------------------------
 QString pqExodusIIPanel::formatDataFor(vtkPVArrayInformation* ai)
 {
   QString info;
@@ -499,15 +546,17 @@ QString pqExodusIIPanel::formatDataFor(vtkPVArrayInformation* ai)
   return info;
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::modeChanged(int value)
 {
-  if ((value >= 0) && (value < this->UI->TimestepValues.size()))
+  if ((value > 0) && (value <= this->UI->TimestepValues.size()))
     {
     this->UI->ModeLabel->setText(
-                            QString("%1").arg(this->UI->TimestepValues[value]));
+      QString("%1").arg(this->UI->TimestepValues[value-1]));
     }
 }
 
+//-----------------------------------------------------------------------------
 void pqExodusIIPanel::onRefresh()
 {
   vtkSMSourceProxy* sp = vtkSMSourceProxy::SafeDownCast(this->proxy());
@@ -521,3 +570,98 @@ void pqExodusIIPanel::onRefresh()
   sp->UpdatePropertyInformation(sp->GetProperty("TimeRange"));
   sp->UpdatePropertyInformation(sp->GetProperty("TimestepValues")); 
 }
+
+//-----------------------------------------------------------------------------
+void pqExodusIIPanel::onSelectionChanged(pqOutputPort* port)
+{
+  this->UI->checkSelected->setEnabled(false);
+  this->UI->uncheckSelected->setEnabled(false);
+
+  if (!port || port->getSource()->getProxy() != this->proxy())
+    {
+    return;
+    }
+
+  vtkSMProxy* activeSelection = port->getSelectionInput();
+  if (!activeSelection ||
+    strcmp(activeSelection->GetXMLName(), "BlockSelectionSource") != 0)
+    {
+    return;
+    }
+
+  this->UI->checkSelected->setEnabled(true);
+  this->UI->uncheckSelected->setEnabled(true);
+}
+
+//-----------------------------------------------------------------------------
+void pqExodusIIPanel::setSelectedBlocksCheckState(bool check/*=true*/)
+{
+  pqSelectionManager* selMan = qobject_cast<pqSelectionManager*>(
+    pqApplicationCore::instance()->manager("SelectionManager"));
+  if (!selMan || !selMan->getSelectedPort())
+    {
+    return;
+    }
+  
+  pqOutputPort* port = selMan->getSelectedPort();
+  vtkSMProxy* activeSelection = port->getSelectionInput();
+  vtkPVDataInformation* dataInfo = port->getDataInformation();
+
+  vtkSMPropertyHelper blocksProp(activeSelection, "Blocks");
+  vtkstd::vector<vtkIdType> block_ids;
+  block_ids.resize(blocksProp.GetNumberOfElements());
+  blocksProp.Get(&block_ids[0], blocksProp.GetNumberOfElements());
+  vtkstd::sort(block_ids.begin(), block_ids.end());
+
+  // if check is true then we are checking only the selected blocks,
+  // if check is false, then we are un-checking the selected blocks, leaving
+  // the selections for the other blocks as they are.
+  if (check)
+    {
+    this->UI->SILModel.setData(
+      this->UI->SILModel.makeIndex(0), Qt::Unchecked,
+      Qt::CheckStateRole);
+    }
+
+  // block selection only has the block ids, now we need to convert the block
+  // ids to names for the blocks (and sets) using the data information.
+  vtkPVCompositeDataInformationIterator* iter =
+    vtkPVCompositeDataInformationIterator::New();
+  iter->SetDataInformation(dataInfo);
+  unsigned int cur_index = 0;
+  for (iter->InitTraversal();
+    !iter->IsDoneWithTraversal() && cur_index < static_cast<unsigned int>(
+      block_ids.size()); iter->GoToNextItem())
+    {
+    if (static_cast<vtkIdType>(
+        iter->GetCurrentFlatIndex()) < block_ids[cur_index])
+      {
+      continue;
+      }
+    if (static_cast<vtkIdType>(
+        iter->GetCurrentFlatIndex()) > block_ids[cur_index])
+      {
+      qDebug() << "Failed to locate block's name for block id: " <<
+        block_ids[cur_index];
+      cur_index++;
+      continue;
+      }
+
+    vtkIdType vertexid =
+      this->UI->SILModel.findVertex(iter->GetCurrentName());
+    if (vertexid != -1)
+      {
+      this->UI->SILModel.setData(this->UI->SILModel.makeIndex(vertexid),
+        check? Qt::Checked : Qt::Unchecked, Qt::CheckStateRole);
+      }
+    else
+      {
+      // if vertexid==-1 from the SIL, then it's possible that this is a name of
+      // one of the sets, since currently, sets are not part of the SIL. Until
+      // the users ask for it, we will leave enabling/disabling the sets out.
+      }
+    cur_index++;
+    }
+
+  iter->Delete();
+}
