@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program:   ParaView
-   Module:    $RCSfile: pqFieldSelectionAdaptor.h,v $
+   Module:    pqFieldSelectionAdaptor.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
