@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqFileChooserWidget.cxx,v $
+   Module:    pqFileChooserWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -34,6 +34,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqFileChooserWidget.h"
 
 // Qt includes
+#include <QFileInfo>
 #include <QHBoxLayout>
 #include <QLineEdit>
 #include <QToolButton>
@@ -44,6 +45,9 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 pqFileChooserWidget::pqFileChooserWidget(QWidget* p)
   : QWidget(p), Server(NULL)
 {
+  this->ForceSingleFile = false;
+  this->UseDirectoryMode = false;
+  
   QHBoxLayout* l = new QHBoxLayout(this);
   l->setMargin(0);
   l->setSpacing(2);
@@ -61,8 +65,7 @@ pqFileChooserWidget::pqFileChooserWidget(QWidget* p)
 
   QObject::connect(this->LineEdit,
                    SIGNAL(textChanged(const QString&)),
-                   this,
-                   SIGNAL(filenameChanged(const QString&)));
+                   this, SLOT(emitFilenamesChanged(const QString&)));
 
 }
 
@@ -71,14 +74,57 @@ pqFileChooserWidget::~pqFileChooserWidget()
 }
 
 
-QString pqFileChooserWidget::filename()
+QStringList pqFileChooserWidget::filenames()
 {
-  return this->LineEdit->text();
+  return pqFileChooserWidget::splitFilenames(this->LineEdit->text());
 }
 
-void pqFileChooserWidget::setFilename(const QString& file)
+void pqFileChooserWidget::setFilenames(const QStringList& files)
 {
-  this->LineEdit->setText(file);
+  if(this->UseDirectoryMode)
+    {
+    if (!files.isEmpty())
+      {
+      this->LineEdit->setText(QFileInfo(files[0]).path());
+      }
+    else
+      {
+      this->LineEdit->setText("");
+      }
+    }
+  else if (this->ForceSingleFile)
+    {
+    if (!files.isEmpty())
+      {
+      this->LineEdit->setText(files[0]);
+      }
+    else
+      {
+      this->LineEdit->setText("");
+      }
+    }
+  else
+    {
+    this->LineEdit->setText(pqFileChooserWidget::joinFilenames(files));
+    }
+}
+
+QString pqFileChooserWidget::singleFilename()
+{
+  QStringList files = this->filenames();
+  if (files.isEmpty())
+    {
+    return "";
+    }
+  else
+    {
+    return files[0];
+    }
+}
+
+void pqFileChooserWidget::setSingleFilename(const QString &file)
+{
+  this->setFilenames(QStringList(file));
 }
 
 QString pqFileChooserWidget::extension()
@@ -108,16 +154,38 @@ void pqFileChooserWidget::chooseFile()
 
   pqFileDialog* dialog = new pqFileDialog(this->Server,
     this, tr("Open File:"), QString(), filters);
-  dialog->setFileMode(pqFileDialog::ExistingFile);
+  if(this->UseDirectoryMode)
+    {
+    dialog->setFileMode(pqFileDialog::Directory);
+    }
+  else if (this->ForceSingleFile)
+    {
+    dialog->setFileMode(pqFileDialog::ExistingFile);
+    }
+  else
+    {
+    dialog->setFileMode(pqFileDialog::ExistingFiles);
+    }
   if(QDialog::Accepted == dialog->exec())
     {
     QStringList files = dialog->getSelectedFiles();
     if(files.size())
       {
-      this->LineEdit->setText(files[0]);
+      this->setFilenames(files);
       }
     }
 }
 
-
-
+void pqFileChooserWidget::emitFilenamesChanged(const QString &fileString)
+{
+  QStringList fileList = pqFileChooserWidget::splitFilenames(fileString);
+  emit this->filenamesChanged(fileList);
+  if (!fileList.empty())
+    {
+    emit this->filenameChanged(fileList[0]);
+    }
+  else
+    {
+    emit this->filenameChanged("");
+    }
+}
