@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqGlyphPanel.cxx,v $
+   Module:    pqGlyphPanel.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -70,7 +70,7 @@ pqGlyphPanel::pqGlyphPanel(pqProxy* object_proxy, QWidget* _parent)
 
   if (!scaleFactor)
     {
-    qDebug() << "Failed to locate ScaleFactor widget.";
+    qWarning() << "Failed to locate ScaleFactor widget.";
     return;
     }
 
@@ -109,6 +109,33 @@ pqGlyphPanel::pqGlyphPanel(pqProxy* object_proxy, QWidget* _parent)
   lockButton->toggle();
   lockButton->toggle();
 
+  // Get the widgets for selecting the scalars and vectors so that we can
+  // set their enabled flags to reflect their usefulness.
+  this->ScalarsWidget = this->findChild<QWidget*>("SelectInputScalars");
+  if (!this->ScalarsWidget)
+    {
+    qWarning() << "Failed to locate Scalars widget.";
+    return;
+    }
+
+  this->VectorsWidget = this->findChild<QWidget*>("SelectInputVectors");
+  if (!this->VectorsWidget)
+    {
+    qWarning() << "Failed to locate Vectors widget.";
+    return;
+    }
+
+  this->OrientWidget = this->findChild<QCheckBox*>("SetOrient");
+  if (!this->OrientWidget)
+    {
+    qWarning() << "Failed to locate Orient widget.";
+    return;
+    }
+
+  QObject::connect(this->propertyManager(), SIGNAL(modified()),
+                   this, SLOT(updateScalarsVectorsEnable()),
+                   Qt::QueuedConnection);
+
   if (object_proxy->modifiedState() == pqProxy::UNINITIALIZED)
     {
     this->updateScaleFactor();
@@ -131,6 +158,8 @@ void pqGlyphPanel::updateScaleFactor()
 
   vtkSMProxy* filterProxy = this->proxy();
   filterProxy->GetProperty("Input")->UpdateDependentDomains();
+  filterProxy->GetProperty("SelectInputScalars")->UpdateDependentDomains();
+  filterProxy->GetProperty("SelectInputVectors")->UpdateDependentDomains();
 
   vtkSMEnumerationDomain* enumDomain = vtkSMEnumerationDomain::SafeDownCast(
     filterProxy->GetProperty("SetScaleMode")->GetDomain("enum"));
@@ -159,9 +188,9 @@ void pqGlyphPanel::updateScaleFactor()
       {
       domain = vtkSMArrayRangeDomain::SafeDownCast(
         scaleFactorProperty->GetDomain("scalar_range"));
-      if (domain->GetMaximumExists(0) && domain->GetMinimumExists(0))
+      if (domain->GetMaximumExists(0) /*&& domain->GetMinimumExists(0)*/)
         {
-        divisor = domain->GetMaximum(0)-domain->GetMinimum(0);
+        divisor = domain->GetMaximum(0)/*-domain->GetMinimum(0)*/;
         }
       }
     break;
@@ -171,10 +200,10 @@ void pqGlyphPanel::updateScaleFactor()
       {
       domain = vtkSMArrayRangeDomain::SafeDownCast(
         scaleFactorProperty->GetDomain("vector_range"));
-      if (domain->GetMaximumExists(3) && domain->GetMinimumExists(3))
+      if (domain->GetMaximumExists(3)/* && domain->GetMinimumExists(3)*/)
         {
         // we use the vector magnitude.
-        divisor = domain->GetMaximum(3)-domain->GetMinimum(3);
+        divisor = domain->GetMaximum(3)/*-domain->GetMinimum(3)*/;
         }
       }
     break;
@@ -184,7 +213,11 @@ void pqGlyphPanel::updateScaleFactor()
     break;
     }
 
-  divisor = (divisor < 1.0)? 1 : divisor;
+  divisor = fabs(divisor);
+  // the divisor can sometimes be very close to 0, which happens in case the
+  // vectors indeed have same value but due to precision issues are not reported
+  // as identical. In that case we just treat it as 0.
+  divisor = (divisor < 0.000000001)? 1 : divisor;
   scalefactor /= divisor;
 
   if (this->ScaleFactorWidget->property("text").toDouble() != scalefactor)
@@ -193,3 +226,39 @@ void pqGlyphPanel::updateScaleFactor()
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqGlyphPanel::updateScalarsVectorsEnable()
+{
+  vtkSMProxy* filterProxy = this->proxy();
+  filterProxy->GetProperty("Input")->UpdateDependentDomains();
+
+  vtkSMEnumerationDomain* enumDomain = vtkSMEnumerationDomain::SafeDownCast(
+                   filterProxy->GetProperty("SetScaleMode")->GetDomain("enum"));
+
+  int valid;
+  int scale_mode = enumDomain->GetEntryValue(
+    this->ScaleModeWidget->currentText().toAscii().data(), valid);
+  if (!valid) return;
+
+  bool orientGlyphs = this->OrientWidget->isChecked();
+
+  if (scale_mode == VTK_SCALE_BY_SCALAR)
+    {
+    this->ScalarsWidget->setEnabled(true);
+    }
+  else
+    {
+    this->ScalarsWidget->setEnabled(false);
+    }
+
+  if (   orientGlyphs
+      || (scale_mode == VTK_SCALE_BY_VECTOR)
+      || (scale_mode == VTK_SCALE_BY_VECTORCOMPONENTS) )
+    {
+    this->VectorsWidget->setEnabled(true);
+    }
+  else
+    {
+    this->VectorsWidget->setEnabled(false);
+    }
+}
