@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqImplicitPlaneWidget.cxx,v $
+   Module:    pqImplicitPlaneWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -332,15 +332,9 @@ void pqImplicitPlaneWidget::select()
 }
 
 //-----------------------------------------------------------------------------
-void pqImplicitPlaneWidget::resetBounds()
+void pqImplicitPlaneWidget::resetBounds(double input_bounds[6])
 {
   vtkSMNewWidgetRepresentationProxy* widget = this->getWidgetProxy();
-  double input_bounds[6];
-  if (!widget || !this->getReferenceInputBounds(input_bounds))
-    {
-    return;
-    }
-
   vtkBoundingBox box(input_bounds);
 
   double input_origin[3];
@@ -448,8 +442,9 @@ void pqImplicitPlaneWidget::onUseCameraNormal()
   vtkSMNewWidgetRepresentationProxy* widget = this->getWidgetProxy();
   if (widget)
     {
-    if (vtkCamera* const camera = 
-      this->renderView()->getRenderViewProxy()->GetActiveCamera())
+    pqRenderView* renView = qobject_cast<pqRenderView*>(this->renderView());
+    if (vtkCamera* const camera = renView? 
+      renView->getRenderViewProxy()->GetActiveCamera() : 0)
       {
       double camera_normal[3];
       camera->GetViewPlaneNormal(camera_normal);
