@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program:   ParaQ
-   Module:    $RCSfile: pqLineWidget.cxx,v $
+   Module:    pqLineWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -47,6 +47,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <vtkPVDataInformation.h>
 #include <vtkSMDoubleVectorProperty.h>
 #include <vtkSMNewWidgetRepresentationProxy.h>
+#include <vtkSMPropertyHelper.h>
 #include <vtkSMProxyProperty.h>
 #include <vtkSMSourceProxy.h>
 
@@ -339,15 +340,9 @@ void pqLineWidget::createWidget(pqServer* server, const QString& xmlname)
 }
 
 //-----------------------------------------------------------------------------
-void pqLineWidget::resetBounds()
+void pqLineWidget::resetBounds(double bounds[6])
 {
   vtkSMNewWidgetRepresentationProxy* widget = this->getWidgetProxy();
-  double bounds[6];
-  if (!widget || !this->getReferenceInputBounds(bounds))
-    {
-    return;
-    }
-
   if(vtkSMDoubleVectorProperty* const place_widget =
     vtkSMDoubleVectorProperty::SafeDownCast(
       widget->GetProperty("PlaceWidget")))
@@ -402,3 +397,16 @@ void pqLineWidget::onWidgetVisibilityChanged(bool visible)
   this->Implementation->UI.visible->setChecked(visible);
   this->Implementation->UI.visible->blockSignals(false);
 }
+
+//-----------------------------------------------------------------------------
+void pqLineWidget::setLineColor(const QColor& color)
+{
+  vtkSMProxy* widget = this->getWidgetProxy();
+  vtkSMPropertyHelper(widget,
+    "LineColor").Set(0, color.redF());
+ vtkSMPropertyHelper(widget,
+    "LineColor").Set(1,color.greenF());
+ vtkSMPropertyHelper(widget,
+    "LineColor").Set(2 , color.blueF());
+  widget->UpdateVTKObjects(); 
+}
