@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqMultiView.cxx,v $
+   Module:    pqMultiView.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -37,6 +37,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QBuffer>
 #include <QDataStream>
 #include <QLayout>
+#include <QShortcut>
 #include <QSignalMapper>
 #include <QSplitter>
 #include <QString>
@@ -59,7 +60,7 @@ public:
     {
     }
 
-  // overrides default behaviour when this->HideDecorations is true.
+  // overrides default behavior when this->HideDecorations is true.
   virtual QSize sizeHint () const
     {
     return this->HideDecorations? QSize(): QSplitterHandle::sizeHint();
@@ -131,8 +132,19 @@ void pqMultiView::Index::setFromString(const QString& str)
 //-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 pqMultiView::pqMultiView(QWidget* p)
-  : QStackedWidget(p)
+  : Superclass(p)
 {
+  this->FullScreenParent = 0;
+  this->FullScreenWidget = 0;
+
+  QObject::connect(this, SIGNAL(frameAdded(pqMultiViewFrame*)),
+    this, SLOT(updateFrameNames()));
+  QObject::connect(this, SIGNAL(frameRemoved(pqMultiViewFrame*)),
+    this, SLOT(updateFrameNames()));
+  QObject::connect(this, SIGNAL(afterSplitView(const Index&, 
+    Qt::Orientation, float, const Index&)),
+    this, SLOT(updateFrameNames()));
+
   this->SplitterFrame= new QFrame(this);
   this->SplitterFrame->setObjectName("SplitterFrame");
   this->addWidget(this->SplitterFrame);
@@ -163,6 +175,7 @@ pqMultiView::pqMultiView(QWidget* p)
   this->CurrentMaximizedFrame=0;
 }
 
+//-----------------------------------------------------------------------------
 void pqMultiView::init()
 {
   QWidget *w = this->SplitterFrame->layout()->itemAt(0)->widget();
@@ -179,11 +192,12 @@ void pqMultiView::init()
 //-----------------------------------------------------------------------------
 pqMultiView::~pqMultiView()
 {
-  // emit signals for removing frames
   QList<pqMultiViewFrame*> frames = this->findChildren<pqMultiViewFrame*>();
   foreach(pqMultiViewFrame* v, frames)
     {
-    this->removeWidget(v);
+    emit this->preFrameRemoved(qobject_cast<pqMultiViewFrame*>(v));
+    emit this->frameRemoved(qobject_cast<pqMultiViewFrame*>(v));
+    delete v;
     }
 }
 
@@ -403,21 +417,13 @@ pqMultiView::Index pqMultiView::splitView(pqMultiView::Index index,
     {
     // insert new below or on right of existing one
     newFrame = new pqMultiViewFrame;
-    
-    
     QList<int> sizes_old = splitter->sizes();
-
-
     splitter->insertWidget(splitter->indexOf(w)+1, newFrame);
-    
-
     QList<int> sizes = splitter->sizes();
-
     sizes[splitter->indexOf(w)] = static_cast<int>(
       sizes_old[splitter->indexOf(w)]*(1.0-percent));
     sizes[splitter->indexOf(w)+1] = static_cast<int>(
       sizes_old[splitter->indexOf(w)]*percent);
-
     splitter->setSizes(sizes);
     }
     
@@ -494,6 +500,129 @@ QWidget* pqMultiView::widgetOfIndex(Index index)
   return w;
 }
 
+//-----------------------------------------------------------------------------
+//
+// This method computes values of ViewPosition and ViewSize for each
+// frame in the multiview.  The values are computed so that gaps
+// created by splitter handles and view titlebars are eliminated. 
+// This method calls the recursive method computeCompactSizesHelper.
+//
+// ViewPosition is stored as a QPoint and ViewSize is stored as a QSize.
+//
+void pqMultiView::computeCompactSizes(
+  QMap<pqMultiViewFrame*, QPair<QPoint,QSize> >& ViewInfo) const
+{
+  // Call the recursive helper method
+  this->computeCompactSizesHelper(
+    ViewInfo,
+    qobject_cast<QSplitter*>(this->getMultiViewWidget()),
+    0, 0, 0, 0);
+}
+
+//-----------------------------------------------------------------------------
+//
+// A recursive method to compute ViewSize and ViewPosition for each frame
+// in the multiview.  The multiview contains nested splitters which can
+// be thought of as a tree.  The strategy is to keep track of splitter handle
+// locations while visiting each node in depth first order.
+//
+// If there is a splitter handle on the right or bottom, then padding is added
+// to the right or bottom side.  If there is a splitter handle on the left
+// or top, then padding is added on the left or top and ViewPosition is adjusted
+// the same number of pixels as the padding.
+//
+void pqMultiView::computeCompactSizesHelper(
+  QMap<pqMultiViewFrame*, QPair<QPoint,QSize> >& ViewInfo,
+  QSplitter* splitter, int lflag, int rflag, int tflag, int bflag) const
+{
+  if (!splitter) // Check for null just in case
+    {
+    return;
+    }
+
+  // Get info about this splitter
+  int numFrames = splitter->count();
+  int handlePadding = splitter->handleWidth()/2;
+  Qt::Orientation orientation = splitter->orientation();
+
+  // For each widget in the splitter
+  for (int i = 0; i < numFrames; ++i)
+    {
+
+    // This is the logic that tracks splitter handle
+    // locations.  The flags rflag and bflag are
+    // incremented/decremented, while lflag and bflag
+    // are either 1 or 0.
+    if (i == 0)
+      {
+      if (orientation == Qt::Horizontal) rflag++;
+      if (orientation == Qt::Vertical) bflag++;
+      }
+    else if (i == 1)
+      {
+      if (orientation == Qt::Horizontal) lflag=1;
+      if (orientation == Qt::Vertical) tflag=1;
+      }
+    if (i == numFrames-1)
+      {
+      if (orientation == Qt::Horizontal) rflag--;
+      if (orientation == Qt::Vertical) bflag--;
+      }
+
+    // Get the child widget
+    QWidget * child = splitter->widget(i);
+
+    // The widget is either a splitter of a multi view frame
+    QSplitter * childSplitter = qobject_cast<QSplitter*>(child);
+    pqMultiViewFrame * childFrame = qobject_cast<pqMultiViewFrame*>(child);
+
+    if (childSplitter) // It's a splitter
+      {
+      // Recurse!
+      this->computeCompactSizesHelper(ViewInfo, childSplitter, lflag, rflag, tflag, bflag);
+      }
+    else if (childFrame)  // It's a multi view frame
+      {
+      // Get size of frame
+      QSize viewSize = childFrame->size();
+
+      // Get the position of the frame
+      QPoint position =
+        childFrame->mapTo(this->getMultiViewWidget(), QPoint(0,0));
+
+      // Expand size and adjust position as needed
+      if (rflag)
+        {
+        viewSize.rwidth() += handlePadding;
+        }
+      if (bflag)
+        {
+        viewSize.rheight() += handlePadding;
+        }
+      if (lflag)
+        {
+        viewSize.rwidth() += handlePadding;
+        position.rx() -= handlePadding;
+        }
+      if (tflag)
+        {
+        viewSize.rheight() += handlePadding;
+        position.ry() -= handlePadding;
+        }
+
+      // Store the size and position in the map
+      ViewInfo[childFrame] = qMakePair(position, viewSize);
+      }
+    }
+}
+
+//-----------------------------------------------------------------------------
+// Returns the top level splitter widget
+QWidget* pqMultiView::getMultiViewWidget() const
+{
+  return this->SplitterFrame->layout()->itemAt(0)->widget();
+}
+
 //-----------------------------------------------------------------------------
 Qt::Orientation pqMultiView::widgetOrientation(QWidget* _widget) const
 {
@@ -794,16 +923,9 @@ void pqMultiView::cleanSplitter(QSplitter *splitter, QList<QWidget*> &removed)
 void pqMultiView::setup(pqMultiViewFrame* frame)
 {
   Q_ASSERT(frame != NULL);
-  if(!frame)
-    return;
-
-  // Give the frame a name.
-  QString name;
-  pqMultiView::Index index=this->indexOf(frame);
-  if(!index.isEmpty())
+  if (!frame)
     {
-    name.setNum(index.front());
-    frame->setObjectName(name);
+    return;
     }
 
   QSignalMapper* CloseSignalMapper = new QSignalMapper(frame);
@@ -1006,3 +1128,81 @@ QSize pqMultiView::computeSize(QSize client_size) const
   QSize padding = outerSize-innerSize;
   return client_size + padding;
 }
+
+//-----------------------------------------------------------------------------
+void pqMultiView::updateFrameNames()
+{
+  static int name_uniquer=0;
+  QList<pqMultiViewFrame*> frames = this->findChildren<pqMultiViewFrame*>();
+  foreach (pqMultiViewFrame* frame, frames)
+    {
+    QSplitter* parentWdg = qobject_cast<QSplitter*>(frame->parentWidget());
+    if (parentWdg)
+      {
+      frame->setObjectName(QString::number(parentWdg->indexOf(frame)));
+      }
+    else
+      {
+      frame->setObjectName(QString("orphan%1").arg(name_uniquer++));
+      }
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqMultiView::setCurrentWidget(QWidget* cur_widget)
+{
+  if (this->FullScreenParent && cur_widget)
+    {
+    this->Superclass::addWidget(this->FullScreenWidget);
+    this->FullScreenWidget = cur_widget;
+    
+    delete this->FullScreenParent->layout();
+    cur_widget->setParent(this->FullScreenParent);
+    QVBoxLayout* vbl = new QVBoxLayout(this->FullScreenParent);
+    vbl->setMargin(0);
+    vbl->setSpacing(0);
+    vbl->addWidget(cur_widget);
+    cur_widget->show();
+    }
+  else
+    {
+    this->Superclass::setCurrentWidget(cur_widget);
+    }
+}
+
+//-----------------------------------------------------------------------------
+void pqMultiView::toggleFullScreen()
+{
+  if (!this->FullScreenParent)
+    {
+    QWidget* cur_widget = this->Superclass::currentWidget();
+    this->Superclass::removeWidget(cur_widget);
+    this->FullScreenWidget = cur_widget;
+    this->FullScreenParent = new QWidget(this, Qt::Window);
+    cur_widget->setParent(this->FullScreenParent);
+    QVBoxLayout* vbl = new QVBoxLayout(this->FullScreenParent);
+    vbl->setMargin(0);
+    vbl->setSpacing(0);
+    vbl->addWidget(cur_widget);
+    cur_widget->show();
+
+    QShortcut *esc= new QShortcut(Qt::Key_Escape, this->FullScreenParent);
+    QObject::connect(esc, SIGNAL(activated()), this, SLOT(toggleFullScreen()));
+    QShortcut *f11= new QShortcut(Qt::Key_F11, this->FullScreenParent);
+    QObject::connect(f11, SIGNAL(activated()), this, SLOT(toggleFullScreen()));
+
+    this->FullScreenParent->showFullScreen();
+    this->FullScreenParent->show();
+    }
+  else
+    {
+    this->FullScreenParent->hide();
+    this->FullScreenWidget->hide();
+    this->Superclass::addWidget(this->FullScreenWidget);
+    this->Superclass::setCurrentWidget(this->FullScreenWidget);
+    delete this->FullScreenParent;
+    this->FullScreenParent = 0;
+    this->FullScreenWidget = 0;
+    }
+}
+
