@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqNamedWidgets.cxx,v $
+   Module:    pqNamedWidgets.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -49,6 +49,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QLabel>
 #include <QMetaObject>
 #include <QMetaProperty>
+#include <QHeaderView>
 
 // VTK includes
 
@@ -64,33 +65,38 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMStringListDomain.h"
 #include "vtkSMIntVectorProperty.h"
+#include "vtkSMSILDomain.h"
 
 // ParaView includes
 #include "pq3DWidget.h"
 #include "pqApplicationCore.h"
 #include "pqCollapsedGroup.h"
 #include "pqComboBoxDomain.h"
-#include "pqSignalAdaptorCompositeTreeWidget.h"
 #include "pqDoubleRangeWidget.h"
-#include "pqIntRangeWidget.h"
-#include "pqWidgetRangeDomain.h"
 #include "pqFieldSelectionAdaptor.h"
 #include "pqFileChooserWidget.h"
+#include "pqIntRangeWidget.h"
 #include "pqListWidgetItemObject.h"
 #include "pqObjectPanel.h"
 #include "pqPipelineFilter.h"
 #include "pqPropertyManager.h"
 #include "pqProxySelectionWidget.h"
+#include "pqProxySILModel.h"
+#include "pqSelectionInputWidget.h"
 #include "pqServerManagerModel.h"
 #include "pqServerManagerObserver.h"
+#include "pqSignalAdaptorCompositeTreeWidget.h"
 #include "pqSignalAdaptorSelectionTreeWidget.h"
 #include "pqSignalAdaptors.h"
+#include "pqSILModel.h"
+#include "pqSILWidget.h"
 #include "pqSMAdaptor.h"
 #include "pqSMProxy.h"
 #include "pqSMSignalAdaptors.h"
+#include "pqTreeView.h"
 #include "pqTreeWidget.h"
-#include "pqTreeWidgetItemObject.h"
 #include "pqTreeWidgetSelectionHelper.h"
+#include "pqWidgetRangeDomain.h"
 
 //-----------------------------------------------------------------------------
 void pqNamedWidgets::link(QWidget* parent, pqSMProxy proxy, 
@@ -257,6 +263,22 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         proxyAdaptor, "proxy", SIGNAL(proxyChanged(const QVariant&)),
         proxy, SMProperty);
       }
+
+    pqSelectionInputWidget *selectWidget
+      = qobject_cast<pqSelectionInputWidget*>(object);
+    if (selectWidget)
+      {
+      QString userProperty, userSignal;
+      if (pqNamedWidgets::propertyInformation(object, userProperty, userSignal))
+        {
+        pqNamedWidgets::linkObject(object, userProperty, userSignal,
+                                   proxy, SMProperty, -1, property_manager);
+        QObject::connect(property_manager, SIGNAL(aboutToAccept()),
+          selectWidget, SLOT(preAccept()));
+        QObject::connect(property_manager, SIGNAL(accepted()),
+          selectWidget, SLOT(postAccept()));
+        }
+      }
     }
   else if (pt == pqSMAdaptor::PROXYSELECTION)
     {
@@ -383,6 +405,16 @@ void pqNamedWidgets::linkObject(QObject* object, pqSMProxy proxy,
         proxy, SMProperty);
       }
     }
+  else if (pt == pqSMAdaptor::SIL)
+    {
+    pqSILWidget* tree = qobject_cast<pqSILWidget*>(object);
+    if (tree)
+      {
+      property_manager->registerLink(
+        tree->activeModel(), "values", SIGNAL(valuesChanged()),
+        proxy, SMProperty);
+      }
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -778,9 +810,10 @@ static QLabel* createPanelLabel(QWidget* parent, QString text, QString pname)
 }
 
 //-----------------------------------------------------------------------------
-void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
-                                               vtkSMProxy* pxy)
+void pqNamedWidgets::createWidgets(QGridLayout* panelLayout, vtkSMProxy* pxy)
 {
+  bool row_streched = false; // when set, the extra setRowStretch() at the end
+                             // is skipped
   int rowCount = 0;
   int skippedFirstFileProperty = 0;
   bool isCompoundProxy = pxy->IsA("vtkSMCompoundSourceProxy");
@@ -879,6 +912,8 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
     pqSMAdaptor::PropertyType pt = pqSMAdaptor::getPropertyType(SMProperty);
     QList<QString> domainsTypes = pqSMAdaptor::getDomainTypes(SMProperty);
 
+    vtkPVXMLElement *hints = SMProperty->GetHints();
+
     // skip input properties
     if(pt == pqSMAdaptor::PROXY || pt == pqSMAdaptor::PROXYLIST)
       {
@@ -890,19 +925,31 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
 
     if(pt == pqSMAdaptor::PROXY)
       {
-      // create a combo box with list of proxies
-      QComboBox* combo = new QComboBox(panelLayout->parentWidget());
-      if(informationOnly)
+      if (hints && hints->FindNestedElementByName("SelectionInput"))
         {
-        combo->setEnabled(false);
+        // Create a widget for grabbing the current selection.
+        pqSelectionInputWidget *selectWidget
+          = new pqSelectionInputWidget(panelLayout->parentWidget());
+        selectWidget->setObjectName(propertyName);
+        panelLayout->addWidget(selectWidget, rowCount, 0, 1, -1);
+        rowCount++;
+        }
+      else
+        {
+        // create a combo box with list of proxies
+        QComboBox* combo = new QComboBox(panelLayout->parentWidget());
+        if(informationOnly)
+          {
+          combo->setEnabled(false);
+          }
+        combo->setObjectName(propertyName);
+        QLabel* label = createPanelLabel(panelLayout->parentWidget(),
+                                         propertyLabel,
+                                         propertyName);
+        panelLayout->addWidget(label, rowCount, 0, 1, 1);
+        panelLayout->addWidget(combo, rowCount, 1, 1, 1);
+        rowCount++;
         }
-      combo->setObjectName(propertyName);
-      QLabel* label = createPanelLabel(panelLayout->parentWidget(),
-                                       propertyLabel,
-                                       propertyName);
-      panelLayout->addWidget(label, rowCount, 0, 1, 1);
-      panelLayout->addWidget(combo, rowCount, 1, 1, 1);
-      rowCount++;
       }
     else if(pt == pqSMAdaptor::PROXYLIST)
       {
@@ -1096,7 +1143,6 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         {
         //  what entry widget we should use for a stringvectorproperty
         bool multiLineString = false;
-        vtkPVXMLElement* hints = SMProperty->GetHints();
         if (hints)
           {
           vtkPVXMLElement* widgetHint = hints->FindNestedElementByName("Widget");
@@ -1153,6 +1199,7 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
 
           panelLayout->addWidget(textEdit, rowCount, 0, 1, 2);
           panelLayout->setRowStretch(rowCount, 1);
+          row_streched = true;
           rowCount++;
           }
         }
@@ -1293,6 +1340,15 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
         {
         chooser->setEnabled(false);
         }
+     
+      // If there's a hint on the property indicating that this property expects a
+      // directory name, then, we will use the directory mode.
+      if (SMProperty->IsA("vtkSMStringVectorProperty") && SMProperty->GetHints() && 
+          SMProperty->GetHints()->FindNestedElementByName("UseDirectoryName"))
+        {
+        chooser->setUseDirectoryMode(1);
+        }
+        
       pqServerManagerModel* m =
         pqApplicationCore::instance()->getServerManagerModel();
       chooser->setServer(m->findServer(pxy->GetConnectionID()));
@@ -1308,7 +1364,7 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
     else if(pt == pqSMAdaptor::FIELD_SELECTION)
       {
       QLabel* label = createPanelLabel(panelLayout->parentWidget(),
-                                       "Scalars",
+                                       propertyLabel,
                                        propertyName);
       QComboBox* combo = new QComboBox(panelLayout->parentWidget());
       if(informationOnly)
@@ -1333,11 +1389,36 @@ void pqNamedWidgets::createWidgets(QGridLayout* panelLayout,
       header->setData(0, Qt::DisplayRole, propertyLabel);
       tree->setHeaderItem(header);
       panelLayout->addWidget(tree, rowCount, 0, 1, 2); 
+      panelLayout->setRowStretch(rowCount, 1);
+      row_streched = true;
+      rowCount++;
+      }
+    else if (pt == pqSMAdaptor::SIL)
+      {
+      vtkSMSILDomain* silDomain = vtkSMSILDomain::SafeDownCast(
+        SMProperty->GetDomain("array_list"));
+
+      pqSILWidget* tree = new pqSILWidget( 
+        silDomain->GetSubtree(), panelLayout->parentWidget());
+      tree->setObjectName(propertyName);
+      
+      pqSILModel* silModel = new pqSILModel(tree);
+      
+      // FIXME: This needs to be automated, we want the model to automatically
+      // fetch the SIL when the domain is updated.
+      silModel->update(silDomain->GetSIL());
+      tree->setModel(silModel);
+      panelLayout->addWidget(tree, rowCount, 0, 1, 2); 
+      panelLayout->setRowStretch(rowCount, 1);
+      row_streched = true;
       rowCount++;
       }
     }
   iter->Delete();
-  panelLayout->setRowStretch(rowCount, 1);
+  if (!row_streched)
+    {
+    panelLayout->setRowStretch(rowCount, 1);
+    }
   panelLayout->invalidate();
 }
 
@@ -1387,6 +1468,17 @@ bool pqNamedWidgets::propertyInformation(QObject* object,
      return true;
      }
 
+  // QMetaMethod notifySignal () const is available in Qt 4.5 
+  // This will make determining the signal name easier and more generic.
+  // and the QGroupBox code will not be needed.
+  QGroupBox* grp = qobject_cast<QGroupBox*>(object);
+  if(grp && grp->isCheckable())
+     {
+     property = "checked";
+     signal = SIGNAL(toggled(bool));
+     return true;
+     }
+
   return false;
 }
 
