@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqProxyInformationWidget.cxx,v $
+   Module:    pqProxyInformationWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -139,6 +139,9 @@ void pqProxyInformationWidget::updateInformation()
   this->Ui->filename->setText(tr("NA"));
   this->Ui->filename->setToolTip(tr("NA"));
   this->Ui->filename->setStatusTip(tr("NA"));
+  this->Ui->path->setText(tr("NA"));
+  this->Ui->path->setToolTip(tr("NA"));
+  this->Ui->path->setStatusTip(tr("NA"));
 
   vtkPVDataInformation* dataInformation = NULL;
   pqPipelineSource* source = NULL;
@@ -147,7 +150,7 @@ void pqProxyInformationWidget::updateInformation()
     source = this->OutputPort->getSource();
     if (this->OutputPort->getOutputPortProxy())
       {
-      dataInformation = this->OutputPort->getDataInformation(/*update=*/false);
+      dataInformation = this->OutputPort->getDataInformation();
       }
     }
 
@@ -186,13 +189,25 @@ void pqProxyInformationWidget::updateInformation()
         {
         if (diter->GetDomain()->IsA("vtkSMFileListDomain"))
           {
+          vtkSMProperty* smprop = piter->GetProperty();
+          if (smprop->GetInformationProperty())
+            {
+            smprop = smprop->GetInformationProperty();
+            source->getProxy()->UpdatePropertyInformation(smprop);
+            }
           QString filename = 
-            pqSMAdaptor::getElementProperty(piter->GetProperty()).toString();
+            pqSMAdaptor::getElementProperty(smprop).toString();
+          QString path = vtksys::SystemTools::GetFilenamePath(
+            filename.toAscii().data()).c_str();
+
           this->Ui->properties->show();
           this->Ui->filename->setText(vtksys::SystemTools::GetFilenameName(
               filename.toAscii().data()).c_str());
           this->Ui->filename->setToolTip(filename);
           this->Ui->filename->setStatusTip(filename);
+          this->Ui->path->setText(path);
+          this->Ui->path->setToolTip(path);
+          this->Ui->path->setStatusTip(path);
           break;
           }
         }
@@ -236,6 +251,8 @@ void pqProxyInformationWidget::fillDataInformation(
   this->Ui->type->setText(tr("NA"));
   this->Ui->numberOfCells->setText(tr("NA"));
   this->Ui->numberOfPoints->setText(tr("NA"));
+  this->Ui->numberOfRows->setText(tr("NA"));
+  this->Ui->numberOfColumns->setText(tr("NA"));
   this->Ui->memory->setText(tr("NA"));
   
   this->Ui->dataArrays->clear();
@@ -261,9 +278,27 @@ void pqProxyInformationWidget::fillDataInformation(
   
   QString numPoints = QString("%1").arg(dataInformation->GetNumberOfPoints());
   this->Ui->numberOfPoints->setText(numPoints);
+
+  QString numRows = QString("%1").arg(dataInformation->GetNumberOfRows());
+  this->Ui->numberOfRows->setText(numRows);
+
+  QString numColumns =
+    QString("%1").arg(dataInformation->GetRowDataInformation()->GetNumberOfArrays());
+  this->Ui->numberOfColumns->setText(numColumns);
+
+  if (dataInformation->GetDataSetType() == VTK_TABLE)
+    {
+    this->Ui->dataTypeProperties->setCurrentWidget(
+      this->Ui->Table);
+    }
+  else
+    {
+    this->Ui->dataTypeProperties->setCurrentWidget(
+      this->Ui->DataSet);
+    }
   
   QString memory = QString("%1 MB").arg(dataInformation->GetMemorySize()/1000.0,
-                                     0, 'e', 2);
+                                     0, 'g', 2);
   this->Ui->memory->setText(memory);
 
   if (dataInformation->GetHasTime())
@@ -273,14 +308,22 @@ void pqProxyInformationWidget::fillDataInformation(
       QString("Current data time step: <b>%1</b>").arg(dataInformation->GetTime()));
     }
 
-  vtkPVDataSetAttributesInformation* info[2];
+  vtkPVDataSetAttributesInformation* info[6];
   info[0] = dataInformation->GetPointDataInformation();
   info[1] = dataInformation->GetCellDataInformation();
+  info[2] = dataInformation->GetVertexDataInformation();
+  info[3] = dataInformation->GetEdgeDataInformation();
+  info[4] = dataInformation->GetRowDataInformation();
+  info[5] = dataInformation->GetFieldDataInformation();
 
-  QPixmap pixmaps[2] = 
+  QPixmap pixmaps[6] = 
     {
     QPixmap(":/pqWidgets/Icons/pqPointData16.png"),
-    QPixmap(":/pqWidgets/Icons/pqCellData16.png")
+    QPixmap(":/pqWidgets/Icons/pqCellData16.png"),
+    QPixmap(":/pqWidgets/Icons/pqPointData16.png"),
+    QPixmap(":/pqWidgets/Icons/pqCellData16.png"),
+    QPixmap(":/pqWidgets/Icons/pqSpreadsheet16.png"),
+    QPixmap(":/pqWidgets/Icons/pqGlobalData16.png")
     };
 
   if (dataInformation->IsDataStructured())
@@ -310,7 +353,7 @@ void pqProxyInformationWidget::fillDataInformation(
       }
     }
 
-  for(int k=0; k<2; k++)
+  for(int k=0; k<6; k++)
     {
     if(info[k])
       {
@@ -367,9 +410,9 @@ void pqProxyInformationWidget::fillDataInformation(
   else
     {
     xrange = QString("%1 to %2 (delta: %3)");
-    xrange = xrange.arg(bounds[0], -1, 'f', 3);
-    xrange = xrange.arg(bounds[1], -1, 'f', 3);
-    xrange = xrange.arg(bounds[1] - bounds[0], -1, 'f', 3);
+    xrange = xrange.arg(bounds[0], -1, 'g', 3);
+    xrange = xrange.arg(bounds[1], -1, 'g', 3);
+    xrange = xrange.arg(bounds[1] - bounds[0], -1, 'g', 3);
     }
   this->Ui->xRange->setText(xrange);
 
@@ -381,9 +424,9 @@ void pqProxyInformationWidget::fillDataInformation(
   else
     {
     yrange = QString("%1 to %2 (delta: %3)");
-    yrange = yrange.arg(bounds[2], -1, 'f', 3);
-    yrange = yrange.arg(bounds[3], -1, 'f', 3);
-    yrange = yrange.arg(bounds[3] - bounds[2], -1, 'f', 3);
+    yrange = yrange.arg(bounds[2], -1, 'g', 3);
+    yrange = yrange.arg(bounds[3], -1, 'g', 3);
+    yrange = yrange.arg(bounds[3] - bounds[2], -1, 'g', 3);
     }
   this->Ui->yRange->setText(yrange);
 
@@ -395,9 +438,9 @@ void pqProxyInformationWidget::fillDataInformation(
   else
     {
     zrange = QString("%1 to %2 (delta: %3)");
-    zrange = zrange.arg(bounds[4], -1, 'f', 3);
-    zrange = zrange.arg(bounds[5], -1, 'f', 3);
-    zrange = zrange.arg(bounds[5] - bounds[4], -1, 'f', 3);
+    zrange = zrange.arg(bounds[4], -1, 'g', 3);
+    zrange = zrange.arg(bounds[5], -1, 'g', 3);
+    zrange = zrange.arg(bounds[5] - bounds[4], -1, 'g', 3);
     }
   this->Ui->zRange->setText(zrange);
 }
