@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqSelectionManager.h,v $
+   Module:    pqSelectionManager.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -108,11 +108,9 @@ public slots:
   void select(pqOutputPort*);
 
 private slots:
-  /// Called when the view fires signal indicating that is has made a selection.
-  void onSelected(pqOutputPort*);
-
   /// Called when server manager item is being deleted.
   void onItemRemoved(pqServerManagerModelItem* item);
+
 private:
   pqSelectionManagerImplementation* Implementation;
 
