@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqSignalAdaptorCompositeTreeWidget.cxx,v $
+   Module:    pqSignalAdaptorCompositeTreeWidget.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -49,35 +49,135 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QtDebug>
 
 // ParaView Includes.
-#include "pqTreeWidgetItemObject.h"
+#include "pqTreeWidgetItem.h"
 #include "pqSMAdaptor.h"
 
-// This TreeItem specialization needs some explaination.
-// Default Qt behaviour for tristate items:
+//-----------------------------------------------------------------------------
+class pqSignalAdaptorCompositeTreeWidget::pqInternal
+{
+public:
+  QPointer<QTreeWidget> TreeWidget;
+  vtkSmartPointer<vtkSMIntVectorProperty> Property;
+  vtkSmartPointer<vtkSMOutputPort> OutputPort;
+  vtkSmartPointer<vtkSMCompositeTreeDomain> Domain;
+  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnect;
+  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnectSelection;
+
+  QList<pqTreeWidgetItem*> Items;
+  int DomainMode;
+};
+
+//-----------------------------------------------------------------------------
+class pqSignalAdaptorCompositeTreeWidget::pqCallbackAdaptor :
+  public pqTreeWidgetItem::pqCallbackHandler
+{
+  int ChangeDepth;// used to avoid repeated valuesChanged() signal firing when
+                  // check states are being changed as a consequence of toggle
+                  // the check state for a non-leaf node.
+  bool BlockCallbacks;
+  bool PrevBlockSignals;
+  pqSignalAdaptorCompositeTreeWidget* Adaptor;
+public:
+  pqCallbackAdaptor(pqSignalAdaptorCompositeTreeWidget* adaptor)
+    {
+    this->Adaptor = adaptor;
+    this->BlockCallbacks = false;
+    this->ChangeDepth = 0;
+    }
+  bool blockCallbacks(bool val)
+    {
+    bool old = this->BlockCallbacks;
+    this->BlockCallbacks = val;
+    return old;
+    }
+  virtual void checkStateChanged(pqTreeWidgetItem* item, int column)
+    {
+    // When a non-leaf node is toggled, Qt changes the check state of all child
+    // nodes. When that's happening we don't want to repeatedly fire
+    // valuesChanged() signal. Hence we use this ChangeDepth magic.
+    this->ChangeDepth--;
+    if (this->ChangeDepth == 0)
+      {
+      this->Adaptor->blockSignals(this->PrevBlockSignals);
+      }
+    if (this->BlockCallbacks == false)
+      {
+      this->Adaptor->updateCheckState(item, column);
+      }
+    }
+
+  virtual void checkStateAboutToChange(
+    pqTreeWidgetItem* /*item*/, int /*column*/)
+    {
+    // When a non-leaf node is toggled, Qt changes the check state of all child
+    // nodes. When that's happening we don't want to repeatedly fire
+    // valuesChanged() signal. Hence we use this ChangeDepth magic.
+    this->ChangeDepth++;
+    if (this->ChangeDepth == 1)
+      {
+      this->PrevBlockSignals = this->Adaptor->blockSignals(true);
+      }
+    }
+
+  virtual bool acceptChange(pqTreeWidgetItem* item,
+    const QVariant& curValue, const QVariant& newValue, int column, int role)
+    {
+    if (this->BlockCallbacks == false)
+      {
+      // * In SINGLE_ITEM, it's an error to not have any item checked. Hence if
+      // the user is un-checking an item, we ensure that atleast one other item
+      // is checked.
+      if (this->Adaptor->CheckMode == pqSignalAdaptorCompositeTreeWidget::SINGLE_ITEM &&
+        role == Qt::CheckStateRole && curValue.toInt() == Qt::Checked &&
+        newValue.toInt() == Qt::Unchecked && 
+        (item->flags() & Qt::ItemIsTristate) == 0)
+        {
+        // ensure that at least one item is always checked.
+        foreach (pqTreeWidgetItem* curitem, this->Adaptor->Internal->Items)
+          {
+          if (item != curitem && curitem->checkState(column) == Qt::Checked)
+            {
+            return true;
+            }
+          }
+        return false;
+        }
+      }
+    return true;
+    }
+};
+
+// This TreeItem specialization needs some explanation.
+// Default Qt behavior for tristate items:
 //   - If all immediate children are checked or partially checked
 //     then the item becomes fully checked.
-// This is not approriate for this widget. A parent item should never be fully
-// checked unless the user explicitly checked it, since otherwise, the behaviour
+// This is not appropriate for this widget. A parent item should never be fully
+// checked unless the user explicitly checked it, since otherwise, the behavior
 // of the filter is to pass the entire subtree through. 
 // This class fixes that issue.
-class pqCompositeTreeWidgetItem : public pqTreeWidgetItemObject
+class pqCompositeTreeWidgetItem : public pqTreeWidgetItem
 {
-  typedef pqTreeWidgetItemObject Superclass;
+  typedef pqTreeWidgetItem Superclass;
   int triStateCheckState;
+  bool inSetData;
 public:
   pqCompositeTreeWidgetItem(QTreeWidget* tree, QStringList _values):
-    Superclass(tree, _values), triStateCheckState(Qt::Unchecked)
+    Superclass(tree, _values), triStateCheckState(-1),
+    inSetData(false)
+
   {
   }
 
   pqCompositeTreeWidgetItem(QTreeWidgetItem* item, QStringList _values):
-    Superclass(item, _values), triStateCheckState(Qt::Unchecked)
+    Superclass(item, _values), triStateCheckState(-1),
+    inSetData(false)
   {
   }
 
   virtual QVariant data(int column, int role) const
     {
     if (role == Qt::CheckStateRole && 
+      this->triStateCheckState != -1 &&
       this->childCount() > 0 && (this->flags() & Qt::ItemIsTristate))
       {
       // superclass implementation of this method only checks the immediate
@@ -95,17 +195,22 @@ public:
         }
       return this->triStateCheckState;
       }
-    return this->Superclass::data(column, role);
 
+    return this->Superclass::data(column, role);
     }
 
-  virtual void setData(int column, int role, const QVariant &value)
+  virtual void setData(int column, int role, const QVariant &in_value)
     {
+    this->inSetData = true;
+    this->triStateCheckState = -1;
+
     // Superclass will also mark all children checked or unchecked.
-    this->Superclass::setData(column, role, value);
+    this->Superclass::setData(column, role, in_value);
 
     if (role == Qt::CheckStateRole && column==0)
       {
+      QVariant value = this->data(column, role);
+
       if (this->flags() & Qt::ItemIsTristate)
         {
         this->triStateCheckState = value.toInt();
@@ -115,7 +220,7 @@ public:
       pqCompositeTreeWidgetItem* itemParent = 
         dynamic_cast<pqCompositeTreeWidgetItem*>(
           static_cast<QTreeWidgetItem*>(this)->parent());
-      while (itemParent)
+      while (itemParent && !itemParent->inSetData)
         {
         itemParent->triStateCheckState = Qt::PartiallyChecked;
         itemParent = 
@@ -123,26 +228,12 @@ public:
             static_cast<QTreeWidgetItem*>(itemParent)->parent());
         }
       }
+    this->inSetData = false;
     }
 
 
 };
 
-//-----------------------------------------------------------------------------
-class pqSignalAdaptorCompositeTreeWidget::pqInternal
-{
-public:
-  QPointer<QTreeWidget> TreeWidget;
-  vtkSmartPointer<vtkSMIntVectorProperty> Property;
-  vtkSmartPointer<vtkSMOutputPort> OutputPort;
-  vtkSmartPointer<vtkSMCompositeTreeDomain> Domain;
-  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnect;
-  vtkSmartPointer<vtkEventQtSlotConnect> VTKConnectSelection;
-
-  QList<pqTreeWidgetItemObject*> Items;
-  int DomainMode;
-};
-
 //-----------------------------------------------------------------------------
 void pqSignalAdaptorCompositeTreeWidget::constructor(
   QTreeWidget* tree, bool autoUpdateVisibility)
@@ -157,9 +248,11 @@ void pqSignalAdaptorCompositeTreeWidget::constructor(
   this->CheckMode = SINGLE_ITEM;
   this->IndexMode = INDEX_MODE_FLAT;
 
-  this->ShowIndex = false;
+  this->ShowFlatIndex = false;
   this->ShowDatasetsInMultiPiece = false;
   this->ShowSelectedElementCounts = false;
+
+  this->CallbackAdaptor = new pqCallbackAdaptor(this);
 }
 
 //-----------------------------------------------------------------------------
@@ -224,10 +317,10 @@ pqSignalAdaptorCompositeTreeWidget::pqSignalAdaptorCompositeTreeWidget(
     }
 
   // * Initialize the widget using the current value.
-  bool prev = this->Internal->TreeWidget->blockSignals(true);
+  bool prev = this->blockSignals(true);
   QList<QVariant> curValues = pqSMAdaptor::getMultipleElementProperty(smproperty);
   this->setValues(curValues);
-  this->Internal->TreeWidget->blockSignals(prev);
+  this->blockSignals(prev);
 }
 
 //-----------------------------------------------------------------------------
@@ -248,7 +341,7 @@ pqSignalAdaptorCompositeTreeWidget::pqSignalAdaptorCompositeTreeWidget(
     return;
     }
 
-  this->ShowIndex = true;
+  this->ShowFlatIndex = true;
   this->ShowDatasetsInMultiPiece = true;
   this->ShowSelectedElementCounts = showSelectedElementCounts;
   this->CheckMode = selectMultiple? MULTIPLE_ITEMS : SINGLE_ITEM;
@@ -265,21 +358,23 @@ pqSignalAdaptorCompositeTreeWidget::pqSignalAdaptorCompositeTreeWidget(
 pqSignalAdaptorCompositeTreeWidget::~pqSignalAdaptorCompositeTreeWidget()
 {
   delete this->Internal;
+  delete this->CallbackAdaptor;
+  this->CallbackAdaptor = 0;
 }
 
 //-----------------------------------------------------------------------------
-void pqSignalAdaptorCompositeTreeWidget::select(unsigned int flatIndex)
+void pqSignalAdaptorCompositeTreeWidget::select(unsigned int flat_index)
 {
   QList<QTreeWidgetItem*> selItems = this->Internal->TreeWidget->selectedItems();
   foreach (QTreeWidgetItem* item, selItems)
     {
     item->setSelected(false);
     }
-  QList<pqTreeWidgetItemObject*> treeitems = this->Internal->Items; 
-  foreach (pqTreeWidgetItemObject* item, treeitems)
+  QList<pqTreeWidgetItem*> treeitems = this->Internal->Items; 
+  foreach (pqTreeWidgetItem* item, treeitems)
     {
     QVariant metadata = item->data(0, FLAT_INDEX);
-    if (metadata.isValid() && metadata.toUInt() == flatIndex)
+    if (metadata.isValid() && metadata.toUInt() == flat_index)
       {
       item->setSelected(true);
       break;
@@ -302,18 +397,49 @@ unsigned int pqSignalAdaptorCompositeTreeWidget::getCurrentFlatIndex(bool* valid
       {
       *valid = true;
       }
-    return selItems[0]->data(0, FLAT_INDEX).toUInt();
+    return this->flatIndex(selItems[0]);
     }
   return 0;
 }
 
+//-----------------------------------------------------------------------------
+unsigned int pqSignalAdaptorCompositeTreeWidget::flatIndex(
+  const QTreeWidgetItem* item) const
+{
+  return item->data(0, FLAT_INDEX).toUInt();
+}
+
+//-----------------------------------------------------------------------------
+unsigned int pqSignalAdaptorCompositeTreeWidget::hierarchicalLevel(
+  const QTreeWidgetItem* item) const
+{
+  QVariant val = item->data(0, AMR_LEVEL_NUMBER).toUInt();
+  return val.toUInt();
+}
+
+//-----------------------------------------------------------------------------
+unsigned int pqSignalAdaptorCompositeTreeWidget::hierarchicalBlockIndex(
+  const QTreeWidgetItem* item) const
+{
+  QVariant val = item->data(0, AMR_BLOCK_INDEX).toUInt();
+  return val.toUInt();
+}
+
+//-----------------------------------------------------------------------------
+QString pqSignalAdaptorCompositeTreeWidget::blockName(
+  const QTreeWidgetItem* item) const
+{
+  QString block_name = item->data(0, BLOCK_NAME).toString();
+  return block_name;
+}
+
 //-----------------------------------------------------------------------------
 QList<QVariant> pqSignalAdaptorCompositeTreeWidget::values() const
 {
   QList<QVariant> reply;
 
-  QList<pqTreeWidgetItemObject*> treeitems = this->Internal->Items; 
-  foreach (pqTreeWidgetItemObject* item, treeitems)
+  QList<pqTreeWidgetItem*> treeitems = this->Internal->Items; 
+  foreach (pqTreeWidgetItem* item, treeitems)
     {
     QVariant nodeType = item->data(0, NODE_TYPE);
     if (!nodeType.isValid())
@@ -333,6 +459,7 @@ QList<QVariant> pqSignalAdaptorCompositeTreeWidget::values() const
     if (this->IndexMode == INDEX_MODE_FLAT)
       {
       QVariant metadata = item->data(0, FLAT_INDEX);
+      //cout << metadata.toInt()  << ": " << item->checkState(0) << endl;
       if (metadata.isValid() && item->checkState(0)== Qt::Checked)
         {
         // metadata has the flat index for the node.
@@ -342,8 +469,8 @@ QList<QVariant> pqSignalAdaptorCompositeTreeWidget::values() const
       }
     else if (this->IndexMode == INDEX_MODE_LEVEL_INDEX)
       {
-      QVariant metadata0 = item->data(0, LEVEL_NUMBER);
-      QVariant metadata1 = item->data(0, DATASET_INDEX);
+      QVariant metadata0 = item->data(0, AMR_LEVEL_NUMBER);
+      QVariant metadata1 = item->data(0, AMR_BLOCK_INDEX);
       if (metadata0.isValid() && metadata1.isValid() && item->checkState(0) == Qt::Checked)
         {
         reply.push_back(metadata0);
@@ -353,7 +480,7 @@ QList<QVariant> pqSignalAdaptorCompositeTreeWidget::values() const
       }
     else if (this->IndexMode == INDEX_MODE_LEVEL)
       {
-      QVariant metadata0 = item->data(0, LEVEL_NUMBER);
+      QVariant metadata0 = item->data(0, AMR_LEVEL_NUMBER);
       if (metadata0.isValid() && item->checkState(0) == Qt::Checked)
         {
         reply.push_back(metadata0);
@@ -374,13 +501,13 @@ inline bool pqItemIsCheckable(QTreeWidgetItem* item)
 //-----------------------------------------------------------------------------
 void pqSignalAdaptorCompositeTreeWidget::setValues(const QList<QVariant>& new_values)
 {
-  bool prev = this->Internal->TreeWidget->blockSignals(true);
-  QList<pqTreeWidgetItemObject*> treeitems = this->Internal->Items; 
+  bool prev = this->blockSignals(true);
+  QList<pqTreeWidgetItem*> treeitems = this->Internal->Items; 
   bool changed = false;
 
   if (this->IndexMode == INDEX_MODE_FLAT)
     {
-    foreach (pqTreeWidgetItemObject* item, treeitems)
+    foreach (pqTreeWidgetItem* item, treeitems)
       {
       QVariant metadata = item->data(0, FLAT_INDEX);
       Qt::CheckState cstate = 
@@ -395,9 +522,9 @@ void pqSignalAdaptorCompositeTreeWidget::setValues(const QList<QVariant>& new_va
     }
   else if (this->IndexMode == INDEX_MODE_LEVEL)
     {
-    foreach (pqTreeWidgetItemObject* item, treeitems)
+    foreach (pqTreeWidgetItem* item, treeitems)
       {
-      QVariant metadata = item->data(0, LEVEL_NUMBER);
+      QVariant metadata = item->data(0, AMR_LEVEL_NUMBER);
       Qt::CheckState cstate = 
         (metadata.isValid() && new_values.contains(metadata))? 
         Qt::Checked : Qt::Unchecked;
@@ -418,10 +545,10 @@ void pqSignalAdaptorCompositeTreeWidget::setValues(const QList<QVariant>& new_va
       pairs.insert(QPair<unsigned int, unsigned int>(level, index));
       }
 
-    foreach (pqTreeWidgetItemObject* item, treeitems)
+    foreach (pqTreeWidgetItem* item, treeitems)
       {
-      QVariant metadata0 = item->data(0, LEVEL_NUMBER);
-      QVariant metadata1 = item->data(0, DATASET_INDEX);
+      QVariant metadata0 = item->data(0, AMR_LEVEL_NUMBER);
+      QVariant metadata1 = item->data(0, AMR_BLOCK_INDEX);
       Qt::CheckState cstate = (metadata0.isValid() && metadata1.isValid() &&
         pairs.contains(QPair<unsigned int, unsigned int>(metadata0.toUInt(), metadata1.toUInt())))?
         Qt::Checked : Qt::Unchecked;
@@ -432,7 +559,7 @@ void pqSignalAdaptorCompositeTreeWidget::setValues(const QList<QVariant>& new_va
         }
       }
     }
-  this->Internal->TreeWidget->blockSignals(prev);
+  this->blockSignals(prev);
   if (changed)
     {
     emit this->valuesChanged();
@@ -442,7 +569,7 @@ void pqSignalAdaptorCompositeTreeWidget::setValues(const QList<QVariant>& new_va
 //-----------------------------------------------------------------------------
 void pqSignalAdaptorCompositeTreeWidget::domainChanged()
 {
-  bool prev = this->Internal->TreeWidget->blockSignals(true);
+  bool prev = this->blockSignals(true);
   QList<QVariant> widgetvalues = this->values();
   this->Internal->Items.clear();
   this->Internal->TreeWidget->clear();
@@ -453,9 +580,11 @@ void pqSignalAdaptorCompositeTreeWidget::domainChanged()
   this->FlatIndex = 0;
   this->LevelNo = 0;
   
-  pqTreeWidgetItemObject* root = new pqCompositeTreeWidgetItem(
+  pqTreeWidgetItem* root = new pqCompositeTreeWidgetItem(
     this->Internal->TreeWidget, QStringList("Root"));
+  root->setCallbackHandler(this->CallbackAdaptor);
   root->setData(0, ORIGINAL_LABEL, "Root");
+  root->setData(0, BLOCK_NAME, QString());
   root->setToolTip(0, root->text(0));
   this->buildTree(root, dInfo);
   this->updateItemFlags();
@@ -463,12 +592,13 @@ void pqSignalAdaptorCompositeTreeWidget::domainChanged()
   
   // now update check state.
   this->setValues(widgetvalues);
-  this->Internal->TreeWidget->blockSignals(prev);
+  this->blockSignals(prev);
 
   if (this->AutoUpdateWidgetVisibility)
     {
     this->Internal->TreeWidget->setVisible(
-      dInfo->GetCompositeDataInformation()->GetDataIsComposite()==1);
+      dInfo?
+      dInfo->GetCompositeDataInformation()->GetDataIsComposite()==1 : 0);
     }
 
   if (this->ShowSelectedElementCounts)
@@ -486,7 +616,7 @@ void pqSignalAdaptorCompositeTreeWidget::domainChanged()
 //-----------------------------------------------------------------------------
 void pqSignalAdaptorCompositeTreeWidget::portInformationChanged()
 {
-  bool prev = this->Internal->TreeWidget->blockSignals(true);
+  bool prev = this->blockSignals(true);
   QList<QVariant> widgetvalues = this->values();
   this->Internal->Items.clear();
   this->Internal->TreeWidget->clear();
@@ -497,9 +627,11 @@ void pqSignalAdaptorCompositeTreeWidget::portInformationChanged()
   this->FlatIndex = 0;
   this->LevelNo = 0;
 
-  pqTreeWidgetItemObject* root = new pqCompositeTreeWidgetItem(
+  pqTreeWidgetItem* root = new pqCompositeTreeWidgetItem(
     this->Internal->TreeWidget, QStringList("Root"));
+  root->setCallbackHandler(this->CallbackAdaptor);
   root->setData(0, ORIGINAL_LABEL, "Root");
+  root->setData(0, BLOCK_NAME, QString());
   root->setToolTip(0, root->text(0));
   this->buildTree(root, dInfo);
   this->updateItemFlags();
@@ -507,7 +639,7 @@ void pqSignalAdaptorCompositeTreeWidget::portInformationChanged()
 
   // now update check state.
   this->setValues(widgetvalues);
-  this->Internal->TreeWidget->blockSignals(prev);
+  this->blockSignals(prev);
 
   if (this->AutoUpdateWidgetVisibility)
     {
@@ -529,7 +661,7 @@ void pqSignalAdaptorCompositeTreeWidget::updateItemFlags()
     return;
     }
 
-  foreach (pqTreeWidgetItemObject* item, this->Internal->Items)
+  foreach (pqTreeWidgetItem* item, this->Internal->Items)
     {
     QVariant vNodeType = item->data(0, NODE_TYPE);
     if (!vNodeType.isValid() || !vNodeType.canConvert<int>())
@@ -537,14 +669,12 @@ void pqSignalAdaptorCompositeTreeWidget::updateItemFlags()
       continue;
       }
 
-    bool can_check=false;
     int nodeType = vNodeType.toInt();
     if (nodeType == LEAF)
       {
       // leaves are always checkable.
       item->setFlags(item->flags()|Qt::ItemIsUserCheckable);
       item->setCheckState(0, Qt::Unchecked);
-      can_check = true;
       }
     else if (nodeType == NON_LEAF)
       {
@@ -555,41 +685,37 @@ void pqSignalAdaptorCompositeTreeWidget::updateItemFlags()
         {
         item->setFlags(item->flags()|Qt::ItemIsUserCheckable|Qt::ItemIsTristate);
         item->setCheckState(0, Qt::Unchecked);
-        can_check = true;
         }
       }
-
-    if (can_check)
-      {
-      QObject::connect(item, SIGNAL(checkedStateChanged(bool)),
-        this, SIGNAL(valuesChanged()), Qt::QueuedConnection);
-      QObject::connect(item, SIGNAL(checkedStateChanged(bool)),
-        this, SLOT(updateCheckState(bool)));
-      }
     }
 }
 
 //-----------------------------------------------------------------------------
 /// Called when an item check state is toggled. This is used only when
 /// this->CheckMode == SINGLE_ITEM. We uncheck all other items.
-void pqSignalAdaptorCompositeTreeWidget::updateCheckState(bool checked)
+void pqSignalAdaptorCompositeTreeWidget::updateCheckState(pqTreeWidgetItem* item,
+  int column)
 {
-  pqTreeWidgetItemObject* item = qobject_cast<pqTreeWidgetItemObject*>(this->sender());
+  this->CallbackAdaptor->blockCallbacks(true);
+  bool checked = item->checkState(column) == Qt::Checked;
   if (item && checked && this->CheckMode == SINGLE_ITEM)
     {
-    foreach (pqTreeWidgetItemObject* curitem, this->Internal->Items)
+    foreach (pqTreeWidgetItem* curitem, this->Internal->Items)
       {
       if (curitem != item && (curitem->flags() & Qt::ItemIsUserCheckable) && 
-        curitem->checkState(0) != Qt::Unchecked)
+        (curitem->checkState(0) != Qt::Unchecked) &&
+        (curitem->flags() & Qt::ItemIsTristate) == 0)
         {
         curitem->setCheckState(0, Qt::Unchecked);
         }
       }
     }
+  this->CallbackAdaptor->blockCallbacks(false);
+  emit this->valuesChanged();
 }
 
 //-----------------------------------------------------------------------------
-void pqSignalAdaptorCompositeTreeWidget::buildTree(pqTreeWidgetItemObject* item,
+void pqSignalAdaptorCompositeTreeWidget::buildTree(pqTreeWidgetItem* item,
   vtkPVDataInformation* info)
 {
   this->Internal->Items.push_back(item);
@@ -623,18 +749,20 @@ void pqSignalAdaptorCompositeTreeWidget::buildTree(pqTreeWidgetItemObject* item,
         // We don't fetch the names for each piece in a vtkMultiPieceDataSet
         // hence we just make up a name,
         QString childLabel = QString("DataSet %1").arg(cc);
-        if (this->ShowIndex)
+        if (this->ShowFlatIndex)
           {
           childLabel = QString("DataSet (%1)").arg(this->FlatIndex);
           }
 
-        pqTreeWidgetItemObject* child = new pqCompositeTreeWidgetItem(item,
+        pqTreeWidgetItem* child = new pqCompositeTreeWidgetItem(item,
           QStringList(childLabel));
+        child->setCallbackHandler(this->CallbackAdaptor);
         child->setToolTip(0, child->text(0));
         child->setData(0, ORIGINAL_LABEL, childLabel);
+        child->setData(0, BLOCK_NAME, QString());
         this->buildTree(child, NULL);
-        child->setData(0, DATASET_INDEX, cc);
-        child->setData(0, LEVEL_NUMBER, this->LevelNo);
+        child->setData(0, AMR_BLOCK_INDEX, cc);
+        child->setData(0, AMR_LEVEL_NUMBER, this->LevelNo);
         }
       }
     else
@@ -657,6 +785,7 @@ void pqSignalAdaptorCompositeTreeWidget::buildTree(pqTreeWidgetItemObject* item,
     {
     vtkPVDataInformation* childInfo = cinfo->GetDataInformation(cc);
     QString childLabel = QString("DataSet %1").arg(cc);
+    QString block_name;
 
     bool is_leaf = true;
     if (childInfo && childInfo->GetCompositeDataInformation()->GetDataIsComposite())
@@ -670,22 +799,25 @@ void pqSignalAdaptorCompositeTreeWidget::buildTree(pqTreeWidgetItemObject* item,
       if (cname[0])
         {
         childLabel = cname;
+        block_name = cname;
         }
       }
 
-    if (this->ShowIndex)
+    if (this->ShowFlatIndex)
       {
       childLabel = QString("%1 (%2)").arg(childLabel).arg(this->FlatIndex);
       }
 
     if (this->Internal->DomainMode != vtkSMCompositeTreeDomain::NON_LEAVES || !is_leaf)
       {
-      pqTreeWidgetItemObject* child = new pqCompositeTreeWidgetItem(item,
+      pqTreeWidgetItem* child = new pqCompositeTreeWidgetItem(item,
         QStringList(childLabel));
+      child->setCallbackHandler(this->CallbackAdaptor);
       child->setData(0, ORIGINAL_LABEL, childLabel);
+      child->setData(0, BLOCK_NAME, block_name);
       child->setToolTip(0, child->text(0));
       this->buildTree(child, cinfo->GetDataInformation(cc));
-      child->setData(0, LEVEL_NUMBER, this->LevelNo);
+      child->setData(0, AMR_LEVEL_NUMBER, this->LevelNo);
       this->LevelNo++;
       }
     else
@@ -712,14 +844,15 @@ void pqSignalAdaptorCompositeTreeWidget::updateSelectionCounts()
   
   // Iterate over the selection data information and then update the labels.
   vtkSMSourceProxy* sourceProxy = this->Internal->Domain->GetSource();
-  if (!sourceProxy->GetSelectionOutput(this->Internal->Domain->GetSourcePort()))
+  if (!sourceProxy ||
+    !sourceProxy->GetSelectionOutput(this->Internal->Domain->GetSourcePort()))
     {
     return;
     }
   vtkPVDataInformation* info = sourceProxy->GetSelectionOutput(
     this->Internal->Domain->GetSourcePort())->GetDataInformation();
 
-  foreach (pqTreeWidgetItemObject* item, this->Internal->Items)
+  foreach (pqTreeWidgetItem* item, this->Internal->Items)
     {
     if (item->data(0, NODE_TYPE).toInt() != LEAF)
       {
