@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program:   ParaView
-   Module:    $RCSfile: pqSignalAdaptorKeyFrameValue.cxx,v $
+   Module:    pqSignalAdaptorKeyFrameValue.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
