@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program:   ParaView
-   Module:    $RCSfile: pqSignalAdaptorKeyFrameValue.h,v $
+   Module:    pqSignalAdaptorKeyFrameValue.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -49,9 +49,9 @@ class pqAnimationCue;
 // to contain the value widget (passed as \c parent to the constructor).
 class PQCOMPONENTS_EXPORT pqSignalAdaptorKeyFrameValue : public QObject
 {
-  Q_OBJECT;
-  Q_PROPERTY(QList<QVariant> values READ values WRITE setValue);
-  Q_PROPERTY(QVariant value READ value WRITE setValue);
+  Q_OBJECT
+  Q_PROPERTY(QList<QVariant> values READ values WRITE setValue)
+  Q_PROPERTY(QVariant value READ value WRITE setValue)
 public:
   // Constructor. 
   // \c lparent is the frame which can be used to pack large widgets
