@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqSplitViewUndoElement.h,v $
+   Module:    pqSplitViewUndoElement.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -45,7 +45,7 @@ class PQCOMPONENTS_EXPORT pqSplitViewUndoElement : public vtkSMUndoElement
 {
 public:
   static pqSplitViewUndoElement* New();
-  vtkTypeRevisionMacro(pqSplitViewUndoElement, vtkSMUndoElement);
+  vtkTypeMacro(pqSplitViewUndoElement, vtkSMUndoElement);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   /// Description:
