@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqTextDisplayPropertiesWidget.h,v $
+   Module:    pqTextDisplayPropertiesWidget.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -44,7 +44,6 @@ public:
   virtual ~pqTextDisplayPropertiesWidget();
 
 protected slots:
-  void onVisibilityChanged(int);
   void onTextLocationChanged(bool);
 
 private:
