@@ -2,7 +2,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqTimerLogDisplay.h,v $
+   Module:    pqTimerLogDisplay.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -43,7 +43,7 @@ class vtkPVTimerInformation;
 
 class PQCOMPONENTS_EXPORT pqTimerLogDisplay : public QDialog
 {
-  Q_OBJECT;
+  Q_OBJECT
 
 public:
   pqTimerLogDisplay(QWidget *p = NULL);
