@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqViewManager.h,v $
+   Module:    pqViewManager.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -34,15 +34,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqMultiView.h"
 
-class pqActiveViewOptionsManager;
 class pqMultiViewFrame;
 class pqServer;
-class pqUndoStack;
 class pqView;
 class vtkImageData;
 class vtkPVXMLElement;
-class vtkSMStateLoaderBase;
-class vtkUndoElement;
+class vtkSMProxyLocator;
 
 /// This class manages all view windows. View windows occupy the central
 /// area in the application are all layed out using split windows. This 
@@ -61,12 +58,6 @@ public:
   /// returns the active view module.
   pqView* getActiveView() const;
 
-  /// Save the state of the view window manager.
-  void saveState(vtkPVXMLElement* root);
-
-  /// Loads the state for the view window manager.
-  bool loadState(vtkPVXMLElement* rwRoot, vtkSMStateLoaderBase* loader);
-
   /// This option is used for testing. When size.isEmpty() is true,
   /// it resets the maximum bounds on the view windows.
   /// This is useful when running tests, so that we are guranteed that 
@@ -80,9 +71,6 @@ public:
   /// Given a frame, returns the view, if any contained in it.
   pqView* getView(pqMultiViewFrame* frame) const;
 
-  /// Set the undo stack used for the application.
-  void setUndoStack(pqUndoStack* stack);
-
   /// Prepare the multiview for a screen capture for the given size. Returns the
   /// magnification to be used while performing the capture, if the
   /// requested size is greater than the widget size. One must call
@@ -108,38 +96,25 @@ public:
   ///   Resets the multi-view to its original state.
   /// \param removed Used to return all the removed widgets.
   virtual void reset(QList<QWidget*> &removed);
+  virtual void reset();
 
-  /// View options manager is used to show the view options dialog for the
-  /// current view. If the manager is not set, then the view options tool button
-  /// will be disabled.
-  void setViewOptionsManager(pqActiveViewOptionsManager* manager);
+protected slots:
+  /// Save the state of the view window manager.
+  void saveState(vtkPVXMLElement* root);
+
+  /// Loads the state for the view window manager.
+  bool loadState(vtkPVXMLElement* rwRoot, vtkSMProxyLocator* loader);
+
+  /// Called when server disconnects, we reset the view layout.
+  void onServerDisconnect();
 
 signals:
   /// Fired when the active view module changes.
   void activeViewChanged(pqView*);
 
-  /// Fired when the user pressed the lookmark button for one of the views
-  void createLookmark(QWidget*);
-
-  /// Fired when the manager begins an undoable change.
-  void beginUndo(const QString& label);
-
-  /// Fired when the manager is done with an undoable change.
-  void endUndo();
-
-  /// Fired to add the elem to the undo stack.
-  void addToUndoStack(vtkUndoElement* elem);
-
-  /// emitted when the manager begins changes that should not get
-  /// recorded on the undo stack.
-  void beginNonUndoableChanges();
-
-  /// emitted when the manager is done with changes that
-  /// should not get recorded on the undo stack.
-  void endNonUndoableChanges();
-
-  /// Fired when the camera button is clicked.
-  void triggerCameraAdjustment(pqView*);
+  /// Fired when setMaxViewWindowSize.  Argument set to true when given a
+  /// nonempty size, false otherwise.
+  void maxViewWindowSizeSet(bool);
 
 private slots:
   /// This will create a view module to fill the frame.
@@ -186,23 +161,24 @@ private slots:
   /// We add an undo element to the stack to undo/redo the split.
   void onSplittingView(const Index&, Qt::Orientation, float, const Index&);
 
-  /// Called when the camera button is clicked
-  virtual void onCameraTriggered();
-
-  /// Called when the view options button is clicked.
-  void onViewOptionsRequested();
+  /// Destroys any frame overlays.
+  void destroyFrameOverlays();
 
 public slots:
-  /// Set the active server. This must be called whenever
-  /// active server changes. The active server is used to 
-  /// determine the server on whcih to create new view modules
-  /// when a frame a split.
-  void setActiveServer(pqServer* server);
+  /// Called to change the active view. If view==null and then if the view
+  /// manager is currently focused on an empty frame, then it does not change
+  /// that. Otherwise the frame containing the view is activated.
+  void setActiveView(pqView* view);
 
 protected:
   /// Event filter callback.
   bool eventFilter(QObject* caller, QEvent* e);
 
+  /// Overridden to call updateViewPositions() after a frame has been maximized
+  /// or minimized.
+  virtual void maximizeWidget(QWidget*);
+  virtual void restoreWidget(QWidget*);
+
   /// This method will either assign an empty frame
   /// to the view module or split the active view
   /// (if any, otherwise splits the first view)
@@ -224,6 +200,11 @@ protected:
   /// on all view modules.
   void updateViewPositions();
 
+  /// Update the GUISize/WindowPosition properties
+  /// using compact values to produce gapless multi views
+  /// on the render server.
+  void updateCompactViewPositions();
+
   /// Updates the context menu.
   void updateConversionActions(pqMultiViewFrame* frame);
 
@@ -231,9 +212,14 @@ protected:
   /// We add an undo element to the stack to undo/redo the close.
   void onFrameRemovedInternal(pqMultiViewFrame*);
 
+  /// Shows the overlay widgets showing all the view sizes.
+  void showFrameOverlays();
 
   QAction* getAction(pqMultiViewFrame* frame,QString name);
 
+  /// need access to the loadState()/saveState() methods.
+  friend class pqCloseViewUndoElement;
+  friend class pqSplitViewUndoElement;
 
 private:
   pqViewManager(pqViewManager&); // Not implemented.
