@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationCue.cxx,v $
+   Module:    pqAnimationCue.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -82,6 +82,10 @@ pqAnimationCue::pqAnimationCue(const QString& group, const QString& name,
     proxy->GetProperty("AnimatedElement"), vtkCommand::ModifiedEvent,
     this, SIGNAL(modified()));
 
+  this->Internal->VTKConnect->Connect(
+    proxy->GetProperty("Enabled"), vtkCommand::ModifiedEvent,
+    this, SLOT(onEnabledModified()));
+
   this->onManipulatorModified();
 }
 
@@ -156,6 +160,27 @@ void pqAnimationCue::setDefaultPropertyValues()
   proxy->UpdateVTKObjects();
 }
 
+//-----------------------------------------------------------------------------
+void pqAnimationCue::setEnabled(bool enable)
+{
+  pqSMAdaptor::setElementProperty(this->getProxy()->GetProperty("Enabled"),
+    enable? 1 : 0);
+  this->getProxy()->UpdateVTKObjects();
+}
+
+//-----------------------------------------------------------------------------
+bool pqAnimationCue::isEnabled() const
+{
+  return pqSMAdaptor::getElementProperty(
+    this->getProxy()->GetProperty("Enabled")).toBool();
+}
+
+//-----------------------------------------------------------------------------
+void pqAnimationCue::onEnabledModified()
+{
+  emit this->enabled(this->isEnabled());
+}
+
 //-----------------------------------------------------------------------------
 void pqAnimationCue::onManipulatorModified()
 {
