@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqAnimationCue.h,v $
+   Module:    pqAnimationCue.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -96,7 +96,16 @@ public:
   /// The default implementation iterates over all properties
   /// of the proxy and sets them to default values. 
   void setDefaultPropertyValues();
-    
+
+  /// Used by editors to trigger keyframesModified() signal after bulk of
+  /// modifications have been made to the cue/key frames.
+  void triggerKeyFramesModified()
+    { emit this->keyframesModified(); }
+   
+  /// Get/Set the enabled state for the cue.
+  void setEnabled(bool enable);
+  bool isEnabled() const;
+
 signals:
   // emitted when something about the keyframes changes.
   void keyframesModified();
@@ -105,10 +114,16 @@ signals:
   // changes.
   void modified();
 
+  /// Fired when the enabled-state of the cue changes.
+  void enabled(bool);
+
 private slots:
-  // Called when the "Manipulator" property is changed.
+  /// Called when the "Manipulator" property is changed.
   void onManipulatorModified();
 
+  /// Called when the "Enabled" property is changed.
+  void onEnabledModified();
+
 private:
   pqAnimationCue(const pqAnimationCue&); // Not implemented.
   void operator=(const pqAnimationCue&); // Not implemented.
