@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayPolicy.cxx,v $
+   Module:    pqDisplayPolicy.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -45,11 +45,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QString>
 
 #include "pqApplicationCore.h"
+#include "pqBarChartView.h"
 #include "pqDataRepresentation.h"
+#include "pqLineChartView.h"
 #include "pqObjectBuilder.h"
 #include "pqOutputPort.h"
 #include "pqPipelineSource.h"
-#include "pqPlotView.h"
 #include "pqRenderView.h"
 #include "pqServer.h"
 #include "pqTwoDRenderView.h"
@@ -68,6 +69,13 @@ pqDisplayPolicy::~pqDisplayPolicy()
 QString pqDisplayPolicy::getPreferredViewType(pqOutputPort* opPort,
   bool update_pipeline) const
 {
+  QString view_type = QString::null;
+
+  if (!opPort)
+    {
+    return view_type;
+    }
+
   pqPipelineSource* source = opPort->getSource();
   if (update_pipeline)
     {
@@ -100,8 +108,6 @@ QString pqDisplayPolicy::getPreferredViewType(pqOutputPort* opPort,
       }
     }
 
-  QString view_type = QString::null;
-
   // HACK: for now, when update_pipeline is false, we don't do any gather
   // information as that can result in progress events which may case Qt paint
   // issues.
@@ -115,11 +121,9 @@ QString pqDisplayPolicy::getPreferredViewType(pqOutputPort* opPort,
     return view_type;
     }
 
-  vtkPVDataInformation* datainfo = update_pipeline?
-    opPort->getDataInformation(true) : opPort->getCachedDataInformation();
+  vtkPVDataInformation* datainfo = opPort->getDataInformation();
   QString className = datainfo?  datainfo->GetDataClassName() : QString();
 
-
   // * Check if we should create the 2D view.
   if ((className == "vtkImageData" || className == "vtkUniformGrid") && 
     datainfo->GetCompositeDataClassName()==0)
@@ -158,7 +162,7 @@ QString pqDisplayPolicy::getPreferredViewType(pqOutputPort* opPort,
     if (dimensionality == 1 && cellDataInfo->GetNumberOfArrays() > 0)
       {
       // Has cell data, mostlikely this is a histogram.
-      view_type = pqPlotView::barChartType();
+      view_type = pqBarChartView::barChartViewType();
       }
     else if (dimensionality == 1 && 
       (pointDataInfo->GetNumberOfArrays() > 0 ||
@@ -166,7 +170,7 @@ QString pqDisplayPolicy::getPreferredViewType(pqOutputPort* opPort,
       datainfo->GetNumberOfPoints() > 1)
       {
       // No cell data, but some point data -- may be a XY line plot.
-      view_type = pqPlotView::XYPlotType();
+      view_type = pqLineChartView::lineChartViewType();
       }
     }
 
@@ -189,6 +193,11 @@ pqView* pqDisplayPolicy::getPreferredView(
       }
     else
       {
+      // if the currentView is empty (no visible representations), destroy it
+      if(currentView && !currentView->getNumberOfVisibleRepresentations())
+        {
+        builder->destroy(currentView);
+        }
       // Create the preferred view only if the current one is not of the same type
       // as the preferred view.
       currentView = builder->createView(view_type, opPort->getServer());
@@ -197,7 +206,7 @@ pqView* pqDisplayPolicy::getPreferredView(
 
   if (!currentView || (currentView && !currentView->canDisplay(opPort)))
     {
-    vtkPVDataInformation* info = opPort->getDataInformation(false);
+    vtkPVDataInformation* info = opPort->getDataInformation();
     // GetDataSetType() == -1 signifies that there's no data to show.
     if (info->GetDataSetType() != -1)
       {
@@ -240,27 +249,12 @@ pqDataRepresentation* pqDisplayPolicy::createPreferredRepresentation(
       }
     }
 
-  // Simply create a display for the view set up the connections and
-  // return.
-  pqDataRepresentation* display = pqApplicationCore::instance()->
-    getObjectBuilder()->createDataRepresentation(opPort, view);
-
-  // If this is the only source displayed in the view, reset the camera to make sure its visible
-  if(view->getNumberOfVisibleRepresentations()==1)
-    {
-    pqRenderViewBase* ren = qobject_cast<pqRenderViewBase*>(view);
-    if (ren)
-      {
-      ren->resetCamera();
-      }
-    }
-
-  return display;
+  return this->setRepresentationVisibility(opPort, view, true);
 }
 
 //-----------------------------------------------------------------------------
 pqDataRepresentation* pqDisplayPolicy::setRepresentationVisibility(
-  pqOutputPort* opPort, pqView* view, bool visible) 
+  pqOutputPort* opPort, pqView* view, bool visible) const
 {
   if (!opPort)
     {
@@ -286,10 +280,15 @@ pqDataRepresentation* pqDisplayPolicy::setRepresentationVisibility(
       }
     if (view)
       {
-      repr = pqApplicationCore::instance()->getObjectBuilder()->
-        createDataRepresentation(opPort, view);
+      repr = this->newRepresentation(opPort, view);
       }
     }
+  if (!repr)
+    {
+    qDebug() << "Cannot show the data in the current view although "
+      "the view reported that it can show the data.";
+    return 0;
+    }
 
   repr->setVisible(visible);
 
@@ -335,7 +334,15 @@ pqDisplayPolicy::VisibilityState pqDisplayPolicy::getVisibility(
       }
     }
 
-  // Default behaviour if no view is present
+  // Default behavior if no view is present
   return Hidden;
 }
 
+
+//-----------------------------------------------------------------------------
+pqDataRepresentation* pqDisplayPolicy::newRepresentation(pqOutputPort* port, 
+  pqView* view) const
+{
+  return pqApplicationCore::instance()->getObjectBuilder()->
+    createDataRepresentation(port, view);
+}
