@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqDisplayPolicy.h,v $
+   Module:    pqDisplayPolicy.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -83,7 +83,7 @@ public:
   /// </ul>
   /// or not of the type preferred by the source, it may create a new view and 
   /// add the displayto new view. \c dont_create_view can be used to 
-  /// override this behaviour.
+  /// override this behavior.
   virtual pqDataRepresentation* createPreferredRepresentation(
     pqOutputPort* port, pqView* view, bool dont_create_view) const;
 
@@ -94,7 +94,7 @@ public:
   /// create new views, we provide this as part of display policy which can 
   /// be easily overridden by creating a new subclass.
   virtual pqDataRepresentation* setRepresentationVisibility(
-    pqOutputPort* opPort, pqView* view, bool visible);
+    pqOutputPort* opPort, pqView* view, bool visible) const;
 
   /// Returns the type for the view that is indicated as the preferred view
   /// for the given output port. May return a null string if the no view type
@@ -127,6 +127,10 @@ protected:
   /// type may be created and returned. A new 3D view will be created if the
   /// view has no preferred view type and the active view cannot show the data.
   virtual pqView* getPreferredView(pqOutputPort* opPort, pqView* view) const;
+
+  /// Create a new representation. port and view cannot be null at this point.
+  virtual pqDataRepresentation* newRepresentation(pqOutputPort* port, 
+    pqView* view) const;
 };
 
 #endif
