@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqFileDialog.cxx,v $
+   Module:    pqFileDialog.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -33,6 +33,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqFileDialog.h"
 #include "pqFileDialogModel.h"
 #include "pqFileDialogFavoriteModel.h"
+#include "pqFileDialogRecentDirsModel.h"
 #include "pqFileDialogFilter.h"
 #include "pqServer.h"
 
@@ -47,6 +48,9 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <QComboBox>
 #include <QAbstractItemView>
 
+#include <vtkstd/string>
+#include <vtksys/SystemTools.hxx>
+
 class pqFileComboBox : public QComboBox
 {
 public:
@@ -122,10 +126,12 @@ class pqFileDialog::pqImplementation : public QObject
 public:
   pqFileDialogModel* const Model;
   pqFileDialogFavoriteModel* const FavoriteModel;
+  pqFileDialogRecentDirsModel* const RecentModel;
   pqFileDialogFilter FileFilter;
   FileMode Mode;
   Ui::pqFileDialog Ui;
   QStringList SelectedFiles;
+  QStringList Filters;
   bool SupressOverwriteWarning;
   
   // remember the last locations we browsed
@@ -136,16 +142,20 @@ public:
     QObject(p),
     Model(new pqFileDialogModel(server, NULL)),
     FavoriteModel(new pqFileDialogFavoriteModel(server, NULL)),
+    RecentModel(new pqFileDialogRecentDirsModel(Model, server, NULL)),
     FileFilter(this->Model),
     Mode(ExistingFile),
     SupressOverwriteWarning(false)
   {
+  QObject::connect(p, SIGNAL(filesSelected(const QStringList&)),
+    this->RecentModel, SLOT(setChosenFiles(const QStringList&)));
   }
   
   ~pqImplementation()
   {
-    delete this->Model;
+    delete this->RecentModel;
     delete this->FavoriteModel;
+    delete this->Model;
   }
   
   QString getStartPath()
@@ -276,6 +286,11 @@ pqFileDialog::pqFileDialog(
   this->Implementation->Ui.Favorites->setModel(this->Implementation->FavoriteModel);
   this->Implementation->Ui.Favorites->setSelectionBehavior(QAbstractItemView::SelectRows);
 
+  this->Implementation->Ui.Recent->setModel(
+    this->Implementation->RecentModel);
+  this->Implementation->Ui.Recent->setSelectionBehavior(
+    QAbstractItemView::SelectRows);
+
   this->setFileMode(ExistingFile);
 
   QObject::connect(this->Implementation->Model,
@@ -299,7 +314,7 @@ pqFileDialog::pqFileDialog(
                    SLOT(onNavigate(const QString&)));
 
   QObject::connect(this->Implementation->Ui.FileType, 
-                   SIGNAL(activated(const QString&)), 
+                   SIGNAL(currentIndexChanged(const QString&)), 
                    this, 
                    SLOT(onFilterChange(const QString&)));
   
@@ -308,6 +323,11 @@ pqFileDialog::pqFileDialog(
                    this, 
                    SLOT(onClickedFavorite(const QModelIndex&)));
 
+  QObject::connect(this->Implementation->Ui.Recent, 
+                   SIGNAL(clicked(const QModelIndex&)), 
+                   this, 
+                   SLOT(onClickedRecent(const QModelIndex&)));
+
   QObject::connect(this->Implementation->Ui.Files, 
                    SIGNAL(clicked(const QModelIndex&)), 
                    this, 
@@ -324,6 +344,11 @@ pqFileDialog::pqFileDialog(
                    this, 
                    SLOT(onActivateFavorite(const QModelIndex&)));
 
+  QObject::connect(this->Implementation->Ui.Recent, 
+                   SIGNAL(activated(const QModelIndex&)), 
+                   this, 
+                   SLOT(onActivateRecent(const QModelIndex&)));
+
   QObject::connect(this->Implementation->Ui.Files, 
                    SIGNAL(activated(const QModelIndex&)), 
                    this, 
@@ -338,10 +363,12 @@ pqFileDialog::pqFileDialog(
   if(filterList.empty())
     {
     this->Implementation->Ui.FileType->addItem("All Files (*)");
+    this->Implementation->Filters << "All Files (*)";
     }
   else
     {
     this->Implementation->Ui.FileType->addItems(filterList);
+    this->Implementation->Filters = filterList;
     }
   this->onFilterChange(this->Implementation->Ui.FileType->currentText());
   
@@ -434,7 +461,6 @@ void pqFileDialog::onContextMenuRequested(const QPoint &menuPos)
   menu.exec(this->Implementation->Ui.Files->mapToGlobal(menuPos));
 }
 
-
 //-----------------------------------------------------------------------------
 void pqFileDialog::setFileMode(pqFileDialog::FileMode mode)
 {
@@ -466,6 +492,26 @@ void pqFileDialog::setFileMode(pqFileDialog::FileMode mode)
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqFileDialog::setRecentlyUsedExtension(const QString& fileExtension)
+{
+  if ( fileExtension == QString() )
+    {
+    // upon the initial use of any kind (e.g., data or screenshot) of dialog
+    // 'fileExtension' is equal /set to an empty string.
+    // In this case, no any user preferences are considered
+    this->Implementation->Ui.FileType->setCurrentIndex(0);
+    }
+  else
+    {
+    int index = this->Implementation->Ui.FileType->findText(fileExtension,
+      Qt::MatchContains);
+    // just in case the provided extension is not in the combobox list
+    index = (index == -1) ? 0 : index;
+    this->Implementation->Ui.FileType->setCurrentIndex(index);
+    }
+}
+
 //-----------------------------------------------------------------------------
 void pqFileDialog::emitFilesSelected(const QStringList& files)
 {
@@ -634,6 +680,12 @@ void pqFileDialog::onClickedFavorite(const QModelIndex&)
   this->Implementation->Ui.Files->clearSelection();
 }
 
+//-----------------------------------------------------------------------------
+void pqFileDialog::onClickedRecent(const QModelIndex&)
+{
+  this->Implementation->Ui.Files->clearSelection();
+}
+
 //-----------------------------------------------------------------------------
 void pqFileDialog::onClickedFile(const QModelIndex&)
 {
@@ -651,6 +703,14 @@ void pqFileDialog::onActivateFavorite(const QModelIndex& index)
     }
 }
 
+//-----------------------------------------------------------------------------
+void pqFileDialog::onActivateRecent(const QModelIndex& index)
+{
+  QString file = this->Implementation->RecentModel->filePath(index);
+  this->onNavigate(file);
+  this->Implementation->Ui.FileName->selectAll();
+}
+
 //-----------------------------------------------------------------------------
 void pqFileDialog::onActivateFile(const QModelIndex& index)
 {
@@ -682,6 +742,45 @@ QString pqFileDialog::fixFileExtension(
   QString wantedExtension =
     extensionWildcard.mid(extensionWildcard.indexOf('.')+1);
 
+
+  if (!ext.isEmpty())
+    {
+    // Ensure that the extension the user added is indeed of one the supported
+    // types. (BUG #7634).
+    QStringList wildCards;
+    foreach (QString curfilter, this->Implementation->Filters)
+      {
+      wildCards += ::GetWildCardsFromFilter(curfilter);
+      }
+    bool pass = false;
+    foreach (QString wildcard, wildCards)
+      {
+      if (wildcard.indexOf('.') != -1)
+        {
+        // we only need to validate the extension, not the filename.
+        wildcard = QString("*.%1").arg(wildcard.mid(wildcard.indexOf('.')+1));
+        QRegExp regEx = QRegExp(wildcard, Qt::CaseInsensitive, QRegExp::Wildcard);
+        if (regEx.exactMatch(fileInfo.fileName()))
+          {
+          pass = true;
+          break;
+          }
+        }
+      else
+        {
+        // we have a filter which does not specify any rule for the extension.
+        // In that case, just assume the extension matched. 
+        pass = true;
+        break;
+        }
+      }
+    if (!pass)
+      {
+      // force adding of the wantedExtension.
+      ext = QString();
+      }
+    }
+
   QString fixedFilename = filename;
   if(ext.isEmpty() && !wantedExtension.isEmpty() &&
     wantedExtension != "*")
@@ -829,13 +928,30 @@ void pqFileDialog::fileSelectionChanged()
 
   this->Implementation->Ui.FileName->setText(fileString);
 }
-  
+
 bool pqFileDialog::selectFile(const QString& f)
 {
+  // We don't use QFileInfo here since it messes the paths up if the client and
+  // the server are heterogeneous systems.
+  vtkstd::string unix_path = f.toAscii().data();
+  vtksys::SystemTools::ConvertToUnixSlashes(unix_path);
+
+  vtkstd::string filename, dirname;
+  vtkstd::string::size_type slashPos = unix_path.rfind("/");
+  if (slashPos != vtkstd::string::npos)
+    {
+    filename = unix_path.substr(slashPos+1);
+    dirname = unix_path.substr(0, slashPos);
+    }
+  else
+    {
+    filename = unix_path;
+    dirname = "";
+    }
+
   QPointer<QDialog> diag = this;
-  QFileInfo info(f);
-  this->Implementation->Model->setCurrentPath(info.absolutePath());
-  this->Implementation->Ui.FileName->setText(info.fileName());
+  this->Implementation->Model->setCurrentPath(dirname.c_str());
+  this->Implementation->Ui.FileName->setText(filename.c_str());
   this->Implementation->SupressOverwriteWarning = true;
   this->accept();
   if(diag && diag->result() != QDialog::Accepted)
