@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqObjectBuilder.cxx,v $
+   Module:    pqObjectBuilder.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -57,12 +57,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqPipelineFilter.h"
 #include "pqPipelineSource.h"
 #include "pqPluginManager.h"
+#include "pqProxyModifiedStateUndoElement.h"
 #include "pqRenderView.h"
 #include "pqScalarBarRepresentation.h"
 #include "pqScalarsToColors.h"
 #include "pqServer.h"
 #include "pqServerManagerModel.h"
 #include "pqSMAdaptor.h"
+#include "pqUndoStack.h"
 #include "pqView.h"
 #include "pqViewModuleInterface.h"
 
@@ -93,7 +95,7 @@ pqPipelineSource* pqObjectBuilder::createSource(const QString& sm_group,
     const QString& sm_name, pqServer* server)
 {
   vtkSMProxy* proxy = 
-    this->createProxyInternal(sm_group, sm_name, server, "sources");
+    this->createProxyInternal(sm_group, sm_name, server, "sources", QString(), QMap<QString,QVariant>());
   if (proxy)
     {
     pqPipelineSource* source = pqApplicationCore::instance()->
@@ -103,6 +105,12 @@ pqPipelineSource* pqObjectBuilder::createSource(const QString& sm_group,
     source->setDefaultPropertyValues();
     source->setModifiedState(pqProxy::UNINITIALIZED);
 
+    pqProxyModifiedStateUndoElement* elem =
+      pqProxyModifiedStateUndoElement::New();
+    elem->MadeUninitialized(source);
+    ADD_UNDO_ELEM(elem);
+    elem->Delete();
+
     emit this->sourceCreated(source);
     emit this->proxyCreated(source);
     return source;
@@ -110,26 +118,23 @@ pqPipelineSource* pqObjectBuilder::createSource(const QString& sm_group,
   return 0;
 }
 
-//-----------------------------------------------------------------------------
 pqPipelineSource* pqObjectBuilder::createFilter(
-  const QString& group, const QString& name, pqPipelineSource* input)
+  const QString& group, const QString& name,
+  QMap<QString, QList<pqOutputPort*> > namedInputs, pqServer* server)
 {
-  QMap<QString, QList<pqOutputPort*> > namedInputs;
-  QList<pqOutputPort*> inputs;
-  inputs.push_back(input->getOutputPort(0));
-  namedInputs["Input"] = inputs;
-
-  return this->createFilter(group, name, namedInputs, input->getServer());
+  return this->createFilter(group, name, namedInputs, server, QMap<QString, QVariant>());
 }
 
+
 //-----------------------------------------------------------------------------
 pqPipelineSource* pqObjectBuilder::createFilter(
   const QString& group, const QString& name,
   QMap<QString, QList<pqOutputPort*> > namedInputs,
-  pqServer* server)
+  pqServer* server,
+  const QMap<QString, QVariant>& properties/*=QMap<QString, QVariant>()*/)
 {
   vtkSMProxy* proxy = 
-    this->createProxyInternal(group, name, server, "sources");
+    this->createProxyInternal(group, name, server, "sources",QString(),properties);
   if (!proxy)
     {
     return 0;
@@ -177,6 +182,18 @@ pqPipelineSource* pqObjectBuilder::createFilter(
   return filter;
 }
 
+//-----------------------------------------------------------------------------
+pqPipelineSource* pqObjectBuilder::createFilter(
+  const QString& group, const QString& name, pqPipelineSource* input, int output_port)
+{
+  QMap<QString, QList<pqOutputPort*> > namedInputs;
+  QList<pqOutputPort*> inputs;
+  inputs.push_back(input->getOutputPort(output_port));
+  namedInputs["Input"] = inputs;
+
+  return this->createFilter(group, name, namedInputs, input->getServer());
+}
+
 inline QString pqObjectBuilderGetPath(const QString& filename, bool use_dir)
 {
   if (use_dir)
@@ -219,7 +236,7 @@ pqPipelineSource* pqObjectBuilder::createReader(const QString& sm_group,
     }
 
   vtkSMProxy* proxy = 
-    this->createProxyInternal(sm_group, sm_name, server, "sources", reg_name);
+    this->createProxyInternal(sm_group, sm_name, server, "sources", reg_name, QMap<QString,QVariant>());
   if (!proxy)
     {
     return 0;
@@ -273,8 +290,15 @@ pqPipelineSource* pqObjectBuilder::createReader(const QString& sm_group,
   reader->setDefaultPropertyValues();
   reader->setModifiedState(pqProxy::UNINITIALIZED);
 
+  pqProxyModifiedStateUndoElement* elem =
+    pqProxyModifiedStateUndoElement::New();
+  elem->MadeUninitialized(reader);
+  ADD_UNDO_ELEM(elem);
+  elem->Delete();
+
   emit this->readerCreated(reader, files[0]);
   emit this->readerCreated(reader, files);
+  emit this->sourceCreated(reader);
   emit this->proxyCreated(reader);
   return reader;
 }
@@ -354,7 +378,8 @@ pqView* pqObjectBuilder::createView(const QString& type,
 
   if (!proxy)
     {
-    qDebug() << "Failed to create a proxy for the requested view type.";
+    qDebug() << "Failed to create a proxy for the requested view type:"
+             << type;
     return NULL;
     }
 
@@ -376,8 +401,8 @@ pqView* pqObjectBuilder::createView(const QString& type,
     }
   else
     {
-    qDebug() << "Cannot locate the pqView for the " 
-      << "view module proxy.";
+    qDebug() << "Cannot locate the pqView for the view module proxy:"
+             << name << "of type" << type;
     }
 
   return view;
@@ -440,14 +465,6 @@ pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
     reprProxy = vtkSMObject::GetProxyManager()->NewProxy(
       "representations", representationType.toAscii().data());
     }
-  else if ( (srcProxyName == "TextSource" || 
-             srcProxyName == "TimeToTextConvertor"
-             || srcProxyName == "TimeToTextConvertorSource") && 
-    qobject_cast<pqRenderView*>(view))
-    {
-    reprProxy = vtkSMObject::GetProxyManager()->NewProxy(
-      "representations", "TextSourceRepresentation");
-    }
   else
     {
     reprProxy = view->getViewProxy()->CreateDefaultRepresentation(
@@ -493,11 +510,11 @@ pqDataRepresentation* pqObjectBuilder::createDataRepresentation(
     findItem<pqDataRepresentation*>(reprProxy);
   if (repr)
     {
+    repr->setDefaultPropertyValues();
+
     // inherit properties from the representation for the input if applicable.
     this->initializeInheritedProperties(repr);
 
-    repr->setDefaultPropertyValues();
-
     emit this->dataRepresentationCreated(repr);
     emit this->proxyCreated(repr);
     }
@@ -560,7 +577,8 @@ pqScalarBarRepresentation* pqObjectBuilder::createScalarBarDisplay(
 
   pqServer* server = view->getServer();
   vtkSMProxy* scalarBarProxy = this->createProxyInternal(
-    "representations", "ScalarBarWidgetRepresentation", server, "scalar_bars");
+    "representations", "ScalarBarWidgetRepresentation", server, "scalar_bars",
+    QString(), QMap<QString,QVariant>());
 
   if (!scalarBarProxy)
     {
@@ -587,7 +605,8 @@ pqScalarBarRepresentation* pqObjectBuilder::createScalarBarDisplay(
 pqAnimationScene* pqObjectBuilder::createAnimationScene(pqServer* server)
 {
   vtkSMProxy* proxy = 
-    this->createProxyInternal("animation", "AnimationScene", server, "animation");
+    this->createProxyInternal("animation", "AnimationScene", server,
+      "animation", QString(), QMap<QString,QVariant>());
   if (proxy)
     {
     proxy->SetServers(vtkProcessModule::CLIENT);
@@ -611,7 +630,7 @@ vtkSMProxy* pqObjectBuilder::createProxy(const QString& sm_group,
     const QString& reg_group, const QString& reg_name/*=QString()*/)
 {
   vtkSMProxy* proxy = this->createProxyInternal(
-    sm_group, sm_name, server, reg_group, reg_name);
+    sm_group, sm_name, server, reg_group, reg_name, QMap<QString,QVariant>());
   if (proxy)
     {
     emit this->proxyCreated(proxy);
@@ -695,7 +714,8 @@ void pqObjectBuilder::destroyAllProxies(pqServer* server)
 //-----------------------------------------------------------------------------
 vtkSMProxy* pqObjectBuilder::createProxyInternal(const QString& sm_group, 
   const QString& sm_name, pqServer* server, 
-  const QString& reg_group, const QString& reg_name/*=QString()*/)
+  const QString& reg_group, const QString& reg_name/*=QString()*/,
+  const QMap<QString, QVariant>& properties/*=QMap<QString, QVariant>()*/)
 {
   if (!server)
     {
@@ -728,6 +748,33 @@ vtkSMProxy* pqObjectBuilder::createProxyInternal(const QString& sm_group,
 
   pxm->RegisterProxy(reg_group.toAscii().data(), 
     actual_regname.toAscii().data(), proxy);
+
+
+  QMap<QString, QVariant>::const_iterator mapIter;
+  for (mapIter = properties.begin(); mapIter != properties.end(); ++mapIter)
+    {
+    QString propertyName = mapIter.key();
+    QVariant propertyValue = mapIter.value();
+
+    vtkSMProperty *prop=proxy->GetProperty(propertyName.toAscii().data());
+
+    if(prop)
+      {
+        switch (pqSMAdaptor::getPropertyType(prop))
+        {
+        case pqSMAdaptor::FILE_LIST:
+          {
+            pqSMAdaptor::setFileListProperty(prop, propertyValue.toStringList());
+          }
+          break;
+        default:
+          break;
+        }
+      }
+    }
+
+
+
   return proxy;
 }
 
@@ -743,7 +790,7 @@ void pqObjectBuilder::destroyProxyInternal(pqProxy* proxy)
 }
 
 //-----------------------------------------------------------------------------
-QString pqObjectBuilder::getFileNamePropertyName(vtkSMProxy* proxy) const
+QString pqObjectBuilder::getFileNamePropertyName(vtkSMProxy* proxy)
 {
   // Find the first property that has a vtkSMFileListDomain. Assume that
   // it is the property used to set the filename.
@@ -895,4 +942,5 @@ void pqObjectBuilder::initializeInheritedProperties(pqDataRepresentation* repr)
         }
       }
     }
+  reprProxy->UpdateVTKObjects();
 }
