@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqPipelineFilter.cxx,v $
+   Module:    pqPipelineFilter.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -42,8 +42,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMDomain.h"
 #include "vtkSMDomainIterator.h"
 #include "vtkSMInputProperty.h"
-#include "vtkSMPropertyIterator.h"
-#include "vtkSMProxy.h"
+#include "vtkSMOrderedPropertyIterator.h"
+#include "vtkSMPVRepresentationProxy.h"
 
 //Qt includes.
 #include <QList>
@@ -56,6 +56,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "pqApplicationCore.h"
 #include "pqServerManagerModel.h"
 #include "pqOutputPort.h"
+#include "pqPipelineRepresentation.h"
 
 uint qHash(QPair<QPointer<pqPipelineSource>, int> arg)
 {
@@ -102,8 +103,8 @@ static void pqPipelineFilterGetInputProperties(QList<const char*> &list,
   vtkSMProxy* proxy, 
   bool skip_optional)
 {
-  vtkSmartPointer<vtkSMPropertyIterator> propIter;
-  propIter.TakeReference(proxy->NewPropertyIterator());
+  vtkSMOrderedPropertyIterator* propIter = vtkSMOrderedPropertyIterator::New();
+  propIter->SetProxy(proxy);
   for (propIter->Begin(); !propIter->IsAtEnd(); propIter->Next())
     {
     vtkSMInputProperty* inputProp = vtkSMInputProperty::SafeDownCast(
@@ -111,7 +112,9 @@ static void pqPipelineFilterGetInputProperties(QList<const char*> &list,
     if (inputProp)
       {
       vtkPVXMLElement* hints = inputProp->GetHints();
-      if (hints && hints->FindNestedElementByName("NoGUI"))
+      if (   hints
+          && (   hints->FindNestedElementByName("NoGUI")
+              || hints->FindNestedElementByName("SelectionInput") ) )
         {
         // hints suggest that this input property is not to be considered by the
         // GUI.
@@ -151,6 +154,7 @@ static void pqPipelineFilterGetInputProperties(QList<const char*> &list,
         }
       }
     }
+  propIter->Delete();
 }
 
 //-----------------------------------------------------------------------------
@@ -233,6 +237,25 @@ int pqPipelineFilter::getNumberOfInputs(const QString& portname) const
   return iter.value().size();
 }
 
+//-----------------------------------------------------------------------------
+QList<pqOutputPort*> pqPipelineFilter::getAllInputs() const
+{
+  QList<pqOutputPort*> list;
+
+  foreach (const pqInternal::InputList& inputs, this->Internal->Inputs)
+    {
+    for (int cc=0; cc < inputs.size(); cc++)
+      {
+      if (inputs[cc] && !list.contains(inputs[cc]))
+        {
+        list.push_back(inputs[cc]);
+        }
+      }
+    }
+
+  return list;
+}
+
 //-----------------------------------------------------------------------------
 QList<pqOutputPort*> pqPipelineFilter::getInputs(const QString& portname) const
 {
@@ -431,3 +454,36 @@ int pqPipelineFilter::replaceInput() const
   return 1; // default value.
 }
 
+//-----------------------------------------------------------------------------
+void pqPipelineFilter::hideInputIfRequired(pqView* view)
+{
+  int replace_input = this->replaceInput();
+  if (replace_input > 0)
+    {
+    // hide input source.
+    QList<pqOutputPort*> inputs = this->getAllInputs();
+    for(int kk=0; kk < inputs.size(); ++kk)
+      {
+      pqOutputPort* input = inputs[kk];
+      pqDataRepresentation* inputRepr = input->getRepresentation(view);
+      if (inputRepr)
+        {
+        pqPipelineRepresentation* sourceDisp =
+          qobject_cast<pqPipelineRepresentation*>(inputRepr);
+        if (sourceDisp && replace_input == 2)
+          {
+          // Conditionally turn off the input. The input should be turned
+          // off if the representation is surface and the opacity is 1.
+          int reprType = sourceDisp->getRepresentationType();
+          if ((reprType != vtkSMPVRepresentationProxy::SURFACE &&
+              reprType != vtkSMPVRepresentationProxy::SURFACE_WITH_EDGES) ||
+            sourceDisp->getOpacity() < 1.0)
+            {
+            continue;
+            }
+          }
+        inputRepr->setVisible(false);
+        }
+      }
+    }
+}
