@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqPipelineFilter.h,v $
+   Module:    pqPipelineFilter.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -71,6 +71,9 @@ public:
   /// Returns the input proxies connected to the given named input port.
   QList<pqOutputPort*> getInputs(const QString& portname) const;
 
+  /// Returns inputs connected to all input ports connections. 
+  QList<pqOutputPort*> getAllInputs() const;
+
   /// Returns a map of input port name to the list of output ports that are set
   /// as the input to that port.
   QMap<QString, QList<pqOutputPort*> > getNamedInputs() const;
@@ -98,6 +101,9 @@ public:
   /// returns false, otherwise true.
   int replaceInput() const;
 
+  /// Hides the representations for the input if required, in the given view.
+  void hideInputIfRequired(pqView* view);
+
 protected slots:
   /// process some change in the input property for the proxy.
   void inputChanged(vtkObject*, unsigned long, void* client_data);
