@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqPluginManager.h,v $
+   Module:    pqPluginManager.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -34,11 +34,15 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #define _pqPluginManager_h
 
 #include <QObject>
-#include <QMultiMap>
 #include <QStringList>
+#include "vtkSmartPointer.h"
 #include "pqCoreExport.h"
 
+class pqPluginManagerInternal;
 class pqServer;
+class vtkObject;
+class vtkPVGUIPluginInterface;
+class vtkPVPluginInformation;
 
 /// extension manager takes care of loading plugins, xml files, and binary qrc files
 /// the plugins may also contain xml files and qrc files
@@ -56,7 +60,8 @@ public:
   /// return status on success, if NOTLOADED was returned, the error is reported
   /// If errorMsg is non-null, then errors are not reported, but the error
   /// message is put in the errorMsg string
-  LoadStatus loadExtension(pqServer* server, const QString& lib, QString* errorMsg=0);
+  LoadStatus loadExtension(pqServer* server, const QString& lib, 
+    QString* errorMsg=0, bool remote=true);
   
   /// attempt to load all available plugins on a server, 
   /// or client plugins if NULL
@@ -68,11 +73,27 @@ public:
   void loadExtensions(const QString& path, pqServer* server);
   
   /// return all GUI interfaces that have been loaded
-  QObjectList interfaces();
+  QObjectList interfaces() const;
+
+  template <class T>
+    QList<T> findInterfaces() const
+      {
+      QList<T> list;
+      QObjectList objList = this->interfaces();
+      foreach (QObject* object, objList)
+        {
+        if (object && qobject_cast<T>(object))
+          {
+          list.push_back(qobject_cast<T>(object));
+          }
+        }
+      return list;
+      }
 
   /// return all the plugins loaded on a server, or locally if NULL is passed in
-  QStringList loadedExtensions(pqServer*);
-
+  QList< vtkPVPluginInformation* > loadedExtensions(pqServer*);
+  QList< vtkPVPluginInformation* > loadedExtensions(QString extensionKey);
+ 
   /// add an extra interface.
   /// these interfaces are appended to the ones loaded from plugins
   void addInterface(QObject* iface);
@@ -83,6 +104,26 @@ public:
   /// Return all the paths that plugins will be searched for.
   QStringList pluginPaths(pqServer*);
 
+  /// Get the pluginInfo that may already exist in this manager by the file name or plugin name.
+  vtkPVPluginInformation* getExistingExtensionByFileName(pqServer*, const QString& lib);
+  vtkPVPluginInformation* getExistingExtensionByFileName(QString extensionKey, const QString& lib);
+  vtkPVPluginInformation* getExistingExtensionByPluginName(pqServer*, const QString& name);
+  vtkPVPluginInformation* getExistingExtensionByPluginName(QString extensionKey, const QString& name);
+  
+  /// Update the plugin AutoLoad state.
+  void updatePluginAutoLoadState(vtkPVPluginInformation* plInfo, int autoLoad);
+
+  // load plugin settings
+  void addPluginFromSettings();
+  // save plugin settings
+  void savePluginSettings(bool clearFirst = true);
+  // check whether the plugin is ready to run.
+  bool isPluginFuntional(vtkPVPluginInformation* plInfo, bool remote);
+  // exclude an extension from being saved with settings.
+  void removePlugin(pqServer* server, const QString& lib, 
+    bool remote=true);
+ 
+  void loadGUIPlugin(vtkPVGUIPluginInterface*);
 signals:
   /// signal for when an interface is loaded
   void guiInterfaceLoaded(QObject* iface);
@@ -95,25 +136,47 @@ signals:
   /// come from a plugin or an xml file
   void serverManagerExtensionLoaded();
 
+  /// notification when plugin information is updated
+  void pluginInfoUpdated();
+
 protected:
-  LoadStatus loadClientExtension(const QString& lib, QString& error);
-  LoadStatus loadServerExtension(pqServer* server, const QString& lib, QString& error);
+  LoadStatus loadServerExtension(pqServer* server, const QString& lib, 
+    vtkPVPluginInformation* pluginInfo, bool remote);
+  LoadStatus loadClientExtension(const QString& lib,
+    vtkPVPluginInformation* pluginInfo);
 
   // add to the list if it isn't already there
-  void addExtension(pqServer* server, const QString& lib);
-
+  void addExtension(pqServer* server, vtkPVPluginInformation* pluginInfo);
+  void addExtension(QString extensionKey, vtkPVPluginInformation* pluginInfo);
+  
   /// Handles pqAutoStartInterface plugins.
   void handleAutoStartPlugins(QObject* iface, bool startup);
-
+  
+  // check whether the required plugins are functional.
+  bool areRequiredPluginsFunctional(vtkPVPluginInformation* plInfo, bool remote);
+  
 protected slots:
   void onServerConnected(pqServer*);
   void onServerDisconnected(pqServer*);
+  void onSMLoadPluginInvoked(vtkObject* caller, unsigned long vtk_event, void* client_data, void* call_data);
   
 private:
-
+  
+  pqPluginManagerInternal* Internal;
+  // return the extension key for identifying the server, 
+  // used for mapping from server to plugins.
+  QString getServerURIKey(pqServer* server);
+  // return the settings key for the plugin, 
+  // used for saving the plugin settings ,
+  // "[serverURI]###[filename]###[AutoLoad]"
+  QString getPluginSettingsKey(vtkPVPluginInformation*);
+  void processPluginSettings(QString& plSettingKey);
+  // load the Auto Load plugins on that server
+  void loadAutoLoadPlugins(pqServer* server);    
+  
   QObjectList Interfaces;
-  QMultiMap<pqServer*, QString> Extensions;
   QObjectList ExtraInterfaces;
+
 };
 
 #endif
