@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqSMAdaptor.cxx,v $
+   Module:    pqSMAdaptor.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -39,6 +39,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // vtk includes
 #include "vtkConfigure.h"   // for 64-bitness
+#include "vtkStringList.h"
+#include "vtkSmartPointer.h"
 
 // server manager includes
 #include "vtkSMArrayListDomain.h"
@@ -68,9 +70,13 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkSMExtentDomain.h"
 #include "vtkSMFileListDomain.h"
 #include "vtkSMCompositeTreeDomain.h"
+#include "vtkSMSILDomain.h"
+
 // ParaView includes
 #include "pqSMProxy.h"
 
+#include <QStringList>
+
 static const int metaId = qRegisterMetaType<QList<QList<QVariant> > >("ListOfList");
 
 pqSMAdaptor::pqSMAdaptor()
@@ -120,10 +126,15 @@ pqSMAdaptor::PropertyType pqSMAdaptor::getPropertyType(vtkSMProperty* Property)
     vtkSMFileListDomain* fileListDomain = NULL;
     vtkSMStringListDomain* stringListDomain = NULL;
     vtkSMCompositeTreeDomain* compositeTreeDomain = NULL;
+    vtkSMSILDomain* silDomain = NULL;
     
     vtkSMDomainIterator* iter = Property->NewDomainIterator();
     for(iter->Begin(); !iter->IsAtEnd(); iter->Next())
       {
+      if (!silDomain)
+        {
+        silDomain = vtkSMSILDomain::SafeDownCast(iter->GetDomain());
+        }
       if(!stringListRangeDomain)
         {
         stringListRangeDomain = vtkSMStringListRangeDomain::SafeDownCast(iter->GetDomain());
@@ -163,9 +174,14 @@ pqSMAdaptor::PropertyType pqSMAdaptor::getPropertyType(vtkSMProperty* Property)
       {
       type = pqSMAdaptor::COMPOSITE_TREE;
       }
-    else if(stringListRangeDomain || 
+    else if (silDomain)
+      {
+      type = pqSMAdaptor::SIL;
+      }
+    else if(!silDomain && (
+      stringListRangeDomain || 
       (VectorProperty && VectorProperty->GetRepeatCommand() && 
-       (stringListDomain || enumerationDomain)))
+       (stringListDomain || enumerationDomain))))
       {
       type = pqSMAdaptor::SELECTION;
       }
@@ -666,20 +682,31 @@ void pqSMAdaptor::setSelectionProperty(vtkSMProperty* Property,
         return;
         }
       }
-    // not found, just put it in the first empty slot
+
+    // Not found, add it...
+    // We will create a vtkStringList for the name,value pair and then
+    // set the string values in one atomic call.
+    vtkSmartPointer<vtkStringList> stringValues = vtkSmartPointer<vtkStringList>::New();
+    StringProperty->GetElements(stringValues);
+    numElems = stringValues->GetLength();
+
+    // First look for an empty slot to add the values
     for(i=0; i<numElems; i+=2)
       {
       const char* elem = StringProperty->GetElement(i);
       if(!elem || elem[0] == '\0')
         {
-        StringProperty->SetElement(i, name.toAscii().data());
-        StringProperty->SetElement(i+1, valueStr.toAscii().data());
+        stringValues->SetString(i, name.toAscii().data());
+        stringValues->SetString(i+1, valueStr.toAscii().data());
+        StringProperty->SetElements(stringValues);
         return;
         }
       }
-    // If we didn't find any empty spots, append to the vector
-    StringProperty->SetElement(numElems, name.toAscii().data());
-    StringProperty->SetElement(numElems+1, valueStr.toAscii().data());
+
+    // Add the values at the end
+    stringValues->SetString(numElems, name.toAscii().data());
+    stringValues->SetString(numElems+1, valueStr.toAscii().data());
+    StringProperty->SetElements(stringValues);
     }
   else if(EnumerationDomain)
     {
@@ -1724,46 +1751,65 @@ QList<QVariant> pqSMAdaptor::getMultipleElementPropertyDomain(
   return domain;
 }
 
-QString pqSMAdaptor::getFileListProperty(vtkSMProperty* Property)
+QStringList pqSMAdaptor::getFileListProperty(vtkSMProperty* Property)
 {
-  QString file;
+  QStringList files;
 
   vtkSMStringVectorProperty* svp;
   svp = vtkSMStringVectorProperty::SafeDownCast(Property);
 
-  if(svp && svp->GetNumberOfElements() > 0)
+  if (svp)
     {
-    file = svp->GetElement(0);
+    for (unsigned int i = 0; i < svp->GetNumberOfElements(); i++)
+      {
+      files.append(svp->GetElement(i));
+      }
     }
-  return file;
+
+  return files;
 }
 
-void pqSMAdaptor::setFileListProperty(vtkSMProperty* Property, QString Value)
+void pqSMAdaptor::setFileListProperty(vtkSMProperty* Property,
+                                      QStringList Value)
 {
   vtkSMStringVectorProperty* svp;
   svp = vtkSMStringVectorProperty::SafeDownCast(Property);
 
-  if(svp && svp->GetNumberOfElements() > 0)
+  if (!svp) return;
+
+  unsigned int i = 0;
+  foreach (QString file, Value)
     {
-    if(!Value.isNull())
-      {
-      svp->SetElement(0, Value.toAscii().data());
-      }
+    if (!svp->GetRepeatCommand() && (i >= svp->GetNumberOfElements())) break;
+    svp->SetElement(i, file.toAscii().data());
+    i++;
+    }
+
+  if (static_cast<int>(svp->GetNumberOfElements()) != Value.size())
+    {
+    svp->SetNumberOfElements(svp->GetNumberOfElements());
     }
 }
 
 void pqSMAdaptor::setUncheckedFileListProperty(vtkSMProperty* Property,
-                                               QString Value)
+                                               QStringList Value)
 {
   vtkSMStringVectorProperty* svp;
   svp = vtkSMStringVectorProperty::SafeDownCast(Property);
 
-  if(svp && svp->GetNumberOfElements() > 0)
+  if (!svp) return;
+
+  unsigned int i = 0;
+  foreach (QString file, Value)
     {
-    if(!Value.isNull())
-      {
-      svp->SetUncheckedElement(0, Value.toAscii().data());
-      }
+    if (!svp->GetRepeatCommand() && (i >= svp->GetNumberOfElements())) break;
+    svp->SetUncheckedElement(i, file.toAscii().data());
+    i++;
+    }
+
+  if (static_cast<int>(svp->GetNumberOfElements()) != Value.size())
+    {
+    svp->SetNumberOfElements(svp->GetNumberOfElements());
     }
   Property->UpdateDependentDomains();
 }
