@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqServerManagerModel.h,v $
+   Module:    pqServerManagerModel.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -46,7 +46,9 @@ class pqServerManagerModelItem;
 class pqServerManagerObserver;
 class pqServerResource;
 class pqView;
+class vtkPVXMLElement;
 class vtkSMProxy;
+class vtkSMProxyLocator;
 
 //BTX
 class pqServerManagerModel;
@@ -158,15 +160,15 @@ public:
     const QMetaObject &mo, QList<void *> *list, pqServer* server=0);
 
   /// Internal method.
-  static pqProxy* findItemHelper(const pqServerManagerModel* const model, 
+  static pqServerManagerModelItem* findItemHelper(const pqServerManagerModel* const model, 
     const QMetaObject& mo, vtkSMProxy* proxy);
 
   /// Internal method.
-  static pqProxy* findItemHelper(const pqServerManagerModel* const model, 
+  static pqServerManagerModelItem* findItemHelper(const pqServerManagerModel* const model, 
     const QMetaObject& mo, vtkClientServerID id);
 
   /// Internal method.
-  static pqProxy* findItemHelper(const pqServerManagerModel* const model, 
+  static pqServerManagerModelItem* findItemHelper(const pqServerManagerModel* const model, 
     const QMetaObject& mo, const QString& name);
 
 signals:
@@ -238,6 +240,9 @@ signals:
   void preConnectionRemoved(pqPipelineSource* source, 
     pqPipelineSource* consumer, int srcOutputPort);
 
+  /// Fired when a source indicates that data was updated i.e. the pipeline was
+  /// updated.
+  void dataUpdated(pqPipelineSource*);
 
 protected slots:
   /// Called when a proxy is registered.
@@ -254,6 +259,10 @@ protected slots:
   /// Called when a server connection is closed.
   virtual void onConnectionClosed(vtkIdType id);
 
+  /// Called when state file is loaded. We need to discover "helper proxies" and
+  /// set up the associations accordingly.
+  virtual void onStateLoaded(vtkPVXMLElement*, vtkSMProxyLocator*);
+
 private:
   pqServerManagerModel(const pqServerManagerModel&); // Not implemented.
   void operator=(const pqServerManagerModel&); // Not implemented.
