@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqServerResource.cxx,v $
+   Module:    pqServerResource.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -116,7 +116,7 @@ public:
     }
   }
 
-  const bool operator==(const pqImplementation& rhs)
+  bool operator==(const pqImplementation& rhs)
   {
     return this->Scheme == rhs.Scheme
       && this->Host == rhs.Host
@@ -129,12 +129,12 @@ public:
       && this->SessionServer == rhs.SessionServer;
   }
   
-  const bool operator!=(const pqImplementation& rhs)
+  bool operator!=(const pqImplementation& rhs)
   {
     return !(*this == rhs);
   }
   
-  const bool operator<(const pqImplementation& rhs)
+  bool operator<(const pqImplementation& rhs)
   {
     if(this->Scheme != rhs.Scheme)
       return this->Scheme < rhs.Scheme;
@@ -201,7 +201,6 @@ pqServerResource::~pqServerResource()
 const QString pqServerResource::toURI() const
 {
   QString result;
-  
   result += this->Implementation->Scheme + ":";
   
   if(this->Implementation->Scheme == "builtin")
