@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqSpreadSheetViewModel.cxx,v $
+   Module:    pqSpreadSheetViewModel.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -35,7 +35,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkEventQtSlotConnect.h"
 #include "vtkIdList.h"
 #include "vtkIdTypeArray.h"
-#include "vtkIndexBasedBlockFilter.h"
 #include "vtkInformation.h"
 #include "vtkPVArrayInformation.h"
 #include "vtkPVCompositeDataInformation.h"
@@ -43,6 +42,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkPVDataInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
 #include "vtkSelection.h"
+#include "vtkSelectionNode.h"
 #include "vtkSmartPointer.h"
 #include "vtkSMInputProperty.h"
 #include "vtkSMSourceProxy.h"
@@ -79,6 +79,7 @@ public:
   this->ActiveBlockNumber = 0;
   this->Dirty = true;
   this->VTKConnect = vtkSmartPointer<vtkEventQtSlotConnect>::New();
+  this->DecimalPrecision = 6;
   }
 
   QPointer<pqDataRepresentation> DataRepresentation;
@@ -87,7 +88,8 @@ public:
   int NumberOfRows;
   QItemSelectionModel SelectionModel;
   vtkIdType ActiveBlockNumber;
-
+  int DecimalPrecision;
+  
   vtkIdType getBlockSize()
     {
     vtkIdType blocksize = pqSMAdaptor::getElementProperty(
@@ -121,7 +123,32 @@ public:
   int getFieldType()
     {
     return pqSMAdaptor::getElementProperty(
-      this->Representation->GetProperty("FieldType")).toInt();
+      this->Representation->GetProperty("FieldAssociation")).toInt();
+    }
+
+  int getNumberOfRows()
+    {
+    if (this->Representation)
+      {
+      vtkPVDataInformation* info =
+        this->Representation->GetRepresentedDataInformation(true);
+      return info->GetNumberOfRows();
+      }
+    return 0;
+    }
+  
+  int getNumberOfColumns()
+    {
+    if (this->Representation)
+      {
+      vtkTable* table = vtkTable::SafeDownCast(
+        this->Representation->GetOutput(this->ActiveBlockNumber));
+      if (table)
+        {
+        return table->GetNumberOfColumns();
+        }
+      }
+    return 0;
     }
 
   QTimer Timer;
@@ -247,25 +274,15 @@ void pqSpreadSheetViewModel::forceUpdate()
   vtkSMSpreadSheetRepresentationProxy* repr = this->Internal->Representation;
   if (repr)
     {
-    vtkTable* table = vtkTable::SafeDownCast(
-      repr->GetOutput(this->Internal->ActiveBlockNumber));
-    this->Internal->NumberOfRows = repr->GetMaximumNumberOfItems();
-    this->Internal->NumberOfColumns = table? table->GetNumberOfColumns()  :0;
-    if (this->Internal->NumberOfColumns == 0 && this->Internal->ActiveBlockNumber != 0)
+    if (this->Internal->ActiveBlockNumber >= repr->GetNumberOfRequiredBlocks() &&
+      this->Internal->ActiveBlockNumber != 0)
       {
-      // it is possible that the current index is invalid (data size may have
-      // shrunk), update the view once again.
+      // Ensure that the active block number if within range.
       this->Internal->ActiveBlockNumber = 0;
-      this->forceUpdate();
-      }
-    
-    // When SelectionOnly is true, the delivered data has an extra
-    // "vtkOriginalIndices" column that needs to be hidden since it does not
-    // make any sense to the user.
-    if (this->Internal->NumberOfColumns && repr->GetSelectionOnly())
-      {
-      this->Internal->NumberOfColumns--;
       }
+
+    this->Internal->NumberOfRows = this->Internal->getNumberOfRows();
+    this->Internal->NumberOfColumns = this->Internal->getNumberOfColumns();
     }
 
   this->Internal->SelectionModel.clear();
@@ -291,7 +308,8 @@ void pqSpreadSheetViewModel::updateSelectionForBlock(vtkIdType blockNumber)
 {
   vtkSMSpreadSheetRepresentationProxy* repr = this->Internal->Representation;
   if (repr && 
-    this->Internal->getFieldType() != vtkIndexBasedBlockFilter::FIELD)
+    (this->Internal->getFieldType() == vtkDataObject::FIELD_ASSOCIATION_CELLS ||
+    this->Internal->getFieldType() == vtkDataObject::FIELD_ASSOCIATION_POINTS))
     {
     // If we are showing only the selected items, then there's not point in
     // highlighting the selected items, since all items are selected. So we
@@ -362,6 +380,9 @@ void pqSpreadSheetViewModel::delayedSelectionUpdate()
       this->Internal->ActiveBlockNumber = blockNumber;
       this->updateSelectionForBlock(blockNumber);
       }
+    
+    // whether or not to allow further selections
+    emit this->selectionOnly( repr->GetSelectionOnly() );
     }
 }
 
@@ -407,7 +428,7 @@ QVariant pqSpreadSheetViewModel::data(
 
     // If displaying field data, check to make sure that the data is valid
     // since its arrays can be of different lengths
-    if(this->Internal->getFieldType() == vtkIndexBasedBlockFilter::FIELD)
+    if(this->Internal->getFieldType() == vtkDataObject::FIELD_ASSOCIATION_NONE)
       {
       if(!this->isDataValid(idx))
         {
@@ -425,12 +446,65 @@ QVariant pqSpreadSheetViewModel::data(
     if (table)
       {
       vtkVariant value = table->GetValue(blockOffset, column);
+      
       QString str = value.ToString().c_str();
       if (value.IsChar() || value.IsUnsignedChar() || value.IsSignedChar())
         {
         // Don't show ASCII characted for char arrays.
         str = QString::number(value.ToInt());
         }
+      else if(value.IsFloat() || value.IsDouble())
+        {
+        str = QString::number(value.ToDouble(), 'g', 
+              this->Internal->DecimalPrecision);        
+        }
+      else if (value.IsArray())
+        {
+        // it's possible that it's a char array, then too we need to do the
+        // number magic.
+        vtkDataArray* array = vtkDataArray::SafeDownCast(value.ToArray());
+        if (array)
+          {
+          switch(array->GetDataType())
+            {
+            case VTK_CHAR :
+            case VTK_UNSIGNED_CHAR :
+            case VTK_SIGNED_CHAR:
+              {
+              str = QString();
+              for (vtkIdType cc=0; cc < array->GetNumberOfTuples(); cc++)
+                {
+                double *tuple = array->GetTuple(cc);
+                for (vtkIdType kk=0; kk < array->GetNumberOfComponents(); kk++)
+                  {
+                  str += QString::number(static_cast<int>(tuple[kk])) + " ";
+                  }
+                str = str.trimmed();
+                }
+              break;
+              }
+            case VTK_DOUBLE :
+            case VTK_FLOAT :
+              {
+              str = QString();
+              for (vtkIdType cc=0; cc < array->GetNumberOfTuples(); cc++)
+                {
+                double *tuple = array->GetTuple(cc);
+                for (vtkIdType kk=0; kk < array->GetNumberOfComponents(); kk++)
+                  {
+                  str += QString::number(static_cast<double>(tuple[kk]), 'g', 
+                         this->Internal->DecimalPrecision) + " ";
+                  }
+                str = str.trimmed();
+                }
+              break;
+              break;
+              }
+            default :
+              break;
+            }
+          }
+        }
       str.replace(" ", "\t");
       return str;
       }
@@ -472,8 +546,28 @@ QVariant pqSpreadSheetViewModel::headerData (int section, Qt::Orientation orient
         }
       else if (title == "vtkOriginalIndices")
         {
-        title = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
-          "Point ID" : "Cell ID";
+        switch (this->Internal->getFieldType())
+          {
+        case vtkDataObject::FIELD_ASSOCIATION_POINTS:
+          title  = "Point ID";
+          break;
+
+        case vtkDataObject::FIELD_ASSOCIATION_CELLS:
+          title = "Cell ID";
+          break;
+
+        case vtkDataObject::FIELD_ASSOCIATION_VERTICES:
+          title = "Vertex ID";
+          break;
+
+        case vtkDataObject::FIELD_ASSOCIATION_EDGES:
+          title = "Edge ID";
+          break;
+
+        case vtkDataObject::FIELD_ASSOCIATION_ROWS:
+          title = "Row ID";
+          break;
+          }
         }
       else if (title == "vtkOriginalCellIds" && repr->GetSelectionOnly())
         {
@@ -502,7 +596,7 @@ QVariant pqSpreadSheetViewModel::headerData (int section, Qt::Orientation orient
 }
 
 //-----------------------------------------------------------------------------
-QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdType vtkindex)
+QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelectionNode* node, vtkIdType vtkindex)
 {
   vtkSMSpreadSheetRepresentationProxy* repr = 
     this->Internal->Representation;
@@ -514,7 +608,7 @@ QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdTy
   const char* column_name = "vtkOriginalIndices";
   if (repr->GetSelectionOnly())
     {
-    column_name = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
+    column_name = (this->Internal->getFieldType() == vtkDataObject::FIELD_ASSOCIATION_POINTS)?
       "vtkOriginalPointIds" : "vtkOriginalCellIds";
     }
   vtkIdTypeArray* indexcolumn = vtkIdTypeArray::SafeDownCast(
@@ -535,9 +629,9 @@ QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdTy
   vtkIdList* ids = vtkIdList::New();
   indexcolumn->LookupValue(vtkindex, ids);
  
-  if (vtkselection->GetProperties()->Has(vtkSelection::PROCESS_ID()) && pidcolumn)
+  if (node->GetProperties()->Has(vtkSelectionNode::PROCESS_ID()) && pidcolumn)
     {
-    int pid = vtkselection->GetProperties()->Get(vtkSelection::PROCESS_ID());
+    int pid = node->GetProperties()->Get(vtkSelectionNode::PROCESS_ID());
     if (pid != -1)
       {
       // remove those ids from the "ids" list that don't have the same process ID
@@ -557,14 +651,14 @@ QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdTy
       }
     }
 
-  if (vtkselection->GetProperties()->Has(vtkSelection::HIERARCHICAL_LEVEL()) &&
-      vtkselection->GetProperties()->Has(vtkSelection::HIERARCHICAL_INDEX()) &&
+  if (node->GetProperties()->Has(vtkSelectionNode::HIERARCHICAL_LEVEL()) &&
+      node->GetProperties()->Has(vtkSelectionNode::HIERARCHICAL_INDEX()) &&
       compositeIndexColumn && compositeIndexColumn->GetNumberOfComponents() == 2)
     {
     unsigned int hid = static_cast<unsigned int>(
-      vtkselection->GetProperties()->Get(vtkSelection::HIERARCHICAL_INDEX()));
+      node->GetProperties()->Get(vtkSelectionNode::HIERARCHICAL_INDEX()));
     unsigned int hlevel = static_cast<unsigned int>(
-      vtkselection->GetProperties()->Get(vtkSelection::HIERARCHICAL_LEVEL()));
+      node->GetProperties()->Get(vtkSelectionNode::HIERARCHICAL_LEVEL()));
     // remove those ids from the "ids" list that don't have the same process ID
     // as the selection.
     for (vtkIdType cc=0; cc < ids->GetNumberOfIds();)
@@ -582,11 +676,11 @@ QModelIndex pqSpreadSheetViewModel::indexFor(vtkSelection* vtkselection, vtkIdTy
         }
       }
     }
-  else if (vtkselection->GetProperties()->Has(vtkSelection::COMPOSITE_INDEX()) && 
+  else if (node->GetProperties()->Has(vtkSelectionNode::COMPOSITE_INDEX()) && 
     compositeIndexColumn)
     {
     unsigned int cid = static_cast<unsigned int>(
-      vtkselection->GetProperties()->Get(vtkSelection::COMPOSITE_INDEX()));
+      node->GetProperties()->Get(vtkSelectionNode::COMPOSITE_INDEX()));
     // remove those ids from the "ids" list that don't have the same process ID
     // as the selection.
     for (vtkIdType cc=0; cc < ids->GetNumberOfIds();)
@@ -625,51 +719,47 @@ QItemSelection pqSpreadSheetViewModel::convertToQtSelection(vtkSelection* vtksel
     return QItemSelection();
     }
 
-  if (vtkselection->GetContentType() == vtkSelection::SELECTIONS)
+  QItemSelection qSel;
+  for (unsigned int cc=0; cc < vtkselection->GetNumberOfNodes(); cc++)
     {
-    QItemSelection qSel;
-    for (unsigned int cc=0; cc < vtkselection->GetNumberOfChildren(); cc++)
+    vtkSelectionNode* node = vtkselection->GetNode(cc);
+    QItemSelection qSelCur;
+    if (node->GetContentType() == vtkSelectionNode::INDICES)
       {
-      vtkSelection* sel = vtkselection->GetChild(cc);
-      qSel.merge(this->convertToQtSelection(sel), QItemSelectionModel::Select);
-      }
-    return qSel;
-    }
-  else if (vtkselection->GetContentType() == vtkSelection::INDICES)
-    {
-    QItemSelection qSel;
-    // Iterate over all indices in the vtk selection, 
-    // Determine the qt model index for each and then add that to the
-    // qt selection.
-    vtkIdTypeArray *indices = vtkIdTypeArray::SafeDownCast(
-      vtkselection->GetSelectionList());
-    for (vtkIdType cc=0; indices && cc < indices->GetNumberOfTuples(); cc++)
-      {
-      vtkIdType idx = indices->GetValue(cc);
-      QModelIndex qtIndex = this->indexFor(vtkselection, idx);
-      if (qtIndex.isValid())
+      // Iterate over all indices in the vtk selection, 
+      // Determine the qt model index for each and then add that to the
+      // qt selection.
+      vtkIdTypeArray *indices = vtkIdTypeArray::SafeDownCast(
+        node->GetSelectionList());
+      for (vtkIdType i=0; indices && i < indices->GetNumberOfTuples(); i++)
         {
-        // cout << "Selecting: " << qtIndex.row() << endl;
-        qSel.select(qtIndex, qtIndex);
+        vtkIdType idx = indices->GetValue(i);
+        QModelIndex qtIndex = this->indexFor(node, idx);
+        if (qtIndex.isValid())
+          {
+          // cout << "Selecting: " << qtIndex.row() << endl;
+          qSelCur.select(qtIndex, qtIndex);
+          }
         }
       }
-    return qSel;
-    }
-  else if (vtkselection->GetContentType() == vtkSelection::BLOCKS)
-    {
-    QItemSelection qSel;
-    vtkUnsignedIntArray* blocks = vtkUnsignedIntArray::SafeDownCast(
-      vtkselection->GetSelectionList());
-    if (blocks && blocks->GetNumberOfTuples() > 0)
+    else if (node->GetContentType() == vtkSelectionNode::BLOCKS)
       {
-      qSel.select(this->createIndex(0, 0),
-        this->createIndex(this->rowCount()-1, 0));
+      vtkUnsignedIntArray* blocks = vtkUnsignedIntArray::SafeDownCast(
+        node->GetSelectionList());
+      if (blocks && blocks->GetNumberOfTuples() > 0)
+        {
+        qSelCur.select(this->createIndex(0, 0),
+          this->createIndex(this->rowCount()-1, 0));
+        }
+      }
+    else
+      {
+      qCritical() << "Unknown selection object.";
       }
-    return qSel;
-    }
-  qCritical() << "Unknown selection object.";
-  return QItemSelection();
 
+    qSel.merge(qSelCur, QItemSelectionModel::Select);
+    }
+  return qSel;
 }
 
 //-----------------------------------------------------------------------------
@@ -706,7 +796,7 @@ QSet<pqSpreadSheetViewModel::vtkIndex> pqSpreadSheetViewModel::getVTKIndices(
       const char* column_name = "vtkOriginalIndices";
       if (repr->GetSelectionOnly())
         {
-        column_name = (this->Internal->getFieldType() == vtkIndexBasedBlockFilter::POINT)?
+        column_name = (this->Internal->getFieldType() == vtkDataObject::FIELD_ASSOCIATION_POINTS)?
           "vtkOriginalPointIds" : "vtkOriginalCellIds";
         }
 
@@ -767,28 +857,15 @@ bool pqSpreadSheetViewModel::isDataValid( const QModelIndex &idx) const
       inputProxy->GetDataInformation(port) : 0;
 
     // Get the appropriate attribute information object
-    vtkPVDataSetAttributesInformation *attrInfo = NULL;
-    if (info)
-      {
-      if (field_type == vtkIndexBasedBlockFilter::FIELD)
-        {
-        attrInfo = info->GetFieldDataInformation();
-        }
-      else if (field_type == vtkIndexBasedBlockFilter::POINT)
-        {
-        attrInfo = info->GetPointDataInformation();
-        }
-      else if (field_type == vtkIndexBasedBlockFilter::CELL)
-        {
-        attrInfo = info->GetCellDataInformation();
-        }
-      }
+    vtkPVDataSetAttributesInformation *attrInfo = info?
+      info->GetAttributeInformation(field_type) : 0;
  
     if(attrInfo)
       {
       // Ensure that the row of this index is less than the length of the 
       // data array associated with its column
-      vtkPVArrayInformation *arrayInfo = attrInfo->GetArrayInformation(table->GetColumnName(idx.column()));
+      vtkPVArrayInformation *arrayInfo = attrInfo->GetArrayInformation(
+        table->GetColumnName(idx.column()));
       if(arrayInfo && idx.row() < arrayInfo->GetNumberOfTuples())
         {
         return true;
@@ -802,6 +879,11 @@ bool pqSpreadSheetViewModel::isDataValid( const QModelIndex &idx) const
 //-----------------------------------------------------------------------------
 void pqSpreadSheetViewModel::resetCompositeDataSetIndex()
 {
+  if (!this->getRepresentation())
+    {
+    return;
+    }
+
   vtkSMProxy* reprProxy = this->getRepresentationProxy();
   int cur_index = pqSMAdaptor::getElementProperty(
     reprProxy->GetProperty("CompositeDataSetIndex")).toInt();
@@ -852,3 +934,19 @@ void pqSpreadSheetViewModel::resetCompositeDataSetIndex()
     reprProxy->GetProperty("CompositeDataSetIndex"), cur_index);
   reprProxy->UpdateVTKObjects();
 }
+   
+//-----------------------------------------------------------------------------
+void pqSpreadSheetViewModel::setDecimalPrecision(int dPrecision)
+{
+  if(this->Internal->DecimalPrecision != dPrecision)
+    {
+    this->Internal->DecimalPrecision = dPrecision;
+    this->forceUpdate();
+    }
+}
+
+//-----------------------------------------------------------------------------
+int pqSpreadSheetViewModel::getDecimalPrecision()
+{
+  return this->Internal->DecimalPrecision;
+}
