@@ -1,13 +1,13 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqStandardViewModules.cxx,v $
+   Module:    pqStandardViewModules.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
 
    ParaView is a free software; you can redistribute it and/or modify it
-   under the terms of the ParaView license version 1.2. 
+   under the terms of the ParaView license version 1.2.
 
    See License_v1.2.txt for the full ParaView license.
    A copy of this license can be obtained by contacting
@@ -32,20 +32,27 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "pqStandardViewModules.h"
 
-#include "vtkSMProxyManager.h"
-#include "vtkSMRenderViewProxy.h"
-
-#include "pqBarChartRepresentation.h"
+#include "pqChartRepresentation.h"
+#include "pqComparativeBarChartView.h"
+#include "pqComparativeLineChartView.h"
 #include "pqComparativeRenderView.h"
-#include "pqComparativePlotView.h"
-#include "pqLineChartRepresentation.h"
-#include "pqPlotView.h"
 #include "pqRenderView.h"
 #include "pqServer.h"
 #include "pqSpreadSheetView.h"
 #include "pqTableView.h"
 #include "pqTextRepresentation.h"
 #include "pqTwoDRenderView.h"
+#include "pqScatterPlotView.h"
+#include "pqXYChartView.h"
+#include "pqXYBarChartView.h"
+#include "vtkSMChartViewProxy.h"
+#include "vtkSMContextViewProxy.h"
+#include "vtkSMXYChartViewProxy.h"
+#include "vtkSMComparativeViewProxy.h"
+#include "vtkSMProxyManager.h"
+#include "vtkSMRenderViewProxy.h"
+
+#include <QDebug>
 
 pqStandardViewModules::pqStandardViewModules(QObject* o)
   : QObject(o)
@@ -58,23 +65,24 @@ pqStandardViewModules::~pqStandardViewModules()
 
 QStringList pqStandardViewModules::viewTypes() const
 {
-  return QStringList() << 
-    pqRenderView::renderViewType() << 
+  return QStringList() <<
+    pqRenderView::renderViewType() <<
     pqTwoDRenderView::twoDRenderViewType() <<
-    pqPlotView::barChartType() << 
-    pqPlotView::XYPlotType() << 
     pqTableView::tableType() <<
+    pqSpreadSheetView::spreadsheetViewType() <<
+    pqScatterPlotView::scatterPlotViewType() <<
+    pqXYChartView::XYChartViewType() <<
+    pqXYBarChartView::XYBarChartViewType() <<
     pqComparativeRenderView::comparativeRenderViewType() <<
-    pqComparativePlotView::comparativeXYPlotViewType() <<
-    pqComparativePlotView::comparativeBarChartViewType() <<
-    pqSpreadSheetView::spreadsheetViewType();
+    pqComparativeBarChartView::comparativeBarChartViewType() <<
+    pqComparativeLineChartView::comparativeLineChartViewType();
 }
 
 QStringList pqStandardViewModules::displayTypes() const
 {
-  return QStringList() 
-    << "BarChartRepresentation"
-    << "XYPlotRepresentation"
+  return QStringList()
+    << "XYChartRepresentation"
+    << "XYBarChartRepresentation"
     << "TextSourceRepresentation";
 }
 
@@ -84,14 +92,6 @@ QString pqStandardViewModules::viewTypeName(const QString& type) const
     {
     return pqRenderView::renderViewTypeName();
     }
-  else if(type == pqPlotView::barChartType())
-    {
-    return pqPlotView::barChartTypeName();
-    }
-  else if(type == pqPlotView::XYPlotType())
-    {
-    return pqPlotView::XYPlotTypeName();
-    }
   else if(type == pqTableView::tableType())
     {
     return pqTableView::tableTypeName();
@@ -100,13 +100,13 @@ QString pqStandardViewModules::viewTypeName(const QString& type) const
     {
     return pqComparativeRenderView::comparativeRenderViewTypeName();
     }
-  else if (type == pqComparativePlotView::comparativeXYPlotViewType())
+  else if (type == pqComparativeBarChartView::comparativeBarChartViewType())
     {
-    return pqComparativePlotView::comparativeXYPlotViewTypeName();
+    return pqComparativeBarChartView::comparativeBarChartViewTypeName();
     }
-  else if (type == pqComparativePlotView::comparativeBarChartViewType())
+  else if (type == pqComparativeLineChartView::comparativeLineChartViewType())
     {
-    return pqComparativePlotView::comparativeBarChartViewTypeName();
+    return pqComparativeLineChartView::comparativeLineChartViewTypeName();
     }
   else if (type == pqSpreadSheetView::spreadsheetViewType())
     {
@@ -116,6 +116,18 @@ QString pqStandardViewModules::viewTypeName(const QString& type) const
     {
     return pqTwoDRenderView::twoDRenderViewTypeName();
     }
+  else if  (type == pqScatterPlotView::scatterPlotViewType())
+    {
+    return pqScatterPlotView::scatterPlotViewTypeName();
+    }
+  else if (type == pqXYChartView::XYChartViewType())
+    {
+    return pqXYChartView::XYChartViewTypeName();
+    }
+  else if (type == pqXYBarChartView::XYBarChartViewType())
+    {
+    return pqXYBarChartView::XYBarChartViewTypeName();
+    }
 
   return QString();
 }
@@ -138,26 +150,18 @@ vtkSMProxy* pqStandardViewModules::createViewProxy(const QString& viewtype,
     {
     root_xmlname = "ComparativeRenderView";
     }
-  else if(viewtype == pqComparativePlotView::comparativeXYPlotViewType())
-    {
-    root_xmlname = "ComparativeXYPlotView";
-    }
-  else if(viewtype == pqComparativePlotView::comparativeBarChartViewType())
+  else if(viewtype == pqComparativeBarChartView::comparativeBarChartViewType())
     {
     root_xmlname = "ComparativeBarChartView";
     }
+  else if(viewtype == pqComparativeLineChartView::comparativeLineChartViewType())
+    {
+    root_xmlname = "ComparativeXYPlotView";
+    }
   else if (viewtype == pqTwoDRenderView::twoDRenderViewType())
     {
     root_xmlname = "2DRenderView";
     }
-  else if(viewtype == pqPlotView::barChartType())
-    {
-    root_xmlname = "BarChartView";
-    }
-  else if(viewtype == pqPlotView::XYPlotType())
-    {
-    root_xmlname = "XYPlotView";
-    }
   else if(viewtype == pqTableView::tableType())
     {
     root_xmlname = "TableView";
@@ -166,6 +170,18 @@ vtkSMProxy* pqStandardViewModules::createViewProxy(const QString& viewtype,
     {
     root_xmlname = "SpreadSheetView";
     }
+  else if (viewtype == pqScatterPlotView::scatterPlotViewType())
+    {
+    root_xmlname = "ScatterPlotRenderView";
+    }
+  else if (viewtype == pqXYChartView::XYChartViewType())
+    {
+    root_xmlname = "XYChartView";
+    }
+  else if (viewtype == pqXYBarChartView::XYBarChartViewType())
+    {
+    root_xmlname = "XYBarChartView";
+    }
 
   if (root_xmlname)
     {
@@ -183,27 +199,16 @@ vtkSMProxy* pqStandardViewModules::createViewProxy(const QString& viewtype,
 
 
 pqView* pqStandardViewModules::createView(const QString& viewtype,
-                                                const QString& group,
-                                                const QString& viewname,
-                                                vtkSMViewProxy* viewmodule,
-                                                pqServer* server,
-                                                QObject* p)
+                                          const QString& group,
+                                          const QString& viewname,
+                                          vtkSMViewProxy* viewmodule,
+                                          pqServer* server,
+                                          QObject* p)
 {
-  if(viewtype == pqPlotView::barChartType())
-    {
-    return new pqPlotView(pqPlotView::barChartType(),
-                              group, viewname, viewmodule, server, p);
-    }
-  else if(viewtype == pqPlotView::XYPlotType())
-    {
-    return new pqPlotView(pqPlotView::XYPlotType(),
-                              group, viewname, viewmodule, server, p);
-    }
-  else if(viewtype == "TableView")
+  if(viewtype == "TableView")
     {
     // return new pqTableView(group, viewname, viewmodule, server, p);
     }
-
   else if (viewtype == pqSpreadSheetView::spreadsheetViewType())
     {
     return new pqSpreadSheetView(
@@ -213,15 +218,19 @@ pqView* pqStandardViewModules::createView(const QString& viewtype,
     {
     return new pqRenderView(group, viewname, viewmodule, server, p);
     }
-  else if (viewtype == pqComparativePlotView::comparativeBarChartViewType())
+  else if (viewtype == pqComparativeBarChartView::comparativeBarChartViewType()
+    && viewmodule->IsA("vtkSMComparativeViewProxy"))
     {
-    return new pqComparativePlotView(pqPlotView::barChartType(),
-                 group, viewname, viewmodule, server, p);
+    return new pqComparativeBarChartView(
+      group, viewname, vtkSMComparativeViewProxy::SafeDownCast(viewmodule),
+      server, p);
     }
-  else if (viewtype == pqComparativePlotView::comparativeXYPlotViewType())
+  else if (viewtype == pqComparativeLineChartView::comparativeLineChartViewType()
+    && viewmodule->IsA("vtkSMComparativeViewProxy"))
     {
-    return new pqComparativePlotView(pqPlotView::XYPlotType(),
-                 group, viewname, viewmodule, server, p);
+    return new pqComparativeLineChartView(
+      group, viewname, vtkSMComparativeViewProxy::SafeDownCast(viewmodule),
+      server, p);
     }
   else if (viewmodule->IsA("vtkSMComparativeViewProxy"))
     {
@@ -232,26 +241,44 @@ pqView* pqStandardViewModules::createView(const QString& viewtype,
   else if (viewmodule->IsA("vtkSMTwoDRenderViewProxy"))
     {
     return new pqTwoDRenderView(
-      group, viewname, viewmodule, server, p); 
+      group, viewname, viewmodule, server, p);
+    }
+  else if (viewmodule->IsA("vtkSMScatterPlotViewProxy"))
+    {
+    return new pqScatterPlotView(
+      group, viewname, viewmodule, server, p);
+    }
+  else if (viewmodule->IsA("vtkSMXYChartViewProxy") &&
+           viewtype == "XYChartView")
+    {
+    return new pqXYChartView(group, viewname,
+                            vtkSMXYChartViewProxy::SafeDownCast(viewmodule),
+                            server, p);
+    }
+  else if (viewmodule->IsA("vtkSMXYChartViewProxy") &&
+           viewtype == "XYBarChartView")
+    {
+    return new pqXYBarChartView(group, viewname,
+                                vtkSMXYChartViewProxy::SafeDownCast(viewmodule),
+                                server, p);
     }
 
+  qDebug() << "Failed to create a proxy" << viewmodule->GetClassName();
   return NULL;
 }
 
-pqDataRepresentation* pqStandardViewModules::createDisplay(const QString& display_type, 
+pqDataRepresentation* pqStandardViewModules::createDisplay(const QString& display_type,
   const QString& group,
   const QString& n,
   vtkSMProxy* proxy,
   pqServer* server,
   QObject* p)
 {
-  if(display_type == "BarChartRepresentation")
+  if (display_type == "XYChartRepresentation" ||
+      display_type == "XYBarChartRepresentation")
     {
-    return new pqBarChartRepresentation(group, n, proxy, server, p);
-    }
-  else if (display_type == "XYPlotRepresentation")
-    {
-    return new pqLineChartRepresentation(group, n, proxy, server, p);
+    // new chart representations.
+    return new pqChartRepresentation(group, n, proxy, server, p);
     }
   else if (display_type == "TextSourceRepresentation")
     {
@@ -260,5 +287,3 @@ pqDataRepresentation* pqStandardViewModules::createDisplay(const QString& displa
 
   return NULL;
 }
-
-
