@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqView.h,v $
+   Module:    pqView.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -44,7 +44,9 @@ class pqUndoStack;
 class pqViewInternal;
 class QWidget;
 class vtkImageData;
+class vtkSMSourceProxy;
 class vtkSMViewProxy;
+class vtkView;
 
 /// This is a PQ abstraction of a generic view module. Subclasses can be
 /// specific for different types of view such as render view, histogram view
@@ -59,6 +61,10 @@ public:
   /// Returns the internal render Module proxy associated with this object.
   vtkSMViewProxy* getViewProxy() const;
 
+  /// Return the client-side vtkView encapsulated by this view (if any),
+  /// or return NULL.
+  virtual vtkView* getClientSideView() const;
+
   /// Return a widget associated with this view
   virtual QWidget* getWidget() = 0;
 
@@ -69,11 +75,6 @@ public:
   /// all undo related signals defined by this class.
   virtual bool supportsUndo() const { return false; }
 
-  /// Returns if this view module can support lookmarks.
-  /// Returns false by default. Subclassess must override
-  /// if that's not the case.
-  virtual bool supportsLookmarks() const { return false; }
-
   /// Returns the type of this view module.
   QString getViewType() const
     { return this->ViewType; }
@@ -96,6 +97,11 @@ public slots:
   /// View modules supporting interaction undo must override this method.
   virtual void redo() {}
 
+  /// Called to reset the view's display.
+  /// For example, reset the camera or zoom level.
+  /// The default implementation does nothing, but subclasses may override.
+  virtual void resetDisplay() {}
+
 public:
   /// Returns true if undo can be done.
   virtual bool canUndo() const {return false;}
@@ -110,7 +116,7 @@ public:
 
   /// Returns the current size of the rendering context.
   /// Default implementation returns the client size ofthe widget. Subclasses
-  /// may override to change this behaviour.
+  /// may override to change this behavior.
   virtual QSize getSize();
 
   /// Capture the view image into a new vtkImageData with the given magnification
@@ -124,7 +130,7 @@ public:
   /// Returns the number representations in the view.
   int getNumberOfRepresentations() const;
 
-  // Returns the number of representations currently visibile in the view.
+  // Returns the number of representations currently visible in the view.
   int getNumberOfVisibleRepresentations() const;
 
   /// Returns the representation for the specified index where
@@ -140,6 +146,11 @@ public:
   /// are queries it.
   virtual bool canDisplay(pqOutputPort* opPort) const =0;
 
+  /// The annotation link used on representations to share the selection
+  /// and annotations. This is only used in views aware of this link.
+  virtual void setAnnotationLink(vtkSMSourceProxy* link);
+  vtkSMSourceProxy* getAnnotationLink();
+
 signals:
   /// Fired after a representation has been added to this view.
   void representationAdded(pqRepresentation*);
