@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqPythonShell.cxx,v $
+   Module:    pqPythonShell.cxx
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -41,12 +41,81 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "vtkPVOptions.h"
 #include "vtkEventQtSlotConnect.h"
 #include "vtkPVPythonInteractiveInterpretor.h"
+#include "vtkStdString.h"
 
 #include <QCoreApplication>
 #include <QResizeEvent>
+#include <QScrollBar>
+#include <QStringListModel>
 #include <QTextCharFormat>
 #include <QVBoxLayout>
 
+
+class pqPythonShellCompleter : public pqConsoleWidgetCompleter
+{
+public:
+  pqPythonShellCompleter(pqPythonShell& p) : Parent(p)
+    {
+    this->setParent(&p);
+    }
+
+  virtual void updateCompletionModel(const QString& completion)
+    {
+    // Start by clearing the model
+    this->setModel(0);
+
+    // Don't try to complete the empty string
+    if (completion.isEmpty())
+      {
+      return;
+      }
+
+    // Search backward through the string for usable characters
+    QString textToComplete;
+    for (int i = completion.length()-1; i >= 0; --i)
+      {
+      QChar c = completion.at(i);
+      if (c.isLetterOrNumber() || c == '.' || c == '_')
+        {
+        textToComplete.prepend(c);
+        }
+      else
+        {
+        break;
+        }
+      }
+
+    // Split the string at the last dot, if one exists
+    QString lookup;
+    QString compareText = textToComplete;
+    int dot = compareText.lastIndexOf('.');
+    if (dot != -1)
+      {
+      lookup = compareText.mid(0, dot);
+      compareText = compareText.mid(dot+1);
+      }
+
+    // Lookup python names
+    QStringList attrs;
+    if (!lookup.isEmpty() || !compareText.isEmpty())
+      {
+      attrs = Parent.getPythonAttributes(lookup);
+      }
+
+    // Initialize the completion model
+    if (!attrs.isEmpty())
+      {
+      this->setCompletionMode(QCompleter::PopupCompletion);
+      this->setModel(new QStringListModel(attrs, this));
+      this->setCaseSensitivity(Qt::CaseInsensitive);
+      this->setCompletionPrefix(compareText.toLower());
+      this->popup()->setCurrentIndex(this->completionModel()->index(0, 0));
+      }
+    }
+  pqPythonShell& Parent;
+};
+
+
 /////////////////////////////////////////////////////////////////////////
 // pqPythonShell::pqImplementation
 
@@ -119,7 +188,7 @@ struct pqPythonShell::pqImplementation
       this->Interpreter->Push(Command.toAscii().data());
   }
 
-  void promptForInput()
+  void promptForInput(const QString& indent=QString())
   {
     QTextCharFormat format = this->Console.getFormat();
     format.setForeground(QColor(0, 0, 0));
@@ -128,12 +197,15 @@ struct pqPythonShell::pqImplementation
     this->Interpreter->MakeCurrent();
     if(!this->MultilineStatement)
       {
-      this->Console.prompt(PyString_AsString(PySys_GetObject(const_cast<char*>("ps1"))));
+      this->Console.prompt(
+        PyString_AsString(PySys_GetObject(const_cast<char*>("ps1"))));
       }
     else
       {
-      this->Console.prompt(PyString_AsString(PySys_GetObject(const_cast<char*>("ps2"))));
+      this->Console.prompt(
+        PyString_AsString(PySys_GetObject(const_cast<char*>("ps2"))));
       }
+    this->Console.printCommand(indent);
     this->Interpreter->ReleaseControl();
   }
 
@@ -162,6 +234,9 @@ pqPythonShell::pqPythonShell(QWidget* Parent) :
   boxLayout->addWidget(&this->Implementation->Console);
 
   this->setObjectName("pythonShell");
+
+  pqPythonShellCompleter* completer = new pqPythonShellCompleter(*this);
+  this->Implementation->Console.setCompleter(completer);
   
   QObject::connect(
     &this->Implementation->Console, SIGNAL(executeCommand(const QString&)), 
@@ -198,6 +273,10 @@ void pqPythonShell::initializeInterpretor(int argc, char* argv[])
   this->Implementation->VTKConnect->Connect(
     this->Implementation->Interpreter, vtkCommand::WarningEvent, 
     this, SLOT(printStdout(vtkObject*, unsigned long, void*, void*))); 
+
+  this->Implementation->VTKConnect->Connect(
+    this->Implementation->Interpreter, vtkCommand::UpdateEvent, 
+    this, SLOT(readInputLine(vtkObject*, unsigned long, void*, void*))); 
 }
 
 void pqPythonShell::clear()
@@ -206,6 +285,15 @@ void pqPythonShell::clear()
   this->Implementation->promptForInput();
 }
 
+void pqPythonShell::makeCurrent()
+{
+  this->Implementation->Interpreter->MakeCurrent();
+}
+void pqPythonShell::releaseControl()
+{
+  this->Implementation->Interpreter->ReleaseControl();
+}
+
 void pqPythonShell::executeScript(const QString& script)
 {
   this->printStdout("\n");
@@ -216,6 +304,67 @@ void pqPythonShell::executeScript(const QString& script)
   this->Implementation->promptForInput();
 }
 
+QStringList pqPythonShell::getPythonAttributes(const QString& objectName)
+{
+  this->makeCurrent();
+
+  PyObject* dict = PyImport_GetModuleDict();
+  PyObject* object = PyDict_GetItemString(dict, "__main__");
+  Py_INCREF(object);
+
+
+  if (!objectName.isEmpty())
+    {
+    QStringList tmpNames = objectName.split('.');
+    for (int i = 0; i < tmpNames.size() && object; ++i)
+      {
+      QByteArray tmpName = tmpNames.at(i).toLatin1();
+      PyObject* prevObj = object;
+      if (PyDict_Check(object))
+        {
+        object = PyDict_GetItemString(object, tmpName.data());
+        Py_XINCREF(object);
+        }
+      else
+        {
+        object = PyObject_GetAttrString(object, tmpName.data());
+        }
+      Py_DECREF(prevObj);
+      }
+    PyErr_Clear();
+    }
+
+  QStringList results;
+  if (object)
+    {
+    PyObject* keys = PyObject_Dir(object);
+    if (keys)
+      {
+      PyObject* key;
+      PyObject* value;
+      QString keystr;
+      int nKeys = PyList_Size(keys);
+      for (int i = 0; i < nKeys; ++i)
+        {
+        key = PyList_GetItem(keys, i);
+        value = PyObject_GetAttr(object, key);
+        if (!value)
+          {
+          continue;
+          }
+
+        results << PyString_AsString(key);
+        Py_DECREF(value);
+        }
+      Py_DECREF(keys);
+      }
+    Py_DECREF(object);
+    }
+
+  this->releaseControl();
+  return results;
+}
+
 void pqPythonShell::printStdout(vtkObject*, unsigned long, void*, void* calldata)
 {
   const char* text = reinterpret_cast<const char*>(calldata);
@@ -223,6 +372,32 @@ void pqPythonShell::printStdout(vtkObject*, unsigned long, void*, void* calldata
   this->Implementation->Interpreter->ClearMessages();
 }
 
+#include <QDialog>
+#include <QLineEdit>
+#include <QVBoxLayout>
+class LineInput : public QDialog {
+public:
+  LineInput(QWidget* parent=NULL) : QDialog(parent, Qt::FramelessWindowHint)
+  {
+    QVBoxLayout* layout = new QVBoxLayout(this);
+    layout->setMargin(0);
+    this->Edit = new QLineEdit;
+    layout->addWidget(this->Edit);
+    this->connect(this->Edit, SIGNAL(returnPressed()), SLOT(accept()));
+  }
+  QLineEdit* Edit;
+};
+
+void pqPythonShell::readInputLine(vtkObject*, unsigned long, void*, void* calldata)
+{
+  vtkStdString* ret = reinterpret_cast<vtkStdString*>(calldata);
+  QPoint pos = this->Implementation->Console.getCursorPosition();
+  LineInput input(this);
+  input.move(this->mapToGlobal(pos));
+  input.exec();
+  *ret = input.Edit->text().toAscii().data();
+}
+
 void pqPythonShell::printStderr(vtkObject*, unsigned long, void*, void* calldata)
 {
   const char* text = reinterpret_cast<const char*>(calldata);
@@ -266,7 +441,15 @@ void pqPythonShell::onExecuteCommand(const QString& Command)
   QString command = Command;
   command.replace(QRegExp("\\s*$"), "");
   this->internalExecuteCommand(command);
-  this->promptForInput();
+
+  // Find the indent for the command.
+  QRegExp regExp("^(\\s+)");
+  QString indent;
+  if (regExp.indexIn(command) != -1)
+    {
+    indent = regExp.cap(1);
+    }
+  this->Implementation->promptForInput(indent);
 }
 
 void pqPythonShell::promptForInput()
