@@ -1,7 +1,7 @@
 /*=========================================================================
 
    Program: ParaView
-   Module:    $RCSfile: pqConsoleWidget.h,v $
+   Module:    pqConsoleWidget.h
 
    Copyright (c) 2005-2008 Sandia Corporation, Kitware Inc.
    All rights reserved.
@@ -37,14 +37,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <QWidget>
 #include <QTextCharFormat>
+#include <QCompleter>
+
+class pqConsoleWidgetCompleter;
 
 /**
-  Qt widget that provides an interactive console - you can send text to the console by calling printString(),
-  and receive user input by connecting to the executeCommand() slot.
+  Qt widget that provides an interactive console - you can send text to the
+  console by calling printString() and receive user input by connecting to the
+  executeCommand() slot.
   
   \sa pqPythonShell, pqOutputWindow
 */
-  
 class QTWIDGETS_EXPORT pqConsoleWidget :
   public QWidget
 {
@@ -58,6 +61,11 @@ public:
   QTextCharFormat getFormat();
   /// Sets formatting that will be used by printString
   void setFormat(const QTextCharFormat& Format);
+
+  /// Set a completer for this console widget
+  void setCompleter(pqConsoleWidgetCompleter* completer);
+
+  QPoint getCursorPosition();
   
 signals:
   /// Signal emitted whenever the user enters a command
@@ -67,6 +75,10 @@ public slots:
   /// Writes the supplied text to the console
   void printString(const QString& Text);
 
+  /// Updates the current command. Unlike printString, this will affect the
+  /// current command being typed.
+  void printCommand(const QString& cmd);
+
   /// Clears the contents of the console
   void clear();
 
@@ -75,6 +87,12 @@ public slots:
   /// an input prompt since this call ensures that the prompt is shown on a new
   /// line.
   void prompt(const QString& text);
+
+  /// Inserts the given completion string at the cursor.  This will replace
+  /// the current word that the cursor is touching with the given text.
+  /// Determines the word using QTextCursor::StartOfWord, EndOfWord.
+  void insertCompletion(const QString& text);
+
 private:
   pqConsoleWidget(const pqConsoleWidget&);
   pqConsoleWidget& operator=(const pqConsoleWidget&);
@@ -86,5 +104,17 @@ private:
   friend class pqImplementation;
 };
 
+
+class QTWIDGETS_EXPORT pqConsoleWidgetCompleter : public QCompleter
+{
+public:
+
+  /// Update the completion model given a string.  The given string
+  /// is the current console text between the cursor and the start of
+  /// the line.
+  virtual void updateCompletionModel(const QString& str) = 0;
+};
+
+
 #endif // !_pqConsoleWidget_h
 
