@@ -1,7 +1,7 @@
 /*=========================================================================
   
   Program:   ParaView
-  Module:    $RCSfile: vtkCommandOptions.h,v $
+  Module:    vtkCommandOptions.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -31,7 +31,7 @@ class VTK_EXPORT vtkCommandOptions : public vtkObject
 {
 public:
   static vtkCommandOptions* New();
-  vtkTypeRevisionMacro(vtkCommandOptions,vtkObject);
+  vtkTypeMacro(vtkCommandOptions,vtkObject);
   void PrintSelf(ostream& os, vtkIndent indent);
 
 //BTX
@@ -92,6 +92,11 @@ protected:
   // Destructor.
   virtual ~vtkCommandOptions();
 
+  // Description:
+  // Prototype for callbacks.
+  typedef int(*CallbackType)(const char* argument, const char* value, 
+    void* call_data);
+
   // Description:
   // Add a command line option.  For each argument added there is a long
   // version --long and a short version -l, a help string, and a variable
@@ -109,6 +114,10 @@ protected:
                    int* var, const char* help, int type=EVERYBODY);
   void AddArgument(const char* longarg, const char* shortarg,
                    char** var, const char* help, int type=EVERYBODY);
+
+  void AddCallback(const char* longarg, const char* shortarg,
+    CallbackType callback, void* call_data, const char* help,
+    int type=EVERYBODY);
   
   // Description:
   // Initialize arguments.
