@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVDataInformation.cxx,v $
+  Module:    vtkPVDataInformation.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -25,10 +25,12 @@
 #include "vtkDataArray.h"
 #include "vtkDataObjectTypes.h"
 #include "vtkDataSet.h"
+#include "vtkExecutive.h"
 #include "vtkGenericDataSet.h"
 #include "vtkGraph.h"
 #include "vtkImageData.h"
 #include "vtkInformation.h"
+#include "vtkMath.h"
 #include "vtkMultiProcessController.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
@@ -43,11 +45,11 @@
 #include "vtkTable.h"
 #include "vtkUniformGrid.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
+#include "vtkPriorityHelper.h"
 
 #include <vtkstd/vector>
 
 vtkStandardNewMacro(vtkPVDataInformation);
-vtkCxxRevisionMacro(vtkPVDataInformation, "$Revision: 1.57 $");
 
 //----------------------------------------------------------------------------
 vtkPVDataInformation::vtkPVDataInformation()
@@ -152,6 +154,34 @@ void vtkPVDataInformation::PrintSelf(ostream& os, vtkIndent indent)
      << endl;
 }
 
+//----------------------------------------------------------------------------
+vtkPVDataSetAttributesInformation*
+vtkPVDataInformation::GetAttributeInformation(int fieldAssociation)
+{
+  switch (fieldAssociation)
+    {
+  case vtkDataObject::FIELD_ASSOCIATION_POINTS:
+    return this->PointDataInformation;
+
+  case vtkDataObject::FIELD_ASSOCIATION_CELLS:
+    return this->CellDataInformation;
+
+  case vtkDataObject::FIELD_ASSOCIATION_VERTICES:
+    return this->VertexDataInformation;
+
+  case vtkDataObject::FIELD_ASSOCIATION_EDGES:
+    return this->EdgeDataInformation;
+
+  case vtkDataObject::FIELD_ASSOCIATION_ROWS:
+    return this->RowDataInformation;
+
+  case vtkDataObject::FIELD_ASSOCIATION_NONE:
+    return this->FieldDataInformation;
+    }
+
+  return 0;
+}
+
 //----------------------------------------------------------------------------
 void vtkPVDataInformation::Initialize()
 {
@@ -535,6 +565,21 @@ void vtkPVDataInformation::CopyFromObject(vtkObject* object)
       dobj = algOutput->GetProducer()->GetOutputDataObject(
         algOutput->GetIndex());
       }
+    else
+      {
+      //Streaming ParaView puts this helper filter into the pipeline.
+      //The helper prevent the whole_extent from being requested and makes
+      //sure that the sub extent that is requested is worthwhile.
+      vtkPriorityHelper *helper = vtkPriorityHelper::SafeDownCast(object);
+      if (helper)
+        {
+        dobj = helper->ConditionallyGetDataObject();
+        if (!dobj)
+          {
+          return;
+          }
+        }
+      }
     }
 
   if (!dobj)
@@ -960,37 +1005,37 @@ void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
 
   this->PointArrayInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->PointDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->CellDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->VertexDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->EdgeDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->RowDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   *css << this->CompositeDataClassName;
   *css << this->CompositeDataSetType;
@@ -999,13 +1044,13 @@ void vtkPVDataInformation::CopyToStream(vtkClientServerStream* css)
 
   this->CompositeDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   dcss.Reset();
 
   this->FieldDataInformation->CopyToStream(&dcss);
   dcss.GetData(&data, &length);
-  *css << vtkClientServerStream::InsertArray(data, length);
+  *css << vtkClientServerStream::InsertArray(data, static_cast<int>(length));
 
   *css << vtkClientServerStream::InsertArray(this->TimeSpan, 2);
 
