@@ -1,7 +1,7 @@
 /*=========================================================================
   
   Program:   ParaView
-  Module:    $RCSfile: vtkPVOptions.h,v $
+  Module:    vtkPVOptions.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -25,6 +25,7 @@
 #include "vtkCommandOptions.h"
 
 class vtkPVOptionsInternal;
+class vtkSelfConnection;
 
 class VTK_EXPORT vtkPVOptions : public vtkCommandOptions
 {
@@ -38,12 +39,13 @@ public:
     PVRENDER_SERVER = 0x10,
     PVDATA_SERVER = 0x20,
     PVBATCH = 0x40,
-    ALLPROCESS = PARAVIEW | PVCLIENT | PVSERVER | PVRENDER_SERVER | PVDATA_SERVER
+    ALLPROCESS = PARAVIEW | PVCLIENT | PVSERVER | PVRENDER_SERVER |
+      PVDATA_SERVER | PVBATCH
   };
 //ETX
   
   static vtkPVOptions* New();
-  vtkTypeRevisionMacro(vtkPVOptions,vtkCommandOptions);
+  vtkTypeMacro(vtkPVOptions,vtkCommandOptions);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   vtkGetMacro(ServerMode, int);
@@ -51,6 +53,7 @@ public:
   vtkGetMacro(ConnectID, int);
   vtkGetMacro(UseOffscreenRendering, int);
   vtkGetMacro(UseStereoRendering, int);
+  vtkGetStringMacro(StereoType);
   vtkGetMacro(ClientMode, int);
   // Description:
   // Get Various ports.
@@ -98,6 +101,11 @@ public:
   // vtkProcessModule needs to set the render module name
   vtkSetStringMacro(RenderModuleName);
 
+  // Description:
+  // Log filename.
+  vtkSetStringMacro(LogFileName);
+  vtkGetStringMacro(LogFileName);
+
   // Description:
   // vtkPVProcessModule needs to set this.
   vtkSetVector2Macro(TileDimensions, int);
@@ -112,6 +120,13 @@ public:
   // Should this run print the version numbers and exit.
   vtkGetMacro(TellVersion, int);
 
+  // Description:
+  // Called to create the primary self-connection. If NULL is returned then
+  // vtkProcessModuleConnectionManager will do the default thing.
+  // Makes it possible for new paraview-based
+  // applications to provide new primary connection types.
+  virtual vtkSelfConnection* NewSelfConnection();
+
 protected:
 //BTX
   // Description:
@@ -200,7 +215,11 @@ private:
   vtkSetStringMacro(GroupFileName);
   char* GroupFileName;
 
+  char* LogFileName;
   int TellVersion;
+
+  vtkSetStringMacro(StereoType);
+  char* StereoType;
 //ETX
 private:
   vtkPVOptions(const vtkPVOptions&); // Not implemented
