@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVXMLElement.cxx,v $
+  Module:    vtkPVXMLElement.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -18,7 +18,6 @@
 #include "vtkObjectFactory.h"
 #include "vtkSmartPointer.h"
 
-vtkCxxRevisionMacro(vtkPVXMLElement, "$Revision: 1.22 $");
 vtkStandardNewMacro(vtkPVXMLElement);
 
 #include <vtkstd/string>
@@ -110,6 +109,22 @@ void vtkPVXMLElement::AddAttribute(const char* attrName, double attrValue)
   this->AddAttribute(attrName, valueStr.str().c_str());
 }
 
+//----------------------------------------------------------------------------
+void vtkPVXMLElement::AddAttribute(const char* attrName, double attrValue,
+  int precision)
+{
+  if (precision <= 0)
+    {
+    this->AddAttribute(attrName, attrValue);
+    }
+  else
+    {
+    vtksys_ios::ostringstream valueStr;
+    valueStr << setprecision(precision) << attrValue << ends;
+    this->AddAttribute(attrName, valueStr.str().c_str());
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkPVXMLElement::AddAttribute(const char* attrName,
                                    const char* attrValue)
@@ -133,8 +148,8 @@ void vtkPVXMLElement::SetAttribute(const char* attrName,
     }
   
   // iterate over the names, and find if the attribute name exists.
-  unsigned int numAttributes = this->Internal->AttributeNames.size();
-  unsigned int i;
+  size_t numAttributes = this->Internal->AttributeNames.size();
+  size_t i;
   for(i=0; i < numAttributes; ++i)
     {
     if(strcmp(this->Internal->AttributeNames[i].c_str(), attrName) == 0)
@@ -214,8 +229,8 @@ void vtkPVXMLElement::AddCharacterData(const char* data, int length)
 //----------------------------------------------------------------------------
 const char* vtkPVXMLElement::GetAttribute(const char* name)
 {
-  unsigned int numAttributes = this->Internal->AttributeNames.size();
-  unsigned int i;
+  size_t numAttributes = this->Internal->AttributeNames.size();
+  size_t i;
   for(i=0; i < numAttributes; ++i)
     {
     if(strcmp(this->Internal->AttributeNames[i].c_str(), name) == 0)
@@ -242,8 +257,8 @@ void vtkPVXMLElement::PrintXML()
 void vtkPVXMLElement::PrintXML(ostream& os, vtkIndent indent)
 {
   os << indent << "<" << (this->Name?this->Name:"NoName");
-  unsigned int numAttributes = this->Internal->AttributeNames.size();
-  unsigned int i;
+  size_t numAttributes = this->Internal->AttributeNames.size();
+  size_t i;
   for(i=0;i < numAttributes; ++i)
     {
     const char* aName = this->Internal->AttributeNames[i].c_str();
@@ -256,7 +271,7 @@ void vtkPVXMLElement::PrintXML(ostream& os, vtkIndent indent)
     os << " " << (aName?aName:"NoName")
        << "=\"" << (aValue?sanitizedValue.c_str():"NoValue") << "\"";
     }
-  unsigned int numberOfNestedElements = this->Internal->NestedElements.size();
+  size_t numberOfNestedElements = this->Internal->NestedElements.size();
   if(numberOfNestedElements > 0)
     {
     os << ">\n";
@@ -288,7 +303,7 @@ vtkPVXMLElement* vtkPVXMLElement::GetParent()
 //----------------------------------------------------------------------------
 unsigned int vtkPVXMLElement::GetNumberOfNestedElements()
 {
-  return this->Internal->NestedElements.size();
+  return static_cast<unsigned int>(this->Internal->NestedElements.size());
 }
 
 //----------------------------------------------------------------------------
@@ -310,8 +325,8 @@ vtkPVXMLElement* vtkPVXMLElement::LookupElement(const char* id)
 //----------------------------------------------------------------------------
 vtkPVXMLElement* vtkPVXMLElement::FindNestedElement(const char* id)
 {
-  unsigned int numberOfNestedElements = this->Internal->NestedElements.size();
-  unsigned int i;
+  size_t numberOfNestedElements = this->Internal->NestedElements.size();
+  size_t i;
   for(i=0;i < numberOfNestedElements;++i)
     {
     const char* nid = this->Internal->NestedElements[i]->GetId();
@@ -587,13 +602,13 @@ void vtkPVXMLElement::Merge(vtkPVXMLElement* element, const char* attributeName)
     }
   
   // add attributes from element to this, or override attribute values on this
-  unsigned int numAttributes = element->Internal->AttributeNames.size();
-  unsigned int numAttributes2 = this->Internal->AttributeNames.size();
+  size_t numAttributes = element->Internal->AttributeNames.size();
+  size_t numAttributes2 = this->Internal->AttributeNames.size();
 
-  for(unsigned int i=0; i < numAttributes; ++i)
+  for(size_t i=0; i < numAttributes; ++i)
     {
     bool found = false;
-    for(unsigned int j=0; !found && j < numAttributes2; ++j)
+    for(size_t j=0; !found && j < numAttributes2; ++j)
       {
       if(element->Internal->AttributeNames[i] ==
         this->Internal->AttributeNames[j])
