@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVXMLElement.h,v $
+  Module:    vtkPVXMLElement.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -32,7 +32,7 @@ struct vtkPVXMLElementInternals;
 class VTK_EXPORT vtkPVXMLElement : public vtkObject
 {
 public:
-  vtkTypeRevisionMacro(vtkPVXMLElement,vtkObject);
+  vtkTypeMacro(vtkPVXMLElement,vtkObject);
   void PrintSelf(ostream& os, vtkIndent indent);
   static vtkPVXMLElement* New();
 
@@ -125,6 +125,7 @@ public:
   void AddAttribute(const char* attrName, const char* attrValue);
   void AddAttribute(const char* attrName, unsigned int attrValue);
   void AddAttribute(const char* attrName, double attrValue);
+  void AddAttribute(const char* attrName, double attrValue, int precision);
   void AddAttribute(const char* attrName, int attrValue);
 #if defined(VTK_USE_64BIT_IDS)
   void AddAttribute(const char* attrName, vtkIdType attrValue);
