@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkProcessModuleConnectionManager.cxx,v $
+  Module:    vtkProcessModuleConnectionManager.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -21,6 +21,7 @@
 #include "vtkCommand.h"
 #include "vtkConnectionIterator.h"
 #include "vtkMPISelfConnection.h"
+#include "vtkMPISelfConnection.h"
 #include "vtkObjectFactory.h"
 #include "vtkProcessModule.h"
 #include "vtkProcessModuleConnection.h"
@@ -33,7 +34,12 @@
 #include "vtkSmartPointer.h"
 #include "vtkSocketCollection.h"
 #include "vtkSocketController.h"
+#include "vtkTimerLog.h"
 
+#include "vtksys/SystemTools.hxx"
+
+#define VTK_CREATE(type, name) \
+  vtkSmartPointer<type> name = vtkSmartPointer<type>::New()
 
 #include <vtkstd/map>
 #include <vtkstd/deque>
@@ -72,7 +78,6 @@ protected:
 
 //-----------------------------------------------------------------------------
 vtkStandardNewMacro(vtkProcessModuleConnectionManager);
-vtkCxxRevisionMacro(vtkProcessModuleConnectionManager, "$Revision: 1.23 $");
 
 //-----------------------------------------------------------------------------
 vtkProcessModuleConnectionManager::vtkProcessModuleConnectionManager()
@@ -116,17 +121,21 @@ int vtkProcessModuleConnectionManager::Initialize(int argc, char** argv,
   dummy->Initialize();
   dummy->Delete();
 
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   // Create and initialize the self connection. This would also initialize
   // the MPIController, if any.
-  vtkSelfConnection* sc = 0;
-  if (this->ClientMode || !vtkProcessModule::GetProcessModule()->GetUseMPI())
+  vtkSelfConnection* sc = pm->GetOptions()->NewSelfConnection();
+  if (!sc)
     {
-    // No MPI needed in on a pure Client.
-    sc = vtkSelfConnection::New();
-    }
-  else
-    {
-    sc = vtkMPISelfConnection::New();
+    if (this->ClientMode || !vtkProcessModule::GetProcessModule()->GetUseMPI())
+      {
+      // No MPI needed in on a pure Client.
+      sc = vtkSelfConnection::New();
+      }
+    else
+      {
+      sc = vtkMPISelfConnection::New();
+      }
     }
   this->SetConnection(vtkProcessModuleConnectionManager::GetSelfConnectionID(),
     sc);
@@ -143,14 +152,13 @@ void vtkProcessModuleConnectionManager::Finalize()
   // Close open server sockets.
   this->StopAcceptingAllConnections();
 
-  vtkConnectionIterator* iter = this->NewIterator();
-  for (iter->Begin(); !iter->IsAtEnd(); iter->Next())
+  while (!this->Internals->IDToConnectionMap.empty())
     {
-    vtkProcessModuleConnection* conn = iter->GetCurrentConnection();
+    vtkProcessModuleConnection* conn =
+      this->Internals->IDToConnectionMap.begin()->second;
     conn->Finalize();
+    this->DropConnection(conn);
     }
-  iter->Delete();
-
 }
 
 //-----------------------------------------------------------------------------
@@ -162,7 +170,10 @@ GetConnectionFromID(vtkIdType connectionID)
   iter->Begin();
   if (iter->IsAtEnd())
     {
-    vtkErrorMacro("Invalid connection ID: " << connectionID);
+    if (connectionID != GetNullConnectionID())
+      {
+      vtkErrorMacro("Invalid connection ID: " << connectionID);
+      }
     iter->Delete();
     return NULL;
     }
@@ -261,14 +272,26 @@ vtkIdType vtkProcessModuleConnectionManager::OpenConnection(
   // Create a RemoteConnection (Server/Client)
   // Set the client socket on its controller.
   // Manage the client socket.
-  vtkClientSocket* cs = vtkClientSocket::New();
-  if (cs->ConnectToServer(hostname, port) == -1)
+  VTK_CREATE(vtkClientSocket, cs);
+  VTK_CREATE(vtkTimerLog, timer);
+  timer->StartTimer();
+  while (1)
     {
-    cs->Delete();
-    return id;
+    if (cs->ConnectToServer(hostname, port) != -1)
+      {
+      id = this->CreateConnection(cs, 0);
+      break;
+      }
+    timer->StopTimer();
+    if (timer->GetElapsedTime() > 60.0)
+      {
+      vtkErrorMacro(<< "Connect timeout.");
+      break;
+      }
+    vtkWarningMacro(<< "Connect failed.  Retrying for "
+                    << (60.0 - timer->GetElapsedTime()) << " more seconds.");
+    vtksys::SystemTools::Delay(1000);
     }
-  id = this->CreateConnection(cs, 0, 1); 
-  cs->Delete();
   return id;
 }
 
@@ -301,7 +324,7 @@ vtkIdType vtkProcessModuleConnectionManager::OpenConnection(
     vtkErrorMacro("Render Server connection failed.");
     return id;
     }
-  id = this->CreateConnection(dcs, rcs, 1);
+  id = this->CreateConnection(dcs, rcs);
   dcs->Delete();
   rcs->Delete();
   return id;
@@ -349,7 +372,7 @@ int vtkProcessModuleConnectionManager::MonitorConnections(
           {
           ds = this->Internals->DataServerConnections.front();
           rs = cc;
-          id = this->CreateConnection(ds, rs, 0);
+          id = this->CreateConnection(ds, rs);
           this->Internals->DataServerConnections.pop_front();
           }
         else
@@ -363,7 +386,7 @@ int vtkProcessModuleConnectionManager::MonitorConnections(
           {
           ds = cc;
           rs = this->Internals->RenderServerConnections.front();
-          id = this->CreateConnection(ds, rs, 0);
+          id = this->CreateConnection(ds, rs);
           this->Internals->RenderServerConnections.pop_front();
           }
         else
@@ -373,7 +396,7 @@ int vtkProcessModuleConnectionManager::MonitorConnections(
         break;
         
       case RENDER_AND_DATA_SERVER:
-        id = this->CreateConnection(cc, 0, 0);
+        id = this->CreateConnection(cc, 0);
         break;
         }
 
@@ -478,19 +501,6 @@ int vtkProcessModuleConnectionManager::IsServerConnection(
   return 1;
 }
 
-//-----------------------------------------------------------------------------
-vtkClientServerID vtkProcessModuleConnectionManager::
-GetConnectionClientServerID(vtkIdType id)
-{
-  vtkProcessModuleConnection* conn = this->GetConnectionFromID(id);
-  if (!conn)
-    {
-    vtkClientServerID nullid;
-    return nullid;
-    }
-  return conn->GetSelfID();
-}
-
 //-----------------------------------------------------------------------------
 vtkIdType vtkProcessModuleConnectionManager::GetConnectionID(
   vtkProcessModuleConnection* conn)
@@ -665,15 +675,14 @@ vtkIdType vtkProcessModuleConnectionManager::GetUniqueConnectionID()
 
 //-----------------------------------------------------------------------------
 vtkIdType vtkProcessModuleConnectionManager::CreateConnection(
-  vtkClientSocket* cs, vtkClientSocket* renderserver_socket, 
-  int connecting_side_handshake)
+  vtkClientSocket* cs, vtkClientSocket* renderserver_socket)
 {
   vtkIdType id = vtkProcessModuleConnectionManager::NullConnectionID;
 
   vtkRemoteConnection* rc = this->NewRemoteConnection();
   if (rc)
     {
-    if (rc->SetSocket(cs, connecting_side_handshake) == 0)
+    if (rc->SetSocket(cs) == 0)
       {
       rc->Delete();
       vtkErrorMacro("Handshake failed. You are probably using mismatching "
@@ -684,8 +693,7 @@ vtkIdType vtkProcessModuleConnectionManager::CreateConnection(
     if (renderserver_socket && vtkServerConnection::SafeDownCast(rc))
       {
       vtkServerConnection* sc = vtkServerConnection::SafeDownCast(rc);
-      if (sc->SetRenderServerSocket(renderserver_socket, 
-          connecting_side_handshake) == 0)
+      if (sc->SetRenderServerSocket(renderserver_socket) == 0)
         {
         rc->Delete();
         vtkErrorMacro("RenderServer Handshake failed.");
@@ -764,7 +772,7 @@ void vtkProcessModuleConnectionManager::DropConnection(
 //-----------------------------------------------------------------------------
 unsigned int vtkProcessModuleConnectionManager::GetNumberOfConnections()
 {
-  return this->Internals->IDToConnectionMap.size();
+  return static_cast<unsigned int>(this->Internals->IDToConnectionMap.size());
 }
 
 //-----------------------------------------------------------------------------
