@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSelectionConverter.cxx,v $
+  Module:    vtkSelectionConverter.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -24,14 +24,15 @@
 #include "vtkPointData.h"
 #include "vtkProcessModule.h"
 #include "vtkSelection.h"
+#include "vtkSelectionNode.h"
 #include "vtkSelectionSerializer.h"
+#include "vtkSmartPointer.h"
 #include "vtkUnsignedIntArray.h"
 
 #include <vtkstd/map>
 #include <vtkstd/set>
 
 vtkStandardNewMacro(vtkSelectionConverter);
-vtkCxxRevisionMacro(vtkSelectionConverter, "$Revision: 1.15 $");
 
 //----------------------------------------------------------------------------
 vtkSelectionConverter::vtkSelectionConverter()
@@ -85,35 +86,52 @@ public:
 void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
   int global_ids)
 {
-  output->Clear();
+  output->Initialize();
+  for (unsigned int i = 0; i < input->GetNumberOfNodes(); ++i)
+    {
+    vtkInformation *nodeProps = input->GetNode(i)->GetProperties();
+    if (!nodeProps->Has(vtkSelectionNode::PROCESS_ID()) ||
+        ( nodeProps->Get(vtkSelectionNode::PROCESS_ID()) ==
+          vtkProcessModule::GetProcessModule()->GetPartitionId() )
+      )
+      {
+      this->Convert(input->GetNode(i), output, global_ids);
+      }
+    }
+}
 
+//----------------------------------------------------------------------------
+void vtkSelectionConverter::Convert(vtkSelectionNode* input, vtkSelection* outputSel,
+  int global_ids)
+{
+  vtkSmartPointer<vtkSelectionNode> output = vtkSmartPointer<vtkSelectionNode>::New();
   vtkInformation* inputProperties =  input->GetProperties();
   vtkInformation* outputProperties = output->GetProperties();
 
   if (global_ids)
     {
     outputProperties->Set(
-      vtkSelection::CONTENT_TYPE(),
-      vtkSelection::GLOBALIDS);
+      vtkSelectionNode::CONTENT_TYPE(),
+      vtkSelectionNode::GLOBALIDS);
     }
   else
     {
     outputProperties->Set(
-      vtkSelection::CONTENT_TYPE(),
-      inputProperties->Get(vtkSelection::CONTENT_TYPE()));
+      vtkSelectionNode::CONTENT_TYPE(),
+      inputProperties->Get(vtkSelectionNode::CONTENT_TYPE()));
     }
 
-
+#if 0
   unsigned int numChildren = input->GetNumberOfChildren();
   for (unsigned int i=0; i<numChildren; i++)
     {
     vtkInformation *childProps = input->GetChild(i)->GetProperties();
-    if (!childProps->Has(vtkSelection::PROCESS_ID()) ||
-        ( childProps->Get(vtkSelection::PROCESS_ID()) ==
+    if (!childProps->Has(vtkSelectionNode::PROCESS_ID()) ||
+        ( childProps->Get(vtkSelectionNode::PROCESS_ID()) ==
           vtkProcessModule::GetProcessModule()->GetPartitionId() )
       )
       {
-      vtkSelection* newOutput = vtkSelection::New();
+      vtkSelection* newOutput = vtkSelectionNode::New();
       this->Convert(input->GetChild(i), newOutput, global_ids);
       if (newOutput->GetNumberOfChildren() > 0)
         {
@@ -129,19 +147,23 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
       newOutput->Delete();
       }
     }
+#endif
 
-  if (inputProperties->Get(vtkSelection::CONTENT_TYPE()) !=
-      vtkSelection::INDICES
-      ||
-      !inputProperties->Has(vtkSelection::FIELD_TYPE())
-      ||
-      inputProperties->Get(vtkSelection::FIELD_TYPE()) !=
-      vtkSelection::CELL)
+  if (inputProperties->Get(vtkSelectionNode::CONTENT_TYPE()) !=
+    vtkSelectionNode::INDICES ||
+    !inputProperties->Has(vtkSelectionNode::FIELD_TYPE()))
     {
     return;
     }
 
-  if (!inputProperties->Has(vtkSelection::SOURCE_ID()) ||
+  if (inputProperties->Get(vtkSelectionNode::FIELD_TYPE()) != vtkSelectionNode::CELL &&
+    inputProperties->Get(vtkSelectionNode::FIELD_TYPE()) != vtkSelectionNode::POINT)
+    {
+    // We can only handle point or cell selections.
+    return;
+    }
+
+  if (!inputProperties->Has(vtkSelectionNode::SOURCE_ID()) ||
       !inputProperties->Has(vtkSelectionSerializer::ORIGINAL_SOURCE_ID()))
     {
     return;
@@ -157,7 +179,7 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
 
   vtkClientServerID id;
-  id.ID = inputProperties->Get(vtkSelection::SOURCE_ID());
+  id.ID = inputProperties->Get(vtkSelectionNode::SOURCE_ID());
   vtkAlgorithm* geomAlg = vtkAlgorithm::SafeDownCast(
     pm->GetObjectFromID(id));
   if (!geomAlg)
@@ -172,24 +194,21 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
     return;
     }
 
-  /// Get the dataset from the input of the geom filter from which
-  /// we get the global ids, if required.
-  id.ID = inputProperties->Get(vtkSelectionSerializer::ORIGINAL_SOURCE_ID());
-  vtkAlgorithm* originalAlg = vtkAlgorithm::SafeDownCast(
-    pm->GetObjectFromID(id));
-  vtkDataSet* originalDS = originalAlg?
-    vtkDataSet::SafeDownCast(originalAlg->GetOutputDataObject(0)) : 0;
-  vtkIdTypeArray* globalIdsArray = originalDS?
-    vtkIdTypeArray::SafeDownCast(originalDS->GetCellData()->GetGlobalIds()): 0;
-
-  if (global_ids && !globalIdsArray)
+  if (global_ids)
     {
+    vtkErrorMacro("Global id selection no longer supported.");
     return;
     }
 
-  vtkUnsignedIntArray* compositeIndexArray = 0;
-  vtkUnsignedIntArray* amrLevelArray = 0;
-  vtkUnsignedIntArray* amrIndexArray = 0;
+  // NOTE: these are cell-data arrays.
+  vtkUnsignedIntArray* compositeIndexArray = vtkUnsignedIntArray::SafeDownCast(
+    ds->GetCellData()->GetArray("vtkCompositeIndex"));
+  // compositeIndexArray may not be present at all if the input dataset is not a
+  // composite dataset.
+  vtkUnsignedIntArray* amrLevelArray = vtkUnsignedIntArray::SafeDownCast(
+    ds->GetCellData()->GetArray("vtkAMRLevel")); // may be null.
+  vtkUnsignedIntArray* amrIndexArray = vtkUnsignedIntArray::SafeDownCast(
+    ds->GetCellData()->GetArray("vtkAMRIndex")); // may be null.
 
   // key == composite index or hierarchical index.
   // value == set of indices.
@@ -200,13 +219,7 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
   vtkIdType numHits = inputList->GetNumberOfTuples() *
     inputList->GetNumberOfComponents();
 
-  vtkIdTypeArray* vertptrs = vtkIdTypeArray::SafeDownCast(
-    input->GetSelectionData()->GetArray("vertptrs"));
-  vtkIdTypeArray* vertlist = vtkIdTypeArray::SafeDownCast(
-    input->GetSelectionData()->GetArray("vertlist"));
-  if (inputProperties->Has(vtkSelection::INDEXED_VERTICES()) &&
-      (inputProperties->Get(vtkSelection::INDEXED_VERTICES()) == 1) &&
-      vertptrs && vertlist)
+  if (inputProperties->Get(vtkSelectionNode::FIELD_TYPE()) == vtkSelectionNode::POINT)
     {
     vtkIdTypeArray* pointMapArray = vtkIdTypeArray::SafeDownCast(
       ds->GetPointData()->GetArray("vtkOriginalPointIds"));
@@ -215,19 +228,7 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
       return;
       }
 
-    // NOTE: this is a cell-data array.
-    compositeIndexArray = vtkUnsignedIntArray::SafeDownCast(
-        ds->GetCellData()->GetArray("vtkCompositeIndex"));
-    // compositeIndexArray may not be present at all if the input dataset is not a
-    // composite dataset.
-
-    // NOTE: these are cell-data arrays.
-    amrLevelArray = vtkUnsignedIntArray::SafeDownCast(
-      ds->GetCellData()->GetArray("vtkAMRLevel")); // may be null.
-    amrIndexArray = vtkUnsignedIntArray::SafeDownCast(
-      ds->GetCellData()->GetArray("vtkAMRIndex")); // may be null.
-
-    outputProperties->Set(vtkSelection::FIELD_TYPE(), vtkSelection::POINT);
+    outputProperties->Set(vtkSelectionNode::FIELD_TYPE(), vtkSelectionNode::POINT);
 
     vtkIdList *idlist = vtkIdList::New();
 
@@ -235,42 +236,38 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
     //vertices which were hit. For those lookup vertex id in original data set.
     for (vtkIdType hitId=0; hitId<numHits; hitId++)
       {
-      vtkIdType ptr = vertptrs->GetValue(hitId);
-      if (ptr != -1)
+      vtkIdType geomPointId = inputList->GetValue(hitId);
+      if (geomPointId < 0 || geomPointId >=  pointMapArray->GetNumberOfTuples())
         {
-        vtkIdType cellIndex = inputList->GetValue(hitId);
-        ds->GetCellPoints(cellIndex, idlist);
-
-        vtkKeyType key;
-        if (amrLevelArray && amrIndexArray)
-          {
-          unsigned int val[2];
-          amrLevelArray->GetTupleValue(cellIndex, &val[0]);
-          amrIndexArray->GetTupleValue(cellIndex, &val[1]);
-          key = vtkKeyType(val[0], val[1]);
-          }
-        else if (compositeIndexArray)
-          {
-          unsigned int composite_index = 0;
-          composite_index = compositeIndexArray->GetValue(cellIndex);
-          key = vtkKeyType(composite_index);
-          }
-
-        vtkIdType npts = vertlist->GetValue(ptr);
-        for (vtkIdType v = 0; v < npts; v++)
-          {
-          vtkIdType idx = vertlist->GetValue(ptr+1+v);
-          vtkIdType ptId = idlist->GetId(idx);
-          vtkIdType originalPtId = ptId;
-          if (pointMapArray)
-            {
-            originalPtId = pointMapArray->GetValue(ptId);
-            }
-
-          vtkstd::set<vtkIdType>& visverts = indices[key];
-          visverts.insert(originalPtId);
-          }
+        continue;
         }
+      vtkIdType pointId = pointMapArray->GetValue(geomPointId);
+
+      // Composite index information is available only on cells, hence, 
+      // for each point we find the first cell it belongs to and then lookup the
+      // composite index.
+      ds->GetPointCells(geomPointId, idlist);
+      vtkIdType geomCellId = 0;
+      if (idlist->GetNumberOfIds() > 0)
+        {
+        geomCellId = idlist->GetId(0);
+        }
+      vtkKeyType key;
+      if (amrLevelArray && amrIndexArray)
+        {
+        unsigned int val[2];
+        amrLevelArray->GetTupleValue(geomCellId, &val[0]);
+        amrIndexArray->GetTupleValue(geomCellId, &val[1]);
+        key = vtkKeyType(val[0], val[1]);
+        }
+      else if (compositeIndexArray)
+        {
+        unsigned int composite_index = 0;
+        composite_index = compositeIndexArray->GetValue(geomCellId);
+        key = vtkKeyType(composite_index);
+        }
+      vtkstd::set<vtkIdType>& visverts = indices[key];
+      visverts.insert(pointId);
       }
     idlist->Delete();
     }
@@ -282,14 +279,7 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
       {
       return;
       }
-
-    compositeIndexArray = vtkUnsignedIntArray::SafeDownCast(
-        ds->GetCellData()->GetArray("vtkCompositeIndex"));
-    amrLevelArray = vtkUnsignedIntArray::SafeDownCast(
-      ds->GetCellData()->GetArray("vtkAMRLevel")); // may be null.
-    amrIndexArray = vtkUnsignedIntArray::SafeDownCast(
-      ds->GetCellData()->GetArray("vtkAMRIndex")); // may be null.
-
+    
     for (vtkIdType hitId=0; hitId<numHits; hitId++)
       {
       vtkIdType geomCellId = inputList->GetValue(hitId);
@@ -310,26 +300,18 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
         key = vtkKeyType(composite_index);
         }
       vtkstd::set<vtkIdType>& cellindices = indices[key];
-      if (global_ids)
-        {
-        vtkIdType globalId = globalIdsArray->GetValue(cellIndex);
-        cellindices.insert(globalId);
-        }
-      else
-        {
-        cellindices.insert(cellIndex);
-        }
+      cellindices.insert(cellIndex);
       }
     }
 
   outputProperties->Set(
-    vtkSelection::SOURCE_ID(),
+    vtkSelectionNode::SOURCE_ID(),
     inputProperties->Get(vtkSelectionSerializer::ORIGINAL_SOURCE_ID()));
 
-  if (inputProperties->Has(vtkSelection::PROCESS_ID()))
+  if (inputProperties->Has(vtkSelectionNode::PROCESS_ID()))
     {
-    outputProperties->Set(vtkSelection::PROCESS_ID(),
-                          inputProperties->Get(vtkSelection::PROCESS_ID()));
+    outputProperties->Set(vtkSelectionNode::PROCESS_ID(),
+                          inputProperties->Get(vtkSelectionNode::PROCESS_ID()));
     }
 
   if (indices.size() > 1)
@@ -337,18 +319,18 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
     indicesType::iterator mit;
     for (mit=indices.begin(); mit != indices.end(); ++mit)
       {
-      vtkSelection* child = vtkSelection::New();
+      vtkSelectionNode* child = vtkSelectionNode::New();
       child->GetProperties()->Copy(outputProperties, /*deep=*/0);
       if (amrLevelArray && amrIndexArray)
         {
-        child->GetProperties()->Set(vtkSelection::HIERARCHICAL_LEVEL(),
+        child->GetProperties()->Set(vtkSelectionNode::HIERARCHICAL_LEVEL(),
           mit->first.HierarchicalIndex[0]);
-        child->GetProperties()->Set(vtkSelection::HIERARCHICAL_INDEX(),
+        child->GetProperties()->Set(vtkSelectionNode::HIERARCHICAL_INDEX(),
           mit->first.HierarchicalIndex[1]);
         }
       else if (compositeIndexArray)
         {
-        child->GetProperties()->Set(vtkSelection::COMPOSITE_INDEX(),
+        child->GetProperties()->Set(vtkSelectionNode::COMPOSITE_INDEX(),
           mit->first.CompositeIndex);
         }
 
@@ -364,7 +346,7 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
       child->SetSelectionList(outputArray);
       outputArray->Delete();
 
-      output->AddChild(child);
+      outputSel->AddNode(child);
       child->Delete();
       }
     }
@@ -381,18 +363,19 @@ void vtkSelectionConverter::Convert(vtkSelection* input, vtkSelection* output,
       }
     if (amrLevelArray && amrIndexArray)
       {
-      outputProperties->Set(vtkSelection::HIERARCHICAL_LEVEL(),
+      outputProperties->Set(vtkSelectionNode::HIERARCHICAL_LEVEL(),
         indices.begin()->first.HierarchicalIndex[0]);
-      outputProperties->Set(vtkSelection::HIERARCHICAL_INDEX(),
+      outputProperties->Set(vtkSelectionNode::HIERARCHICAL_INDEX(),
         indices.begin()->first.HierarchicalIndex[1]);
       }
     else if (compositeIndexArray)
       {
-      outputProperties->Set(vtkSelection::COMPOSITE_INDEX(),
+      outputProperties->Set(vtkSelectionNode::COMPOSITE_INDEX(),
         indices.begin()->first.CompositeIndex);
       }
     output->SetSelectionList(outputArray);
     outputArray->Delete();
+    outputSel->AddNode(output);
     }
 }
 
