@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkServerConnection.cxx,v $
+  Module:    vtkServerConnection.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -35,7 +35,6 @@
 
 
 vtkStandardNewMacro(vtkServerConnection);
-vtkCxxRevisionMacro(vtkServerConnection, "$Revision: 1.15 $");
 //-----------------------------------------------------------------------------
 vtkServerConnection::vtkServerConnection()
 {
@@ -92,8 +91,7 @@ void vtkServerConnection::Finalize()
 }
 
 //-----------------------------------------------------------------------------
-int vtkServerConnection::SetRenderServerSocket(vtkClientSocket* soc, 
-    int connecting_side_handshake)
+int vtkServerConnection::SetRenderServerSocket(vtkClientSocket* soc)
 {
   if (!this->RenderServerSocketController)
     {
@@ -109,12 +107,7 @@ int vtkServerConnection::SetRenderServerSocket(vtkClientSocket* soc,
   comm->SetSocket(soc);
   soc->AddObserver(vtkCommand::ErrorEvent, this->GetObserver());
   comm->AddObserver(vtkCommand::ErrorEvent, this->GetObserver());
-  // TODO: I would like to hide this handshake. May be, vtkSocket must manage it.
-  if (connecting_side_handshake)
-    {
-    return comm->ClientSideHandshake();
-    }
-  return comm->ServerSideHandshake();
+  return comm->Handshake();
 }
 
 //-----------------------------------------------------------------------------
@@ -204,7 +197,7 @@ int vtkServerConnection::SendStreamToServer(vtkSocketController* controller,
   const unsigned char* data;
   size_t len;
   stream.GetData(&data, &len);
-  controller->TriggerRMI(1, (void*)data, len, 
+  controller->TriggerRMI(1, (void*)data, static_cast<int>(len), 
     vtkRemoteConnection::CLIENT_SERVER_RMI_TAG);
   return 0;
 }
@@ -216,7 +209,7 @@ int vtkServerConnection::SendStreamToRoot(vtkSocketController* controller,
   const unsigned char* data;
   size_t len;
   stream.GetData(&data, &len);
-  controller->TriggerRMI(1, (void*)data, len, 
+  controller->TriggerRMI(1, (void*)data, static_cast<int>(len), 
     vtkRemoteConnection::CLIENT_SERVER_ROOT_RMI_TAG);
   return 0;
 }
@@ -340,7 +333,7 @@ void vtkServerConnection::GatherInformationFromController(vtkSocketController* c
   const unsigned char* data;
   size_t length;
   stream.GetData(&data, &length);
-  controller->TriggerRMI(1, (void*)(data), length,
+  controller->TriggerRMI(1, (void*)(data), static_cast<int>(length),
     vtkRemoteConnection::CLIENT_SERVER_GATHER_INFORMATION_RMI_TAG);
   
   int length2 = 0;
@@ -365,23 +358,12 @@ void vtkServerConnection::GatherInformationFromController(vtkSocketController* c
 }
 
 //-----------------------------------------------------------------------------
-int vtkServerConnection::Initialize(int vtkNotUsed(argc), 
-  char** vtkNotUsed(argv), int *vtkNotUsed(partitionId))
+int vtkServerConnection::Initialize(int argc, char** argv, int *partitionId)
 {
+  this->Superclass::Initialize(argc, argv, partitionId);
   // returns 0 on success, 1 on error.
 
-  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   
-  // A remote connnection is assigned a unique vtkClientServerID so that
-  // the connection can be located by the interpreter.
-  this->SelfID = pm->GetUniqueID();
-  vtkClientServerStream stream;
-  stream << vtkClientServerStream::Assign
-    << this->SelfID << this
-    << vtkClientServerStream::End;
-  this->SendStreamToClient(stream);  
-
-
   // Authenticate with DataServer.
   if (!this->AuthenticateWithServer(this->GetSocketController()))
     {
@@ -403,6 +385,7 @@ int vtkServerConnection::Initialize(int vtkNotUsed(argc),
     }
 
   // Collect and keep the server information.
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   vtkPVServerInformation* temp = vtkPVServerInformation::New();
   this->GatherInformation(
     vtkProcessModule::RENDER_SERVER,
@@ -441,13 +424,6 @@ int vtkServerConnection::AuthenticateWithServer(vtkSocketController* controller)
     return 1;
     }
 
-  if (!this->SelfID.ID)
-    {
-    // Sanity check.
-    vtkErrorMacro("SelfID not set.");
-    return 0;
-    }
-  
   // If any send or receive fails we simply give up.
   // Send connect ID to the server.
   int cid = options->GetConnectID();
@@ -502,29 +478,13 @@ int vtkServerConnection::AuthenticateWithServer(vtkSocketController* controller)
   // Receive the number of server processes as an handshake.
   int numServerProcs =  0;
   if (!controller->Receive(&numServerProcs, 1, 1,
-      vtkRemoteConnection::CLIENT_SERVER_COMMUNICATION_TAG))
+      vtkRemoteConnection::CLIENT_SERVER_COMMUNICATION_TAG) || 
+    (numServerProcs == 0))
     {
     vtkErrorMacro("Failed to receive handshake message.");
     return 0;
     }
 
-  // Now, since the authentication has succeeded, tell the server
-  // to assign the same vtkClientServerID to this connection as the Client is.
-  int id = static_cast<int>(this->SelfID.ID);
-  if (!controller->Send(&id, 1, 1,
-      vtkRemoteConnection::CLIENT_SERVER_COMMUNICATION_TAG))
-    {
-    return 0;
-    }
-  match=0;
-  controller->Receive(&match, 1, 1, 
-    vtkRemoteConnection::CLIENT_SERVER_COMMUNICATION_TAG);
-  if (match == 0)
-    {
-    vtkErrorMacro("Failed to assign ID to this connection.");
-    return 0;
-    }
-  
   // Since no of Data Server Processes >= no. of Render Server Processes,
   // this check ensures we only maintain the Data Server processes count.
   this->NumberOfServerProcesses = 
@@ -708,7 +668,7 @@ void vtkServerConnection::PushUndo(const char* label, vtkPVXMLElement* uelem)
   size_t len;
   stream.GetData(&data, &len);
   
-  controller->TriggerRMI(1, (void*)(data), len, 
+  controller->TriggerRMI(1, (void*)(data), static_cast<int>(len), 
     vtkRemoteConnection::CLIENT_SERVER_PUSH_UNDO_XML_TAG);
 }
 
