@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkExtractHistogram.h,v $
+  Module:    vtkExtractHistogram.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,7 +16,7 @@
 #ifndef __vtkExtractHistogram_h
 #define __vtkExtractHistogram_h
 
-#include "vtkRectilinearGridAlgorithm.h"
+#include "vtkTableAlgorithm.h"
 
 //BTX
 class vtkDoubleArray;
@@ -34,11 +34,11 @@ struct vtkEHInternals;
 // the boundaries between each histogram bin, and a vtkUnsignedLongArray
 // named "bin_values" which will contain the value for each bin.
 
-class VTK_EXPORT vtkExtractHistogram : public vtkRectilinearGridAlgorithm
+class VTK_EXPORT vtkExtractHistogram : public vtkTableAlgorithm
 {
 public:
   static vtkExtractHistogram* New();
-  vtkTypeRevisionMacro(vtkExtractHistogram, vtkRectilinearGridAlgorithm);
+  vtkTypeMacro(vtkExtractHistogram, vtkTableAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -51,6 +51,19 @@ public:
   // Controls the number of bins N in the output histogram data
   vtkSetClampMacro(BinCount, int, 1, VTK_LARGE_INTEGER);
   vtkGetMacro(BinCount, int);
+
+  // Description:
+  // Get/Set custom bin ranges to use. These are used only when
+  // UseCustomBinRanges is set to true.
+  vtkSetVector2Macro(CustomBinRanges, double);
+  vtkGetVector2Macro(CustomBinRanges, double);
+
+  // Description:
+  // When set to true, CustomBinRanges will  be used instead of using the full
+  // range for the selected array. By default, set to false.
+  vtkSetMacro(UseCustomBinRanges, bool);
+  vtkGetMacro(UseCustomBinRanges, bool);
+  vtkBooleanMacro(UseCustomBinRanges, bool);
   
   // Description:
   // This option controls whether the algorithm calculates averages
@@ -67,20 +80,10 @@ protected:
 
   virtual int FillInputPortInformation (int port, vtkInformation *info);
 
-  // convenience method
-  virtual int RequestInformation(vtkInformation* request,
-                                 vtkInformationVector** inputVector,
-                                 vtkInformationVector* outputVector);
-
   virtual int RequestData(vtkInformation *request, 
                           vtkInformationVector **inputVector, 
                           vtkInformationVector *outputVector);
 
-  virtual int RequestUpdateExtent(vtkInformation*,
-                                  vtkInformationVector**,
-                                  vtkInformationVector*);
-
-
   // Initialize the bin_extents using the data range for the selected 
   // array.
   virtual bool InitializeBinExtents(
@@ -94,6 +97,10 @@ protected:
     double min, double max,
     vtkFieldData* field);
 
+  void FillBinExtents(vtkDoubleArray* bin_extents, double min, double max);
+
+  double CustomBinRanges[2];
+  bool UseCustomBinRanges;
   int Component;
   int BinCount;
   int CalculateAverages;
