@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkFileSeriesReader.h,v $
+  Module:    vtkFileSeriesReader.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -59,7 +59,7 @@ class VTK_EXPORT vtkFileSeriesReader : public vtkDataObjectAlgorithm
 {
 public:
   static vtkFileSeriesReader* New();
-  vtkTypeRevisionMacro(vtkFileSeriesReader, vtkDataObjectAlgorithm);
+  vtkTypeMacro(vtkFileSeriesReader, vtkDataObjectAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -98,6 +98,10 @@ public:
   // Returns the name of a file with index idx.
   virtual const char* GetFileName(unsigned int idx);
 
+  // Description:
+  // Returns the most recent filename used.
+  vtkGetStringMacro(CurrentFileName);
+
   // Description:
   // Get/set the filename for the meta-file.  Has no effect unless UseMetaFile
   // is true.
@@ -120,6 +124,13 @@ public:
   vtkSetStringMacro(FileNameMethod);
   vtkGetStringMacro(FileNameMethod);
 
+  // Description:
+  // If true, then treat file series like it does not contain any time step 
+  // values. False by default.
+  vtkGetMacro(IgnoreReaderTime, int);
+  vtkSetMacro(IgnoreReaderTime, int);
+  vtkBooleanMacro(IgnoreReaderTime, int);
+
 protected:
   vtkFileSeriesReader();
   ~vtkFileSeriesReader();
@@ -152,11 +163,14 @@ protected:
   // Reads a metadata file and returns a list of filenames (in filesToRead).  If
   // the file could not be read correctly, 0 is returned.
   virtual int ReadMetaDataFile(const char *metafilename,
-                               vtkStringArray *filesToRead);
+                               vtkStringArray *filesToRead,
+                               int maxFilesToRead = VTK_LARGE_INTEGER);
 
   virtual void SetReaderFileName(const char* fname);
   vtkAlgorithm* Reader;
 
+  vtkSetStringMacro(CurrentFileName);
+  char* CurrentFileName;
   char* FileNameMethod;
 
   char *MetaFileName;
@@ -167,6 +181,8 @@ protected:
   // Re-reads information from the metadata file, if necessary.
   virtual void UpdateMetaData();
 
+  int IgnoreReaderTime;
+
 private:
   vtkFileSeriesReader(const vtkFileSeriesReader&); // Not implemented.
   void operator=(const vtkFileSeriesReader&); // Not implemented.
