@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkHierarchicalFractal.cxx,v $
+  Module:    vtkHierarchicalFractal.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -36,7 +36,6 @@
 
 #include <assert.h>
 
-vtkCxxRevisionMacro(vtkHierarchicalFractal, "$Revision: 1.11 $");
 vtkStandardNewMacro(vtkHierarchicalFractal);
 
 //----------------------------------------------------------------------------
@@ -52,6 +51,7 @@ vtkHierarchicalFractal::vtkHierarchicalFractal()
   this->Levels = vtkIntArray::New();
   this->TwoDimensional = 1;
   this->Asymetric = 1;
+  this->Overlap = 1;
   
   this->TopLevelSpacing[0] = 1.0;
   this->TopLevelSpacing[1] = 1.0;
@@ -452,7 +452,7 @@ int vtkHierarchicalFractal::RequestData(
   // return the specific process number
   int piece=info->Get(vtkStreamingDemandDrivenPipeline::UPDATE_PIECE_NUMBER());
   int numPieces =info->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
-  
+
   float ox = -1.75;
   float oy = -1.25;
   float oz = 0.0;
@@ -719,11 +719,15 @@ void vtkHierarchicalFractal::Traverse(int &blockId,
     }
     
   int subOnFace[6];
-  
+  int generateBlock  = 1;
   if (this->TwoDimensional)
     {
     if (this->TwoDTest(bds, level, this->MaximumLevel))
       {
+      if ( ! this->Overlap)
+        {
+        generateBlock = 0;
+        }
       ++level;
       // Traverse the 4 new blocks.
       subOnFace[0]=onFace[0];
@@ -749,7 +753,7 @@ void vtkHierarchicalFractal::Traverse(int &blockId,
 //      subOnFace[3]=onFace[3];
       this->Traverse(blockId, level, output, x2,x3,y2,y3,z0,z0,subOnFace);
       }
-    else
+    if (generateBlock)
       {
       if (this->BlockCount >= this->StartBlock
           && this->BlockCount <= this->EndBlock)
@@ -771,6 +775,10 @@ void vtkHierarchicalFractal::Traverse(int &blockId,
         this->Levels->InsertValue(blockId, level);
         ++blockId;
         }
+      else if (this->EndBlock != -1)
+        {
+        this->AppedDataSetToLevel(output, level, ext, NULL);
+        }
       ++this->BlockCount;
       }
     }
@@ -781,6 +789,10 @@ void vtkHierarchicalFractal::Traverse(int &blockId,
         this->LineTest(-1.05088,0.85595,0.87104, -0.61430,1.00347,0.59553, bds,
                        level, this->MaximumLevel) )
       { // break block into eight.
+      if ( ! this->Overlap)
+        {
+        generateBlock = 0;
+        }
       ++level;
       // Traverse the 8 new blocks.
       subOnFace[0]=onFace[0];
@@ -830,7 +842,7 @@ void vtkHierarchicalFractal::Traverse(int &blockId,
 //      subOnFace[3]=onFace[3];
       this->Traverse(blockId, level, output, x2,x3,y2,y3,z2,z3,subOnFace);
       }
-    else
+    if (generateBlock)
       {
       if (this->BlockCount >= this->StartBlock
           && this->BlockCount <= this->EndBlock)
@@ -1063,13 +1075,16 @@ void vtkHierarchicalFractal::AddBlockIdArray(vtkCompositeDataSet *output)
   int blockId = 0;
   vtkSmartPointer<vtkCompositeDataIterator> iter;
   iter.TakeReference(output->NewIterator());
+  iter->SkipEmptyNodesOff();
   for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); 
     iter->GoToNextItem(), blockId++)
     {
     vtkUniformGrid *grid;
     grid=vtkUniformGrid::SafeDownCast(iter->GetCurrentDataObject());
-    assert("check: grid_exists" && grid!=0);
-
+    if (!grid)
+      {
+      continue;
+      }
 
     vtkIntArray* array = vtkIntArray::New();
     int numCells=grid->GetNumberOfCells();
@@ -1101,21 +1116,21 @@ void vtkHierarchicalFractal::AddDepthArray(vtkHierarchicalBoxDataSet* output)
       vtkAMRBox temp;
       vtkUniformGrid *grid;
       grid=vtkUniformGrid::SafeDownCast(output->GetDataSet(level,block, temp));
-      assert("check: grid_exists" && grid!=0);
-      
-      
-      vtkIntArray* array = vtkIntArray::New();
-      int numCells=grid->GetNumberOfCells();
-      array->Allocate(numCells);
-      int cell=0;
-      while(cell<numCells)
+      if (grid)
         {
-        array->InsertNextValue(level);
-        ++cell;
+        vtkIntArray* array = vtkIntArray::New();
+        int numCells=grid->GetNumberOfCells();
+        array->Allocate(numCells);
+        int cell=0;
+        while(cell<numCells)
+          {
+          array->InsertNextValue(level);
+          ++cell;
+          }
+        array->SetName("Depth");
+        grid->GetCellData()->AddArray(array);
+        array->Delete();
         }
-      array->SetName("Depth");
-      grid->GetCellData()->AddArray(array);
-      array->Delete();
       ++block;
       }
     ++level;
@@ -1461,15 +1476,8 @@ unsigned int vtkHierarchicalFractal::AppedDataSetToLevel(
     }
   else if (hbDS)
     {
-    int loCorner[3], hiCorner[3];
-    loCorner[0] = extents[0];
-    hiCorner[0] = extents[1];
-    loCorner[1] = extents[2];
-    hiCorner[1] = extents[3];
-    loCorner[2] = extents[4];
-    hiCorner[2] = extents[5];
-
-    vtkAMRBox box(this->TwoDimensional? 2:3,loCorner,hiCorner);
+    int dim=this->TwoDimensional? 2:3;
+    vtkAMRBox box(dim,extents);
     index = hbDS->GetNumberOfDataSets(level);
     hbDS->SetDataSet(level, index, box, vtkUniformGrid::SafeDownCast(dataset));
     }
