@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkIceTRenderManager.cxx,v $
+  Module:    vtkIceTRenderManager.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -16,18 +16,19 @@
  Copyright (c) Sandia Corporation
  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
 ----------------------------------------------------------------------------*/
+#include "vtkIceTRenderManager.h"
 
 #include "vtkCallbackCommand.h"
 #include "vtkCamera.h"
 #include "vtkDoubleArray.h"
 #include "vtkFloatArray.h"
 #include "vtkIceTContext.h"
-#include "vtkIceTRenderManager.h"
 #include "vtkIceTRenderer.h"
 #include "vtkIntArray.h"
 #include "vtkMath.h"
-#include "vtkMPI.h"
 #include "vtkMPIController.h"
+#include "vtkMPI.h"
+#include "vtkMultiProcessStream.h"
 #include "vtkObjectFactory.h"
 #include "vtkPerspectiveTransform.h"
 #include "vtkPKdTree.h"
@@ -40,7 +41,7 @@
 #include <GL/ice-t.h>
 
 #include "vtkgl.h"
-
+#include "assert.h"
 #include <vtkstd/algorithm>
 
 #ifndef GL_BGRA
@@ -50,21 +51,6 @@
 #define MY_MAX(x, y)    ((x) < (y) ? (y) : (x))
 #define MY_MIN(x, y)    ((x) < (y) ? (x) : (y))
 
-//******************************************************************
-// Hidden structures.
-//******************************************************************
-struct IceTWindowInformation {
-  int TilesDirty;
-};
-const int ICET_WIN_INFO_SIZE = sizeof(struct IceTWindowInformation)/sizeof(int);
-
-struct IceTRendererInformation {
-  int Strategy;
-  int ComposeOperation;
-};
-const int ICET_REN_INFO_SIZE
-  = sizeof(struct IceTRendererInformation)/sizeof(int);
-
 // ******************************************************************
 // Callback commands.
 // ******************************************************************
@@ -94,7 +80,6 @@ static void vtkIceTRenderManagerReconstructWindowImage(vtkObject *,
 // vtkIceTRenderManager implementation.
 //******************************************************************
 
-vtkCxxRevisionMacro(vtkIceTRenderManager, "$Revision: 1.43 $");
 vtkStandardNewMacro(vtkIceTRenderManager);
 
 vtkCxxSetObjectMacro(vtkIceTRenderManager, TileViewportTransform,
@@ -501,26 +486,26 @@ void vtkIceTRenderManager::ComputeTileViewportTransform()
   // tile, in which case just show something.
 
   // The mullion size as a fraction of total display size.
+  // Here we are assuming that each of the tiles is of the same size.
   double mullions[2];
-  mullions[0] = (double)this->TileMullions[0]/(double)this->FullImageSize[0];
-  mullions[1] = (double)this->TileMullions[1]/(double)this->FullImageSize[1];
-
-  // The spacing of the tiles (including the mullions).
-  double tileSpacing[2];
-  tileSpacing[0] = 1.0/(double)this->TileDimensions[0];
-  tileSpacing[1] = 1.0/(double)this->TileDimensions[1];
+  mullions[0] = static_cast<double>(this->TileMullions[0])/
+    (this->FullImageSize[0] * this->TileDimensions[0]);
+  mullions[1] = static_cast<double>(this->TileMullions[1])/
+    (this->FullImageSize[1] * this->TileDimensions[1]);
 
   // The size of the tile as a fraction of the total display size.
   double tileSize[2];
-  tileSize[0] = tileSpacing[0] - mullions[0];
-  tileSize[1] = tileSpacing[1] - mullions[1];
+  tileSize[0] = 1.0/(double)this->TileDimensions[0];
+  tileSize[1] = 1.0/(double)this->TileDimensions[1];
+
+  // The spacing of the tiles (including the mullions).
+  double tileSpacing[2];
+  tileSpacing[0] = tileSize[0] + mullions[0];
+  tileSpacing[1] = tileSize[1] + mullions[1];
 
-  // At this time, the render window's tile parameters (which 2D annotation will
-  // use for placement) does not support mullions.  Punt, and don't use mullions
-  // for 2D annotation.
   this->RenderWindow->SetTileScale(this->TileDimensions);
-  this->RenderWindow->SetTileViewport(0.0, 0.0, tileSpacing[0], tileSpacing[1]);
-
+  this->RenderWindow->SetTileViewport(0.0, 0.0, tileSize[0], tileSize[1]);
+  
   for (int y = 0; y < this->TileDimensions[1]; y++)
     {
     for (int x = 0; x < this->TileDimensions[0]; x++)
@@ -543,13 +528,10 @@ void vtkIceTRenderManager::ComputeTileViewportTransform()
           {
           // RenderWindow tiles from lower left instead of upper left.
           y = this->TileDimensions[1] - y - 1;
-          // At this time, the render window's tile parameters (which 2D
-          // annotation will use for placement) does not support mullions.
-          // Punt, and don't use mullions for 2D annotation.
           this->RenderWindow->SetTileViewport(x*tileSpacing[0],
                                               y*tileSpacing[1],
-                                              x*tileSpacing[0]+tileSpacing[0],
-                                              y*tileSpacing[1]+tileSpacing[1]);
+                                              x*tileSpacing[0]+tileSize[0],
+                                              y*tileSpacing[1]+tileSize[1]);
           }
 
         return;
@@ -835,111 +817,114 @@ double vtkIceTRenderManager::GetCompositeTime()
 }
 
 //-----------------------------------------------------------------------------
-
-void vtkIceTRenderManager::SendWindowInformation()
+void vtkIceTRenderManager::CollectWindowInformation(vtkMultiProcessStream& stream)
 {
   vtkDebugMacro("Sending Window Information");
 
-  this->Superclass::SendWindowInformation();
+  this->Superclass::CollectWindowInformation(stream);
 
-  struct IceTWindowInformation info;
-  info.TilesDirty = this->TilesDirty;
-
-  int numProcs = this->Controller->GetNumberOfProcesses();
-  for (int id = 0; id < numProcs; id++)
+  // insert the tag to ensure we reading back the correct information.
+  stream << vtkProcessModule::IceTWinInfo;
+  stream << this->TilesDirty;
+  if (this->TilesDirty)
     {
-    if (id == this->RootProcessId) continue;
-
-    this->Controller->Send((int *)&info, ICET_WIN_INFO_SIZE, id,
-                           vtkProcessModule::IceTWinInfo);
-    if (this->TilesDirty)
+    stream << this->TileDimensions[0] << this->TileDimensions[1];
+    for (int x = 0; x < this->TileDimensions[0]; x++)
       {
-      this->Controller->Send(&this->TileDimensions[0], 1, id,
-                             vtkProcessModule::IceTNumTilesX);
-      this->Controller->Send(&this->TileDimensions[1], 1, id,
-                             vtkProcessModule::IceTNumTilesY);
-      for (int x = 0; x < this->TileDimensions[0]; x++)
+      for (int y=0; y < this->TileDimensions[1]; y++)
         {
-        this->Controller->Send(this->TileRanks[x], this->TileDimensions[1], id,
-                               vtkProcessModule::IceTTileRanks);
+        stream << (this->TileRanks[x])[y];
         }
       }
     }
+  stream << vtkProcessModule::IceTWinInfo;
 }
 
 //-----------------------------------------------------------------------------
 
-void vtkIceTRenderManager::ReceiveWindowInformation()
+bool vtkIceTRenderManager::ProcessWindowInformation(vtkMultiProcessStream& stream)
 {
   vtkDebugMacro("Receiving Window Information");
 
-  this->Superclass::ReceiveWindowInformation();
-
-  struct IceTWindowInformation info;
-  this->Controller->Receive((int *)&info, ICET_WIN_INFO_SIZE,
-                            this->RootProcessId,
-                            vtkProcessModule::IceTWinInfo);
-  if (info.TilesDirty)
+  if (!this->Superclass::ProcessWindowInformation(stream))
     {
-    int NewNumTilesX, NewNumTilesY;
-    this->Controller->Receive(&NewNumTilesX, 1, 0,
-                              vtkProcessModule::IceTNumTilesX);
-    this->Controller->Receive(&NewNumTilesY, 1, 0,
-                              vtkProcessModule::IceTNumTilesY);
-    this->SetTileDimensions(NewNumTilesX, NewNumTilesY);
+    return false;
+    }
+
+  int tag;
+  stream >> tag;
+  if (tag != vtkProcessModule::IceTWinInfo)
+    {
+    vtkErrorMacro("Incorrect tag received. Aborting for debugging purposes.");
+    return false;
+    }
+
+  int tilesDirty;
+  stream >> tilesDirty;
+  if (tilesDirty)
+    {
+    int newNumTilesX, newNumTilesY;
+    stream >> newNumTilesX >> newNumTilesY;
+    this->SetTileDimensions(newNumTilesX, newNumTilesY);
     for (int x = 0; x < this->TileDimensions[0]; x++)
       {
-      this->Controller->Receive(this->TileRanks[x], this->TileDimensions[1], 0,
-                                vtkProcessModule::IceTTileRanks);
+      for (int y=0; y < this->TileDimensions[1]; y++)
+        {
+        stream >> (this->TileRanks[x])[y];
+        }
       }
     }
+  stream >> tag;
+  if (tag != vtkProcessModule::IceTWinInfo)
+    {
+    vtkErrorMacro("Incorrect tag received. Aborting for debugging purposes.");
+    return false;
+    }
+  return true;
 }
 
 //-----------------------------------------------------------------------------
-
-void vtkIceTRenderManager::SendRendererInformation(vtkRenderer *_ren)
+void vtkIceTRenderManager::CollectRendererInformation(vtkRenderer *_ren,
+  vtkMultiProcessStream& stream)
 {
   vtkDebugMacro("Sending renderer information for " << _ren);
 
-  this->Superclass::SendRendererInformation(_ren);
+  this->Superclass::CollectRendererInformation(_ren, stream);
 
   vtkIceTRenderer *ren = vtkIceTRenderer::SafeDownCast(_ren);
-  if (!ren) return;
-
-  struct IceTRendererInformation info;
-  info.Strategy = ren->GetStrategy();
-  info.ComposeOperation = ren->GetComposeOperation();
-
-  int numProcs = this->Controller->GetNumberOfProcesses();
-  for (int id = 0; id < numProcs; id++)
+  if (!ren)
     {
-    if (id == this->RootProcessId) continue;
-
-    this->Controller->Send((int *)&info, ICET_REN_INFO_SIZE, id,
-                           vtkProcessModule::IceTRenInfo);
+    return;
     }
+
+  stream << ren->GetStrategy()
+         << ren->GetComposeOperation();
 }
 
 //-----------------------------------------------------------------------------
-
-void vtkIceTRenderManager::ReceiveRendererInformation(vtkRenderer *_ren)
+bool vtkIceTRenderManager::ProcessRendererInformation(vtkRenderer *_ren,
+  vtkMultiProcessStream& stream)
 {
   vtkDebugMacro("Receiving renderer information for " << _ren);
 
-  this->Superclass::ReceiveRendererInformation(_ren);
+  if (!this->Superclass::ProcessRendererInformation(_ren, stream))
+    {
+    return false;
+    }
 
   vtkIceTRenderer *ren = vtkIceTRenderer::SafeDownCast(_ren);
-  if (!ren) return;
-
-  struct IceTRendererInformation info;
-  this->Controller->Receive((int *)&info, ICET_REN_INFO_SIZE, 0,
-                            vtkProcessModule::IceTRenInfo);
-  ren->SetStrategy(info.Strategy);
-  ren->SetComposeOperation(info.ComposeOperation);
+  if (ren) 
+    {
+    int strategy;
+    int compose_operation;
+    stream >> strategy >> compose_operation;
+    ren->SetStrategy(strategy);
+    ren->SetComposeOperation(compose_operation);
+    }
+  return true;
 }
 
 //-----------------------------------------------------------------------------
-
 void vtkIceTRenderManager::PreRenderProcessing()
 {
   vtkDebugMacro("PreRenderProcessing");
@@ -1069,6 +1054,7 @@ void vtkIceTRenderManager::PreRenderProcessing()
 void vtkIceTRenderManager::PostRenderProcessing()
 {
   vtkDebugMacro("PostRenderProcessing");
+  vtkTimerLog::MarkStartEvent("Compositing");
 
   this->Controller->Barrier();
 
@@ -1090,6 +1076,8 @@ void vtkIceTRenderManager::PostRenderProcessing()
     this->RenderWindow->SwapBuffersOn();
     }
   this->RenderWindow->Frame();
+
+  vtkTimerLog::MarkEndEvent("Compositing");
 }
 
 //-----------------------------------------------------------------------------
