@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkIceTRenderer.cxx,v $
+  Module:    vtkIceTRenderer.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -14,12 +14,13 @@
 =========================================================================*/
 /* -*- c++ -*- *******************************************************/
 
-/* $Id: vtkIceTRenderer.cxx,v 1.27 2008-03-15 06:14:40 dcthomp Exp $ */
+/* Id */
 
 #include "vtkIceTRenderer.h"
 
 #include "vtkCamera.h"
 #include "vtkCommand.h"
+#include "vtkHardwareSelector.h"
 #include "vtkIceTContext.h"
 #include "vtkIntArray.h"
 #include "vtkLightCollection.h"
@@ -27,6 +28,7 @@
 #include "vtkObjectFactory.h"
 #include "vtkPKdTree.h"
 #include "vtkRenderWindow.h"
+#include "vtkTimerLog.h"
 
 #include <GL/ice-t.h>
 
@@ -46,7 +48,6 @@ static vtkIceTRenderer *currentRenderer;
 // vtkIceTRenderer implementation.
 //******************************************************************
 
-vtkCxxRevisionMacro(vtkIceTRenderer, "$Revision: 1.27 $");
 vtkStandardNewMacro(vtkIceTRenderer);
 
 vtkCxxSetObjectMacro(vtkIceTRenderer, SortingKdTree, vtkPKdTree);
@@ -73,6 +74,10 @@ vtkIceTRenderer::vtkIceTRenderer()
     {
     this->PhysicalViewport[i] = 0;
     }
+
+  // Disable the gradient and texuted background color for now.
+  this->Superclass::TexturedBackgroundOff();
+  this->Superclass::GradientBackgroundOff();
 }
 
 vtkIceTRenderer::~vtkIceTRenderer()
@@ -178,6 +183,7 @@ void vtkIceTRenderer::GetTiledSizeAndOrigin(int *width, int *height,
 void vtkIceTRenderer::DeviceRender()
 {
   vtkDebugMacro("In vtkIceTRenderer::DeviceRender");
+  vtkTimerLog::MarkStartEvent("IceT Dev Render");
 
   //Update the Camera once.  ICE-T will shift the viewpoint around.
   this->Superclass::ClearLights();
@@ -190,6 +196,7 @@ void vtkIceTRenderer::DeviceRender()
     glMatrixMode(GL_MODELVIEW);
     glPopMatrix();
     this->InvokeEvent(vtkCommand::EndEvent,NULL);
+    vtkTimerLog::MarkEndEvent("IceT Dev Render");
     return;
     }
 
@@ -283,6 +290,11 @@ void vtkIceTRenderer::DeviceRender()
     icetDisable(ICET_ORDERED_COMPOSITE);
     }
 
+  // \NOTE: This suppose to fix the issue with volume rendering
+  // on the tile display mode where every tile has to render and composite
+  // but for some reason it didnot work.
+//  icetDisable(ICET_FLOATING_VIEWPORT);
+
   //Make sure we tell ICE-T what the background color is.  If the background
   //is black, also make it transparent so that we can skip fixing it.
   GLint in_buffers;
@@ -342,6 +354,8 @@ void vtkIceTRenderer::DeviceRender()
 
   //This is also traditionally done in UpdateGeometry().
   this->RenderTime.Modified();
+
+  vtkTimerLog::MarkEndEvent("IceT Dev Render");
 }
 
 //-----------------------------------------------------------------------------
@@ -360,8 +374,13 @@ void vtkIceTRenderer::Clear()
   vtkDebugMacro("Clear Color: " << bgcolor[0] << ", " << bgcolor[1]
                 << ", " << bgcolor[2] << ", " << bgcolor[3]);
   glClearColor(bgcolor[0], bgcolor[1], bgcolor[2], bgcolor[3]);
-  glClearDepth(1.0);
-  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+  GLbitfield  clear_mask = GL_COLOR_BUFFER_BIT;
+  if (!this->GetPreserveDepthBuffer())
+    {
+    glClearDepth(static_cast<GLclampf>(1.0));
+    clear_mask |= GL_DEPTH_BUFFER_BIT;
+    }
+  glClear(clear_mask);
 }
 
 //-----------------------------------------------------------------------------
@@ -425,13 +444,14 @@ int vtkIceTRenderer::UpdateGeometry()
 {
   vtkDebugMacro("In vtkIceTRenderer::UpdateGeometry()");
 
-  if (this->SelectMode != vtkRenderer::NOT_SELECTING)
+  if (this->Selector)
     {
-    //we are doing a visible polygon selection instead of a normal render
-    int ret = this->UpdateGeometryForSelection();
-    vtkDebugMacro( << "Rendered " << 
-                   this->NumberOfPropsRendered << " actors" );    
-    return ret;
+    // When selector is present, we are performing a selection,
+    // so do the selection rendering pass instead of the normal passes.
+    // Delegate the rendering of the props to the selector itself.
+    this->NumberOfPropsRendered = this->Selector->Render(this,
+      this->PropArray, this->PropArrayCount);
+    return this->NumberOfPropsRendered;
     }
 
   int i;
@@ -552,7 +572,7 @@ int vtkIceTRenderer::UpdateGeometry()
 int vtkIceTRenderer::UpdateTranslucentPolygonalGeometry()
 {
   int result=0;
-  // loop through props and give them a chance to 
+  // loop through props and give them a chance to
   // render themselves as translucent geometry
   for (int i = 0; i < this->PropArrayCount; i++ )
     {
