@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMPIMoveData.cxx,v $
+  Module:    vtkMPIMoveData.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -20,39 +20,45 @@
 #include "vtkCharArray.h"
 #include "vtkDataSetReader.h"
 #include "vtkDataSetWriter.h"
+#include "vtkDirectedGraph.h"
+#include "vtkGraphReader.h"
+#include "vtkGraphWriter.h"
 #include "vtkImageAppend.h"
 #include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
+#include "vtkMergeGraphs.h"
 #include "vtkMPIMToNSocketConnection.h"
 #include "vtkMultiProcessController.h"
 #include "vtkObjectFactory.h"
 #include "vtkOutlineFilter.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
+#include "vtkProcessModule.h"
 #include "vtkSmartPointer.h"
 #include "vtkSocketCommunicator.h"
 #include "vtkSocketController.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkTimerLog.h"
 #include "vtkToolkits.h"
+#include "vtkUndirectedGraph.h"
 #include "vtkUnstructuredGrid.h"
-
+#include "vtk_zlib.h"
 #include <vtksys/ios/sstream>
-#define EXTENT_HEADER_SIZE 360
 
 #ifdef VTK_USE_MPI
 #include "vtkMPICommunicator.h"
 #include "vtkAllToNRedistributePolyData.h"
 #endif
 
-vtkCxxRevisionMacro(vtkMPIMoveData, "$Revision: 1.20 $");
+
+bool vtkMPIMoveData::UseZLibCompression = false;
+
+
 vtkStandardNewMacro(vtkMPIMoveData);
 
 vtkCxxSetObjectMacro(vtkMPIMoveData,Controller, vtkMultiProcessController);
-vtkCxxSetObjectMacro(vtkMPIMoveData,ClientDataServerSocketController, vtkSocketController);
 vtkCxxSetObjectMacro(vtkMPIMoveData,MPIMToNSocketConnection, vtkMPIMToNSocketConnection);
-
 //-----------------------------------------------------------------------------
 vtkMPIMoveData::vtkMPIMoveData()
 {
@@ -88,15 +94,27 @@ vtkMPIMoveData::vtkMPIMoveData()
 vtkMPIMoveData::~vtkMPIMoveData()
 {
   this->SetController(0);
-  this->SetClientDataServerSocketController(0);
+  this->ClientDataServerSocketController = 0;
   this->SetMPIMToNSocketConnection(0);
   this->ClearBuffer();
 }
 
+//----------------------------------------------------------------------------
+void vtkMPIMoveData::SetUseZLibCompression(bool b)
+{
+  vtkMPIMoveData::UseZLibCompression = b;
+}
+
+//----------------------------------------------------------------------------
+bool vtkMPIMoveData::GetUseZLibCompression()
+{
+  return vtkMPIMoveData::UseZLibCompression;
+}
+
 //----------------------------------------------------------------------------
 int vtkMPIMoveData::FillInputPortInformation(int, vtkInformation *info)
 {
-  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataSet");
+  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkDataObject");
   info->Set(vtkAlgorithm::INPUT_IS_OPTIONAL(), 1);
   return 1;
 }
@@ -134,6 +152,22 @@ int vtkMPIMoveData::RequestDataObject(vtkInformation*,
       }
     outputCopy = vtkImageData::New();
     }
+  else if (this->OutputDataType == VTK_DIRECTED_GRAPH)
+    {
+    if (output && output->IsA("vtkDirectedGraph"))
+      {
+      return 1;
+      }
+    outputCopy = vtkDirectedGraph::New();
+    }
+  else if (this->OutputDataType == VTK_UNDIRECTED_GRAPH)
+    {
+    if (output && output->IsA("vtkUndirectedGraph"))
+      {
+      return 1;
+      }
+    outputCopy = vtkUndirectedGraph::New();
+    }
   else
     {
     vtkErrorMacro("Unrecognized output type: " << this->OutputDataType
@@ -167,6 +201,19 @@ int vtkMPIMoveData::RequestInformation(vtkInformation*,
   return 1;
 }
 
+//-----------------------------------------------------------------------------
+void vtkMPIMoveData::DetermineClientDataServerController()
+{
+  this->ClientDataServerSocketController = 0;
+  if (this->Server != vtkMPIMoveData::RENDER_SERVER)
+    {
+    vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+    this->ClientDataServerSocketController =
+      pm? pm->GetActiveSocketController() : 0;
+    }
+}
+
+//-----------------------------------------------------------------------------
 // This filter  is going to replace the many variations of collection fitlers.
 // It handles collection and duplication.
 // It handles poly data and unstructured grid.
@@ -200,17 +247,17 @@ int vtkMPIMoveData::RequestData(vtkInformation*,
                                 vtkInformationVector** inputVector,
                                 vtkInformationVector* outputVector)
 {
+  this->DetermineClientDataServerController();
+
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
 
-  vtkDataSet* input = 0;
-  vtkDataSet* output = vtkDataSet::SafeDownCast(
-    outInfo->Get(vtkDataObject::DATA_OBJECT()));
+  vtkDataObject* input = 0;
+  vtkDataObject* output = outInfo->Get(vtkDataObject::DATA_OBJECT());
 
   if (inputVector[0]->GetNumberOfInformationObjects() > 0)
     {
-    input = vtkDataSet::SafeDownCast(
-      inputVector[0]->GetInformationObject(0)->Get(
-        vtkDataObject::DATA_OBJECT()));
+    input = inputVector[0]->GetInformationObject(0)->Get(
+        vtkDataObject::DATA_OBJECT());
     }
 
 
@@ -423,8 +470,8 @@ int vtkMPIMoveData::RequestData(vtkInformation*,
 //-----------------------------------------------------------------------------
 // Use LANL filter to redistribute the data.
 // We will marshal more than once, but that is OK.
-void vtkMPIMoveData::DataServerAllToN(vtkDataSet* inData,
-                                      vtkDataSet* outData, int n)
+void vtkMPIMoveData::DataServerAllToN(vtkDataObject* inData,
+                                      vtkDataObject* outData, int n)
 {
   vtkMultiProcessController* controller = this->Controller;
   vtkPolyData* input = vtkPolyData::SafeDownCast(inData);
@@ -476,8 +523,8 @@ void vtkMPIMoveData::DataServerAllToN(vtkDataSet* inData,
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::DataServerGatherAll(vtkDataSet* input,
-                                         vtkDataSet* output)
+void vtkMPIMoveData::DataServerGatherAll(vtkDataObject* input,
+                                         vtkDataObject* output)
 {
   int numProcs= this->Controller->GetNumberOfProcesses();
 
@@ -538,8 +585,8 @@ void vtkMPIMoveData::DataServerGatherAll(vtkDataSet* input,
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::DataServerGatherToZero(vtkDataSet* input,
-                                            vtkDataSet* output)
+void vtkMPIMoveData::DataServerGatherToZero(vtkDataObject* input,
+                                            vtkDataObject* output)
 {
   int numProcs= this->Controller->GetNumberOfProcesses();
   if (numProcs == 1)
@@ -616,7 +663,7 @@ void vtkMPIMoveData::DataServerGatherToZero(vtkDataSet* input,
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::DataServerSendToRenderServer(vtkDataSet* output)
+void vtkMPIMoveData::DataServerSendToRenderServer(vtkDataObject* output)
 {
   vtkSocketCommunicator* com =
       this->MPIMToNSocketConnection->GetSocketCommunicator();
@@ -639,7 +686,7 @@ void vtkMPIMoveData::DataServerSendToRenderServer(vtkDataSet* output)
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::RenderServerReceiveFromDataServer(vtkDataSet* output)
+void vtkMPIMoveData::RenderServerReceiveFromDataServer(vtkDataObject* output)
 {
   vtkSocketCommunicator* com =
       this->MPIMToNSocketConnection->GetSocketCommunicator();
@@ -671,7 +718,7 @@ void vtkMPIMoveData::RenderServerReceiveFromDataServer(vtkDataSet* output)
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::DataServerZeroSendToRenderServerZero(vtkDataSet* data)
+void vtkMPIMoveData::DataServerZeroSendToRenderServerZero(vtkDataObject* data)
 {
   int myId = this->Controller->GetLocalProcessId();
 
@@ -699,7 +746,7 @@ void vtkMPIMoveData::DataServerZeroSendToRenderServerZero(vtkDataSet* data)
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::RenderServerZeroReceiveFromDataServerZero(vtkDataSet* data)
+void vtkMPIMoveData::RenderServerZeroReceiveFromDataServerZero(vtkDataObject* data)
 {
   int myId = this->Controller->GetLocalProcessId();
 
@@ -736,7 +783,7 @@ void vtkMPIMoveData::RenderServerZeroReceiveFromDataServerZero(vtkDataSet* data)
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::DataServerSendToClient(vtkDataSet* output)
+void vtkMPIMoveData::DataServerSendToClient(vtkDataObject* output)
 {
   int myId = this->Controller->GetLocalProcessId();
 
@@ -744,13 +791,13 @@ void vtkMPIMoveData::DataServerSendToClient(vtkDataSet* output)
     {
     vtkTimerLog::MarkStartEvent("Dataserver sending to client");
 
-    vtkSmartPointer<vtkDataSet> tosend = output;
+    vtkSmartPointer<vtkDataObject> tosend = output;
     if (this->DeliverOutlineToClient)
       {
       // reduce data using outline filter.
       if (output->IsA("vtkPolyData"))
         {
-        vtkDataSet* clone = output->NewInstance();
+        vtkDataSet* clone = vtkPolyData::SafeDownCast(output)->NewInstance();
         clone->ShallowCopy(output);
 
         vtkOutlineFilter* filter = vtkOutlineFilter::New();
@@ -780,7 +827,7 @@ void vtkMPIMoveData::DataServerSendToClient(vtkDataSet* output)
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::ClientReceiveFromDataServer(vtkDataSet* output)
+void vtkMPIMoveData::ClientReceiveFromDataServer(vtkDataObject* output)
 {
   vtkCommunicator* com = 0;
   com = this->ClientDataServerSocketController->GetCommunicator();
@@ -812,7 +859,7 @@ void vtkMPIMoveData::ClientReceiveFromDataServer(vtkDataSet* output)
 
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::RenderServerZeroBroadcast(vtkDataSet* data)
+void vtkMPIMoveData::RenderServerZeroBroadcast(vtkDataObject* data)
 {
   (void)data; // shut up warning
   int numProcs= this->Controller->GetNumberOfProcesses();
@@ -895,80 +942,114 @@ void vtkMPIMoveData::ClearBuffer()
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::MarshalDataToBuffer(vtkDataSet* data)
+void vtkMPIMoveData::MarshalDataToBuffer(vtkDataObject* data)
 {
+  vtkDataSet* dataSet = vtkDataSet::SafeDownCast(data);
+  vtkImageData* imageData = vtkImageData::SafeDownCast(data);
+  vtkGraph* graph = vtkGraph::SafeDownCast(data);
+
   // Protect from empty data.
-  if (data->GetNumberOfPoints() == 0)
+  if ((dataSet && dataSet->GetNumberOfPoints() == 0) || (graph && graph->GetNumberOfVertices() == 0))
     {
     this->NumberOfBuffers = 0;
     }
 
-  vtkImageData* imageData = vtkImageData::SafeDownCast(data);
-
   // Copy input to isolate reader from the pipeline.
-  vtkDataSet* d = data->NewInstance();
-  d->CopyStructure(data);
-  d->GetPointData()->PassData(data->GetPointData());
-  d->GetCellData()->PassData(data->GetCellData());
-  // Marshal with writer.
-  vtkDataSetWriter *writer = vtkDataSetWriter::New();
+  vtkDataWriter* writer = 0;
+  if (dataSet)
+    {
+    vtkDataSet* d = dataSet->NewInstance();
+    d->CopyStructure(dataSet);
+    d->GetPointData()->PassData(dataSet->GetPointData());
+    d->GetCellData()->PassData(dataSet->GetCellData());
+    writer = vtkDataSetWriter::New();
+    writer->SetInput(d);
+    d->Delete();
+    if (imageData)
+      {
+      // We add the image extents to the header, since the writer doesn't preserve
+      // the extents.
+      int *extent = imageData->GetExtent();
+      double* origin = imageData->GetOrigin();
+      vtksys_ios::ostringstream stream;
+      stream << "EXTENT " << extent[0] << " " <<
+        extent[1] << " " <<
+        extent[2] << " " <<
+        extent[3] << " " <<
+        extent[4] << " " <<
+        extent[5];
+      stream << " ORIGIN: " << origin[0] << " " << origin[1] << " " << origin[2];
+      writer->SetHeader(stream.str().c_str());
+      }
+    }
+  if (graph)
+    {
+    vtkGraph* g = graph->NewInstance();
+    g->ShallowCopy(graph);
+    writer = vtkGraphWriter::New();
+    writer->SetInput(g);
+    g->Delete();
+    }
   writer->SetFileTypeToBinary();
   writer->WriteToOutputStringOn();
-  writer->SetInput(d);
   writer->Write();
-  // Get string.
-  this->NumberOfBuffers = 1;
-  this->BufferLengths = new vtkIdType[1];
-  this->BufferLengths[0] = writer->GetOutputStringLength();
-  this->BufferOffsets = new vtkIdType[1];
-  this->BufferOffsets[0] = 0;
 
-  if (imageData)
+  char* buffer =NULL;
+  vtkIdType buffer_length = 0;
+
+  if (vtkMPIMoveData::UseZLibCompression)
     {
-    // we need to add marshall extents separately, since the writer doesn't
-    // preserve extents.
-    int *extent = imageData->GetExtent();
-    double* origin = imageData->GetOrigin();
-    vtksys_ios::ostringstream stream;
-    stream << "EXTENT " << extent[0] << " " <<
-      extent[1] << " " <<
-      extent[2] << " " <<
-      extent[3] << " " <<
-      extent[4] << " " <<
-      extent[5];
-    stream << " ORIGIN: " << origin[0] << " " << origin[1] << " " << origin[2];
+    vtkTimerLog::MarkStartEvent("Zlib compress");
+    // Use z-lib compression.
+    uLongf out_size =compressBound(writer->GetOutputStringLength());
+    buffer = new char[out_size + 8]; 
+    memcpy(buffer, "zlib0000", 8);
 
-    if (stream.str().size() >= EXTENT_HEADER_SIZE)
+    compress2(reinterpret_cast<Bytef*>(buffer + 8), 
+      &out_size,
+      reinterpret_cast<const Bytef*>(writer->GetOutputString()),
+      writer->GetOutputStringLength(), /* compression_level */ Z_DEFAULT_COMPRESSION);
+    vtkTimerLog::MarkEndEvent("Zlib compress");
+    int in_size = static_cast<int>(writer->GetOutputStringLength());
+    for (int cc=0; cc < 4; cc++)
       {
-      vtkErrorMacro("Extent message too long!");
-      this->Buffers = writer->RegisterAndGetOutputString();
-      }
-    else
-      {
-      char extentHeader[EXTENT_HEADER_SIZE];
-      strcpy(extentHeader, stream.str().c_str());
-      this->BufferLengths[0] += EXTENT_HEADER_SIZE;
-      this->Buffers = new char[this->BufferLengths[0]];
-      memcpy(this->Buffers, extentHeader, EXTENT_HEADER_SIZE);
-      memcpy(this->Buffers+EXTENT_HEADER_SIZE, writer->GetOutputString(),
-        writer->GetOutputStringLength());
+      // the first 4 bytes in the header are "zlib" which helps the receiver
+      // identify that zlib compression has been used.
+      // the next 4 bytes are the original length since zlib doesn't provide
+      // that to the receiver.
+      buffer[4+cc] = (in_size & 0x0ff);
+      in_size = in_size >> 8;
       }
+    buffer_length = out_size + 8;
     }
   else
     {
-    this->Buffers = writer->RegisterAndGetOutputString();
+    buffer_length = writer->GetOutputStringLength();
+    buffer = writer->RegisterAndGetOutputString();
     }
+
+  // Get string.
+  this->NumberOfBuffers = 1;
+  this->BufferLengths = new vtkIdType[1];
+  this->BufferLengths[0] = buffer_length;
+  this->BufferOffsets = new vtkIdType[1];
+  this->BufferOffsets[0] = 0;
+  this->Buffers = buffer;
   this->BufferTotalLength = this->BufferLengths[0];
 
-  d->Delete();
-  d = 0;
   writer->Delete();
   writer = 0;
 }
 
 //-----------------------------------------------------------------------------
-void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
+void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataObject* data)
 {
+  vtkDataSet *dataSet = vtkDataSet::SafeDownCast(data);
+  vtkPolyData *polyData = vtkPolyData::SafeDownCast(data);
+  vtkUnstructuredGrid *unstructuredGrid = vtkUnstructuredGrid::SafeDownCast(data);
+  vtkImageData *imageData = vtkImageData::SafeDownCast(data);
+  vtkGraph *graph = vtkGraph::SafeDownCast(data);
+
   if (this->NumberOfBuffers == 0 || this->Buffers == 0)
     {
     data->Initialize();
@@ -979,21 +1060,26 @@ void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
   vtkAppendPolyData* appendPd = NULL;
   vtkAppendFilter*   appendUg = NULL;
   vtkImageAppend* appendId = NULL;
+  vtkMergeGraphs* mergeGraphs = NULL;
   if (this->NumberOfBuffers > 1)
     {
-    if (data->IsA("vtkPolyData"))
+    if (polyData)
       {
       appendPd = vtkAppendPolyData::New();
       }
-    else if (data->IsA("vtkUnstructuredGrid"))
+    else if (unstructuredGrid)
       {
       appendUg = vtkAppendFilter::New();
       }
-    else if (data->IsA("vtkImageData"))
+    else if (imageData)
       {
       appendId = vtkImageAppend::New();
       appendId->PreserveExtentsOn();
       }
+    else if (graph)
+      {
+      mergeGraphs = vtkMergeGraphs::New();
+      }
     else
       {
       vtkErrorMacro("This filter only handles unstructured data.");
@@ -1003,44 +1089,75 @@ void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
 
   for (int idx = 0; idx < this->NumberOfBuffers; ++idx)
     {
-    // Setup a reader.
-    vtkDataSetReader *reader = vtkDataSetReader::New();
-    reader->ReadFromInputStringOn();
-
     char* bufferArray = this->Buffers+this->BufferOffsets[idx];
     vtkIdType bufferLength = this->BufferLengths[idx];
 
+    char* realBuffer = 0;
+    if (bufferLength > 4 && strncmp(bufferArray, "zlib", 4) == 0)
+      {
+      // sender used zlib compression. Decompress it.
+      vtkIdType compressed_length = bufferLength - 8; // remove the zlib header.
+      vtkIdType uncompressed_length = 0;
+      for (int cc=0; cc < 4; cc++)
+        {
+        uncompressed_length = uncompressed_length | 
+          ((0xff & (bufferArray[4+cc])) <<8 *cc);
+        }
+
+      // using zlib compression.
+      realBuffer = new char[uncompressed_length];
+      uLongf destLen = uncompressed_length;
+      vtkTimerLog::MarkStartEvent("Zlib uncompress");
+      uncompress(reinterpret_cast<Bytef*>(realBuffer), &destLen,
+        reinterpret_cast<const Bytef*>(bufferArray+8), compressed_length);
+      vtkTimerLog::MarkEndEvent("Zlib uncompress");
+
+      bufferArray = realBuffer;
+      bufferLength = uncompressed_length;
+      }
+
+    // Setup a reader.
+    vtkDataReader *reader = NULL;
+    if (dataSet)
+      {
+      reader = vtkDataSetReader::New();
+      }
+    else if (graph)
+      {
+      reader = vtkGraphReader::New();
+      }
+    reader->ReadFromInputStringOn();
+
     int extent[6]= {0, 0, 0, 0, 0, 0};
     float origin[3] = {0, 0, 0};
     bool extentAvailable = false;
-    if (bufferLength >= EXTENT_HEADER_SIZE &&
-      strncmp(bufferArray, "EXTENT", 6)==0)
-      {
-      sscanf(bufferArray, "EXTENT %d %d %d %d %d %d ORIGIN %f %f %f", &extent[0], &extent[1],
-        &extent[2], &extent[3], &extent[4], &extent[5],
-        &origin[0], &origin[1], &origin[2]);
-      bufferArray += EXTENT_HEADER_SIZE;
-      bufferLength -= EXTENT_HEADER_SIZE;
-      extentAvailable = true;
-      }
-
     vtkCharArray* mystring = vtkCharArray::New();
     mystring->SetArray(bufferArray, bufferLength, 1);
     reader->SetInputArray(mystring);
     reader->Modified(); // For append loop
     reader->Update();
+
+    if (imageData)
+      {
+      sscanf(reader->GetHeader(),
+        "EXTENT %d %d %d %d %d %d ORIGIN %f %f %f", &extent[0], &extent[1],
+        &extent[2], &extent[3], &extent[4], &extent[5],
+        &origin[0], &origin[1], &origin[2]);
+      extentAvailable = true;
+      }
+
     if (appendPd)
       {
-      appendPd->AddInput(reader->GetPolyDataOutput());
+      appendPd->AddInput(vtkPolyData::SafeDownCast(reader->GetOutputDataObject(0)));
       }
     else if (appendUg)
       {
-      appendUg->AddInput(reader->GetUnstructuredGridOutput());
+      appendUg->AddInput(vtkUnstructuredGrid::SafeDownCast(reader->GetOutputDataObject(0)));
       }
     else if (appendId)
       {
       vtkImageData* curInput = vtkImageData::SafeDownCast(
-        reader->GetOutput());
+        reader->GetOutputDataObject(0));
       if (curInput->GetNumberOfPoints() >0)
         {
         if (extentAvailable)
@@ -1058,26 +1175,49 @@ void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
           }
         }
       }
-    else
+    else if (mergeGraphs)
       {
-      vtkDataSet* out = reader->GetOutput();
-      data->CopyStructure(out);
-      data->GetPointData()->PassData(out->GetPointData());
-      data->GetCellData()->PassData(out->GetCellData());
+      if (!mergeGraphs->GetInputDataObject(0, 0))
+        {
+        mergeGraphs->SetInput(0, reader->GetOutputDataObject(0));
+        }
+      else
+        {
+        mergeGraphs->SetInput(1, reader->GetOutputDataObject(0));
+        mergeGraphs->Update();
+        vtkGraph* output = mergeGraphs->GetOutput();
+        vtkGraph* outputCopy = output->NewInstance();
+        outputCopy->ShallowCopy(output);
+        mergeGraphs->SetInput(0, outputCopy);
+        }
+      }
+    else if (dataSet)
+      {
+      vtkDataSet* out = vtkDataSet::SafeDownCast(reader->GetOutputDataObject(0));
+      dataSet->CopyStructure(out);
+      dataSet->GetPointData()->PassData(out->GetPointData());
+      dataSet->GetCellData()->PassData(out->GetCellData());
+      }
+    else if (graph)
+      {
+      vtkGraph* out = vtkGraph::SafeDownCast(reader->GetOutputDataObject(0));
+      graph->ShallowCopy(out);
       }
     mystring->Delete();
     mystring = 0;
     reader->Delete();
     reader = NULL;
+    delete [] realBuffer;
+    realBuffer = 0;
     }
 
   if (appendPd)
     {
     vtkDataSet* out = appendPd->GetOutput();
     out->Update();
-    data->CopyStructure(out);
-    data->GetPointData()->PassData(out->GetPointData());
-    data->GetCellData()->PassData(out->GetCellData());
+    polyData->CopyStructure(out);
+    polyData->GetPointData()->PassData(out->GetPointData());
+    polyData->GetCellData()->PassData(out->GetCellData());
     appendPd->Delete();
     appendPd = NULL;
     }
@@ -1085,9 +1225,9 @@ void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
     {
     vtkDataSet* out = appendUg->GetOutput();
     out->Update();
-    data->CopyStructure(out);
-    data->GetPointData()->PassData(out->GetPointData());
-    data->GetCellData()->PassData(out->GetCellData());
+    unstructuredGrid->CopyStructure(out);
+    unstructuredGrid->GetPointData()->PassData(out->GetPointData());
+    unstructuredGrid->GetCellData()->PassData(out->GetCellData());
     appendUg->Delete();
     appendUg = NULL;
     }
@@ -1095,12 +1235,19 @@ void vtkMPIMoveData::ReconstructDataFromBuffer(vtkDataSet* data)
     {
     appendId->Update();
     vtkDataSet* out = appendId->GetOutput();
-    data->CopyStructure(out);
-    data->GetPointData()->PassData(out->GetPointData());
-    data->GetCellData()->PassData(out->GetCellData());
+    imageData->CopyStructure(out);
+    imageData->GetPointData()->PassData(out->GetPointData());
+    imageData->GetCellData()->PassData(out->GetCellData());
     appendId->Delete();
     appendId = NULL;
     }
+  if (mergeGraphs)
+    {
+    vtkGraph* out = vtkGraph::SafeDownCast(mergeGraphs->GetInputDataObject(0, 0));
+    graph->ShallowCopy(out);
+    mergeGraphs->Delete();
+    mergeGraphs = NULL;
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -1121,6 +1268,18 @@ void vtkMPIMoveData::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << "VTK_UNSTRUCTURED_GRID";
     }
+  else if (this->OutputDataType == VTK_IMAGE_DATA)
+    {
+    os << "VTK_IMAGE_DATA";
+    }
+  else if (this->OutputDataType == VTK_DIRECTED_GRAPH)
+    {
+    os << "VTK_DIRECTED_GRAPH";
+    }
+  else if (this->OutputDataType == VTK_UNDIRECTED_GRAPH)
+    {
+    os << "VTK_UNDIRECTED_GRAPH";
+    }
   else
     {
     os << "Unrecognized output type " << this->OutputDataType;
