@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkMPIMoveData.h,v $
+  Module:    vtkMPIMoveData.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -21,7 +21,7 @@
 #ifndef __vtkMPIMoveData_h
 #define __vtkMPIMoveData_h
 
-#include "vtkDataSetAlgorithm.h"
+#include "vtkPassInputTypeAlgorithm.h"
 
 class vtkMultiProcessController;
 class vtkSocketController;
@@ -29,11 +29,11 @@ class vtkMPIMToNSocketConnection;
 class vtkDataSet;
 class vtkIndent;
 
-class VTK_EXPORT vtkMPIMoveData : public vtkDataSetAlgorithm
+class VTK_EXPORT vtkMPIMoveData : public vtkPassInputTypeAlgorithm
 {
 public:
   static vtkMPIMoveData *New();
-  vtkTypeRevisionMacro(vtkMPIMoveData, vtkDataSetAlgorithm);
+  vtkTypeMacro(vtkMPIMoveData, vtkPassInputTypeAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -50,7 +50,6 @@ public:
   // MPIMToNSocketConnection==0 => Client-DataServer.
   // MPIMToNSocketConnection==1 => Client-DataServer-RenderServer.
   void SetController(vtkMultiProcessController* controller);
-  void SetClientDataServerSocketController(vtkSocketController* sdc);
   void SetMPIMToNSocketConnection(vtkMPIMToNSocketConnection* sc);
   
   // Description:
@@ -80,12 +79,6 @@ public:
   vtkSetMacro(OutputDataType, int);
   vtkGetMacro(OutputDataType, int);
 
-  // Description:
-  // Legacy API for ParaView 1.4
-  void SetPassThrough(int v) 
-    {if(v){this->SetMoveModeToPassThrough();} else {this->SetMoveModeToClone();}}
-  void SetSocketController(vtkSocketController* c) {this->SetClientDataServerSocketController(c);}
-
   // Description:
   // Sometimes, the data may be too huge to deliver to the client. In that case,
   // the client can request that only the outline for the data may be delivered
@@ -94,6 +87,13 @@ public:
   vtkSetMacro(DeliverOutlineToClient, int);
   vtkGetMacro(DeliverOutlineToClient, int);
 
+  // Description:
+  // When set to true, zlib compression is used. False by default.
+  // This value has any effect only on the data-sender processes. The receiver
+  // always checks the received data to see if zlib decompression is required.
+  static void SetUseZLibCompression(bool b);
+  static bool GetUseZLibCompression();
+
 //BTX
   enum MoveModes {
     PASS_THROUGH=0,
@@ -106,6 +106,12 @@ protected:
   vtkMPIMoveData();
   ~vtkMPIMoveData();
 
+  // Description:
+  // This method is called on each RequestData(). It Initializes the
+  // ClientDataServerSocketController with the socket communicator for the
+  // active connection.
+  void DetermineClientDataServerController();
+
   virtual int RequestDataObject(vtkInformation* request, 
                            vtkInformationVector** inputVector, 
                            vtkInformationVector* outputVector);
@@ -121,16 +127,16 @@ protected:
   vtkSocketController* ClientDataServerSocketController;
   vtkMPIMToNSocketConnection* MPIMToNSocketConnection;
 
-  void DataServerAllToN(vtkDataSet* inData, vtkDataSet* outData, int n);
-  void DataServerGatherAll(vtkDataSet* input, vtkDataSet* output);
-  void DataServerGatherToZero(vtkDataSet* input, vtkDataSet* output);
-  void DataServerSendToRenderServer(vtkDataSet* output);
-  void RenderServerReceiveFromDataServer(vtkDataSet* output);
-  void DataServerZeroSendToRenderServerZero(vtkDataSet* data);
-  void RenderServerZeroReceiveFromDataServerZero(vtkDataSet* data);
-  void RenderServerZeroBroadcast(vtkDataSet* data);
-  void DataServerSendToClient(vtkDataSet* output);
-  void ClientReceiveFromDataServer(vtkDataSet* output);
+  void DataServerAllToN(vtkDataObject* inData, vtkDataObject* outData, int n);
+  void DataServerGatherAll(vtkDataObject* input, vtkDataObject* output);
+  void DataServerGatherToZero(vtkDataObject* input, vtkDataObject* output);
+  void DataServerSendToRenderServer(vtkDataObject* output);
+  void RenderServerReceiveFromDataServer(vtkDataObject* output);
+  void DataServerZeroSendToRenderServerZero(vtkDataObject* data);
+  void RenderServerZeroReceiveFromDataServerZero(vtkDataObject* data);
+  void RenderServerZeroBroadcast(vtkDataObject* data);
+  void DataServerSendToClient(vtkDataObject* output);
+  void ClientReceiveFromDataServer(vtkDataObject* output);
 
   int        NumberOfBuffers;
   vtkIdType* BufferLengths;
@@ -139,8 +145,8 @@ protected:
   vtkIdType  BufferTotalLength;
 
   void ClearBuffer();
-  void MarshalDataToBuffer(vtkDataSet* data);
-  void ReconstructDataFromBuffer(vtkDataSet* data);
+  void MarshalDataToBuffer(vtkDataObject* data);
+  void ReconstructDataFromBuffer(vtkDataObject* data);
 
   int MoveMode;
   int Server;
@@ -162,6 +168,8 @@ private:
 
   vtkMPIMoveData(const vtkMPIMoveData&); // Not implemented
   void operator=(const vtkMPIMoveData&); // Not implemented
+
+  static bool UseZLibCompression;
 };
 
 #endif
