@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkNetworkImageSource.h,v $
+  Module:    vtkNetworkImageSource.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -36,7 +36,7 @@ class VTK_EXPORT vtkNetworkImageSource : public vtkImageAlgorithm
 {
 public:
   static vtkNetworkImageSource* New();
-  vtkTypeRevisionMacro(vtkNetworkImageSource, vtkImageAlgorithm);
+  vtkTypeMacro(vtkNetworkImageSource, vtkImageAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
