@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPExtractHistogram.h,v $
+  Module:    vtkPExtractHistogram.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,7 +28,7 @@ class VTK_EXPORT vtkPExtractHistogram : public vtkExtractHistogram
 {
 public:
   static vtkPExtractHistogram* New();
-  vtkTypeRevisionMacro(vtkPExtractHistogram, vtkExtractHistogram);
+  vtkTypeMacro(vtkPExtractHistogram, vtkExtractHistogram);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
