@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVClipDataSet.h,v $
+  Module:    vtkPVClipDataSet.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -15,17 +15,17 @@
 // .NAME vtkPVClipDataSet - Clip filter
 //
 // .SECTION Description
-// This is a subclass of vtkClipDataSet that allows selection of input scalars
+// This is a subclass of vtkTableBasedClipDataSet that allows selection of input scalars
 
 #ifndef __vtkPVClipDataSet_h
 #define __vtkPVClipDataSet_h
 
-#include "vtkClipDataSet.h"
+#include "vtkTableBasedClipDataSet.h"
 
-class VTK_EXPORT vtkPVClipDataSet : public vtkClipDataSet
+class VTK_EXPORT vtkPVClipDataSet : public vtkTableBasedClipDataSet
 {
 public:
-  vtkTypeRevisionMacro(vtkPVClipDataSet,vtkClipDataSet);
+  vtkTypeMacro(vtkPVClipDataSet,vtkTableBasedClipDataSet);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
