@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVDesktopDeliveryClient.h,v $
+  Module:    vtkPVDesktopDeliveryClient.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -36,17 +36,21 @@
 // vtkPVDesktopDeliveryServer
 //
 
+// Enable timing of image delivery
+// #define vtkPVDesktopDeliveryTIME
+
 #ifndef __vtkPVDesktopDeliveryClient_h
 #define __vtkPVDesktopDeliveryClient_h
 
-#include "vtkParallelRenderManager.h"
+#include "vtkPVClientServerRenderManager.h"
 
 class vtkCommand;
+class vtkImageCompressor;
 
-class VTK_EXPORT vtkPVDesktopDeliveryClient : public vtkParallelRenderManager
+class VTK_EXPORT vtkPVDesktopDeliveryClient : public vtkPVClientServerRenderManager
 {
 public:
-  vtkTypeRevisionMacro(vtkPVDesktopDeliveryClient, vtkParallelRenderManager);
+  vtkTypeMacro(vtkPVDesktopDeliveryClient, vtkPVClientServerRenderManager);
   virtual void PrintSelf(ostream &os, vtkIndent indent);
 
   static vtkPVDesktopDeliveryClient *New();
@@ -60,13 +64,6 @@ public:
   // ParallelRendering is turned back on when UseCompositing is turned on.
   virtual void SetUseCompositing(int v);
 
-  // Description:
-  // Set/Get the controller that is attached to a vtkDesktopDeliveryServer.
-  // This object will assume that the controller has two processors, and
-  // that the controller on the opposite side of the controller has been
-  // given to the server object.
-  virtual void SetController(vtkMultiProcessController *controller);
-
   virtual void ComputeVisiblePropBounds(vtkRenderer *ren, double bounds[6]);
 
   // Description:
@@ -94,35 +91,6 @@ public:
         + this->TransferTime + this->ImageProcessingTime);
   }
 
-  // For ParaView
-  void SetSquirtLevel (int l)
-  { 
-    if (l == 0)
-      {
-      this->SquirtOff();
-      }
-    else
-      {
-      this->SquirtOn(); 
-      this->SetSquirtCompressionLevel(l-1);
-      }
-  }
-
-  // Description:
-  // Enables or disables SQUIRT compression for image delivery.  By
-  // default, compression is off.  Note that this function may be replaced
-  // with a more universal image compression at a later date.
-  vtkGetMacro(Squirt, int);
-  vtkSetMacro(Squirt, int);
-  vtkBooleanMacro(Squirt, int);
-
-  // Description:
-  // Sets the compression level used by SQUIRT.  Higher values result in
-  // better compression but lower resolution in the color space (the size
-  // of the image is unaffected by this option).
-  vtkGetMacro(SquirtCompressionLevel, int);
-  vtkSetClampMacro(SquirtCompressionLevel, int, 0, 5);
-
   // Description:
   // The client may have many render windows and associated desktop delivery
   // client objects that all attach to the same desktop delivery server.
@@ -153,6 +121,13 @@ public:
   vtkGetVector2Macro(GUISize, int);
   vtkSetVector2Macro(GUISize, int);
 
+  vtkGetVector2Macro(GUISizeCompact, int);
+  vtkSetVector2Macro(GUISizeCompact, int);
+  vtkGetVector2Macro(ViewSizeCompact, int);
+  vtkSetVector2Macro(ViewSizeCompact, int);
+  vtkGetVector2Macro(ViewPositionCompact, int);
+  vtkSetVector2Macro(ViewPositionCompact, int);
+
   virtual void SetImageReductionFactorForUpdateRate(double desiredUpdateRate);
   float GetZBufferValue(int x, int y);
 
@@ -174,15 +149,14 @@ protected:
   double RemoteImageProcessingTime;
   double TransferTime;
 
-  virtual void SendWindowInformation();
-  virtual void SendRendererInformation(vtkRenderer *);
+  virtual void CollectWindowInformation(vtkMultiProcessStream& stream);
+  virtual void CollectRendererInformation(vtkRenderer *, vtkMultiProcessStream&);
 
-  // Squirt options (probably to be replaced later).
-  int Squirt;
-  int SquirtCompressionLevel;
-  vtkUnsignedCharArray *SquirtBuffer;
-
-  void SquirtDecompress(vtkUnsignedCharArray *in, vtkUnsignedCharArray *out);
+  //BTX
+  #if defined vtkPVDesktopDeliveryTIME
+  double CreationTime;
+  #endif
+  //ETX
 
   int UseCompositing;
 
@@ -192,6 +166,10 @@ protected:
   int WindowPosition[2];
   int GUISize[2];
 
+  int GUISizeCompact[2];
+  int ViewSizeCompact[2];
+  int ViewPositionCompact[2];
+
   int ReceivedImageFromServer;
   vtkCommand *ReceiveImageCallback;
   
