@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVEnSightMasterServerReader.cxx,v $
+  Module:    vtkPVEnSightMasterServerReader.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -43,7 +43,6 @@
 
 //----------------------------------------------------------------------------
 vtkStandardNewMacro(vtkPVEnSightMasterServerReader);
-vtkCxxRevisionMacro(vtkPVEnSightMasterServerReader, "$Revision: 1.28 $");
 
 vtkCxxSetObjectMacro(vtkPVEnSightMasterServerReader, Controller,
                      vtkMultiProcessController);
@@ -293,7 +292,8 @@ int vtkPVEnSightMasterServerReader::RequestInformation(
     }
   if(vtkPVEnSightMasterServerReaderSyncValues(
        &*this->Internal->TimeSetValues.begin(),
-       this->Internal->TimeSetValues.size(), this->NumberOfPieces,
+       static_cast<int>(this->Internal->TimeSetValues.size()),
+       this->NumberOfPieces,
        this->Controller) != VTK_OK)
     {
     vtkErrorMacro("Time set values do not match on all nodes.");
