@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPVExtractSelection.h,v $
+  Module:    vtkPVExtractSelection.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -37,10 +37,12 @@
 
 #include "vtkExtractSelection.h"
 
+class vtkSelectionNode;
+
 class VTK_EXPORT vtkPVExtractSelection : public vtkExtractSelection
 {
 public:
-  vtkTypeRevisionMacro(vtkPVExtractSelection,vtkExtractSelection);
+  vtkTypeMacro(vtkPVExtractSelection,vtkExtractSelection);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -69,17 +71,17 @@ protected:
 
   virtual int FillOutputPortInformation(int port, vtkInformation* info);
 
-  vtkSelection* LocateSelection(unsigned int level,
+  vtkSelectionNode* LocateSelection(unsigned int level,
     unsigned int index, vtkSelection* sel);
-  vtkSelection* LocateSelection(unsigned int composite_index, vtkSelection* sel);
+  vtkSelectionNode* LocateSelection(unsigned int composite_index, vtkSelection* sel);
 
 private:
   vtkPVExtractSelection(const vtkPVExtractSelection&);  // Not implemented.
   void operator=(const vtkPVExtractSelection&);  // Not implemented.
 
-  class vtkSelectionVector;
-  void RequestDataInternal(vtkSelectionVector& outputs,
-    vtkDataSet* geomOutput, vtkSelection* sel);
+  class vtkSelectionNodeVector;
+  void RequestDataInternal(vtkSelectionNodeVector& outputs,
+    vtkDataSet* geomOutput, vtkSelectionNode* sel);
 
   // Returns the combined content type for the selection.
   int GetContentType(vtkSelection* sel);
