@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVGenericRenderWindowInteractor.h,v $
+  Module:    vtkPVGenericRenderWindowInteractor.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -28,7 +28,7 @@ class VTK_EXPORT vtkPVGenericRenderWindowInteractor : public vtkRenderWindowInte
 {
 public:
   static vtkPVGenericRenderWindowInteractor *New();
-  vtkTypeRevisionMacro(vtkPVGenericRenderWindowInteractor, vtkRenderWindowInteractor);
+  vtkTypeMacro(vtkPVGenericRenderWindowInteractor, vtkRenderWindowInteractor);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   void SetPVRenderView(vtkPVRenderViewProxy *view);
