@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVGeometryFilter.cxx,v $
+  Module:    vtkPVGeometryFilter.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -18,10 +18,12 @@
 #include "vtkCallbackCommand.h"
 #include "vtkCellArray.h"
 #include "vtkCellData.h"
+#include "vtkCleanArrays.h"
 #include "vtkCommand.h"
 #include "vtkCompositeDataIterator.h"
 #include "vtkCompositeDataPipeline.h"
 #include "vtkCompositeDataSet.h"
+#include "vtkCompositeDataSet.h"
 #include "vtkDataSetSurfaceFilter.h"
 #include "vtkFloatArray.h"
 #include "vtkGarbageCollector.h"
@@ -34,14 +36,12 @@
 #include "vtkImageData.h"
 #include "vtkInformation.h"
 #include "vtkInformationVector.h"
-#include "vtkCompositeDataSet.h"
 #include "vtkMultiProcessController.h"
 #include "vtkObjectFactory.h"
 #include "vtkOutlineSource.h"
 #include "vtkPointData.h"
 #include "vtkPolyData.h"
 #include "vtkPolygon.h"
-#include "vtkPVTrivialProducer.h"
 #include "vtkRectilinearGrid.h"
 #include "vtkRectilinearGridOutlineFilter.h"
 #include "vtkSmartPointer.h"
@@ -49,18 +49,80 @@
 #include "vtkStripper.h"
 #include "vtkStructuredGrid.h"
 #include "vtkStructuredGridOutlineFilter.h"
+#include "vtkTimerLog.h"
 #include "vtkUnsignedCharArray.h"
-#include "vtkUnstructuredGrid.h"
 #include "vtkUnsignedIntArray.h"
+#include "vtkAlgorithmOutput.h"
+#include "vtkUnstructuredGrid.h"
 
 #include <vtkstd/map>
+#include <vtkstd/vector>
 #include <vtkstd/string>
+#include <assert.h>
 
-vtkCxxRevisionMacro(vtkPVGeometryFilter, "$Revision: 1.85 $");
 vtkStandardNewMacro(vtkPVGeometryFilter);
 
 vtkCxxSetObjectMacro(vtkPVGeometryFilter, Controller, vtkMultiProcessController);
 
+class vtkPVGeometryFilter::vtkPolyDataVector :
+  public vtkstd::vector<vtkPolyData*>
+{
+};
+
+class vtkPVGeometryFilter::BoundsReductionOperation : public vtkCommunicator::Operation
+{
+public:
+  // Subclasses must overload this method, which performs the actual
+  // operations.  The methods should first do a reintepret cast of the arrays
+  // to the type suggestsed by \c datatype (which will be one of the VTK type
+  // identifiers like VTK_INT, etc.).  Both arrays are considered top be
+  // length entries.  The method should perform the operation A*B (where * is
+  // a placeholder for whatever operation is actually performed) and store the
+  // result in B.  The operation is assumed to be associative.  Commutativity
+  // is specified by the Commutative method.
+  virtual void Function(const void *A, void *B, vtkIdType length,
+    int datatype)
+    {
+    assert((datatype == VTK_DOUBLE) && (length==6));
+    (void)datatype;
+    (void)length;
+    const double* bdsA = reinterpret_cast<const double*>(A);
+    double* bdsB = reinterpret_cast<double*>(B);
+    if (bdsA[0] < bdsB[0])
+      {
+      bdsB[0] = bdsA[0];
+      }
+    if (bdsA[1] > bdsB[1])
+      {
+      bdsB[1] = bdsA[1];
+      }
+    if (bdsA[2] < bdsB[2])
+      {
+      bdsB[2] = bdsA[2];
+      }
+    if (bdsA[3] > bdsB[3])
+      {
+      bdsB[3] = bdsA[3];
+      }
+    if (bdsA[4] < bdsB[4])
+      {
+      bdsB[4] = bdsA[4];
+      }
+    if (bdsA[5] > bdsB[5])
+      {
+      bdsB[5] = bdsA[5];
+      } 
+    }
+
+  // Description:
+  // Subclasses override this method to specify whether their operation
+  // is commutative.  It should return 1 if commutative or 0 if not.
+  virtual int Commutative()
+    {
+    return 1;
+    }
+};
+
 //----------------------------------------------------------------------------
 vtkPVGeometryFilter::vtkPVGeometryFilter ()
 {
@@ -89,6 +151,11 @@ vtkPVGeometryFilter::vtkPVGeometryFilter ()
   this->PassThroughPointIds = 1;
   this->ForceUseStrips = 0;
   this->StripModFirstPass = 1;
+  this->MakeOutlineOfInput = 0;
+
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_RANGES(), 1);
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_BOUNDS(), 1);  
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_TOPOLOGY(), 1);
 }
 
 //----------------------------------------------------------------------------
@@ -128,7 +195,10 @@ void vtkPVGeometryFilter::InternalProgressCallback(vtkAlgorithm *algorithm)
 {
   // This limits progress for only the DataSetSurfaceFilter.
   float progress = algorithm->GetProgress();
-  this->UpdateProgress(progress);
+  if (progress > 0 && progress < 1)
+    {
+    this->UpdateProgress(progress);
+    }
   if (this->AbortExecute)
     {
     algorithm->SetAbortExecute(1);
@@ -219,32 +289,28 @@ static void vtkFillPartialArrays(vtkDataSetAttributes* dsa,
 }
 
 //----------------------------------------------------------------------------
-vtkCompositeDataSet* vtkPVGeometryFilter::FillPartialArrays(
-  vtkCompositeDataSet* input)
+void vtkPVGeometryFilter::FillPartialArrays(
+  vtkPVGeometryFilter::vtkPolyDataVector& inputs)
 {
-  if (!input)
+  if (inputs.size() == 0)
     {
-    return NULL;
+    return;
     }
 
   vtkArrayMap arrayMapPD;
   vtkArrayMap arrayMapCD;
 
-
-  vtkCompositeDataSet* output = input->NewInstance();
-  output->CopyStructure(input);
-  vtkCompositeDataIterator* iter = input->NewIterator();
-
   // build arrayMap with representative arrays for each array name.
   // As far as active attributes are concerned, we only fill in the scalars or
   // vectors. For appendPolyData to work correctly, it is essential that the
   // active scalars/vectors are the same array on all the inputs. To avoid
   // contention, we choose the first available scalars/vectors.
   vtkstd::string activePScalars, activeCScalars, activePVectors, activeCVectors;
-  
-  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+ 
+  for (vtkPolyDataVector::iterator iter = inputs.begin();
+    iter!= inputs.end(); ++iter)
     {
-    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
+    vtkPolyData* ds = *iter;
     if (ds)
       {
       if (activePScalars=="" && ds->GetPointData()->GetScalars() &&
@@ -272,19 +338,13 @@ vtkCompositeDataSet* vtkPVGeometryFilter::FillPartialArrays(
       ::vtkBuildArrayMap(ds->GetPointData(), arrayMapPD);
       ::vtkBuildArrayMap(ds->GetCellData(), arrayMapCD);
       }
-
-    vtkDataSet* clone = ds->NewInstance();
-    clone->ShallowCopy(ds);
-    output->SetDataSet(iter, clone);
-    clone->Delete();
     }
-  iter->Delete();
 
   // Now fill partial arrays.
-  iter = output->NewIterator();
-  for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem())
+  for (vtkPolyDataVector::iterator iter = inputs.begin();
+    iter!= inputs.end(); ++iter)
     {
-    vtkDataSet* ds = vtkDataSet::SafeDownCast(iter->GetCurrentDataObject());
+    vtkPolyData* ds = *iter;
     if (ds)
       {
       ::vtkFillPartialArrays(ds->GetPointData(), arrayMapPD, ds->GetNumberOfPoints());
@@ -308,14 +368,11 @@ vtkCompositeDataSet* vtkPVGeometryFilter::FillPartialArrays(
         }
       }
     }
-
-  iter->Delete();
-  return output;
 }
 
 //----------------------------------------------------------------------------
 int vtkPVGeometryFilter::RequestInformation(
-  vtkInformation*, vtkInformationVector**, vtkInformationVector* outputVector)
+  vtkInformation*, vtkInformationVector** vtkNotUsed(inVectors), vtkInformationVector* outputVector)
 {
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
 
@@ -326,28 +383,77 @@ int vtkPVGeometryFilter::RequestInformation(
   return 1;
 }
 
+//----------------------------------------------------------------------------
+int vtkPVGeometryFilter::RequestUpdateExtent(vtkInformation* request,
+  vtkInformationVector** inputVector,
+  vtkInformationVector* outputVector)
+{
+  /* 
+  vtkUnstructuredGrid* ug_input =
+    vtkUnstructuredGrid::GetData(inputVector[0], 0);
+  vtkCompositeDataSet* cd_input =
+    vtkCompositeDataSet::GetData(inputVector[0], 0);
+  if (ug_input || cd_input)
+    {
+    vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
+    vtkInformation *outInfo = outputVector->GetInformationObject(0);
+    int numPieces, ghostLevels;
+    numPieces =
+      outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_PIECES());
+    ghostLevels =
+      outInfo->Get(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS());
+    if (numPieces > 1)
+      {
+      ++ghostLevels;
+      }
+    inInfo->Set(vtkStreamingDemandDrivenPipeline::UPDATE_NUMBER_OF_GHOST_LEVELS(),
+      ghostLevels);
+    }
+
+  */
+  return this->Superclass::RequestUpdateExtent(request, inputVector,
+    outputVector);
+}
+
 //----------------------------------------------------------------------------
 void vtkPVGeometryFilter::ExecuteBlock(
   vtkDataObject* input, vtkPolyData* output, int doCommunicate)
 {
+  if (this->UseOutline && this->MakeOutlineOfInput)
+    {    
+    vtkAlgorithmOutput *pport = input->GetProducerPort();
+    vtkDataObject *insin = NULL;
+    if (pport)
+      {
+      vtkAlgorithm *alg = pport->GetProducer();
+      if (alg && 
+          alg->GetNumberOfInputPorts() && 
+          alg->GetNumberOfInputConnections(0))
+        {
+        insin = alg->GetInputDataObject(0,0);
+        }
+      }
+    if (insin)
+      {
+      input = insin;
+      }
+    }
+
   if (input->IsA("vtkImageData"))
     {
     this->ImageDataExecute(static_cast<vtkImageData*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
 
   if (input->IsA("vtkStructuredGrid"))
     {
     this->StructuredGridExecute(static_cast<vtkStructuredGrid*>(input), output);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
 
   if (input->IsA("vtkRectilinearGrid"))
     {
     this->RectilinearGridExecute(static_cast<vtkRectilinearGrid*>(input),output);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
 
@@ -355,32 +461,27 @@ void vtkPVGeometryFilter::ExecuteBlock(
     {
     this->UnstructuredGridExecute(
       static_cast<vtkUnstructuredGrid*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
 
   if (input->IsA("vtkPolyData"))
     {
     this->PolyDataExecute(static_cast<vtkPolyData*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
   if (input->IsA("vtkHyperOctree"))
     {
     this->OctreeExecute(static_cast<vtkHyperOctree*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
   if (input->IsA("vtkDataSet"))
     {
     this->DataSetExecute(static_cast<vtkDataSet*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
   if (input->IsA("vtkGenericDataSet"))
     {
     this->GenericDataSetExecute(static_cast<vtkGenericDataSet*>(input), output, doCommunicate);
-    this->ExecuteCellNormals(output, doCommunicate);
     return;
     }
 }
@@ -390,6 +491,14 @@ int vtkPVGeometryFilter::RequestData(vtkInformation* request,
                                      vtkInformationVector** inputVector,
                                      vtkInformationVector* outputVector)
 {
+  vtkInformation* info = outputVector->GetInformationObject(0);
+  vtkPolyData *output = vtkPolyData::SafeDownCast(
+    info->Get(vtkDataObject::DATA_OBJECT()));
+  if (!output) 
+    {
+    return 0;
+    }
+
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
 
   vtkDataObject* inputDobj = inInfo->Get(vtkDataObject::DATA_OBJECT());
@@ -397,13 +506,18 @@ int vtkPVGeometryFilter::RequestData(vtkInformation* request,
     vtkCompositeDataSet::SafeDownCast(inputDobj);
   if (compInput) 
     {
-    return this->RequestCompositeData(request, inputVector, outputVector);
-    }
+    vtkGarbageCollector::DeferredCollectionPush();
+    vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::RequestData");
+    this->RequestCompositeData(request, inputVector, outputVector);
+    this->ExecuteCellNormals(output, 0);
+    this->RemoveGhostCells(output);
+    vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::RequestData");
 
-  vtkInformation* info = outputVector->GetInformationObject(0);
-  vtkPolyData *output = vtkPolyData::SafeDownCast(
-    info->Get(vtkDataObject::DATA_OBJECT()));
-  if (!output) {return 0;}
+    vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::GarbageCollect");
+    vtkGarbageCollector::DeferredCollectionPop();
+    vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::GarbageCollect");
+    return 1;
+    }
 
   vtkDataObject *input = vtkDataSet::SafeDownCast(inputDobj);
   if (!input)
@@ -414,13 +528,9 @@ int vtkPVGeometryFilter::RequestData(vtkInformation* request,
     }
 
   this->ExecuteBlock(input, output, 1);
+  this->ExecuteCellNormals(output, 1);
+  this->RemoveGhostCells(output);
 
-  vtkDataArray* ghost = output->GetCellData()->GetArray("vtkGhostLevels");
-  if (ghost)
-    {
-    output->RemoveGhostCells(1);
-    }
-  
   return 1;
 }
 
@@ -429,6 +539,8 @@ int vtkPVGeometryFilter::RequestCompositeData(vtkInformation*,
                                               vtkInformationVector** inputVector,
                                               vtkInformationVector* outputVector)
 {
+  vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::RequestCompositeData");
+
   vtkInformation* inInfo = inputVector[0]->GetInformationObject(0);
 
   vtkInformation* info = outputVector->GetInformationObject(0);
@@ -445,31 +557,70 @@ int vtkPVGeometryFilter::RequestCompositeData(vtkInformation*,
     return 0;
     }
 
-  // fills partial array.
-  vtkSmartPointer<vtkCompositeDataSet> constructuredInput;
-  constructuredInput.TakeReference(this->FillPartialArrays(mgInput));
-
-  if (this->CheckAttributes(constructuredInput))
+  vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::CheckAttributes");
+  if (this->CheckAttributes(mgInput))
     {
     return 0;
     }
+  vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::CheckAttributes");
+
+  vtkPolyDataVector blocks;
+  vtkPolyDataVector::iterator blocksIter;
 
-  vtkAppendPolyData* append = vtkAppendPolyData::New();
   int numInputs = 0;
-
-  int retVal = 0;
-  if (this->ExecuteCompositeDataSet(constructuredInput, append, numInputs))
+  if (this->ExecuteCompositeDataSet(mgInput, blocks, numInputs))
     {
-    if (numInputs > 0)
+    vtkCleanArrays* cleaner = vtkCleanArrays::New();
+    if (blocks.size() > 0)
       {
-      append->Update();
-      }
-    output->ShallowCopy(append->GetOutput());
-    append->Delete();
-    retVal = 1;
-    }
+      // fills partial array.
+      vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::FillPartialArrays");
+      this->FillPartialArrays(blocks);
+      vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::FillPartialArrays");
 
-  return retVal;
+      vtkTimerLog::MarkStartEvent("Append Blocks");
+      vtkPolyData* appenderOutput = vtkPolyData::New();
+
+      // We avoid setting the polydatas as input to vtkAppendPolyData and using
+      // it as a traditional vtkAlgorithm since if there are a large number of
+      // blocks, it results in excessive garbage collection which slow thing
+      // down considerably.
+      vtkAppendPolyData* append = vtkAppendPolyData::New();
+      append->ExecuteAppend(appenderOutput,
+        &blocks[0],
+        static_cast<int>(blocks.size()));
+      append->Delete();
+
+      for (blocksIter = blocks.begin(); blocksIter != blocks.end();
+        ++blocksIter)
+        {
+        (*blocksIter)->FastDelete();
+        }
+      vtkTimerLog::MarkEndEvent("Append Blocks");
+
+      // vtkAppendPolyData will throw errors if it has no input.
+      cleaner->SetInputConnection(appenderOutput->GetProducerPort());
+      appenderOutput->Delete();
+      }
+    else
+      {
+      // it's legal for some processes to have no blocks. However, in that case
+      // too, we need to ensure that vtkCleanArrays is executed on those
+      // processes as well to avoid deadlocks. Hence we setup an empty polydata
+      // as input.
+      vtkPolyData* emptyInput = vtkPolyData::New();
+      cleaner->SetInput(emptyInput);
+      emptyInput->Delete();
+      }
+    cleaner->Update();
+    output->ShallowCopy(cleaner->GetOutput());
+    cleaner->RemoveAllInputs();
+    cleaner->Delete();
+    vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::RequestCompositeData");
+    return 1;
+    }
+  vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::RequestCompositeData");
+  return 0;
 }
 
 //----------------------------------------------------------------------------
@@ -506,9 +657,10 @@ void vtkPVGeometryFilter::AddHierarchicalIndex(vtkPolyData* pd,
 //----------------------------------------------------------------------------
 int vtkPVGeometryFilter::ExecuteCompositeDataSet(
   vtkCompositeDataSet* mgInput, 
-  vtkAppendPolyData* append, 
+  vtkPolyDataVector &outputs,
   int& numInputs)
 {
+  vtkTimerLog::MarkStartEvent("vtkPVGeometryFilter::ExecuteCompositeDataSet");
   vtkSmartPointer<vtkCompositeDataIterator> iter;
   iter.TakeReference(mgInput->NewIterator());
 
@@ -521,7 +673,7 @@ int vtkPVGeometryFilter::ExecuteCompositeDataSet(
     // iter skips empty blocks automatically.
     totNumBlocks++;
     }
- 
+
   unsigned int group = 0;
   for (iter->InitTraversal(); !iter->IsDoneWithTraversal(); iter->GoToNextItem(), group++)
     {
@@ -530,6 +682,7 @@ int vtkPVGeometryFilter::ExecuteCompositeDataSet(
     
     vtkPolyData* tmpOut = vtkPolyData::New();
     this->ExecuteBlock(block, tmpOut, 0);
+    //// assert(tmpOut->GetReferenceCount() == 1);
 
     if (hdIter)
       {
@@ -541,14 +694,13 @@ int vtkPVGeometryFilter::ExecuteCompositeDataSet(
       this->AddCompositeIndex(tmpOut, iter->GetCurrentFlatIndex());
       }
 
-    append->AddInput(tmpOut);
-    // Call FastDelete() instead of Delete() to avoid garbage
-    // collection checks. This improves the preformance significantly
-    tmpOut->FastDelete();
+    outputs.push_back(tmpOut);
+
     numInputs++;
     this->UpdateProgress(static_cast<float>(numInputs)/totNumBlocks);
     }
 
+  vtkTimerLog::MarkEndEvent("vtkPVGeometryFilter::ExecuteCompositeDataSet");
   return 1;
 }
 
@@ -583,27 +735,16 @@ void vtkPVGeometryFilter::ExecuteCellNormals(vtkPolyData* output, int doCommunic
     }
   if( this->Controller && doCommunicate )
     {
-    // An MPI gather or reduce would be easier ...
-    if (this->Controller->GetLocalProcessId() == 0)  
+    int reduced_skip = 0;
+    if (!this->Controller->AllReduce(&skip, &reduced_skip, 1,
+        vtkCommunicator::MAX_OP))
       {
-      int tmp, idx;
-      for (idx = 1; idx < this->Controller->GetNumberOfProcesses(); ++idx)
-        {
-        this->Controller->Receive(&tmp, 1, idx, 89743);
-        if (tmp)
-          {
-          skip = 1;
-          }
-        }
-      for (idx = 1; idx < this->Controller->GetNumberOfProcesses(); ++idx)
-        {
-        this->Controller->Send(&skip, 1, idx, 89744);
-        }
+      vtkErrorMacro("Failed to reduce correctly.");
+      skip = 1;
       }
     else
       {
-      this->Controller->Send(&skip, 1, 0, 89743);
-      this->Controller->Receive(&skip, 1, 0, 89744);
+      skip = reduced_skip;
       }
     }
   if (skip)
@@ -675,46 +816,19 @@ void vtkPVGeometryFilter::DataSetExecute(
 
   input->GetBounds(bds);
 
+  vtkPVGeometryFilter::BoundsReductionOperation operation;
   if ( procid && doCommunicate )
     {
     // Satellite node
-    this->Controller->Send(bds, 6, 0, 792390);
+    this->Controller->Reduce(bds, NULL, 6, &operation, 0);
     }
   else
     {
-    int idx;
-    double tmp[6];
-    
-    if (doCommunicate)
+    if (this->Controller && doCommunicate)
       {
-      for (idx = 1; idx < numProcs; ++idx)
-        {
-        this->Controller->Receive(tmp, 6, idx, 792390);
-        if (tmp[0] < bds[0])
-          {
-          bds[0] = tmp[0];
-          }
-        if (tmp[1] > bds[1])
-          {
-          bds[1] = tmp[1];
-          }
-        if (tmp[2] < bds[2])
-          {
-          bds[2] = tmp[2];
-          }
-        if (tmp[3] > bds[3])
-          {
-          bds[3] = tmp[3];
-          }
-        if (tmp[4] < bds[4])
-          {
-          bds[4] = tmp[4];
-          }
-        if (tmp[5] > bds[5])
-          {
-          bds[5] = tmp[5];
-          }
-        }
+      double tmp[6];
+      this->Controller->Reduce(bds, tmp, 6, &operation, 0);
+      memcpy(bds, tmp, 6*sizeof(double));
       }
 
     if (bds[1] >= bds[0] && bds[3] >= bds[2] && bds[5] >= bds[4])
@@ -772,46 +886,19 @@ void vtkPVGeometryFilter::GenericDataSetExecute(
 
   input->GetBounds(bds);
 
+  vtkPVGeometryFilter::BoundsReductionOperation operation;
   if ( procid && doCommunicate )
     {
     // Satellite node
-    this->Controller->Send(bds, 6, 0, 792390);
+    this->Controller->Reduce(bds, NULL, 6, &operation, 0);
     }
   else
     {
-    int idx;
-    double tmp[6];
-    
     if (doCommunicate)
       {
-      for (idx = 1; idx < numProcs; ++idx)
-        {
-        this->Controller->Receive(tmp, 6, idx, 792390);
-        if (tmp[0] < bds[0])
-          {
-          bds[0] = tmp[0];
-          }
-        if (tmp[1] > bds[1])
-          {
-          bds[1] = tmp[1];
-          }
-        if (tmp[2] < bds[2])
-          {
-          bds[2] = tmp[2];
-          }
-        if (tmp[3] > bds[3])
-          {
-          bds[3] = tmp[3];
-          }
-        if (tmp[4] < bds[4])
-          {
-          bds[4] = tmp[4];
-          }
-        if (tmp[5] > bds[5])
-          {
-          bds[5] = tmp[5];
-          }
-        }
+      double tmp[6];
+      this->Controller->Reduce(bds, tmp, 6, &operation, 0);
+      memcpy(bds, tmp, 6*sizeof(double));
       }
 
     // only output in process 0.
@@ -823,41 +910,6 @@ void vtkPVGeometryFilter::GenericDataSetExecute(
     }
 }
 
-//----------------------------------------------------------------------------
-void vtkPVGeometryFilter::DataSetSurfaceExecute(vtkDataSet* input, 
-                                                vtkPolyData* output)
-{
-  vtkDataSet* clone = input->NewInstance();
-
-  // If we directly pass clone to the input of the DataSetSurfaceFilter, then
-  // for structured datasets, the information about whole extent of the data is
-  // lost. This is so, because vtkTrivialProducer sets that the whole
-  // extent for the output to be same as the extent of the data. This results is
-  // false boundary surfaces for such data when number of processes > 1. To
-  // overcome that issue, we are creating a new produces which still reports the
-  // "real" whole extent, however, has a special extent translator that never
-  // results in a request for the whole extent, only that provided by the data.
-  vtkPVTrivialProducer* producer = vtkPVTrivialProducer::New();
-  producer->SetWholeExtent(input->GetWholeExtent());
-  producer->SetOutput(clone);
-
-  clone->ShallowCopy(input);
-  this->DataSetSurfaceFilter->SetInputConnection(producer->GetOutputPort());
-  
-  producer->Delete();
-  clone->Delete();
-
-  // Observe the progress of the internal filter.
-  this->DataSetSurfaceFilter->AddObserver(vtkCommand::ProgressEvent, 
-                                          this->InternalProgressObserver);
-  this->DataSetSurfaceFilter->Update();
-  // The internal filter is finished.  Remove the observer.
-  this->DataSetSurfaceFilter->RemoveObserver(this->InternalProgressObserver);
-  this->DataSetSurfaceFilter->SetInput(0);
-
-  output->ShallowCopy(this->DataSetSurfaceFilter->GetOutput());
-}
-
 //----------------------------------------------------------------------------
 void vtkPVGeometryFilter::ImageDataExecute(vtkImageData *input,
                                            vtkPolyData* output,
@@ -884,7 +936,11 @@ void vtkPVGeometryFilter::ImageDataExecute(vtkImageData *input,
 //      !this->UseOutline)
   if (!this->UseOutline)
     {
-    this->DataSetSurfaceExecute(input, output);
+    if (input->GetNumberOfCells() > 0)
+      {
+      this->DataSetSurfaceFilter->StructuredExecute(input,
+        output, input->GetExtent(), ext);
+      }
     this->OutlineFlag = 0;
     return;
     }  
@@ -894,7 +950,8 @@ void vtkPVGeometryFilter::ImageDataExecute(vtkImageData *input,
   // Otherwise, let OutlineSource do all the work
   //
   
-  if (output->GetUpdatePiece() == 0 || !doCommunicate)
+  if (ext[1] >= ext[0] && ext[3] >= ext[2] && ext[5] >= ext[4] &&
+    (output->GetUpdatePiece() == 0 || !doCommunicate))
     {
     spacing = input->GetSpacing();
     origin = input->GetOrigin();
@@ -912,6 +969,7 @@ void vtkPVGeometryFilter::ImageDataExecute(vtkImageData *input,
 
     output->SetPoints(outline->GetOutput()->GetPoints());
     output->SetLines(outline->GetOutput()->GetLines());
+    output->SetPolys(outline->GetOutput()->GetPolys());
     outline->Delete();
     }
   else
@@ -935,7 +993,11 @@ void vtkPVGeometryFilter::StructuredGridExecute(vtkStructuredGrid* input,
 //      !this->UseOutline)
   if (!this->UseOutline)
     {
-    this->DataSetSurfaceExecute(input, output);
+    if (input->GetNumberOfCells() > 0)
+      {
+      this->DataSetSurfaceFilter->StructuredExecute(input, output, input->GetExtent(),
+        input->GetWholeExtent());
+      }
     this->OutlineFlag = 0;
     return;
     }  
@@ -971,7 +1033,11 @@ void vtkPVGeometryFilter::RectilinearGridExecute(vtkRectilinearGrid* input,
 //      !this->UseOutline)
   if (!this->UseOutline)
     {
-    this->DataSetSurfaceExecute(input, output);
+    if (input->GetNumberOfCells() > 0)
+      {
+      this->DataSetSurfaceFilter->StructuredExecute(input, output,
+        input->GetExtent(), input->GetWholeExtent());
+      }
     this->OutlineFlag = 0;
     return;
     }  
@@ -1000,7 +1066,10 @@ void vtkPVGeometryFilter::UnstructuredGridExecute(
   if (!this->UseOutline)
     {
     this->OutlineFlag = 0;
-    this->DataSetSurfaceExecute(input, output);
+    if (input->GetNumberOfCells() > 0)
+      {
+      this->DataSetSurfaceFilter->UnstructuredGridExecute(input, output);
+      }
     return;
     }
   
@@ -1226,3 +1295,13 @@ void vtkPVGeometryFilter::SetUseStrips(int newvalue)
     this->StripSettingMTime.Modified();
     }
 }
+
+//----------------------------------------------------------------------------
+void vtkPVGeometryFilter::RemoveGhostCells(vtkPolyData* output)
+{
+  vtkDataArray* ghost = output->GetCellData()->GetArray("vtkGhostLevels");
+  if (ghost)
+    {
+    output->RemoveGhostCells(1);
+    }
+}
