@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVGlyphFilter.cxx,v $
+  Module:    vtkPVGlyphFilter.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -31,7 +31,6 @@
 #include "vtkProcessModule.h"
 #include "vtkUniformGrid.h"
 
-vtkCxxRevisionMacro(vtkPVGlyphFilter, "$Revision: 1.34 $");
 vtkStandardNewMacro(vtkPVGlyphFilter);
 
 //-----------------------------------------------------------------------------
