@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVTrackballZoom.h,v $
+  Module:    vtkPVTrackballZoom.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -27,7 +27,7 @@ class VTK_EXPORT vtkPVTrackballZoom : public vtkCameraManipulator
 {
 public:
   static vtkPVTrackballZoom *New();
-  vtkTypeRevisionMacro(vtkPVTrackballZoom, vtkCameraManipulator);
+  vtkTypeMacro(vtkPVTrackballZoom, vtkCameraManipulator);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
