@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPythonProgrammableFilter.cxx,v $
+  Module:    vtkPythonProgrammableFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -14,6 +14,7 @@
 =========================================================================*/
 #include "vtkPythonProgrammableFilter.h"
 
+#include "vtkCommand.h"
 #include "vtkDataObject.h"
 #include "vtkDataObjectTypes.h"
 #include "vtkDataSet.h"
@@ -31,7 +32,6 @@
 #include <vtkstd/map>
 #include <vtkstd/string>
 
-vtkCxxRevisionMacro(vtkPythonProgrammableFilter, "$Revision: 1.32 $");
 vtkStandardNewMacro(vtkPythonProgrammableFilter);
 
 //----------------------------------------------------------------------------
@@ -41,33 +41,44 @@ typedef vtkstd::map<vtkstd::string, vtkstd::string> ParametersT;
 class vtkPythonProgrammableFilterImplementation
 {
 public:
-  vtkPythonProgrammableFilterImplementation() :
-    Interpretor(NULL)
-  {
-  }
-
-  void DestroyInterpretor()
-    {
-      if (this->Interpretor)
-        {
-        // The following is necessary because the Delete() may
-        // cause the destruction of vtkPythonProgrammableFilter
-        // which calls DestroyInterpretor() in its destructor.
-        // If this->Interpretor is not set to 0, it will be
-        // deleted a second time causing segmentation fault.
-        vtkPVPythonInterpretor* interp = this->Interpretor;
-        this->Interpretor = 0;
-        interp->Delete();
-        }
-    }
-
-  vtkPVPythonInterpretor* Interpretor;
-  
   // Stores name-value parameters that will be passed to running scripts
   ParametersT Parameters;
-
 };
 
+vtkPVPythonInterpretor* vtkPythonProgrammableFilter::GlobalPipelineInterpretor = 0;
+
+// Upon disconnect, remove the global interp
+class vtkPythonProgrammableFilterObserver : public vtkCommand
+{
+public:
+  virtual void Execute(vtkObject*, unsigned long, void*)
+    {
+    if (vtkPythonProgrammableFilter::GlobalPipelineInterpretor)
+      {
+      vtkPythonProgrammableFilter::GlobalPipelineInterpretor->Delete();
+      vtkPythonProgrammableFilter::GlobalPipelineInterpretor = 0;
+      }
+    }
+};
+
+//----------------------------------------------------------------------------
+vtkPVPythonInterpretor* vtkPythonProgrammableFilter::GetGlobalPipelineInterpretor()
+{
+  if (!GlobalPipelineInterpretor)
+    {
+    vtkPythonProgrammableFilter::GlobalPipelineInterpretor = vtkPVPythonInterpretor::New();
+    vtkPythonProgrammableFilter::GlobalPipelineInterpretor->SetCaptureStreams(true);
+    const char* argv0 = vtkProcessModule::GetProcessModule()->
+      GetOptions()->GetArgv0();
+    vtkPythonProgrammableFilter::GlobalPipelineInterpretor->InitializeSubInterpretor(
+      1, (char**)&argv0);
+    vtkPythonProgrammableFilterObserver* obs = new vtkPythonProgrammableFilterObserver;
+    vtkProcessModule::GetProcessModule()->AddObserver(vtkCommand::ExitEvent, obs);
+    obs->UnRegister(0);
+    }
+  return vtkPythonProgrammableFilter::GlobalPipelineInterpretor;
+}
+
 //----------------------------------------------------------------------------
 vtkPythonProgrammableFilter::vtkPythonProgrammableFilter() :
   Implementation(new vtkPythonProgrammableFilterImplementation())
@@ -86,7 +97,6 @@ vtkPythonProgrammableFilter::~vtkPythonProgrammableFilter()
   this->SetInformationScript(NULL);
   this->SetPythonPath(0);
 
-  this->Implementation->DestroyInterpretor();
   delete this->Implementation;
 }
 
@@ -161,9 +171,6 @@ int vtkPythonProgrammableFilter::RequestInformation(
   vtkInformationVector**, 
   vtkInformationVector* outputVector)
 {
-  // Start with a new interpretor
-  this->Implementation->DestroyInterpretor();
-
   vtkInformation* outInfo = outputVector->GetInformationObject(0);
 
   // Setup ExtentTranslator so that all downstream piece requests are
@@ -230,21 +237,25 @@ void vtkPythonProgrammableFilter::Exec(const char* script,
     return;
     }
 
-  if (this->Implementation->Interpretor == NULL)
+
+  // Prepend the paths defined in PythonPath to sys.path
+  if (this->PythonPath)
     {
-    this->Implementation->Interpretor = vtkPVPythonInterpretor::New();
-    this->Implementation->Interpretor->SetCaptureStreams(true);
-    const char* argv0 = vtkProcessModule::GetProcessModule()->
-      GetOptions()->GetArgv0();
-    this->Implementation->Interpretor->InitializeSubInterpretor(
-      1, (char**)&argv0);
-    if (this->PythonPath)
+    vtkstd::string pathscript;
+    pathscript += "import sys\n";
+    vtkstd::vector<vtksys::String> paths = vtksys::SystemTools::SplitString(
+      this->PythonPath, ';');
+    for (unsigned int cc=0; cc < static_cast<unsigned int>(paths.size()); cc++)
       {
-      vtkstd::vector<vtksys::String> paths = vtksys::SystemTools::SplitString(
-        this->PythonPath, ';');
-      for (unsigned int cc=0; cc < static_cast<unsigned int>(paths.size()); cc++)
+      if (!paths[cc].empty())
         {
-        this->Implementation->Interpretor->AddPythonPath(paths[cc].c_str());
+        pathscript += "if not ";
+        pathscript += paths[cc];
+        pathscript += " in sys.path:\n";
+        pathscript += "  sys.path.insert(0, ";
+        pathscript += paths[cc];
+        pathscript += ")\n";
+        vtkPythonProgrammableFilter::GetGlobalPipelineInterpretor()->RunSimpleString(pathscript.c_str());
         }
       }
     }
@@ -255,7 +266,7 @@ void vtkPythonProgrammableFilter::Exec(const char* script,
   fscript += funcname;
 
   // Set the parameters defined by user.
-  fscript += "(self):\n";
+  fscript += "(self, inputs = None, output = None):\n";
   for(ParametersT::const_iterator parameter = 
         this->Implementation->Parameters.begin();
       parameter != this->Implementation->Parameters.end();
@@ -296,10 +307,22 @@ void vtkPythonProgrammableFilter::Exec(const char* script,
       }
     }
   fscript += "\n";
-  this->Implementation->Interpretor->RunSimpleString(fscript.c_str());
+  vtkPythonProgrammableFilter::GetGlobalPipelineInterpretor()->RunSimpleString(fscript.c_str());
 
   vtkstd::string runscript;
-  runscript = "from paraview import vtk\n";
+
+  runscript += "from paraview import vtk\n";
+  runscript += "from paraview import servermanager\n";
+  runscript += "if servermanager.progressObserverTag:\n";
+  runscript += "  servermanager.ToggleProgressPrinting()\n";
+  runscript += "hasnumpy = True\n";
+  runscript += "try:\n";
+  runscript += "  from numpy import *\n";
+  runscript += "except ImportError:\n";
+  runscript += "  hasnumpy = False\n";
+  runscript += "if hasnumpy:\n";
+  runscript += "  from paraview.vtk import dataset_adapter\n";
+  runscript += "  from paraview.vtk.algorithms import *\n";
 
   // Set self to point to this
   char addrofthis[1024];
@@ -310,16 +333,38 @@ void vtkPythonProgrammableFilter::Exec(const char* script,
     {
     aplus += 2; //skip over "0x"
     }
+
+  // Call the function
+  runscript += "myarg = ";
+  runscript += "vtk.vtkProgrammableFilter('";
+  runscript += aplus;
+  runscript += "')\n";
+  runscript += "if hasnumpy:\n";
+  runscript += "  inputs = []\n";
+  runscript += "  index = 0\n";
+  int numinps = this->GetNumberOfInputConnections(0);
+  for (int i=0; i<numinps; i++)
+    {
+    runscript += 
+      "  inputs.append(dataset_adapter.WrapDataObject(myarg.GetInputDataObject(0, index)))\n";
+    runscript += "  index += 1\n";
+    }
+  runscript += "  output = dataset_adapter.WrapDataObject(myarg.GetOutputDataObject(0))\n";
+  runscript += "else:\n";
+  runscript += "  inputs = None\n";
+  runscript += "  output = None\n";
   
   // Call the function
   runscript += funcname;
-  runscript += "(vtk.vtkProgrammableFilter('";
-  runscript += aplus;
-  runscript += "'))\n";
+  runscript += "(myarg, inputs, output)\n";
+  runscript += "del inputs\n";
+  runscript += "del output\n";
+  runscript += "del myarg\n";
+  runscript += "import gc\n";
+  runscript += "gc.collect()\n";
 
-  this->Implementation->Interpretor->RunSimpleString(runscript.c_str());
-
-  this->Implementation->Interpretor->FlushMessages();
+  vtkPythonProgrammableFilter::GetGlobalPipelineInterpretor()->RunSimpleString(runscript.c_str());
+  vtkPythonProgrammableFilter::GetGlobalPipelineInterpretor()->FlushMessages();
 }
 
 //----------------------------------------------------------------------------
