@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkRedistributePolyData.cxx,v $
+  Module:    vtkRedistributePolyData.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -42,7 +42,6 @@
 #include "vtkMultiProcessController.h"
 
 vtkStandardNewMacro(vtkRedistributePolyData);
-vtkCxxRevisionMacro(vtkRedistributePolyData, "$Revision: 1.27 $");
 
 vtkCxxSetObjectMacro(vtkRedistributePolyData, Controller, 
                      vtkMultiProcessController);
