@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkTimeToTextConvertor.h,v $
+  Module:    vtkTimeToTextConvertor.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -29,7 +29,7 @@ class VTK_EXPORT vtkTimeToTextConvertor : public vtkTableAlgorithm
 {
 public:
   static vtkTimeToTextConvertor* New();
-  vtkTypeRevisionMacro(vtkTimeToTextConvertor, vtkTableAlgorithm);
+  vtkTypeMacro(vtkTimeToTextConvertor, vtkTableAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
