@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkUpdateSuppressorPipeline.cxx,v $
+  Module:    vtkUpdateSuppressorPipeline.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -20,7 +20,6 @@
 
 #include "vtkProcessModule.h"
 
-vtkCxxRevisionMacro(vtkUpdateSuppressorPipeline, "$Revision: 1.10 $");
 vtkStandardNewMacro(vtkUpdateSuppressorPipeline);
 
 
