@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: ServersServerManagerPrintSelf.cxx,v $
+  Module:    ServersServerManagerPrintSelf.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -67,7 +67,6 @@
 #include "vtkSMMaterialLoaderProxy.h"
 #include "vtkSMNullProxy.h"
 #include "vtkSMNumberOfGroupsDomain.h"
-#include "vtkSMNumberOfPartsDomain.h"
 #include "vtkSMObject.h"
 #include "vtkSMOrderedPropertyIterator.h"
 #include "vtkSMOutputPort.h"
@@ -77,7 +76,6 @@
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMPropertyLink.h"
 #include "vtkSMPropertyModificationUndoElement.h"
-#include "vtkSMPropertyStatusManager.h"
 #include "vtkSMProxyGroupDomain.h"
 #include "vtkSMProxy.h"
 #include "vtkSMProxyDefinitionIterator.h"
@@ -88,7 +86,6 @@
 #include "vtkSMProxyProperty.h"
 #include "vtkSMProxyRegisterUndoElement.h"
 #include "vtkSMProxyUnRegisterUndoElement.h"
-#include "vtkSMPVDWriterProxy.h"
 #include "vtkSMPVLookupTableProxy.h"
 #include "vtkSMPWriterProxy.h"
 #include "vtkSMRampKeyFrameProxy.h"
@@ -108,7 +105,6 @@
 #include "vtkSMStringListRangeDomain.h"
 #include "vtkSMStringVectorProperty.h"
 #include "vtkSMSubPropertyIterator.h"
-#include "vtkSMSummaryHelperProxy.h"
 #include "vtkSMTesting.h"
 #include "vtkSMTimeRangeInformationHelper.h"
 #include "vtkSMTimeStepsInformationHelper.h"
@@ -121,8 +117,6 @@
 #include "vtkSMVectorProperty.h"
 #include "vtkSMWidgetRepresentationProxy.h"
 #include "vtkSMWriterProxy.h"
-#include "vtkSMXDMFInformationHelper.h"
-#include "vtkSMXDMFPropertyDomain.h"
 #include "vtkSMXMLParser.h"
 #include "vtkSMXMLPVAnimationWriterProxy.h"
 
@@ -208,7 +202,6 @@ int main(int , char *[])
   c = vtkSMMaterialLoaderProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMNullProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMNumberOfGroupsDomain::New(); c->Print( cout ); c->Delete();
-  c = vtkSMNumberOfPartsDomain::New(); c->Print( cout ); c->Delete();
   c = vtkSMObject::New(); c->Print( cout ); c->Delete();
   c = vtkSMOrderedPropertyIterator::New(); c->Print( cout ); c->Delete();
   c = vtkSMOutputPort::New(); c->Print( cout ); c->Delete();
@@ -218,7 +211,6 @@ int main(int , char *[])
   c = vtkSMPropertyIterator::New(); c->Print( cout ); c->Delete();
   c = vtkSMPropertyLink::New(); c->Print( cout ); c->Delete();
   c = vtkSMPropertyModificationUndoElement::New(); c->Print( cout ); c->Delete();
-  c = vtkSMPropertyStatusManager::New(); c->Print( cout ); c->Delete();
   c = vtkSMProxyGroupDomain::New(); c->Print( cout ); c->Delete();
   c = vtkSMProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMProxyDefinitionIterator::New(); c->Print( cout ); c->Delete();
@@ -229,7 +221,6 @@ int main(int , char *[])
   c = vtkSMProxyProperty::New(); c->Print( cout ); c->Delete();
   c = vtkSMProxyRegisterUndoElement::New(); c->Print( cout ); c->Delete();
   c = vtkSMProxyUnRegisterUndoElement::New(); c->Print( cout ); c->Delete();
-  c = vtkSMPVDWriterProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMPVLookupTableProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMPWriterProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMRampKeyFrameProxy::New(); c->Print( cout ); c->Delete();
@@ -249,7 +240,6 @@ int main(int , char *[])
   c = vtkSMStringListRangeDomain::New(); c->Print( cout ); c->Delete();
   c = vtkSMStringVectorProperty::New(); c->Print( cout ); c->Delete();
   c = vtkSMSubPropertyIterator::New(); c->Print( cout ); c->Delete();
-  c = vtkSMSummaryHelperProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMTesting::New(); c->Print( cout ); c->Delete();
   c = vtkSMTimeRangeInformationHelper::New(); c->Print( cout ); c->Delete();
   c = vtkSMTimeStepsInformationHelper::New(); c->Print( cout ); c->Delete();
@@ -262,8 +252,6 @@ int main(int , char *[])
   c = vtkSMVectorProperty::New(); c->Print( cout ); c->Delete();
   c = vtkSMWidgetRepresentationProxy::New(); c->Print( cout ); c->Delete();
   c = vtkSMWriterProxy::New(); c->Print( cout ); c->Delete();
-  c = vtkSMXDMFInformationHelper::New(); c->Print( cout ); c->Delete();
-  c = vtkSMXDMFPropertyDomain::New(); c->Print( cout ); c->Delete();
   c = vtkSMXMLParser::New(); c->Print( cout ); c->Delete();
   c = vtkSMXMLPVAnimationWriterProxy::New(); c->Print( cout ); c->Delete();
 
