@@ -19,9 +19,9 @@ if view.GetProperty("RemoteRenderThreshold"):
 repr = servermanager.CreateRepresentation(sphere, view);
 repr.Input.append(sphere)
 
+view.UseOffscreenRenderingForScreenshotsOff()
 view.ResetCamera()
 view.StillRender()
 
 if not SMPythonTesting.DoRegressionTesting(view.SMProxy):
-    # This will lead to VTK object leaks.
-    sys.exit(1)
+    raise SMPythonTesting.TestError, 'Test failed.'
