@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkDummyProcessModuleHelper.cxx,v $
+  Module:    vtkDummyProcessModuleHelper.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -15,7 +15,6 @@
 #include "vtkDummyProcessModuleHelper.h"
 
 #include "vtkObjectFactory.h"
-vtkCxxRevisionMacro(vtkDummyProcessModuleHelper, "$Revision: 1.1 $");
 vtkStandardNewMacro(vtkDummyProcessModuleHelper);
 
 //----------------------------------------------------------------------------
