@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMApplication.cxx,v $
+  Module:    vtkSMApplication.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -15,11 +15,17 @@
 #include "vtkSMApplication.h"
 
 #include "vtkPVConfig.h" // To get PARAVIEW_USE_*
+#include "vtkPVEnvironmentInformation.h"
 
 #include "vtkClientServerStream.h"
 #include "vtkDirectory.h"
 #include "vtkObjectFactory.h"
+#include "vtkProcessModule.h"
+#include "vtkSmartPointer.h"
+#include "vtkSMProxy.h"
+#include "vtkSMPluginManager.h"
 #include "vtkSMProxyManager.h"
+#include "vtkSMStringVectorProperty.h"
 #include "vtkSMXMLParser.h"
 
 #include "vtkProcessModule.h"
@@ -32,6 +38,10 @@
 
 #include "vtkStdString.h"
 
+#ifdef VTK_USE_QVTK
+# include "vtkQtInitialization.h"
+#endif
+
 struct vtkSMApplicationInternals
 {
   struct ConfFile
@@ -41,21 +51,34 @@ struct vtkSMApplicationInternals
   };
 
   vtkstd::vector<ConfFile> Files;
+  vtkSmartPointer<vtkPVEnvironmentInformation> EnvInfo ;
+  vtkSmartPointer<vtkSMPluginManager> PluginManager ;
 };
 
 vtkStandardNewMacro(vtkSMApplication);
-vtkCxxRevisionMacro(vtkSMApplication, "$Revision: 1.19 $");
 
 //---------------------------------------------------------------------------
 vtkSMApplication::vtkSMApplication()
 {
   this->Internals = new vtkSMApplicationInternals;
+  this->Internals->EnvInfo = vtkSmartPointer<vtkPVEnvironmentInformation>::New();
+#ifdef VTK_USE_QVTK
+  this->QtInitializer = NULL;
+#endif
 }
 
 //---------------------------------------------------------------------------
 vtkSMApplication::~vtkSMApplication()
 {
   delete this->Internals;
+
+#ifdef VTK_USE_QVTK
+  if (this->QtInitializer)
+    {
+    this->QtInitializer->Delete();
+    this->QtInitializer = 0;
+    }
+#endif
 }
 
 extern "C" { void vtkPVServerManager_Initialize(vtkClientServerInterpreter*); }
@@ -72,7 +95,7 @@ void vtkSMApplication::AddConfigurationFile(const char* fname, const char* dir)
 //---------------------------------------------------------------------------
 unsigned int vtkSMApplication::GetNumberOfConfigurationFiles()
 {
-  return this->Internals->Files.size();
+  return static_cast<unsigned int>(this->Internals->Files.size());
 }
 
 //---------------------------------------------------------------------------
@@ -94,16 +117,32 @@ void vtkSMApplication::ParseConfigurationFiles()
     }
 }
 
+//---------------------------------------------------------------------------
+void vtkSMApplication::EnsureQApplicationIsInitialized()
+{
+#ifdef VTK_USE_QVTK
+  if (!this->QtInitializer)
+    {
+    this->QtInitializer  = vtkQtInitialization::New();
+    }
+#endif
+}
+
 //---------------------------------------------------------------------------
 void vtkSMApplication::Initialize()
 {
+
+#ifndef PARAVIEW_MINIMAL_BUILD
   vtkPVServerManager_Initialize(
     vtkProcessModule::GetProcessModule()->GetInterpreter());
+#endif
 
   vtkSMProxyManager* proxyM = vtkSMProxyManager::New();
   this->SetProxyManager(proxyM);
   this->SetApplication(this);
 
+  this->Internals->PluginManager = vtkSmartPointer<vtkSMPluginManager>::New();
+  
   // Load the generated modules
 #include "vtkParaViewIncludeModulesToSMApplication.h"
 
@@ -192,6 +231,53 @@ void vtkSMApplication::Finalize()
   //this->GetProcessModule()->FinalizeInterpreter();
   this->SetProxyManager(0);
 
+#ifdef VTK_USE_QVTK
+  if (this->QtInitializer)
+    {
+    this->QtInitializer->Delete();
+    this->QtInitializer = 0;
+    }
+#endif
+}
+
+//---------------------------------------------------------------------------
+const char* vtkSMApplication::GetSettingsRoot(
+  vtkIdType connectionId)
+{
+  vtkSMProxyManager* pxm = this->GetProxyManager();
+  if (!pxm)
+    {
+    vtkErrorMacro("No global proxy manager defined. Can not parse file");
+    return 0;
+    }
+  vtkSMProxy* helper = pxm->NewProxy("misc", "EnvironmentInformationHelper");
+  helper->SetConnectionID(connectionId);
+  helper->SetServers(vtkProcessModule::DATA_SERVER_ROOT);
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    helper->GetProperty("Variable"));
+  if(!svp)
+    {
+    helper->UnRegister(NULL);
+    return NULL;
+    }  
+#ifdef _WIN32
+  svp->SetElement(0,"APPDATA");
+#else
+  svp->SetElement(0,"HOME");
+#endif
+  helper->UpdateVTKObjects();
+  
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  pm->GatherInformation(helper->GetConnectionID(),
+    vtkProcessModule::DATA_SERVER, this->Internals->EnvInfo, helper->GetID());
+  helper->UnRegister(NULL);
+  return this->Internals->EnvInfo->GetVariable();
+}
+
+//---------------------------------------------------------------------------
+vtkSMPluginManager* vtkSMApplication::GetPluginManager()
+{
+  return this->Internals->PluginManager;
 }
 
 //---------------------------------------------------------------------------
