@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMBlockDeliveryRepresentationProxy.h,v $
+  Module:    vtkSMBlockDeliveryRepresentationProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -31,7 +31,7 @@ class VTK_EXPORT vtkSMBlockDeliveryRepresentationProxy :
 {
 public:
   static vtkSMBlockDeliveryRepresentationProxy* New();
-  vtkTypeRevisionMacro(vtkSMBlockDeliveryRepresentationProxy, 
+  vtkTypeMacro(vtkSMBlockDeliveryRepresentationProxy,
     vtkSMDataRepresentationProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
@@ -44,23 +44,6 @@ public:
   // Indicates if the block is available on the client.
   virtual bool IsAvailable(vtkIdType blockid);
 
-  // Description:
-  // Get/Set the current block number.
-  // This simply passes it to the block filter. Overridden to ensure that the
-  void SetFieldType(int);
-
-  // Description:
-  // Get/Set the process id to pull data from.
-  // This simply passes it to the block filter. ProcessID is used only if
-  // FieldType is Field data.
-  void SetProcessID(int);
-
-  // Description:
-  // Get/Set the composite dataset index. 
-  // This simply passes it to the block filter. 
-  void SetCompositeDataSetIndex(int idx);
-  void SetCompositeDataSetIndex() { this->SetCompositeDataSetIndex(0); }
-
   // Description:
   // Set the cache size as the maximum number of blocks to cache at a given
   // time. When cache size exceeds this number, the least-recently-accessed
@@ -84,6 +67,11 @@ public:
   // cleaned.
   void CleanCache();
 
+  // Description:
+  // Returns the number of blocks that are needed to fetch the entire input
+  // dataset given the current block size.
+  vtkIdType GetNumberOfRequiredBlocks();
+
 //BTX
 protected:
   vtkSMBlockDeliveryRepresentationProxy();
@@ -105,7 +93,8 @@ protected:
   // Ensures that the block of data is available on the client.
   void Fetch(vtkIdType block);
 
-  vtkSMSourceProxy* BlockFilter;
+  vtkSMSourceProxy* PreProcessor;
+  vtkSMSourceProxy* Streamer;
   vtkSMSourceProxy* Reduction;
 
   bool CacheDirty;
