@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMCameraProxy.cxx,v $
+  Module:    vtkSMCameraProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -20,7 +20,6 @@
 #include "vtkCamera.h"
 
 vtkStandardNewMacro(vtkSMCameraProxy);
-vtkCxxRevisionMacro(vtkSMCameraProxy, "$Revision: 1.4 $");
 //-----------------------------------------------------------------------------
 vtkSMCameraProxy::vtkSMCameraProxy()
 {
@@ -67,6 +66,10 @@ void vtkSMCameraProxy::UpdatePropertyInformation()
   dvp = vtkSMDoubleVectorProperty::SafeDownCast(
     this->GetProperty("CameraParallelScaleInfo"));
   dvp->SetElement(0, camera->GetParallelScale());
+
+  dvp = vtkSMDoubleVectorProperty::SafeDownCast(
+    this->GetProperty("CameraViewAngleInfo"));
+  dvp->SetElement(0, camera->GetViewAngle());
 }
 
 //-----------------------------------------------------------------------------
