@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMClientDeliveryStrategyProxy.cxx,v $
+  Module:    vtkSMClientDeliveryStrategyProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -24,12 +24,13 @@
 #include "vtkSMSourceProxy.h"
 
 vtkStandardNewMacro(vtkSMClientDeliveryStrategyProxy);
-vtkCxxRevisionMacro(vtkSMClientDeliveryStrategyProxy, "$Revision: 1.14 $");
 //----------------------------------------------------------------------------
 vtkSMClientDeliveryStrategyProxy::vtkSMClientDeliveryStrategyProxy()
 {
   this->ReductionProxy = 0;
   this->CollectProxy = 0;
+  this->PostCollectUpdateSuppressor = 0;
+  this->CollectedDataValid = false;
   this->SetEnableLOD(false);
 }
 
@@ -38,6 +39,7 @@ vtkSMClientDeliveryStrategyProxy::~vtkSMClientDeliveryStrategyProxy()
 {
   this->CollectProxy = 0;
   this->ReductionProxy = 0;
+  this->PostCollectUpdateSuppressor = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -75,6 +77,7 @@ void vtkSMClientDeliveryStrategyProxy::SetPostGatherHelper(
 
     pm->SendStream(this->ConnectionID, this->ReductionProxy->GetServers(),
       stream);
+    this->CollectedDataValid = false;
     }
 }
 
@@ -86,6 +89,7 @@ void vtkSMClientDeliveryStrategyProxy::SetPostGatherHelper(vtkSMProxy* helper)
   pp->RemoveAllProxies();
   pp->AddProxy(helper);
   this->ReductionProxy->UpdateVTKObjects();
+  this->CollectedDataValid = false;
 }
 
 //----------------------------------------------------------------------------
@@ -96,11 +100,14 @@ void vtkSMClientDeliveryStrategyProxy::SetPreGatherHelper(vtkSMProxy* helper)
   pp->RemoveAllProxies();
   pp->AddProxy(helper);
   this->ReductionProxy->UpdateVTKObjects();
+  this->CollectedDataValid = false;
 }
+
 //----------------------------------------------------------------------------
 void vtkSMClientDeliveryStrategyProxy::BeginCreateVTKObjects()
 {
   this->Superclass::BeginCreateVTKObjects();
+  this->UpdateSuppressor->SetServers(this->Servers);
 
   this->CollectProxy = vtkSMSourceProxy::SafeDownCast(
     this->GetSubProxy("Collect"));
@@ -111,30 +118,26 @@ void vtkSMClientDeliveryStrategyProxy::BeginCreateVTKObjects()
     this->Servers | vtkProcessModule::CLIENT);
   this->ReductionProxy->SetServers(this->Servers);
 
-  this->UpdateSuppressor->SetServers(this->Servers|vtkProcessModule::CLIENT);
+  this->PostCollectUpdateSuppressor = vtkSMSourceProxy::SafeDownCast(
+    this->GetSubProxy("PostCollectUpdateSuppressor"));
+  this->PostCollectUpdateSuppressor->SetServers(
+    this->Servers|vtkProcessModule::CLIENT);
 }
 
 //----------------------------------------------------------------------------
 void vtkSMClientDeliveryStrategyProxy::CreatePipeline(vtkSMSourceProxy* input,
   int outputport)
 {
+  this->Superclass::CreatePipeline(input, outputport);
+
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
   vtkClientServerStream stream;
 
-  this->Connect(input, this->ReductionProxy, "Input", outputport);
+  this->Connect(this->Superclass::GetOutput(), this->ReductionProxy);
   this->Connect(this->ReductionProxy, this->CollectProxy);
+  this->Connect(this->CollectProxy, this->PostCollectUpdateSuppressor);
 
-  // Connects the CollectProxy to UpdateSuppressor.
-  this->Superclass::CreatePipeline(this->CollectProxy, 0);
-
-  // Now we need to set up some default parameters on these filters.
-  stream << vtkClientServerStream::Invoke
-         << this->CollectProxy->GetID() 
-         << "SetProcessModuleConnection"
-         << pm->GetConnectionClientServerID(this->GetConnectionID())
-         << vtkClientServerStream::End;
-  pm->SendStream(this->ConnectionID, this->CollectProxy->GetServers(), stream);
-
+  // Now set the MPI controller.
   stream  << vtkClientServerStream::Invoke
           << pm->GetProcessModuleID() << "GetController"
           << vtkClientServerStream::End;
@@ -148,52 +151,56 @@ void vtkSMClientDeliveryStrategyProxy::CreatePipeline(vtkSMSourceProxy* input,
 //----------------------------------------------------------------------------
 void vtkSMClientDeliveryStrategyProxy::UpdatePipeline()
 {
-  this->UpdatePipelineInternal(this->CollectProxy, 
-                               this->UpdateSuppressor);
+  if (this->vtkSMClientDeliveryStrategyProxy::GetDataValid())
+    {
+    return;
+    }
+
   this->Superclass::UpdatePipeline();
+  this->UpdatePipelineInternal(this->CollectProxy, 
+                               this->PostCollectUpdateSuppressor);
+
+  this->PostCollectUpdateSuppressor->InvokeCommand("ForceUpdate");
+  this->PostCollectUpdateSuppressor->UpdatePipeline();
+  this->CollectedDataValid = true;
 }
 
 //----------------------------------------------------------------------------
 void vtkSMClientDeliveryStrategyProxy::UpdatePipelineInternal(
   vtkSMSourceProxy* collect, vtkSMSourceProxy* )
 {
-  vtkSMSourceProxy* input = this->Input;
-  if (input)
+  vtkPVDataInformation* inputInfo = this->GetRepresentedDataInformation();
+  this->ReductionProxy->UpdatePipeline();
+  vtkPVDataInformation* outputInfo = this->ReductionProxy->GetDataInformation(0);
+  int dataType = outputInfo->GetDataSetType();
+  int cDataType = outputInfo->GetCompositeDataSetType();
+  if (cDataType > 0)
     {
-    input->UpdatePipeline();
-    vtkPVDataInformation* inputInfo = input->GetDataInformation(this->OutputPort);
-    this->ReductionProxy->UpdatePipeline();
-    vtkPVDataInformation* outputInfo = this->ReductionProxy->GetDataInformation(0);
-    int dataType = outputInfo->GetDataSetType();
-    int cDataType = outputInfo->GetCompositeDataSetType();
-    if (cDataType > 0)
-      {
-      dataType = cDataType;
-      }
-
-    vtkClientServerStream stream;
-
-    stream << vtkClientServerStream::Invoke
-           << collect->GetID() << "SetOutputDataType" << dataType
-           << vtkClientServerStream::End;
-    if (dataType == VTK_STRUCTURED_POINTS ||
-        dataType == VTK_STRUCTURED_GRID   ||
-        dataType == VTK_RECTILINEAR_GRID  ||
-        dataType == VTK_IMAGE_DATA)
-      {
-      const int* extent = inputInfo->GetExtent();
-      stream << vtkClientServerStream::Invoke
-             << collect->GetID() 
-             << "SetWholeExtent" 
-             << vtkClientServerStream::InsertArray(extent, 6)
-             << vtkClientServerStream::End;
-      }
-
-    vtkProcessModule::GetProcessModule()->SendStream(
-      this->ConnectionID,
-      collect->GetServers(), 
-      stream);
+    dataType = cDataType;
     }
+
+  vtkClientServerStream stream;
+
+  stream << vtkClientServerStream::Invoke
+         << collect->GetID() << "SetOutputDataType" << dataType
+         << vtkClientServerStream::End;
+  if (dataType == VTK_STRUCTURED_POINTS ||
+      dataType == VTK_STRUCTURED_GRID   ||
+      dataType == VTK_RECTILINEAR_GRID  ||
+      dataType == VTK_IMAGE_DATA)
+    {
+    const int* extent = inputInfo->GetExtent();
+    stream << vtkClientServerStream::Invoke
+           << collect->GetID() 
+           << "SetWholeExtent" 
+           << vtkClientServerStream::InsertArray(extent, 6)
+           << vtkClientServerStream::End;
+    }
+
+  vtkProcessModule::GetProcessModule()->SendStream(
+    this->ConnectionID,
+    collect->GetServers(), 
+    stream);
 }
 
 //----------------------------------------------------------------------------
