@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMDataLabelRepresentationProxy.cxx,v $
+  Module:    vtkSMDataLabelRepresentationProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -33,7 +33,6 @@
 #include "vtkSMIntVectorProperty.h"
 
 vtkStandardNewMacro(vtkSMDataLabelRepresentationProxy);
-vtkCxxRevisionMacro(vtkSMDataLabelRepresentationProxy, "Revision: 1.1$");
 
 //-----------------------------------------------------------------------------
 vtkSMDataLabelRepresentationProxy::vtkSMDataLabelRepresentationProxy()
@@ -304,21 +303,6 @@ void vtkSMDataLabelRepresentationProxy::SetupDefaults()
   vtkClientServerStream stream;
   vtkSMIntVectorProperty* ivp;
 
-  // Collect filter needs the socket controller use to communicate between
-  // data-server root and the client.
-  stream  << vtkClientServerStream::Invoke
-    << pm->GetProcessModuleID() 
-    << "GetSocketController"
-    << pm->GetConnectionClientServerID(this->ConnectionID)
-    << vtkClientServerStream::End;
-  stream  << vtkClientServerStream::Invoke
-    << this->CollectProxy->GetID()
-    << "SetSocketController"
-    << vtkClientServerStream::LastResult
-    << vtkClientServerStream::End;
-  pm->SendStream(this->ConnectionID, 
-    vtkProcessModule::CLIENT_AND_SERVERS, stream);
-
   // Collect filter needs the MPIMToNSocketConnection to communicate between
   // render server and data server nodes.
   stream  << vtkClientServerStream::Invoke
@@ -356,61 +340,6 @@ void vtkSMDataLabelRepresentationProxy::SetupDefaults()
     }
   ivp->SetElement(0, 2); // Clone mode.
   this->CollectProxy->UpdateVTKObjects();
-
-  ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->TextPropertyProxy->GetProperty("FontSize"));
-  if (!ivp)
-    {
-    vtkErrorMacro("Failed to find property FontSize on TextPropertyProxy.");
-    return;
-    }
-  ivp->SetElement(0, 18);
-
-  ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->TextPropertyProxy->GetProperty("Justification"));
-  if (!ivp)
-  {
-    vtkErrorMacro("Failed to find property Justification on CellTextPropertyProxy.");
-    return;
-  }
-  ivp->SetElement(0, 1); //Center justified
-  this->TextPropertyProxy->UpdateVTKObjects();
-
-  ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->GetProperty("CellLabelVisibility"));
-  ivp->SetElement(0, 0);    
-  this->UpdateProperty("CellLabelVisibility");
-
-  // Set default Cell Text property
-  ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->CellTextPropertyProxy->GetProperty("FontSize"));
-  if (!ivp)
-    {
-    vtkErrorMacro("Failed to find property FontSize on CellTextPropertyProxy.");
-    return;
-    }
-  ivp->SetElement(0, 24);
-
-  ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->CellTextPropertyProxy->GetProperty("Justification"));
-  if (!ivp)
-  {
-    vtkErrorMacro("Failed to find property Justification on CellTextPropertyProxy.");
-    return;
-  }
-  ivp->SetElement(0, 1); //Center justified
-
-  vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
-    this->CellTextPropertyProxy->GetProperty("Color"));
-  if (!dvp)
-    {
-    vtkErrorMacro("Failed to find property Color on CellTextPropertyProxy.");
-    return;
-    }
-  dvp->SetElements3(0.0, 1.0, 0.0);
-
-  this->CellTextPropertyProxy->UpdateVTKObjects();
-
 }
 
 //-----------------------------------------------------------------------------
@@ -430,25 +359,6 @@ void vtkSMDataLabelRepresentationProxy::Update(
     }
 
   // check if we should UseCache
-
-  if (this->ViewInformation)
-  {
-  if (this->ViewInformation->Has(vtkSMViewProxy::USE_CACHE()))
-    {
-    if(this->ViewInformation->Get(vtkSMViewProxy::USE_CACHE())>0)
-      {
-      if (this->ViewInformation->Has(vtkSMViewProxy::CACHE_TIME()))
-        {
-        vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
-          this->UpdateSuppressorProxy->GetProperty("CacheUpdate"));
-        dvp->SetElement(0, this->ViewInformation->Get(vtkSMViewProxy::CACHE_TIME()));
-        this->UpdateSuppressorProxy->UpdateProperty("CacheUpdate", 1);
-        return;
-        }
-      }
-    }
-  }
-
   if (this->GeometryIsValid || !this->UpdateSuppressorProxy)
     {
     return;
@@ -499,13 +409,13 @@ void vtkSMDataLabelRepresentationProxy::InvalidateGeometryInternal(int useCache)
   if (!useCache)
     {
     this->GeometryIsValid = 0;
-    if (this->UpdateSuppressorProxy)
-      {
-      vtkSMProperty *p = 
-        this->UpdateSuppressorProxy->GetProperty("RemoveAllCaches");
-      p->Modified();
-      this->UpdateSuppressorProxy->UpdateVTKObjects();
-      }
+    //if (this->UpdateSuppressorProxy)
+    //  {
+    //  vtkSMProperty *p = 
+    //    this->UpdateSuppressorProxy->GetProperty("RemoveAllCaches");
+    //  p->Modified();
+    //  this->UpdateSuppressorProxy->UpdateVTKObjects();
+    //  }
     }
 }
 
