@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMDataLabelRepresentationProxy.h,v $
+  Module:    vtkSMDataLabelRepresentationProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -28,7 +28,7 @@ class VTK_EXPORT vtkSMDataLabelRepresentationProxy : public vtkSMDataRepresentat
 {
 public:
   static vtkSMDataLabelRepresentationProxy* New();
-  vtkTypeRevisionMacro(vtkSMDataLabelRepresentationProxy, vtkSMDataRepresentationProxy);
+  vtkTypeMacro(vtkSMDataLabelRepresentationProxy, vtkSMDataRepresentationProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
