@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMDoubleVectorProperty.h,v $
+  Module:    vtkSMDoubleVectorProperty.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -32,7 +32,7 @@ class VTK_EXPORT vtkSMDoubleVectorProperty : public vtkSMVectorProperty
 {
 public:
   static vtkSMDoubleVectorProperty* New();
-  vtkTypeRevisionMacro(vtkSMDoubleVectorProperty, vtkSMVectorProperty);
+  vtkTypeMacro(vtkSMDoubleVectorProperty, vtkSMVectorProperty);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -57,6 +57,7 @@ public:
   // or one or more of the values is not in all domains.
   // Returns 1 otherwise.
   int SetElements(const double* values);
+  int SetElements(const double* values, unsigned int numValues);
   double *GetElements();
 
   // Description:
@@ -124,6 +125,15 @@ public:
   vtkGetMacro(ArgumentIsArray, int);
   vtkSetMacro(ArgumentIsArray, int);
 
+
+  // Description:
+  // Get/Set the precision to use while saving the state for this property. If
+  // 0 (or negative), then default ostream precision will be used. Precision can
+  // be defined in ServerManager configuration XML using the attribute
+  // "precision".
+  vtkSetMacro(Precision, int);
+  vtkGetMacro(Precision, int);
+
   // Description: 
   // Copy all property values.
   virtual void Copy(vtkSMProperty* src);
@@ -155,12 +165,13 @@ protected:
 
   vtkSMDoubleVectorPropertyInternals* Internals;
 
+  int Precision;
   int ArgumentIsArray;
 
   // Description:
   // Updates state from an XML element. Returns 0 on failure.
   virtual int LoadState(vtkPVXMLElement* element, 
-    vtkSMStateLoaderBase* loader, int loadLastPushedValues=0);
+    vtkSMProxyLocator* loader, int loadLastPushedValues=0);
 
   virtual void ChildSaveState(vtkPVXMLElement* parent, int saveLastPushedValues);
 
@@ -170,17 +181,10 @@ protected:
   // before a domain check is performed.
   virtual void SetNumberOfUncheckedElements(unsigned int num);
 
-  // Description:
-  // If SetNumberCommand is set, it is called before Command
-  // with the number of arguments as the parameter.
-  vtkSetStringMacro(SetNumberCommand);
-  vtkGetStringMacro(SetNumberCommand);
-
   // Subclass may override this if ResetToDefault can reset to default
   // value specified in the configuration file.
   virtual void ResetToDefaultInternal();
 
-  char* SetNumberCommand;
 
   bool Initialized;
 
