@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMEnumerationDomain.cxx,v $
+  Module:    vtkSMEnumerationDomain.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -24,7 +24,6 @@
 #include "vtkStdString.h"
 
 vtkStandardNewMacro(vtkSMEnumerationDomain);
-vtkCxxRevisionMacro(vtkSMEnumerationDomain, "$Revision: 1.11 $");
 
 struct vtkSMEnumerationDomainInternals
 {
@@ -54,21 +53,42 @@ vtkSMEnumerationDomain::~vtkSMEnumerationDomain()
 //---------------------------------------------------------------------------
 unsigned int vtkSMEnumerationDomain::GetNumberOfEntries()
 {
-  return this->EInternals->Entries.size();
+  return static_cast<unsigned int>(this->EInternals->Entries.size());
 }
 
 //---------------------------------------------------------------------------
 int vtkSMEnumerationDomain::GetEntryValue(unsigned int idx)
 {
+  if (idx >= static_cast<unsigned int>(this->EInternals->Entries.size()))
+    {
+    vtkErrorMacro("Invalid idx: " << idx);
+    return 0;
+    }
   return this->EInternals->Entries[idx].Value;
 }
 
 //---------------------------------------------------------------------------
 const char* vtkSMEnumerationDomain::GetEntryText(unsigned int idx)
 {
+  if (idx >= static_cast<unsigned int>(this->EInternals->Entries.size()))
+    {
+    vtkErrorMacro("Invalid idx: " << idx);
+    return NULL;
+    }
   return this->EInternals->Entries[idx].Text.c_str();
 }
 
+//---------------------------------------------------------------------------
+const char* vtkSMEnumerationDomain::GetEntryTextForValue(int value)
+{
+  unsigned int idx = 0;
+  if (!this->IsInDomain(value, idx))
+    {
+    return NULL;
+    }
+  return this->GetEntryText(idx);
+}
+
 //---------------------------------------------------------------------------
 int vtkSMEnumerationDomain::HasEntryText(const char* text)
 {
@@ -243,6 +263,22 @@ void vtkSMEnumerationDomain::Update(vtkSMProperty* prop)
     }
 }
 
+//---------------------------------------------------------------------------
+int vtkSMEnumerationDomain::SetDefaultValues(vtkSMProperty* prop)
+{
+  vtkSMIntVectorProperty* ivp = vtkSMIntVectorProperty::SafeDownCast(prop);
+  if (ivp && this->GetNumberOfEntries() > 0)
+    {
+    unsigned int idx=0;
+    if (!this->IsInDomain(ivp->GetDefaultValue(0), idx))
+      {
+      ivp->SetElement(0, this->GetEntryValue(0));
+      return 1;
+      }
+    }
+  return this->Superclass::SetDefaultValues(prop);
+}
+
 //---------------------------------------------------------------------------
 void vtkSMEnumerationDomain::PrintSelf(ostream& os, vtkIndent indent)
 {
