@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMFieldDataDomain.cxx,v $
+  Module:    vtkSMFieldDataDomain.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -19,17 +19,22 @@
 #include "vtkPVArrayInformation.h"
 #include "vtkPVDataInformation.h"
 #include "vtkPVDataSetAttributesInformation.h"
+#include "vtkPVXMLElement.h"
+#include "vtkSmartPointer.h"
 #include "vtkSMDomainIterator.h"
 #include "vtkSMInputArrayDomain.h"
 #include "vtkSMInputProperty.h"
+#include "vtkSMIntVectorProperty.h"
 #include "vtkSMSourceProxy.h"
 
 vtkStandardNewMacro(vtkSMFieldDataDomain);
-vtkCxxRevisionMacro(vtkSMFieldDataDomain, "$Revision: 1.8 $");
 
 //---------------------------------------------------------------------------
 vtkSMFieldDataDomain::vtkSMFieldDataDomain()
 {
+  this->EnableFieldDataSelection = false;
+  this->DisableUpdateDomainEntries = false;
+  this->DefaultValue = -1;
 }
 
 //---------------------------------------------------------------------------
@@ -47,7 +52,7 @@ int vtkSMFieldDataDomain::CheckForArray(
   int num = info->GetNumberOfArrays();
   for (int idx = 0; idx < num; ++idx)
     {
-    if ( iad->IsFieldValid(sp, outputport, info->GetArrayInformation(idx), 1) )
+    if (iad == 0 || iad->IsFieldValid(sp, outputport, info->GetArrayInformation(idx), 1) )
       {
       return 1;
       }
@@ -69,31 +74,72 @@ void vtkSMFieldDataDomain::Update(vtkSMSourceProxy* sp,
     return;
     }
 
-  if (this->CheckForArray(sp, outputport, info->GetPointDataInformation(), iad))
+  bool has_pd = 0 !=
+    this->CheckForArray(sp, outputport, info->GetPointDataInformation(), iad);
+  bool has_cd = 0 !=
+    this->CheckForArray(sp, outputport, info->GetCellDataInformation(), iad);
+  bool has_vd = 0 !=
+    this->CheckForArray(sp, outputport, info->GetVertexDataInformation(), iad);
+  bool has_ed = 0 !=
+    this->CheckForArray(sp, outputport, info->GetEdgeDataInformation(), iad);
+  bool has_rd = 0 !=
+    this->CheckForArray(sp, outputport, info->GetRowDataInformation(), iad);
+
+  if (this->DisableUpdateDomainEntries || has_pd)
     {
     this->AddEntry("Point Data", vtkDataObject::FIELD_ASSOCIATION_POINTS);
     }
-
-  if (this->CheckForArray(sp, outputport, info->GetCellDataInformation(), iad))
+  if (this->DisableUpdateDomainEntries || has_cd)
     {
     this->AddEntry("Cell Data",  vtkDataObject::FIELD_ASSOCIATION_CELLS);
     }
 
-  if (this->CheckForArray(sp, outputport, info->GetVertexDataInformation(), iad))
+  if (this->DisableUpdateDomainEntries || has_vd)
     {
     this->AddEntry("Vertex Data", vtkDataObject::FIELD_ASSOCIATION_VERTICES);
     }
 
-  if (this->CheckForArray(sp, outputport, info->GetEdgeDataInformation(), iad))
+  if (this->DisableUpdateDomainEntries || has_ed)
     {
     this->AddEntry("Edge Data", vtkDataObject::FIELD_ASSOCIATION_EDGES);
     }
 
-  if (this->CheckForArray(sp, outputport, info->GetRowDataInformation(), iad))
+  if (this->DisableUpdateDomainEntries || has_rd)
     {
     this->AddEntry("Row Data", vtkDataObject::FIELD_ASSOCIATION_ROWS);
     }
 
+  if (this->EnableFieldDataSelection)
+    {
+    this->AddEntry("Field Data", vtkDataObject::FIELD_ASSOCIATION_NONE);
+    }
+
+  this->DefaultValue = -1;
+  if (has_pd)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_POINTS;
+    }
+  else if (has_cd)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_CELLS;
+    }
+  else if (has_vd)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_VERTICES;
+    }
+  else if (has_ed)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_EDGES;
+    }
+  else if (has_rd)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_ROWS;
+    }
+  else if (this->EnableFieldDataSelection)
+    {
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_NONE;
+    }
+
   this->InvokeModified();
 }
 
@@ -102,7 +148,8 @@ void vtkSMFieldDataDomain::Update(vtkSMProxyProperty* pp,
                                   vtkSMSourceProxy* sp,
                                   int outputport)
 {
-  vtkSMDomainIterator* di = pp->NewDomainIterator();
+  vtkSmartPointer<vtkSMDomainIterator> di;
+  di.TakeReference(pp->NewDomainIterator());
   di->Begin();
   while (!di->IsAtEnd())
     {
@@ -111,11 +158,13 @@ void vtkSMFieldDataDomain::Update(vtkSMProxyProperty* pp,
     if (iad)
       {
       this->Update(sp, iad, outputport);
-      break;
+      return;
       }
     di->Next();
     }
-  di->Delete();
+
+  // No vtkSMInputArrayDomain present.
+  this->Update(sp, NULL, outputport);
 }
 
 //---------------------------------------------------------------------------
@@ -162,6 +211,59 @@ void vtkSMFieldDataDomain::Update(vtkSMProperty*)
     }
 }
 
+//---------------------------------------------------------------------------
+int vtkSMFieldDataDomain::SetDefaultValues(vtkSMProperty* prop)
+{
+  vtkSMIntVectorProperty* ivp = vtkSMIntVectorProperty::SafeDownCast(prop);
+  if (ivp && this->DefaultValue != -1)
+    {
+    ivp->SetElement(0, this->DefaultValue);
+    return 1;
+    }
+
+  return this->Superclass::SetDefaultValues(prop);
+}
+
+
+//---------------------------------------------------------------------------
+int vtkSMFieldDataDomain::ReadXMLAttributes(
+  vtkSMProperty* prop, vtkPVXMLElement* element)
+{
+  if (!this->Superclass::ReadXMLAttributes(prop, element))
+    {
+    return 0;
+    }
+
+  int enable_field_data=0;
+  if (element->GetScalarAttribute("enable_field_data", &enable_field_data))
+    {
+    this->EnableFieldDataSelection = (enable_field_data!=0)? true : false;
+    }
+  int disable_update_domain_entries=0;
+  if (element->GetScalarAttribute("disable_update_domain_entries",
+      &disable_update_domain_entries))
+    {
+    this->DisableUpdateDomainEntries = (disable_update_domain_entries!=0)? true : false;
+    }
+
+  if (this->DisableUpdateDomainEntries)
+    {
+    // this is a traditional enumeration. Fill it up with values.
+    this->AddEntry("Point Data", vtkDataObject::FIELD_ASSOCIATION_POINTS);
+    this->AddEntry("Cell Data",  vtkDataObject::FIELD_ASSOCIATION_CELLS);
+    this->AddEntry("Vertex Data", vtkDataObject::FIELD_ASSOCIATION_VERTICES);
+    this->AddEntry("Edge Data", vtkDataObject::FIELD_ASSOCIATION_EDGES);
+    this->AddEntry("Row Data", vtkDataObject::FIELD_ASSOCIATION_ROWS);
+    if (this->EnableFieldDataSelection)
+      {
+      this->AddEntry("Field Data", vtkDataObject::FIELD_ASSOCIATION_NONE);
+      }
+    this->DefaultValue = vtkDataObject::FIELD_ASSOCIATION_POINTS;
+    }
+
+  return 1;
+}
+
 //---------------------------------------------------------------------------
 void vtkSMFieldDataDomain::PrintSelf(ostream& os, vtkIndent indent)
 {
