@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMIdTypeVectorProperty.cxx,v $
+  Module:    vtkSMIdTypeVectorProperty.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -21,7 +21,6 @@
 #include <vtkstd/vector>
 
 vtkStandardNewMacro(vtkSMIdTypeVectorProperty);
-vtkCxxRevisionMacro(vtkSMIdTypeVectorProperty, "$Revision: 1.21 $");
 
 struct vtkSMIdTypeVectorPropertyInternals
 {
@@ -85,6 +84,14 @@ void vtkSMIdTypeVectorProperty::AppendCommandToStream(
       << objectId << this->CleanCommand
       << vtkClientServerStream::End;
     }
+
+  if (this->SetNumberCommand)
+    {
+    *str << vtkClientServerStream::Invoke 
+         << objectId << this->SetNumberCommand 
+         << this->GetNumberOfElements() / this->NumberOfElementsPerCommand
+         << vtkClientServerStream::End;
+    }
   
   if (!this->RepeatCommand)
     {
@@ -164,13 +171,13 @@ void vtkSMIdTypeVectorProperty::SetNumberOfElements(unsigned int num)
 //---------------------------------------------------------------------------
 unsigned int vtkSMIdTypeVectorProperty::GetNumberOfUncheckedElements()
 {
-  return this->Internals->UncheckedValues.size();
+  return static_cast<unsigned int>(this->Internals->UncheckedValues.size());
 }
 
 //---------------------------------------------------------------------------
 unsigned int vtkSMIdTypeVectorProperty::GetNumberOfElements()
 {
-  return this->Internals->Values.size();
+  return static_cast<unsigned int>(this->Internals->Values.size());
 }
 
 //---------------------------------------------------------------------------
@@ -344,7 +351,7 @@ int vtkSMIdTypeVectorProperty::ReadXMLAttributes(vtkSMProxy* parent,
         this->Internals->UpdateLastPushedValues();
         this->Internals->UpdateDefaultValues();
         }
-      else
+      else if (!this->Initialized)
         {
         vtkErrorMacro("No default value is specified for property: "
                       << this->GetXMLName()
@@ -359,7 +366,7 @@ int vtkSMIdTypeVectorProperty::ReadXMLAttributes(vtkSMProxy* parent,
 
 //---------------------------------------------------------------------------
 int vtkSMIdTypeVectorProperty::LoadState(vtkPVXMLElement* element,
-  vtkSMStateLoaderBase* loader, int loadLastPushedValues/*=0*/)
+  vtkSMProxyLocator* loader, int loadLastPushedValues/*=0*/)
 {
   int prevImUpdate = this->ImmediateUpdate;
 
@@ -443,7 +450,7 @@ void vtkSMIdTypeVectorProperty::ChildSaveState(vtkPVXMLElement* propertyElement,
 
   if (saveLastPushedValues)
     {
-    size = this->Internals->LastPushedValues.size();
+    size = static_cast<unsigned int>(this->Internals->LastPushedValues.size());
     
     vtkPVXMLElement* element = vtkPVXMLElement::New();
     element->SetName("LastPushedValues");
