@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMInputArrayDomain.h,v $
+  Module:    vtkSMInputArrayDomain.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -47,7 +47,7 @@ class VTK_EXPORT vtkSMInputArrayDomain : public vtkSMDomain
 {
 public:
   static vtkSMInputArrayDomain* New();
-  vtkTypeRevisionMacro(vtkSMInputArrayDomain, vtkSMDomain);
+  vtkTypeMacro(vtkSMInputArrayDomain, vtkSMDomain);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -94,6 +94,7 @@ public:
     VERTEX = 3,
     EDGE = 4,
     ROW = 5,
+    NONE = 6,
     LAST_ATTRIBUTE_TYPE
   };
 //ETX
