@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMIntVectorProperty.cxx,v $
+  Module:    vtkSMIntVectorProperty.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -22,7 +22,6 @@
 #include <vtkstd/vector>
 
 vtkStandardNewMacro(vtkSMIntVectorProperty);
-vtkCxxRevisionMacro(vtkSMIntVectorProperty, "$Revision: 1.38 $");
 
 struct vtkSMIntVectorPropertyInternals
 {
@@ -87,6 +86,14 @@ void vtkSMIntVectorProperty::AppendCommandToStream(
       << objectId << this->CleanCommand
       << vtkClientServerStream::End;
     }
+
+  if (this->SetNumberCommand)
+    {
+    *str << vtkClientServerStream::Invoke 
+         << objectId << this->SetNumberCommand 
+         << this->GetNumberOfElements() / this->NumberOfElementsPerCommand
+         << vtkClientServerStream::End;
+    }
   
   if (!this->RepeatCommand)
     {
@@ -167,13 +174,13 @@ void vtkSMIntVectorProperty::SetNumberOfElements(unsigned int num)
 //---------------------------------------------------------------------------
 unsigned int vtkSMIntVectorProperty::GetNumberOfUncheckedElements()
 {
-  return this->Internals->UncheckedValues.size();
+  return static_cast<unsigned int>(this->Internals->UncheckedValues.size());
 }
 
 //---------------------------------------------------------------------------
 unsigned int vtkSMIntVectorProperty::GetNumberOfElements()
 {
-  return this->Internals->Values.size();
+  return static_cast<unsigned int>(this->Internals->Values.size());
 }
 
 //---------------------------------------------------------------------------
@@ -182,10 +189,19 @@ int vtkSMIntVectorProperty::GetElement(unsigned int idx)
   return this->Internals->Values[idx];
 }
 
+//---------------------------------------------------------------------------
+int vtkSMIntVectorProperty::GetDefaultValue(int idx)
+{
+  if (idx >= 0 && idx < (int)this->Internals->DefaultValues.size())
+    {
+    return this->Internals->DefaultValues[idx];
+    }
+  return 0;
+}
+
 //---------------------------------------------------------------------------
 int *vtkSMIntVectorProperty::GetElements()
 {
-  
   return (this->Internals->Values.size() > 0)?
     &this->Internals->Values[0] : NULL;
 }
@@ -354,7 +370,7 @@ int vtkSMIntVectorProperty::ReadXMLAttributes(vtkSMProxy* parent,
         this->Internals->UpdateLastPushedValues();
         this->Internals->UpdateDefaultValues();
         }
-      else
+      else if (!this->Initialized)
         {
         vtkErrorMacro("No default value is specified for property: "
                       << this->GetXMLName()
@@ -369,7 +385,7 @@ int vtkSMIntVectorProperty::ReadXMLAttributes(vtkSMProxy* parent,
 
 //---------------------------------------------------------------------------
 int vtkSMIntVectorProperty::LoadState(vtkPVXMLElement* element,
-  vtkSMStateLoaderBase* loader, int loadLastPushedValues/*=0*/)
+  vtkSMProxyLocator* loader, int loadLastPushedValues/*=0*/)
 {
   int prevImUpdate = this->ImmediateUpdate;
 
@@ -453,7 +469,7 @@ void vtkSMIntVectorProperty::ChildSaveState(vtkPVXMLElement* propertyElement,
 
   if (saveLastPushedValues)
     {
-    size = this->Internals->LastPushedValues.size();
+    size = static_cast<unsigned int>(this->Internals->LastPushedValues.size());
     
     vtkPVXMLElement* element = vtkPVXMLElement::New();
     element->SetName("LastPushedValues");
