@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMOutputPort.cxx,v $
+  Module:    vtkSMOutputPort.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -14,19 +14,22 @@
 =========================================================================*/
 #include "vtkSMOutputPort.h"
 
+#include "vtkAlgorithm.h"
 #include "vtkClientServerStream.h"
 #include "vtkCollection.h"
 #include "vtkCollectionIterator.h"
 #include "vtkCommand.h"
+#include "vtkDataObject.h"
 #include "vtkObjectFactory.h"
 #include "vtkProcessModule.h"
 #include "vtkPVClassNameInformation.h"
 #include "vtkPVDataInformation.h"
+#include "vtkPVTemporalDataInformation.h"
 #include "vtkPVXMLElement.h"
+#include "vtkSMSourceProxy.h"
 
 //----------------------------------------------------------------------------
 vtkStandardNewMacro(vtkSMOutputPort);
-vtkCxxRevisionMacro(vtkSMOutputPort, "$Revision: 1.8 $");
 
 
 //----------------------------------------------------------------------------
@@ -36,18 +39,22 @@ vtkSMOutputPort::vtkSMOutputPort()
 
   this->ClassNameInformation = vtkPVClassNameInformation::New();
   this->DataInformation = vtkPVDataInformation::New();
+  this->TemporalDataInformation = vtkPVTemporalDataInformation::New();
   this->ClassNameInformationValid = 0;
   this->DataInformationValid = false;
+  this->TemporalDataInformationValid = false;
   this->PortIndex = 0;
-
+  this->SourceProxy = 0;
   this->DataObjectProxy = 0;
 }
 
 //----------------------------------------------------------------------------
 vtkSMOutputPort::~vtkSMOutputPort()
 {
+  this->SetSourceProxy(0);
   this->ClassNameInformation->Delete();
   this->DataInformation->Delete();
+  this->TemporalDataInformation->Delete();
   if (this->DataObjectProxy)
     {
     this->DataObjectProxy->Delete();
@@ -107,10 +114,9 @@ vtkPVXMLElement* vtkSMOutputPort::SaveRevivalState(vtkPVXMLElement* root)
 }
 
 //----------------------------------------------------------------------------
-int vtkSMOutputPort::LoadRevivalState(vtkPVXMLElement* revivalElem, 
-  vtkSMStateLoaderBase* loader)
+int vtkSMOutputPort::LoadRevivalState(vtkPVXMLElement* revivalElem)
 {
-  if (!this->Superclass::LoadRevivalState(revivalElem, loader))
+  if (!this->Superclass::LoadRevivalState(revivalElem))
     {
     return 0;
     }
@@ -194,6 +200,16 @@ vtkPVDataInformation* vtkSMOutputPort::GetDataInformation()
   return this->DataInformation;
 }
 
+//----------------------------------------------------------------------------
+vtkPVTemporalDataInformation* vtkSMOutputPort::GetTemporalDataInformation()
+{
+  if (!this->TemporalDataInformationValid)
+    {
+    this->GatherTemporalDataInformation();
+    }
+  return this->TemporalDataInformation;
+}
+
 //----------------------------------------------------------------------------
 vtkPVClassNameInformation* vtkSMOutputPort::GetClassNameInformation()
 {
@@ -215,11 +231,11 @@ void vtkSMOutputPort::InvalidateDataInformation()
 {
   this->DataInformationValid = false;
   this->ClassNameInformationValid = false;
+  this->TemporalDataInformationValid = false;
 }
 
 //----------------------------------------------------------------------------
-// vtkPVPart used to update before gathering this information ...
-void vtkSMOutputPort::GatherDataInformation(int vtkNotUsed(doUpdate))
+void vtkSMOutputPort::GatherDataInformation()
 {
   if (this->GetID().IsNull())
     {
@@ -238,6 +254,25 @@ void vtkSMOutputPort::GatherDataInformation(int vtkNotUsed(doUpdate))
   pm->SendCleanupPendingProgress(this->ConnectionID);
 }
 
+//----------------------------------------------------------------------------
+void vtkSMOutputPort::GatherTemporalDataInformation()
+{
+  if (this->GetID().IsNull())
+    {
+    vtkErrorMacro("Part has no associated object, can not gather info.");
+    return;
+    }
+
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  pm->SendPrepareProgress(this->ConnectionID);
+  this->TemporalDataInformation->Initialize();
+  pm->GatherInformation(this->ConnectionID, this->Servers, 
+                        this->TemporalDataInformation, this->GetID());
+
+  this->TemporalDataInformationValid = true;
+  pm->SendCleanupPendingProgress(this->ConnectionID);
+}
+
 //----------------------------------------------------------------------------
 void vtkSMOutputPort::GatherClassNameInformation()
 {
@@ -248,32 +283,42 @@ void vtkSMOutputPort::GatherClassNameInformation()
     }
   vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
 
-  // Temporarily assign an id to the output object so that we
-  // can obtain it's name.
-  vtkClientServerStream stream;
-  stream << vtkClientServerStream::Invoke
-         << this->GetProducerID()
-         << "GetOutputDataObject"
-         << this->PortIndex
-         << vtkClientServerStream::End;
-  vtkClientServerID uid = pm->GetUniqueID();
-  stream << vtkClientServerStream::Assign 
-         << uid << vtkClientServerStream::LastResult
-         << vtkClientServerStream::End;
+  vtkObjectBase* cso = this->GetSourceProxy()->GetClientSideObject();
+  if (cso)
+    {
+    this->ClassNameInformation->CopyFromObject(
+      vtkAlgorithm::SafeDownCast(cso)->GetOutputDataObject(
+        this->PortIndex));
+    }
+  else
+    {
+    // Temporarily assign an id to the output object so that we
+    // can obtain it's name.
+    vtkClientServerStream stream;
+    stream << vtkClientServerStream::Invoke
+           << this->GetProducerID()
+           << "GetOutputDataObject"
+           << this->PortIndex
+           << vtkClientServerStream::End;
+    vtkClientServerID uid = pm->GetUniqueID();
+    stream << vtkClientServerStream::Assign 
+           << uid << vtkClientServerStream::LastResult
+           << vtkClientServerStream::End;
 
-  pm->SendStream(this->ConnectionID, this->Servers, stream);
+    pm->SendStream(this->ConnectionID, this->Servers, stream);
 
-  pm->GatherInformation(this->ConnectionID, 
-                        this->Servers,
-                        this->ClassNameInformation, 
-                        uid);
+    pm->GatherInformation(this->ConnectionID, 
+                          this->Servers,
+                          this->ClassNameInformation, 
+                          uid);
+
+    // Unassign the id.
+    stream << vtkClientServerStream::Delete 
+           << uid 
+           << vtkClientServerStream::End;
+    pm->SendStream(this->ConnectionID, this->Servers, stream);
+    }
   this->ClassNameInformationValid = 1;
-
-  // Unassign the id.
-  stream << vtkClientServerStream::Delete 
-         << uid 
-         << vtkClientServerStream::End;
-  pm->SendStream(this->ConnectionID, this->Servers, stream);
 }
 
 //----------------------------------------------------------------------------
@@ -306,7 +351,7 @@ void vtkSMOutputPort::InsertExtractPiecesIfNecessary()
            << this->GetProducerID() << "UpdateInformation"
            << vtkClientServerStream::End;
     pm->SendStream(this->ConnectionID, this->Servers, stream);
-    this->GatherDataInformation(0);
+    this->GatherDataInformation();
     if (this->DataInformation->GetCompositeDataClassName())
       {
       return;
@@ -369,7 +414,7 @@ void vtkSMOutputPort::InsertExtractPiecesIfNecessary()
     pm->SendStream(this->ConnectionID,
                    this->Servers, 
                    stream);
-    this->GatherDataInformation(0);
+    this->GatherDataInformation();
     if (this->DataInformation->GetCompositeDataClassName())
       {
       return;
@@ -437,7 +482,7 @@ void vtkSMOutputPort::InsertExtractPiecesIfNecessary()
     pm->SendStream(this->ConnectionID,
                    this->Servers, 
                    stream);
-    this->GatherDataInformation(0);
+    this->GatherDataInformation();
     stream << vtkClientServerStream::Invoke
            << this->GetExecutiveID() 
            << "GetMaximumNumberOfPieces"
@@ -627,8 +672,8 @@ void vtkSMOutputPort::UpdatePipelineInternal(double time,
 void vtkSMOutputPort::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os,indent);
-
   os << indent << "PortIndex: " << this->PortIndex << endl;
+  os << indent << "SourceProxy: " << this->SourceProxy << endl;
 }
 
 
