@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMPVRepresentationProxy.cxx,v $
+  Module:    vtkSMPVRepresentationProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -14,12 +14,14 @@
 =========================================================================*/
 #include "vtkSMPVRepresentationProxy.h"
 
+#include "vtkCollection.h"
 #include "vtkCommand.h"
 #include "vtkObjectFactory.h"
 #include "vtkProcessModule.h"
 #include "vtkProp3D.h"
 #include "vtkProperty.h"
 #include "vtkPVXMLElement.h"
+#include "vtkSmartPointer.h"
 #include "vtkSMEnumerationDomain.h"
 #include "vtkSMIntVectorProperty.h"
 #include "vtkSMProxyProperty.h"
@@ -65,14 +67,27 @@ public:
   // proxies.
   typedef vtkstd::set<vtkSMDataRepresentationProxy*> RepresentationProxiesSet;
   RepresentationProxiesSet UniqueRepresentationProxies;
+
+
+  vtkstd::map<vtkstd::string, int> TraditionalValues;
+  vtkInternals()
+    {
+    this->TraditionalValues["Points"] = 0;
+    this->TraditionalValues["Wireframe"] = 1;
+    this->TraditionalValues["Surface"] = 2;
+    this->TraditionalValues["Outline"] = 3;
+    this->TraditionalValues["Volume"] = 4;
+    this->TraditionalValues["Surface With Edges"] = 5;
+    this->TraditionalValues["Slice"] = 6;
+    }
 };
 
 vtkStandardNewMacro(vtkSMPVRepresentationProxy);
-vtkCxxRevisionMacro(vtkSMPVRepresentationProxy, "$Revision: 1.21 $");
 //----------------------------------------------------------------------------
 vtkSMPVRepresentationProxy::vtkSMPVRepresentationProxy()
 {
-  this->Representation = VTK_SURFACE;
+  this->Representation = vtkSMPVRepresentationProxy::SURFACE;
+  this->BackfaceRepresentation = vtkSMPVRepresentationProxy::FOLLOW_FRONTFACE;
   this->CubeAxesRepresentation = 0;
   this->CubeAxesVisibility = 0;
   this->ActiveRepresentation = 0;
@@ -98,6 +113,11 @@ void vtkSMPVRepresentationProxy::SetViewInformation(vtkInformation* info)
     (*iter)->SetViewInformation(info);
     }
 
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->SetViewInformation(info);
+    }
+
   if (this->CubeAxesRepresentation)
     {
     this->CubeAxesRepresentation->SetViewInformation(info);
@@ -110,15 +130,37 @@ bool vtkSMPVRepresentationProxy::EndCreateVTKObjects()
   vtkSMProxy* inputProxy = this->GetInputProxy();
   this->CubeAxesRepresentation = vtkSMDataRepresentationProxy::SafeDownCast(
     this->GetSubProxy("CubeAxesRepresentation"));
-  this->Connect(inputProxy, this->CubeAxesRepresentation,
-    "Input", this->OutputPort);
-  vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility", 0);
+  if (this->CubeAxesRepresentation)
+    {
+    this->Connect(inputProxy, this->CubeAxesRepresentation,
+      "Input", this->OutputPort);
+    vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility", 0);
+    }
+
+  vtkCommand* observer = this->GetObserver();
+
+  this->BackfaceSurfaceRepresentation
+    = vtkSMDataRepresentationProxy::SafeDownCast(
+                            this->GetSubProxy("BackfaceSurfaceRepresentation"));
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->Connect(inputProxy, this->BackfaceSurfaceRepresentation,
+                  "Input", this->OutputPort);
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", 0);
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "FrontfaceCulling", 1);
+
+    this->BackfaceSurfaceRepresentation->AddObserver(vtkCommand::StartEvent,
+                                                     observer);
+    this->BackfaceSurfaceRepresentation->AddObserver(vtkCommand::EndEvent,
+                                                     observer);
+    }
 
   vtkSMSurfaceRepresentationProxy* surfaceRepr = 
     vtkSMSurfaceRepresentationProxy::SafeDownCast(
       this->GetSubProxy("SurfaceRepresentation"));
 
-  vtkCommand* observer = this->GetObserver();
   vtkInternals::RepresentationProxiesSet::iterator iter;
   for (iter = this->Internals->UniqueRepresentationProxies.begin();
     iter != this->Internals->UniqueRepresentationProxies.end(); ++iter)
@@ -159,7 +201,15 @@ bool vtkSMPVRepresentationProxy::EndCreateVTKObjects()
 //----------------------------------------------------------------------------
 bool vtkSMPVRepresentationProxy::AddToView(vtkSMViewProxy* view)
 {
-  this->CubeAxesRepresentation->AddToView(view);
+  if (this->CubeAxesRepresentation)
+    {
+    this->CubeAxesRepresentation->AddToView(view);
+    }
+
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->AddToView(view);
+    }
 
   vtkInternals::RepresentationProxiesSet::iterator iter;
   for (iter = this->Internals->UniqueRepresentationProxies.begin();
@@ -181,7 +231,15 @@ bool vtkSMPVRepresentationProxy::RemoveFromView(vtkSMViewProxy* view)
     (*iter)->RemoveFromView(view);
     }
 
-  this->CubeAxesRepresentation->RemoveFromView(view);
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->RemoveFromView(view);
+    }
+
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesRepresentation->RemoveFromView(view);
+    }
   return this->Superclass::RemoveFromView(view);
 }
 
@@ -215,10 +273,98 @@ void vtkSMPVRepresentationProxy::SetRepresentation(int repr)
 
     vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation, "Visibility", 
       this->GetVisibility());
+
+    // Make sure the backface visibility and all culling, which also depends on
+    // the front face representation, is set correctly.
+    this->SetBackfaceRepresentation(this->BackfaceRepresentation);
+
     this->Modified();
     }
 }
 
+//-----------------------------------------------------------------------------
+void vtkSMPVRepresentationProxy::SetBackfaceRepresentation(int repr)
+{
+  if (this->BackfaceRepresentation != repr)
+    {
+    this->BackfaceRepresentation = repr;
+    this->Modified();
+    }
+
+  if (!this->BackfaceSurfaceRepresentation)
+    {
+    return;
+    }
+
+  if (!this->ActiveRepresentationIsSurface())
+    {
+    // Not rendering surfaces.
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", 0);
+    }
+  else if (this->BackfaceRepresentation == FOLLOW_FRONTFACE)
+    {
+    // Let the "frontface" surface also be the backface.
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", 0);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "BackfaceCulling", 0);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "FrontfaceCulling", 0);
+    }
+  else if (this->BackfaceRepresentation == CULL_BACKFACE)
+    {
+    // Turn off the back face.
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", 0);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "BackfaceCulling", 1);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "FrontfaceCulling", 0);
+    }
+  else if (this->BackfaceRepresentation == CULL_FRONTFACE)
+    {
+    // Just let SurfaceRepresentation be the backface
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", 0);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "BackfaceCulling", 0);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "FrontfaceCulling", 1);
+    }
+  else
+    {
+    // Both surfaces will be rendered.
+    vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                     "Visibility", this->GetVisibility());
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "BackfaceCulling", 1);
+    vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation,
+                                     "FrontfaceCulling", 0);
+    switch (this->BackfaceRepresentation)
+      {
+      case POINTS:
+        vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                         "Representation", VTK_POINTS);
+        break;
+      case WIREFRAME:
+        vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                         "Representation", VTK_WIREFRAME);
+        break;
+      case SURFACE_WITH_EDGES:
+        vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                         "Representation",
+                                         VTK_SURFACE_WITH_EDGES);
+        break;
+      case SURFACE:
+      default:
+        vtkSMPVRepresentationProxySetInt(this->BackfaceSurfaceRepresentation,
+                                         "Representation", VTK_SURFACE);
+        break;
+      }
+    }
+}
+
 //----------------------------------------------------------------------------
 void vtkSMPVRepresentationProxy::SetVisibility(int visible)
 {
@@ -227,9 +373,14 @@ void vtkSMPVRepresentationProxy::SetVisibility(int visible)
     vtkSMPVRepresentationProxySetInt(this->ActiveRepresentation, "Visibility", 
       visible);
     }
-  vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility",
-    visible && this->CubeAxesVisibility);
-  this->CubeAxesRepresentation->UpdateVTKObjects();
+  if (this->CubeAxesRepresentation)
+    {    
+    vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility",
+      visible && this->CubeAxesVisibility);
+    this->CubeAxesRepresentation->UpdateVTKObjects();
+    }
+  // Visibility is a bit more complicated for back face.
+  this->SetBackfaceRepresentation(this->BackfaceRepresentation);
      
   this->Superclass::SetVisibility(visible);
 }
@@ -237,10 +388,13 @@ void vtkSMPVRepresentationProxy::SetVisibility(int visible)
 //----------------------------------------------------------------------------
 void vtkSMPVRepresentationProxy::SetCubeAxesVisibility(int visible)
 {
-  this->CubeAxesVisibility = visible;
-  vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility",
-    visible && this->GetVisibility());
-  this->CubeAxesRepresentation->UpdateVTKObjects();
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesVisibility = visible;
+    vtkSMPVRepresentationProxySetInt(this->CubeAxesRepresentation, "Visibility",
+      visible && this->GetVisibility());
+    this->CubeAxesRepresentation->UpdateVTKObjects();
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -250,7 +404,15 @@ void vtkSMPVRepresentationProxy::Update(vtkSMViewProxy* view)
     {
     this->ActiveRepresentation->Update(view);
     }
-  this->CubeAxesRepresentation->Update(view);
+  if (   this->BackfaceSurfaceRepresentation
+      && this->ActiveRepresentationIsSurface() )
+    {
+    this->BackfaceSurfaceRepresentation->Update(view);
+    }
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesRepresentation->Update(view);
+    }
 
   this->Superclass::Update(view);
 }
@@ -298,7 +460,15 @@ void vtkSMPVRepresentationProxy::SetUpdateTime(double time)
     (*iter)->SetUpdateTime(time);
     }
 
-  this->CubeAxesRepresentation->SetUpdateTime(time);
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->SetUpdateTime(time);
+    }
+
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesRepresentation->SetUpdateTime(time);
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -312,7 +482,16 @@ void vtkSMPVRepresentationProxy::SetUseViewUpdateTime(bool use)
     {
     (*iter)->SetUseViewUpdateTime(use);
     }
-  this->CubeAxesRepresentation->SetUseViewUpdateTime(use);
+
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->SetUseViewUpdateTime(use);
+    }
+
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesRepresentation->SetUseViewUpdateTime(use);
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -326,7 +505,16 @@ void vtkSMPVRepresentationProxy::SetViewUpdateTime(double time)
     {
     (*iter)->SetViewUpdateTime(time);
     }
-  this->CubeAxesRepresentation->SetViewUpdateTime(time);
+
+  if (this->BackfaceSurfaceRepresentation)
+    {
+    this->BackfaceSurfaceRepresentation->SetViewUpdateTime(time);
+    }
+
+  if (this->CubeAxesRepresentation)
+    {    
+    this->CubeAxesRepresentation->SetViewUpdateTime(time);
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -337,6 +525,12 @@ void vtkSMPVRepresentationProxy::MarkDirty(vtkSMProxy* modifiedProxy)
     this->ActiveRepresentation->MarkDirty(modifiedProxy);
     }
 
+  if (   (modifiedProxy != this) && this->BackfaceSurfaceRepresentation
+      && this->ActiveRepresentationIsSurface() )
+    {
+    this->BackfaceSurfaceRepresentation->MarkDirty(modifiedProxy);
+    }
+
   this->Superclass::MarkDirty(modifiedProxy);
 }
 
@@ -408,6 +602,15 @@ bool vtkSMPVRepresentationProxy::HasVisibleProp3D(vtkProp3D* prop)
       }
     }
 
+  if (   this->GetVisibility() && this->BackfaceSurfaceRepresentation
+      && this->ActiveRepresentationIsSurface() )
+    {
+    vtkSMPropRepresentationProxy* repr
+      = vtkSMPropRepresentationProxy::SafeDownCast(
+                                           this->BackfaceSurfaceRepresentation);
+    if (repr && repr->HasVisibleProp3D(prop)) return true;
+    }
+
   return false;
 }
 
@@ -418,67 +621,150 @@ bool vtkSMPVRepresentationProxy::GetBounds(double bounds[6])
 }
 
 //----------------------------------------------------------------------------
-int vtkSMPVRepresentationProxy::ReadXMLAttributes(
+int vtkSMPVRepresentationProxy::CreateSubProxiesAndProperties(
   vtkSMProxyManager* pm, vtkPVXMLElement* element)
 {
-  if (!this->Superclass::ReadXMLAttributes(pm, element))
+  if (!this->Superclass::CreateSubProxiesAndProperties(pm, element))
     {
     return 0;
     }
 
-  // <Option representation="101" subproxy="SurfaceLICRepresentation" value="2" />
-  unsigned int numElemens = element->GetNumberOfNestedElements();
-  for (unsigned int cc=0; cc < numElemens; cc++)
+  // Locate the representation-type domain.
+  vtkSMProperty* repProp = this->GetProperty("Representation");
+  vtkSMEnumerationDomain* enumDomain = repProp?
+    vtkSMEnumerationDomain::SafeDownCast(repProp->GetDomain("enum")): 0;
+
+  // <RepresentationType subproxy="OutlineRepresentation" text="Outline"
+  //    subtype="0"/> 
+  unsigned int numElements = element->GetNumberOfNestedElements();
+  for (unsigned int cc=0; cc < numElements; cc++)
     {
     vtkPVXMLElement* child = element->GetNestedElement(cc);
-    if (child && child->GetName() && strcmp(child->GetName(), "Option") == 0)
+    if (child && child->GetName() && strcmp(child->GetName(),
+        "RepresentationType") == 0)
       {
-      int representation = -1;
-      if (!child->GetScalarAttribute("representation", &representation))
-        {
-        vtkErrorMacro("Missing required attribute 'representation'");
-        return 0;
-        }
       const char* name = child->GetAttribute("subproxy");
       vtkSMDataRepresentationProxy* subproxy = 
         vtkSMDataRepresentationProxy::SafeDownCast(this->GetSubProxy(name));
       if (!subproxy)
         {
-        vtkErrorMacro("Missing data representation subproxy '" << name << "'");
+        vtkErrorMacro("Missing data representation subproxy '"
+          << (name? name : "<null>")
+          << "' when processing RepresentationType element.");
+        return 0;
+        }
+      const char* text = child->GetAttribute("text");
+      if (!text)
+        {
+        vtkErrorMacro(
+          "Missing required 'text' attribute on RepresentationType element");
         return 0;
         }
 
-      const char* text = child->GetAttribute("text");
-      int value = -1;
-      child->GetScalarAttribute("value", &value);
+      // read optional subtype.
+      int subtype = -1;
+      child->GetScalarAttribute("subtype", &subtype);
+
+      // this ensures that when new representations are added from plugins
+      // everyone gets a unique id.
+      int representation = USER_DEFINED +
+        this->Internals->RepresentationProxies.size();
+
+      // Handle traditional values explicitly; this makes backwards
+      // compatibility easier.
+      if (this->Internals->TraditionalValues.find(text) !=
+        this->Internals->TraditionalValues.end())
+        {
+        representation = this->Internals->TraditionalValues[text];
+        }
+      if (enumDomain)
+        {
+        enumDomain->AddEntry(text, representation);
+        }
       this->Internals->RepresentationProxies[representation] =
-        vtkInternals::vtkValue(subproxy, value, text);
+        vtkInternals::vtkValue(subproxy, subtype, text);
       this->Internals->UniqueRepresentationProxies.insert(subproxy);
       }
     }
 
-  // Now update the "Representation" property domain to reflect the options
-  // available.
-  vtkSMProperty* repProp = this->GetProperty("Representation");
-  if (repProp)
+  return 1;
+}
+
+//-----------------------------------------------------------------------------
+bool vtkSMPVRepresentationProxy::ActiveRepresentationIsSurface()
+{
+  vtkSMSurfaceRepresentationProxy *activeSurfaceRep
+    = vtkSMSurfaceRepresentationProxy::SafeDownCast(this->ActiveRepresentation);
+  return (activeSurfaceRep != NULL);
+}
+
+//----------------------------------------------------------------------------
+int vtkSMPVRepresentationProxy::LoadState(
+  vtkPVXMLElement* element, vtkSMProxyLocator* locator)
+{
+  if (!this->Superclass::LoadState(element, locator))
     {
-    vtkSMEnumerationDomain* enumDomain = vtkSMEnumerationDomain::SafeDownCast(
-      repProp->GetDomain("enum"));
-    if (enumDomain)
+    return 0;
+    }
+
+  // Representation types can be added through plugins. Hence the available
+  // representation types and the (text==value) mapping may be different at time
+  // of loading the state than at the time of saving the state. So we parse the
+  // state to obtain the text for the state-selected representation type and set
+  // the property to the value that corresponds to the state-indicated
+  // representation type (using the text as the invariant).
+
+  vtkSMIntVectorProperty* repProp = vtkSMIntVectorProperty::SafeDownCast(
+   this->GetProperty("Representation"));
+  vtkSMEnumerationDomain* enumDomain = repProp?
+    vtkSMEnumerationDomain::SafeDownCast(repProp->GetDomain("enum")): 0;
+  if (!enumDomain)
+    {
+    return 1;
+    }
+  unsigned int numElements = element->GetNumberOfNestedElements();
+  for (unsigned int cc=0; cc < numElements; cc++)
+    {
+    vtkPVXMLElement* currentElement = element->GetNestedElement(cc);
+    const char* name =  currentElement->GetName();
+    if (name && strcmp(name, "Property") == 0 && 
+      currentElement->GetAttribute("name") &&
+      strcmp(currentElement->GetAttribute("name"), "Representation") == 0 &&
+      currentElement->FindNestedElementByName("Domain"))
       {
-      // Add strings for all types >= USER_DEFINED.
-      vtkInternals::RepresentationProxiesMap::iterator iter;
-      for (iter = this->Internals->RepresentationProxies.begin();
-        iter != this->Internals->RepresentationProxies.end(); ++iter)
+      // Using the domain in the state file, determine what was the
+      // representation type the user wanted and then set it on the property.
+      const char* repr_text = 0;
+
+      vtkSmartPointer<vtkCollection> entries = vtkSmartPointer<vtkCollection>::New();
+      currentElement->FindNestedElementByName("Domain")->GetElementsByName(
+        "Entry", entries);
+      for (int kk=0; kk < entries->GetNumberOfItems() && !repr_text; kk++)
         {
-        if (iter->first >= vtkSMPVRepresentationProxy::USER_DEFINED &&
-          !iter->second.Text.empty())
+        vtkPVXMLElement* entry = vtkPVXMLElement::SafeDownCast(
+          entries->GetItemAsObject(kk));
+        int value;
+        if (entry->GetScalarAttribute("value", &value) &&
+          value == repProp->GetElement(0))
           {
-          enumDomain->AddEntry(iter->second.Text.c_str(), iter->first);
+          repr_text = entry->GetAttribute("text");
           }
         }
+      if (repr_text)
+        {
+        if (!enumDomain->HasEntryText(repr_text))
+          {
+          vtkWarningMacro("Cannot restore representation type to \'"
+            << repr_text << "\' since possibly some plugins are missing.");
+          return 1;
+          }
+
+        repProp->SetElement(0, enumDomain->GetEntryValueForText(repr_text));
+        }
+      break;
       }
     }
+
   return 1;
 }
 
@@ -486,33 +772,38 @@ int vtkSMPVRepresentationProxy::ReadXMLAttributes(
 void vtkSMPVRepresentationProxy::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
+#define PRINT_REP_CASE(type) case type: os << #type << endl; break;
   os << indent << "Representation: " ;
   switch (this->Representation)
     {
-  case SURFACE:
-    os << "Surface";
-    break;
-
-  case WIREFRAME:
-    os << "Wireframe";
-    break;
-
-  case POINTS:
-    os << "Points";
-    break;
-
-  case OUTLINE:
-    os << "Outline";
-    break;
-
-  case VOLUME:
-    os << "Volume";
-    break;
-
-  default:
-    os << "(unknown)";
+    PRINT_REP_CASE(SURFACE);
+    PRINT_REP_CASE(WIREFRAME);
+    PRINT_REP_CASE(POINTS);
+    PRINT_REP_CASE(OUTLINE);
+    PRINT_REP_CASE(VOLUME);
+    PRINT_REP_CASE(SURFACE_WITH_EDGES);
+    PRINT_REP_CASE(SLICE);
+    PRINT_REP_CASE(USER_DEFINED);
+    default:
+      os << "(unknown)" << endl;;
+    }
+  os << indent << "BackfaceRepresentation: " ;
+  switch (this->BackfaceRepresentation)
+    {
+    PRINT_REP_CASE(SURFACE);
+    PRINT_REP_CASE(WIREFRAME);
+    PRINT_REP_CASE(POINTS);
+    PRINT_REP_CASE(OUTLINE);
+    PRINT_REP_CASE(VOLUME);
+    PRINT_REP_CASE(SURFACE_WITH_EDGES);
+    PRINT_REP_CASE(SLICE);
+    PRINT_REP_CASE(USER_DEFINED);
+    PRINT_REP_CASE(FOLLOW_FRONTFACE);
+    PRINT_REP_CASE(CULL_BACKFACE);
+    PRINT_REP_CASE(CULL_FRONTFACE);
+    default:
+      os << "(unknown)" << endl;;
     }
-  os << endl;
   os << indent << "CubeAxesVisibility: " << this->CubeAxesVisibility << endl;
 }
 
