@@ -1,7 +1,7 @@
 /*=========================================================================
 
 Program:   ParaView
-Module:    $RCSfile: vtkSMPWriterProxy.h,v $
+Module:    vtkSMPWriterProxy.h
 
 Copyright (c) Kitware, Inc.
 All rights reserved.
@@ -16,6 +16,8 @@ PURPOSE.  See the above copyright notice for more information.
 // writing.
 // .SECTION Description
 // vtkSMPWriterProxy is the proxy for all writers that can write in parallel.
+// The only extra functionality provided by this class is that it sets up the
+// piece information on the writer.
 
 #ifndef __vtkSMPWriterProxy_h
 #define __vtkSMPWriterProxy_h
@@ -26,22 +28,9 @@ class VTK_EXPORT vtkSMPWriterProxy : public vtkSMWriterProxy
 {
 public:
   static vtkSMPWriterProxy* New();
-  vtkTypeRevisionMacro(vtkSMPWriterProxy, vtkSMWriterProxy);
+  vtkTypeMacro(vtkSMPWriterProxy, vtkSMWriterProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
-  virtual void AddInput(unsigned int inputPort,
-                        vtkSMSourceProxy* input, 
-                        unsigned int outputPort,
-                        const char* method);
-  virtual void AddInput(vtkSMSourceProxy* input,
-                        const char* method)
-  {
-    this->AddInput(0, input, 0, method);
-  }
-
-  virtual void UpdatePipeline();
-  virtual void UpdatePipeline(double time);
-
 protected:
   vtkSMPWriterProxy();
   ~vtkSMPWriterProxy();
