@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProperty.cxx,v $
+  Module:    vtkSMProperty.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -16,7 +16,7 @@
 
 #include "vtkClientServerStream.h"
 #include "vtkObjectFactory.h"
-#include "vtkPVServerManagerInstantiator.h"
+#include "vtkInstantiator.h"
 #include "vtkPVXMLElement.h"
 #include "vtkSmartPointer.h"
 #include "vtkSMDocumentation.h"
@@ -33,9 +33,7 @@
 #include "vtkSMPropertyInternals.h"
 
 vtkStandardNewMacro(vtkSMProperty);
-vtkCxxRevisionMacro(vtkSMProperty, "$Revision: 1.59 $");
 
-vtkCxxSetObjectMacro(vtkSMProperty, Proxy, vtkSMProxy);
 vtkCxxSetObjectMacro(vtkSMProperty, InformationHelper, vtkSMInformationHelper);
 vtkCxxSetObjectMacro(vtkSMProperty, InformationProperty, vtkSMProperty);
 vtkCxxSetObjectMacro(vtkSMProperty, Documentation, vtkSMDocumentation);
@@ -66,6 +64,8 @@ vtkSMProperty::vtkSMProperty()
   this->Hints = 0;
   this->BlockModifiedEvents = false;
   this->PendingModifiedEvents = false;
+
+  this->Proxy = 0;
 }
 
 //---------------------------------------------------------------------------
@@ -76,11 +76,11 @@ vtkSMProperty::~vtkSMProperty()
   this->SetXMLName(0);
   this->SetXMLLabel(0);
   this->DomainIterator->Delete();
-  this->SetProxy(0);
   this->SetInformationHelper(0);
   this->SetInformationProperty(0);
   this->SetDocumentation(0);
   this->SetHints(0);
+  this->SetParent(0);
 }
 
 //-----------------------------------------------------------------------------
@@ -147,6 +147,21 @@ vtkSMDomain* vtkSMProperty::GetDomain(const char* name)
   return it->second.GetPointer();
 }
 
+//---------------------------------------------------------------------------
+vtkSMDomain* vtkSMProperty::FindDomain(const char* classname)
+{
+  vtkSmartPointer<vtkSMDomainIterator> iter;
+  iter.TakeReference(this->NewDomainIterator());
+  for (iter->Begin(); !iter->IsAtEnd(); iter->Next())
+    {
+    if (iter->GetDomain()->IsA(classname))
+      {
+      return iter->GetDomain();
+      }
+    }
+  return 0;
+}
+
 //---------------------------------------------------------------------------
 unsigned int vtkSMProperty::GetNumberOfDomains()
 {
@@ -322,13 +337,9 @@ void vtkSMProperty::CreatePrettyLabel(const char* xmlname)
 }
 
 //---------------------------------------------------------------------------
-int vtkSMProperty::ReadXMLAttributes(vtkSMProxy* proxy,
+int vtkSMProperty::ReadXMLAttributes(vtkSMProxy* vtkNotUsed(proxy),
                                      vtkPVXMLElement* element)
 {
-  // Set during xml parsing only. Used in NewProperty() which is
-  // called by domains to get required properties.
-  this->SetProxy(proxy);
-
   const char* xmlname = element->GetAttribute("name");
   if(xmlname) 
     { 
@@ -461,13 +472,12 @@ int vtkSMProperty::ReadXMLAttributes(vtkSMProxy* proxy,
       }
     }
 
-  this->SetProxy(0);
   return 1;
 }
 
 //---------------------------------------------------------------------------
 int vtkSMProperty::LoadState(vtkPVXMLElement* propertyElement, 
-  vtkSMStateLoaderBase* loader, int vtkNotUsed(loadLastPushedValues))
+  vtkSMProxyLocator* loader, int vtkNotUsed(loadLastPushedValues))
 {
   // Process the domains.
   unsigned int numElems = propertyElement->GetNumberOfNestedElements();
@@ -568,7 +578,7 @@ void vtkSMProperty::ResetToDefault()
 void vtkSMProperty::PrintSelf(ostream& os, vtkIndent indent)
 {
   this->Superclass::PrintSelf(os, indent);
-
+  os << indent << "Proxy: " << this->Proxy << endl;
   os << indent << "Command: " 
      << (this->Command ? this->Command : "(null)") << endl;
   os << indent << "ImmediateUpdate:" << this->ImmediateUpdate << endl;
