@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMPropertyHelper.cxx,v $
+  Module:    vtkSMPropertyHelper.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -48,6 +48,9 @@
 #include "vtkSMIntVectorProperty.h"
 #include "vtkSMProxy.h"
 #include "vtkSMStringVectorProperty.h"
+#include "vtkStringList.h"
+
+#include <vtksys/ios/sstream>
 
 #include <assert.h>
 inline unsigned int vtkSMPropertyHelperMin(unsigned int x, unsigned int y)
@@ -55,16 +58,28 @@ inline unsigned int vtkSMPropertyHelperMin(unsigned int x, unsigned int y)
   return x < y? x : y;
 }
 
+#define vtkSMPropertyHelperWarningMacro(blah)\
+  if (this->Quiet == false) \
+  {\
+    vtkGenericWarningMacro(blah)\
+  }
+
+
 //----------------------------------------------------------------------------
-vtkSMPropertyHelper::vtkSMPropertyHelper(vtkSMProxy* proxy, const char* pname)
+vtkSMPropertyHelper::vtkSMPropertyHelper(vtkSMProxy* proxy, const char* pname,
+  bool quiet)
 {
   this->Proxy = proxy;
   this->Property = proxy->GetProperty(pname);
   this->Type = vtkSMPropertyHelper::NONE;
+  this->DoubleValues = NULL;
+  this->IntValues = NULL;
+  this->IdTypeValues = NULL;
+  this->Quiet = quiet;
 
   if (!this->Property)
     {
-    vtkGenericWarningMacro("Failed to locate property: " << pname);
+    vtkSMPropertyHelperWarningMacro("Failed to locate property: " << pname);
     }
   else if (this->Property->IsA("vtkSMIntVectorProperty"))
     {
@@ -92,16 +107,18 @@ vtkSMPropertyHelper::vtkSMPropertyHelper(vtkSMProxy* proxy, const char* pname)
     }
   else
     {
-    vtkGenericWarningMacro("Unhandled property type : " << this->Property->GetClassName());
+    vtkSMPropertyHelperWarningMacro("Unhandled property type : " << this->Property->GetClassName());
     }
 }
 
 //----------------------------------------------------------------------------
 vtkSMPropertyHelper::~vtkSMPropertyHelper()
 {
+  delete [] this->IntValues;
+  delete [] this->IdTypeValues;
+  delete [] this->DoubleValues;
 }
 
-
 //----------------------------------------------------------------------------
 void vtkSMPropertyHelper::UpdateValueFromServer()
 {
@@ -138,7 +155,7 @@ void vtkSMPropertyHelper::SetNumberOfElements(unsigned int elems)
     SM_TEMPLATE_MACRO_VP(SMProperty->SetNumberOfElements(elems));
     SM_TEMPLATE_MACRO_PP(SMProperty->SetNumberOfProxies(elems));
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -151,7 +168,7 @@ unsigned int vtkSMPropertyHelper::GetNumberOfElements()
     SM_TEMPLATE_MACRO_PP(return SMProperty->GetNumberOfProxies());
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
@@ -164,7 +181,7 @@ void vtkSMPropertyHelper::Set(unsigned int index, int value)
     {
     SM_TEMPLATE_MACRO_NUM(SMProperty->SetElement(index, static_cast<VTK_TT>(value)));
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -177,7 +194,7 @@ int vtkSMPropertyHelper::GetAsInt(unsigned int index /*=0*/)
       return static_cast<int>(SMProperty->GetElement(index)));
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
@@ -198,12 +215,26 @@ unsigned int vtkSMPropertyHelper::Get(int *values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
 }
 
+//----------------------------------------------------------------------------
+const int* vtkSMPropertyHelper::GetAsIntPtr()
+{
+  delete [] this->IntValues;
+  this->IntValues = NULL;
+  int num_elems = this->GetNumberOfElements();
+  if (num_elems)
+    {
+    this->IntValues = new int[num_elems];
+    this->Get(this->IntValues, num_elems);
+    }
+  return this->IntValues;
+}
+
 //----------------------------------------------------------------------------
 void vtkSMPropertyHelper::Set(const int* values, unsigned int count)
 {
@@ -221,7 +252,7 @@ void vtkSMPropertyHelper::Set(const int* values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -232,7 +263,7 @@ void vtkSMPropertyHelper::Set(unsigned int index, double value)
     {
     SM_TEMPLATE_MACRO_NUM(SMProperty->SetElement(index, static_cast<VTK_TT>(value)));
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -245,7 +276,7 @@ double vtkSMPropertyHelper::GetAsDouble(unsigned int index /*=0*/)
       return static_cast<double>(SMProperty->GetElement(index)));
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
@@ -266,12 +297,26 @@ unsigned int vtkSMPropertyHelper::Get(double *values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
 }
 
+//----------------------------------------------------------------------------
+const double* vtkSMPropertyHelper::GetAsDoublePtr()
+{
+  delete [] this->DoubleValues;
+  this->DoubleValues = NULL;
+  int num_elems = this->GetNumberOfElements();
+  if (num_elems)
+    {
+    this->DoubleValues= new double[num_elems];
+    this->Get(this->DoubleValues, num_elems);
+    }
+  return this->DoubleValues;
+}
+
 //----------------------------------------------------------------------------
 void vtkSMPropertyHelper::Set(const double* values, unsigned int count)
 {
@@ -289,7 +334,7 @@ void vtkSMPropertyHelper::Set(const double* values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -302,7 +347,7 @@ void vtkSMPropertyHelper::Set(unsigned int index, vtkIdType value)
     SM_TEMPLATE_MACRO_NUM(SMProperty->SetElement(index, static_cast<VTK_TT>(value)));
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -323,7 +368,7 @@ void vtkSMPropertyHelper::Set(const vtkIdType* values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -342,7 +387,7 @@ unsigned int vtkSMPropertyHelper::Get(vtkIdType* values, unsigned int count)
     );
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
@@ -358,12 +403,26 @@ vtkIdType vtkSMPropertyHelper::GetAsIdType(unsigned int index /*=0*/)
       return static_cast<vtkIdType>(SMProperty->GetElement(index)));
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 
   return 0;
 }
 
+//----------------------------------------------------------------------------
+const vtkIdType* vtkSMPropertyHelper::GetAsIdTypePtr()
+{
+  delete [] this->IdTypeValues;
+  this->IdTypeValues = NULL;
+  int num_elems = this->GetNumberOfElements();
+  if (num_elems)
+    {
+    this->IdTypeValues = new vtkIdType[num_elems];
+    this->Get(this->IdTypeValues, num_elems);
+    }
+  return this->IdTypeValues;
+}
+
 //----------------------------------------------------------------------------
 void vtkSMPropertyHelper::Set(unsigned int index, const char* value)
 {
@@ -375,7 +434,7 @@ void vtkSMPropertyHelper::Set(unsigned int index, const char* value)
     }
   else
     {
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -389,7 +448,7 @@ const char* vtkSMPropertyHelper::GetAsString(unsigned int index /*=0*/)
     return svp->GetElement(index);
     }
 
-  vtkGenericWarningMacro("Call not supported for the current property type.");
+  vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
   return 0;
 }
 
@@ -409,7 +468,7 @@ void vtkSMPropertyHelper::Set(unsigned int index, vtkSMProxy* value,
     }
   else
     {
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -429,7 +488,7 @@ void vtkSMPropertyHelper::Set(vtkSMProxy** value, unsigned int count,
     }
   else
     {
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -448,7 +507,22 @@ void vtkSMPropertyHelper::Add(vtkSMProxy* value, unsigned int outputport/*=0*/)
     }
   else
     {
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
+    }
+}
+
+//----------------------------------------------------------------------------
+void vtkSMPropertyHelper::Remove(vtkSMProxy* value)
+{
+  if (this->Type == vtkSMPropertyHelper::PROXY ||
+    this->Type == vtkSMPropertyHelper::INPUT)
+    {
+    vtkSMProxyProperty* pp = static_cast<vtkSMProxyProperty*>(this->Property);
+    pp->RemoveProxy(value);
+    }
+  else
+    {
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
 }
 
@@ -465,7 +539,7 @@ vtkSMProxy* vtkSMPropertyHelper::GetAsProxy(unsigned int index/*=0*/)
     }
 
   default:
-    vtkGenericWarningMacro("Call not supported for the current property type.");
+    vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
     }
   return 0;
 }
@@ -479,8 +553,278 @@ unsigned int vtkSMPropertyHelper::GetOutputPort(unsigned int index/*=0*/)
     return ip->GetOutputPortForConnection(index);
     }
 
-  vtkGenericWarningMacro("Call not supported for the current property type.");
+  vtkSMPropertyHelperWarningMacro("Call not supported for the current property type.");
   return 0;
 }
 
+//----------------------------------------------------------------------------
+void vtkSMPropertyHelper::SetStatus(const char* key, int value)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+  if (svp->GetNumberOfElementsPerCommand() != 2)
+    {
+    vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != 2");
+    return;
+    }
+
+  if (!svp->GetRepeatCommand())
+    {
+    vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+    return;
+    }
+
+
+  vtksys_ios::ostringstream str;
+  str << value;
+
+  for (unsigned int cc=0; (cc+1) < svp->GetNumberOfElements(); cc+=2)
+    {
+    if (strcmp(svp->GetElement(cc), key) == 0)
+      {
+      svp->SetElement(cc+1, str.str().c_str());
+      return;
+      }
+    }
+
+  vtkStringList* list = vtkStringList::New();
+  svp->GetElements(list);
+  list->AddString(key);
+  list->AddString(str.str().c_str());
+  svp->SetElements(list);
+  list->Delete();
+}
+
+//----------------------------------------------------------------------------
+int vtkSMPropertyHelper::GetStatus(const char* key, int default_value/*=0*/)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return default_value;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+  while (svp)
+    {
+    if (svp->GetNumberOfElementsPerCommand() != 2)
+      {
+      vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != 2");
+      return default_value;
+      }
+
+    if (!svp->GetRepeatCommand())
+      {
+      vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+      return default_value;
+      }
+
+    for (unsigned int cc=0; (cc+1) < svp->GetNumberOfElements(); cc+=2)
+      {
+      if (strcmp(svp->GetElement(cc), key) == 0)
+        {
+        return atoi(svp->GetElement(cc+1));
+        }
+      }
+
+    // Now check if the information_property has the value.
+    svp = svp->GetInformationOnly() == 0? 
+      vtkSMStringVectorProperty::SafeDownCast(svp->GetInformationProperty()) : 0;
+    }
+
+  return default_value;
+}
+
+//----------------------------------------------------------------------------
+void vtkSMPropertyHelper::SetStatus(const char* key, double *values,
+  int num_values)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+  if (svp->GetNumberOfElementsPerCommand() != num_values+1)
+    {
+    vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != " << num_values + 1);
+    return;
+    }
+
+  if (!svp->GetRepeatCommand())
+    {
+    vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+    return;
+    }
+
+
+  vtkStringList* list = vtkStringList::New();
+  svp->GetElements(list);
+
+  bool append = true;
+  for (unsigned int cc=0; (cc+num_values+1) <= svp->GetNumberOfElements();
+    cc+=(num_values+1))
+    {
+    if (strcmp(svp->GetElement(cc), key) == 0)
+      {
+      for (int kk=0; kk < num_values; kk++)
+        {
+        vtksys_ios::ostringstream str;
+        str << values[kk];
+        list->SetString(cc+kk+1, str.str().c_str());
+        }
+      append = false;
+      }
+    }
+
+  if (append)
+    {
+    list->AddString(key);
+    for (int kk=0; kk < num_values; kk++)
+      {
+      vtksys_ios::ostringstream str;
+      str << values[kk];
+      list->AddString(str.str().c_str());
+      }
+    }
+  svp->SetElements(list);
+  list->Delete();
+}
+
+//----------------------------------------------------------------------------
+bool vtkSMPropertyHelper::GetStatus(const char* key, double *values, int num_values)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return false;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+
+  while (svp)
+    {
+    if (svp->GetNumberOfElementsPerCommand() != num_values+1)
+      {
+      vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != " << num_values + 1);
+      return false;
+      }
+
+    if (!svp->GetRepeatCommand())
+      {
+      vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+      return false;
+      }
+
+    for (unsigned int cc=0; (cc+num_values+1) <= svp->GetNumberOfElements();
+      cc+=(num_values+1))
+      {
+      if (strcmp(svp->GetElement(cc), key) == 0)
+        {
+        for (int kk=0; kk < num_values; kk++)
+          {
+          values[kk] = atof(svp->GetElement(cc+kk+1));
+          }
+        return true;
+        }
+      }
+
+    // Now check if the information_property has the value.
+    svp = svp->GetInformationOnly() == 0? 
+      vtkSMStringVectorProperty::SafeDownCast(svp->GetInformationProperty()) : 0;
+    }
+
+  return false;
+}
+
+//----------------------------------------------------------------------------
+void vtkSMPropertyHelper::SetStatus(const char* key, const char* value)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+  if (svp->GetNumberOfElementsPerCommand() != 2)
+    {
+    vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != 2");
+    return;
+    }
+
+  if (!svp->GetRepeatCommand())
+    {
+    vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+    return;
+    }
+
+  for (unsigned int cc=0; (cc+1) < svp->GetNumberOfElements(); cc+=2)
+    {
+    if (strcmp(svp->GetElement(cc), key) == 0)
+      {
+      svp->SetElement(cc+1, value);
+      return;
+      }
+    }
+
+  vtkStringList* list = vtkStringList::New();
+  svp->GetElements(list);
+  list->AddString(key);
+  list->AddString(value);
+  svp->SetElements(list);
+  list->Delete();
+}
+
+//----------------------------------------------------------------------------
+const char* vtkSMPropertyHelper::GetStatus(const char* key, const char* default_value)
+{
+  if (this->Type != vtkSMPropertyHelper::STRING)
+    {
+    vtkSMPropertyHelperWarningMacro("Status properties can only be vtkSMStringVectorProperty.");
+    return default_value;
+    }
+
+  vtkSMStringVectorProperty* svp = vtkSMStringVectorProperty::SafeDownCast(
+    this->Property);
+  while (svp)
+    {
+    if (svp->GetNumberOfElementsPerCommand() != 2)
+      {
+      vtkSMPropertyHelperWarningMacro("NumberOfElementsPerCommand != 2");
+      return default_value;
+      }
+
+    if (!svp->GetRepeatCommand())
+      {
+      vtkSMPropertyHelperWarningMacro("Property is non-repeatable.");
+      return default_value;
+      }
+
+    for (unsigned int cc=0; (cc+1) < svp->GetNumberOfElements(); cc+=2)
+      {
+      if (strcmp(svp->GetElement(cc), key) == 0)
+        {
+        return svp->GetElement(cc+1);
+        }
+      }
+
+    // Now check if the information_property has the value.
+    svp = svp->GetInformationOnly() == 0? 
+      vtkSMStringVectorProperty::SafeDownCast(svp->GetInformationProperty()) : 0;
+    }
+
+  return default_value;
+}
 
