@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxy.cxx,v $
+  Module:    vtkSMProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -27,8 +27,8 @@
 #include "vtkSMDocumentation.h"
 #include "vtkSMInputProperty.h"
 #include "vtkSMPropertyIterator.h"
+#include "vtkSMProxyLocator.h"
 #include "vtkSMProxyManager.h"
-#include "vtkSMStateLoaderBase.h"
 
 #include "vtkSMProxyInternals.h"
 
@@ -38,7 +38,6 @@
 #include <vtksys/ios/sstream>
 
 vtkStandardNewMacro(vtkSMProxy);
-vtkCxxRevisionMacro(vtkSMProxy, "$Revision: 1.105 $");
 
 vtkCxxSetObjectMacro(vtkSMProxy, XMLElement, vtkPVXMLElement);
 vtkCxxSetObjectMacro(vtkSMProxy, Hints, vtkPVXMLElement);
@@ -164,6 +163,7 @@ vtkSMProxy::~vtkSMProxy()
       vtkSMProxyProperty::SafeDownCast(
         prop)->RemoveConsumerFromPreviousProxies(this);
       }
+    prop->SetParent(0);
     }
   delete this->Internals;
   this->SetVTKClassName(0);
@@ -422,7 +422,9 @@ vtkClientServerID vtkSMProxy::GetID()
 //---------------------------------------------------------------------------
 vtkObjectBase* vtkSMProxy::GetClientSideObject()
 {
-  return vtkProcessModule::GetProcessModule()->GetObjectFromID(this->GetID());
+  return vtkProcessModule::GetProcessModule()->GetObjectFromID(
+    this->GetID(), true); // the second argument means that no error
+                          // will be printed if the ID is invalid.
 }
 
 //---------------------------------------------------------------------------
@@ -531,6 +533,7 @@ void vtkSMProxy::RemoveProperty(const char* name)
     this->Internals->Properties.find(name);
   if (it != this->Internals->Properties.end())
     {
+    it->second.Property->SetParent(0);
     this->Internals->Properties.erase(it);
     }
 
@@ -618,6 +621,7 @@ void vtkSMProxy::AddPropertyToSelf(
       {
       oldProp->RemoveObserver(it->second.ObserverTag);
       }
+    oldProp->SetParent(0);
     }
 
   unsigned int tag=0;
@@ -630,6 +634,8 @@ void vtkSMProxy::AddPropertyToSelf(
   tag = prop->AddObserver(vtkCommand::ModifiedEvent, obs);
   obs->Delete();
 
+  prop->SetParent(this);
+
   vtkSMProxyInternals::PropertyInfo newEntry;
   newEntry.Property = prop;
   newEntry.ObserverTag = tag;
@@ -641,6 +647,96 @@ void vtkSMProxy::AddPropertyToSelf(
   this->Internals->PropertyNamesInOrder.push_back(name);
 }
 
+//---------------------------------------------------------------------------
+bool vtkSMProxy::UpdatePropertyInternal(const char* name, bool force,
+  vtkClientServerStream& stream)
+{
+  // This will ensure that the SelfID is assigned properly.
+  this->GetSelfID();
+
+  vtkSMProxyInternals::PropertyInfoMap::iterator it =
+    this->Internals->Properties.find(name);
+  if (it == this->Internals->Properties.end())
+    {
+    // Search exposed subproxy properties.
+    vtkSMProxyInternals::ExposedPropertyInfoMap::iterator eiter = 
+      this->Internals->ExposedProperties.find(name);
+    if (eiter == this->Internals->ExposedProperties.end())
+      {
+      return false;
+      }
+    const char* subproxy_name =  eiter->second.SubProxyName.c_str();
+    const char* property_name = eiter->second.PropertyName.c_str();
+    vtkSMProxy * sp = this->GetSubProxy(subproxy_name);
+    if (sp && sp->UpdatePropertyInternal(property_name, force, stream))
+      {
+      this->MarkModified(this);
+      return true;
+      }
+    return false;
+    }
+
+  if (!it->second.ModifiedFlag && !force)
+    {
+    return false;
+    }
+
+  // In case this property is a self property and causes
+  // another UpdateVTKObjects(), make sure that it does
+  // not cause recursion. If this is not set, UpdateVTKObjects()
+  // that is caused by UpdateProperty() can end up calling trying
+  // to push the same property.
+  it->second.ModifiedFlag = 0;
+
+  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
+  vtkSMProperty* prop = it->second.Property.GetPointer();
+  if (prop->GetUpdateSelf())
+    {
+    vtkClientServerStream str;
+    prop->AppendCommandToStream(this, &str, this->GetSelfID());
+    if (str.GetNumberOfMessages() > 0)
+      {
+      pm->SendStream(this->ConnectionID, vtkProcessModule::CLIENT, str);
+
+      // Fire event to let everyone know that a property has been updated.
+      this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)name);
+      if (!this->InUpdateVTKObjects) // if updating multiple properties
+        {                            // MarkModified() is called only once.
+        this->MarkModified(this);
+        }
+      return true;
+      }
+    }
+  else
+    {
+    if (this->VTKObjectID.IsNull())
+      {
+      // Make sure that server side objects exist before
+      // pushing values to them
+      this->CreateVTKObjects();
+      }
+    if (!this->VTKObjectID.IsNull())
+      {
+      int old_count = stream.GetNumberOfMessages();
+      prop->AppendCommandToStream(this, &stream, this->VTKObjectID);
+      if (stream.GetNumberOfMessages() > old_count)
+        {
+        //pm->SendStream(this->ConnectionID, this->Servers, str);
+
+        // Fire event to let everyone know that a property has been updated.
+        this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)name);
+        if (!this->InUpdateVTKObjects) // if updating multiple properties
+          {                            // MarkModified() is called only once.
+          this->MarkModified(this);
+          }
+        return true;  
+        }
+      }
+    }
+
+  return false;
+}
+
 //---------------------------------------------------------------------------
 bool vtkSMProxy::UpdateProperty(const char* name, int force)
 {
@@ -803,34 +899,30 @@ void vtkSMProxy::UpdatePropertyInformation(vtkSMProperty* prop)
     }
   if (!found)
     {
+    // Check if the property is an exposed property
+    const char *exposed_name = this->GetPropertyName(prop);
+    if (exposed_name)
+      {
+      vtkSMProxyInternals::ExposedPropertyInfoMap::iterator eiter = 
+      this->Internals->ExposedProperties.find(exposed_name);
+      if (eiter != this->Internals->ExposedProperties.end())
+        {
+        const char* subproxy_name =  eiter->second.SubProxyName.c_str();
+        const char* property_name = eiter->second.PropertyName.c_str();
+        vtkSMProxy * sp = this->GetSubProxy(subproxy_name);
+        if (sp)
+          {
+          sp->UpdatePropertyInformation(sp->GetProperty(property_name, 0));
+          }
+        }
+      }
+
     return;
     }
 
   this->CreateVTKObjects();
-
-  if (this->ObjectsCreated)
-    {
-    if (prop->GetInformationOnly())
-      {
-      if (prop->GetUpdateSelf())
-        {
-        prop->UpdateInformation(this->ConnectionID, 
-                                vtkProcessModule::CLIENT, 
-                                this->GetSelfID());
-        }
-      else
-        {
-        prop->UpdateInformation(this->ConnectionID,
-                                this->Servers, 
-                                this->VTKObjectID);
-        }
-      prop->UpdateDependentDomains();
-      }
-    // I cannot understand why updating a property information
-    // should mark a proxy modified. I am removing this line for now.
-    // If we run into problems, we'll have to investigate.
-    // this->MarkModified(this);
-    }
+  this->UpdatePropertyInformationInternal(prop);
+  prop->UpdateDependentDomains();
 }
 
 //---------------------------------------------------------------------------
@@ -850,20 +942,7 @@ void vtkSMProxy::UpdatePropertyInformation()
     ++it)
     {
     vtkSMProperty* prop = it->second.Property.GetPointer();
-    if (prop->GetInformationOnly())
-      {
-      if (prop->GetUpdateSelf())
-        {
-        prop->UpdateInformation(this->ConnectionID,
-          vtkProcessModule::CLIENT, this->GetSelfID());
-        }
-      else
-        {
-        prop->UpdateInformation(this->ConnectionID,
-                                this->Servers, 
-                                this->VTKObjectID);
-        }
-      }
+    this->UpdatePropertyInformationInternal(prop);
     }
 
   // Make sure all dependent domains are updated. UpdateInformation()
@@ -887,6 +966,27 @@ void vtkSMProxy::UpdatePropertyInformation()
     }
 }
 
+//---------------------------------------------------------------------------
+void vtkSMProxy::UpdatePropertyInformationInternal(vtkSMProperty* prop)
+{
+  if (this->ObjectsCreated)
+    {
+    if (prop->GetInformationOnly())
+      {
+      if (prop->GetUpdateSelf())
+        {
+        prop->UpdateInformation(this->ConnectionID, 
+          vtkProcessModule::CLIENT, this->GetSelfID());
+        }
+      else
+        {
+        prop->UpdateInformation(this->ConnectionID,
+          this->Servers, this->VTKObjectID);
+        }
+      }
+    }
+}
+
 //---------------------------------------------------------------------------
 int vtkSMProxy::ArePropertiesModified(int selfOnly /*=0*/)
 {
@@ -912,6 +1012,21 @@ int vtkSMProxy::ArePropertiesModified(int selfOnly /*=0*/)
 
 //---------------------------------------------------------------------------
 void vtkSMProxy::UpdateVTKObjects()
+{
+  vtkClientServerStream stream;
+  this->UpdateVTKObjects(stream);
+  if (stream.GetNumberOfMessages() > 0)
+    {
+    //cout << "Message Count: " << stream.GetNumberOfMessages() << endl;
+    vtkProcessModule::GetProcessModule()->SendStream(
+      this->ConnectionID,
+      this->Servers,
+      stream);
+    }
+}
+
+//---------------------------------------------------------------------------
+void vtkSMProxy::UpdateVTKObjects(vtkClientServerStream& buffer)
 {
   if (this->InUpdateVTKObjects)
     {
@@ -995,10 +1110,10 @@ void vtkSMProxy::UpdateVTKObjects()
       vtkSMProperty* prop = it->second.Property.GetPointer();
       if (!prop->GetInformationOnly())
         {
-        this->UpdateProperty(it->first.c_str());
+        this->UpdatePropertyInternal(it->first.c_str(), false, buffer);
         }
       }
-    modified = true;    
+    modified = true;
     }
 
   this->InUpdateVTKObjects = 0;
@@ -1010,7 +1125,7 @@ void vtkSMProxy::UpdateVTKObjects()
   // subproxy can ever set any property on the parent proxy.
   if (this->ArePropertiesModified(1))
     {
-    this->UpdateVTKObjects();
+    this->UpdateVTKObjects(buffer);
     }
 
   vtkSMProxyInternals::ProxyMap::iterator it2 =
@@ -1018,7 +1133,14 @@ void vtkSMProxy::UpdateVTKObjects()
   for( ; it2 != this->Internals->SubProxies.end(); it2++)
     {
     modified = modified || it2->second.GetPointer()->ArePropertiesModified();
-    it2->second.GetPointer()->UpdateVTKObjects();
+    if (it2->second.GetPointer()->Servers == this->Servers)
+      {
+      it2->second.GetPointer()->UpdateVTKObjects(buffer);
+      }
+    else
+      {
+      it2->second.GetPointer()->UpdateVTKObjects();
+      }
     }
 
   if (modified)
@@ -1247,7 +1369,7 @@ void vtkSMProxy::ReviveVTKObjects()
 //---------------------------------------------------------------------------
 unsigned int vtkSMProxy::GetNumberOfSubProxies()
 {
-  return this->Internals->SubProxies.size();
+  return static_cast<unsigned int>(this->Internals->SubProxies.size());
 }
 
 //---------------------------------------------------------------------------
@@ -1314,7 +1436,8 @@ vtkSMProxy* vtkSMProxy::GetSubProxy(const char* name)
 }
 
 //---------------------------------------------------------------------------
-void vtkSMProxy::AddSubProxy(const char* name, vtkSMProxy* proxy)
+void vtkSMProxy::AddSubProxy(const char* name, vtkSMProxy* proxy, 
+  int override)
 {
   // Check if the proxy already exists. If it does, we will
   // replace it
@@ -1323,7 +1446,10 @@ void vtkSMProxy::AddSubProxy(const char* name, vtkSMProxy* proxy)
 
   if (it != this->Internals->SubProxies.end())
     {
-    vtkWarningMacro("Proxy " << name  << " already exists. Replacing");
+    if (!override)
+      {
+      vtkWarningMacro("Proxy " << name  << " already exists. Replacing");
+      }
     // needed to remove any observers.
     this->RemoveSubProxy(name);
     }
@@ -1453,6 +1579,7 @@ void vtkSMProxy::ExecuteSubProxyEvent(vtkSMProxy* subproxy,
       {
       // UpdatePropertyEvent is fired only for exposed properties.
       this->InvokeEvent(vtkCommand::UpdatePropertyEvent, (void*)exposed_name);
+      this->MarkModified(this);
       }
     }
 
@@ -1486,13 +1613,13 @@ void vtkSMProxy::AddConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
 }
 
 //---------------------------------------------------------------------------
-void vtkSMProxy::RemoveConsumer(vtkSMProperty* property, vtkSMProxy* proxy)
+void vtkSMProxy::RemoveConsumer(vtkSMProperty* property, vtkSMProxy*)
 {
   vtkstd::vector<vtkSMProxyInternals::ConnectionInfo>::iterator i = 
     this->Internals->Consumers.begin();
   for(; i != this->Internals->Consumers.end(); i++)
     {
-    if ( i->Property == property && i->Proxy == proxy )
+    if ( i->Property == property )
       {
       this->Internals->Consumers.erase(i);
       break;
@@ -1510,7 +1637,7 @@ void vtkSMProxy::RemoveAllConsumers()
 //---------------------------------------------------------------------------
 unsigned int vtkSMProxy::GetNumberOfConsumers()
 {
-  return this->Internals->Consumers.size();
+  return static_cast<unsigned int>(this->Internals->Consumers.size());
 }
 
 //---------------------------------------------------------------------------
@@ -1566,7 +1693,7 @@ void vtkSMProxy::RemoveProducer(vtkSMProperty* property, vtkSMProxy* proxy)
 //---------------------------------------------------------------------------
 unsigned int vtkSMProxy::GetNumberOfProducers()
 {
-  return this->Internals->Producers.size();
+  return static_cast<unsigned int>(this->Internals->Producers.size());
 }
 
 //---------------------------------------------------------------------------
@@ -1587,16 +1714,16 @@ void vtkSMProxy::PostUpdateData()
   unsigned int numProducers = this->GetNumberOfProducers();
   for (unsigned int i=0; i<numProducers; i++)
     {
-    if (this->GetVTKClassName() && this->GetProducerProxy(i)->GetVTKClassName())
-      {
-      }
     if (this->GetProducerProxy(i)->NeedsUpdate)
       {
       this->GetProducerProxy(i)->PostUpdateData();
       }
     }
-  this->InvokeEvent(vtkCommand::UpdateDataEvent, 0);
-  this->NeedsUpdate = false;
+  if (this->NeedsUpdate)
+    {
+    this->InvokeEvent(vtkCommand::UpdateDataEvent, 0);
+    this->NeedsUpdate = false;
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -1691,7 +1818,7 @@ vtkSMProperty* vtkSMProxy::NewProperty(const char* name)
 
 //----------------------------------------------------------------------------
 vtkSMProperty* vtkSMProxy::NewProperty(const char* name, 
-  vtkPVXMLElement* propElement)
+                                       vtkPVXMLElement* propElement)
 {
   vtkSMProperty* property = this->GetProperty(name, 1);
   if (property)
@@ -1779,6 +1906,26 @@ void vtkSMProxy::ReadCoreXMLAttributes(vtkPVXMLElement* element)
     this->SetXMLLabel(xmllabel);
     }
 
+  const char* processes = element->GetAttribute("processes");
+  if (processes)
+    {
+    vtkTypeUInt32 uiprocesses = 0;
+    vtkstd::string strprocesses = processes;
+    if (strprocesses.find("client") != vtkstd::string::npos)
+      {
+      uiprocesses |= vtkProcessModule::CLIENT;
+      }
+    if (strprocesses.find("renderserver") != vtkstd::string::npos)
+      {
+      uiprocesses |= vtkProcessModule::RENDER_SERVER;
+      }
+    if (strprocesses.find("dataserver") != vtkstd::string::npos)
+      {
+      uiprocesses |= vtkProcessModule::DATA_SERVER;
+      }
+    this->SetServersSelf(uiprocesses);
+    }
+
   // Locate documentation.
   for (unsigned int cc=0; cc < element->GetNumberOfNestedElements(); ++cc)
     {
@@ -1811,6 +1958,7 @@ int vtkSMProxy::ReadXMLAttributes(
     {
     return 0;
     }
+
   this->SetXMLElement(0);
   return 1;
 }
@@ -1859,6 +2007,11 @@ int vtkSMProxy::CreateSubProxiesAndProperties(vtkSMProxyManager* pm,
         const char* name = subElement->GetAttribute("name");
         const char* pname = subElement->GetAttribute("proxyname");
         const char* gname = subElement->GetAttribute("proxygroup");
+        int override = 0;
+        if (!subElement->GetScalarAttribute("override", &override))
+          {
+          override = 0;
+          }
         if (pname && !gname)
           {
           vtkErrorMacro("proxygroup not specified. Subproxy cannot be created.");
@@ -1886,9 +2039,9 @@ int vtkSMProxy::CreateSubProxiesAndProperties(vtkSMProxyManager* pm,
                           << (pname?pname:"(none"));
             return 0;
             }
+          this->AddSubProxy(name, subproxy, override);
           this->SetupSharedProperties(subproxy, propElement);
           this->SetupExposedProperties(name, propElement);
-          this->AddSubProxy(name, subproxy);
           subproxy->Delete();
           }
         }
@@ -1943,7 +2096,28 @@ void vtkSMProxy::SetupExposedProperties(const char* subproxy_name,
          // use the property name as the exposed name.
          exposed_name = name;
          }
-       this->ExposeSubProxyProperty(subproxy_name, name, exposed_name);
+       int override = 0;
+       if (!propertyElement->GetScalarAttribute("override", &override))
+         {
+         override = 0;
+         }
+
+       if (propertyElement->GetAttribute("default_values"))
+         {
+         vtkSMProxy* subproxy = this->GetSubProxy(subproxy_name);
+         vtkSMProperty* prop = subproxy->GetProperty(name);
+         if (!prop)
+           {
+           vtkWarningMacro("Failed to locate property '" << name
+             << "' on subproxy '" << subproxy_name << "'");
+           return;
+           }
+         if (!prop->ReadXMLAttributes(subproxy, propertyElement))
+           {
+           return;
+           }
+         }
+       this->ExposeSubProxyProperty(subproxy_name, name, exposed_name, override);
        }
     }
 }
@@ -2002,8 +2176,8 @@ void vtkSMProxy::SetupSharedProperties(vtkSMProxy* subproxy,
 }
 
 //---------------------------------------------------------------------------
-int vtkSMProxy::LoadState(vtkPVXMLElement* proxyElement, 
-                          vtkSMStateLoaderBase* loader)
+int vtkSMProxy::LoadState(
+  vtkPVXMLElement* proxyElement, vtkSMProxyLocator* locator)
 {
   unsigned int numElems = proxyElement->GetNumberOfNestedElements();
   int servers =0;
@@ -2034,18 +2208,19 @@ int vtkSMProxy::LoadState(vtkPVXMLElement* proxyElement,
         vtkDebugMacro("Property " << prop_name<< " does not exist.");
         continue;
         }
-      if (!property->LoadState(currentElement, loader))
+      if (!property->LoadState(currentElement, locator))
         {
         return 0;
         }
       }
     else if (strcmp(name, "SubProxy") == 0)
       {
-      this->LoadSubProxyState(currentElement, loader);
+      this->LoadSubProxyState(currentElement, locator);
       }
-    else if (strcmp(name, "RevivalState") == 0 && loader->GetReviveProxies())
+    else if (strcmp(name, "RevivalState") == 0 &&
+      locator && locator->GetReviveProxies())
       {
-      if (!this->LoadRevivalState(currentElement, loader))
+      if (!this->LoadRevivalState(currentElement))
         {
         return 0;
         }
@@ -2055,8 +2230,7 @@ int vtkSMProxy::LoadState(vtkPVXMLElement* proxyElement,
 }
 
 //---------------------------------------------------------------------------
-int vtkSMProxy::LoadRevivalState(vtkPVXMLElement* revivalElem,
-  vtkSMStateLoaderBase* loader)
+int vtkSMProxy::LoadRevivalState(vtkPVXMLElement* revivalElem)
 {
   if (this->ObjectsCreated)
     {
@@ -2118,8 +2292,7 @@ int vtkSMProxy::LoadRevivalState(vtkPVXMLElement* revivalElem,
                       << ". Cannot revive the subproxy.");
         return 0;
         }
-      if (!subProxy->LoadRevivalState(
-            currentElement->GetNestedElement(0), loader))
+      if (!subProxy->LoadRevivalState(currentElement->GetNestedElement(0)))
         {
         return 0;
         }
@@ -2130,8 +2303,29 @@ int vtkSMProxy::LoadRevivalState(vtkPVXMLElement* revivalElem,
   return 1;
 }
 
+//---------------------------------------------------------------------------
+int vtkSMProxy::RevertState(
+  vtkPVXMLElement* vtkNotUsed(element),
+  vtkSMProxyLocator* vtkNotUsed(locator))
+{
+  //vtkWarningMacro("RevertState not supported by this proxy");
+  return 0;
+}
+
 //---------------------------------------------------------------------------
 vtkPVXMLElement* vtkSMProxy::SaveState(vtkPVXMLElement* root)
+{
+  vtkSMPropertyIterator *iter=this->NewPropertyIterator();
+  vtkPVXMLElement *proxyXml=this->SaveState(root,iter,1);
+  iter->Delete();
+  return proxyXml;
+}
+
+//---------------------------------------------------------------------------
+vtkPVXMLElement* vtkSMProxy::SaveState(
+    vtkPVXMLElement* root,
+    vtkSMPropertyIterator *iter,
+    int saveSubProxies)
 {
   vtkPVXMLElement* proxyElement = vtkPVXMLElement::New();
   proxyElement->SetName("Proxy");
@@ -2140,10 +2334,15 @@ vtkPVXMLElement* vtkSMProxy::SaveState(vtkPVXMLElement* root)
   proxyElement->AddAttribute("id", this->GetSelfIDAsString());
   proxyElement->AddAttribute("servers", 
     static_cast<unsigned int>(this->Servers));
-  vtkSMPropertyIterator* iter = this->NewPropertyIterator();
 
-  while (!iter->IsAtEnd())
+  for (iter->Begin(); !iter->IsAtEnd(); iter->Next())
     {
+    if (!iter->GetProperty())
+      {
+      vtkWarningMacro("Missing property with name: " << iter->GetKey()
+        << " on " << this->GetXMLName());
+      continue;
+      }
     if (!iter->GetProperty()->GetIsInternal())
       {
       vtksys_ios::ostringstream propID;
@@ -2151,18 +2350,19 @@ vtkPVXMLElement* vtkSMProxy::SaveState(vtkPVXMLElement* root)
       iter->GetProperty()->SaveState(proxyElement, iter->GetKey(),
         propID.str().c_str());
       }
-    iter->Next();
     }
 
-  iter->Delete();
-
   if (root)
     {
     root->AddNestedElement(proxyElement);
     proxyElement->Delete();
     }
 
-  this->SaveSubProxyState(proxyElement);
+  if (saveSubProxies)
+    {
+    this->SaveSubProxyState(proxyElement);
+    }
+
   return proxyElement;
 }
 
@@ -2219,7 +2419,7 @@ void vtkSMProxy::SaveSubProxyState(vtkPVXMLElement* root)
 
 //---------------------------------------------------------------------------
 void vtkSMProxy::LoadSubProxyState(vtkPVXMLElement* subproxyElement, 
-  vtkSMStateLoaderBase* loader)
+  vtkSMProxyLocator* locator)
 {
   const char* name = subproxyElement->GetAttribute("name");
   if (name)
@@ -2235,7 +2435,7 @@ void vtkSMProxy::LoadSubProxyState(vtkPVXMLElement* subproxyElement,
         vtkPVXMLElement* nested = subproxyElement->GetNestedElement(cc);
         if (nested->GetName() && strcmp(nested->GetName(), "SubProxy")==0)
           {
-          proxy->LoadSubProxyState(nested, loader);
+          proxy->LoadSubProxyState(nested, locator);
           }
         }
       }
@@ -2318,7 +2518,8 @@ void vtkSMProxy::Copy(vtkSMProxy* src, const char* exceptionClass,
 
 //---------------------------------------------------------------------------
 void vtkSMProxy::ExposeSubProxyProperty(const char* subproxy_name, 
-  const char* property_name, const char* exposed_name)
+  const char* property_name, const char* exposed_name,
+  int override)
 {
   if (!subproxy_name || !property_name || !exposed_name)
     {
@@ -2330,10 +2531,13 @@ void vtkSMProxy::ExposeSubProxyProperty(const char* subproxy_name,
     this->Internals->ExposedProperties.find(exposed_name);
   if (iter != this->Internals->ExposedProperties.end())
     {
-    vtkWarningMacro("An exposed property with the name \"" << exposed_name
-      << "\" already exists. It will be replaced.");
+    if (!override)
+      {
+      vtkWarningMacro("An exposed property with the name \"" << exposed_name
+        << "\" already exists. It will be replaced.");
+      }
     }
-  
+
   vtkSMProxyInternals::ExposedPropertyInfo info;
   info.SubProxyName = subproxy_name;
   info.PropertyName = property_name;
