@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxy.h,v $
+  Module:    vtkSMProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -133,13 +133,14 @@ class vtkSMProperty;
 class vtkSMPropertyIterator;
 class vtkSMProxyManager;
 class vtkSMProxyObserver;
-class vtkSMStateLoaderBase;
+class vtkClientServerStream;
+class vtkSMProxyLocator;
 
 class VTK_EXPORT vtkSMProxy : public vtkSMObject
 {
 public:
   static vtkSMProxy* New();
-  vtkTypeRevisionMacro(vtkSMProxy, vtkSMObject);
+  vtkTypeMacro(vtkSMProxy, vtkSMObject);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -321,7 +322,6 @@ public:
   // actual objects.
   virtual void MarkAllPropertiesAsModified();
   
-//BTX
   // Description:
   // Set server ids on self and sub-proxies.
   virtual void SetServers(vtkTypeUInt32 servers);
@@ -330,7 +330,6 @@ public:
   // Return the servers.
   vtkTypeUInt32 GetServers();
 
-//ETX
   // Description:
   // Set the server connection ID on self and sub-proxies.
   virtual void SetConnectionID(vtkIdType id);
@@ -422,8 +421,40 @@ public:
   // Description:
   // Saves the state of the proxy. This state can be reloaded
   // to create a new proxy that is identical the present state of this proxy.
+  // The resulting proxy's XML hieratchy is returned, in addition if the root
+  // argument is not NULL then it's also inserted as a nested element.
+  // This call saves all a proxy's properties, including exposed properties
+  // and sub-proxies. More control is provided by the following overload.
   virtual vtkPVXMLElement* SaveState(vtkPVXMLElement* root);
 
+  // Description:
+  // Saves the state of the proxy. This state can be reloaded
+  // to create a new proxy that is identical the present state of this proxy.
+  // The resulting proxy's XML hieratchy is returned, in addition if the root
+  // argument is not NULL then it's also inserted as a nested element.
+  // The iterator argument is provided as a means for subsetting properties.
+  // The saveSubProxies argument turns on and off the saving of sub-proxies.
+  virtual vtkPVXMLElement* SaveState(
+        vtkPVXMLElement* root,
+        vtkSMPropertyIterator *iter,
+        int saveSubProxies);
+
+  // Description:
+  // Loads the proxy state from the XML element. Returns 0 on failure.
+  // \c locator is used to locate other proxies that may be referred to in the
+  // state XML (which happens in case of properties of type vtkSMProxyProperty
+  // or subclasses). If locator is NULL, then such properties are left
+  // unchanged.
+  virtual int LoadState(vtkPVXMLElement* element, vtkSMProxyLocator* locator);
+
+  // Description:
+  // Same as LoadState except that the proxy will try to undo the changes
+  // recorded in the state. Default implementation simply throws an warning.
+  // Subclasses that don't have traditional properties but fire
+  // vtkCommand::StateChangedEvent to capture state changes, need to override
+  // this method to support undo/redo.
+  virtual int RevertState(vtkPVXMLElement* element, vtkSMProxyLocator* locator);
+
   // Description:
   // Dirty means this algorithm will execute during next update.
   // This all marks all consumers as dirty.
@@ -431,8 +462,10 @@ public:
 
   // Description:
   // Returns the client side object associated with the VTKObjectID (if any).
+  // Returns 0 otherwise.
   vtkObjectBase* GetClientSideObject();
 
+//BTX
 protected:
   vtkSMProxy();
   ~vtkSMProxy();
@@ -449,11 +482,21 @@ protected:
   // Description:
   // Expose a subproxy property from the base proxy. The property with the name
   // "property_name" on the subproxy with the name "subproxy_name" is exposed 
-  // with the name "exposed_name".
+  // with the name "exposed_name". 
+  // If the overrideOK flag is set, then no warning is printed when a new 
+  // exposed property replaces a preexisting one.
   void ExposeSubProxyProperty(const char* subproxy_name, 
-    const char* property_name, const char* exposed_name);
+                              const char* property_name,
+                              const char* exposed_name,
+                              int overrideOK = 0);
+
+
+  // Description:
+  // Pushes the property. For non update-self properties, the command is
+  // collected in the stream and not pushed directly.
+  bool UpdatePropertyInternal(const char* name, bool force,
+    vtkClientServerStream& stream);
 
-//BTX
   // Description:
   // These classes have been declared as friends to minimize the
   // public interface exposed by vtkSMProxy. Each of these classes
@@ -467,16 +510,16 @@ protected:
   friend class vtkSMPart;
   friend class vtkSMProperty;
   friend class vtkSMPropertyIterator;
+  friend class vtkSMNamedPropertyIterator;
   friend class vtkSMProxyManager;
   friend class vtkSMProxyObserver;
   friend class vtkSMProxyProperty;
   friend class vtkSMProxyRegisterUndoElement;
   friend class vtkSMProxyUnRegisterUndoElement;
   friend class vtkSMSourceProxy;
-  friend class vtkSMStateLoaderBase;
+  friend class vtkSMUndoRedoStateLoader;
   // -- PVEE only
   friend class vtkWSMApplication;
-//ETX
 
   // Description:
   // Assigned by the XML parser. The name assigned in the XML
@@ -501,8 +544,7 @@ protected:
   // It is possible to set the SelfID for a proxy. However then the setter
   // has the responsiblity to ensure that the ID is going to be unique 
   // for the lifetime of the proxy. Also the SelfID can be set, only before
-  // an ID was assigned to the proxy. This is used by vtkSMStateLoaderBase
-  // and subclasses.
+  // an ID was assigned to the proxy. 
   void SetSelfID(vtkClientServerID id);
 
   // Description:
@@ -572,7 +614,7 @@ protected:
   // is modified, it invokes all observers and the observers set the
   // appropriate flags in the proxies. 
   // Changes the modified flag of a property. Used by the observers
-  void SetPropertyModifiedFlag(const char* name, int flag);
+  virtual void SetPropertyModifiedFlag(const char* name, int flag);
 
   // Description:
   // Add a property to either self (subProxyName = 0) or a sub-proxy.
@@ -595,7 +637,10 @@ protected:
 
   // Description:
   // Add a sub-proxy.
-  void AddSubProxy(const char* name, vtkSMProxy* proxy);
+  // If the overrideOK flag is set, then no warning is printed when a new 
+  // subproxy replaces a preexisting one.
+  void AddSubProxy(const char* name, vtkSMProxy* proxy,
+                   int overrideOK=0);
 
   // Description:
   // Remove a sub-proxy.
@@ -632,7 +677,7 @@ protected:
   // Description:
   // Load the subproxy state.
   void LoadSubProxyState(vtkPVXMLElement* subproxyElement, 
-    vtkSMStateLoaderBase* loader);
+    vtkSMProxyLocator* locator);
 
   // Description:
   // Called by a proxy property, this adds the property,proxy
@@ -704,18 +749,26 @@ protected:
   virtual void UpdatePipelineInformation();
 
   // Description:
-  // Updates state from an XML element. Returns 0 on failure.
-  virtual int LoadState(vtkPVXMLElement* element, vtkSMStateLoaderBase* loader);
+  // Internal UpdateVTKObjects() which collapses all server-side sends into the
+  // stream.
+  virtual void UpdateVTKObjects(vtkClientServerStream& stream);
 
+  // Description:
   // Loads the revival state for the proxy.
   // RevivalState includes the entire state saved by calling 
   // SaveState() as well additional information such as server side
   // object ID. This makes it possible to restore the servermanager state
   // while reusing server side object id.
-  virtual int LoadRevivalState(vtkPVXMLElement* revivalElement, 
-    vtkSMStateLoaderBase* loader);
+  virtual int LoadRevivalState(vtkPVXMLElement* revivalElement);
+
+  // Description:
+  // Called to update the property information on the property. It is assured
+  // that the property passed in as an argument is a self property. Both the
+  // overloads of UpdatePropertyInformation() call this method, so subclass can
+  // override this method to perform special tasks.
+  virtual void UpdatePropertyInformationInternal(vtkSMProperty*);
  
-  int CreateSubProxiesAndProperties(vtkSMProxyManager* pm, 
+  virtual int CreateSubProxiesAndProperties(vtkSMProxyManager* pm, 
     vtkPVXMLElement *element);
 
   char* Name;
