@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxyDefinitionIterator.cxx,v $
+  Module:    vtkSMProxyDefinitionIterator.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -19,7 +19,6 @@
 #include "vtkSMProxyManagerInternals.h"
 
 vtkStandardNewMacro(vtkSMProxyDefinitionIterator);
-vtkCxxRevisionMacro(vtkSMProxyDefinitionIterator, "$Revision: 1.3 $");
 
 class vtkSMProxyDefinitionIteratorInternals
 {
