@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxyManagerInternals.h,v $
+  Module:    vtkSMProxyManagerInternals.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -18,9 +18,10 @@
 
 #include "vtkPVXMLElement.h"
 #include "vtkSmartPointer.h"
+#include "vtkSMGlobalPropertiesManager.h"
 #include "vtkSMLink.h"
 #include "vtkSMProxy.h"
-#include "vtkSMProxyManagerExtension.h"
+#include "vtkSMProxySelectionModel.h"
 
 #include <vtkstd/map>
 #include <vtkstd/set>
@@ -86,6 +87,7 @@ class vtkSMProxyManagerProxyInfo : public vtkObjectBase
 public:
   vtkSmartPointer<vtkSMProxy> Proxy;
   unsigned long ModifiedObserverTag;
+  unsigned long StateChangedObserverTag;
   unsigned long UpdateObserverTag;
   unsigned long UpdateInformationObserverTag;
 
@@ -116,6 +118,7 @@ private:
   vtkSMProxyManagerProxyInfo()
     {
     this->ModifiedObserverTag = 0;
+    this->StateChangedObserverTag = 0;
     this->UpdateObserverTag = 0;
     this->UpdateInformationObserverTag = 0;
     }
@@ -127,6 +130,11 @@ private:
       this->Proxy.GetPointer()->RemoveObserver(this->ModifiedObserverTag);
       this->ModifiedObserverTag = 0;
       }
+    if (this->StateChangedObserverTag && this->Proxy.GetPointer())
+      {
+      this->Proxy.GetPointer()->RemoveObserver(this->StateChangedObserverTag);
+      this->StateChangedObserverTag = 0;
+      }
     if (this->UpdateObserverTag && this->Proxy.GetPointer())
       {
       this->Proxy.GetPointer()->RemoveObserver(this->UpdateObserverTag);
@@ -203,9 +211,16 @@ struct vtkSMProxyManagerInternals
     LinkType;
   LinkType RegisteredLinkMap;
 
-  // Data structure for proxy manager extensions.
-  typedef vtkstd::vector<vtkSmartPointer<vtkSMProxyManagerExtension> > ExtensionsType;
-  ExtensionsType Extensions;
+  // Data structure for selection models.
+  typedef vtkstd::map<vtkstd::string, vtkSmartPointer<vtkSMProxySelectionModel> >
+    SelectionModelsType;
+  SelectionModelsType SelectionModels;
+
+  // Data structure for storing GlobalPropertiesManagers.
+  typedef vtkstd::map<vtkstd::string,
+          vtkSmartPointer<vtkSMGlobalPropertiesManager> >
+            GlobalPropertiesManagersType;
+  GlobalPropertiesManagersType GlobalPropertiesManagers;
 
   // Helper method to retrieve the proxy element.
   vtkPVXMLElement* GetProxyElement(const char* groupName, const char* proxyName)
@@ -231,14 +246,6 @@ struct vtkSMProxyManagerInternals
         }
       }
 
-    vtkSMProxyManagerInternals::ExtensionsType::iterator extensionsIter;
-    for (extensionsIter = this->Extensions.begin();
-      extensionsIter != this->Extensions.end(); ++extensionsIter)
-      {
-      element = extensionsIter->GetPointer()->GetProxyElement(
-        groupName, proxyName, element);
-      }
-
     return element;
     }
 };
