@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxyManagerReviver.cxx,v $
+  Module:    vtkSMProxyManagerReviver.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -16,7 +16,6 @@
 
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkSMProxyManagerReviver, "$Revision: 1.1 $");
 //-----------------------------------------------------------------------------
 vtkSMProxyManagerReviver::vtkSMProxyManagerReviver()
 {
