@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMProxyProperty.h,v $
+  Module:    vtkSMProxyProperty.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -56,7 +56,7 @@ class VTK_EXPORT vtkSMProxyProperty : public vtkSMProperty
 {
 public:
   static vtkSMProxyProperty* New();
-  vtkTypeRevisionMacro(vtkSMProxyProperty, vtkSMProperty);
+  vtkTypeMacro(vtkSMProxyProperty, vtkSMProperty);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -151,6 +151,14 @@ public:
   // used while copying over the values from the two proxy properties.
   virtual void DeepCopy(vtkSMProperty* src, const char* exceptionClass, 
     int proxyPropertyCopyFlag);
+
+  // Description:
+  // If RepeatCommand is true, the command is invoked multiple times,
+  // once for every proxy.
+  vtkGetMacro(RepeatCommand, int);
+  vtkSetMacro(RepeatCommand, int);
+  vtkBooleanMacro(RepeatCommand, int);
+
 protected:
   vtkSMProxyProperty();
   ~vtkSMProxyProperty();
@@ -178,7 +186,7 @@ protected:
   // Description:
   // Updates state from an XML element. Returns 0 on failure.
   virtual int LoadState(vtkPVXMLElement* element, 
-    vtkSMStateLoaderBase* loader, int loadLastPushedValues=0);
+    vtkSMProxyLocator* loader, int loadLastPushedValues=0);
 
   // Description:
   // Saves the state of the object in XML format. 
@@ -187,6 +195,13 @@ protected:
   // undo/redo state.
   virtual void ChildSaveState(vtkPVXMLElement* parent, int saveLastPushedValues);
 
+  // Description:
+  // Called by ChildSaveState to save the XML for every proxy. Gives the
+  // subclass an opportunity to add additional attributes to the XML element
+  // (such as output port info).
+  virtual vtkPVXMLElement* SaveProxyElementState(
+    unsigned int idx, bool use_previous_proxies);
+
   // Description:
   // Previous proxies are used by the ProxyProperty internally.  This is a
   // collection of proxies to whcih the owner proxy of this property get
@@ -241,8 +256,6 @@ protected:
   virtual int ReadXMLAttributes(vtkSMProxy* parent, 
                                 vtkPVXMLElement* element);
 
-  vtkSetMacro(RepeatCommand, int);
-  vtkGetMacro(RepeatCommand, int);
   int RepeatCommand;
 
   // When set to true, the property will push a NULL i.e. 0 when there are no
