@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMRenderViewHelper.cxx,v $
+  Module:    vtkSMRenderViewHelper.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -18,7 +18,6 @@
 #include "vtkSMRenderViewProxy.h"
 
 vtkStandardNewMacro(vtkSMRenderViewHelper);
-vtkCxxRevisionMacro(vtkSMRenderViewHelper, "$Revision: 1.2 $");
 //----------------------------------------------------------------------------
 vtkSMRenderViewHelper::vtkSMRenderViewHelper()
 {
