@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMRenderViewProxy.cxx,v $
+  Module:    vtkSMRenderViewProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -32,6 +32,7 @@
 #include "vtkInformation.h"
 #include "vtkInformationIntegerKey.h"
 #include "vtkInstantiator.h"
+#include "vtkMultiProcessController.h"
 #include "vtkObjectFactory.h"
 #include "vtkPointData.h"
 #include "vtkProcessModuleConnectionManager.h"
@@ -42,16 +43,20 @@
 #include "vtkPVOpenGLExtensionsInformation.h"
 #include "vtkPVOptions.h"
 #include "vtkPVServerInformation.h"
-#include "vtkPVVisibleCellSelector.h"
+#include "vtkPVXMLElement.h"
 #include "vtkRendererCollection.h"
 #include "vtkRenderer.h"
 #include "vtkRenderWindow.h"
 #include "vtkSelection.h"
+#include "vtkSelectionNode.h"
 #include "vtkSmartPointer.h"
 #include "vtkSMDataRepresentationProxy.h"
 #include "vtkSMDoubleVectorProperty.h"
+#include "vtkSMEnumerationDomain.h"
+#include "vtkSMHardwareSelector.h"
 #include "vtkSMInputProperty.h"
 #include "vtkSMIntVectorProperty.h"
+#include "vtkSMPropertyHelper.h"
 #include "vtkSMPropertyIterator.h"
 #include "vtkSMPropRepresentationProxy.h"
 #include "vtkSMProxyManager.h"
@@ -89,7 +94,6 @@ inline bool SetIntVectorProperty(vtkSMProxy* proxy, const char* pname,
 }
 
 //-----------------------------------------------------------------------------
-vtkCxxRevisionMacro(vtkSMRenderViewProxy, "$Revision: 1.75 $");
 vtkStandardNewMacro(vtkSMRenderViewProxy);
 
 vtkInformationKeyMacro(vtkSMRenderViewProxy, LOD_RESOLUTION, Integer);
@@ -173,6 +177,11 @@ vtkSMRenderViewProxy::~vtkSMRenderViewProxy()
 vtkSMRepresentationStrategy* vtkSMRenderViewProxy::NewStrategyInternal(
   int dataType)
 {
+  if (this->NewStrategyHelper)
+    {
+    return this->NewStrategyHelper->NewStrategyInternal(dataType);
+    }
+
   vtkSMProxyManager* pxm = vtkSMObject::GetProxyManager();
   vtkSMRepresentationStrategy* strategy = 0;
 
@@ -352,12 +361,20 @@ void vtkSMRenderViewProxy::EndCreateVTKObjects()
     {
     SetIntVectorProperty(this->RenderWindowProxy, "StereoCapableWindow", 1);
     SetIntVectorProperty(this->RenderWindowProxy, "StereoRender", 1);
+    vtkSMEnumerationDomain* domain = vtkSMEnumerationDomain::SafeDownCast(
+      this->RenderWindowProxy->GetProperty("StereoType")->GetDomain("enum"));
+    if (domain && domain->HasEntryText(pvoptions->GetStereoType()))
+      {
+      SetIntVectorProperty(this->RenderWindowProxy, "StereoType",
+        domain->GetEntryValueForText(pvoptions->GetStereoType()));
+      }
     }
 
   SetIntVectorProperty(this->Renderer2DProxy, "Erase", 0);
   SetIntVectorProperty(this->Renderer2DProxy, "Layer", 2);
 
-  SetIntVectorProperty(this->RendererProxy, "DepthPeeling", 1);
+  // This property is now exposed, hence it's not safe to modify it.
+  //SetIntVectorProperty(this->RendererProxy, "DepthPeeling", 1);
   SetIntVectorProperty(this->RenderWindowProxy, "NumberOfLayers", 3);
    
   this->Connect(this->RendererProxy, this->RenderWindowProxy, "Renderer");
@@ -528,7 +545,6 @@ void vtkSMRenderViewProxy::BeginInteractiveRender()
   // size information.
   this->SetUseLOD(this->GetLODDecision());
 
-  this->GetRenderer()->ResetCameraClippingRange();
   this->Superclass::BeginInteractiveRender();
 }
 
@@ -545,11 +561,6 @@ void vtkSMRenderViewProxy::EndInteractiveRender()
 void vtkSMRenderViewProxy::BeginStillRender()
 {
   vtkRenderWindow *renWindow = this->GetRenderWindow(); 
-  // Still Render can get called some funky ways.
-  // Interactive renders get called through the PVInteractorStyles
-  // which cal ResetCameraClippingRange on the Renderer.
-  // We could convert them to call a method on the module directly ...
-  this->GetRenderer()->ResetCameraClippingRange();
   renWindow->SetDesiredUpdateRate(0.002);
 
   this->SetUseLOD(false);
@@ -571,10 +582,22 @@ void vtkSMRenderViewProxy::PerformRender()
   if ( this->MeasurePolygonsPerSecond )
     {
     this->RenderTimer->StartTimer();
-    }
+    } 
 
-  vtkRenderWindow *renWindow = this->GetRenderWindow(); 
-  renWindow->Render();
+  this->GetRenderer()->ResetCameraClippingRange();
+
+  vtkClientServerStream stream;
+  stream << vtkClientServerStream::Invoke
+         << this->RenderWindowProxy->GetID()
+         << "Render"
+         << vtkClientServerStream::End;
+  vtkProcessModule::GetProcessModule()->SendStream(
+    this->ConnectionID,
+    vtkProcessModule::CLIENT,
+    stream);
+
+  //vtkRenderWindow *renWindow = this->GetRenderWindow(); 
+  //renWindow->Render();
 
   if ( this->MeasurePolygonsPerSecond )
     {
@@ -773,6 +796,15 @@ vtkPVClientServerIdCollectionInformation* vtkSMRenderViewProxy
 //-----------------------------------------------------------------------------
 void vtkSMRenderViewProxy::ResetCamera()
 {
+  // Dont' call UpdateAllRepresentations() explicitly, since
+  // UpdateAllRepresentations() results in delivering the data to rendering
+  // nodes as well. All we want is for the pipeline to be updated until enough
+  // information about the bounds can be known. That's already done by
+  // vtkSMRepresentationProxy::GetBounds(). So we don't need to call update on
+  // representations explicitly here.
+  // This fixes BUG #9055.
+  // this->UpdateAllRepresentations();
+
   double bds[6];
   this->ComputeVisiblePropBounds(bds);
   if (bds[0] <= bds[1] && bds[2] <= bds[3] && bds[4] <= bds[5])
@@ -1044,7 +1076,17 @@ vtkImageData* vtkSMRenderViewProxy::CaptureWindow(int magnification)
   w2i->SetMagnification(magnification);
   w2i->ReadFrontBufferOff();
   w2i->ShouldRerenderOff();
-  w2i->Update();
+
+  // BUG #8715: We go through this indirection since the active connection needs
+  // to be set during update since it may request re-renders if magnification >1.
+  vtkClientServerStream stream;
+  stream << vtkClientServerStream::Invoke
+         << w2i << "Update"
+         << vtkClientServerStream::End;
+  vtkProcessModule::GetProcessModule()->SendStream(
+    this->ConnectionID,
+    vtkProcessModule::CLIENT,
+    stream);
 
   vtkImageData* capture = vtkImageData::New();
   capture->ShallowCopy(w2i->GetOutput());
@@ -1074,7 +1116,8 @@ vtkImageData* vtkSMRenderViewProxy::CaptureWindow(int magnification)
         }
       }
 
-    if (invalid_image)
+    if (invalid_image && 
+      vtkProcessModule::GetProcessModule()->GetNumberOfLocalPartitions() == 1)
       {
       // free up current image.
       capture->Delete();
@@ -1119,30 +1162,9 @@ int vtkSMRenderViewProxy::WriteImage(const char* filename,
     return vtkErrorCode::UnknownError;
     }
 
-  vtkObject* object = vtkInstantiator::CreateInstance(writerName);
-  if (!object)
-    {
-    vtkErrorMacro("Failed to create Writer " << writerName);
-    return vtkErrorCode::UnknownError;
-    }
-  vtkImageWriter* writer = vtkImageWriter::SafeDownCast(object);
-  if (!writer)
-    {
-    vtkErrorMacro("Object is not a vtkImageWriter: " << object->GetClassName());
-    object->Delete();
-    return vtkErrorCode::UnknownError;
-    }
-
-  vtkImageData* shot = this->CaptureWindow(magnification);
-  writer->SetInput(shot);
-  writer->SetFileName(filename);
-  writer->Write();
-  int error_code = writer->GetErrorCode();
-
-  writer->Delete();
-  shot->Delete();
-  
-  return error_code;
+  vtkSmartPointer<vtkImageData> shot;
+  shot.TakeReference(this->CaptureWindow(magnification));
+  return vtkSMUtilities::SaveImageOnProcessZero(shot, filename, writerName);
 }
 
 //-----------------------------------------------------------------------------
@@ -1268,18 +1290,7 @@ const char* vtkSMRenderViewProxy::IsSelectVisibleCellsAvailable()
 //-----------------------------------------------------------------------------
 const char* vtkSMRenderViewProxy::IsSelectVisiblePointsAvailable()
 {
-  const char* msg = this->IsSelectVisibleCellsAvailable();
-  if (msg)
-    {
-    return msg;
-    }
-
-  vtkRenderWindow *rwin = this->GetRenderWindow();
-  if (!rwin || !rwin->GetStencilCapable())
-    {
-    return "Stencil buffers are not supported.";
-    }
-  return NULL;
+  return this->IsSelectVisibleCellsAvailable();
 }
 
 //-----------------------------------------------------------------------------
@@ -1287,19 +1298,18 @@ vtkSelection* vtkSMRenderViewProxy::NewSelectionForProp(
   vtkSelection* selection, vtkClientServerID propId)
 {
   vtkSelection* newSelection = vtkSelection::New();
-  newSelection->GetProperties()->Copy(selection->GetProperties(), 0);
-  unsigned int numChildren = selection->GetNumberOfChildren();
-  for(unsigned int i=0; i<numChildren; i++)
+  unsigned int numNodes = selection->GetNumberOfNodes();
+  for(unsigned int i=0; i<numNodes; i++)
     {
-    vtkSelection* child = selection->GetChild(i);
-    vtkInformation* properties = child->GetProperties();
-    if (properties->Has(vtkSelection::PROP_ID()) &&
-      properties->Get(vtkSelection::PROP_ID()) == (int)propId.ID)
+    vtkSelectionNode* node = selection->GetNode(i);
+    vtkInformation* properties = node->GetProperties();
+    if (properties->Has(vtkSelectionNode::PROP_ID()) &&
+      properties->Get(vtkSelectionNode::PROP_ID()) == (int)propId.ID)
       {
-      vtkSelection* newChildSelection = vtkSelection::New();
-      newChildSelection->ShallowCopy(child);
-      newSelection->AddChild(newChildSelection);
-      newChildSelection->Delete();
+      vtkSelectionNode* newNode = vtkSelectionNode::New();
+      newNode->ShallowCopy(node);
+      newSelection->AddNode(newNode);
+      newNode->Delete();
       }
     }
 
@@ -1311,20 +1321,20 @@ static void vtkSMRenderViewProxyShrinkSelection(vtkSelection* sel)
 {
   vtkstd::map<int, int> propToPixelCount;
   
-  unsigned int numChildren = sel->GetNumberOfChildren();
+  unsigned int numNodes = sel->GetNumberOfNodes();
   unsigned int cc;
 
   int choosenPropId = -1;
   int maxPixels = -1;
-  for (cc=0; cc < numChildren; cc++)
+  for (cc=0; cc < numNodes; cc++)
     {
-    vtkSelection* child = sel->GetChild(cc);
-    vtkInformation* properties = child->GetProperties();
-    if (properties->Has(vtkSelection::PIXEL_COUNT()) && 
-      properties->Has(vtkSelection::PROP_ID()))
+    vtkSelectionNode* node = sel->GetNode(cc);
+    vtkInformation* properties = node->GetProperties();
+    if (properties->Has(vtkSelectionNode::PIXEL_COUNT()) && 
+      properties->Has(vtkSelectionNode::PROP_ID()))
       {
-      int numPixels = properties->Get(vtkSelection::PIXEL_COUNT());
-      int prop_id = properties->Get(vtkSelection::PROP_ID());
+      int numPixels = properties->Get(vtkSelectionNode::PIXEL_COUNT());
+      int prop_id = properties->Get(vtkSelectionNode::PROP_ID());
       if (propToPixelCount.find(prop_id) != propToPixelCount.end())
         {
         numPixels += propToPixelCount[prop_id];
@@ -1339,24 +1349,24 @@ static void vtkSMRenderViewProxyShrinkSelection(vtkSelection* sel)
       }
     }
 
-  vtkstd::vector<vtkSmartPointer<vtkSelection> > choosenChildren;
+  vtkstd::vector<vtkSmartPointer<vtkSelectionNode> > choosenNodes;
   if (choosenPropId != -1)
     {
-    for (cc=0; cc < numChildren; cc++)
+    for (cc=0; cc < numNodes; cc++)
       {
-      vtkSelection* child = sel->GetChild(cc);
-      vtkInformation* properties = child->GetProperties();
-      if (properties->Has(vtkSelection::PROP_ID()) && 
-        properties->Get(vtkSelection::PROP_ID()) == choosenPropId)
+      vtkSelectionNode* node = sel->GetNode(cc);
+      vtkInformation* properties = node->GetProperties();
+      if (properties->Has(vtkSelectionNode::PROP_ID()) && 
+        properties->Get(vtkSelectionNode::PROP_ID()) == choosenPropId)
         {
-        choosenChildren.push_back(child);
+        choosenNodes.push_back(node);
         }
       }
     }
-  sel->RemoveAllChildren();
-  for (cc=0; cc <choosenChildren.size(); cc++)
+  sel->RemoveAllNodes();
+  for (cc=0; cc <choosenNodes.size(); cc++)
     {
-    sel->AddChild(choosenChildren[cc]);
+    sel->AddNode(choosenNodes[cc]);
     }
 }
 
@@ -1489,14 +1499,16 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
     worldP[index*4+2], worldP[index*4+3]);
 
   vtkSelection* frustumSel = vtkSelection::New();
-  frustumSel->GetProperties()->Set(
-    vtkSelection::CONTENT_TYPE(), vtkSelection::FRUSTUM);
+  vtkSelectionNode* frustumNode = vtkSelectionNode::New();
+  frustumSel->AddNode(frustumNode);
+  frustumNode->GetProperties()->Set(
+    vtkSelectionNode::CONTENT_TYPE(), vtkSelectionNode::FRUSTUM);
   if (ofPoints)
     {
-    frustumSel->GetProperties()->Set(
-      vtkSelection::FIELD_TYPE(), vtkSelection::POINT);
+    frustumNode->GetProperties()->Set(
+      vtkSelectionNode::FIELD_TYPE(), vtkSelectionNode::POINT);
     }
-  frustumSel->SetSelectionList(frustcorners);
+  frustumNode->SetSelectionList(frustcorners);
   frustcorners->Delete();
 
   // 2) Figure out which representation is "selected".
@@ -1506,8 +1518,6 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
   double bounds[6];
 
   vtkSelection* frustumParent = vtkSelection::New();
-  frustumParent->GetProperties()->Set(
-    vtkSelection::CONTENT_TYPE(), vtkSelection::SELECTIONS);
 
   // Now we just use the first selected representation,
   // until we have other mechanisms to select one.
@@ -1523,6 +1533,12 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
       {
       continue;
       }
+    if (repr->GetProperty("Pickable") &&
+      vtkSMPropertyHelper(repr, "Pickable").GetAsInt() == 0)
+      {
+      // skip non-pickable representations.
+      continue;
+      }
     vtkPVDataInformation* datainfo = repr->GetRepresentedDataInformation(true);
     if (!datainfo)
       {
@@ -1532,7 +1548,7 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
 
     if(FrustumExtractor->OverallBoundsTest(bounds))
       {
-      frustumParent->AddChild(frustumSel);
+      frustumParent->AddNode(frustumNode);
 
       vtkSMProxy* selectionSource = repr->ConvertSelection(frustumParent);
       if (!selectionSource)
@@ -1556,6 +1572,7 @@ bool vtkSMRenderViewProxy::SelectFrustum(unsigned int x0,
     }
 
   frustumSel->Delete();
+  frustumNode->Delete();
   frustumParent->Delete();
   FrustumExtractor->Delete();
   return true;
@@ -1568,7 +1585,7 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
   if (!this->IsSelectionAvailable())
     {
     vtkSelection *selection = vtkSelection::New();
-    selection->Clear();
+    selection->Initialize();
     return selection;
     }
 
@@ -1602,7 +1619,7 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
       {
       continue;
       }
-    vtkIdType numCells = gi->GetNumberOfCells();
+    vtkIdType numCells = ofPoints? gi->GetNumberOfPoints() : gi->GetNumberOfCells();
     if (numCells > maxNumCells)
       {
       maxNumCells = numCells;
@@ -1610,13 +1627,11 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     }
   iter->Delete();
 
-  vtkIdType needs_2_passes = (maxNumCells+1)>>24;//more than 2^24-1 cells
-  vtkIdType needs_3_passes = needs_2_passes>>24; //more than 2^48-1
-
   vtkSMProxyManager* proxyManager = vtkSMObject::GetProxyManager();  
-  vtkSMProxy *vcsProxy = proxyManager->NewProxy("PropPickers", "PVVisibleCellSelector");
+  vtkSMHardwareSelector *vcsProxy = vtkSMHardwareSelector::SafeDownCast(
+      proxyManager->NewProxy("PropPickers", "HardwareSelector"));
   vcsProxy->SetConnectionID(this->ConnectionID);
-  vcsProxy->SetServers(this->Servers);
+  vcsProxy->SetServers(vtkProcessModule::CLIENT|vtkProcessModule::RENDER_SERVER);
 
   //don't let the RenderSyncManager control back/front buffer swapping so that
   //we can do it here instead.
@@ -1633,41 +1648,15 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     renderSyncManager->UpdateVTKObjects();
     }
 
-  //how we get access to the renderers
-  vtkSMProxyProperty *setRendererMethod = vtkSMProxyProperty::SafeDownCast(
-    vcsProxy->GetProperty("SetRenderer"));
-  setRendererMethod->AddProxy(this->RendererProxy);
+  // Set default property values for the selector.
+  int area[4] = {x0, y0, x1, y1};
+  vtkSMPropertyHelper(vcsProxy, "Renderer").Set(this->RendererProxy);
+  vtkSMPropertyHelper(vcsProxy, "Area").Set(area, 4);
+  vtkSMPropertyHelper(vcsProxy, "FieldAssociation").Set(ofPoints? 0 : 1);
+  vtkSMPropertyHelper(vcsProxy, "NumberOfProcesses").Set(numProcessors);
+  vtkSMPropertyHelper(vcsProxy, "NumberOfIDs").Set(maxNumCells);
   vcsProxy->UpdateVTKObjects();   
 
-  //how we put the renderers into selection mode
-  vtkSMIntVectorProperty *setModeMethod = vtkSMIntVectorProperty::SafeDownCast(
-    vcsProxy->GetProperty("SetSelectMode"));
-  vtkSMProperty *setPIdMethod = vcsProxy->GetProperty("LookupProcessorId");
-
-  //I'm using the auto created PVVisCellSelectors in the vcsProxy above just
-  //to set the SelectionMode of the remote renderers.
-  //I use this one below to convert the composited images that arrive here
-  //into a selection.
-  vtkPVVisibleCellSelector *pti = vtkPVVisibleCellSelector::New();
-  pti->SetRenderer(this->GetRenderer());
-  pti->SetArea(x0,y0,x1,y1);
-  pti->GetArea(x0,y0,x1,y1);
-
-  //Use the back buffer
-  int usefrontbuf = 0;
-  if (!usefrontbuf)
-    {
-    this->GetRenderWindow()->SwapBuffersOff();
-    }
-
-  //Set background to black to indicate misses.
-  //vtkRenderer::UpdateGeometryForSelection does this also, but in that case
-  //iceT ignores it. Thus I duplicate the same effect here.
-  double origBG[3];
-  this->GetRenderer()->GetBackground(origBG);
-  double black[3] = {0.0,0.0,0.0};
-  this->SetBackgroundColorCM(black);
-
   //don't draw the scalar bar, text annotation, orientation axes
   vtkRendererCollection *rcol = this->RenderWindow->GetRenderers();
   int numlayers = rcol->GetNumberOfItems();
@@ -1705,52 +1694,7 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     this->ForceTriStripUpdate = 0;
     }
 
-  if (ofPoints)
-    {
-    pti->SetDoVertices(1);
-    }
-
-  unsigned char *buf;  
-  for (int p = 0; p < 6; p++)
-    {
-    if ((p==0) && (numProcessors==1))
-      {
-      p++;
-      }
-    if ((p==2) && (needs_3_passes==0))
-      {
-      p++;
-      }
-    if ((p==3) && (needs_2_passes==0))
-      {
-      p++;
-      }
-    if ((p==5) && (!ofPoints))
-      {
-      break;
-      }
-    //put into a selection mode
-    setModeMethod->SetElements1(p+1);
-    if (p==0)
-      {
-      setPIdMethod->Modified();
-      }
-    vcsProxy->UpdateVTKObjects();   
-
-    //draw
-    this->StillRender();  
-
-    //get the intermediate results
-    //renderwindow allocates this buffer
-    buf = this->GetRenderWindow()->GetRGBACharPixelData(x0,y0,x1,y1,usefrontbuf); 
-    //pti will deallocate the buffer when it is deleted
-    pti->SavePixelBuffer(p, buf); 
-    }
-  
-  //restore original rendering state
-  //clear selection mode to resume normal rendering
-  setModeMethod->SetElements1(0);
-  vcsProxy->UpdateVTKObjects();   
+  vtkSelection* selection = vcsProxy->Select(); 
 
   //Turn stripping back on if we had turned it off
   if (use_strips)
@@ -1774,12 +1718,6 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     }
   delete[] renOldVis;
 
-  this->SetBackgroundColorCM(origBG);
-  if (!usefrontbuf)
-    {
-    this->GetRenderWindow()->SwapBuffersOn();
-    }
-
   if (setAllowBuffSwap != NULL)
     {
     //let the RenderSyncManager control back/front buffer swapping
@@ -1787,15 +1725,7 @@ vtkSelection* vtkSMRenderViewProxy::SelectVisibleCells(unsigned int x0,
     renderSyncManager->UpdateVTKObjects();
     }
   
-  //convert the intermediate results into  the selection data structure 
-  pti->ComputeSelectedIds();
-  vtkSelection *selection = vtkSelection::New();
-  pti->GetSelectedIds(selection);
-
-  //cleanup
-  pti->Delete();
   vcsProxy->Delete();
-
   return selection;
 }
 
@@ -1861,6 +1791,33 @@ vtkSMRepresentationProxy* vtkSMRenderViewProxy::CreateDefaultRepresentation(
       pxm->NewProxy("representations", "GeometryRepresentation"));
     }
 
+  vtkPVXMLElement* hints = source->GetHints();
+  if (hints)
+    {
+    // If the source has an hint as follows, then it's a text producer and must
+    // be is display-able.
+    //  <Hints>
+    //    <OutputPort name="..." index="..." type="text" />
+    //  </Hints>
+
+    unsigned int numElems = hints->GetNumberOfNestedElements(); 
+    for (unsigned int cc=0; cc < numElems; cc++)
+      {
+      int index;
+      vtkPVXMLElement* child = hints->GetNestedElement(cc);
+      if (child->GetName() &&
+        strcmp(child->GetName(), "OutputPort") == 0 &&
+        child->GetScalarAttribute("index", &index) &&
+        index == opport &&
+        child->GetAttribute("type") &&
+        strcmp(child->GetAttribute("type"), "text") == 0)
+        {
+        return vtkSMRepresentationProxy::SafeDownCast(
+          pxm->NewProxy("representations", "TextSourceRepresentation"));
+        }
+      }
+    }
+
   return 0;
 }
 
@@ -1884,6 +1841,10 @@ const char* vtkSMRenderViewProxy::GetSuggestedViewType(vtkIdType connectionID)
         // tiled-display
         renderViewName = "IceTMultiDisplayRenderView";
         }
+      else if (server_info->GetNumberOfMachines())
+        {
+        renderViewName = "CaveRenderView";
+        }
       else
         {
         // regular client-server
