@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMRenderViewProxy.h,v $
+  Module:    vtkSMRenderViewProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -40,7 +40,7 @@ class VTK_EXPORT vtkSMRenderViewProxy : public vtkSMViewProxy
 {
 public:
   static vtkSMRenderViewProxy* New();
-  vtkTypeRevisionMacro(vtkSMRenderViewProxy, vtkSMViewProxy);
+  vtkTypeMacro(vtkSMRenderViewProxy, vtkSMViewProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -74,9 +74,11 @@ public:
   vtkGetObjectMacro(Renderer2D, vtkRenderer);
   vtkGetObjectMacro(RenderWindow, vtkRenderWindow);
   vtkGetObjectMacro(Interactor, vtkPVGenericRenderWindowInteractor);
+  vtkGetObjectMacro(ActiveCamera, vtkCamera);
+
   vtkGetObjectMacro(RendererProxy, vtkSMProxy);
   vtkGetObjectMacro(Renderer2DProxy, vtkSMProxy);
-  vtkGetObjectMacro(ActiveCamera, vtkCamera);
+  vtkGetObjectMacro(RenderWindowProxy, vtkSMProxy);
   vtkGetObjectMacro(InteractorProxy, vtkSMProxy);
   
   // Description:
@@ -144,6 +146,16 @@ public:
   // Called when saving server manager state.
   // Overridden to SynchronizeCameraProperties before saving the properties.
   virtual vtkPVXMLElement* SaveState(vtkPVXMLElement* root);
+  
+  // Description:
+  // Un-hide the method. See vtkSMProxy for description.
+    virtual vtkPVXMLElement* SaveState(
+        vtkPVXMLElement* root,
+        vtkSMPropertyIterator *iter,
+        int saveSubProxies)
+      {
+      return this->Superclass::SaveState(root,iter,saveSubProxies);
+      }
 
   // Description:
   // Returns an image data that contains a "screenshot" of the window.
@@ -182,7 +194,7 @@ public:
   // Checks if color depth is sufficient to support selection.
   // If not, will return 0 and any calls to SelectVisibleCells will 
   // quietly return an empty selection.
-  bool IsSelectionAvailable();
+  virtual bool IsSelectionAvailable();
 
   //BTX
   // Description:
@@ -243,8 +255,6 @@ public:
   // Returns a new proxy.
   virtual vtkSMRepresentationProxy* CreateDefaultRepresentation(
     vtkSMProxy*, int opport);
-  virtual vtkSMRepresentationProxy* CreateDefaultRepresentation(vtkSMProxy* proxy)
-    { return this->Superclass::CreateDefaultRepresentation(proxy); }
 
   // Description:
   // Generally each view type is different class of view eg. bar char view, line
@@ -259,6 +269,15 @@ public:
   // Overridden to choose the correct type of render view.
  virtual const char* GetSuggestedViewType(vtkIdType connectionID);
 
+  // Description:
+  // Called by AddRepresentation(). Subclasses can override to add 
+  // observers etc. //DDM TODO Do I have to make this public?
+  virtual void AddRepresentationInternal(vtkSMRepresentationProxy* rep);
+
+  // Description:
+  // Reset the camera clipping range.
+  void ResetCameraClippingRange();
+
 //BTX
 protected:
   vtkSMRenderViewProxy();
@@ -272,11 +291,6 @@ protected:
   // Called at the end of CreateVTKObjects().
   virtual void EndCreateVTKObjects();
 
-  // Description:
-  // Called by AddRepresentation(). Subclasses can override to add 
-  // observers etc.
-  virtual void AddRepresentationInternal(vtkSMRepresentationProxy* rep);
-
   // Description:
   // Creates a new vtkSMRepresentationStrategy subclass based on the type
   // requested.
@@ -309,10 +323,6 @@ protected:
   // each render.
   void SynchronizeRenderers();
 
-  // Description:
-  // Reset the camera clipping range.
-  void ResetCameraClippingRange();
-
   // Description:
   // Returns a new selection consisting of all the selections with the given
   // prop id in the surfaceSelection.
@@ -348,7 +358,8 @@ protected:
   int ForceTriStripUpdate;
   int UseImmediateMode;
   double LODThreshold;
-  
+
+public:  
   // Description:
   // Method called before/after Still Render is called.
   // Used to perform some every-still-render-setup actions.
@@ -366,13 +377,17 @@ protected:
   // render.
   virtual void PerformRender();
 
+  // Description:
+  // For internal use only. 
+  void CalculatePolygonsPerSecond(double time); //DDM TODO Do I have to make this public?
+
+protected:
   vtkTimerLog *RenderTimer;
   double LastPolygonsPerSecond;
   double MaximumPolygonsPerSecond;
   double AveragePolygonsPerSecond;
   double AveragePolygonsPerSecondAccumulated;
   vtkIdType AveragePolygonsPerSecondCount;
-  void CalculatePolygonsPerSecond(double time);
   int MeasurePolygonsPerSecond;
   int UseOffscreenRenderingForScreenshots;
   bool LightKitAdded;
