@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMRepresentationProxy.cxx,v $
+  Module:    vtkSMRepresentationProxy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -20,7 +20,6 @@
 #include "vtkInformation.h"
 #include "vtkPVDataInformation.h"
 
-vtkCxxRevisionMacro(vtkSMRepresentationProxy, "$Revision: 1.10 $");
 vtkCxxSetObjectMacro(vtkSMRepresentationProxy, ViewInformation, vtkInformation);
 //----------------------------------------------------------------------------
 vtkSMRepresentationProxy::vtkSMRepresentationProxy()
