@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMSelectionHelper.h,v $
+  Module:    vtkSMSelectionHelper.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -23,6 +23,7 @@
 #include "vtkSMObject.h"
 
 class vtkSelection;
+class vtkSelectionNode;
 class vtkSMProxy;
 class vtkSMSourceProxy;
 
@@ -30,7 +31,7 @@ class VTK_EXPORT vtkSMSelectionHelper : public vtkSMObject
 {
 public:
   static vtkSMSelectionHelper* New();
-  vtkTypeRevisionMacro(vtkSMSelectionHelper, vtkSMObject);
+  vtkTypeMacro(vtkSMSelectionHelper, vtkSMObject);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -80,7 +81,11 @@ public:
   // Description:
   // Updates output to be a combination of (input | output) if the two selection
   // sources are mergeable. Returns true if merge successful.
-  static bool MergeSelection(vtkSMSourceProxy* output, vtkSMSourceProxy* input);
+  // dataSource and outputport are needed if a conversion is needed to make the
+  // input expandable to the type of the output.
+  static bool MergeSelection(
+    vtkSMSourceProxy* output, vtkSMSourceProxy* input,
+    vtkSMSourceProxy* dataSource, int outputport);
 
 protected:
   vtkSMSelectionHelper() {};
@@ -95,7 +100,7 @@ private:
   void operator=(const vtkSMSelectionHelper&); // Not implemented.
 
   static vtkSMProxy* NewSelectionSourceFromSelectionInternal(
-    vtkIdType connectionId, vtkSelection* selection, vtkSMProxy* selSource=0);
+    vtkIdType connectionId, vtkSelectionNode* selection, vtkSMProxy* selSource=0);
 
   static vtkSMProxy* ConvertInternal(
     vtkSMSourceProxy* inSource, vtkSMSourceProxy* dataSource,
