@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMSimpleParallelStrategy.cxx,v $
+  Module:    vtkSMSimpleParallelStrategy.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -21,24 +21,26 @@
 #include "vtkProcessModule.h"
 #include "vtkSMDoubleVectorProperty.h"
 #include "vtkSMIceTMultiDisplayRenderViewProxy.h"
-#include "vtkSMIntVectorProperty.h"
 #include "vtkSMSourceProxy.h"
+#include "vtkSMPropertyHelper.h"
 
 vtkStandardNewMacro(vtkSMSimpleParallelStrategy);
-vtkCxxRevisionMacro(vtkSMSimpleParallelStrategy, "$Revision: 1.23 $");
 //----------------------------------------------------------------------------
 vtkSMSimpleParallelStrategy::vtkSMSimpleParallelStrategy()
 {
-  this->PreCollectUpdateSuppressor = 0;
+  this->PostCollectUpdateSuppressor = 0;
   this->Collect = 0;
 
-  this->PreCollectUpdateSuppressorLOD = 0;
+  this->PostCollectUpdateSuppressorLOD = 0;
   this->CollectLOD = 0;
 
   this->UseCompositing = false;
 
   this->LODClientRender = false;
   this->LODClientCollect = true;
+
+  this->CollectedDataValid = false;
+  this->CollectedLODDataValid = false;
 }
 
 //----------------------------------------------------------------------------
@@ -50,24 +52,29 @@ vtkSMSimpleParallelStrategy::~vtkSMSimpleParallelStrategy()
 void vtkSMSimpleParallelStrategy::BeginCreateVTKObjects()
 {
   this->Superclass::BeginCreateVTKObjects();
+  this->UpdateSuppressor->SetServers(vtkProcessModule::DATA_SERVER);
+  if (this->UpdateSuppressorLOD)
+    {
+    this->UpdateSuppressorLOD->SetServers(vtkProcessModule::DATA_SERVER);
+    }
 
-  this->PreCollectUpdateSuppressor =
-    vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("PreCollectUpdateSuppressor"));
   this->Collect = 
     vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("Collect"));
+  this->PostCollectUpdateSuppressor =
+    vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("PostCollectUpdateSuppressor")); 
 
-  this->PreCollectUpdateSuppressorLOD =
-    vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("PreCollectUpdateSuppressorLOD"));
   this->CollectLOD = 
     vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("CollectLOD"));
+  this->PostCollectUpdateSuppressorLOD =
+    vtkSMSourceProxy::SafeDownCast(this->GetSubProxy("PostCollectUpdateSuppressorLOD"));
 
-  this->PreCollectUpdateSuppressor->SetServers(vtkProcessModule::DATA_SERVER);
   this->Collect->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
+  this->PostCollectUpdateSuppressor->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
 
   if (this->CollectLOD)
     {
-    this->PreCollectUpdateSuppressorLOD->SetServers(vtkProcessModule::DATA_SERVER);
     this->CollectLOD->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
+    this->PostCollectUpdateSuppressorLOD->SetServers(vtkProcessModule::CLIENT_AND_SERVERS);
     }
   else
     {
@@ -79,29 +86,27 @@ void vtkSMSimpleParallelStrategy::BeginCreateVTKObjects()
 void vtkSMSimpleParallelStrategy::CreatePipeline(vtkSMSourceProxy* input,
   int outputport)
 {
-  // We set up the pipeline as such
-  //    INPUT --> Collect --> UpdateSuppressor
-  this->CreatePipelineInternal(input, outputport,
+  this->Superclass::CreatePipeline(input, outputport);
+
+  // Extend the superclass's pipeline as follows
+  //    SUPERCLASS --> Collect --> PostCollectUpdateSuppressor 
+  this->CreatePipelineInternal(this->Superclass::GetOutput(), 0,
                                this->Collect, 
-                               this->UpdateSuppressor);
+                               this->PostCollectUpdateSuppressor);
 
-  // Connect the PreCollectUpdateSuppressor to the input.
-  this->Connect(input, this->PreCollectUpdateSuppressor, "Input", outputport);
 }
 
 //----------------------------------------------------------------------------
 void vtkSMSimpleParallelStrategy::CreateLODPipeline(vtkSMSourceProxy* input, 
   int outputport)
 {
-  // We set up the pipeline as such
-  //    INPUT --> LODDecimator --> Collect --> UpdateSuppressor
-  this->Connect(input, this->LODDecimator, "Input", outputport);
-  this->CreatePipelineInternal(this->LODDecimator, 0,
-                               this->CollectLOD, 
-                               this->UpdateSuppressorLOD);
+  this->Superclass::CreateLODPipeline(input, outputport);
 
-  // Connect the PreCollectUpdateSuppressorLOD to the decimator.
-  this->Connect(this->LODDecimator, this->PreCollectUpdateSuppressorLOD, "Input", 0);
+  // Extend the superclass's pipeline as follows
+  //    SUPERCLASS --> CollectLOD --> PostCollectUpdateSuppressorLOD 
+  this->CreatePipelineInternal(this->Superclass::GetLODOutput(), 0,
+                               this->CollectLOD, 
+                               this->PostCollectUpdateSuppressorLOD);
 }
 
 //----------------------------------------------------------------------------
@@ -117,22 +122,7 @@ void vtkSMSimpleParallelStrategy::CreatePipelineInternal(
   this->Connect(collect, updatesuppressor);
 
   // Now we need to set up some default parameters on these filters.
-
-  // Collect filter needs the socket controller use to communicate between
-  // data-server root and the client.
-  stream  << vtkClientServerStream::Invoke
-          << pm->GetProcessModuleID() 
-          << "GetSocketController"
-          << pm->GetConnectionClientServerID(this->ConnectionID)
-          << vtkClientServerStream::End;
-  stream  << vtkClientServerStream::Invoke
-          << collect->GetID()
-          << "SetSocketController"
-          << vtkClientServerStream::LastResult
-          << vtkClientServerStream::End;
-  pm->SendStream(this->ConnectionID, 
-    vtkProcessModule::CLIENT_AND_SERVERS, stream);
-
+  
   // Collect filter needs the MPIMToNSocketConnection to communicate between
   // render server and data server nodes.
   stream  << vtkClientServerStream::Invoke
@@ -164,17 +154,13 @@ void vtkSMSimpleParallelStrategy::CreatePipelineInternal(
 
 
 //----------------------------------------------------------------------------
-void vtkSMSimpleParallelStrategy::UpdatePipeline()
+int vtkSMSimpleParallelStrategy::GetMoveMode()
 {
   // Based on the compositing decision made by the render view,
   // decide where the data should be delivered for rendering.
-
   bool usecompositing = this->GetUseCompositing();
   // cout << "usecompositing: " << usecompositing << endl;
 
-  vtkSMIntVectorProperty* ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->Collect->GetProperty("MoveMode"));
-
   int move_mode = 0;
   if (usecompositing)
     {
@@ -195,33 +181,17 @@ void vtkSMSimpleParallelStrategy::UpdatePipeline()
       // cout << "COLLECT" << endl;
       }
     }
-  
-  ivp->SetElement(0, move_mode);
-  this->Collect->UpdateProperty("MoveMode");
-
-  // It is essential to mark the Collect filter explicitly modified.
-  vtkClientServerStream stream;
-  stream  << vtkClientServerStream::Invoke
-          << this->Collect->GetID()
-          << "Modified"
-          << vtkClientServerStream::End;
-  vtkProcessModule::GetProcessModule()->SendStream(
-    this->ConnectionID, this->Collect->GetServers(), stream);
-
-  this->Superclass::UpdatePipeline();
+  return move_mode;
 }
 
 //----------------------------------------------------------------------------
-void vtkSMSimpleParallelStrategy::UpdateLODPipeline()
+int vtkSMSimpleParallelStrategy::GetLODMoveMode()
 {
   // Based on the compositing decision made by the render view,
   // decide where the data should be delivered for rendering.
 
   bool usecompositing = this->GetUseCompositing();
 
-  vtkSMIntVectorProperty* ivp = vtkSMIntVectorProperty::SafeDownCast(
-    this->CollectLOD->GetProperty("MoveMode"));
-
   int move_mode = 0;
   if (usecompositing)
     {
@@ -251,8 +221,48 @@ void vtkSMSimpleParallelStrategy::UpdateLODPipeline()
       // cout << "LOD COLLECT" << endl;
       }
     }
+  return move_mode;
+}
 
-  ivp->SetElement(0, move_mode);
+//----------------------------------------------------------------------------
+void vtkSMSimpleParallelStrategy::UpdatePipeline()
+{
+  if (this->vtkSMSimpleParallelStrategy::GetDataValid())
+    {
+    return;
+    }
+
+  this->Superclass::UpdatePipeline();
+
+  vtkSMPropertyHelper(this->Collect, "MoveMode").Set(this->GetMoveMode()); 
+  this->Collect->UpdateProperty("MoveMode");
+
+  // It is essential to mark the Collect filter explicitly modified.
+  vtkClientServerStream stream;
+  stream  << vtkClientServerStream::Invoke
+          << this->Collect->GetID()
+          << "Modified"
+          << vtkClientServerStream::End;
+  vtkProcessModule::GetProcessModule()->SendStream(
+    this->ConnectionID, this->Collect->GetServers(), stream);
+
+  this->PostCollectUpdateSuppressor->InvokeCommand("ForceUpdate");
+  // This is called for its side-effects; i.e. to force a PostUpdateData()
+  this->PostCollectUpdateSuppressor->UpdatePipeline();
+  this->CollectedDataValid = true;
+}
+
+//----------------------------------------------------------------------------
+void vtkSMSimpleParallelStrategy::UpdateLODPipeline()
+{
+  if (this->vtkSMSimpleParallelStrategy::GetLODDataValid())
+    {
+    return;
+    }
+
+  this->Superclass::UpdateLODPipeline();
+
+  vtkSMPropertyHelper(this->CollectLOD, "MoveMode").Set(this->GetLODMoveMode());
   this->CollectLOD->UpdateProperty("MoveMode");
 
   // It is essential to mark the Collect filter explicitly modified.
@@ -272,10 +282,12 @@ void vtkSMSimpleParallelStrategy::UpdateLODPipeline()
   vtkProcessModule::GetProcessModule()->SendStream(
     this->ConnectionID, this->CollectLOD->GetServers(), stream);
 
-  this->Superclass::UpdateLODPipeline();
+  this->PostCollectUpdateSuppressorLOD->InvokeCommand("ForceUpdate");
+  // This is called for its side-effects; i.e. to force a PostUpdateData()
+  this->PostCollectUpdateSuppressorLOD->UpdatePipeline();
+  this->CollectedLODDataValid = true;
 }
 
-
 //----------------------------------------------------------------------------
 void vtkSMSimpleParallelStrategy::SetUseCompositing(bool compositing)
 {
@@ -346,101 +358,6 @@ void vtkSMSimpleParallelStrategy::ProcessViewInformation()
   this->Superclass::ProcessViewInformation();
 }
 
-//----------------------------------------------------------------------------
-void vtkSMSimpleParallelStrategy::GatherInformation(vtkPVInformation* info)
-{
-  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
-  if (this->GetUseCache())
-    {
-    // when using cache, if the cached time is available in the update
-    // suppressor that caches, then we use the information from that update
-    // suppressor, else we follow the usual procedure.
-    vtkClientServerStream stream;
-    stream << vtkClientServerStream::Invoke
-           << this->UpdateSuppressor->GetID()
-           << "IsCached"
-           << this->CacheTime
-           << vtkClientServerStream::End;
-    pm->SendStream(this->ConnectionID, 
-      vtkProcessModule::DATA_SERVER_ROOT, stream);
-
-    vtkClientServerStream values;
-    int is_cached=false;
-    if (pm->GetLastResult(this->ConnectionID,
-        vtkProcessModule::DATA_SERVER_ROOT).GetArgument(0, 0, &values) &&
-      values.GetArgument(0, 1, &is_cached) && 
-      is_cached)
-      {
-      this->SomethingCached = true;
-      vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
-        this->UpdateSuppressor->GetProperty("CacheUpdate"));
-      dvp->SetElement(0, this->CacheTime);
-      this->UpdateSuppressor->UpdateProperty("CacheUpdate", 1);
-      pm->GatherInformation(this->ConnectionID,
-        vtkProcessModule::DATA_SERVER_ROOT,
-        info,
-        this->UpdateSuppressor->GetID());
-      return;
-      }
-    }
-
-  // Update the pipeline partially until before the Collect proxy
-  this->PreCollectUpdateSuppressor->InvokeCommand("ForceUpdate");
-  // This is called for its side-effects; i.e. to force a PostUpdateData()
-  this->PreCollectUpdateSuppressor->UpdatePipeline();
-  pm->GatherInformation(this->ConnectionID,
-    vtkProcessModule::DATA_SERVER_ROOT,
-    info,
-    this->PreCollectUpdateSuppressor->GetID());
-}
-
-//----------------------------------------------------------------------------
-void vtkSMSimpleParallelStrategy::GatherLODInformation(vtkPVInformation* info)
-{
-  vtkProcessModule* pm = vtkProcessModule::GetProcessModule();
-  // First update the pipeline partially until before the Collect proxy
-  if (this->GetUseCache())
-    {
-    // when using cache, if the cached time is available in the update
-    // suppressor that caches, then we use the information from that update
-    // suppressor, else we follow the usual procedure.
-    vtkClientServerStream stream;
-    stream << vtkClientServerStream::Invoke
-           << this->UpdateSuppressorLOD->GetID()
-           << "IsCached"
-           << this->CacheTime
-           << vtkClientServerStream::End;
-    pm->SendStream(this->ConnectionID, 
-      vtkProcessModule::DATA_SERVER_ROOT, stream);
-
-    vtkClientServerStream values;
-    int is_cached=false;
-    if (pm->GetLastResult(this->ConnectionID,
-        vtkProcessModule::DATA_SERVER_ROOT).GetArgument(0, 0, &values) &&
-      values.GetArgument(0, 1, &is_cached) && 
-      is_cached)
-      {
-      this->SomethingCached = true;
-      vtkSMDoubleVectorProperty* dvp = vtkSMDoubleVectorProperty::SafeDownCast(
-        this->UpdateSuppressorLOD->GetProperty("CacheUpdate"));
-      dvp->SetElement(0, this->CacheTime);
-      this->UpdateSuppressorLOD->UpdateProperty("CacheUpdate", 1);
-      pm->GatherInformation(this->ConnectionID,
-        vtkProcessModule::DATA_SERVER_ROOT,
-        info,
-        this->UpdateSuppressorLOD->GetID());
-      return;
-      }
-    }
-
-  // Update the pipeline partially until before the Collect proxy
-  this->PreCollectUpdateSuppressorLOD->InvokeCommand("ForceUpdate");
-  pm->GatherInformation(this->ConnectionID,
-    vtkProcessModule::DATA_SERVER_ROOT,
-    info,
-    this->PreCollectUpdateSuppressorLOD->GetID());
-}
-
 //----------------------------------------------------------------------------
 void vtkSMSimpleParallelStrategy::PrintSelf(ostream& os, vtkIndent indent)
 {
