@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMSimpleParallelStrategy.h,v $
+  Module:    vtkSMSimpleParallelStrategy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -26,9 +26,19 @@ class VTK_EXPORT vtkSMSimpleParallelStrategy : public vtkSMSimpleStrategy
 {
 public:
   static vtkSMSimpleParallelStrategy* New();
-  vtkTypeRevisionMacro(vtkSMSimpleParallelStrategy, vtkSMSimpleStrategy);
+  vtkTypeMacro(vtkSMSimpleParallelStrategy, vtkSMSimpleStrategy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
+  // Description:
+  // Get the output from the strategy.
+  virtual vtkSMSourceProxy* GetOutput()
+    { return this->PostCollectUpdateSuppressor; }
+
+  // Description:
+  // Get low resolution output.
+  virtual vtkSMSourceProxy* GetLODOutput()
+    { return this->PostCollectUpdateSuppressorLOD; }
+
 //BTX
 protected:
   vtkSMSimpleParallelStrategy();
@@ -48,18 +58,6 @@ protected:
   // flag.
   virtual void CreateLODPipeline(vtkSMSourceProxy* input, int outputport);
 
-  // Description:
-  // Gather the information of the displayed data (non-LOD).
-  // Update the part of the pipeline needed to gather full information
-  // and then gather that information. 
-  virtual void GatherInformation(vtkPVInformation*);
-
-  // Description:
-  // Gather the information of the displayed data (lod);
-  // Update the part of the pipeline needed to gather full information
-  // and then gather that information. 
-  virtual void GatherLODInformation(vtkPVInformation*);
-
   // Description:
   // Update the LOD pipeline.
   // Overridden to pass correct collection decision to the Collect filter
@@ -72,6 +70,22 @@ protected:
   // based on UseCompositing flag.
   virtual void UpdatePipeline();
 
+  // Description:
+  // Invalidates the LOD pipeline.
+  virtual void InvalidateLODPipeline()
+    { 
+    this->Superclass::InvalidateLODPipeline();
+    this->CollectedLODDataValid = false;
+    }
+
+  // Description:
+  // Invalidates the full resolution pipeline.
+  virtual void InvalidatePipeline()
+    {
+    this->Superclass::InvalidatePipeline();
+    this->CollectedDataValid = false;
+    }
+
   // Description:
   // Called when ever the view information changes.
   // The strategy should update it's state based on the current view information
@@ -84,30 +98,66 @@ protected:
   void SetUseCompositing(bool);
   vtkGetMacro(UseCompositing, bool);
 
+  // Description:
+  // When LODClientCollect is on (default) it implies than when rendering
+  // Low-res, if the compositing decision is to deliver to client, then it's
+  // acceptable to deliver the LOD data to client. If set to false, only the
+  // outline will be delivered to the client.
   void SetLODClientCollect(bool);
+
+  // Description:
+  // This flag is set when rendering in tile-display mode. It implies that LOD
+  // data must be rendered on the client side. TODO: this flag's name needs to
+  // be fixed to be more sensible. 
   void SetLODClientRender(bool);
 
-  vtkSMSourceProxy* PreCollectUpdateSuppressor;
+  virtual bool GetDataValid()
+    { 
+    return this->CollectedDataValid && this->Superclass::GetDataValid();
+    }
+
+  virtual bool GetLODDataValid()
+    { 
+    return this->CollectedLODDataValid &&
+      this->Superclass::GetLODDataValid(); 
+    }
+
+  // Description:
+  // Determine where the data is to be delivered for rendering the current
+  // frame.
+  virtual int GetMoveMode();
+  virtual int GetLODMoveMode();
+
+  // Since LOD and full res pipeline have exactly the same setup, we have this
+  // common method. //DDM TODO Why public?
+  void CreatePipelineInternal(vtkSMSourceProxy* input, int outputport,
+    vtkSMSourceProxy* collect,
+    vtkSMSourceProxy* updatesuppressor);
+
   vtkSMSourceProxy* Collect;
+  vtkSMSourceProxy* PostCollectUpdateSuppressor;
 
-  vtkSMSourceProxy* PreCollectUpdateSuppressorLOD;
   vtkSMSourceProxy* CollectLOD;
+  vtkSMSourceProxy* PostCollectUpdateSuppressorLOD;
+  
+  // In client-server (or parallel) we want to avoid data-movement when caching,
+  // hence we use the PostCollectCacheKeeper.
+  // This is directly liked to the CacheKeeper (using property linking).
+  vtkSMSourceProxy* PostCollectCacheKeeper;
 
+  bool CollectedDataValid;
+  bool CollectedLODDataValid;
   bool UseCompositing;
   bool LODClientRender; // when set, indicates that data must be made available on client,
                      // irrespective of UseCompositing flag.
 
   bool LODClientCollect; // When set, the data delivered to client is outline of the data
                       // not the whole data.
+
 private:
   vtkSMSimpleParallelStrategy(const vtkSMSimpleParallelStrategy&); // Not implemented
   void operator=(const vtkSMSimpleParallelStrategy&); // Not implemented
 
-  // Since LOD and full res pipeline have exactly the same setup, we have this
-  // common method.
-  void CreatePipelineInternal(vtkSMSourceProxy* input, int outputport,
-    vtkSMSourceProxy* collect,
-    vtkSMSourceProxy* updatesuppressor);
 //ETX
 };
 
