@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMStringVectorProperty.cxx,v $
+  Module:    vtkSMStringVectorProperty.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -18,12 +18,12 @@
 #include "vtkObjectFactory.h"
 #include "vtkPVXMLElement.h"
 #include "vtkProcessModule.h"
+#include "vtkStringList.h"
 
 #include <vtkstd/vector>
 #include "vtkStdString.h"
 
 vtkStandardNewMacro(vtkSMStringVectorProperty);
-vtkCxxRevisionMacro(vtkSMStringVectorProperty, "$Revision: 1.44 $");
 
 struct vtkSMStringVectorPropertyInternals
 {
@@ -207,6 +207,70 @@ const char* vtkSMStringVectorProperty::GetElement(unsigned int idx)
   return this->Internals->Values[idx].c_str();
 }
 
+//---------------------------------------------------------------------------
+void vtkSMStringVectorProperty::GetElements(vtkStringList* list)
+{
+  list->RemoveAllItems();
+
+  unsigned int numElems = this->GetNumberOfElements();
+  for (unsigned int cc=0; cc < numElems; cc++)
+    {
+    list->AddString(this->GetElement(cc));
+    }
+}
+
+//---------------------------------------------------------------------------
+int vtkSMStringVectorProperty::SetElements(vtkStringList* list)
+{
+  unsigned int count = static_cast<unsigned int>(list->GetLength());
+  unsigned int numElems = this->GetNumberOfElements();
+
+  if (this->Initialized && count == numElems)
+    {
+    // Check is cur values are same as the new values.
+    int modified = 0;
+    for (unsigned int cc=0; cc < numElems; cc++)
+      {
+      const char* value = list->GetString(cc)? list->GetString(cc) : "";
+      if (this->Internals->Values[cc] != value)
+        {
+        modified = 1;
+        break;
+        }
+      }
+    if (!modified)
+      {
+      // nothing changed.
+      return 1;
+      }
+    }
+
+  if ( vtkSMProperty::GetCheckDomains() )
+    {
+    this->SetNumberOfUncheckedElements(count);
+    for(unsigned int cc=0; cc<count; cc++)
+      {
+      this->SetUncheckedElement(cc, list->GetString(cc)?
+        list->GetString(cc):"");
+      }
+
+    if (!this->IsInDomains())
+      {
+      this->SetNumberOfUncheckedElements(this->GetNumberOfElements());
+      return 0;
+      }
+    }
+
+  this->Internals->Resize(count);
+  for (unsigned int cc=0; cc < count; cc++)
+    {
+    this->Internals->Values[cc] = list->GetString(cc)? list->GetString(cc) : "";
+    }
+  this->Initialized = true;
+  this->Modified();
+  return 1;
+}
+
 //---------------------------------------------------------------------------
 const char* vtkSMStringVectorProperty::GetUncheckedElement(unsigned int idx)
 {
@@ -261,7 +325,7 @@ int vtkSMStringVectorProperty::SetElements(unsigned int count, const char* value
       {
       this->SetUncheckedElement(cc, values[cc]?values[cc]:"");
       }
-    
+
     if (!this->IsInDomains())
       {
       this->SetNumberOfUncheckedElements(this->GetNumberOfElements());
@@ -289,7 +353,7 @@ int vtkSMStringVectorProperty::SetElement(unsigned int idx, const char* value)
 
   unsigned int numElems = this->GetNumberOfElements();
 
-  if (this->Initialized && 
+  if (this->Initialized &&
       idx < numElems && strcmp(value, this->GetElement(idx)) == 0)
     {
     return 1;
@@ -301,7 +365,7 @@ int vtkSMStringVectorProperty::SetElement(unsigned int idx, const char* value)
       {
       this->SetUncheckedElement(i, this->GetElement(i));
       }
-    
+
     this->SetUncheckedElement(idx, value);
     if (!this->IsInDomains())
       {
@@ -309,7 +373,7 @@ int vtkSMStringVectorProperty::SetElement(unsigned int idx, const char* value)
       return 0;
       }
     }
-  
+
   if (idx >= this->GetNumberOfElements())
     {
     this->SetNumberOfElements(idx+1);
@@ -385,7 +449,7 @@ int vtkSMStringVectorProperty::ReadXMLAttributes(vtkSMProxy* proxy,
           pos1 += delim.size();
           }
         pos2 = initVal.find(delimiter, pos1);
-        vtkStdString v = pos1 == pos2 ? "" : initVal.substr(pos1, pos2);
+        vtkStdString v = pos1 == pos2 ? "" : initVal.substr(pos1, pos2-pos1);
         this->Internals->DefaultValues.push_back(v);
         this->SetElement(i, v.c_str());
         pos1 = pos2;
@@ -396,7 +460,7 @@ int vtkSMStringVectorProperty::ReadXMLAttributes(vtkSMProxy* proxy,
       this->SetElement(0, tmp);
       this->Internals->DefaultValues.push_back(tmp);
       }
-    this->Internals->UpdateLastPushedValues(); 
+    this->Internals->UpdateLastPushedValues();
     }
   return 1;
 }
@@ -404,7 +468,7 @@ int vtkSMStringVectorProperty::ReadXMLAttributes(vtkSMProxy* proxy,
 //---------------------------------------------------------------------------
 const char* vtkSMStringVectorProperty::GetDefaultValue(int idx)
 {
-  if(idx >= 0 && idx < (int)this->Internals->DefaultValues.size() && 
+  if(idx >= 0 && idx < (int)this->Internals->DefaultValues.size() &&
     !this->Internals->DefaultValues[idx].empty())
     {
     return this->Internals->DefaultValues[idx].c_str();
@@ -414,7 +478,7 @@ const char* vtkSMStringVectorProperty::GetDefaultValue(int idx)
 
 //---------------------------------------------------------------------------
 int vtkSMStringVectorProperty::LoadState(vtkPVXMLElement* element,
-  vtkSMStateLoaderBase* loader, int loadLastPushedValues/*=0*/)
+  vtkSMProxyLocator* loader, int loadLastPushedValues/*=0*/)
 {
   int prevImUpdate = this->ImmediateUpdate;
 
@@ -429,7 +493,7 @@ int vtkSMStringVectorProperty::LoadState(vtkPVXMLElement* element,
     for (unsigned int i=0; i < numElems; i++)
       {
       vtkPVXMLElement* currentElement = element->GetNestedElement(i);
-      if (currentElement->GetName() && 
+      if (currentElement->GetName() &&
         strcmp(currentElement->GetName(), "LastPushedValues") == 0)
         {
         actual_element = currentElement;
@@ -443,7 +507,7 @@ int vtkSMStringVectorProperty::LoadState(vtkPVXMLElement* element,
       }
     element = actual_element;
     }
-  
+
   bool prev = this->SetBlockModifiedEvents(true);
   unsigned int numElems = element->GetNumberOfNestedElements();
   for (unsigned int i=0; i<numElems; i++)
@@ -491,7 +555,7 @@ void vtkSMStringVectorProperty::ChildSaveState(vtkPVXMLElement* propertyElement,
     vtkPVXMLElement* elementElement = vtkPVXMLElement::New();
     elementElement->SetName("Element");
     elementElement->AddAttribute("index", i);
-    elementElement->AddAttribute("value", 
+    elementElement->AddAttribute("value",
                                  (this->GetElement(i)?this->GetElement(i):""));
     propertyElement->AddNestedElement(elementElement);
     elementElement->Delete();
@@ -500,7 +564,7 @@ void vtkSMStringVectorProperty::ChildSaveState(vtkPVXMLElement* propertyElement,
   if (saveLastPushedValues)
     {
     size = this->Internals->LastPushedValues.size();
-    
+
     vtkPVXMLElement* element = vtkPVXMLElement::New();
     element->SetName("LastPushedValues");
     element->AddAttribute("number_of_elements", size);
@@ -509,7 +573,7 @@ void vtkSMStringVectorProperty::ChildSaveState(vtkPVXMLElement* propertyElement,
       vtkPVXMLElement* elementElement = vtkPVXMLElement::New();
       elementElement->SetName("Element");
       elementElement->AddAttribute("index", cc);
-      elementElement->AddAttribute("value", 
+      elementElement->AddAttribute("value",
         this->Internals->LastPushedValues[cc]);
       element->AddNestedElement(elementElement);
       elementElement->Delete();
