@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMSurfaceRepresentationProxy.h,v $
+  Module:    vtkSMSurfaceRepresentationProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -33,7 +33,7 @@ class VTK_EXPORT vtkSMSurfaceRepresentationProxy :
 {
 public:
   static vtkSMSurfaceRepresentationProxy* New();
-  vtkTypeRevisionMacro(vtkSMSurfaceRepresentationProxy, vtkSMPropRepresentationProxy);
+  vtkTypeMacro(vtkSMSurfaceRepresentationProxy, vtkSMPropRepresentationProxy);
   virtual void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -109,6 +109,11 @@ public:
   // Returns the proxy for the prop.
   vtkGetObjectMacro(Prop3D, vtkSMProxy);
 
+  // Description:
+  // Returns the proxy for the vtkProperty.
+  virtual vtkSMProxy *GetPropertyProxy() 
+    { return this->Property; }
+
   // Description:
   // HACK: vtkSMAnimationSceneGeometryWriter needs acces to the processed data
   // so save out. This method should return the proxy that goes in as the input
