@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMTextWidgetRepresentationProxy.h,v $
+  Module:    vtkSMTextWidgetRepresentationProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -26,7 +26,7 @@ class VTK_EXPORT vtkSMTextWidgetRepresentationProxy : public vtkSMNewWidgetRepre
 {
 public:
   static vtkSMTextWidgetRepresentationProxy* New();
-  vtkTypeRevisionMacro(vtkSMTextWidgetRepresentationProxy, vtkSMNewWidgetRepresentationProxy);
+  vtkTypeMacro(vtkSMTextWidgetRepresentationProxy, vtkSMNewWidgetRepresentationProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
   
 protected:
