@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkSMUnstructuredGridVolumeRepresentationProxy.h,v $
+  Module:    vtkSMUnstructuredGridVolumeRepresentationProxy.h
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -28,7 +28,7 @@ class VTK_EXPORT vtkSMUnstructuredGridVolumeRepresentationProxy :
 {
 public:
   static vtkSMUnstructuredGridVolumeRepresentationProxy* New();
-  vtkTypeRevisionMacro(vtkSMUnstructuredGridVolumeRepresentationProxy, 
+  vtkTypeMacro(vtkSMUnstructuredGridVolumeRepresentationProxy,
     vtkSMPropRepresentationProxy);
   void PrintSelf(ostream& os, vtkIndent indent);
 
