@@ -1,5 +1,5 @@
 /* -*- c -*- *****************************************************************
-** $Id: BoundsBehindViewer.c,v 1.2 2005-12-16 19:02:08 kmorel Exp $
+** Id
 **
 ** Copyright (C) 2005 Sandia Corporation
 ** Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
@@ -17,9 +17,6 @@
 #include "test_codes.h"
 #include "glwin.h"
 
-#include <GL/gl.h>
-#include <GL/glu.h>
-
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -46,14 +43,10 @@ static void PrintMatrix(float *mat)
     }
 }
 
-int BoundsBehindViewer(int argc, char * argv[])
+int BoundsBehindViewer(int argc, char * vargv[])
 {
     float mat[16];
 
-    /* To remove warning */
-    (void)argc;
-    (void)argv;
-
     GLint rank;
     icetGetIntegerv(ICET_RANK, &rank);
 
