@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   ParaView
-  Module:    $RCSfile: vtkPVPythonInterpretor.cxx,v $
+  Module:    vtkPVPythonInterpretor.cxx
 
   Copyright (c) Kitware, Inc.
   All rights reserved.
@@ -75,20 +75,16 @@ static void vtkPythonAppInitPrependPythonPath(const char* dir)
 }
 
 //----------------------------------------------------------------------------
-static void vtkPythonAppInitPrependPath(const char* self_dir)
+static bool vtkPythonAppInitPrependPath2(const vtkstd::string& prefix, 
+  const vtkstd::string& path)
 {
-  // Try to put the VTK python module location in sys.path.
-  vtkstd::string pkg_prefix = self_dir;
-#if defined(CMAKE_INTDIR)
-  pkg_prefix += "/..";
-#endif
   vtkstd::string package_dir;
-  package_dir = pkg_prefix + "/../Utilities/VTKPythonWrapping";
+  package_dir = prefix + "/../" + path;
   package_dir = vtksys::SystemTools::CollapseFullPath(package_dir.c_str());
   if (!vtksys::SystemTools::FileIsDirectory(package_dir.c_str()))
     {
     // This is the right path for app bundles on OS X
-    package_dir = pkg_prefix + "/../../../../Utilities/VTKPythonWrapping";
+    package_dir = prefix + "/../../../../" + path;
     package_dir = vtksys::SystemTools::CollapseFullPath(package_dir.c_str());
     }
   if(vtksys::SystemTools::FileIsDirectory(package_dir.c_str()))
@@ -96,6 +92,23 @@ static void vtkPythonAppInitPrependPath(const char* self_dir)
     // This executable is running from the build tree.  Prepend the
     // library directory and package directory to the search path.
     vtkPythonAppInitPrependPythonPath(package_dir.c_str());
+    return true;
+    }
+  return false;
+}
+//----------------------------------------------------------------------------
+static void vtkPythonAppInitPrependPath(const char* self_dir)
+{
+  // Try to put the VTK python module location in sys.path.
+  vtkstd::string pkg_prefix = self_dir;
+#if defined(CMAKE_INTDIR)
+  pkg_prefix += "/..";
+#endif
+  vtkPythonAppInitPrependPath2(pkg_prefix, "Utilities/mpi4py");
+  if (vtkPythonAppInitPrependPath2(pkg_prefix, "Utilities/VTKPythonWrapping"))
+    {
+    // This executable is running from the build tree.  Prepend the
+    // library directory to the search path.
     vtkPythonAppInitPrependPythonPath(VTK_PYTHON_LIBRARY_DIR);
     }
   else
@@ -109,7 +122,7 @@ static void vtkPythonAppInitPrependPath(const char* self_dir)
     // packages in sys.path himself/herself.
     const char* inst_dirs[] = {
       "/paraview",
-      "/../Resources/paraview", // MacOS
+      "/../Python/paraview", // MacOS bundle
       "/../lib/paraview-" PARAVIEW_VERSION "/paraview",
       "/../../lib/paraview-" PARAVIEW_VERSION "/paraview",
       "/lib/python" VTK_PYTHON_VERSION "/site-packages/paraview", // UNIX --prefix
@@ -119,13 +132,20 @@ static void vtkPythonAppInitPrependPath(const char* self_dir)
       0
     };
     vtkstd::string prefix = self_dir;
-    // On OS X distributions, the libraries are in a different directory
-    // than the module. They are in the same place as the executable.
 #if defined(__APPLE__)
     vtkPythonAppInitPrependPythonPath(self_dir);
-#endif 
+    // On OS X distributions, the libraries are in a different directory
+    // than the module. They are in a place relative to the executable.
+    vtkstd::string libs_dir = vtkstd::string(self_dir) + "/../Libraries";
+    libs_dir = vtksys::SystemTools::CollapseFullPath(libs_dir.c_str());
+    if(vtksys::SystemTools::FileIsDirectory(libs_dir.c_str()))
+      {
+      vtkPythonAppInitPrependPythonPath(libs_dir.c_str());
+      }
+#endif
     for(const char** dir = inst_dirs; *dir; ++dir)
       {
+      vtkstd::string package_dir;
       package_dir = prefix;
       package_dir += *dir;
       package_dir = vtksys::SystemTools::CollapseFullPath(package_dir.c_str());
@@ -207,7 +227,7 @@ public:
         "Please call ReleaseControl() befor calling MakeCurrent().");
       return;
       }
-    
+
     if (this->Interpretor)
       {
       this->PreviousInterpretor = PyThreadState_Swap(this->Interpretor);
@@ -223,7 +243,6 @@ public:
 
 //-----------------------------------------------------------------------------
 vtkStandardNewMacro(vtkPVPythonInterpretor);
-vtkCxxRevisionMacro(vtkPVPythonInterpretor, "$Revision: 1.25 $");
 
 //-----------------------------------------------------------------------------
 vtkPVPythonInterpretor::vtkPVPythonInterpretor()
@@ -240,6 +259,14 @@ vtkPVPythonInterpretor::~vtkPVPythonInterpretor()
   this->SetExecutablePath(0);
 }
 
+//-----------------------------------------------------------------------------
+vtkStdString vtkPVPythonInterpretor::GetInputLine()
+{
+  vtkStdString ret;
+  this->InvokeEvent(vtkCommand::UpdateEvent, &ret);
+  return ret;
+}
+
 //-----------------------------------------------------------------------------
 void vtkPVPythonInterpretor::DumpError(const char* str)
 {
@@ -252,7 +279,7 @@ void vtkPVPythonInterpretor::DumpError(const char* str)
     if (this->Internal->Messages.size() > 0 &&
       this->Internal->Messages.back().IsError)
       {
-      this->Internal->Messages.back().Message += str; 
+      this->Internal->Messages.back().Message += str;
       }
     else
       {
@@ -274,7 +301,7 @@ void vtkPVPythonInterpretor::DumpOutput(const char* str)
     if (this->Internal->Messages.size() > 0 &&
       !this->Internal->Messages.back().IsError)
       {
-      this->Internal->Messages.back().Message += str; 
+      this->Internal->Messages.back().Message += str;
       }
     else
       {
@@ -287,7 +314,7 @@ void vtkPVPythonInterpretor::DumpOutput(const char* str)
 //-----------------------------------------------------------------------------
 void vtkPVPythonInterpretor::FlushMessages()
 {
-  vtkstd::vector<vtkPythonMessage>::iterator iter = 
+  vtkstd::vector<vtkPythonMessage>::iterator iter =
     this->Internal->Messages.begin();
 
   for (; iter != this->Internal->Messages.end(); ++iter)
@@ -336,17 +363,27 @@ void vtkPVPythonInterpretor::InitializeInternal()
 
   if (this->CaptureStreams)
     {
+    // HACK: Calling PyRun_SimpleString for the first time for some reason results in
+    // a "\n" message being generated which is causing the error dialog to
+    // popup. So we flush that message out of the system before setting up the
+    // callbacks.
+    // The cast is necessary because PyRun_SimpleString() hasn't always been
+    // const-correct.
+    PyRun_SimpleString(const_cast<char*>(""));
     vtkPVPythonInterpretorWrapper* wrapperOut = vtkWrapInterpretor(this);
     wrapperOut->DumpToError = false;
 
     vtkPVPythonInterpretorWrapper* wrapperErr = vtkWrapInterpretor(this);
     wrapperErr->DumpToError = true;
 
-    // Redirect Python's stdout and stderr
-    PySys_SetObject(const_cast<char*>("stdout"), 
+    // Redirect Python's stdout and stderr and stdin
+    PySys_SetObject(const_cast<char*>("stdout"),
       reinterpret_cast<PyObject*>(wrapperOut));
 
-    PySys_SetObject(const_cast<char*>("stderr"), 
+    PySys_SetObject(const_cast<char*>("stderr"),
+      reinterpret_cast<PyObject*>(wrapperErr));
+
+    PySys_SetObject(const_cast<char*>("stdin"),
       reinterpret_cast<PyObject*>(wrapperErr));
 
     Py_DECREF(wrapperOut);
@@ -355,7 +392,7 @@ void vtkPVPythonInterpretor::InitializeInternal()
 }
 
 //-----------------------------------------------------------------------------
-int vtkPVPythonInterpretor::InitializeSubInterpretor(int vtkNotUsed(argc), 
+int vtkPVPythonInterpretor::InitializeSubInterpretor(int vtkNotUsed(argc),
   char** argv)
 {
   if (this->Internal->Interpretor)
@@ -398,10 +435,16 @@ void vtkPVPythonInterpretor::AddPythonPath(const char* path)
     }
 
   this->MakeCurrent();
-  vtkPythonAppInitPrependPythonPath(path);
+  this->AddPythonPathInternal(path);
   this->ReleaseControl();
 }
 
+//-----------------------------------------------------------------------------
+void vtkPVPythonInterpretor::AddPythonPathInternal(const char* path)
+{
+  vtkPythonAppInitPrependPythonPath(path);
+}
+
 //-----------------------------------------------------------------------------
 int vtkPVPythonInterpretor::PyMain(int argc, char** argv)
 {
@@ -427,7 +470,7 @@ void vtkPVPythonInterpretor::MakeCurrent()
 //-----------------------------------------------------------------------------
 void vtkPVPythonInterpretor::ReleaseControl()
 {
-  this->Internal->ReleaseControl(); 
+  this->Internal->ReleaseControl();
 }
 
 //-----------------------------------------------------------------------------
@@ -439,7 +482,7 @@ void vtkPVPythonInterpretor::RunSimpleString(const char* const script)
   // http://sourceforge.net/tracker/?group_id=5470&atid=105470&func=detail&aid=1167922
   vtkstd::string buffer = script ? script : "";
   buffer.erase(vtkstd::remove(buffer.begin(), buffer.end(), '\r'), buffer.end());
-  
+
   // The cast is necessary because PyRun_SimpleString() hasn't always been const-correct
   PyRun_SimpleString(const_cast<char*>(buffer.c_str()));
 
@@ -457,7 +500,7 @@ void vtkPVPythonInterpretor::RunSimpleFile(const char* const filename)
     vtkErrorMacro("Failed to open file " << filename);
     return;
     }
- 
+
   // The cast is necessary because PyRun_SimpleFile() hasn't always been const-correct
   PyRun_SimpleFile(fp, const_cast<char*>(filename));
 
