@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfAttribute.cxx,v 1.14 2008-05-16 15:09:50 clarke Exp $  */
-/*  Date : $Date: 2008-05-16 15:09:50 $ */
-/*  Version : $Revision: 1.14 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -36,12 +36,15 @@ XdmfAttribute::XdmfAttribute() {
   this->Values = NULL;
   this->ShapeDesc = new XdmfDataDesc();
   this->Active = 0;
+  this->LightDataLimit = 100;
+  this->Units = NULL;    // Ian Curington, HR Wallingford Ltd.
   }
 
 XdmfAttribute::~XdmfAttribute() {
   if( this->ValuesAreMine && this->Values )  delete this->Values;
   delete this->ShapeDesc;
-  }
+  if(this->Units) delete [] this->Units;  // Ian Curington, HR Wallingford Ltd.
+}
 
 XdmfInt32
 XdmfAttribute::Release(){
@@ -63,45 +66,65 @@ XdmfAttribute::Insert( XdmfElement *Child){
     return(XDMF_FAIL);
 }
 
+XdmfDataItem * XdmfAttribute::GetDataItem(){
+    XdmfDataItem *di = NULL;
+    XdmfXmlNode Node = this->DOM->FindDataElement(0, this->GetElement());
+    
+    if(Node) {
+        di = (XdmfDataItem *)this->GetCurrentXdmfElement(Node);
+    } 
+    if(!di){
+        di = new XdmfDataItem;
+        Node = this->DOM->InsertNew(this->GetElement(), "DataItem");
+        di->SetDOM(this->DOM);
+        di->SetElement(Node);
+    }
+    return(di);
+}
 
 XdmfInt32
 XdmfAttribute::Build(){
     if(XdmfElement::Build() != XDMF_SUCCESS) return(XDMF_FAIL);
+    if (this->GetActive())
+      {
+      this->Set("Active", "1");
+      }
     this->Set("AttributeType", this->GetAttributeTypeAsString());
     this->Set("Center", this->GetAttributeCenterAsString());
+    if(this->BuildFromDataXml() == XDMF_SUCCESS) return(XDMF_SUCCESS);
     if(this->Values){
-        XdmfDataItem    *di = NULL;
-        XdmfXmlNode     node;
-        //! Is there already a DataItem
-        node = this->DOM->FindDataElement(0, this->GetElement());
-        if(node) {
-            di = (XdmfDataItem *)this->GetCurrentXdmfElement(node);
-        }
-        if(!di){
-            di = new XdmfDataItem;
-            node = this->DOM->InsertNew(this->GetElement(), "DataItem");
-            di->SetDOM(this->DOM);
-            di->SetElement(node);
-        }
+        XdmfDataItem * di = this->GetDataItem(); 
         di->SetArray(this->Values);
-        if(this->Values->GetNumberOfElements() > 100) di->SetFormat(XDMF_FORMAT_HDF);
+        if(this->Values->GetNumberOfElements() > this->LightDataLimit) di->SetFormat(XDMF_FORMAT_HDF);
         di->Build();
-
+        this->SetCurrentXdmfElement(di->GetElement(), NULL);
+        delete di;
     }
+// PATCH September 09, Ian Curington, HR Wallingford Ltd.
+	if(this->Units)
+	{
+		this->Set("Units", this->GetUnits());
+	}
+// end patch
+
     return(XDMF_SUCCESS);
 }
 
 XdmfConstString
 XdmfAttribute::GetAttributeTypeAsString( void ){
   switch ( this->AttributeType ){
-    case XDMF_ATTRIBUTE_TYPE_SCALAR :
+    case XDMF_ATTRIBUTE_TYPE_SCALAR  :
       return("Scalar");
-    case XDMF_ATTRIBUTE_TYPE_VECTOR :
+    case XDMF_ATTRIBUTE_TYPE_VECTOR  :
       return("Vector");
-    case XDMF_ATTRIBUTE_TYPE_TENSOR :
+    case XDMF_ATTRIBUTE_TYPE_TENSOR  :
       return("Tensor");
-    case XDMF_ATTRIBUTE_TYPE_MATRIX :
+    case XDMF_ATTRIBUTE_TYPE_MATRIX  :
       return("Matrix");
+    case XDMF_ATTRIBUTE_TYPE_TENSOR6 :
+      return("Tensor6");
+    case XDMF_ATTRIBUTE_TYPE_GLOBALID :
+      return("GlobalId");
     default :
       break;
     }
@@ -128,6 +151,12 @@ if( XDMF_WORD_CMP( attributeType, "Scalar" ) ) {
   this->ShapeDesc->SetShape( 2, Dimensions );
 } else if( XDMF_WORD_CMP( attributeType, "Matrix" ) ) {
   this->AttributeType = XDMF_ATTRIBUTE_TYPE_MATRIX;
+} else if( XDMF_WORD_CMP( attributeType, "Tensor6" ) ) {
+  this->AttributeType = XDMF_ATTRIBUTE_TYPE_TENSOR6;
+  Dimensions[0] = 6;
+} else if( XDMF_WORD_CMP( attributeType, "GlobalId" ) ) {
+  this->AttributeType = XDMF_ATTRIBUTE_TYPE_GLOBALID;
+  Dimensions[0] = 1;
 } else {
   XdmfErrorMessage("Unknown Attribute Type " << attributeType );
   return( XDMF_FAIL );
@@ -210,6 +239,18 @@ if( Attribute ){
 } else {
   this->AttributeType = XDMF_ATTRIBUTE_TYPE_SCALAR;
 }
+free((void*)Attribute);
+
+// PATCH September 09, Ian Curinton HR Wallingford Ltd.
+Attribute = this->Get( "Units" );
+if( Attribute ){
+  this->SetUnits( Attribute );
+} else {
+  if(this->Units) delete [] this->Units;
+  this->Units = NULL;
+}
+// end patch
+free((void*)Attribute);
 
 Attribute = this->Get( "Active" );
 this->Active = 0;
@@ -218,6 +259,7 @@ if ( Attribute ){
     this->Active = 1;
   }
 }
+free((void*)Attribute);
 
 Attribute = this->Get( "Center" );
 if( Attribute ){
@@ -225,6 +267,8 @@ if( Attribute ){
 } else {
   this->AttributeCenter = XDMF_ATTRIBUTE_CENTER_NODE;
 }
+free((void*)Attribute);
+
 Attribute = this->Get( "Dimensions" );
 if( Attribute ){
   this->ShapeDesc->SetShapeFromString( Attribute );
@@ -237,12 +281,14 @@ if( Attribute ){
     Attribute = this->DOM->Get( ValuesNode, "Dimensions" );
     if(!Attribute){
         XdmfErrorMessage("Dimensions of Attribute not set in XML or DataItem");
+        free((void*)Attribute);
         return(XDMF_FAIL);
     }else{
         this->ShapeDesc->SetShapeFromString( Attribute );
     }
 }
 if(!this->Name) this->SetName(GetUnique("Attribute_"));
+free((void*)Attribute);
 return( XDMF_SUCCESS );
 }
 
@@ -267,9 +313,7 @@ if( this->AttributeType == XDMF_ATTRIBUTE_TYPE_NONE ){
 ValuesNode = this->DOM->FindDataElement( 0, Element );
 if( ValuesNode ){
   ValueReader.SetDOM( this->DOM );
-#ifndef XDMF_NO_MPI
   ValueReader.SetDsmBuffer(this->DsmBuffer);
-#endif
   if( this->ValuesAreMine && this->Values ){
     delete this->Values;
     this->Values = NULL;
