@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfAttribute.h,v 1.7 2008-01-31 21:58:55 clarke Exp $  */
-/*  Date : $Date: 2008-01-31 21:58:55 $ */
-/*  Version : $Revision: 1.7 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -33,6 +33,8 @@
 #define XDMF_ATTRIBUTE_TYPE_VECTOR  2
 #define XDMF_ATTRIBUTE_TYPE_TENSOR  3
 #define XDMF_ATTRIBUTE_TYPE_MATRIX  4
+#define XDMF_ATTRIBUTE_TYPE_TENSOR6 5
+#define XDMF_ATTRIBUTE_TYPE_GLOBALID 6
 
 // Where Values are Assigned
 #define XDMF_ATTRIBUTE_CENTER_GRID  0
@@ -43,6 +45,7 @@
 
 class XdmfTopology;
 class XdmfDataDesc;
+class XdmfDataItem;
 class XdmfArray;
 
 //! Class for Scalar, Vector, and Tensor Computed Data
@@ -55,7 +58,7 @@ class XdmfArray;
     \verbatim
     XML Element Name : Attribute
     XML Attribute : Name
-    XML Attribute : AttributeType = Scalar* | Vector | Tensor | Tensor6 | Matrix
+    XML Attribute : AttributeType = Scalar* | Vector | Tensor | Tensor6 | Matrix | GlobalId
     XML Attribute : Center = Node* | Cell | Grid | Face | Edge
 
     Example :
@@ -85,6 +88,13 @@ public:
 //! Return the Attribute Type
   XdmfGetValueMacro( AttributeType, XdmfInt32 );
 
+//! Set the number of values to be written to Light Data before switching to Heavy Data
+  XdmfSetValueMacro(LightDataLimit, XdmfInt32)
+//! Gets the number of values to be written to Light Data before switching to Heavy Data
+  XdmfGetValueMacro(LightDataLimit, XdmfInt32)
+
+//! Sets if the Attribute is Active
+  XdmfSetValueMacro( Active, XdmfInt32 );
 //! Return the if the Attribute is Active
   XdmfGetValueMacro( Active, XdmfInt32 );
 
@@ -109,6 +119,13 @@ public:
 //! Returns the Center of the Attribute
   XdmfGetValueMacro( AttributeCenter, XdmfInt32 );
 
+// PATCH September 09, Ian Curington, HR Wallingford Ltd.
+//! Get the Units
+    XdmfGetStringMacro(Units);
+//! Set the Units
+    XdmfSetStringMacro(Units);
+// end patch
+
 //! Returns the Shape of the attribute
   XdmfDataDesc *GetShapeDesc( void ) { return( this->ShapeDesc ); };
 
@@ -126,6 +143,7 @@ public:
 //! Release Big Data
  XdmfInt32 Release();
 protected:
+  XdmfDataItem *GetDataItem();
 
   XdmfInt32  AttributeType;
   XdmfInt32  AttributeCenter;
@@ -133,6 +151,8 @@ protected:
   XdmfInt32  ValuesAreMine;
   XdmfArray  *Values;
   XdmfInt32  Active;
+  XdmfInt32  LightDataLimit;
+  XdmfString Units;      // Ian Curington, HR Wallingford Ltd.
 };
 
 #endif // __XdmfAttribute_h
