@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfDataDesc.cxx,v 1.6 2008-07-04 13:25:54 biddisco Exp $  */
-/*  Date : $Date: 2008-07-04 13:25:54 $ */
-/*  Version : $Revision: 1.6 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -24,19 +24,22 @@
 /*******************************************************************/
 #include "XdmfDataDesc.h"
 
+#include <cstring>
 #include <stdlib.h>
 
 XdmfDataDesc::XdmfDataDesc() {
+#ifndef XDMF_MEMORY_DEBUG
   H5dont_atexit();
+#endif
   this->DataSpace = H5I_BADID;
   this->DataType = H5Tcopy(H5T_NATIVE_FLOAT);
   this->SelectionType = XDMF_SELECTALL;
   this->NextOffset = 0;
   this->Rank = 1;
+  this->Compression = 0;
   this->ShapeString = 0;
-#ifndef XDMF_NO_MPI
+  this->HeavyDataSetName = 0;
   this->DsmBuffer = NULL;
-#endif
 }
 
 XdmfDataDesc::~XdmfDataDesc() {
@@ -48,7 +51,8 @@ H5E_BEGIN_TRY {
     this->DataSpace = H5I_BADID;
   }
 } H5E_END_TRY;
-  this->SetShapeString(0);
+  delete [] this->ShapeString;
+  delete [] this->HeavyDataSetName;
 }
 
 XdmfInt32
@@ -193,10 +197,10 @@ ostrstream   StringOutput;
 XdmfInt64  i, rank, Dimensions[ XDMF_MAX_DIMENSION ];
 
 rank = this->GetShape( Dimensions );
-for( i = 0 ; i < rank ; i++ ){
+for( i = 0 ; i < rank - 1 ; i++ ){
   StringOutput << XDMF_64BIT_CAST Dimensions[i] << " ";
   }
-StringOutput << ends;
+StringOutput << XDMF_64BIT_CAST Dimensions[i] << ends;
 this->SetShapeString(StringOutput.str());
 StringOutput.rdbuf()->freeze(0);
 return( this->ShapeString );
@@ -572,9 +576,14 @@ dataType =  H5Tget_member_type( this->DataType, Index );
 if( HDF5TypeToXdmfType(dataType) == XDMF_COMPOUND_TYPE ) {
   rank = H5Tget_array_ndims( dataType );
   if( rank <= 0 ){
+    H5Tclose(dataType);
     return( XDMF_FAIL );
     }
+#if (H5_VERS_MAJOR>1)||((H5_VERS_MAJOR==1)&&(H5_VERS_MINOR>=8))
+  H5Tget_array_dims2(dataType, Dims);
+#else
   H5Tget_array_dims(dataType, Dims, NULL );
+#endif
   for( i = 0 ; i < rank ; i++ ){
     Dimensions[i] = Dims[i];
   }
@@ -582,6 +591,7 @@ if( HDF5TypeToXdmfType(dataType) == XDMF_COMPOUND_TYPE ) {
   rank =  1;
   Dimensions[0] = 1;
 }
+H5Tclose(dataType);
 return( rank );
 }
 
@@ -633,7 +643,9 @@ if( Index >  ( H5Tget_nmembers( this->DataType ) - 1 ) ){
   XdmfErrorMessage("Index is Greater than Number of Members");
   return( 0 );
   }
-Length =  H5Tget_size( H5Tget_member_type( this->DataType, Index ) );
+hid_t dataType = H5Tget_member_type( this->DataType, Index );
+Length =  H5Tget_size(dataType);
+H5Tclose(dataType);
 if( Length <= 0 ){
   XdmfErrorMessage("Error Getting Length");
   RetVal = XDMF_FAIL;
@@ -718,7 +730,11 @@ if( ( rank == 1 ) && ( *Dimensions == 1 ) ){
   status = H5Tinsert( this->DataType,
     Name,
     Offset,
+#if (!H5_USE_16_API_DEFAULT && ((H5_VERS_MAJOR>1)||((H5_VERS_MAJOR==1)&&(H5_VERS_MINOR>=8))))
+    H5Tarray_create( HNumberType, rank, Dims));
+#else
     H5Tarray_create( HNumberType, rank, Dims, NULL ));
+#endif
 }
 if( status < 0 ){
   return( XDMF_FAIL );
