@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfDataItem.cxx,v 1.21 2008-08-29 15:08:58 clarke Exp $  */
-/*  Date : $Date: 2008-08-29 15:08:58 $ */
-/*  Version : $Revision: 1.21 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -35,6 +35,7 @@
 #ifdef XDMF_USE_MYSQL
 #include "XdmfValuesMySQL.h"
 #endif
+#include "XdmfValuesBinary.h"
 
 #include <libxml/tree.h>
 
@@ -51,17 +52,20 @@ XdmfDataItem::XdmfDataItem() {
     this->HeavyDataSetName = NULL;
     this->ItemType = XDMF_ITEM_UNIFORM;
     this->Function = NULL;
+    this->ColumnMajor = 0;
+    this->TransposeInMemory = 1;
 }
 
 XdmfDataItem::~XdmfDataItem() {
     XdmfDebug(".... Deleteing DataItem " << this);
     if(this->Array && this->ArrayIsMine ){
-        XdmfDebug("Deleteing array " << this->Array);
+        XdmfDebug("Deleting array " << this->Array);
         delete this->Array;
         XdmfDebug("Done Deleteing array");
     }
     if(this->DataDesc && this->DataDescIsMine) delete this->DataDesc;
     if(this->Values) delete this->Values;
+    delete [] this->HeavyDataSetName;
 }
 
 XdmfInt32
@@ -93,7 +97,7 @@ XdmfDataItem::GetArray(XdmfInt32 Create){
     return(this->Array);
 }
 
-XdmfInt32 
+XdmfInt32
 XdmfDataItem::SetArray(XdmfArray *anArray){
     if(this->Array && this->ArrayIsMine) delete this->Array;
     this->ArrayIsMine = 0;
@@ -101,7 +105,7 @@ XdmfDataItem::SetArray(XdmfArray *anArray){
     return(XDMF_SUCCESS);
 }
 
-XdmfInt32 
+XdmfInt32
 XdmfDataItem::SetDataDesc(XdmfDataDesc *aDataDesc){
     if(this->DataDesc && this->DataDescIsMine) delete this->DataDesc;
     this->DataDescIsMine = 0;
@@ -119,9 +123,7 @@ XdmfDataItem::Copy(XdmfElement *Source){
     // this->SetDOM(ds->GetDOM());
     this->SetFormat(ds->GetFormat());
     this->SetHeavyDataSetName(ds->GetHeavyDataSetName());
-#ifndef XDMF_NO_MPI
     this->SetDsmBuffer(ds->GetDsmBuffer());
-#endif
     this->DataDesc->CopyType(ds->GetDataDesc());
     this->DataDesc->CopyShape(ds->GetDataDesc());
     this->DataDesc->CopySelection(ds->GetDataDesc());
@@ -154,7 +156,8 @@ XdmfInt32 XdmfDataItem::UpdateInformationFunction(){
         case XDMF_ITEM_COORDINATES :
             break;
     }
-return(XDMF_SUCCESS);
+    delete Value;
+    return(XDMF_SUCCESS);
 }
 
 XdmfInt32 XdmfDataItem::UpdateInformationCollection(){
@@ -166,11 +169,12 @@ XdmfInt32 XdmfDataItem::UpdateInformationTree(){
 }
 
 XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
-    XdmfConstString Value;
+    XdmfConstString Value = 0;
     XdmfInt32   Precision = 4;
 
     Value = this->Get("Precision");
     if(Value) Precision = atoi(Value);
+    free((void*)Value);
     Value = this->Get("NumberType");
     // Try DataType
     if(!Value) Value = this->Get("DataType");
@@ -181,12 +185,18 @@ XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
         this->DataDesc->SetNumberType(XDMF_INT8_TYPE);
     } else if(XDMF_WORD_CMP(Value, "UChar")){
         this->DataDesc->SetNumberType(XDMF_UINT8_TYPE);
+    } else if(XDMF_WORD_CMP(Value, "Short")){
+        this->DataDesc->SetNumberType(XDMF_INT16_TYPE);
+    } else if(XDMF_WORD_CMP(Value, "UShort")){
+        this->DataDesc->SetNumberType(XDMF_UINT16_TYPE);
     } else if(XDMF_WORD_CMP(Value, "Int")){
         if(Precision == 8){
             this->DataDesc->SetNumberType(XDMF_INT64_TYPE);
         }else{
             this->DataDesc->SetNumberType(XDMF_INT32_TYPE);
         }
+    } else if(XDMF_WORD_CMP(Value, "UInt")){
+        this->DataDesc->SetNumberType(XDMF_UINT32_TYPE);
     } else {
         if(Precision == 8){
             this->DataDesc->SetNumberType(XDMF_FLOAT64_TYPE);
@@ -194,6 +204,7 @@ XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
             this->DataDesc->SetNumberType(XDMF_FLOAT32_TYPE);
         }
     }
+    free((void*)Value);;
     Value = this->Get("Format");
     // Currently XML or HDF5
     if(XDMF_WORD_CMP(Value, "HDF")){
@@ -206,16 +217,36 @@ XdmfInt32 XdmfDataItem::UpdateInformationUniform(){
         this->SetFormat(XDMF_FORMAT_XML);
     } else if(XDMF_WORD_CMP(Value, "MYSQL")){
         this->SetFormat(XDMF_FORMAT_MYSQL);
+    } else if(XDMF_WORD_CMP(Value, "BINARY")){
+        this->SetFormat(XDMF_FORMAT_BINARY);
     }else if(Value){
         XdmfErrorMessage("Unsupported DataItem Format :" << Value);
+        free((void*)Value);
         return(XDMF_FAIL);
     }
+    free((void*)Value);
     return(XDMF_SUCCESS);
 }
 
 XdmfInt32 XdmfDataItem::UpdateInformation(){
     XdmfConstString Value;
 
+	Value = this->Get("Major");
+	if(Value)
+	{
+		if(XDMF_WORD_CMP(Value,"Column"))
+			this->ColumnMajor = 1;
+		else if(XDMF_WORD_CMP(Value,"Row"))
+			this->ColumnMajor = 0;
+		else
+		{
+        XdmfErrorMessage("invalid major");
+        free((void*)Value);
+        return(XDMF_FAIL);
+		}
+	}
+	free((void*)Value);
+	XdmfDebug("Major = " << this->ColumnMajor);
 
     XdmfDebug("XdmfDataItem::UpdateInformation()");
     if(XdmfElement::UpdateInformation() != XDMF_SUCCESS) return(XDMF_FAIL);
@@ -260,10 +291,12 @@ XdmfInt32 XdmfDataItem::UpdateInformation(){
                 break;
             }
             XdmfErrorMessage("Unknown DataItem Type = " << Value);
+            free((void*)Value);
             return(XDMF_FAIL);
         } while(0);
     }
-    if(this->GetIsReference() && 
+    free((void*)Value);
+    if(this->GetIsReference() &&
         (this->ReferenceElement != this->Element) &&
         (this->GetReferenceObject(this->Element) != this)){
         XdmfDebug("Reference DataItem Copied Info from another ReferenceObject");
@@ -277,6 +310,7 @@ XdmfInt32 XdmfDataItem::UpdateInformation(){
     }
     if(!this->DataDesc) this->DataDesc = new XdmfDataDesc();
     this->DataDesc->SetShapeFromString(Value);
+    free((void*)Value);;
     switch(this->ItemType){
         case XDMF_ITEM_UNIFORM :
             return(this->UpdateInformationUniform());
@@ -432,6 +466,7 @@ XdmfInt32 XdmfDataItem::UpdateFunction(){
     if(Value && ReturnArray){
         ReturnArray->ReformFromString(Value);
     }
+    free((void*)Value);;
     // If only a portion of the DataItem was requested
     // the XdmfValues did not reflect this selection since
     // DataDesc was used to select HyperSlad | Coordinates | Function
@@ -490,15 +525,14 @@ XdmfInt32 XdmfDataItem::Update(){
             this->Values->SetDebug(this->GetDebug());
             // this->SetDsmBuffer(this->Values->GetDsmBuffer());
             // cout << "Setting Values Dsm to " << this->DsmBuffer << endl;
-#ifndef XDMF_NO_MPI
             this->Values->SetDsmBuffer(this->DsmBuffer);
-#endif
             XdmfDebug("Reading Data");
             if(!((XdmfValuesHDF *)this->Values)->Read(this->Array)){
                 XdmfErrorMessage("Reading Values Failed");
                 return(XDMF_FAIL);
             }
             this->SetHeavyDataSetName(this->Values->GetHeavyDataSetName());
+            this->Array->SetHeavyDataSetName(this->Values->GetHeavyDataSetName());
             break;
         case XDMF_FORMAT_XML :
             this->Values->SetDebug(this->GetDebug());
@@ -518,11 +552,134 @@ XdmfInt32 XdmfDataItem::Update(){
             XdmfErrorMessage("XdmfValuesMySQL not enabled in this Xdmf");
             return(XDMF_FAIL);
 #endif
+
+            break;
+        case XDMF_FORMAT_BINARY :
+            this->Values->SetDebug(this->GetDebug());
+            if(!((XdmfValuesBinary *)this->Values)->Read(this->Array)){
+                XdmfErrorMessage("Reading Values Failed");
+                return(XDMF_FAIL);
+            }
             break;
         default :
             XdmfErrorMessage("Unsupported Data Format");
             return(XDMF_FAIL);
     }
+
+	// Support for Fortran matrices (2D arrays) added by Dominik Szczerba, June 2009.
+	// Can be further optimized by e.g. in situ transpose methods
+	// discussed on http://en.wikipedia.org/wiki/In-place_matrix_transposition
+	if(ColumnMajor)
+	{
+		if(TransposeInMemory)
+		{
+			XdmfDebug("Transposing Array " << this->GetHeavyDataSetName());
+			XdmfInt32 rank = 0;
+			XdmfInt64 dims[XDMF_MAX_DIMENSION];
+			rank = GetShape(dims);
+			XdmfDebug("rank = " << rank);
+			if(rank!=2)
+			{
+				XdmfErrorMessage("transpose is only implemented for rank 2 arrays");
+				return(XDMF_FAIL);
+			}
+			// for(int i=0; i<rank; i++)
+				// cerr << "DEBUG: dims[" << i << "] = " << dims[i] << endl;
+
+			const int NI = dims[0], NJ = dims[1];
+
+			void* ArrayT;
+			switch(this->Array->GetNumberType())
+			{
+				case XDMF_FLOAT64_TYPE:
+					ArrayT = (XdmfFloat64*)new XdmfFloat64[NI*NJ];
+					break;
+				case XDMF_INT32_TYPE:
+					ArrayT = (XdmfInt32*)new XdmfInt32[NI*NJ];
+					break;
+				// TODO implement the other types
+				default:
+					XdmfErrorMessage("unknown data type");
+					return(XDMF_FAIL);
+			}
+
+			// XdmfPointer ArrayPointer, ArrayPointerT;
+			XdmfPointer ArrayPointer;
+
+			for(int j=0; j<NJ; j++)
+			{
+				for(int i=0; i<NI; i++)
+				{
+					const int idx  = NI*j+i;
+					const int idxT = NJ*i+j;
+					// cerr << "DEBUG: idx/idxT = " << idx << " " << idxT << endl;
+					switch(this->Array->GetNumberType())
+					{
+						case XDMF_FLOAT64_TYPE:
+							ArrayPointer = (XdmfFloat64*)this->Array->GetDataPointer(idxT);
+							((XdmfFloat64*)ArrayT)[idx] = *(XdmfFloat64*)ArrayPointer;
+							break;
+						case XDMF_INT32_TYPE:
+							ArrayPointer = (XdmfInt32*)this->Array->GetDataPointer(idxT);
+							((XdmfInt32*)ArrayT)[idx] = *(XdmfInt32*)ArrayPointer;
+							break;
+						// TODO implement the other types
+						default:
+							XdmfErrorMessage("unknown data type");
+							return(XDMF_FAIL);
+					}
+				}
+			}
+
+			std::swap(dims[0], dims[1]);
+			Array->Reform(rank,dims);
+
+			for(int l=0; l<NI*NJ; l++)
+			{
+				switch(this->Array->GetNumberType())
+				{
+					case XDMF_FLOAT64_TYPE:
+						// cerr << "DEBUG: l = " << l  << " " << ((XdmfFloat64*)ArrayT)[l] << endl;
+						this->Array->SetValue(l,((XdmfFloat64*)ArrayT)[l]);
+						break;
+					case XDMF_INT32_TYPE:
+						// cerr << "DEBUG: l = " << l << " " << ((XdmfInt32*)ArrayT)[l] << endl;
+						this->Array->SetValue(l,((XdmfInt32*)ArrayT)[l]);
+						break;
+					// TODO implement the other types
+					default:
+						XdmfErrorMessage("unknown data type");
+						return(XDMF_FAIL);
+				}
+			}
+
+			// delete [] ArrayT;
+			switch(this->Array->GetNumberType())
+			{
+				case XDMF_FLOAT64_TYPE:
+					delete [] (XdmfFloat64*)ArrayT;
+					break;
+				case XDMF_INT32_TYPE:
+					delete [] (XdmfInt32*)ArrayT;
+					break;
+				// TODO implement the other types
+				default:
+					XdmfErrorMessage("unknown data type");
+					return(XDMF_FAIL);
+			}
+
+			XdmfDebug("done transpose");
+
+		}
+		else
+		{
+			XdmfErrorMessage("out-of-core transpose not implemented yet");
+			return(XDMF_FAIL);
+		}
+
+
+	}
+
     return(XDMF_SUCCESS);
 }
 
@@ -592,12 +749,33 @@ XdmfInt32 XdmfDataItem::Build(){
         case 4 :
             this->Set("Precision", "4");
             break;
+	case 2 :
+	    this->Set("Precision", "2");
+	    break;
         case 1 :
             this->Set("Precision", "1");
             break;
         default :
             break;
     }
+    switch (this->Format) {
+        case XDMF_FORMAT_HDF :
+            this->Set("Format", "HDF");
+            break;
+        case XDMF_FORMAT_XML :
+            this->Set("Format", "XML");
+            break;
+        case XDMF_FORMAT_MYSQL :
+            this->Set("Format", "MYSQL");
+            break;
+        case XDMF_FORMAT_BINARY :
+            this->Set("Format", "BINARY");
+            break;
+        default :
+            XdmfErrorMessage("Unsupported Data Format");
+            return(XDMF_FAIL);
+    }
+    if(this->BuildFromDataXml(1) == XDMF_SUCCESS) return(XDMF_SUCCESS);
     if(this->CheckValues(this->Format) != XDMF_SUCCESS){
         XdmfErrorMessage("Error Accessing Internal XdmfValues");
         return(XDMF_FAIL);
@@ -607,9 +785,7 @@ XdmfInt32 XdmfDataItem::Build(){
         case XDMF_FORMAT_HDF :
             XdmfDebug("Writing Values in HDF Format");
             Values->SetHeavyDataSetName(this->GetHeavyDataSetName());
-#ifndef XDMF_NO_MPI
             Values->SetDsmBuffer(this->GetDsmBuffer());
-#endif
             if(((XdmfValuesHDF *)Values)->Write(this->Array) != XDMF_SUCCESS){
                 XdmfErrorMessage("Writing Values Failed");
                 return(XDMF_FAIL);
@@ -659,6 +835,9 @@ XdmfDataItem::CheckValues(XdmfInt32 aFormat){
                 return(XDMF_FAIL);
 #endif
                 break;
+            case XDMF_FORMAT_BINARY :
+                this->Values = (XdmfValues *)new XdmfValuesBinary();
+                break;
             default :
                 XdmfErrorMessage("Unsupported Data Format");
                 return(XDMF_FAIL);
