@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfGrid.cxx,v 1.25 2008-09-04 15:46:47 dave.demarle Exp $  */
-/*  Date : $Date: 2008-09-04 15:46:47 $ */
-/*  Version : $Revision: 1.25 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -29,8 +29,10 @@
 #include "XdmfArray.h"
 #include "XdmfTopology.h"
 #include "XdmfGeometry.h"
+#include "XdmfInformation.h"
 #include "XdmfAttribute.h"
 #include "XdmfTime.h"
+#include "XdmfSet.h"
 
 XdmfGrid *HandleToXdmfGrid( XdmfString Source ){
   XdmfObject  *TempObj;
@@ -53,14 +55,19 @@ XdmfGrid::XdmfGrid() {
   this->TopologyIsMine = 1;
   this->Time = new XdmfTime;
   this->TimeIsMine = 1;
+  this->Sets = (XdmfSet **)calloc(1, sizeof( XdmfSet * ));
   this->Attribute = (XdmfAttribute **)calloc(1, sizeof( XdmfAttribute * ));
   this->Children = (XdmfGrid **)calloc(1, sizeof( XdmfGrid * ));
+  this->Informations = (XdmfInformation **)calloc(1, sizeof( XdmfInformation * ));
   this->AssignedAttribute = NULL;
+  this->NumberOfSets = 0;
   this->NumberOfAttributes = 0;
+  this->NumberOfInformations = 0;
   this->GridType = XDMF_GRID_UNSET;
   this->CollectionType = XDMF_GRID_COLLECTION_UNSET;
   this->NumberOfChildren = 0;
   this->Debug = 0;
+  this->BuildTime = 0;
   }
 
 XdmfGrid::~XdmfGrid() {
@@ -68,12 +75,26 @@ XdmfGrid::~XdmfGrid() {
   if( this->GeometryIsMine && this->Geometry ) delete this->Geometry;
   if( this->TopologyIsMine && this->Topology ) delete this->Topology;
   if( this->TimeIsMine && this->Time ) delete this->Time;
-  for ( Index = 0; Index < this->NumberOfAttributes; Index ++ )
-    {
-    delete this->Attribute[Index];
+  for ( Index = 0; Index < this->NumberOfAttributes; Index ++ ){
+    if (this->Attribute[Index]->GetDeleteOnGridDelete()){
+        delete this->Attribute[Index];
     }
-  free(this->Attribute);
   }
+  free(this->Attribute);
+  for ( Index = 0; Index < this->NumberOfChildren; Index ++ ){
+    if (this->Children[Index]->GetDeleteOnGridDelete()){
+        delete this->Children[Index];
+    }
+  }
+  free(this->Children);
+  for ( Index = 0; Index < this->NumberOfSets; Index ++ ){
+    if (this->Sets[Index]->GetDeleteOnGridDelete()){
+        delete this->Sets[Index];
+    }
+  }
+  free(this->Sets);
+  free(this->Informations);
+}
 
 XdmfInt32
 XdmfGrid::Release(){
@@ -86,6 +107,7 @@ XdmfGrid::Release(){
     }
    return(XDMF_SUCCESS);
 }
+
 XdmfInt32
 XdmfGrid::InsertTopology(){
     if(!this->Topology->GetElement()){
@@ -116,12 +138,47 @@ XdmfGrid::Insert( XdmfElement *Child){
         XDMF_WORD_CMP(Child->GetElementName(), "Geometry") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Topology") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Attribute") ||
+        XDMF_WORD_CMP(Child->GetElementName(), "Region") ||
+        XDMF_WORD_CMP(Child->GetElementName(), "Set") ||
         XDMF_WORD_CMP(Child->GetElementName(), "DataItem") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Time") ||
         XDMF_WORD_CMP(Child->GetElementName(), "Information")
         )){
         XdmfInt32   status = XdmfElement::Insert(Child);
-        if((status = XDMF_SUCCESS) && XDMF_WORD_CMP(Child->GetElementName(), "Grid")){
+        if((status == XDMF_SUCCESS) && XDMF_WORD_CMP(Child->GetElementName(), "Set")){
+            XdmfSet *ChildSet = (XdmfSet *)Child;
+            this->NumberOfSets++;
+            this->Sets = ( XdmfSet **)realloc( this->Sets,
+                this->NumberOfSets * sizeof( XdmfSet * ));
+            if(!this->Sets) {
+                XdmfErrorMessage("Realloc of Set List Failed");
+                return(XDMF_FAIL);
+            }
+            this->Sets[this->NumberOfSets - 1] = ChildSet;
+            }
+        if((status == XDMF_SUCCESS) && XDMF_WORD_CMP(Child->GetElementName(), "Information")){
+            XdmfInformation *ChildInfo = (XdmfInformation *)Child;
+            this->NumberOfInformations++;
+            this->Informations = ( XdmfInformation **)realloc( this->Informations,
+                this->NumberOfInformations * sizeof( XdmfInformation * ));
+            if(!this->Informations) {
+                XdmfErrorMessage("Realloc of Information List Failed");
+                return(XDMF_FAIL);
+            }
+            this->Informations[this->NumberOfInformations - 1] = ChildInfo;
+            }
+        if((status == XDMF_SUCCESS) && XDMF_WORD_CMP(Child->GetElementName(), "Attribute")){
+            XdmfAttribute *ChildAttr = (XdmfAttribute *)Child;
+            this->NumberOfAttributes++;
+            this->Attribute = ( XdmfAttribute **)realloc( this->Attribute,
+                this->NumberOfAttributes * sizeof( XdmfAttribute * ));
+            if(!this->Attribute) {
+                XdmfErrorMessage("Realloc of Attribute List Failed");
+                return(XDMF_FAIL);
+            }
+            this->Attribute[this->NumberOfAttributes - 1] = ChildAttr;
+            }
+        if((status == XDMF_SUCCESS) && XDMF_WORD_CMP(Child->GetElementName(), "Grid")){
             XdmfGrid *ChildGrid = (XdmfGrid *)Child;
             XdmfInt32 nchild = this->NumberOfChildren + 1;
             this->Children = (XdmfGrid **)realloc(this->Children, nchild * sizeof(XdmfGrid *));
@@ -133,8 +190,9 @@ XdmfGrid::Insert( XdmfElement *Child){
             }
             return(XDMF_SUCCESS);
         }
+        if (status == XDMF_SUCCESS) return(XDMF_SUCCESS);        
     }else{
-        XdmfErrorMessage("Grid can only Insert Grid | Geometry | Topology | DataItem | Information elements, not a " << Child->GetElementName());
+        XdmfErrorMessage("Grid can only Insert Grid | Geometry | Topology | Attribute | Set | Region | DataItem | Information elements, not a " << Child->GetElementName());
     }
     return(XDMF_FAIL);
 }
@@ -143,6 +201,10 @@ XdmfInt32
 XdmfGrid::Build(){
     if(XdmfElement::Build() != XDMF_SUCCESS) return(XDMF_FAIL);
     this->Set("GridType", this->GetGridTypeAsString());
+    if(this->GridType == XDMF_GRID_COLLECTION){
+        this->Set("CollectionType", this->GetCollectionTypeAsString());
+    }
+    /*
     if((this->GridType & XDMF_GRID_MASK) == XDMF_GRID_UNIFORM){
         if(this->InsertTopology() != XDMF_SUCCESS) return(XDMF_FAIL);
         this->Topology->Build();
@@ -150,7 +212,14 @@ XdmfGrid::Build(){
         this->Geometry->Build();
     }else{
     }
+    */
     if(this->BuildTime && this->Time){
+            if(!this->Time->GetDOM()){
+                if(this->Insert(this->Time) != XDMF_SUCCESS){
+                    XdmfErrorMessage("Grid Cannot Insert current XdmfTime");
+                    return(XDMF_FAIL);
+                }
+            }
             if(this->Time->Build() != XDMF_SUCCESS) return(XDMF_FAIL);
     }
     return(XDMF_SUCCESS);
@@ -243,9 +312,7 @@ XdmfGrid::AssignAttribute( XdmfAttribute *attribute ){
 XdmfInt32 Status = 0;
 
 if( attribute ){
-#ifndef XDMF_NO_MPI
   if(!attribute->GetDsmBuffer()) attribute->SetDsmBuffer(this->DsmBuffer);
-#endif
   attribute->Update();
   // Status = attribute->SetBaseAttribute( this, this->BaseGrid );
   this->AssignedAttribute = attribute;
@@ -316,6 +383,7 @@ if( XDMF_WORD_CMP(this->GetElementType(), "Grid") == 0){
 attribute = this->Get("GridType");
 if(!attribute) attribute = this->Get("Type");
 if( XDMF_WORD_CMP(attribute, "Collection") ){
+    free((void*)attribute);
     this->GridType = XDMF_GRID_COLLECTION;
     attribute = this->Get("CollectionType");
     if(attribute){
@@ -334,11 +402,13 @@ if( XDMF_WORD_CMP(attribute, "Collection") ){
 }else{
     if(attribute){
         XdmfErrorMessage("Unknown Grid Type " << attribute);
+        free((void*)attribute);
         return(XDMF_FAIL);
     }
     // If Type is NULL use default
     this->GridType = XDMF_GRID_UNIFORM;
 }
+free((void*)attribute);
 if( this->GridType & XDMF_GRID_MASK){
     // SubSet Tree or Collection
     XdmfInt32  i, nchild;
@@ -365,18 +435,36 @@ if( this->GridType & XDMF_GRID_MASK){
     }
     anElement = this->DOM->FindElement("Geometry", 0, this->Element);
     for(i=0 ; i < nchild ; i++){
-        node = this->DOM->FindElement("Grid", i, this->Element);
+        if (i==0) {
+          node = this->DOM->FindElement("Grid", i, this->Element);
+        } else {
+          node = this->DOM->FindNextElement("Grid", node);
+        }
+
         if(!node) {
             XdmfErrorMessage("Can't find Child Grid #" << i);
             return(XDMF_FAIL);
         }
         this->Children[i] = new XdmfGrid;
-        if(this->Children[i]->SetDOM(this->DOM) == XDMF_FAIL) return(XDMF_FAIL);
-        if(this->Children[i]->SetElement(node) == XDMF_FAIL) return(XDMF_FAIL);
+        this->Children[i]->SetDeleteOnGridDelete(true);
+        if(this->Children[i]->SetDOM(this->DOM) == XDMF_FAIL) {
+          delete this->Children[i]; 
+          return(XDMF_FAIL);
+        }
+        if(this->Children[i]->SetElement(node) == XDMF_FAIL) {
+          delete this->Children[i];
+          return(XDMF_FAIL);
+        }
         // cout << "Calling update info for child " << i << endl;
-        if(this->Children[i]->UpdateInformation() == XDMF_FAIL) return(XDMF_FAIL);
-        if(this->Time->GetTimeType() != XDMF_TIME_UNSET){
-                if(this->Children[i]->GetTime()->SetTimeFromParent(this->Time, i) != XDMF_SUCCESS) return(XDMF_FAIL);
+        if(this->Children[i]->UpdateInformation() == XDMF_FAIL) {
+          delete this->Children[i];
+          return(XDMF_FAIL);
+        }
+        if(this->Time->GetTimeType() != XDMF_TIME_UNSET) {
+          if(this->Children[i]->GetTime()->SetTimeFromParent(this->Time, i) != XDMF_SUCCESS) {
+            delete this->Children[i];
+            return(XDMF_FAIL);
+          }
         }
     }
     if((this->GridType & XDMF_GRID_MASK) == XDMF_GRID_SUBSET){
@@ -396,6 +484,7 @@ if( this->GridType & XDMF_GRID_MASK){
             select = this->DOM->FindElement("DataItem", 0, this->Element);
             if(!select){
                 XdmfErrorMessage("Section = DataItem but DataItem == 0");
+                free((void*)attribute);
                 return(XDMF_FAIL);
             }
         }else{
@@ -409,6 +498,7 @@ if( this->GridType & XDMF_GRID_MASK){
             }
 
         }
+        free((void*)attribute);
         target = this->Children[0];
         if(!target){
             XdmfErrorMessage("No Target Grid Spceified for Subset");
@@ -475,8 +565,8 @@ if( this->GridType & XDMF_GRID_MASK){
         }
     }
 }
-// Get Attributes
 if(!this->Name) this->SetName( GetUnique("Grid_" ) );
+// Get Attributes
 XdmfInt32 OldNumberOfAttributes = this->NumberOfAttributes;
 this->NumberOfAttributes = this->DOM->FindNumberOfElements("Attribute", this->Element );
 if( this->NumberOfAttributes > 0 ){
@@ -492,14 +582,47 @@ if( this->NumberOfAttributes > 0 ){
       this->NumberOfAttributes * sizeof( XdmfAttribute * ));
   for( Index = 0 ; Index < this->NumberOfAttributes ; Index++ ){
     iattribute = new XdmfAttribute;
-
+    iattribute->SetDeleteOnGridDelete(true);
     this->Attribute[Index] = iattribute;
-    AttributeElement = this->DOM->FindElement( "Attribute", Index, this->Element );
+    if (Index == 0) {
+      AttributeElement = this->DOM->FindElement( "Attribute", Index, this->Element );
+    } else {
+      AttributeElement = this->DOM->FindNextElement( "Attribute", AttributeElement);
+    }
     iattribute->SetDOM( this->DOM );    
     iattribute->SetElement( AttributeElement );
     iattribute->UpdateInformation();
     }
 }
+// Get Sets
+XdmfInt32 OldNumberOfSets = this->NumberOfSets;
+this->NumberOfSets = this->DOM->FindNumberOfElements("Set", this->Element );
+if( this->NumberOfSets > 0 ){
+  XdmfInt32  Index;
+  XdmfSet  *iSet;
+  XdmfXmlNode    lSetElement;
+
+  for ( Index = 0; Index < OldNumberOfSets; Index ++ )
+    {
+    delete this->Sets[Index];
+    }
+  this->Sets = ( XdmfSet **)realloc( this->Sets,
+      this->NumberOfSets * sizeof( XdmfSet * ));
+  for( Index = 0 ; Index < this->NumberOfSets ; Index++ ){
+    iSet = new XdmfSet;
+    iSet->SetDeleteOnGridDelete(true);
+    this->Sets[Index] = iSet;
+    if (Index==0) {
+      lSetElement = this->DOM->FindElement( "Set", Index, this->Element );
+    } else {
+      lSetElement = this->DOM->FindNextElement( "Set", lSetElement);
+    }
+    iSet->SetDOM( this->DOM );    
+    iSet->SetElement( lSetElement );
+    iSet->UpdateInformation();
+    }
+}
+
 return( XDMF_SUCCESS );
 }
 
@@ -520,9 +643,7 @@ if((this->GridType & XDMF_GRID_MASK) != XDMF_GRID_UNIFORM){
     XdmfInt32   i;
     // SubSet, Tree or Collection
     for(i=0; i < this->NumberOfChildren ; i++){
-#ifndef XDMF_NO_MPI
         if(!this->Children[i]->GetDsmBuffer()) this->Children[i]->SetDsmBuffer(this->DsmBuffer);
-#endif
         if(this->Children[i]->Update() == XDMF_FAIL){
             XdmfErrorMessage("Error in Update() of Child Grid " << i);
             return(XDMF_FAIL);
@@ -572,7 +693,7 @@ if((this->GridType & XDMF_GRID_MASK) != XDMF_GRID_UNIFORM){
                     len = o1 - o;
                     if(len > cellsize){
                         cellsize = len + 1;
-                        delete cell;
+                        delete [] cell;
                         cell = new XdmfInt64[ cellsize ];
                     }
 //                    cout << " Conns = " << target->GetTopology()->GetConnectivity()->GetValues(o, len) << endl;
@@ -587,22 +708,18 @@ if((this->GridType & XDMF_GRID_MASK) != XDMF_GRID_UNIFORM){
                 }
                 newconn->SetNumberOfElements(total);
                 this->Topology->SetConnectivity(newconn);
-                delete cell;
+                delete [] cell;
             }
         }
     }
     return(XDMF_SUCCESS);
 }
-#ifndef XDMF_NO_MPI
 if(!this->Topology->GetDsmBuffer()) this->Topology->SetDsmBuffer(this->DsmBuffer);
-#endif
 if(this->Topology->Update() == XDMF_FAIL){
     XdmfErrorMessage("Error in Update() of Topology");
     return(XDMF_FAIL);
 }
-#ifndef XDMF_NO_MPI
 if(!this->Geometry->GetDsmBuffer()) this->Geometry->SetDsmBuffer(this->DsmBuffer);
-#endif
 if(this->Geometry->Update() == XDMF_FAIL){
     XdmfErrorMessage("Error in Update() of Geometry");
     return(XDMF_FAIL);
@@ -624,6 +741,16 @@ XdmfGrid::GetChild(XdmfInt32 Index){
 return(NULL);
 }
 
+XdmfInformation *
+XdmfGrid::GetInformation(XdmfInt32 Index){
+    if(Index < this->NumberOfInformations){
+        return(this->Informations[Index]);
+    }else{
+        XdmfErrorMessage("Grid has " << this->NumberOfInformations << " children. Index " << Index << " is out of range");
+    }
+    return(NULL);
+}
+
 XdmfInt32
 XdmfGrid::IsUniform(){
     if(this->GridType & XDMF_GRID_MASK) return(XDMF_FALSE);
