@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfObject.h,v 1.8 2008-02-08 15:25:42 clarke Exp $  */
-/*  Date : $Date: 2008-02-08 15:25:42 $ */
-/*  Version : $Revision: 1.8 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -89,7 +89,9 @@
 # endif
 
 # if defined(WIN32) && !defined(__CYGWIN__)
-#  include "winsock.h"
+/* winsock.h seems not being used in Xdmf, but
+*  will cause conflicts with projects that includes "winsock2.h" */
+/* #  include "winsock.h" */
 /* String comparison routine. */
 #  define STRCASECMP _stricmp
 #  define STRNCASECMP _strnicmp
@@ -291,6 +293,16 @@ need to make sure ....
 //  }
 #endif /* SWIGTCL */
 
+#ifdef SWIGCSHARP
+%echo "Applying Special C# Typemaps .... "
+%typemap(cscode) XdmfObject %{
+  // Ensure that the GC doesn't collect any inserted XdmfElement instance set from C#
+  public void setCMemOwn(bool val) {
+    swigCMemOwn = val;
+  }
+%}
+#endif
+
 #ifdef SWIGPYTHON
 %echo "Applying Python Typemaps ..... "
 %include typemaps.i
@@ -366,7 +378,7 @@ need to make sure ....
   }
 
 #define XDMF_WORD_TRIM( a ) { \
-  int             StringLength; \
+  size_t          StringLength; \
   char            *fp; \
   char            *st; \
   char            *ed; \
