@@ -2,9 +2,9 @@
 /*                               XDMF                              */
 /*                   eXtensible Data Model and Format              */
 /*                                                                 */
-/*  Id : $Id: XdmfTopology.cxx,v 1.17 2008-05-02 14:58:15 clarke Exp $  */
-/*  Date : $Date: 2008-05-02 14:58:15 $ */
-/*  Version : $Revision: 1.17 $ */
+/*  Id : Id  */
+/*  Date : $Date$ */
+/*  Version : $Revision$ */
 /*                                                                 */
 /*  Author:                                                        */
 /*     Jerry A. Clarke                                             */
@@ -50,6 +50,7 @@ XdmfTopology::XdmfTopology() {
   this->ConnectivityIsMine = 1;
   this->OrderIsDefault = 1;
   this->BaseOffset = 0;
+  this->LightDataLimit = 100;
   }
 
 XdmfTopology::~XdmfTopology() {
@@ -67,12 +68,28 @@ XdmfTopology::Release(){
     return(XDMF_SUCCESS);
 }
 
+XdmfDataItem * XdmfTopology::GetDataItem(){
+    XdmfDataItem *di = NULL;
+    XdmfXmlNode Node = this->DOM->FindDataElement(0, this->GetElement());
+
+    if(Node) {
+        di = (XdmfDataItem *)this->GetCurrentXdmfElement(Node);
+    }
+    if(!di){
+        di = new XdmfDataItem;
+        Node = this->DOM->InsertNew(this->GetElement(), "DataItem");
+        di->SetDOM(this->DOM);
+        di->SetElement(Node);
+    }
+    return(di);
+} 
+
 XdmfInt32
 XdmfTopology::Build(){
     if(XdmfElement::Build() != XDMF_SUCCESS) return(XDMF_FAIL);
     this->Set("TopologyType", this->GetTopologyTypeAsString());
     if(this->GetNumberOfElements()){
-        this->Set("NumberOfElements", this->Shape->GetShapeAsString());
+        this->Set("Dimensions", this->Shape->GetShapeAsString());
     }
     if( this->OrderIsDefault == 0 ){
         this->Set("Order", this->GetOrderAsString());
@@ -83,27 +100,18 @@ XdmfTopology::Build(){
         OffsetStream << this->BaseOffset << ends;
         this->Set("BaseOffset", Offset);
     }
+    if(this->BuildFromDataXml() == XDMF_SUCCESS) return(XDMF_SUCCESS);
     if(this->Connectivity){
-        XdmfDataItem    *di = NULL;
-        XdmfXmlNode     node;
-        //! Is there already a DataItem
-        node = this->DOM->FindDataElement(0, this->GetElement());
-        if(node) {
-            di = (XdmfDataItem *)this->GetCurrentXdmfElement(node);
-        }
-        if(!di){
-            di = new XdmfDataItem;
-            node = this->DOM->InsertNew(this->GetElement(), "DataItem");
-            di->SetDOM(this->DOM);
-            di->SetElement(node);
-        }
+        XdmfDataItem * di = this->GetDataItem();
         di->SetArray(this->Connectivity);
-        if(this->Connectivity->GetNumberOfElements() > 100) di->SetFormat(XDMF_FORMAT_HDF);
+        if(this->Connectivity->GetNumberOfElements() > this->LightDataLimit) di->SetFormat(XDMF_FORMAT_HDF);
         di->Build();
-
+        this->SetCurrentXdmfElement(di->GetElement(), NULL);
+        delete di;
     }
     return(XDMF_SUCCESS);
 }
+
 XdmfInt32
 XdmfTopology::Insert( XdmfElement *Child){
     if(Child && (
@@ -166,16 +174,30 @@ if ( XDMF_WORD_CMP( topologyType, "NOTOPOLOGY") ){
   newTopologyType = XDMF_EDGE_3;
 } else if( XDMF_WORD_CMP( topologyType, "TRI_6") ){
   newTopologyType = XDMF_TRI_6;
+} else if( XDMF_WORD_CMP( topologyType, "TRIANGLE_6") ){
+  newTopologyType = XDMF_TRI_6;
 } else if( XDMF_WORD_CMP( topologyType, "QUAD_8") ){
   newTopologyType = XDMF_QUAD_8;
+} else if( XDMF_WORD_CMP( topologyType, "QUADRILATERAL_8") ){
+  newTopologyType = XDMF_QUAD_8;
 } else if( XDMF_WORD_CMP( topologyType, "TET_10") ){
   newTopologyType = XDMF_TET_10;
+} else if( XDMF_WORD_CMP( topologyType, "TETRAHEDRON_10") ){
+  newTopologyType = XDMF_TET_10;
 } else if( XDMF_WORD_CMP( topologyType, "PYRAMID_13") ){
   newTopologyType = XDMF_PYRAMID_13;
 } else if( XDMF_WORD_CMP( topologyType, "WEDGE_15") ){
   newTopologyType = XDMF_WEDGE_15;
+} else if( XDMF_WORD_CMP( topologyType, "WEDGE_18") ){
+  newTopologyType = XDMF_WEDGE_18;
 } else if( XDMF_WORD_CMP( topologyType, "HEX_20") ){
   newTopologyType = XDMF_HEX_20;
+} else if( XDMF_WORD_CMP( topologyType, "HEXAHEDRON_20") ){
+  newTopologyType = XDMF_HEX_20;
+} else if( XDMF_WORD_CMP( topologyType, "HEXAHEDRON_24") ){
+  newTopologyType = XDMF_HEX_24;
+} else if( XDMF_WORD_CMP( topologyType, "HEXAHEDRON_27") ){
+  newTopologyType = XDMF_HEX_27;
 } else if( XDMF_WORD_CMP( topologyType, "MIXED") ){
   newTopologyType = XDMF_MIXED;
 } else if( XDMF_WORD_CMP( topologyType, "2DSMESH") ){
@@ -230,13 +252,19 @@ XdmfTopology::GetTopologyTypeAsString( void ) {
     case  XDMF_QUAD_8 :
       return("Quadrilateral_8");
     case  XDMF_TET_10 :
-      return("Tetrehedron_10");
+      return("Tetrahedron_10");
     case  XDMF_PYRAMID_13 :
       return("Pyramid_13");
     case  XDMF_WEDGE_15 :
       return("Wedge_15");
+    case  XDMF_WEDGE_18 :
+      return("Wedge_18");
     case  XDMF_HEX_20 :
       return("Hexahedron_20");
+    case  XDMF_HEX_24 :
+      return("Hexahedron_24");
+    case  XDMF_HEX_27 :
+      return("Hexahedron_27");  
     case XDMF_MIXED :
       return("Mixed");
     case  XDMF_2DSMESH :
@@ -310,9 +338,18 @@ XdmfInt32  nodesPerElement = 0;
     case  XDMF_WEDGE_15 :
       nodesPerElement = 15;
       break;
+    case  XDMF_WEDGE_18 :
+      nodesPerElement = 18;
+      break;
     case  XDMF_HEX_20 :
       nodesPerElement = 20;
       break;
+    case  XDMF_HEX_24 :
+      nodesPerElement = 24;
+      break;
+    case  XDMF_HEX_27 :
+      nodesPerElement = 27;
+      break;
     case  XDMF_MIXED :
       nodesPerElement = 0;
       break;
@@ -480,9 +517,18 @@ if(this->TopologyType == XDMF_MIXED){
             case  XDMF_WEDGE_15 :
               npe = 15;
               break;
+            case  XDMF_WEDGE_18 :
+              npe = 18;
+              break;
             case  XDMF_HEX_20 :
               npe = 20;
               break;
+            case  XDMF_HEX_24 :
+              npe = 24;
+              break;
+            case  XDMF_HEX_27 :
+              npe = 27;
+              break;
             default :
                 XdmfErrorMessage("Unknown Cell Type = " << celltype);
                 break;
@@ -540,17 +586,21 @@ Attribute = this->Get( "TopologyType" );
 if(!Attribute) Attribute = this->Get( "Type" );
 if( this->SetTopologyTypeFromString( Attribute ) == XDMF_FAIL ){
   XdmfErrorMessage("Bad Topology Type : " << Attribute );
+  free((void*)Attribute);
   return( XDMF_FAIL );
   }
+free((void*)Attribute);
 // Set Shape Either Way
 Attribute = this->Get( "NumberOfElements" );
 if( Attribute ){
   this->GetShapeDesc()->SetShapeFromString( Attribute );
   }
+free((void*)Attribute);
 Attribute = this->Get( "Dimensions" );
 if( Attribute ){
   this->GetShapeDesc()->SetShapeFromString( Attribute );
 }
+free((void*)Attribute);
 Attribute = this->Get( "NodesPerElement" );
 if( Attribute ){
   XdmfInt64 nodesPerElement;
@@ -558,14 +608,17 @@ if( Attribute ){
   nodesPerElement = strtol( Attribute, (XdmfString *)NULL, 0 );
   this->SetNodesPerElement( nodesPerElement );
   }
+free((void*)Attribute);
 Attribute = this->Get( "Order" );
 if( Attribute ){
   this->SetOrderFromString( Attribute );
   }
+free((void*)Attribute);
 Attribute = this->Get( "BaseOffset" );
 if( Attribute ){
   this->BaseOffset = strtol( Attribute, (XdmfString *)NULL, 0);
   }
+free((void*)Attribute);
 if(!this->Name) this->SetName(GetUnique("Topology_"));
 return( XDMF_SUCCESS );
 }
@@ -591,9 +644,7 @@ if( this->GetClass() == XDMF_UNSTRUCTURED ){
     // cout << "Connection DataItem = " << &Connections << endl;
     XdmfDebug("Reading Connections from DataItem");
     if(Connections.SetDOM(this->DOM) == XDMF_FAIL) return(XDMF_FAIL);
-#ifndef XDMF_NO_MPI
     Connections.SetDsmBuffer(this->DsmBuffer);
-#endif
     if( this->ConnectivityIsMine && this->Connectivity ) delete this->Connectivity;
     if(Connections.SetElement(ConnectionElement, 0) == XDMF_FAIL) return(XDMF_FAIL);
     if(Connections.UpdateInformation() == XDMF_FAIL) return(XDMF_FAIL);
@@ -651,3 +702,109 @@ if( this->GetClass() == XDMF_UNSTRUCTURED ){
 }
 return( XDMF_SUCCESS );
 }
+
+XdmfInt32
+XdmfTopology::GetEdgesPerElement() {
+
+  switch ( this->TopologyType ){
+    case  XDMF_NOTOPOLOGY :
+      return 0;
+    case  XDMF_POLYVERTEX :
+      return 0;
+    case  XDMF_POLYLINE :
+      return this->NodesPerElement - 1;
+    case  XDMF_POLYGON :
+      return this->NodesPerElement;
+    case  XDMF_TRI :
+      return 3;
+    case  XDMF_QUAD :
+      return 4;
+    case  XDMF_TET :
+      return 6;
+    case  XDMF_PYRAMID :
+      return 8;
+    case  XDMF_WEDGE :
+      return 9;
+    case  XDMF_HEX :
+      return 12;
+    case  XDMF_EDGE_3 :
+      return 1;
+    case  XDMF_TRI_6 :
+      return 3;
+    case  XDMF_QUAD_8 :
+      return 4;
+    case  XDMF_TET_10 :
+      return 6;
+    case  XDMF_PYRAMID_13 :
+      return 8;
+    case  XDMF_WEDGE_15 :
+    case  XDMF_WEDGE_18 :
+      return 9;
+    case  XDMF_HEX_20 :
+    case  XDMF_HEX_24 :
+    case  XDMF_HEX_27 :
+      return 12;
+    case  XDMF_MIXED :
+    case  XDMF_2DSMESH :
+    case  XDMF_2DRECTMESH :
+    case  XDMF_2DCORECTMESH :
+    case  XDMF_3DSMESH :
+    case  XDMF_3DRECTMESH :
+    case  XDMF_3DCORECTMESH :
+    default :
+      return 0;
+  }
+}
+
+XdmfInt32
+XdmfTopology::GetFacesPerElement() {
+
+  switch ( this->TopologyType ){
+    case  XDMF_NOTOPOLOGY :
+      return 0;
+    case  XDMF_POLYVERTEX :
+      return 0;
+    case  XDMF_POLYLINE :
+      return 0;
+    case  XDMF_POLYGON :
+      return 1;
+    case  XDMF_TRI :
+      return 1;
+    case  XDMF_QUAD :
+      return 1;
+    case  XDMF_TET :
+      return 4;
+    case  XDMF_PYRAMID :
+      return 5;
+    case  XDMF_WEDGE :
+      return 5;
+    case  XDMF_HEX :
+      return 6;
+    case  XDMF_EDGE_3 :
+      return 0;
+    case  XDMF_TRI_6 :
+      return 1;
+    case  XDMF_QUAD_8 :
+      return 1;
+    case  XDMF_TET_10 :
+      return 4;
+    case  XDMF_PYRAMID_13 :
+      return 5;
+    case  XDMF_WEDGE_15 :
+    case  XDMF_WEDGE_18 :
+      return 5;
+    case  XDMF_HEX_20 :
+    case  XDMF_HEX_24 :
+    case  XDMF_HEX_27 :
+      return 6;
+    case  XDMF_MIXED :
+    case  XDMF_2DSMESH :
+    case  XDMF_2DRECTMESH :
+    case  XDMF_2DCORECTMESH :
+    case  XDMF_3DSMESH :
+    case  XDMF_3DRECTMESH :
+    case  XDMF_3DCORECTMESH :
+    default :
+      return 0;
+  }
+}
