@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBoundingBox.cxx,v $
+  Module:    vtkBoundingBox.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -16,6 +16,7 @@
 #include <assert.h>
 #include <math.h>
 
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::AddPoint(double px, double py, double pz)
 {
   double p[3];
@@ -25,6 +26,7 @@ void vtkBoundingBox::AddPoint(double px, double py, double pz)
   this->AddPoint(p);
 }
 
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::AddPoint(double p[3])
 {
   int i;
@@ -42,6 +44,7 @@ void vtkBoundingBox::AddPoint(double p[3])
     }
 }
 
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::AddBox(const vtkBoundingBox &bbox)
 {
   int i;
@@ -58,8 +61,7 @@ void vtkBoundingBox::AddBox(const vtkBoundingBox &bbox)
       }
     }
 }
-
-
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::AddBounds(double bounds[6])
 {
   if (bounds[0] < this->MinPnt[0])
@@ -92,7 +94,8 @@ void vtkBoundingBox::AddBounds(double bounds[6])
     this->MaxPnt[2] = bounds[5];
     }
 }
-    
+
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::SetBounds(double xMin, double xMax,
                               double yMin, double yMax,
                               double zMin, double zMax)
@@ -114,6 +117,7 @@ void vtkBoundingBox::SetBounds(double xMin, double xMax,
     }
 }
 
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::SetMinPoint(double x, double y, double z)
 {
   this->MinPnt[0] = x;
@@ -136,6 +140,7 @@ void vtkBoundingBox::SetMinPoint(double x, double y, double z)
   
 }
 
+// ---------------------------------------------------------------------------
 void vtkBoundingBox::SetMaxPoint(double x, double y, double z)
 {
   this->MaxPnt[0] = x;
@@ -154,9 +159,19 @@ void vtkBoundingBox::SetMaxPoint(double x, double y, double z)
   if (z < this->MinPnt[2])
     {
     this->MinPnt[2] = z;
-    }  
+    }
 }
-
+// ---------------------------------------------------------------------------
+void vtkBoundingBox::Inflate(double delta)
+{
+  this->MinPnt[0] -= delta;
+  this->MaxPnt[0] += delta;
+  this->MinPnt[1] -= delta;
+  this->MaxPnt[1] += delta;
+  this->MinPnt[2] -= delta;
+  this->MaxPnt[2] += delta;
+}
+// ---------------------------------------------------------------------------
 int vtkBoundingBox::IntersectBox(const vtkBoundingBox &bbox)
 {
   // if either box is not valid don't do the opperation
@@ -180,7 +195,7 @@ int vtkBoundingBox::IntersectBox(const vtkBoundingBox &bbox)
              (this->MinPnt[i] <= bbox.MaxPnt[i]))
       {
       intersects = 1;
-      pMin[i] = this->MinPnt[i];      
+      pMin[i] = this->MinPnt[i];
       }
     if ((bbox.MaxPnt[i] >= this->MinPnt[i]) &&
         (bbox.MaxPnt[i] <= this->MaxPnt[i]))
@@ -209,6 +224,7 @@ int vtkBoundingBox::IntersectBox(const vtkBoundingBox &bbox)
   return 1;
 }
 
+// ---------------------------------------------------------------------------
 int vtkBoundingBox::Intersects(const vtkBoundingBox &bbox) const
 {
   // if either box is not valid they don't intersect
@@ -244,6 +260,26 @@ int vtkBoundingBox::Intersects(const vtkBoundingBox &bbox) const
   return 1;
 }
 
+// ---------------------------------------------------------------------------
+int vtkBoundingBox::Contains(const vtkBoundingBox &bbox) const
+{
+  // if either box is not valid or they don't intersect
+  if (!this->Intersects(bbox))
+    {
+    return 0;
+    }
+  const double* pt = bbox.GetMinPoint();
+  if (!this->ContainsPoint(pt[0],pt[1],pt[2]))
+    {
+    return 0;
+    }
+  pt = bbox.GetMaxPoint();
+  if (!this->ContainsPoint(pt[0],pt[1],pt[2]))
+    {
+    return 0;
+    }
+  return 1;
+}
 
 // ---------------------------------------------------------------------------
 double vtkBoundingBox::GetMaxLength() const
