@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkFunctionParser.cxx,v $
+  Module:    vtkFunctionParser.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -17,7 +17,6 @@
 
 #include <ctype.h>
 
-vtkCxxRevisionMacro(vtkFunctionParser, "$Revision: 1.40 $");
 vtkStandardNewMacro(vtkFunctionParser);
 
 static double vtkParserVectorErrorResult[3] = { VTK_PARSER_ERROR_RESULT, 
@@ -1327,7 +1326,8 @@ void vtkFunctionParser::RemoveSpaces()
   
   delete [] this->Function;
   this->Function = new char[this->FunctionLength+1];
-  strncpy(this->Function, tempString, this->FunctionLength);
+  strncpy(this->Function, tempString,
+          static_cast<size_t>(this->FunctionLength));
   this->Function[this->FunctionLength] = '\0';
   delete [] tempString;
 }
@@ -1335,43 +1335,71 @@ void vtkFunctionParser::RemoveSpaces()
 int vtkFunctionParser::OperatorWithinVariable(int idx)
 {
   int i;
-  char *tmpString;
+  char *tmpString = NULL;
   int start, end;
   
-  for (i = 0; i < this->NumberOfScalarVariables; i++)
+  for ( i = 0;  i < this->NumberOfScalarVariables;  i ++ )
     {
-    if (strchr(this->ScalarVariableNames[i], this->Function[idx]) != 0)
+    end = 0;
+
+    if (  strchr( this->ScalarVariableNames[i], this->Function[idx] ) != 0  )
       {
-      tmpString = strstr(this->Function, this->ScalarVariableNames[i]);
-      if (tmpString)
+      if (    (  tmpString = strstr( this->Function, this->ScalarVariableNames[i] )  )    )
         {
-        start = static_cast<int>(tmpString - this->Function);
-        end = start + static_cast<int>(strlen(this->ScalarVariableNames[i]));
-        if (start <= idx && end >= idx)
+        do
           {
-          return 1;
-          }
-        }
-      }
-    }
-  for (i = 0; i < this->NumberOfVectorVariables; i++)
-    {
-    if (strchr(this->VectorVariableNames[i], this->Function[idx]) != 0)
-      {
-      tmpString = strstr(this->Function, this->VectorVariableNames[i]);
-      if (tmpString)
-        {
-        start = static_cast<int>(tmpString - this->Function);
-        end = start + static_cast<int>(strlen(this->VectorVariableNames[i]));
-        if (start <= idx && end >= idx)
-          {
-          return 1;
-          }
+          if (tmpString)
+            {
+            start = static_cast<int>(tmpString - this->Function);
+            end   = start + static_cast<int>( strlen(this->ScalarVariableNames[i]) );
+
+            // the variable being investigated does contain an operator (at idx)
+            if ( start <= idx && idx <= end )  return  1;       
+
+            // just in case of one or even more occurrences of the
+            // variable name (being investigated) preceding "idx" in this->Function[]
+            // As suggested by 7islands, a greedy search is used here
+            if ( end <= idx )  // to save strstr()  whenever possible
+            tmpString = strstr( this->Function + end, this->ScalarVariableNames[i] );
+            }
+          else  break;
+          } while ( end <= idx );
         }
       }
     }
   
-  return 0;
+  for ( i = 0;  i < this->NumberOfVectorVariables;  i ++ )
+    {
+    end = 0;
+
+    if (  strchr( this->VectorVariableNames[i], this->Function[idx] ) != 0  )
+      {
+      if (    (  tmpString = strstr( this->Function, this->VectorVariableNames[i] )  )    )
+        {
+        do
+          {
+          if (tmpString)
+            {
+            start = static_cast<int>(tmpString - this->Function);
+            end   = start + static_cast<int>( strlen(this->VectorVariableNames[i]) );
+
+            // the variable being investigated does contain an operator (at idx)
+            if ( start <= idx && idx <= end )  return  1;       
+
+            // just in case of one or even more occurrences of the
+            // variable name (being investigated) preceding "idx" in this->Function[]
+            // As suggested by 7islands, a greedy search is used here
+            if ( end <= idx )  // to save strstr()  whenever possible
+            tmpString = strstr( this->Function + end, this->VectorVariableNames[i] );
+            }
+          else  break;
+          } while ( end <= idx );
+        }
+      }
+    }
+
+  tmpString = NULL;
+  return  0;
 }
 
 int vtkFunctionParser::CheckSyntax()
@@ -1411,28 +1439,32 @@ int vtkFunctionParser::CheckSyntax()
       }
 
     // Check for math function
-    if ((functionNumber = this->GetMathFunctionNumber(index)))
+    if ((functionNumber = this->GetMathFunctionNumberByCheckingParenthesis(index)))
       {
       if ((functionNumber == VTK_PARSER_MIN) ||
-          (functionNumber == VTK_PARSER_MAX))
+          (functionNumber == VTK_PARSER_MAX) ||
+          (functionNumber == VTK_PARSER_CROSS))
         {
         expectCommaOnParenthesisCount[parenthesisCount+1] = 1;
         }
-      if ((functionNumber == VTK_PARSER_IF) ||
-          (functionNumber == VTK_PARSER_CROSS))
+      if (functionNumber == VTK_PARSER_IF)
         {
         expectTwoCommasOnParenthesisCount[parenthesisCount+1] = 1;
         }
       index += this->GetMathFunctionStringLength(functionNumber);
       currentChar = this->Function[index];
-      if ( currentChar != '(' )
-        {
-        vtkErrorMacro("Syntax error: input to math function not in "
-                      << "parentheses; see position " << index);
-        delete [] expectCommaOnParenthesisCount;
-        delete [] expectTwoCommasOnParenthesisCount;
-        return 0;
-        }
+      
+      // == currentChar should always be '(' here == a fix to Bug #9208
+      // since GetMathFunctionNumberByCheckingParenthesis() is employed above
+      
+      //if ( currentChar != '(' )
+      //  {
+      //  vtkErrorMacro("Syntax error: input to math function not in "
+      //                << "parentheses; see position " << index);
+      //  delete [] expectCommaOnParenthesisCount;
+      //  delete [] expectTwoCommasOnParenthesisCount;
+      //  return 0;
+      //  }
       }
     
     // Check for opening parenthesis
@@ -1447,7 +1479,10 @@ int vtkFunctionParser::CheckSyntax()
     if(isdigit(currentChar) ||
        (currentChar == '.' && isdigit(this->Function[index+1])))
       {
-      strtod(&this->Function[index], &ptr);
+      double value=strtod(&this->Function[index], &ptr);
+      // ignore the return value, we just try to figure out
+      // the position of the pointer after the double value.
+      static_cast<void>(value);
       index += int(ptr-&this->Function[index]);
       currentChar = this->Function[index];
       }
@@ -1719,7 +1754,8 @@ void vtkFunctionParser::BuildInternalSubstringStructure(int beginIndex,
               {
               this->BuildInternalSubstringStructure(beginIndex2+1, i-1);
               this->BuildInternalSubstringStructure(i+1, endIndex-1);
-              this->AddInternalByte(mathFunctionNum);
+              this->AddInternalByte(
+                static_cast<unsigned char>(mathFunctionNum));
               this->StackPointer--;
               return;
               }
@@ -1757,7 +1793,8 @@ void vtkFunctionParser::BuildInternalSubstringStructure(int beginIndex,
                 this->BuildInternalSubstringStructure(i+1, secondCommaIndex-1);
                 // first arg
                 this->BuildInternalSubstringStructure(beginIndex2+1, i-1);
-                this->AddInternalByte(mathFunctionNum);
+                this->AddInternalByte(
+                  static_cast<unsigned char>(mathFunctionNum));
                 this->StackPointer--;
                 return;
                 }
@@ -1766,7 +1803,7 @@ void vtkFunctionParser::BuildInternalSubstringStructure(int beginIndex,
           } // VTK_PARSER_IF, ...
 
         this->BuildInternalSubstringStructure(beginIndex2+1, endIndex-1);
-        this->AddInternalByte(mathFunctionNum);
+        this->AddInternalByte(static_cast<unsigned char>(mathFunctionNum));
         return;
         } // if (this->IsSubstringCompletelyEnclosed ... )
       } // if (mathFunctionNum > 0)
@@ -1888,7 +1925,10 @@ int vtkFunctionParser::IsSubstringCompletelyEnclosed(int beginIndex,
 }
 
 int vtkFunctionParser::GetMathFunctionNumber(int currentIndex)
-{
+{ 
+  // For addition of any new math function, please update 
+  // function GetMathFunctionNumberByCheckingParenthesis()
+  
   if (strncmp(&this->Function[currentIndex], "abs", 3) == 0)
     {
     return VTK_PARSER_ABSOLUTE_VALUE;
@@ -1991,6 +2031,81 @@ int vtkFunctionParser::GetMathFunctionNumber(int currentIndex)
   return 0;
 }
 
+int vtkFunctionParser::GetMathFunctionNumberByCheckingParenthesis
+  ( int currentIndex )
+{ 
+  // This function assumes that RemoveSpaces() has been called and 
+  // hence involves the check on the '(' that immediately follows a
+  // valid function. Addressing '(' here instead of in CheckSyntax()
+  // allows for early detection of grammar errors, i.e., lack of '(',
+  // and hence simplifies the parsing process.
+  
+  // For addition of any new math function, please update NUMBFUNCS
+  // and add an entry to each of the three arrays below.
+  
+  const  int  NUMBFUNCS                =  24;
+  
+  static int  charsLens[NUMBFUNCS]     = { 4,       4,        5,       6,
+                                           3,       6,        4,       5,
+                                           4,       5,        4,       5,
+                                           4,       5,        5,       5,
+                                           5,       4,        4,       6,
+                                           5,       4,        5,       3
+                                         };
+                                  
+  static int  funcNumbs[NUMBFUNCS]     = { VTK_PARSER_ABSOLUTE_VALUE,
+                                           VTK_PARSER_EXPONENT,
+                                           VTK_PARSER_CEILING,
+                                           VTK_PARSER_FLOOR,
+                                    
+                                           VTK_PARSER_LOGARITHME,
+                                           VTK_PARSER_LOGARITHM10,
+                                           VTK_PARSER_LOGARITHM,
+                                           VTK_PARSER_SQUARE_ROOT,
+                                    
+                                           VTK_PARSER_SINE,
+                                           VTK_PARSER_HYPERBOLIC_SINE,
+                                           VTK_PARSER_COSINE,
+                                           VTK_PARSER_HYPERBOLIC_COSINE,
+                                    
+                                           VTK_PARSER_TANGENT,
+                                           VTK_PARSER_HYPERBOLIC_TANGENT,
+                                           VTK_PARSER_ARCSINE,
+                                           VTK_PARSER_ARCCOSINE,
+                                    
+                                           VTK_PARSER_ARCTANGENT,
+                                           VTK_PARSER_MIN,
+                                           VTK_PARSER_MAX,
+                                           VTK_PARSER_CROSS,
+                                    
+                                           VTK_PARSER_SIGN,
+                                           VTK_PARSER_MAGNITUDE,
+                                           VTK_PARSER_NORMALIZE,
+                                           VTK_PARSER_IF
+                                         };
+  
+  static char funcNames[NUMBFUNCS][10] = { "abs(",  "exp(",   "ceil(", "floor(", 
+                                           "ln(",   "log10(", "log(",  "sqrt(", 
+                                           "sin(",  "sinh(",  "cos(",  "cosh(",
+                                           "tan(",  "tanh(",  "asin(", "acos(",
+                                           "atan(", "min(",   "max(",  "cross(",
+                                           "sign(", "mag(",   "norm(", "if("
+                                         };
+                                  
+  int   isMatched = 0;
+  int   retNumber = 0;
+  for ( int i = 0; i < NUMBFUNCS && isMatched == 0; i ++ )
+    {
+    isMatched = (  strncmp( this->Function + currentIndex, 
+                            funcNames[i], charsLens[i] 
+                          ) == 0  
+                )  ?  1  :  0;
+    retNumber = isMatched * funcNumbs[i];
+    }
+  
+  return retNumber;
+}
+
 int vtkFunctionParser::GetMathFunctionStringLength(int mathFunctionNumber)
 {
   switch (mathFunctionNumber)
@@ -2100,7 +2215,7 @@ int vtkFunctionParser::FindEndOfMathConstant(int beginIndex)
   return beginIndex;
 }
 
-int vtkFunctionParser::GetElementaryOperatorNumber(char op)
+unsigned char vtkFunctionParser::GetElementaryOperatorNumber(char op)
 {
   static const char* const operators = "+-*/^";
   int i;
@@ -2130,7 +2245,7 @@ int vtkFunctionParser::GetElementaryOperatorNumber(char op)
     {
     if (operators[i] == op)
       {
-      return VTK_PARSER_ADD + i;
+      return static_cast<unsigned char>(VTK_PARSER_ADD + i);
       }
     }
   if (op == '.')
@@ -2141,7 +2256,7 @@ int vtkFunctionParser::GetElementaryOperatorNumber(char op)
   return 0;
 }
 
-int vtkFunctionParser::GetOperandNumber(int currentIndex)
+unsigned char vtkFunctionParser::GetOperandNumber(int currentIndex)
 {
   int i, variableIndex = -1;
 
@@ -2202,7 +2317,8 @@ int vtkFunctionParser::GetOperandNumber(int currentIndex)
     }
   if (variableIndex >= 0)
     {
-    return VTK_PARSER_BEGIN_VARIABLES + variableIndex;
+    return static_cast<unsigned char>(
+      VTK_PARSER_BEGIN_VARIABLES + variableIndex);
     }
 
   for (i = 0; i < this->NumberOfVectorVariables; i++)
@@ -2220,7 +2336,9 @@ int vtkFunctionParser::GetOperandNumber(int currentIndex)
     }
   if (variableIndex >= 0)
     {
-    return VTK_PARSER_BEGIN_VARIABLES + variableIndex + this->NumberOfScalarVariables;
+    return static_cast<unsigned char>(
+      VTK_PARSER_BEGIN_VARIABLES + variableIndex
+      + this->NumberOfScalarVariables);
     }
   
   return 0;
