@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkObject.cxx,v $
+  Module:    vtkObject.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,7 +19,8 @@
 #include "vtkGarbageCollector.h"
 #include "vtkTimeStamp.h"
 
-vtkCxxRevisionMacro(vtkObject, "$Revision: 1.98 $");
+#include <vtkstd/map>
+
 
 // Initialize static member that controls warning display
 static int vtkObjectGlobalWarningDisplay = 1;
@@ -61,7 +62,7 @@ int vtkObject::GetGlobalWarningDisplay()
 class vtkObserver
 {
  public:
-  vtkObserver():Command(0),Event(0),Tag(0),Next(0),Priority(0.0), Visited(0) {}
+  vtkObserver():Command(0),Event(0),Tag(0),Next(0),Priority(0.0) {}
   ~vtkObserver();
   void PrintSelf(ostream& os, vtkIndent indent);
   
@@ -70,7 +71,6 @@ class vtkObserver
   unsigned long Tag;
   vtkObserver *Next;
   float Priority;
-  int Visited;
 };
 
 void vtkObserver::PrintSelf(ostream& os, vtkIndent indent)
@@ -465,16 +465,37 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
 {
   int focusHandled = 0;
 
+  // When we invoke an event, the observer may add or remove observers.  To make
+  // sure that the iteration over the observers goes smoothly, we capture any
+  // change to the list with the ListModified ivar.  However, an observer may
+  // also do something that causes another event to be invoked in this object.
+  // That means that this method will be called recursively, which means that we
+  // will obliterate the ListModified flag that the first call is relying on.
+  // To get around this, save the previous ListModified value on the stack and
+  // then restore it before leaving.
+  int saveListModified = this->ListModified;
   this->ListModified = 0;
 
+  // We also need to save what observers we have called on the stack (least it
+  // get overridden in the event invocation).  Also make sure that we do not
+  // invoke any new observers that were added during another observer's
+  // invocation.
+  typedef vtkstd::map<unsigned long, bool> VisitedMapType;
+  VisitedMapType visited;
   vtkObserver *elem = this->Start;
   while (elem)
     {
-    elem->Visited = 0;
-    elem=elem->Next;
+    visited.insert(vtkstd::make_pair(elem->Tag, false));
+    elem = elem->Next;
     }
 
-  // Loop once or twice, giving preference to focus holders, if any.
+  // Loop two or three times, giving preference to passive observers 
+  // and focus holders, if any.
+  //
+  // 0. Passive observer loop
+  //   Loop over all observers and execute those that are passive observers.
+  //   These observers should not affect the state of the system in any way,
+  //   and should not be allowed to abort the event.
   //
   // 1. Focus loop
   //   If there is a focus holder, loop over all observers and execute
@@ -486,22 +507,53 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
   //   remaining observers. This loop will always get executed when there
   //   is no focus holder.
 
+  // 0. Passive observer loop
+  //
+  elem = this->Start;
+  vtkObserver *next;
+  while (elem)
+    {
+    // store the next pointer because elem could disappear due to Command
+    next = elem->Next;
+    VisitedMapType::iterator vIter = visited.find(elem->Tag);
+    if ((vIter != visited.end()) && (vIter->second == false) &&
+        elem->Command->GetPassiveObserver() &&
+        (elem->Event == event || elem->Event == vtkCommand::AnyEvent))
+      {
+      vIter->second = true;
+      vtkCommand* command = elem->Command;
+      command->Register(command);
+      elem->Command->Execute(self,event,callData);
+      command->UnRegister();
+      }
+    if (this->ListModified)
+      {
+      vtkGenericWarningMacro(<<"Passive observer should not call AddObserver or RemoveObserver in callback.");
+      elem = this->Start;
+      this->ListModified = 0;
+      }
+    else
+      {
+      elem = next;
+      }
+    }
+
   // 1. Focus loop
   //
   if (this->Focus1 || this->Focus2)
     {
     elem = this->Start;
-    vtkObserver *next;
     while (elem)
       {
       // store the next pointer because elem could disappear due to Command
       next = elem->Next;
-      if (!elem->Visited &&
+      VisitedMapType::iterator vIter = visited.find(elem->Tag);
+      if ((vIter != visited.end()) && (vIter->second == false) &&
           ((this->Focus1 == elem->Command) || (this->Focus2 == elem->Command)) &&
           (elem->Event == event || elem->Event == vtkCommand::AnyEvent))
         {
         focusHandled = 1;
-        elem->Visited = 1;
+        vIter->second = true;
         vtkCommand* command = elem->Command;
         command->Register(command);
         command->SetAbortFlag(0);
@@ -511,6 +563,7 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
         if(command->GetAbortFlag())
           {
           command->UnRegister();
+          this->ListModified = saveListModified;
           return 1;
           }
         command->UnRegister();
@@ -532,15 +585,16 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
   if (!focusHandled)
     {
     elem = this->Start;
-    vtkObserver *next;
     while (elem)
       {
       // store the next pointer because elem could disappear due to Command
       next = elem->Next;
-      if (!elem->Visited &&
+      VisitedMapType::iterator vIter = visited.find(elem->Tag);
+      if ((vIter != visited.end()) && (vIter->second == false) &&
+          !elem->Command->GetPassiveObserver() &&
           (elem->Event == event || elem->Event == vtkCommand::AnyEvent))
         {
-        elem->Visited = 1;
+        vIter->second = true;
         vtkCommand* command = elem->Command;
         command->Register(command);
         command->SetAbortFlag(0);
@@ -550,6 +604,7 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
         if(command->GetAbortFlag())
           {
           command->UnRegister();
+          this->ListModified = saveListModified;
           return 1;
           }
         command->UnRegister();
@@ -566,6 +621,7 @@ int vtkSubjectHelper::InvokeEvent(unsigned long event, void *callData,
       }
     }
 
+  this->ListModified = saveListModified;
   return 0;
 }
 
