@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPythonUtil.cxx,v $
+  Module:    vtkPythonUtil.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,6 +19,7 @@
 
 #include "vtkSystemIncludes.h"
 
+#include "vtkDataArray.h"
 #include "vtkObject.h"
 #include "vtkObjectFactory.h"
 #include "vtkSmartPointerBase.h"
@@ -30,6 +31,14 @@
 #include <vtkstd/map>
 #include <vtkstd/string>
 
+// Silent warning like
+// "dereferencing type-punned pointer will break strict-aliasing rules"
+// it happens because this kind of expression: (long *)&ptr
+// pragma GCC diagnostic is available since gcc>=4.2
+#if defined(__GNUG__) && (__GNUC__>4) || (__GNUC__==4 && __GNUC_MINOR__>=2)
+#pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#endif
+
 #if defined ( _MSC_VER )
 #  define vtkConvertPtrToLong(x) ((long)(PtrToUlong(x)))
 #else
@@ -40,7 +49,10 @@
 // warnings.  Python documentation says these should not be necessary.
 // We define it as a macro in case the length needs to change across
 // python versions.
-#if   PY_VERSION_HEX >= 0x02030000
+#if   PY_VERSION_HEX >= 0x02060000 // for tp_version_tag
+#define VTK_PYTHON_UTIL_SUPRESS_UNINITIALIZED \
+  0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0, 0,
+#elif   PY_VERSION_HEX >= 0x02030000
 #define VTK_PYTHON_UTIL_SUPRESS_UNINITIALIZED \
   0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 0,
 #elif PY_VERSION_HEX >= 0x02020000
@@ -50,6 +62,10 @@
 #define VTK_PYTHON_UTIL_SUPRESS_UNINITIALIZED
 #endif
 
+#if PY_VERSION_HEX < 0x02050000
+  typedef int Py_ssize_t;
+#endif
+
 //#define VTKPYTHONDEBUG
 
 //--------------------------------------------------------------------
@@ -358,55 +374,6 @@ static PyObject *PyVTKObject_PyGetAttr(PyVTKObject *self, PyObject *attr)
   return NULL;
 }
 
-//--------------------------------------------------------------------
-class vtkPythonDeleteCommand : public vtkCommand
-{
-public:
-  static vtkPythonDeleteCommand *New(PyVTKObject *obj) {
-    return new vtkPythonDeleteCommand(obj); };
-
-  void Execute(vtkObject *caller, unsigned long, void *);
-
-private:
-  vtkPythonDeleteCommand(PyVTKObject *obj) : Self(obj) {};
-
-  PyVTKObject *Self;
-};
-
-void vtkPythonDeleteCommand::Execute(vtkObject *caller,
-                                     unsigned long vtkNotUsed(id),
-                                     void *vtkNotUsed(data))
-{
-  if (this->Self->vtk_ptr != caller)
-    {
-    vtkGenericWarningMacro("Python vs. VTK mismatch for " << caller);
-    return;
-    }
-
-#ifndef VTK_NO_PYTHON_THREADS
-#if (PY_MAJOR_VERSION > 2) || \
-((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
-  PyGILState_STATE state = PyGILState_Ensure();
-#endif
-#endif
-
-  vtkPythonDeleteObjectFromHash((PyObject *)this->Self);
-  Py_DECREF((PyObject *)this->Self->vtk_class);
-  Py_DECREF(this->Self->vtk_dict);
-#if (PY_MAJOR_VERSION >= 2)
-  PyObject_Del(this->Self);
-#else
-  PyMem_DEL(this->Self);
-#endif  
-
-#ifndef VTK_NO_PYTHON_THREADS
-#if (PY_MAJOR_VERSION > 2) || \
-((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
-  PyGILState_Release(state);
-#endif
-#endif
-}
-
 //--------------------------------------------------------------------
 static void PyVTKObject_PyDelete(PyVTKObject *self)
 {
@@ -416,10 +383,11 @@ static void PyVTKObject_PyDelete(PyVTKObject *self)
     PyObject_ClearWeakRefs((PyObject *) self);
     }
 #endif
-  self->vtk_ptr->Delete();
-  // the rest of the delection is handled when the VTK-level object
-  // is destroyed
+
+  // A python object owning a VTK object reference is getting
+  // destroyed.  Remove the python object's VTK object reference.
   vtkPythonDeleteObjectFromHash((PyObject *)self);
+
   Py_DECREF((PyObject *)self->vtk_class);
   Py_DECREF(self->vtk_dict);
 #if (PY_MAJOR_VERSION >= 2)
@@ -429,6 +397,83 @@ static void PyVTKObject_PyDelete(PyVTKObject *self)
 #endif  
 }
 
+static Py_ssize_t
+array_getsegcount(PyObject *pself, Py_ssize_t *lenp)
+{
+  PyVTKObject *self = (PyVTKObject*)pself;
+  vtkDataArray *da = vtkDataArray::SafeDownCast(self->vtk_ptr);
+  if (da)
+    {
+    if (lenp)
+      {
+      *lenp = da->GetNumberOfTuples()*
+        da->GetNumberOfComponents()*
+        da->GetDataTypeSize();
+      }
+
+    return 1;
+    }
+
+  if (lenp)
+    {
+    *lenp = 0;
+    }
+  return 0;
+}
+
+static Py_ssize_t
+array_getreadbuf(PyObject *pself, Py_ssize_t segment, void **ptrptr)
+{
+  if (segment != 0) 
+    {
+    PyErr_SetString(PyExc_ValueError,
+                    "accessing non-existing array segment");
+    return -1;
+    }
+
+  PyVTKObject *self = (PyVTKObject*)pself;
+  vtkDataArray *da = vtkDataArray::SafeDownCast(self->vtk_ptr);
+  if (da)
+    {
+    *ptrptr = da->GetVoidPointer(0);
+    return da->GetNumberOfTuples()*
+      da->GetNumberOfComponents()*
+      da->GetDataTypeSize();
+    }
+  return -1;
+}
+
+
+static Py_ssize_t
+array_getwritebuf(PyObject *pself, Py_ssize_t segment, void **ptrptr)
+{
+  return array_getreadbuf(pself, segment, ptrptr);
+}
+
+static Py_ssize_t
+array_getcharbuf(PyObject *, Py_ssize_t , const char **)
+{
+  return -1;
+}
+
+static PyBufferProcs array_as_buffer = {
+#if PY_VERSION_HEX >= 0x02050000
+  (readbufferproc)array_getreadbuf,    /*bf_getreadbuffer*/
+  (writebufferproc)array_getwritebuf,  /*bf_getwritebuffer*/
+  (segcountproc)array_getsegcount,            /*bf_getsegcount*/
+  (charbufferproc)array_getcharbuf,    /*bf_getcharbuffer*/
+  #if PY_VERSION_HEX >= 0x02060000
+   (getbufferproc)0, /* bf_getbuffer */
+   (releasebufferproc)0 /* bf_releasebuffer */
+  #endif
+#else
+  (getreadbufferproc)array_getreadbuf,    /*bf_getreadbuffer*/
+  (getwritebufferproc)array_getwritebuf,  /*bf_getwritebuffer*/
+  (getsegcountproc)array_getsegcount,         /*bf_getsegcount*/
+  (getcharbufferproc)array_getcharbuf,    /*bf_getcharbuffer*/
+#endif
+};
+
 //--------------------------------------------------------------------
 static PyTypeObject PyVTKObjectType = {
   PyObject_HEAD_INIT(&PyType_Type)
@@ -450,7 +495,7 @@ static PyTypeObject PyVTKObjectType = {
   (reprfunc)PyVTKObject_PyString,        // tp_string
   (getattrofunc)PyVTKObject_PyGetAttr,   // tp_getattro
   (setattrofunc)PyVTKObject_PySetAttr,   // tp_setattro
-  0,                                     // tp_as_buffer
+  &array_as_buffer,                      // tp_as_buffer
 #if PY_VERSION_HEX >= 0x02010000
   Py_TPFLAGS_HAVE_WEAKREFS,              // tp_flags
 #else
@@ -476,20 +521,22 @@ int PyVTKObject_Check(PyObject *obj)
 PyObject *PyVTKObject_New(PyObject *pyvtkclass, vtkObjectBase *ptr)
 {
   PyVTKClass *vtkclass = (PyVTKClass *)pyvtkclass;
-
-  if (ptr)
+  bool haveRef = false;
+  if(!ptr)
     {
-    ptr->Register(NULL);
-    }
-  else if (vtkclass->vtk_new != NULL)
-    {
-    ptr = vtkclass->vtk_new();
-    }
-  else
-    {
-    PyErr_SetString(PyExc_TypeError,
-                    (char*)"this is an abstract class and cannot be instantiated");
-    return 0;
+    // Create a new instance of this class since we were not given one.
+    if(vtkclass->vtk_new)
+      {
+      ptr = vtkclass->vtk_new();
+      haveRef = true;
+      }
+    else
+      {
+      PyErr_SetString(
+        PyExc_TypeError,
+        (char*)"this is an abstract class and cannot be instantiated");
+      return 0;
+      }
     }
 #if (PY_MAJOR_VERSION >= 2)
   PyVTKObject *self = PyObject_New(PyVTKObject, &PyVTKObjectType);
@@ -521,11 +568,16 @@ PyObject *PyVTKObject_New(PyObject *pyvtkclass, vtkObjectBase *ptr)
   self->vtk_weakreflist = NULL;
 #endif
 
+  // A python object owning a VTK object reference is getting
+  // created.  Add the python object's VTK object reference.
   vtkPythonAddObjectToHash((PyObject *)self, ptr);
-  /* I'll reinstate this later
-  ptr->AddObserver(vtkCommand::DeleteEvent,
-                   vtkPythonDeleteCommand::New(self));
-  */
+
+  // The hash now owns a reference so we can free ours.
+  if(haveRef)
+    {
+    ptr->Delete();
+    }
+
   return (PyObject *)self;
 }
 
@@ -880,7 +932,10 @@ static PyObject *vtkBuildDocString(char *docstring[])
   int *m;
   int total = 0;
 
-  for (n = 0; docstring[n] != NULL; n++);
+  for (n = 0; docstring[n] != NULL; n++)
+    {
+    ;
+    }
 
   m = new int[n];
 
@@ -1151,7 +1206,10 @@ static PyObject *PyVTKSpecialObject_PyGetAttr(PyVTKSpecialObject *self,
       PyObject *lst;
       int i, n;
 
-      for (n = 0; meth && meth[n].ml_name; n++);
+      for (n = 0; meth && meth[n].ml_name; n++)
+        {
+        ;
+        }
 
       if ((lst = PyList_New(n)) != NULL)
         {
@@ -1365,7 +1423,10 @@ void vtkPythonDeleteObjectFromHash(PyObject *obj)
                          << obj->vtk_ptr);
 #endif  
 
-  vtkPythonHash->ObjectHash->erase(ptr);
+  if (vtkPythonHash)
+    {
+    vtkPythonHash->ObjectHash->erase(ptr);
+    }
 }
 
 //--------------------------------------------------------------------
@@ -1553,7 +1614,7 @@ PyObject *vtkPythonGetObjectFromObject(PyObject *arg, const char *type)
       }      
     if (i <= 0)
       {
-      i = sscanf(ptrText,"%lx",(long *)&ptr);
+      i = sscanf(ptrText,"%p",&ptr);
       }
     if (i <= 0)
       {
@@ -1593,21 +1654,38 @@ void *vtkPythonUnmanglePointer(char *ptrText, int *len, const char *type)
 {
   int i; 
   void *ptr;
-  char typeCheck[128];
-  if (*len < 128)
+  char typeCheck[256];
+  typeCheck[0] = '\0';
+
+  // Do some minimal checks that it might be a swig pointer.
+  if (*len < 256 && *len > 4 && ptrText[0] == '_')
     {
-    i = sscanf(ptrText,"_%lx_%s",(long *)&ptr,typeCheck);
-    if (strcmp(type,typeCheck) == 0)
-      { // sucessfully unmangle
-      *len = 0;
-      return ptr;
+    // Verify that this is a terminated string
+    for (i = *len-1; i >= 0; --i)
+      {
+      if (ptrText[i] == '\0')
+        {
+        break;
+        }
       }
-    else if (i == 2)
-      { // mangled pointer of wrong type
-      *len = -1;
-      return NULL;
+
+    // If the string is terminated, do a full check for a swig pointer
+    if (i >= 0)
+      {
+      i = sscanf(ptrText,"_%lx_%s",(long *)&ptr,typeCheck);
+      if (strcmp(type,typeCheck) == 0)
+        { // sucessfully unmangle
+        *len = 0;
+        return ptr;
+        }
+      else if (i == 2)
+        { // mangled pointer of wrong type
+        *len = -1;
+        return NULL;
+        }
       }
     }
+
   // couldn't unmangle: return string as void pointer if it didn't look
   // like a SWIG mangled pointer
   return (void *)ptrText;
@@ -1620,7 +1698,7 @@ void *vtkPythonUnmanglePointer(char *ptrText, int *len, const char *type)
 // list is modified.
 
 template<class T>
-static inline
+inline
 int vtkPythonCheckFloatArray(PyObject *args, int i, T *a, int n)
 {
   int changed = 0;
@@ -1652,7 +1730,7 @@ int vtkPythonCheckFloatArray(PyObject *args, int i, T *a, int n)
 }
 
 template<class T>
-static inline
+inline
 int vtkPythonCheckIntArray(PyObject *args, int i, T *a, int n)
 {
   int changed = 0;
@@ -1685,7 +1763,7 @@ int vtkPythonCheckIntArray(PyObject *args, int i, T *a, int n)
 
 #if defined(VTK_TYPE_USE_LONG_LONG) || defined(VTK_TYPE_USE___INT64)
 template<class T>
-static inline
+inline
 int vtkPythonCheckLongArray(PyObject *args, int i, T *a, int n)
 {
   int changed = 0;
@@ -1824,6 +1902,14 @@ void vtkPythonVoidFunc(void *arg)
   PyObject *arglist, *result;
   PyObject *func = (PyObject *)arg;
 
+  // Sometimes it is possible for the function to be invoked after
+  // Py_Finalize is called, this will cause nasty errors so we return if
+  // the interpreter is not initialized.
+  if (Py_IsInitialized() == 0)
+    {
+    return;
+    }
+
 #ifndef VTK_NO_PYTHON_THREADS
 #if (PY_MAJOR_VERSION > 2) || \
 ((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
@@ -1863,6 +1949,14 @@ void vtkPythonVoidFuncArgDelete(void *arg)
 {
   PyObject *func = (PyObject *)arg;
 
+  // Sometimes it is possible for the function to be invoked after
+  // Py_Finalize is called, this will cause nasty errors so we return if
+  // the interpreter is not initialized.
+  if (Py_IsInitialized() == 0)
+    {
+    return;
+    }
+
 #ifndef VTK_NO_PYTHON_THREADS
 #if (PY_MAJOR_VERSION > 2) || \
 ((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
@@ -1887,11 +1981,12 @@ void vtkPythonVoidFuncArgDelete(void *arg)
 vtkPythonCommand::vtkPythonCommand()
 { 
   this->obj = NULL;
+  this->ThreadState = NULL;
 }
 
 vtkPythonCommand::~vtkPythonCommand()
-{ 
-  if (this->obj)
+{
+  if (this->obj && Py_IsInitialized())
     {
     Py_DECREF(this->obj);
     }
@@ -1903,12 +1998,25 @@ void vtkPythonCommand::SetObject(PyObject *o)
   this->obj = o; 
 }
 
+void vtkPythonCommand::SetThreadState(PyThreadState *ts)
+{ 
+  this->ThreadState = ts; 
+}
+
 void vtkPythonCommand::Execute(vtkObject *ptr, unsigned long eventtype,
                                void *CallData)
 {
   PyObject *arglist, *result, *obj2;
   const char *eventname;
 
+  // Sometimes it is possible for the command to be invoked after
+  // Py_Finalize is called, this will cause nasty errors so we return if
+  // the interpreter is not initialized.
+  if (Py_IsInitialized() == 0)
+    {
+    return;
+    }
+
 #ifndef VTK_NO_PYTHON_THREADS
 #if (PY_MAJOR_VERSION > 2) || \
 ((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
@@ -1916,6 +2024,15 @@ void vtkPythonCommand::Execute(vtkObject *ptr, unsigned long eventtype,
 #endif
 #endif
   
+  // If a threadstate has been set using vtkPythonCommand::SetThreadState,
+  // then swap it in here.  See the email to vtk-developers@vtk.org from
+  // June 18, 2009 with subject "Py_NewInterpreter and vtkPythonCallback issue"
+  PyThreadState* prevThreadState = NULL;
+  if (this->ThreadState)
+    {
+    prevThreadState = PyThreadState_Swap(this->ThreadState);
+    }
+
   if (ptr && ptr->GetReferenceCount() > 0)
     {
     obj2 = vtkPythonGetObjectFromPointer(ptr);
@@ -2010,6 +2127,12 @@ void vtkPythonCommand::Execute(vtkObject *ptr, unsigned long eventtype,
     PyErr_Print();
     }
 
+  // If we did the swap near the top of this function then swap back now.
+  if (this->ThreadState)
+    {
+    PyThreadState_Swap(prevThreadState);
+    }
+
 #ifndef VTK_NO_PYTHON_THREADS
 #if (PY_MAJOR_VERSION > 2) || \
 ((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
