@@ -1,10 +1,10 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkStringArray.cxx,v $
+  Module:    vtkStringArray.cxx
   Language:  C++
-  Date:      $Date: 2008-07-17 13:22:30 $
-  Version:   $Revision: 1.16 $
+  Date:      $Date$
+  Version:   $Revision$
 
   Copyright 2004 Sandia Corporation.
   Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
@@ -73,7 +73,6 @@ public:
   bool Rebuild;
 };
 
-vtkCxxRevisionMacro(vtkStringArray, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkStringArray);
 
 //-----------------------------------------------------------------------------
@@ -356,8 +355,8 @@ vtkStdString * vtkStringArray::ResizeAndExtend(vtkIdType sz)
   if(this->Array)
     {
     // can't use memcpy here
-    int numCopy = (newSize < this->Size ? newSize : this->Size);
-    for (int i = 0; i < numCopy; ++i)
+    vtkIdType numCopy = (newSize < this->Size ? newSize : this->Size);
+    for (vtkIdType i = 0; i < numCopy; ++i)
       {
       newArray[i] = this->Array[i];
       }
@@ -405,9 +404,9 @@ int vtkStringArray::Resize(vtkIdType sz)
 
   if(this->Array)
     {
-    int numCopy = (newSize < this->Size ? newSize : this->Size);
+    vtkIdType numCopy = (newSize < this->Size ? newSize : this->Size);
 
-    for (int i = 0; i < numCopy; ++i)
+    for (vtkIdType i = 0; i < numCopy; ++i)
       {
       newArray[i] = this->Array[i];
       }
@@ -487,30 +486,30 @@ int vtkStringArray::GetDataTypeSize( void )
 // ----------------------------------------------------------------------------
 unsigned long vtkStringArray::GetActualMemorySize( void )
 {
-  unsigned long totalSize = 0;
-  unsigned long  numPrims = this->GetSize();
+  size_t totalSize = 0;
+  size_t  numPrims = static_cast<size_t>(this->GetSize());
 
-  for (unsigned long i = 0; i < numPrims; ++i)
+  for (size_t i = 0; i < numPrims; ++i)
     {
     totalSize += sizeof( vtkStdString );
-    totalSize += static_cast<unsigned long>(this->Array[i].size()) *
-      sizeof( vtkStdString::value_type );
+    totalSize += this->Array[i].size() *sizeof( vtkStdString::value_type );
     }
 
-  return static_cast<unsigned long>(ceil( totalSize / 1024.0 )); // kilobytes
+  return static_cast<unsigned long>(
+    ceil(static_cast<double>(totalSize) / 1024.0 )); // kilobytes
 }
 
 // ----------------------------------------------------------------------------
-unsigned long vtkStringArray::GetDataSize()
+vtkIdType vtkStringArray::GetDataSize()
 {
-  unsigned long size = 0;
-  unsigned long numStrs = this->GetMaxId() + 1;
-  for (unsigned long i=0; i < numStrs; i++)
+  size_t size = 0;
+  size_t numStrs = static_cast<size_t>(this->GetMaxId() + 1);
+  for(size_t i=0; i < numStrs; i++)
     {
-    size += static_cast<unsigned long>(this->Array[i].size()) + 1;
-      // (+1) for termination character.
+    size += this->Array[i].size() + 1;
+    // (+1) for termination character.
     }
-  return size;
+  return static_cast<vtkIdType>(size);
 }
 
 // ----------------------------------------------------------------------------
@@ -712,8 +711,8 @@ vtkIdType vtkStringArray::LookupValue(vtkStdString value)
     ++cached;
     }
 
-  int numComps = this->GetNumberOfComponents();
-  vtkIdType numTuples = this->GetNumberOfTuples();
+  int numComps = this->Lookup->SortedArray->GetNumberOfComponents();
+  vtkIdType numTuples = this->Lookup->SortedArray->GetNumberOfTuples();
   vtkStdString* ptr = this->Lookup->SortedArray->GetPointer(0);
   vtkStdString* ptrEnd = ptr + numComps*numTuples;
   vtkStdString* found = vtkstd::lower_bound(ptr, ptrEnd, value);
