@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDataSetAttributes.cxx,v $
+  Module:    vtkDataSetAttributes.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,8 +29,10 @@
 #include "vtkFloatArray.h"
 #include "vtkIdTypeArray.h"
 #include "vtkObjectFactory.h"
+#include "vtkInformation.h"
+
+#include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkDataSetAttributes, "$Revision: 1.27 $");
 vtkStandardNewMacro(vtkDataSetAttributes);
 
 //--------------------------------------------------------------------------
@@ -124,7 +126,6 @@ void vtkDataSetAttributes::CopyAllOff(int ctype)
 // not copied.
 void vtkDataSetAttributes::DeepCopy(vtkFieldData *fd)
 {
-
   this->Initialize(); //free up memory
 
   vtkDataSetAttributes* dsa = vtkDataSetAttributes::SafeDownCast(fd);
@@ -167,7 +168,6 @@ void vtkDataSetAttributes::DeepCopy(vtkFieldData *fd)
     {
     this->vtkFieldData::DeepCopy(fd);
     }
-          
 }
 
 //--------------------------------------------------------------------------
@@ -357,6 +357,7 @@ vtkFieldData::BasicIterator  vtkDataSetAttributes::ComputeRequiredArrays(
         }
       }
     }
+
   vtkFieldData::BasicIterator it(copyFlags, numArrays);
   delete[] copyFlags;
   return it;
@@ -383,7 +384,7 @@ void vtkDataSetAttributes::PassData(vtkFieldData* fd)
     // Note that NULL data arrays are not copied
 
     vtkFieldData::BasicIterator it = this->ComputeRequiredArrays(dsa, PASSDATA);
-    
+
     if ( it.GetListSize() > this->NumberOfArrays )
       {
       this->AllocateArrays(it.GetListSize());
@@ -618,6 +619,10 @@ void vtkDataSetAttributes::InternalCopyAllocate(vtkDataSetAttributes* pd,
         newAA = aa->NewInstance();
         newAA->SetNumberOfComponents(aa->GetNumberOfComponents());
         newAA->SetName(aa->GetName());
+        if (aa->HasInformation())
+          {
+          newAA->CopyInformation(aa->GetInformation(),/*deep=*/1);
+          }
         if ( sze > 0 )
           {
           newAA->Allocate(sze*aa->GetNumberOfComponents(),ext);
@@ -1481,6 +1486,10 @@ void vtkDataSetAttributes::InternalCopyAllocate(
       newAA = vtkAbstractArray::CreateArray(list.FieldTypes[i]);
       newAA->SetName(list.Fields[i]);
       newAA->SetNumberOfComponents(list.FieldComponents[i]);
+      if (list.FieldInformation[i])
+        {
+        newAA->CopyInformation(list.FieldInformation[i],/*deep=*/1);
+        }
 
       if ( sze > 0 )
         {
@@ -1490,11 +1499,13 @@ void vtkDataSetAttributes::InternalCopyAllocate(
         {
         newAA->Allocate(list.NumberOfTuples,ext);
         }
+
       if ( (newDA = vtkDataArray::SafeDownCast(newAA)) )
         {
         newDA->SetLookupTable(list.LUT[i]);
         }
 
+
       // If attribute data, do something extra
       if ( i < NUM_ATTRIBUTES )
         {
@@ -1514,7 +1525,7 @@ void vtkDataSetAttributes::InternalCopyAllocate(
         if ( (this->GetFlag(list.Fields[i]) != 0) &&
              !(this->DoCopyAllOff && (this->GetFlag(list.Fields[i]) != 1)) )
           {
-          list.FieldIndices[i] = this->AddArray(newDA);
+          list.FieldIndices[i] = this->AddArray(newAA);
           }
         else
           {
@@ -1542,7 +1553,7 @@ void vtkDataSetAttributes::CopyData(vtkDataSetAttributes::FieldList& list,
   int i;
   for (i=0; i < list.NumberOfFields; i++)
     {
-    if ( list.FieldIndices[i] >= 0 )
+    if ( list.FieldIndices[i] >= 0 && list.DSAIndices[idx][i] >= 0 )
       {
       toDA = this->GetAbstractArray(list.FieldIndices[i]);
       fromDA = fromDSA->GetAbstractArray(list.DSAIndices[idx][i]);
@@ -1566,7 +1577,7 @@ void vtkDataSetAttributes::InterpolatePoint(
 
   for (int i=0; i < list.NumberOfFields; i++)
     {
-    if ( list.FieldIndices[i] >= 0 )
+    if ( list.FieldIndices[i] >= 0 && list.DSAIndices[idx][i] >= 0 )
       {
       toArray = this->GetAbstractArray(list.FieldIndices[i]);
       fromArray = fromPd->GetAbstractArray(list.DSAIndices[idx][i]);
@@ -1575,16 +1586,70 @@ void vtkDataSetAttributes::InterpolatePoint(
     }
 }
 
-// FieldList support ---------------------------------------------------------
+//--------------------------------------------------------------------------
+const char* vtkDataSetAttributes::GetAttributeTypeAsString(int attributeType)
+{
+  if (attributeType < 0 || attributeType >= NUM_ATTRIBUTES)
+    {
+    vtkGenericWarningMacro("Bad attribute type.");
+    return NULL;
+    }
+  return vtkDataSetAttributes::AttributeNames[attributeType];
+}
+
+//--------------------------------------------------------------------------
+const char* vtkDataSetAttributes::GetLongAttributeTypeAsString(int attributeType)
+{
+  if (attributeType < 0 || attributeType >= NUM_ATTRIBUTES)
+    {
+    vtkGenericWarningMacro("Bad attribute type.");
+    return NULL;
+    }
+  return vtkDataSetAttributes::LongAttributeNames[attributeType];
+}
+
+//=============================================================================
+vtkDataSetAttributes::FieldList::FieldList(int numInputs)
+{
+  this->Fields = 0;
+  this->FieldTypes = 0;
+  this->FieldComponents = 0;
+  this->FieldIndices = 0;
+  this->NumberOfFields = 0;
+  this->LUT = 0;
+  this->FieldInformation = 0;
+  this->DSAIndices = 0;
+  this->NumberOfDSAIndices = 0;
+  //
+  if (numInputs)
+    {
+    this->NumberOfDSAIndices = numInputs;
+    this->DSAIndices = new int*[numInputs];
+    int i;
+    for (i=0; i<numInputs; i++)
+      {
+      this->DSAIndices[i] = 0;
+      }
+    }
+}
+
+//--------------------------------------------------------------------------
+vtkDataSetAttributes::FieldList::~FieldList()
+{
+  this->ClearFields();
+  delete [] this->DSAIndices;
+  this->DSAIndices = 0;
+}
+
+//----------------------------------------------------------------------------
 // To perform intersection of attribute data, use IntializeFieldList() to grab
 // an initial vtkDataSetAttributes. Then use IntersectFieldList() to add (and 
 // intersect) additional vtkDataSetAttributes.
-void vtkDataSetAttributes::FieldList::InitializeFieldList(vtkDataSetAttributes* dsa)
+void vtkDataSetAttributes::FieldList::InitializeFieldList(
+  vtkDataSetAttributes* dsa)
 {
-  int i, idx;
-
+  int i;
   this->ClearFields();
-
   // Allocate space for the arrays plus five attributes
   this->NumberOfFields = dsa->GetNumberOfArrays() + NUM_ATTRIBUTES;
   this->Fields = new char*[this->NumberOfFields];
@@ -1592,23 +1657,27 @@ void vtkDataSetAttributes::FieldList::InitializeFieldList(vtkDataSetAttributes*
   this->FieldComponents = new int [this->NumberOfFields];
   this->FieldIndices = new int [this->NumberOfFields];
   this->LUT = new vtkLookupTable* [this->NumberOfFields];
+  this->FieldInformation = new vtkInformation* [this->NumberOfFields];
   for(i=0; i < this->NumberOfFields; i++)
     {
     this->Fields[i] = 0;
     this->FieldTypes[i] = -1;
     this->FieldComponents[i] = 0;
     this->FieldIndices[i] = -1;
+    this->LUT[i] = 0;
+    this->FieldInformation[i] = 0;
     }
   this->CurrentInput = 0;
   this->NumberOfTuples = 0;
-  
+
   //there may be no data hence dsa->Data
   for(i=0; dsa->Data && i < dsa->GetNumberOfArrays(); i++) 
     {
-    if ( (idx=dsa->IsArrayAnAttribute(i)) != -1 ) //it's an attribute
+    int attrType = dsa->IsArrayAnAttribute(i);
+    if ( attrType != -1 ) //it's an attribute
       {
-      this->FieldIndices[idx] = idx;
-      this->SetField(idx, dsa->Data[i]);
+      this->FieldIndices[attrType] = i;
+      this->SetField(attrType, dsa->Data[i]);
       }
     else
       {
@@ -1616,18 +1685,228 @@ void vtkDataSetAttributes::FieldList::InitializeFieldList(vtkDataSetAttributes*
       this->SetField(NUM_ATTRIBUTES+i, dsa->Data[i]);
       }
     }
-  
+
   // The first dataset is added to the field list
   this->IntersectFieldList(dsa);
 }
 
+//--------------------------------------------------------------------------
+void vtkDataSetAttributes::FieldList::UnionFieldList(vtkDataSetAttributes* dsa)
+{
+  vtkAbstractArray* aa;
+  vtkDataArray* da;
+  // Keep a running total of the number of tuples...might be useful
+  // for later allocation.
+  if ( (aa=dsa->GetAbstractArray(0)) ) 
+    {
+    this->NumberOfTuples += aa->GetNumberOfTuples();
+    }
+
+  // unlike Intersection, with Union the the total number of fields may change,
+  // so we have to be careful with that.
+  vtkstd::vector<int> dsaIndices;
+  dsaIndices.resize(this->NumberOfFields, -1);
+
+  // Intersect the active attributes. (Even though we are taking a union, we
+  // intersect the active attributes).
+  int attributeIndices[NUM_ATTRIBUTES];
+  dsa->GetAttributeIndices(attributeIndices);
+  for (int i=0; i < NUM_ATTRIBUTES; i++)
+    {
+    if (this->FieldIndices[i] >= 0)
+      {
+      da = dsa->GetAttribute(i);
+      if ((da) && (da->GetDataType() == this->FieldTypes[i]) && 
+          (da->GetNumberOfComponents() == this->FieldComponents[i]))
+        {
+        dsaIndices[i] = attributeIndices[i];
+        }
+      else
+        {
+        // doh! A attribute is not available in this new dsa. But it was
+        // available until now.
+
+        // In InitializeFieldList(), if an array is an active attribute, its
+        // information is noted only in the first "set". Now since we are
+        // marking it as not-an-attribute, we still don't want to loose the
+        // array. So we enable it in the second "set". But all DSAIndices until
+        // the CurrentInput referred to this array in it's location in the
+        // first "set" so we have to move those as well. That's what's
+        // happening here.
+        int offset = this->FieldIndices[i];
+        this->FieldIndices[NUM_ATTRIBUTES + offset] = this->FieldIndices[i];
+        this->Fields[offset+NUM_ATTRIBUTES] = this->Fields[i];
+        this->FieldTypes[offset+NUM_ATTRIBUTES] = this->FieldTypes[i];
+        this->FieldComponents[offset+NUM_ATTRIBUTES] = this->FieldComponents[i];
+        this->LUT[offset+NUM_ATTRIBUTES] = this->LUT[i];
+        this->FieldInformation[offset+NUM_ATTRIBUTES] = this->FieldInformation[i];
+
+        this->FieldIndices[i] = -1; //Attribute not present
+        this->Fields[i] = NULL;
+        this->FieldTypes[i] = -1;
+        this->FieldComponents[i] = 0;
+        this->LUT[i] = NULL;
+        this->FieldInformation[i] = NULL;
+
+        for (int cc=0; cc < this->CurrentInput && cc < this->NumberOfDSAIndices;
+          cc++)
+          {
+          this->DSAIndices[cc][offset+NUM_ATTRIBUTES] = this->DSAIndices[cc][i];
+          this->DSAIndices[cc][i] = -1;
+          }
+        }
+      }
+    }
+
+  vtkstd::vector<bool> dsaMarkedArrays;
+  dsaMarkedArrays.resize(dsa->GetNumberOfArrays(), false);
+
+  // * Try to match the existing fields with those in dsa.
+  for (int i = NUM_ATTRIBUTES; i < this->NumberOfFields; i++)
+    {
+    // FieldIndices should really be a bool.
+    if (this->FieldIndices[i] < 0)
+      {
+      continue;
+      }
+    int index;
+    aa = dsa->GetAbstractArray(this->Fields[i], index);
+    if ((aa) && (aa->GetDataType() == this->FieldTypes[i]) &&
+      (aa->GetNumberOfComponents() == this->FieldComponents[i]))
+      {
+      dsaIndices[i] = index;
+      dsaMarkedArrays[index] = true;
+      }
+    }
+  
+  // * Now every array in dsaMarkedArrays that has a false, implies that it did not
+  // match with any of the existing fields. So those will be appended to the
+  // end of the field list.
+  vtkstd::vector<int> dsaPendingIndices;
+  for (size_t cc=0; cc < dsaMarkedArrays.size(); cc++)
+    {
+    if (dsaMarkedArrays[cc] == false)
+      {
+      dsaPendingIndices.push_back(static_cast<int>(cc));
+      }
+    }
+
+  if (dsaPendingIndices.size() != 0)
+    {
+    size_t old_size = dsaIndices.size();
+    size_t new_size = old_size + dsaPendingIndices.size();
+
+    // * If dsaPendingIndices != empty, then we need to grow the num of fields.
+    this->GrowBy(static_cast<unsigned int>(dsaPendingIndices.size()));
+    
+    dsaIndices.resize(new_size, -1);
+    for (size_t cc=0; cc < dsaPendingIndices.size(); cc++)
+      {
+      this->FieldIndices[old_size+cc] =
+        static_cast<int>((old_size+cc) - NUM_ATTRIBUTES);
+      this->SetField(static_cast<int>(old_size+cc),
+        dsa->GetAbstractArray(dsaPendingIndices[cc]));
+      dsaIndices[old_size + cc] = dsaPendingIndices[cc];
+      }
+    }
+ 
+  this->DSAIndices[this->CurrentInput] = new int [this->NumberOfFields];
+  memcpy(this->DSAIndices[this->CurrentInput], &dsaIndices[0],
+    sizeof(int)*this->NumberOfFields);
+
+  this->CurrentInput++;
+}
+
+//--------------------------------------------------------------------------
+void vtkDataSetAttributes::FieldList::GrowBy(unsigned int delta)
+{
+  if (delta == 0)
+    {
+    return;
+    }
+
+  int old_size = this->NumberOfFields;
+  int new_size = this->NumberOfFields + delta;
+
+  char** newFields = new char*[new_size];
+  int* newFieldTypes = new int[new_size];
+  int* newFieldComponents = new int [new_size];
+  int* newFieldIndices = new int [new_size];
+  vtkLookupTable** newLUT = new vtkLookupTable* [new_size];
+  vtkInformation** newFieldInformation = new vtkInformation* [new_size];
+
+  // copy the old fields.
+  for(int i=0; i < old_size; i++)
+    {
+    if (this->Fields[i])
+      {
+      newFields[i] = strdup(this->Fields[i]);
+      }
+    else
+      {
+      newFields[i] = NULL;
+      }
+    }
+  memcpy(newFieldTypes, this->FieldTypes, sizeof(int)*old_size);
+  memcpy(newFieldComponents, this->FieldComponents, sizeof(int)*old_size);
+  memcpy(newFieldIndices, this->FieldIndices, sizeof(int)*old_size);
+  memcpy(newLUT, this->LUT, sizeof(vtkLookupTable*)*old_size);
+  memcpy(newFieldInformation, this->FieldInformation,
+    sizeof(vtkInformation*)*old_size);
+
+  // initialize the rest.
+  for (int i=old_size; i < new_size; i++)
+    {
+    newFields[i] = NULL;
+    newFieldTypes[i] = -1;
+    newFieldComponents[i] = 0;
+    newFieldIndices[i] = -1;
+    newLUT[i] = NULL;
+    newFieldInformation[i] = NULL;
+    }
+
+  int **newDSAIndices = new int*[this->NumberOfDSAIndices];
+  for (int cc=0; cc < this->NumberOfDSAIndices; cc++)
+    {
+    if (this->DSAIndices[cc] != NULL)
+      {
+      newDSAIndices[cc] = new int[new_size];
+      memcpy(newDSAIndices[cc], this->DSAIndices[cc], sizeof(int)*old_size);
+      for (int kk=old_size; kk < new_size; kk++)
+        {
+        newDSAIndices[cc][kk] = -1;
+        }
+      }
+    else
+      {
+      newDSAIndices[cc] = NULL;
+      }
+    }
+
+  int currentInput = this->CurrentInput;
+  int numberOfDSAIndices = this->NumberOfDSAIndices;
+
+  this->ClearFields();
+
+  this->NumberOfFields = new_size;
+  this->NumberOfDSAIndices = numberOfDSAIndices;
+  this->CurrentInput = currentInput;
+  this->Fields = newFields;
+  this->FieldTypes = newFieldTypes;
+  this->FieldComponents = newFieldComponents;
+  this->FieldIndices = newFieldIndices;
+  this->LUT = newLUT;
+  this->FieldInformation = newFieldInformation;
+  this->DSAIndices = newDSAIndices;
+}
+
 //--------------------------------------------------------------------------
 void vtkDataSetAttributes::FieldList::IntersectFieldList(vtkDataSetAttributes* dsa)
 {
   int i;
   vtkDataArray *da;
   vtkAbstractArray* aa;
-  
+
   // Initialize the indices for this dataset
   this->DSAIndices[this->CurrentInput] = new int [this->NumberOfFields];
   for (i=0; i < this->NumberOfFields; i++)
@@ -1641,7 +1920,7 @@ void vtkDataSetAttributes::FieldList::IntersectFieldList(vtkDataSetAttributes* d
     {
     this->NumberOfTuples += da->GetNumberOfTuples();
     }
-  
+
   // Intersect the attributes
   int attributeIndices[NUM_ATTRIBUTES];
   dsa->GetAttributeIndices(attributeIndices);
@@ -1680,7 +1959,6 @@ void vtkDataSetAttributes::FieldList::IntersectFieldList(vtkDataSetAttributes* d
       }
     }
 
-    
   this->CurrentInput++;
 }
 
@@ -1690,33 +1968,6 @@ int vtkDataSetAttributes::FieldList::IsAttributePresent(int attrType)
   return this->FieldIndices[attrType];
 }
 
-
-//--------------------------------------------------------------------------
-vtkDataSetAttributes::FieldList::FieldList(int numInputs)
-{
-  this->Fields = 0;
-  this->FieldTypes = 0;
-  this->FieldComponents = 0;
-  this->FieldIndices = 0;
-  this->NumberOfFields = 0;
-  this->LUT = 0;
-  this->NumberOfDSAIndices = numInputs;
-  this->DSAIndices = new int*[numInputs];
-  int i;
-  for (i=0; i<numInputs; i++)
-    {
-    this->DSAIndices[i] = 0;
-    }
-}
-
-//--------------------------------------------------------------------------
-vtkDataSetAttributes::FieldList::~FieldList()
-{
-  this->ClearFields();
-  delete [] this->DSAIndices;
-  this->DSAIndices = 0;
-}
-
 //--------------------------------------------------------------------------
 void vtkDataSetAttributes::FieldList::ClearFields()
 {
@@ -1737,43 +1988,41 @@ void vtkDataSetAttributes::FieldList::ClearFields()
       this->DSAIndices[i] = 0;
       }
     }
-  delete [] this->LUT;
-  this->LUT = 0;
+  //
   delete [] this->Fields;
   this->Fields = 0;
+
+  delete [] this->FieldInformation;
+  this->FieldInformation = 0;
+
+  delete [] this->LUT;
+  this->LUT = 0;
+
   delete [] this->FieldTypes;
   this->FieldTypes = 0;
+
   delete [] this->FieldComponents;
   this->FieldComponents = 0;
+
   delete [] this->FieldIndices;
   this->FieldIndices = 0;
-  
+
   this->NumberOfFields = 0;
   this->CurrentInput = 0;
 }
 
 //--------------------------------------------------------------------------
-void vtkDataSetAttributes::FieldList::SetField(int index, 
-  vtkAbstractArray *aa)
+void vtkDataSetAttributes::FieldList::SetField(
+        int index,
+        vtkAbstractArray *aa)
 {
-  const char* name=aa->GetName();
-  int dataType=aa->GetDataType();
-  vtkLookupTable *lut = 0;
-  
-  if (vtkDataArray::SafeDownCast(aa))
-    {
-    lut=vtkDataArray::SafeDownCast(aa)->GetLookupTable();
-    }
-  
+  // Store the field name
   if ( this->Fields[index] )
     {
     delete [] this->Fields[index];
     this->Fields[index] = 0;
     }
-  
-  this->FieldTypes[index] = dataType;
-  this->FieldComponents[index] = aa->GetNumberOfComponents();
-  this->LUT[index] = lut;
+  const char* name=aa->GetName();
   if (name)
     {
     int len = static_cast<int>(strlen(name));
@@ -1783,11 +2032,22 @@ void vtkDataSetAttributes::FieldList::SetField(int index,
       strcpy(this->Fields[index], name);
       }
     }
-  else
+  // Store the data type
+  this->FieldTypes[index] = aa->GetDataType();
+  // Store the components
+  this->FieldComponents[index] = aa->GetNumberOfComponents();
+  // Store the lookup table
+  this->LUT[index]=0;
+  if (vtkDataArray::SafeDownCast(aa))
     {
-    this->Fields[index] = 0;
+    this->LUT[index]=vtkDataArray::SafeDownCast(aa)->GetLookupTable();
+    }
+  // Store the information
+  this->FieldInformation[index] = 0;
+  if (aa->HasInformation())
+    {
+    this->FieldInformation[index] = aa->GetInformation();
     }
-    
 }
 
 //--------------------------------------------------------------------------
@@ -1805,32 +2065,30 @@ void vtkDataSetAttributes::FieldList::RemoveField(const char *name)
       {
       delete [] this->Fields[i];
       this->Fields[i] = 0;
+      this->FieldTypes[i] = -1;
+      this->FieldComponents[i] = 0;
       this->FieldIndices[i] = -1;
+      this->LUT[i] = 0;
+      this->FieldInformation[i] = 0;
       return;
       }
     }
 }
 
-
 //--------------------------------------------------------------------------
-const char* vtkDataSetAttributes::GetAttributeTypeAsString(int attributeType)
+void vtkDataSetAttributes::FieldList::PrintSelf(ostream &os, vtkIndent indent)
 {
-  if (attributeType < 0 || attributeType >= NUM_ATTRIBUTES)
+  os << indent << "Number of Fields:" << this->NumberOfFields << endl;
+  vtkIndent nextIndent=indent.GetNextIndent();
+  for (int i=0; i<this->NumberOfFields; ++i)
     {
-    vtkGenericWarningMacro("Bad attribute type.");
-    return NULL;
+    os << indent << "Field " << i << " {" << endl
+       << nextIndent
+       << (this->Fields[i]==0?"NULL":this->Fields[i]) << ", "
+       << this->FieldTypes[i] << ", "
+       << this->FieldComponents[i] << ", "
+       << this->FieldIndices[i] << ", "
+       << this->FieldInformation[i]
+       << "}" << endl;
     }
-  return vtkDataSetAttributes::AttributeNames[attributeType];
 }
-
-//--------------------------------------------------------------------------
-const char* vtkDataSetAttributes::GetLongAttributeTypeAsString(int attributeType)
-{
-  if (attributeType < 0 || attributeType >= NUM_ATTRIBUTES)
-    {
-    vtkGenericWarningMacro("Bad attribute type.");
-    return NULL;
-    }
-  return vtkDataSetAttributes::LongAttributeNames[attributeType];
-}
-
