@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAppendPolyData.cxx,v $
+  Module:    vtkAppendPolyData.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -25,7 +25,6 @@
 #include "vtkPolyData.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
 
-vtkCxxRevisionMacro(vtkAppendPolyData, "$Revision: 1.103 $");
 vtkStandardNewMacro(vtkAppendPolyData);
 
 //----------------------------------------------------------------------------
@@ -105,19 +104,9 @@ void vtkAppendPolyData::SetInputByNumber(int num, vtkPolyData *input)
 }
 
 //----------------------------------------------------------------------------
-// This method is much too long, and has to be broken up!
-// Append data sets into single polygonal data set.
-int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
-                                   vtkInformationVector **inputVector,
-                                   vtkInformationVector *outputVector)
+int vtkAppendPolyData::ExecuteAppend(vtkPolyData* output,
+    vtkPolyData* inputs[], int numInputs)
 {
-  // get the info object
-  vtkInformation *outInfo = outputVector->GetInformationObject(0);
-
-  // get the ouptut
-  vtkPolyData *output = vtkPolyData::SafeDownCast(
-    outInfo->Get(vtkDataObject::DATA_OBJECT()));
-
   int idx;
   vtkPolyData *ds;
   vtkPoints  *inPts;
@@ -142,7 +131,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   vtkIdType *pPolys;
   vtkIdType npts = 0;
   vtkIdType ptId, cellId;
-  
+
   vtkDebugMacro(<<"Appending polydata");
 
   // loop over all data sets, checking to see what point data is available.
@@ -155,15 +144,11 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
 
   vtkIdType numVerts = 0, numLines = 0, numStrips = 0;
 
-  int numInputs = this->GetNumberOfInputConnections(0);
-  vtkInformation *inInfo;
-
   // These Field lists are very picky.  Count the number of non empty inputs
   // so we can initialize them properly.
   for (idx = 0; idx < numInputs; ++idx)
     {
-    inInfo = inputVector[0]->GetInformationObject(idx);
-    ds = vtkPolyData::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    ds = inputs[idx];
     if (ds != NULL)
       {
       if ( ds->GetNumberOfPoints() > 0)
@@ -180,12 +165,11 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   // These are used to determine which fields are available for appending
   vtkDataSetAttributes::FieldList ptList(countPD);
   vtkDataSetAttributes::FieldList cellList(countCD);  
-  
+
   countPD = countCD = 0;
   for (idx = 0; idx < numInputs; ++idx)
     {
-    inInfo = inputVector[0]->GetInformationObject(idx);
-    ds = vtkPolyData::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    ds = inputs[idx]; 
     if (ds != NULL)
       {
       // Skip points and cells if there are no points.  Empty inputs may have no arrays.
@@ -204,7 +188,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
           }
         ++countPD;
         } // for a data set that has points
-        
+
       // Although we cannot have cells without points ... let's not nest.
       if (ds->GetNumberOfCells() > 0 )  
         {
@@ -221,7 +205,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
         numLines += ds->GetNumberOfLines();
         numPolys += ds->GetNumberOfPolys();
         numStrips += ds->GetNumberOfStrips();
-        
+
         inCD = ds->GetCellData();
         if ( countCD == 0 )
           {
@@ -252,8 +236,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   // Keep track of types for fast point append
   for (idx = 0; idx < numInputs; ++idx)
     {
-    inInfo = inputVector[0]->GetInformationObject(idx);
-    ds = vtkPolyData::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    ds = inputs[idx];
     if (ds != NULL && ds->GetNumberOfPoints()>0)
       {
       if ( firstType )
@@ -262,7 +245,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
         pointtype = ds->GetPoints()->GetData()->GetDataType();
         }
       ttype = ds->GetPoints()->GetData()->GetDataType();
-      
+
       if ( ttype != pointtype )
         {
         AllSame = 0;
@@ -296,45 +279,71 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
 
   // These are created manually for faster execution
   // Uses the properties of the last input
+  vtkDataArray *inDA=0;
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::SCALARS) > -1 )
     {
+    inDA=inPD->GetScalars();
     outputPD->CopyScalarsOff();
-    newPtScalars = inPD->GetScalars()->NewInstance();
-    newPtScalars->SetNumberOfComponents(inPD->GetScalars()->GetNumberOfComponents());
-    newPtScalars->SetName(inPD->GetScalars()->GetName());
+    newPtScalars = inDA->NewInstance();
+    newPtScalars->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtScalars->SetName(inDA->GetName());
     newPtScalars->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtScalars->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::VECTORS) > -1 )
     {
+    inDA=inPD->GetVectors();
     outputPD->CopyVectorsOff();
-    newPtVectors = inPD->GetVectors()->NewInstance();
-    newPtVectors->SetNumberOfComponents(inPD->GetVectors()->GetNumberOfComponents());
-    newPtVectors->SetName(inPD->GetVectors()->GetName());
+    newPtVectors = inDA->NewInstance();
+    newPtVectors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtVectors->SetName(inDA->GetName());
     newPtVectors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtVectors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TENSORS) > -1 )
     {
+    inDA=inPD->GetTensors();
     outputPD->CopyTensorsOff();
-    newPtTensors = inPD->GetTensors()->NewInstance();
-    newPtTensors->SetNumberOfComponents(inPD->GetTensors()->GetNumberOfComponents());
-    newPtTensors->SetName(inPD->GetTensors()->GetName());
+    newPtTensors = inDA->NewInstance();
+    newPtTensors->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTensors->SetName(inDA->GetName());
     newPtTensors->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTensors->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::NORMALS) > -1 )
     {
+    inDA=inPD->GetNormals();
     outputPD->CopyNormalsOff();
-    newPtNormals = inPD->GetNormals()->NewInstance();
-    newPtNormals->SetNumberOfComponents(inPD->GetNormals()->GetNumberOfComponents());
-    newPtNormals->SetName(inPD->GetNormals()->GetName());
+    newPtNormals = inDA->NewInstance();
+    newPtNormals->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtNormals->SetName(inDA->GetName());
     newPtNormals->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtNormals->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
   if ( ptList.IsAttributePresent(vtkDataSetAttributes::TCOORDS) > -1 )
     {
+    inDA=inPD->GetTCoords();
     outputPD->CopyTCoordsOff();
-    newPtTCoords = inPD->GetTCoords()->NewInstance();
-    newPtTCoords->SetNumberOfComponents(inPD->GetTCoords()->GetNumberOfComponents());
-    newPtTCoords->SetName(inPD->GetTCoords()->GetName());
+    newPtTCoords = inDA->NewInstance();
+    newPtTCoords->SetNumberOfComponents(inDA->GetNumberOfComponents());
+    newPtTCoords->SetName(inDA->GetName());
     newPtTCoords->SetNumberOfTuples(numPts);
+    if (inDA->HasInformation())
+      {
+      newPtTCoords->CopyInformation(inDA->GetInformation(),/*deep=*/1);
+      }
     }
 
   // Allocate the point and cell data
@@ -351,8 +360,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   for (idx = 0; idx < numInputs; ++idx)
     {
     this->UpdateProgress(0.2 + 0.8*idx/numInputs);
-    inInfo = inputVector[0]->GetInformationObject(idx);
-    ds = vtkPolyData::SafeDownCast(inInfo->Get(vtkDataObject::DATA_OBJECT()));
+    ds = inputs[idx];
     // this check is not necessary, but I'll put it in anyway
     if (ds != NULL)
       {
@@ -461,10 +469,10 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
           outputCD->CopyData(cellList,inCD,countCD,cellId,outCellId);
           }
         ++countCD;
-        
+
         // copy the cells
         pPolys = this->AppendCells(pPolys, inPolys, ptOffset);
-        
+
         // These other cell arrays could be made efficient like polys ...
         for (inVerts->InitTraversal(); inVerts->GetNextCell(npts,pts); )
           {
@@ -474,7 +482,7 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
             newVerts->InsertCellPoint(pts[i]+ptOffset);
             }
           }
-        
+
         for (inLines->InitTraversal(); inLines->GetNextCell(npts,pts); )
           {
           newLines->InsertNextCell(npts);
@@ -559,6 +567,34 @@ int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
   return 1;
 }
 
+//----------------------------------------------------------------------------
+// This method is much too long, and has to be broken up!
+// Append data sets into single polygonal data set.
+int vtkAppendPolyData::RequestData(vtkInformation *vtkNotUsed(request),
+                                   vtkInformationVector **inputVector,
+                                   vtkInformationVector *outputVector)
+{
+  // get the info object
+  // get the ouptut
+  vtkPolyData *output = vtkPolyData::GetData(outputVector, 0);
+
+  int numInputs = inputVector[0]->GetNumberOfInformationObjects();
+  if (numInputs == 1)
+    {
+    output->ShallowCopy(vtkPolyData::GetData(inputVector[0], 0));
+    return 1;
+    }
+
+  vtkPolyData** inputs = new vtkPolyData*[numInputs];
+  for (int idx = 0; idx < numInputs; ++idx)
+    {
+    inputs[idx] = vtkPolyData::GetData(inputVector[0], idx);
+    }
+  int retVal = this->ExecuteAppend(output, inputs, numInputs);
+  delete [] inputs;
+  return retVal;
+}
+
 //----------------------------------------------------------------------------
 int vtkAppendPolyData::RequestUpdateExtent(vtkInformation *vtkNotUsed(request),
                                            vtkInformationVector **inputVector,
