@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkClipDataSet.cxx,v $
+  Module:    vtkClipDataSet.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -34,10 +34,11 @@
 #include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkStreamingDemandDrivenPipeline.h"
+#include "vtkIncrementalPointLocator.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkClipDataSet, "$Revision: 1.51 $");
 vtkStandardNewMacro(vtkClipDataSet);
 vtkCxxSetObjectMacro(vtkClipDataSet,ClipFunction,vtkImplicitFunction);
 
@@ -71,6 +72,8 @@ vtkClipDataSet::vtkClipDataSet(vtkImplicitFunction *cf)
     &vtkClipDataSet::InternalProgressCallbackFunction);
   this->InternalProgressObserver->SetClientData(this);
 
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_RANGES(), 1);
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_BOUNDS(), 1);
 }
 
 //----------------------------------------------------------------------------
@@ -152,7 +155,7 @@ int vtkClipDataSet::RequestData(
   vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
   vtkInformation *outInfo = outputVector->GetInformationObject(0);
 
-  // get the input and ouptut
+  // get the input and output
   vtkDataSet *realInput = vtkDataSet::SafeDownCast(
     inInfo->Get(vtkDataObject::DATA_OBJECT()));
   // We have to create a copy of the input because clip requires being
@@ -324,13 +327,20 @@ int vtkClipDataSet::RequestData(
         }
       cellScalars->Delete();
       newPoints->Delete();
-      vtkErrorMacro(<<"Cannot clip without clip function or input scalars");
+      // When processing composite datasets with partial arrays, this warning is
+      // not applicable, hence disabling it.
+      // vtkErrorMacro(<<"Cannot clip without clip function or input scalars");
       return 1;
       }
     }
-    
-  if ( !this->GenerateClipScalars && 
-       !this->GetInputArrayToProcess(0,inputVector))
+   
+  // Typically we pass input scalars to the output unharmed. If
+  // this->GenerateClipScalars is on, then the input scalars are replaced by the
+  // "ClipDataSetScalars" array (I am not convinced we really want replace the
+  // input scalars, we should pass them as well). If we are clipping by scalar
+  // itself, then the scalar is passed only if this->GenerateClipScalars is on. 
+  // Refer to BUG #8494.
+  if (!this->GenerateClipScalars && !this->ClipFunction)
     {
     outPD->CopyScalarsOff();
     }
@@ -550,7 +560,7 @@ int vtkClipDataSet::ClipPoints(vtkDataSet* input,
 //----------------------------------------------------------------------------
 // Specify a spatial locator for merging points. By default, 
 // an instance of vtkMergePoints is used.
-void vtkClipDataSet::SetLocator(vtkPointLocator *locator)
+void vtkClipDataSet::SetLocator(vtkIncrementalPointLocator *locator)
 {
   if ( this->Locator == locator)
     {
@@ -675,24 +685,24 @@ int vtkClipDataSet::ProcessRequest(vtkInformation* request,
     {
     // compute the priority for this UpdateExtent
     vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
-    if (!inInfo)
+    vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
+    double inPriority = 1;
+    if (inInfo->Has(vtkStreamingDemandDrivenPipeline::PRIORITY()))
+      {
+      inPriority = inInfo->Get(vtkStreamingDemandDrivenPipeline::
+                            PRIORITY());
+      }
+    if (!inPriority)
       {
       return 1;
       }
-
-    double inPrior = 1;
-    if (inInfo->Has(vtkStreamingDemandDrivenPipeline::PRIORITY()))
-      {
-      inPrior = inInfo->Get(vtkStreamingDemandDrivenPipeline::
-                            PRIORITY());
-      }
-
     // Get bounds and evaluate implicit function. If all bounds
     // evaluate to a value smaller than input value, this piece
     // has priority set to 0.
 
     static double bounds[] = {-1.0,1.0, -1.0,1.0, -1.0,1.0};
-    double prior = 1;
+    double priority = 1;
 
     // determine geometric bounds of this piece
     double *wBBox = NULL;
@@ -733,8 +743,7 @@ int vtkClipDataSet::ProcessRequest(vtkInformation* request,
       else
         {
         //cerr << "Need geometric bounds meta information to evaluate priority" << endl;
-        outputVector->GetInformationObject(0)->
-          Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPrior);
+        outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPriority);
         return 1;
         }
       }
@@ -743,8 +752,7 @@ int vtkClipDataSet::ProcessRequest(vtkInformation* request,
     if (!fPtr)
       {
       //cerr << "Can not evaluate priority for that clip type" << endl;
-      outputVector->GetInformationObject(0)->
-        Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPrior);
+      outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPriority);
       return 1;
       }
 
@@ -758,22 +766,17 @@ int vtkClipDataSet::ProcessRequest(vtkInformation* request,
     fVal[6] = fPtr->EvaluateFunction(bounds[1],bounds[3],bounds[4]);
     fVal[7] = fPtr->EvaluateFunction(bounds[1],bounds[3],bounds[5]);
 
-    prior = 0;
+    priority = 0;
     int i;
     for (i=0; i<8;i++)
       {
       if (fVal[i]>this->Value)
         {
-        prior = inPrior;
+        priority = inPriority;
         break;
         }
       }
-    if (prior != inPrior)
-      {
-      //cerr << "rejected something!" << endl;
-      }
-    outputVector->GetInformationObject(0)->
-      Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),prior);
+    outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),priority);
     return 1;
     }
 
