@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkContourFilter.cxx,v $
+  Module:    vtkContourFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -40,10 +40,10 @@
 #include "vtkTimerLog.h"
 #include "vtkUniformGrid.h"
 #include "vtkUnstructuredGrid.h"
+#include "vtkIncrementalPointLocator.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkContourFilter, "$Revision: 1.128 $");
 vtkStandardNewMacro(vtkContourFilter);
 vtkCxxSetObjectMacro(vtkContourFilter,ScalarTree,vtkScalarTree);
 
@@ -70,6 +70,9 @@ vtkContourFilter::vtkContourFilter()
   // by default process active point scalars
   this->SetInputArrayToProcess(0,0,0,vtkDataObject::FIELD_ASSOCIATION_POINTS,
                                vtkDataSetAttributes::SCALARS);
+
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_RANGES(), 1);
+  this->GetInformation()->Set(vtkAlgorithm::PRESERVES_BOUNDS(), 1);
 }
 
 vtkContourFilter::~vtkContourFilter()
@@ -295,8 +298,8 @@ int vtkContourFilter::RequestData(
       this->SynchronizedTemplates3D->SetComputeScalars(this->ComputeScalars);      
       this->SynchronizedTemplates3D->
         SetInputArrayToProcess(0,this->GetInputArrayInformation(0));
-      return 
-        this->SynchronizedTemplates3D->ProcessRequest(request,inputVector,outputVector);
+
+      return this->SynchronizedTemplates3D->ProcessRequest(request,inputVector,outputVector);
       }
     } //if image data
   
@@ -370,6 +373,11 @@ int vtkContourFilter::RequestData(
 
     cgrid = vtkContourGrid::New();
     cgrid->SetInput(input);
+    if ( this->Locator )
+      {
+      cgrid->SetLocator( this->Locator );
+      }
+      
     for (i = 0; i < numContours; i++)
       {
       cgrid->SetValue(i, values[i]);
@@ -566,7 +574,7 @@ int vtkContourFilter::RequestData(
 
 // Specify a spatial locator for merging points. By default, 
 // an instance of vtkMergePoints is used.
-void vtkContourFilter::SetLocator(vtkPointLocator *locator)
+void vtkContourFilter::SetLocator(vtkIncrementalPointLocator *locator)
 {
   if ( this->Locator == locator ) 
     {
@@ -617,10 +625,8 @@ int vtkContourFilter::ProcessRequest(vtkInformation* request,
     {
     // compute the priority for this UE
     vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
-    if (!inInfo)
-      {
-      return 1;
-      }
+    vtkInformation *outInfo = outputVector->GetInformationObject(0);
+
     // get the range of the input if available
     vtkInformation *fInfo = 
       vtkDataObject::GetActiveFieldInformation
@@ -630,31 +636,50 @@ int vtkContourFilter::ProcessRequest(vtkInformation* request,
       {
       return 1;
       }
-    double *range = fInfo->Get(vtkDataObject::FIELD_RANGE());
+
+    double *range = fInfo->Get(vtkDataObject::PIECE_FIELD_RANGE());
     int numContours = this->ContourValues->GetNumberOfContours();
     if (range && numContours)
       {
       // compute the priority
       // get the incoming priority if any
-      double inPrior = 1;
+      double inPriority = 1;
       if (inInfo->Has(vtkStreamingDemandDrivenPipeline::PRIORITY()))
         {
-        inPrior = inInfo->Get(vtkStreamingDemandDrivenPipeline::PRIORITY());
+        inPriority = inInfo->Get(vtkStreamingDemandDrivenPipeline::PRIORITY());
         }
+      outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPriority);
+      if (!inPriority)
+        {
+        return 1;
+        }
+
       // do any contours intersect the range?
       double *values=this->ContourValues->GetValues();
-      double prior = 0;
       int i;
       for (i=0; i < numContours; i++)
         {
         if (values[i] >= range[0] && values[i] <= range[1])
           {
-          prior = inPrior;
-          break;
+          return 1;
           }
         }
-      outputVector->GetInformationObject(0)->
-        Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),prior);
+
+      double inRes = 1.0;
+      if (inInfo->Has(
+                      vtkStreamingDemandDrivenPipeline::UPDATE_RESOLUTION()))
+        {
+        inRes = inInfo->Get(
+                            vtkStreamingDemandDrivenPipeline::UPDATE_RESOLUTION());
+        }
+      if (inRes == 1.0)
+        {
+        outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),0.0);
+        }
+      else
+        {
+        outInfo->Set(vtkStreamingDemandDrivenPipeline::PRIORITY(),inPriority*0.1);
+        }
       }
     return 1;
     }
