@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkExtractArraysOverTime.cxx,v $
+  Module:    vtkExtractArraysOverTime.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,8 +29,9 @@
 #include "vtkObjectFactory.h"
 #include "vtkOnePieceExtentTranslator.h"
 #include "vtkPointData.h"
-#include "vtkRectilinearGrid.h"
+#include "vtkTable.h"
 #include "vtkSelection.h"
+#include "vtkSelectionNode.h"
 #include "vtkSmartPointer.h"
 #include "vtkStdString.h"
 #include "vtkStreamingDemandDrivenPipeline.h"
@@ -77,7 +78,7 @@ public: // vtkValue is made public due to a bug in VS 6.0
     {
   public:
     vtkstd::string Label;
-    vtkSmartPointer<vtkRectilinearGrid> Output;
+    vtkSmartPointer<vtkTable> Output;
     vtkSmartPointer<vtkUnsignedCharArray> ValidMaskArray;
     vtkSmartPointer<vtkDoubleArray> PointCoordinatesArray;
     };
@@ -89,7 +90,7 @@ private:
   int FieldType;
   int ContentType;
 
-  void AddTimeStepInternal(unsigned int cid, double time, vtkDataSet* data);
+  void AddTimeStepInternal(unsigned int cid, double time, vtkDataObject* data);
   void AddTimeStepInternalForLocations(unsigned int composite_index, 
     double time, vtkDataSet* input);
   vtkValue* GetOutput(const vtkKey& key, vtkDataSetAttributes* inDSA);
@@ -188,23 +189,29 @@ public:
         // TODO; To add information about where which cell/pt this grid came
         // from.
 
-        value.Output->GetPointData()->RemoveArray(
-          value.ValidMaskArray->GetName());
-        value.Output->GetPointData()->AddArray(value.ValidMaskArray);
+        // Remove vtkOriginalCellIds or vtkOriginalPointIds arrays which were
+        // added by vtkExtractSelection.
+        value.Output->GetRowData()->RemoveArray("vtkOriginalCellIds");
+        value.Output->GetRowData()->RemoveArray("vtkOriginalPointIds");
 
-        value.Output->GetPointData()->RemoveArray(
+        value.Output->GetRowData()->RemoveArray(
+          value.ValidMaskArray->GetName());
+        value.Output->GetRowData()->AddArray(value.ValidMaskArray);
+
+        value.Output->GetRowData()->RemoveArray(
           this->TimeArray->GetName());
-        value.Output->GetPointData()->AddArray(this->TimeArray);
+        value.Output->GetRowData()->AddArray(this->TimeArray);
+
  
         if (value.PointCoordinatesArray)
           {
-          value.Output->GetPointData()->RemoveArray(
+          value.Output->GetRowData()->RemoveArray(
             value.PointCoordinatesArray->GetName());
-          value.Output->GetPointData()->AddArray(value.PointCoordinatesArray);
+          value.Output->GetRowData()->AddArray(value.PointCoordinatesArray);
           }
 
         this->RemoveInvalidPoints(value.ValidMaskArray,
-          value.Output->GetPointData());
+          value.Output->GetRowData());
         output->SetBlock(cc, value.Output.GetPointer());
         output->GetMetaData(cc)->Set(vtkCompositeDataSet::NAME(),
           value.Label.c_str());
@@ -233,7 +240,7 @@ void vtkExtractArraysOverTime::vtkInternal::AddFastPathTimeline(
   stream << "GlobalID: " << gid;
   value->Label = stream.str();
 
-  vtkDataSetAttributes* outputAttributes = value->Output->GetPointData();
+  vtkDataSetAttributes* outputAttributes = value->Output->GetRowData();
 
   for (int j=0; j<numFieldArrays; j++)
     {  
@@ -259,6 +266,10 @@ void vtkExtractArraysOverTime::vtkInternal::AddFastPathTimeline(
     // Mark all pts as valid.
     value->ValidMaskArray->FillComponent(0, 1);
     }
+
+  // Fast-path does not provide us with the point coordinate information, so
+  // we cannot provide that to the output.
+  value->PointCoordinatesArray = 0;
 }
 
 //----------------------------------------------------------------------------
@@ -267,9 +278,9 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStep(
 {
   this->TimeArray->SetTuple1(this->CurrentTimeIndex, time);
 
-  if (data && data->IsA("vtkDataSet"))
+  if (data && (data->IsA("vtkDataSet") || data->IsA("vtkTable")))
     {
-    this->AddTimeStepInternal(0, time, static_cast<vtkDataSet*>(data));
+    this->AddTimeStepInternal(0, time, data);
     }
   else if (data && data->IsA("vtkCompositeDataSet"))
     {
@@ -282,6 +293,11 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStep(
         {
         this->AddTimeStepInternal(iter->GetCurrentFlatIndex(), time, ds);
         }
+      else if (
+        vtkTable* table = vtkTable::SafeDownCast(iter->GetCurrentDataObject()))
+        {
+        this->AddTimeStepInternal(iter->GetCurrentFlatIndex(), time, table);
+        }
       }
     iter->Delete();
     }
@@ -293,6 +309,12 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStep(
 void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternalForLocations(
   unsigned int vtkNotUsed(composite_index), double vtkNotUsed(time), vtkDataSet* input)
 {
+  if (!input)
+    {
+    vtkGenericWarningMacro("Ignoring since input is not a vtkDataset.");
+    return;
+    }
+
   vtkDataSetAttributes* inDSA = input->GetPointData();
   vtkCharArray* validMask = vtkCharArray::SafeDownCast(
     inDSA->GetArray("vtkValidPointMask"));
@@ -322,11 +344,11 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternalForLocations(
     // not the selected cell/point id.
     vtkKey key(0, cc);
 
-    // This will allocate a new vtkRectilinearGrid is none is present
+    // This will allocate a new vtkTable is none is present
     vtkValue* value = this->GetOutput(key, inDSA);
 
-    vtkRectilinearGrid* output = value->Output;
-    output->GetPointData()->CopyData(inDSA, cc, this->CurrentTimeIndex);
+    vtkTable* output = value->Output;
+    output->GetRowData()->CopyData(inDSA, cc, this->CurrentTimeIndex);
 
     // Mark the entry valid.
     value->ValidMaskArray->SetValue(this->CurrentTimeIndex, 1);
@@ -346,32 +368,42 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternalForLocations(
 
 //----------------------------------------------------------------------------
 void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
-  unsigned int composite_index, double time, vtkDataSet* input)
+  unsigned int composite_index, double time, vtkDataObject* input)
 {
-  if (this->ContentType == vtkSelection::LOCATIONS)
+  if (this->ContentType == vtkSelectionNode::LOCATIONS)
     {
-    this->AddTimeStepInternalForLocations(composite_index, time, input);
+    this->AddTimeStepInternalForLocations(composite_index, time, 
+      vtkDataSet::SafeDownCast(input));
     return;
     }
 
   vtkDataSetAttributes* inDSA = 0;
   const char* idarrayname = 0;
-  if (this->FieldType == vtkSelection::CELL)
+  if (this->FieldType == vtkSelectionNode::CELL)
     {
-    inDSA = input->GetCellData();
+    inDSA = vtkDataSet::SafeDownCast(input)->GetCellData();
     idarrayname = "vtkOriginalCellIds";
     }
+  else if (this->FieldType == vtkSelectionNode::POINT)
+    {
+    inDSA = vtkDataSet::SafeDownCast(input)->GetPointData();
+    idarrayname = "vtkOriginalPointIds";
+    }
+  else if (this->FieldType == vtkSelectionNode::ROW)
+    {
+    inDSA = vtkTable::SafeDownCast(input)->GetRowData();
+    idarrayname = "vtkOriginalRowIds";
+    }
   else
     {
-    inDSA = input->GetPointData();
-    idarrayname = "vtkOriginalPointIds";
+    vtkGenericWarningMacro("Ignoring since unsupported field type.");
+    return;
     }
 
-
   vtkIdTypeArray* idsArray = 
     vtkIdTypeArray::SafeDownCast(inDSA->GetArray(idarrayname));
 
-  if (this->ContentType == vtkSelection::GLOBALIDS)
+  if (this->ContentType == vtkSelectionNode::GLOBALIDS)
     {
     idsArray = vtkIdTypeArray::SafeDownCast(inDSA->GetGlobalIds());
     }
@@ -382,8 +414,6 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
     return;
     }
 
-
-
   vtkIdType numIDs = idsArray->GetNumberOfTuples();
   if (numIDs <= 0)
     {
@@ -395,10 +425,10 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
     vtkIdType curid = idsArray->GetValue(cc);
     vtkKey key(composite_index, curid);
 
-    // This will allocate a new vtkRectilinearGrid is none is present
+    // This will allocate a new vtkTable is none is present
     vtkValue* value= this->GetOutput(key, inDSA);
-    vtkRectilinearGrid* output = value->Output;
-    output->GetPointData()->CopyData(inDSA, cc, this->CurrentTimeIndex);
+    vtkTable* output = value->Output;
+    output->GetRowData()->CopyData(inDSA, cc, this->CurrentTimeIndex);
 
     // Mark the entry valid.
     value->ValidMaskArray->SetValue(this->CurrentTimeIndex, 1);
@@ -406,7 +436,7 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
     // Record the point coordinate if we are tracking a point.
     if (value->PointCoordinatesArray)
       {
-      double *point = input->GetPoint(cc);
+      double *point = vtkDataSet::SafeDownCast(input)->GetPoint(cc);
       value->PointCoordinatesArray->SetTuple(this->CurrentTimeIndex, point);
       }
 
@@ -415,7 +445,7 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
     if (value->Label.empty())
       {
       vtksys_ios::ostringstream stream;
-      if (this->ContentType == vtkSelection::GLOBALIDS)
+      if (this->ContentType == vtkSelectionNode::GLOBALIDS)
         {
         vtkIdTypeArray* gidsArray = vtkIdTypeArray::SafeDownCast(
           inDSA->GetGlobalIds());
@@ -431,8 +461,21 @@ void vtkExtractArraysOverTime::vtkInternal::AddTimeStepInternal(
           {
           stream << "Block: " << composite_index << " ; ";
           }
-        stream << (this->FieldType == vtkSelection::CELL?
-          "Cell : " : "Point : ") << curid;
+        switch (this->FieldType)
+          {
+        case vtkSelectionNode::CELL:
+          stream << "Cell : ";
+          break;
+
+        case vtkSelectionNode::POINT:
+          stream << "Point : ";
+          break;
+
+        case vtkSelectionNode::ROW:
+          stream << "Row: " ;
+          break;
+          }
+        stream << curid;
         value->Label = stream.str();
         }
       }
@@ -450,19 +493,17 @@ vtkExtractArraysOverTime::vtkInternal::GetOutput(
   if (iter == this->OutputGrids.end())
     {
     vtkValue value;
-    vtkRectilinearGrid *output = vtkRectilinearGrid::New();
+    vtkTable *output = vtkTable::New();
     value.Output.TakeReference(output);
 
-    output->SetDimensions(this->NumberOfTimeSteps, 1, 1); 
-    
-    vtkPointData *opd = output->GetPointData();
-    if (this->ContentType == vtkSelection::LOCATIONS)
+    vtkDataSetAttributes *rowData = output->GetRowData();
+    if (this->ContentType == vtkSelectionNode::LOCATIONS)
       {
-      opd->InterpolateAllocate(inDSA, this->NumberOfTimeSteps);
+      rowData->InterpolateAllocate(inDSA, this->NumberOfTimeSteps);
       }
     else
       {
-      opd->CopyAllocate(inDSA, this->NumberOfTimeSteps);
+      rowData->CopyAllocate(inDSA, this->NumberOfTimeSteps);
       }
 
     // Add an array to hold the time at each step
@@ -476,26 +517,8 @@ vtkExtractArraysOverTime::vtkInternal::GetOutput(
       timeArray->SetName("Time");
       }
     
-    // Assign this array as the x-coords
-    output->SetXCoordinates(timeArray);
-
-    // Assign dummy y and z coordinates
-    vtkDoubleArray* yCoords = vtkDoubleArray::New();
-    yCoords->SetNumberOfComponents(1);
-    yCoords->SetNumberOfTuples(1);
-    yCoords->SetTuple1(0, 0.0);
-    output->SetYCoordinates(yCoords);
-    yCoords->Delete();
-
-    vtkDoubleArray* zCoords = vtkDoubleArray::New();
-    zCoords->SetNumberOfComponents(1);
-    zCoords->SetNumberOfTuples(1);
-    zCoords->SetTuple1(0, 0.0);
-    output->SetZCoordinates(zCoords);
-    zCoords->Delete();
-
-    if (this->FieldType == vtkSelection::POINT || 
-      this->ContentType == vtkSelection::LOCATIONS)
+    if (this->FieldType == vtkSelectionNode::POINT || 
+      this->ContentType == vtkSelectionNode::LOCATIONS)
       {
       // These are the point coordinates of the original data
       vtkDoubleArray* coordsArray = vtkDoubleArray::New();
@@ -509,10 +532,13 @@ vtkExtractArraysOverTime::vtkInternal::GetOutput(
         {
         coordsArray->SetName("Point Coordinates");
         }
-      if (this->ContentType == vtkSelection::LOCATIONS)
+      if (this->ContentType == vtkSelectionNode::LOCATIONS)
         {
         coordsArray->SetName("Probe Coordinates");
         }
+      coordsArray->FillComponent(0, 0.0);
+      coordsArray->FillComponent(1, 0.0);
+      coordsArray->FillComponent(2, 0.0);
       value.PointCoordinatesArray.TakeReference(coordsArray);
       }
 
@@ -534,7 +560,6 @@ vtkExtractArraysOverTime::vtkInternal::GetOutput(
 }
 
 //****************************************************************************
-vtkCxxRevisionMacro(vtkExtractArraysOverTime, "$Revision: 1.23 $");
 vtkStandardNewMacro(vtkExtractArraysOverTime);
 //----------------------------------------------------------------------------
 vtkExtractArraysOverTime::vtkExtractArraysOverTime()
@@ -545,7 +570,7 @@ vtkExtractArraysOverTime::vtkExtractArraysOverTime()
   this->SetNumberOfInputPorts(2);
 
   this->ContentType = -1;
-  this->FieldType = vtkSelection::CELL;
+  this->FieldType = vtkSelectionNode::CELL;
 
   this->Error = vtkExtractArraysOverTime::NoError;
 
@@ -664,19 +689,19 @@ int vtkExtractArraysOverTime::RequestUpdateExtent(
                   this->Internal->FastPathIDs[this->Internal->FastPathIDIndex]);
 
     // Create a key for the data type
-    if (this->FieldType == vtkSelection::CELL)
+    if (this->FieldType == vtkSelectionNode::CELL)
       {
       inInfo1->Set(vtkStreamingDemandDrivenPipeline::FAST_PATH_OBJECT_TYPE(),
                    "CELL");
       }
-    else if(this->FieldType == vtkSelection::POINT)
+    else if(this->FieldType == vtkSelectionNode::POINT)
       {
       inInfo1->Set(vtkStreamingDemandDrivenPipeline::FAST_PATH_OBJECT_TYPE(),
                    "POINT");
       }
 
     // Create a key for the type of id
-    assert(this->ContentType == vtkSelection::GLOBALIDS);
+    assert(this->ContentType == vtkSelectionNode::GLOBALIDS);
     inInfo1->Set(vtkStreamingDemandDrivenPipeline::FAST_PATH_ID_TYPE(),
       "GLOBAL");
     this->WaitingForFastPathData = true;
@@ -721,7 +746,13 @@ int vtkExtractArraysOverTime::RequestData(
       }
 
     // Only GLOBALIDS based selection support fast path.
-    if (this->ContentType != vtkSelection::GLOBALIDS)
+    if (this->ContentType != vtkSelectionNode::GLOBALIDS)
+      {
+      this->UseFastPath = false;
+      }
+    // Only point or cell data is supported for fast path.
+    if (this->FieldType != vtkSelectionNode::POINT &&
+      this->FieldType != vtkSelectionNode::CELL)
       {
       this->UseFastPath = false;
       }
@@ -911,11 +942,11 @@ void vtkExtractArraysOverTime::CollectFastPathInput(vtkDataObject* input)
   // Get the right attribute and number of elements
   switch (this->FieldType)
     {
-    case vtkSelection::CELL:
+    case vtkSelectionNode::CELL:
       inputAttributes = input->GetCellData();
       break;
 
-    case vtkSelection::POINT:
+    case vtkSelectionNode::POINT:
       inputAttributes = input->GetPointData();
       break;
     default:
@@ -931,7 +962,7 @@ void vtkExtractArraysOverTime::CollectFastPathInput(vtkDataObject* input)
 
     }
 
-  vtkRectilinearGrid* output = vtkRectilinearGrid::New();
+  vtkTable* output = vtkTable::New();
   
   outputAttributes = output->GetPointData();
     
@@ -1005,51 +1036,41 @@ static bool vtkUpdateFastPathIDsInternal(
     return true;
     }
 
-  vtkInformation* selProperties = selection->GetProperties();
-
-  if (selProperties->Has(vtkSelection::PROCESS_ID()) &&
-    piece != selProperties->Get(vtkSelection::PROCESS_ID()) &&
-    selProperties->Get(vtkSelection::PROCESS_ID()) != -1)
+  for (unsigned int n = 0; n < selection->GetNumberOfNodes(); n++)
     {
-    // Not the piece we are concerned with. Ignore.
-    return true;
-    }
+    vtkSelectionNode* node = selection->GetNode(n);
+    vtkInformation* selProperties = node->GetProperties();
 
-  if (selection->GetContentType() == vtkSelection::SELECTIONS)
-    {
-    unsigned int numChildren = selection->GetNumberOfChildren();
-    for (unsigned int cc=0; cc < numChildren; cc++)
+    if (selProperties->Has(vtkSelectionNode::PROCESS_ID()) &&
+      piece != selProperties->Get(vtkSelectionNode::PROCESS_ID()) &&
+      selProperties->Get(vtkSelectionNode::PROCESS_ID()) != -1)
       {
-      vtkSelection* child = selection->GetChild(cc);
-      if (!vtkUpdateFastPathIDsInternal(child, piece, ids, cids))
-        {
-        return false;
-        }
+      // Not the piece we are concerned with. Ignore.
+      continue;
       }
-    return true;
-    }
 
-  unsigned int composite_index = 0;
-  if (selProperties->Has(vtkSelection::COMPOSITE_INDEX()))
-    {
-    composite_index = static_cast<unsigned int>(
-      selProperties->Get(vtkSelection::COMPOSITE_INDEX()));
-    }
+    unsigned int composite_index = 0;
+    if (selProperties->Has(vtkSelectionNode::COMPOSITE_INDEX()))
+      {
+      composite_index = static_cast<unsigned int>(
+        selProperties->Get(vtkSelectionNode::COMPOSITE_INDEX()));
+      }
 
-  vtkIdTypeArray* idArray = vtkIdTypeArray::SafeDownCast(
-    selection->GetSelectionList());
-  if (!idArray || idArray->GetNumberOfTuples() == 0)
-    {
-    // "Empty selection";
-    return false;
-    }
+    vtkIdTypeArray* idArray = vtkIdTypeArray::SafeDownCast(
+      node->GetSelectionList());
+    if (!idArray || idArray->GetNumberOfTuples() == 0)
+      {
+      // "Empty selection";
+      continue;
+      }
 
-  vtkIdType numValues = idArray->GetNumberOfTuples();
-  for (vtkIdType cc=0; cc < numValues; cc++)
-    {
-    vtkIdType selectedId = idArray->GetValue(cc);
-    ids.push_back(selectedId);
-    cids.push_back(composite_index);
+    vtkIdType numValues = idArray->GetNumberOfTuples();
+    for (vtkIdType cc=0; cc < numValues; cc++)
+      {
+      vtkIdType selectedId = idArray->GetValue(cc);
+      ids.push_back(selectedId);
+      cids.push_back(composite_index);
+      }
     }
 
   return true;
@@ -1074,13 +1095,13 @@ bool vtkExtractArraysOverTime::UpdateFastPathIDs(vtkInformationVector** inputV,
   vtkInformation* inInfo2 = inputV[1]->GetInformationObject(0);
   vtkSelection* selection = vtkSelection::GetData(inInfo2);
 
-  if ((this->ContentType == vtkSelection::INDICES) ||        
-    (this->ContentType == vtkSelection::GLOBALIDS))
+  if ((this->ContentType == vtkSelectionNode::INDICES) ||        
+    (this->ContentType == vtkSelectionNode::GLOBALIDS))
     {
     bool status = ::vtkUpdateFastPathIDsInternal(selection, piece, 
       this->Internal->FastPathIDs,
       this->Internal->FastPathCompositeIDs);
-    if (this->ContentType == vtkSelection::GLOBALIDS)
+    if (this->ContentType == vtkSelectionNode::GLOBALIDS)
       {
       // composite ids are not needed for global ids.
       this->Internal->FastPathCompositeIDs.clear();
@@ -1094,41 +1115,28 @@ bool vtkExtractArraysOverTime::UpdateFastPathIDs(vtkInformationVector** inputV,
 //----------------------------------------------------------------------------
 int vtkExtractArraysOverTime::DetermineSelectionType(vtkSelection* sel)
 {
-  int selContentType = sel->GetContentType();
-  if (selContentType == vtkSelection::SELECTIONS)
+  int contentType = -1;
+  int fieldType = -1;
+  unsigned int numNodes = sel->GetNumberOfNodes();
+  for (unsigned int cc = 0; cc < numNodes; cc++)
     {
-    int contentType = -1;
-    int fieldType = -1;
-
-    unsigned int numChildren = sel->GetNumberOfChildren();
-    for (unsigned int cc=0; cc < numChildren; cc++)
+    vtkSelectionNode* node = sel->GetNode(cc);
+    if (node)
       {
-      vtkSelection* child = sel->GetChild(cc);
-      if (child)
+      int nodeFieldType = node->GetFieldType();
+      int nodeContentType = node->GetContentType();
+      if ((fieldType != -1 && fieldType != nodeFieldType) ||
+        (contentType != -1 && contentType != nodeContentType))
         {
-        int childFieldType = child->GetFieldType();
-        int childContentType = child->GetContentType();
-
-        if ((fieldType != -1 && fieldType != childFieldType) ||
-          (contentType != -1 && contentType != childContentType))
-          {
-          vtkErrorMacro("All vtkSelection instances within a composite vtkSelection"
-            " must have the same ContentType and FieldType.");
-          return 0;
-          }
-        fieldType = childFieldType;
-        contentType = childContentType;
+        vtkErrorMacro("All vtkSelectionNode instances within a vtkSelection"
+          " must have the same ContentType and FieldType.");
+        return 0;
         }
+      fieldType = nodeFieldType;
+      contentType = nodeContentType;
       }
-
-    this->ContentType = contentType;
-    this->FieldType = fieldType;
     }
-  else
-    {
-    this->ContentType = selContentType;
-    this->FieldType = sel->GetFieldType();
-    }
-
+  this->ContentType = contentType;
+  this->FieldType = fieldType;
   return 1;
 }
