@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkProbeFilter.cxx,v $
+  Module:    vtkProbeFilter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -27,7 +27,6 @@
 
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkProbeFilter, "$Revision: 1.94 $");
 vtkStandardNewMacro(vtkProbeFilter);
 
 class vtkProbeFilter::vtkVectorOfArrays : 
@@ -50,6 +49,8 @@ vtkProbeFilter::vtkProbeFilter()
 
   this->PointList = 0;
   this->CellList = 0;
+
+  this->UseNullPoint = true;
 }
 
 //----------------------------------------------------------------------------
@@ -100,7 +101,7 @@ int vtkProbeFilter::RequestData(
   vtkInformation *sourceInfo = inputVector[1]->GetInformationObject(0);
   vtkInformation *outInfo = outputVector->GetInformationObject(0);
 
-  // get the input and ouptut
+  // get the input and output
   vtkDataSet *input = vtkDataSet::SafeDownCast(
     inInfo->Get(vtkDataObject::DATA_OBJECT()));
   vtkDataSet *source = vtkDataSet::SafeDownCast(
@@ -182,12 +183,16 @@ void vtkProbeFilter::InitializeForProbing(vtkDataSet* input,
 
   outPD->AddArray(this->MaskPoints);
 
+  // Since we haven't resize the point arrays, we need to fill them up with
+  // nulls whenever we have a miss when probing.
+  this->UseNullPoint = true;
+
   // BUG FIX: JB.
   // Output gets setup from input, but when output is imagedata, scalartype
   // depends on source scalartype not input scalartype
   if (output->IsA("vtkImageData"))
     {
-    vtkImageData *out = (vtkImageData*)output;
+    vtkImageData *out = static_cast<vtkImageData *>(output);
     vtkDataArray *s = outPD->GetScalars();
     if (s)
       {
@@ -246,6 +251,23 @@ void vtkProbeFilter::ProbeEmptyPoints(vtkDataSet *input,
   tol2 = source->GetLength();
   tol2 = tol2 ? tol2*tol2 / 1000.0 : 0.001;
 
+  // the actual sampling rate needs to be considered for a
+  // more appropriate / accurate selection of the tolerance.
+  // Otherwise the tolerance simply determined above might be
+  // so large as to cause incorrect cell location
+  double bounds[6];
+  source->GetBounds(bounds);
+  double minRes = 10000000000.0;
+  double axisRes[3];
+  for ( int  i = 0;  i < 3;  i ++ )
+    {
+    axisRes[i] = ( bounds[i * 2 + 1] - bounds[i * 2] ) / numPts;
+    if ( (axisRes[i] > 0.0) && (axisRes[i] < minRes) )
+      minRes = axisRes[i];
+    }
+  double minRes2 = minRes * minRes;
+  tol2 = tol2 > minRes2 ? minRes2 : tol2;
+
   // Loop over all input points, interpolating source data
   //
   int abort=0;
@@ -254,7 +276,7 @@ void vtkProbeFilter::ProbeEmptyPoints(vtkDataSet *input,
     {
     if ( !(ptId % progressInterval) )
       {
-      this->UpdateProgress((double)ptId/numPts);
+      this->UpdateProgress(static_cast<double>(ptId)/numPts);
       abort = GetAbortExecute();
       }
 
@@ -299,7 +321,10 @@ void vtkProbeFilter::ProbeEmptyPoints(vtkDataSet *input,
       }
     else
       {
-      outPD->NullPoint(ptId);
+      if (this->UseNullPoint)
+        {
+        outPD->NullPoint(ptId);
+        }
       }
     }
 
