@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkAxisActor.cxx,v $
+  Module:    vtkAxisActor.cxx
   Thanks:    Kathleen Bonnell, B Division, Lawrence Livermore Nat'l Laboratory
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
@@ -33,7 +33,6 @@
 // ****************************************************************
 
 vtkStandardNewMacro(vtkAxisActor);
-vtkCxxRevisionMacro(vtkAxisActor, "$Revision: 1.6 $");
 vtkCxxSetObjectMacro(vtkAxisActor, Camera, vtkCamera); 
 
 // ****************************************************************
@@ -1110,7 +1109,7 @@ bool vtkAxisActor::BuildTickPointsForYType(double p1[3], double p2[3],
 
   y = this->MajorStart;
   numTicks = 0;
-  while (y < p2[1] && numTicks < VTK_MAX_TICKS)
+  while (y <= p2[1] && numTicks < VTK_MAX_TICKS)
     {
     yPoint1[1] = xPoint[1] = yPoint2[1] = zPoint[1] = y;
     // yx portion
@@ -1151,7 +1150,7 @@ bool vtkAxisActor::BuildTickPointsForYType(double p1[3], double p2[3],
     }
   y = this->MajorStart;
   numTicks = 0;
-  while (y < p2[1] && numTicks < VTK_MAX_TICKS)
+  while (y <= p2[1] && numTicks < VTK_MAX_TICKS)
     {
     yPoint1[1] = xPoint[1] = yPoint2[1] = zPoint[1] = y;
     // yx portion
@@ -1266,7 +1265,7 @@ bool vtkAxisActor::BuildTickPointsForZType(double p1[3], double p2[3],
 
   z = this->MajorStart;
   numTicks = 0;
-  while (z < p2[2] && numTicks < VTK_MAX_TICKS)
+  while (z <= p2[2] && numTicks < VTK_MAX_TICKS)
     {
     zPoint1[2] = zPoint2[2] = xPoint[2] = yPoint[2] = z;
     // zx-portion
@@ -1308,7 +1307,7 @@ bool vtkAxisActor::BuildTickPointsForZType(double p1[3], double p2[3],
     }
   z = this->MajorStart;
   numTicks = 0;
-  while (z < p2[2] && numTicks < VTK_MAX_TICKS)
+  while (z <= p2[2] && numTicks < VTK_MAX_TICKS)
     {
     zPoint1[2] = zPoint2[2] = xPoint[2] = yPoint[2] = z;
     // zx-portion
