@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCaptionActor2D.cxx,v $
+  Module:    vtkCaptionActor2D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -32,7 +32,6 @@
 #include "vtkTextProperty.h"
 #include "vtkViewport.h"
 
-vtkCxxRevisionMacro(vtkCaptionActor2D, "$Revision: 1.35 $");
 vtkStandardNewMacro(vtkCaptionActor2D);
 
 vtkCxxSetObjectMacro(vtkCaptionActor2D,LeaderGlyph,vtkPolyData);
@@ -59,6 +58,7 @@ vtkCaptionActor2D::vtkCaptionActor2D()
   this->Caption = NULL;
   this->Border = 1;
   this->Leader = 1;
+  this->AttachEdgeOnly = 0;
   this->ThreeDimensionalLeader = 1;
   this->LeaderGlyphSize = 0.025;
   this->MaximumLeaderGlyphSize = 20;
@@ -81,7 +81,7 @@ vtkCaptionActor2D::vtkCaptionActor2D()
   this->TextActor->GetPositionCoordinate()->SetReferenceCoordinate(NULL);
   this->TextActor->GetPosition2Coordinate()->SetCoordinateSystemToDisplay();
   this->TextActor->GetPosition2Coordinate()->SetReferenceCoordinate(NULL);
-  this->TextActor->SetScaledText(1);
+  this->TextActor->SetTextScaleModeToProp();
   this->TextActor->SetTextProperty(this->CaptionTextProperty);
 
   this->BorderPolyData = vtkPolyData::New();
@@ -282,7 +282,8 @@ int vtkCaptionActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
   minPt[1] = p2[1];
 
   pt[0] = p2[0]; pt[1] = p2[1]; pt[2] = minPt[2] = 0.0;
-  if ( (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
+  if ( !this->AttachEdgeOnly && 
+    (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
     {
     minD2 = d2;
     minPt[0] = pt[0]; minPt[1] = pt[1]; 
@@ -296,7 +297,8 @@ int vtkCaptionActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
     }
 
   pt[0] = p3[0];
-  if ( (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
+  if ( !this->AttachEdgeOnly && 
+    (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
     {
     minD2 = d2;
     minPt[0] = pt[0]; minPt[1] = pt[1]; 
@@ -310,7 +312,8 @@ int vtkCaptionActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
     }
 
   pt[1] = p3[1];
-  if ( (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
+  if ( !this->AttachEdgeOnly && 
+    (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
     {
     minD2 = d2;
     minPt[0] = pt[0]; minPt[1] = pt[1]; 
@@ -324,7 +327,8 @@ int vtkCaptionActor2D::RenderOpaqueGeometry(vtkViewport *viewport)
     }
 
   pt[0] = p2[0];
-  if ( (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
+  if ( !this->AttachEdgeOnly && 
+    (d2 = vtkMath::Distance2BetweenPoints(p1,pt)) < minD2 )
     {
     minD2 = d2;
     minPt[0] = pt[0]; minPt[1] = pt[1]; 
@@ -509,6 +513,7 @@ void vtkCaptionActor2D::PrintSelf(ostream& os, vtkIndent indent)
     }
   os << indent << "Padding: " << this->Padding << "\n";
   os << indent << "Border: " << (this->Border ? "On\n" : "Off\n");
+  os << indent << "AttachEdgeOnly: " << (this->AttachEdgeOnly ? "On\n" : "Off\n");
 }
 
 //----------------------------------------------------------------------------
