@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkRIBExporter.cxx,v $
+  Module:    vtkRIBExporter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -42,7 +42,6 @@
 
 #include <vtksys/ios/sstream>
 
-vtkCxxRevisionMacro(vtkRIBExporter, "$Revision: 1.64 $");
 vtkStandardNewMacro(vtkRIBExporter);
 
 typedef double RtColor[3];
@@ -1402,9 +1401,9 @@ void vtkRIBExporter::ModifyArrayName(char *newname, const char* name)
   int cc = 0;
   for ( cc =0; name[cc]; cc++ )
     {
-    if ( name[cc] >= 'A' && name[cc] <= 'Z' ||
-         name[cc] >= '0' && name[cc] <= '9' ||
-         name[cc] >= 'a' && name[cc] <= 'z' )
+      if ( (name[cc] >= 'A' && name[cc] <= 'Z') ||
+           (name[cc] >= '0' && name[cc] <= '9') ||
+           (name[cc] >= 'a' && name[cc] <= 'z') )
       {
       newname[cc] = name[cc];
       }
