@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkEnSightReader.cxx,v $
+  Module:    vtkEnSightReader.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -33,7 +33,6 @@
 #include <vtkstd/string>
 #include <vtkstd/vector>
 
-vtkCxxRevisionMacro(vtkEnSightReader, "$Revision: 1.82 $");
 
 //----------------------------------------------------------------------------
 typedef vtkstd::vector< vtkSmartPointer<vtkIdList> > vtkEnSightReaderCellIdsTypeBase;
@@ -91,6 +90,10 @@ vtkEnSightReader::vtkEnSightReader()
   
   this->InitialRead = 1;
   this->NumberOfNewOutputs = 0;
+  
+  this->PreviousTimeStepInFile   = 1;
+  this->ForwardTimeStepShiftIS   = NULL;
+  this->ForwardTimeStepShiftMode = FORWARD_TIME_STEP_SHIFT_NON;
 }
 
 //----------------------------------------------------------------------------
@@ -166,6 +169,12 @@ vtkEnSightReader::~vtkEnSightReader()
   this->FileSets = NULL;
 
   this->ActualTimeValue = 0.0;
+  
+  if ( this->ForwardTimeStepShiftIS )
+    {
+    delete this->ForwardTimeStepShiftIS;
+    this->ForwardTimeStepShiftIS = NULL;
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -206,15 +215,15 @@ int vtkEnSightReader::RequestData(
     this->ActualTimeValue = steps[cnt];
     }
 
-  cout << "Executing with: " << this->ActualTimeValue << endl;
-
-  int i, timeSet, fileSet, timeStep, timeStepInFile, fileNum;
-  vtkDataArray *times;
-  vtkIdList *numStepsList, *filenameNumbers;
-  float newTime;
-  int numSteps;
-  char* fileName;
-  int filenameNum;
+  int    i, timeSet, fileSet, timeStep, timeStepInFile, fileNum;
+  int    numSteps;
+  int    filenameNum;
+  float  newTime;
+  char * fileName      = NULL;
+  vtkDataArray * times = NULL;
+  vtkIdList *    numStepsList    = NULL;
+  vtkIdList *    filenameNumbers = NULL;
+  static char prevFileName[300]  = { "" }; // for forward time step shifting
   
   if ( ! this->CaseFileRead)
     {
@@ -233,14 +242,23 @@ int vtkEnSightReader::RequestData(
     
     if (this->UseTimeSets)
       {
+      // The TIME section may declare multiple time sets, each coupled with
+      // an index. We first need to obtain its corresponding index in a list.
       timeSet = this->TimeSetIds->IsId(this->GeometryTimeSet);
       if (timeSet >= 0)
         {
+        // Each time set declares some time values
         times = this->TimeSets->GetItem(timeSet);
+        
+        // access the beginning time step value
         this->GeometryTimeValue = times->GetComponent(0, 0);
+        
         for (i = 1; i < times->GetNumberOfTuples(); i++)
           {
           newTime = times->GetComponent(i, 0);
+          
+          // If the value of the current time-array item is no greater than
+          // the target time step, we need to update the time step.
           if (newTime <= this->ActualTimeValue && 
               newTime > this->GeometryTimeValue)
             {
@@ -249,10 +267,22 @@ int vtkEnSightReader::RequestData(
             timeStepInFile++;
             }
           }
+        
+        // obtain the target file name
         if (this->TimeSetFileNameNumbers->GetNumberOfItems() > 0)
           {
+          // The TIME section declares possibly multiple time sets and the
+          // files referenced by each time set are accessed via some filename
+          // numbers that constitute a list. Thus the entire TIME section may 
+          // include multiple such lists.
+          
+          // obtain the index of the list (of filename numbers) for the
+          // target time set
           int collectionNum = this->TimeSetsWithFilenameNumbers->
             IsId(this->GeometryTimeSet);
+            
+          // get the list (of filename numbers) for the target time set and 
+          // then obtain the filename number of the target file via (timeStep - 1)
           if (collectionNum > -1)
             {
             filenameNumbers =
@@ -261,16 +291,33 @@ int vtkEnSightReader::RequestData(
             this->ReplaceWildcards(fileName, filenameNum);
             }
           }
+          
+        // --------------------------------------------------------------------
+        // If FILE SETS are declared, we need to determine the target file name
+        // by means of the method below.
+        // --------------------------------------------------------------------
       
         // There can only be file sets if there are also time sets.
         if (this->UseFileSets)
           {
+          // The FILE section may declare multiple file sets, each coupled
+          // with an index.
           fileSet = this->FileSets->IsId(this->GeometryFileSet);
+          
+          // Each file set may include multiple filename indices, each coupled
+          // with the number of steps covered by the file. These numbers, of 
+          // the entire file set, are stored in a list.
           numStepsList = static_cast<vtkIdList*>(this->FileSetNumberOfSteps->
                                                  GetItemAsObject(fileSet));
-          
+                                                 
+          // number of all steps of this file set, index of the target file,
+          // and the index of the time step in the target file
           if (timeStep > numStepsList->GetId(0))
             {
+            // Given the file set (determined above) covering the target
+            // time step, its FIRST file just does not include the target
+            // time step. Thus we need to skip at least one file to locate 
+            // the target file.
             numSteps = numStepsList->GetId(0);
             timeStepInFile -= numSteps;
             for (i = 1; i < numStepsList->GetNumberOfIds(); i++)
@@ -283,10 +330,23 @@ int vtkEnSightReader::RequestData(
                 }
               }
             }
+            
+          // the index of the target file has been determined, i.e., fileNum.
           if (this->FileSetFileNameNumbers->GetNumberOfItems() > 0)
             {
+            // The FILE section declares possibly multiple file sets and
+            // the files referenced by each file set are accessed via
+            // some filename numbers that constitute a list. The entire 
+            // FILE section may include mutiple such lists.
+            
+            // obtain the index of the list (of filename numbers) for the
+            // target file set
             int collectionNum = this->FileSetsWithFilenameNumbers->
               IsId(this->GeometryFileSet);
+              
+            // get the list (of filename numbers) for the target file set
+            // and then obtain the filename number of the target file
+            // via (fileNum - 1)
             if (collectionNum > -1)
               {
               filenameNumbers = this->FileSetFileNameNumbers->
@@ -299,14 +359,46 @@ int vtkEnSightReader::RequestData(
         }
       }
     
+    // set the forward time step shifting mode
+    // for accelerated data loading (bug #9289)
+    if (    (  strcmp( prevFileName, fileName ) == 0  )
+         && (  timeStepInFile >  this->PreviousTimeStepInFile  )
+         && (  numStepsList   != NULL  ) // in case each time step of the time-
+                                         // varying geometry is stored in a
+                                         // single file (bug #10117)
+         && (  timeStepInFile <= numStepsList->GetId( fileNum - 1 )  )
+       )
+      {
+      this->ForwardTimeStepShiftMode = FORWARD_TIME_STEP_SHIFT_YES;
+      if (    numStepsList 
+           && numStepsList->GetId( fileNum - 1 ) == timeStepInFile
+         )
+        {
+        this->ForwardTimeStepShiftMode = FORWARD_TIME_STEP_SHIFT_END;
+        }
+      }
+    else
+      {
+      this->ForwardTimeStepShiftMode = FORWARD_TIME_STEP_SHIFT_NON;
+      }
+    
+    
     if (!this->ReadGeometryFile(fileName, timeStepInFile, output))
       {
       vtkErrorMacro("error reading geometry file");
       delete [] fileName;
+      fileName = NULL;
       return 0;
       }
+     
+    // update the geometry file name and the in-file time step in support
+    // of forward time step shifting for accelerated data loading (bug #9289) 
+    strcpy( prevFileName, fileName );
+    prevFileName[ strlen( fileName ) ] = '\0';
+    this->PreviousTimeStepInFile = timeStepInFile;
     
     delete [] fileName;
+    fileName = NULL;
     }
   if (this->MeasuredFileName)
     {
@@ -1066,99 +1158,279 @@ int vtkEnSightReader::ReadCaseFile()
       }
     }
   
+  // 'TIME' section includes the following sub-sections
+  // 'time set: <int>'
+  // 'number of steps: <int>'
+  // 'filename numbers:' --- either inline or not
+  // 'filename start number: <int>' --- 'start' may be combined with 'increment'
+  // 'filename increment: <int>'    --- to serve as an alternative to 'numbers'
+  // 'time values:' --- either inline or not
   if (strncmp(line, "TIME", 4) == 0)
     {
-    // found TIME section
+    // (only) upon the access to the FIRST 'timeStep', min and max are initialized
     int firstTimeStep = 1;
     
     this->UseTimeSetsOn();
-    while(this->ReadNextDataLine(line) != 0 &&
-          strncmp(line, "FILE", 4) != 0)
+    
+    int lineScanResult = 0;    
+    int lineReadResult = this->ReadNextDataLine(line);
+    while (  lineReadResult != 0  &&  strncmp(line, "FILE", 4)  !=  0  )
       {
-      sscanf(line, "%*s %*s %d", &timeSet);
+      // 'time set: <int>' --- to obtain timeSet, an index
+      lineScanResult = sscanf(line, "%*s %s %d", subLine, &timeSet);
+      if ( lineScanResult != 2 || 
+        strncmp(line, "time", 4) != 0 || strcmp(subLine, "set:") != 0 )
+        {
+        vtkErrorMacro("Error with vtkEnSightReader: 'time set' not found!!!");
+        return  0;
+        }        
       this->TimeSetIds->InsertNextId(timeSet);
-      this->ReadNextDataLine(line);
-      sscanf(line, "%*s %*s %*s %d", &numTimeSteps);
-      this->ReadNextDataLine(line);
-      if (strncmp(line, "filename", 8) == 0)
+      
+      // 'number of steps: <int>' --- to obtain numTimeSteps
+      // i.e.., the number of 'filename numbers' and equally that of 'time values'
+      if ( this->ReadNextDataLine(line) == 0 )
+        {
+        vtkErrorMacro("Error with vtkEnSightReader: 'number of steps' not found!!!");
+        return  0;
+        }
+       
+      lineScanResult = sscanf(line, "%*s %*s %s %d", subLine, &numTimeSteps);
+      if ( lineScanResult != 2 ||
+        strncmp(line, "number", 6) != 0 || strcmp(subLine, "steps:") != 0 )
+        {
+        vtkErrorMacro("Error with vtkEnSightReader: 'number of steps' not found!!!");
+        return  0;
+        }
+      
+      // 'filename numbers:' ==or the combination of the following two lines==
+      // 'filename start number: <int>'
+      // 'filename increment: <int>'
+      // --- to obtain a sequence of filenameNum(s) which might span multiple lines
+      if ( this->ReadNextDataLine(line) == 0 )
+        {
+        vtkErrorMacro("Error with vtkEnSightReader: 'filename ......' not found!!!");
+        return  0;
+        }
+        
+      if ( strncmp(line, "filename", 8) == 0 )
         {
         vtkIdList *filenameNumbers = vtkIdList::New();
         this->TimeSetsWithFilenameNumbers->InsertNextId(timeSet);
-        sscanf(line, "%*s %s", subLine);
-        if (strncmp(subLine, "numbers", 7) == 0)
+        
+        if ( sscanf(line, "%*s %s", subLine) != 1 )
           {
-          strcpy(formatLine, "%*s %*s");
-          strcpy(subLine, "%*s %*s");
-          for (i = 0; i < numTimeSteps; i++)
+          vtkErrorMacro("Error with vtkEnSightReader: 'filename ......' not found!!!");
+          return  0;
+          }
+        
+        // 'filename numbers:'
+        if (strncmp(subLine, "numbers", 7) == 0)
+          {   
+          // Filename numbers may be provided on the line(s) following
+          // 'filename numbers:', as is usually the case --- not "inline". Thus we need
+          // to go to the FIRST line that indeed contains filename numbers.
+          if ( sscanf(line, "%*s %*s %d", &filenameNum) != 1 )
             {
-            strcat(formatLine, " %d");
-            sscanf(line, formatLine, &filenameNum);
+            // not "inline"
+            if ( this->ReadNextDataLine(line) == 0 )
+              {
+              vtkErrorMacro("Error with vtkEnSightReader: filename numbers not found!!!");
+              return  0;
+              }
+            // access the sub-strings from the very beginning
+            strcpy(formatLine, "");
+            strcpy(subLine, "");
+            }
+          else
+            {
+            // "inline" ----> skip the first two sub-strings: 'filename numbers:'
+            strcpy(formatLine, "%*s %*s ");
+            strcpy(subLine, "%*s %*s ");
+            }
+          
+          for ( i = 0; i < numTimeSteps; i++ )
+            {
+            strcat(formatLine, "%d ");
+            
+            // More lines might be needed to provide the remaining filename numbers
+            // and then formatLine and subLine need to be updated. 'while' is used here 
+            // instead of 'if' in case of any invalid filename numbers.
+            while ( sscanf(line, formatLine, &filenameNum) != 1 )
+              {
+              if ( this->ReadNextDataLine(line) == 0 )
+                {
+                vtkErrorMacro("Error with vtkEnSightReader: insufficient filename numbers!!!");
+                return  0;
+                }
+              
+              // in case of insufficient filename numbers
+              if ( strncmp(line, "filename start", 14) == 0 ||
+                   strncmp(line, "filename increment", 18) == 0 ||
+                   strncmp(line, "time values", 11) == 0 ||
+                   strncmp(line, "time set", 8) == 0 ||
+                   strncmp(line, "FILE", 4) == 0
+                 )
+                {
+                vtkErrorMacro("Error with vtkEnSightReader: insufficient filename numbers!!!");
+                return  0;
+                } 
+              
+              // to access a new line              
+              strcpy(formatLine, "%d ");
+              strcpy(subLine, "");
+              }
+              
             filenameNumbers->InsertNextId(filenameNum);
-            strcat(subLine, " %*d");
+            strcat(subLine, "%*d ");
             strcpy(formatLine, subLine);
             }
           }
         else
+          // subLine == "start" ----> 'filename start number: <int>' followed by
+          // 'filename increment: <int>'
           {
-          sscanf(line, "%*s %*s %*s %d", &filenameNum);
-          this->ReadNextDataLine(line);
-          sscanf(line, "%*s %*s %d", &increment);
-          for (i = 0; i < numTimeSteps; i++)
+          if ( strcmp(subLine, "start") != 0 ||
+            sscanf(line, "%*s %*s %*s %d", &filenameNum) != 1 )
+            {
+            vtkErrorMacro("Error with vtkEnSightReader: 'filename start number' not found!!!");
+            return  0;
+            }
+             
+          if ( this->ReadNextDataLine(line) == 0 )
+            {
+            vtkErrorMacro("Error with vtkEnSightReader: 'filename increment' not found!!!");
+            return  0;
+            }
+          
+          lineScanResult = sscanf(line, "%*s %s %d", subLine, &increment);       
+          if ( lineScanResult != 2 || strcmp(subLine, "increment:") != 0 )
+            {
+            vtkErrorMacro("Error with vtkEnSightReader: 'filename increment' not found!!!");
+            return  0;
+            }
+            
+          for ( i = 0; i < numTimeSteps; i++ )
             {
             filenameNumbers->InsertNextId(filenameNum + i*increment);
             }
           }
         this->TimeSetFileNameNumbers->AddItem(filenameNumbers);
         filenameNumbers->Delete();
-        this->ReadLine(line);
+       
+        // To ignore redundant filename numbers, just if any,
+        // and check if 'time values' are subsequently provided as expected.
+        do
+          {
+          lineReadResult = this->ReadNextDataLine(line);
+          } while ( lineReadResult != 0 && strncmp(line, "time values", 11) != 0 );
+      
+        if ( lineReadResult == 0 )
+          {
+          vtkErrorMacro("Error with vtkEnSightReader: 'time values' not found!!!");
+          return  0;
+          }  
         }
+      
+      // 'time values:' --- to obtain timeStep(s)
       vtkFloatArray *timeValues = vtkFloatArray::New();
       timeValues->SetNumberOfComponents(1);
       timeValues->SetNumberOfTuples(numTimeSteps);
-      strcpy(formatLine, "%*s %*s");
-      strcpy(subLine, "%*s %*s");
-      for (i = 0; i < numTimeSteps; i++)
+      
+      // Time values may be provided on the line(s) following  'time values:',
+      // as is usually the case --- not "inline". Thus we need to go to the
+      // FIRST line that indeed contains time values.
+      if ( sscanf(line, "%*s %*s %f", &timeStep) != 1 )
         {
-        strcat(formatLine, " %f");
-        if (sscanf(line, formatLine, &timeStep) != 1)
+        // not "inline"
+        if ( this->ReadNextDataLine(line) == 0 )
           {
-          this->ReadNextDataLine(line);
-          strcpy(formatLine, " %f");
-          strcpy(subLine, "");
-          while (sscanf(line, formatLine, &timeStep) < 1)
-            {
-            // The time values may start on the next line after "time values:".
-            this->ReadNextDataLine(line);
-            }
+          vtkErrorMacro("Error with vtkEnSightReader: time values not found!!!");
+          return  0;
           }
+        // access the sub-strings from the very beginning
+        strcpy(formatLine, "");
+        strcpy(subLine, "");
+        }
+      else
+        {
+        // "inline" ----> skip the first two sub-strings: 'time values:'
+        strcpy(formatLine, "%*s %*s ");
+        strcpy(subLine, "%*s %*s ");
+        }
+      
+      for ( i = 0; i < numTimeSteps; i++ )
+        {
+        strcat(formatLine, "%f ");
+        
+        // More lines might be needed to provide the remaining time values
+        // and then formatLine and subLine need to be updated. 'while' is used
+        // here instead of 'if' in case of any invalid time values.
+        while ( sscanf(line, formatLine, &timeStep) != 1 )
+          {
+          if ( this->ReadNextDataLine(line) == 0 )
+            {
+            vtkErrorMacro("Error with vtkEnSightReader: insufficient time values!!!");
+            return  0;
+            }
+          
+          // in case of insufficient time values
+          if ( strncmp(line, "time set", 8) == 0 || strncmp(line, "FILE", 4) == 0 )
+            {
+            vtkErrorMacro("Error with vtkEnSightReader: insufficient time values!!!");
+            return  0;
+            } 
+          
+          // to access a new line               
+          strcpy(formatLine, "%f ");
+          strcpy(subLine, "");
+          }
+
+        timeValues->SetComponent(i, 0, timeStep);
+        strcat(subLine, "%*f ");
+        strcpy(formatLine, subLine);
+        
+        // init min and max only upon the access to the FIRST 'timeStep'  
         if (firstTimeStep)
           {
           this->MinimumTimeValue = timeStep;
           this->MaximumTimeValue = timeStep;
           firstTimeStep = 0;
           // Set this as default TimeValue.
-          if(!this->TimeValueInitialized)
+          if ( !this->TimeValueInitialized )
             {
             this->SetTimeValue(timeStep);
             }
           }
         else
           {
-          if (timeStep < this->MinimumTimeValue)
+          if ( timeStep < this->MinimumTimeValue )
             {
             this->MinimumTimeValue = timeStep;
             }
-          else if (timeStep > this->MaximumTimeValue)
+          else
+          if ( timeStep > this->MaximumTimeValue )
             {
             this->MaximumTimeValue = timeStep;
             }
           }
-        timeValues->SetComponent(i, 0, timeStep);
-        strcat(subLine, " %*f");
-        strcpy(formatLine, subLine);
         }
       this->TimeSets->AddItem(timeValues);
       timeValues->Delete();
+      
+      // The follwing line MUST *NOT* be modified as it enables the loop
+      // "while ( lineReadResult != 0 && strncmp(line, "FILE", 4) != 0 )".
+      // It also enables the check of redundant time-step values.
+      lineReadResult = this->ReadNextDataLine(line);
+      
+      // To ignore redundant time-step values, if any, to fix bug #0007091
+      while ( lineReadResult != 0 &&
+              strncmp(line, "time set", 8) != 0 &&
+              strncmp(line, "FILE", 4) != 0
+            )
+        {
+        lineReadResult = this->ReadNextDataLine(line);
+        }
+      
       }
     }
   
