@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkXMLReader.h,v $
+  Module:    vtkXMLReader.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -31,11 +31,12 @@ class vtkDataSetAttributes;
 class vtkXMLDataElement;
 class vtkXMLDataParser;
 class vtkInformationVector;
+class vtkInformation;
 
 class VTK_IO_EXPORT vtkXMLReader : public vtkAlgorithm
 {
 public:
-  vtkTypeRevisionMacro(vtkXMLReader,vtkAlgorithm);
+  vtkTypeMacro(vtkXMLReader,vtkAlgorithm);
   void PrintSelf(ostream& os, vtkIndent indent);
   
   // Description:
@@ -134,7 +135,11 @@ protected:
   // Create a vtkAbstractArray from its cooresponding XML representation.
   // Does not allocate.
   vtkAbstractArray* CreateArray(vtkXMLDataElement* da);
-  
+
+  // Create a vtkInformationKey from its coresponding XML representation.
+  // Stores it in the instance of vtkInformationProvided. Does not allocate.
+  int CreateInformationKey(vtkXMLDataElement *eInfoKey, vtkInformation *info);
+
   // Internal utility methods.
   virtual int OpenVTKFile();
   virtual void CloseVTKFile();
