@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkBoostGraphAdapter.h,v $
+  Module:    vtkBoostGraphAdapter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -32,6 +32,7 @@
 #include "vtkDirectedGraph.h"
 #include "vtkDistributedGraphHelper.h"
 #include "vtkDataObject.h"
+#include "vtkDataArray.h"
 #include "vtkDoubleArray.h"
 #include "vtkFloatArray.h"
 #include "vtkIdTypeArray.h"
@@ -42,7 +43,6 @@
 #include "vtkTree.h"
 #include "vtkUndirectedGraph.h"
 #include "vtkVariant.h"
-#include <stddef.h> // for ptrdiff_t
 
 namespace boost {
   //===========================================================================
@@ -85,6 +85,28 @@ namespace boost {
   vtkPropertyMapMacro(vtkIdTypeArray, vtkIdType)
   vtkPropertyMapMacro(vtkDoubleArray, double)
   vtkPropertyMapMacro(vtkFloatArray, float)
+  
+  // vtkDataArray
+  template<>
+  struct property_traits<vtkDataArray*>
+  {
+    typedef double value_type;
+    typedef double reference;
+    typedef vtkIdType  key_type;
+    typedef read_write_property_map_tag category;
+  };
+
+  inline double 
+  get(vtkDataArray * const& arr, vtkIdType key)
+  {
+    return arr->GetTuple1(key);
+  }                     
+
+  inline void
+  put(vtkDataArray *arr, vtkIdType key, const double& value)
+  {
+    arr->SetTuple1(key, value);
+  }
 
   // vtkAbstractArray as a property map of vtkVariants
   template<>
@@ -109,7 +131,7 @@ namespace boost {
   }
 }
 
-#include <vtksys/stl/utility>
+#include <vtksys/stl/utility> // STL Header
 
 #include <boost/config.hpp>
 #include <boost/iterator/iterator_facade.hpp>
@@ -799,6 +821,41 @@ namespace boost {
   {
     return key;
   }
+  
+  //===========================================================================
+  // Helper for vtkGraph property maps
+  // Automatically multiplies the property value by some value (default 1)
+  template<typename PMap>
+  class vtkGraphPropertyMapMultiplier
+  {
+  public:
+    vtkGraphPropertyMapMultiplier(PMap m, float multi=1) : pmap(m),multiplier(multi){}
+    PMap pmap;
+    float multiplier;
+    typedef typename property_traits<PMap>::value_type value_type;
+    typedef typename property_traits<PMap>::reference reference;
+    typedef typename property_traits<PMap>::key_type key_type;
+    typedef typename property_traits<PMap>::category category;
+  };
+
+  template<typename PMap>
+  inline typename property_traits<PMap>::reference
+  get(
+    vtkGraphPropertyMapMultiplier<PMap> multi,
+    const typename property_traits<PMap>::key_type key)
+  {
+    return multi.multiplier * get(multi.pmap, key);
+  } 
+
+  template<typename PMap>
+  inline void
+  put(
+    vtkGraphPropertyMapMultiplier<PMap> multi,
+    const typename property_traits<PMap>::key_type key,
+    const typename property_traits<PMap>::value_type & value)
+  {
+    put(multi.pmap, key, value);
+  }
 
   // Allow algorithms to automatically extract vtkGraphIndexMap from a
   // VTK graph
@@ -855,5 +912,12 @@ namespace boost {
 
 } // namespace boost
 
+#include <boost/version.hpp>
+#if BOOST_VERSION > 104000
+#include <boost/property_map/vector_property_map.hpp>
+#else
+#include <boost/vector_property_map.hpp>
+#endif
+
 
 #endif // __vtkBoostGraphAdapter_h
