@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: ExerciseMultiProcessController.cxx,v $
+  Module:    ExerciseMultiProcessController.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -76,6 +76,27 @@ void MatrixMultArray(const T *A, T *B, vtkIdType length)
     A += 4;  B += 4;
     }
 }
+
+// Specialize for floats for greater precision.
+VTK_TEMPLATE_SPECIALIZE
+void MatrixMultArray(const float *A, float *B, vtkIdType length)
+{
+  double *tmpA = new double[length];
+  double *tmpB = new double[length];
+  for (vtkIdType i = 0; i < length; i++)
+    {
+    tmpA[i] = static_cast<double>(A[i]);
+    tmpB[i] = static_cast<double>(B[i]);
+    }
+  MatrixMultArray(tmpA, tmpB, length);
+  for (vtkIdType i = 0; i < length; i++)
+    {
+    B[i] = static_cast<float>(tmpB[i]);
+    }
+  delete[] tmpA;
+  delete[] tmpB;
+}
+
 class MatrixMultOperation : public vtkCommunicator::Operation
 {
 public:
@@ -104,7 +125,7 @@ inline T myAbs(T x)
 
 VTK_TEMPLATE_SPECIALIZE inline int AreEqual(float a, float b)
 {
-  float tolerance = myAbs(0.0001f*a);
+  float tolerance = myAbs(0.01f*a);
   return (myAbs(a-b) <= tolerance);
 }
 
@@ -1031,6 +1052,22 @@ int ExerciseMultiProcessController(vtkMultiProcessController *controller)
     args.retval = 1;
     }
 
+  int color = (group1->GetLocalProcessId() >= 0) ? 1 : 2;
+  vtkMultiProcessController *subcontroller
+    = controller->PartitionController(color, 0);
+  subcontroller->SetSingleMethod(Run, &args);
+  subcontroller->SingleMethodExecute();
+  subcontroller->Delete();
+
+  try
+    {
+    CheckSuccess(controller, !args.retval);
+    }
+  catch (ExerciseMultiProcessControllerError)
+    {
+    args.retval = 1;
+    }
+
   return args.retval;
 }
 
