@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkDistributedDataFilter.h,v $
+  Module:    vtkDistributedDataFilter.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -99,12 +99,10 @@ class vtkModelMetadata;
 class vtkMultiProcessController;
 class vtkPKdTree;
 class vtkUnstructuredGrid;
-class vtkUnstructuredGrid;
 
 class VTK_PARALLEL_EXPORT vtkDistributedDataFilter: public vtkDataObjectAlgorithm
 {
-  vtkTypeRevisionMacro(vtkDistributedDataFilter, 
-    vtkDataObjectAlgorithm);
+  vtkTypeMacro(vtkDistributedDataFilter, vtkDataObjectAlgorithm);
 
 public:
   void PrintSelf(ostream& os, vtkIndent indent);
