@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkCamera.cxx,v $
+  Module:    vtkCamera.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -19,15 +19,37 @@
 #include "vtkGraphicsFactory.h"
 #include "vtkPerspectiveTransform.h"
 #include "vtkTransform.h"
+#include "vtkCallbackCommand.h"
 
 #include <math.h>
 
-vtkCxxRevisionMacro(vtkCamera, "$Revision: 1.116 $");
 
 //----------------------------------------------------------------------------
 // Needed when we don't use the vtkStandardNewMacro.
 vtkInstantiatorNewMacro(vtkCamera);
 
+//-----------------------------------------------------------------------------
+class vtkCameraCallbackCommand : public vtkCommand
+{
+public:
+  static vtkCameraCallbackCommand *New()
+    { return new vtkCameraCallbackCommand; };
+  vtkCamera *Self;
+  void Execute(vtkObject *, unsigned long, void *)
+    {
+      if (this->Self)
+        {
+        this->Self->Modified();
+        this->Self->ComputeViewTransform();
+        this->Self->ComputeDistance();
+        this->Self->ComputeCameraLightTransform();
+        }
+    }
+protected:
+  vtkCameraCallbackCommand() { this->Self = NULL; };
+  ~vtkCameraCallbackCommand() {};
+};
+
 //----------------------------------------------------------------------------
 // Construct camera instance with its focal point at the origin, 
 // and position=(0,0,1). The view up is along the y-axis, 
@@ -75,9 +97,11 @@ vtkCamera::vtkCamera()
 
   this->Transform = vtkPerspectiveTransform::New();
   this->ViewTransform = vtkTransform::New();
-  this->PerspectiveTransform = vtkPerspectiveTransform::New();
+  this->ProjectionTransform = vtkPerspectiveTransform::New();
   this->CameraLightTransform = vtkTransform::New();
   this->UserTransform = NULL;
+  this->UserViewTransform = NULL;
+  this->UserViewTransformCallbackCommand = NULL;
 
   // initialize the ViewTransform
   this->ComputeViewTransform();
@@ -90,13 +114,24 @@ vtkCamera::~vtkCamera()
 {
   this->Transform->Delete();
   this->ViewTransform->Delete();
-  this->PerspectiveTransform->Delete();
+  this->ProjectionTransform->Delete();
   this->CameraLightTransform->Delete();
   if (this->UserTransform)
     {
     this->UserTransform->UnRegister(this);
     this->UserTransform = NULL;
     }
+  if (this->UserViewTransform)
+    {
+    this->UserViewTransform->RemoveObserver(
+      this->UserViewTransformCallbackCommand);
+    this->UserViewTransform->UnRegister(this);
+    this->UserViewTransform = NULL;
+    }
+  if (this->UserViewTransformCallbackCommand)
+    {
+    this->UserViewTransformCallbackCommand->Delete();
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -142,6 +177,7 @@ void vtkCamera::SetPosition(double x, double y, double z)
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
 void vtkCamera::SetUserTransform(vtkHomogeneousTransform *transform)
 {
   if (transform == this->UserTransform) 
@@ -161,6 +197,39 @@ void vtkCamera::SetUserTransform(vtkHomogeneousTransform *transform)
   this->Modified();
 }
 
+//----------------------------------------------------------------------------
+void vtkCamera::SetUserViewTransform(vtkHomogeneousTransform *transform)
+{
+  if (transform == this->UserViewTransform) 
+    { 
+    return; 
+    }
+  if (this->UserViewTransform) 
+    {
+    this->UserViewTransform->RemoveObserver(
+      this->UserViewTransformCallbackCommand);
+    this->UserViewTransform->Delete();
+    this->UserViewTransform = NULL;
+    }
+  if (transform)
+    {
+    this->UserViewTransform = transform;
+    this->UserViewTransform->Register(this);
+    if (!this->UserViewTransformCallbackCommand)
+      {
+      this->UserViewTransformCallbackCommand = vtkCameraCallbackCommand::New();
+      this->UserViewTransformCallbackCommand->Self = this;
+      }
+    this->UserViewTransform->AddObserver(
+      vtkCommand::ModifiedEvent,
+      this->UserViewTransformCallbackCommand);
+    }
+  this->Modified();
+  this->ComputeViewTransform();
+  this->ComputeDistance();
+  this->ComputeCameraLightTransform();
+}
+
 //----------------------------------------------------------------------------
 void vtkCamera::SetFocalPoint(double x, double y, double z)
 {
@@ -231,6 +300,10 @@ void vtkCamera::ComputeViewTransform()
 {
   // main view through the camera
   this->Transform->Identity();
+  if (this->UserViewTransform)
+    {
+    this->Transform->Concatenate(this->UserViewTransform);
+    }
   this->Transform->SetupCamera(this->Position, this->FocalPoint, this->ViewUp);
   this->ViewTransform->SetMatrix(this->Transform->GetMatrix());
 }
@@ -687,14 +760,14 @@ void vtkCamera::SetWindowCenter(double x, double y)
 //----------------------------------------------------------------------------
 void vtkCamera::SetObliqueAngles(double alpha, double beta)
 {
-  alpha *= vtkMath::DoubleDegreesToRadians();
-  beta *= vtkMath::DoubleDegreesToRadians();
+  alpha = vtkMath::RadiansFromDegrees( alpha );
+  beta = vtkMath::RadiansFromDegrees( beta );
 
-  double cotbeta = cos(beta) / sin(beta);
-  double dxdz = cos(alpha) * cotbeta;
-  double dydz = sin(alpha) * cotbeta;
+  double cotbeta = cos( beta ) / sin( beta );
+  double dxdz = cos( alpha ) * cotbeta;
+  double dydz = sin( alpha ) * cotbeta;
 
-  this->SetViewShear(dxdz, dydz, 1.0);
+  this->SetViewShear( dxdz, dydz, 1.0 );
 }
 
 //----------------------------------------------------------------------------
@@ -726,118 +799,169 @@ void vtkCamera::SetViewShear(double d[3])
   this->SetViewShear(d[0], d[1], d[2]);
 }
 
+#ifndef VTK_LEGACY_REMOVE
 //----------------------------------------------------------------------------
-// Compute the perspective transform matrix. This is used in converting
+// Compute the projection transform matrix. This is used in converting
 // between view and world coordinates.
 void vtkCamera::ComputePerspectiveTransform(double aspect,
                                             double nearz, double farz)
 {
-  this->PerspectiveTransform->Identity();
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::ComputePerspectiveTransform,"VTK 5.4",vtkCamera::ComputeProjectionTransform);
+  this->ComputeProjectionTransform(aspect,nearz,farz);
+}
+#endif
+
+//----------------------------------------------------------------------------
+// Compute the projection transform matrix. This is used in converting
+// between view and world coordinates.
+void vtkCamera::ComputeProjectionTransform(double aspect,
+                                           double nearz, double farz)
+{
+  this->ProjectionTransform->Identity();
 
   // apply user defined transform last if there is one
-  if (this->UserTransform)
+  if ( this->UserTransform )
     {
-    this->PerspectiveTransform->Concatenate(this->UserTransform->GetMatrix());
+    this->ProjectionTransform->Concatenate( this->UserTransform->GetMatrix() );
     }
 
   // adjust Z-buffer range
-  this->PerspectiveTransform->AdjustZBuffer(-1, +1, nearz, farz);
+  this->ProjectionTransform->AdjustZBuffer( -1, +1, nearz, farz );
 
-  if (this->ParallelProjection)
+  if ( this->ParallelProjection)
     {
     // set up a rectangular parallelipiped
 
-    double width = this->ParallelScale*aspect;
+    double width = this->ParallelScale * aspect;
     double height = this->ParallelScale;
 
-    double xmin = (this->WindowCenter[0]-1.0)*width;
-    double xmax = (this->WindowCenter[0]+1.0)*width;
-    double ymin = (this->WindowCenter[1]-1.0)*height;
-    double ymax = (this->WindowCenter[1]+1.0)*height;
+    double xmin = ( this->WindowCenter[0] - 1.0 ) * width;
+    double xmax = ( this->WindowCenter[0] + 1.0 ) * width;
+    double ymin = ( this->WindowCenter[1] - 1.0 ) * height;
+    double ymax = ( this->WindowCenter[1] + 1.0 ) * height;
 
-    this->PerspectiveTransform->Ortho(xmin,xmax,ymin,ymax,
+    this->ProjectionTransform->Ortho( xmin, xmax, ymin, ymax,
                                       this->ClippingRange[0],
-                                      this->ClippingRange[1]);
+                                      this->ClippingRange[1] );
     }
   else
     {
     // set up a perspective frustum
 
-    double tmp = tan(this->ViewAngle*vtkMath::DoubleDegreesToRadians()/2);
+    double tmp = tan( vtkMath::RadiansFromDegrees( this->ViewAngle ) / 2. );
     double width;
     double height;
-    if (this->UseHorizontalViewAngle)
+    if ( this->UseHorizontalViewAngle )
       {
-      width = this->ClippingRange[0]*tmp;
-      height = this->ClippingRange[0]*tmp/aspect;
+      width = this->ClippingRange[0] * tmp;
+      height = this->ClippingRange[0] * tmp / aspect;
       }
     else
       {
-      width = this->ClippingRange[0]*tmp*aspect;
-      height = this->ClippingRange[0]*tmp;
+      width = this->ClippingRange[0] * tmp * aspect;
+      height = this->ClippingRange[0] * tmp;
       }
 
-    double xmin = (this->WindowCenter[0]-1.0)*width;
-    double xmax = (this->WindowCenter[0]+1.0)*width;
-    double ymin = (this->WindowCenter[1]-1.0)*height;
-    double ymax = (this->WindowCenter[1]+1.0)*height;
+    double xmin = ( this->WindowCenter[0] - 1.0 ) * width;
+    double xmax = ( this->WindowCenter[0] + 1.0 ) * width;
+    double ymin = ( this->WindowCenter[1] - 1.0 ) * height;
+    double ymax = ( this->WindowCenter[1] + 1.0 ) * height;
 
-    this->PerspectiveTransform->Frustum(xmin, xmax, ymin, ymax,
+    this->ProjectionTransform->Frustum( xmin, xmax, ymin, ymax,
                                         this->ClippingRange[0],
-                                        this->ClippingRange[1]);
+                                        this->ClippingRange[1] );
     }
 
-  if (this->Stereo)
+  if ( this->Stereo )
     {
     // set up a shear for stereo views
-    if (this->LeftEye)
+    if ( this->LeftEye )
       {
-      this->PerspectiveTransform->Stereo(-this->EyeAngle/2,
-                                         this->Distance);
+      this->ProjectionTransform->Stereo( -this->EyeAngle/2,
+                                          this->Distance );
       }
     else
       {
-      this->PerspectiveTransform->Stereo(+this->EyeAngle/2,
-                                         this->Distance);
+      this->ProjectionTransform->Stereo( +this->EyeAngle/2,
+                                          this->Distance );
       }
     }
 
-  if (this->ViewShear[0] != 0.0 || this->ViewShear[1] != 0.0)
+  if ( this->ViewShear[0] != 0.0 || this->ViewShear[1] != 0.0 )
     {
-    this->PerspectiveTransform->Shear(this->ViewShear[0],
+    this->ProjectionTransform->Shear( this->ViewShear[0],
                                       this->ViewShear[1],
-                                      this->ViewShear[2]*this->Distance);
+                                      this->ViewShear[2] * this->Distance );
     }
 
 }
 
+#ifndef VTK_LEGACY_REMOVE
 //----------------------------------------------------------------------------
-// Return the perspective transform matrix. See ComputePerspectiveTransform.
+// Return the projection transform matrix. See ComputeProjectionTransform.
 vtkMatrix4x4 *vtkCamera::GetPerspectiveTransformMatrix(double aspect,
                                                        double nearz,
                                                        double farz)
 {
-  this->ComputePerspectiveTransform(aspect, nearz, farz);
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::GetPerspectiveTransformMatrix,"VTK 5.4",
+                           vtkCamera::GetProjectionTransformMatrix);
+  return this->GetProjectionTransformMatrix(aspect,nearz,farz);
+}
+#endif
+
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
+vtkMatrix4x4 *vtkCamera::GetProjectionTransformMatrix(double aspect,
+                                                      double nearz,
+                                                      double farz)
+{
+  this->ComputeProjectionTransform(aspect, nearz, farz);
 
   // return the transform
-  return this->PerspectiveTransform->GetMatrix();
+  return this->ProjectionTransform->GetMatrix();
 }
 
 //----------------------------------------------------------------------------
-// Return the perspective transform matrix. See ComputePerspectiveTransform.
+// Return the projection transform object. See ComputeProjectionTransform.
+vtkPerspectiveTransform *vtkCamera::GetProjectionTransformObject(double aspect,
+                                                                 double nearz,
+                                                                 double farz)
+{
+  this->ComputeProjectionTransform(aspect, nearz, farz);
+
+  // return the transform
+  return this->ProjectionTransform;
+}
+
+#ifndef VTK_LEGACY_REMOVE
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
 vtkMatrix4x4 *vtkCamera::GetCompositePerspectiveTransformMatrix(double aspect,
                                                                 double nearz,
                                                                 double farz)
 {
-  // turn off stereo, the CompositePerspectiveTransformMatrix is used for
+  VTK_LEGACY_REPLACED_BODY(vtkCamera::GetCompositePerspectiveTransformMatrix,
+                           "VTK 5.4",
+                           vtkCamera::GetCompositeProjectionTransformMatrix);
+  return this->GetCompositeProjectionTransformMatrix(aspect,nearz,farz);
+}
+#endif
+
+//----------------------------------------------------------------------------
+// Return the projection transform matrix. See ComputeProjectionTransform.
+vtkMatrix4x4 *vtkCamera::GetCompositeProjectionTransformMatrix(double aspect,
+                                                               double nearz,
+                                                               double farz)
+{
+  // turn off stereo, the CompositeProjectionTransformMatrix is used for
   // picking, not for rendering.
   int stereo = this->Stereo;
   this->Stereo = 0;
 
   this->Transform->Identity();
-  this->Transform->Concatenate(this->GetPerspectiveTransformMatrix(aspect,
-                                                                   nearz,
-                                                                   farz));
+  this->Transform->Concatenate(this->GetProjectionTransformMatrix(aspect,
+                                                                  nearz,
+                                                                  farz));
   this->Transform->Concatenate(this->GetViewTransformMatrix());
 
   this->Stereo = stereo;
@@ -918,7 +1042,7 @@ void vtkCamera::GetFrustumPlanes(double aspect, double planes[24])
   // get the composite perspective matrix
   vtkMatrix4x4::DeepCopy(
     *matrix, 
-    this->GetCompositePerspectiveTransformMatrix(aspect,-1,+1));
+    this->GetCompositeProjectionTransformMatrix(aspect,-1,+1));
   
   // transpose the matrix for use with normals
   vtkMatrix4x4::Transpose(*matrix,*matrix);
@@ -951,6 +1075,255 @@ void vtkCamera::ViewingRaysModified()
   this->ViewingRaysMTime.Modified();
 }
 
+// ----------------------------------------------------------------------------
+// Description:
+// Copy the properties of `source' into `this'.
+// Copy pointers of matrices.
+// \pre source_exists!=0
+// \pre not_this: source!=this
+void vtkCamera::ShallowCopy(vtkCamera *source)
+{
+  assert("pre: source_exists" && source!=0);
+  assert("pre: not_this" && source!=this);
+  
+  this->PartialCopy(source);
+  
+  // Shallow copy of matrices:
+  if(this->UserTransform!=0)
+    {
+    this->UserTransform->Delete();
+    }
+  this->UserTransform=source->UserTransform;
+  if(this->UserTransform!=0)
+    {
+    this->UserTransform->Register(this);
+    }
+  if(this->UserViewTransform!=0)
+    {
+    this->UserViewTransform->Delete();
+    }
+  this->UserViewTransform=source->UserViewTransform;
+  if(this->UserViewTransform!=0)
+    {
+    this->UserViewTransform->Register(this);
+    }
+  
+  if(this->ViewTransform!=0)
+    {
+    this->ViewTransform->Delete();
+    }
+  this->ViewTransform=source->ViewTransform;
+  if(this->ViewTransform!=0)
+    {
+    this->ViewTransform->Register(this);
+    }
+  
+  if(this->ProjectionTransform!=0)
+    {
+    this->ProjectionTransform->Delete();
+    }
+  this->ProjectionTransform=source->ProjectionTransform;
+  if(this->ProjectionTransform!=0)
+    {
+    this->ProjectionTransform->Register(this);
+    }
+  
+  if(this->Transform!=0)
+    {
+    this->Transform->Delete();
+    }
+  this->Transform=source->Transform;
+  if(this->Transform!=0)
+    {
+    this->Transform->Register(this);
+    }
+  
+  if(this->CameraLightTransform!=0)
+    {
+    this->CameraLightTransform->Delete();
+    }
+  this->CameraLightTransform=source->CameraLightTransform;
+  if(this->CameraLightTransform!=0)
+    {
+    this->CameraLightTransform->Register(this);
+    }
+}
+  
+// ----------------------------------------------------------------------------
+// Description:
+// Copy the properties of `source' into `this'.
+// Copy the contents of the matrices.
+// \pre source_exists!=0
+// \pre not_this: source!=this
+void vtkCamera::DeepCopy(vtkCamera *source)
+{
+  assert("pre: source_exists" && source!=0);
+  assert("pre: not_this" && source!=this);
+  
+  this->PartialCopy(source);
+  
+  // Deep copy the matrices:
+  if(source->UserTransform==0)
+    {
+    if(this->UserTransform!=0)
+      {
+      this->UserTransform->UnRegister(this);
+      this->UserTransform=0;
+      }
+    }
+  else
+    {
+    if(this->UserTransform==0)
+      {
+      this->UserTransform=
+        static_cast<vtkHomogeneousTransform *>(
+          source->UserTransform->MakeTransform());
+      }
+     this->UserTransform->DeepCopy(source->UserTransform);
+    }
+
+  if(source->UserViewTransform==0)
+    {
+    if(this->UserViewTransform!=0)
+      {
+      this->UserViewTransform->UnRegister(this);
+      this->UserViewTransform=0;
+      }
+    }
+  else
+    {
+    if(this->UserViewTransform==0)
+      {
+      this->UserViewTransform=
+        static_cast<vtkHomogeneousTransform *>(
+          source->UserViewTransform->MakeTransform());
+      }
+     this->UserViewTransform->DeepCopy(source->UserViewTransform);
+    }
+  
+  if(source->ViewTransform==0)
+    {
+    if(this->ViewTransform!=0)
+      {
+      this->ViewTransform->UnRegister(this);
+      this->ViewTransform=0;
+      }
+    }
+  else
+    {
+    if(this->ViewTransform==0)
+      {
+      this->ViewTransform=
+        static_cast<vtkTransform *>(
+          source->ViewTransform->MakeTransform());
+      }
+     this->ViewTransform->DeepCopy(source->ViewTransform);
+    }
+  
+  if(source->ProjectionTransform==0)
+    {
+    if(this->ProjectionTransform!=0)
+      {
+      this->ProjectionTransform->UnRegister(this);
+      this->ProjectionTransform=0;
+      }
+    }
+  else
+    {
+    if(this->ProjectionTransform==0)
+      {
+      this->ProjectionTransform=
+        static_cast<vtkPerspectiveTransform *>(
+          source->ProjectionTransform->MakeTransform());
+      }
+     this->ProjectionTransform->DeepCopy(source->ProjectionTransform);
+    }
+  
+  if(source->Transform==0)
+    {
+    if(this->Transform!=0)
+      {
+      this->Transform->UnRegister(this);
+      this->Transform=0;
+      }
+    }
+  else
+    {
+    if(this->Transform==0)
+      {
+      this->Transform=
+        static_cast<vtkPerspectiveTransform *>(
+          source->Transform->MakeTransform());
+      }
+     this->Transform->DeepCopy(source->Transform);
+    }
+  
+  if(source->CameraLightTransform==0)
+    {
+    if(this->CameraLightTransform!=0)
+      {
+      this->CameraLightTransform->UnRegister(this);
+      this->CameraLightTransform=0;
+      }
+    }
+  else
+    {
+    if(this->CameraLightTransform==0)
+      {
+      this->CameraLightTransform=
+        static_cast<vtkTransform *>(
+          source->CameraLightTransform->MakeTransform());
+      }
+     this->CameraLightTransform->DeepCopy(source->CameraLightTransform);
+    }
+}
+
+// ----------------------------------------------------------------------------
+// Description:
+// Copy the ivars. Do nothing for the matrices.
+// Called by ShallowCopy() and DeepCopy()
+// \pre source_exists!=0
+// \pre not_this: source!=this
+void vtkCamera::PartialCopy(vtkCamera *source)
+{
+  assert("pre: source_exists" && source!=0);
+  assert("pre: not_this" && source!=this);
+  
+  int i;
+  
+  i=0;
+  while(i<2)
+    {
+    this->WindowCenter[i]=source->WindowCenter[i];
+    this->ObliqueAngles[i]=source->ObliqueAngles[i];
+    this->ClippingRange[i]=source->ClippingRange[i];
+    ++i;
+    }
+  i=0;
+  while(i<3)
+    {
+    this->FocalPoint[i]=source->FocalPoint[i];
+    this->Position[i]=source->Position[i];
+    this->ViewUp[i]=source->ViewUp[i];
+    this->DirectionOfProjection[i]=source->DirectionOfProjection[i];
+    this->ViewPlaneNormal[i]=source->ViewPlaneNormal[i];
+    this->ViewShear[i]=source->ViewShear[i];
+    ++i;
+    }
+  
+  this->ViewAngle=source->ViewAngle;
+  this->EyeAngle=source->EyeAngle;
+  this->ParallelProjection=source->ParallelProjection;
+  this->ParallelScale=source->ParallelScale;
+  this->Stereo=source->Stereo;
+  this->LeftEye=source->LeftEye;
+  this->Thickness=source->Thickness;
+  this->Distance=source->Distance;
+  this->UseHorizontalViewAngle=source->UseHorizontalViewAngle;
+  this->FocalDisk=source->FocalDisk;
+  this->ViewingRaysMTime=source->ViewingRaysMTime;
+}
+
 //----------------------------------------------------------------------------
 void vtkCamera::PrintSelf(ostream& os, vtkIndent indent)
 {
@@ -989,6 +1362,14 @@ void vtkCamera::PrintSelf(ostream& os, vtkIndent indent)
     {
     os << "(none)\n";
     }
+  if (this->UserViewTransform)
+    {
+    os << this->UserViewTransform << "\n";
+    }
+  else
+    {
+    os << "(none)\n";
+    }
   os << indent << "ViewPlaneNormal: (" << this->ViewPlaneNormal[0]
      << ", " << this->ViewPlaneNormal[1]
      << ", " << this->ViewPlaneNormal[2] << ")\n";
