@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkChooserPainter.cxx,v $
+  Module:    vtkChooserPainter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -28,7 +28,6 @@
 #include "vtkStandardPolyDataPainter.h"
 #include "vtkTStripsPainter.h"
 
-vtkCxxRevisionMacro(vtkChooserPainter, "$Revision: 1.7 $");
 vtkStandardNewMacro(vtkChooserPainter);
 
 vtkCxxSetObjectMacro(vtkChooserPainter, VertPainter, vtkPolyDataPainter);
@@ -310,7 +309,7 @@ void vtkChooserPainter::RenderInternal(vtkRenderer* renderer, vtkActor* actor,
 
   this->ProgressOffset = 0.0;
   this->TimeToDraw = 0.0;
-  if (typeflags & vtkPainter::VERTS)
+  if ((typeflags & vtkPainter::VERTS) && numVerts>0 )
     {
     //cout << this << "Verts" << endl;
     this->ProgressScaleFactor = static_cast<double>(numVerts)/total_cells;
@@ -320,7 +319,7 @@ void vtkChooserPainter::RenderInternal(vtkRenderer* renderer, vtkActor* actor,
     this->ProgressOffset += this->ProgressScaleFactor;
     }
   
-  if (typeflags & vtkPainter::LINES)
+  if ((typeflags & vtkPainter::LINES) && numLines>0 )
     {
     //cout << this << "Lines" << endl;
     this->ProgressScaleFactor = static_cast<double>(numLines)/total_cells;
@@ -331,12 +330,14 @@ void vtkChooserPainter::RenderInternal(vtkRenderer* renderer, vtkActor* actor,
     }
   
   
-  if (typeflags & vtkPainter::POLYS)
+  if ((typeflags & vtkPainter::POLYS) && numPolys>0 )
     {
     //cout << this << "Polys" << endl;
     this->ProgressScaleFactor = static_cast<double>(numPolys)/total_cells;
-    if (this->UseLinesPainterForWireframes && 
-      actor->GetProperty()->GetRepresentation() == VTK_WIREFRAME)
+    if (   this->UseLinesPainterForWireframes
+        && (actor->GetProperty()->GetRepresentation() == VTK_WIREFRAME)
+        && !actor->GetProperty()->GetBackfaceCulling()
+        && !actor->GetProperty()->GetFrontfaceCulling() )
       {
       this->LinePainter->Render(renderer, actor, vtkPainter::POLYS,
         forceCompileOnly);
@@ -351,7 +352,7 @@ void vtkChooserPainter::RenderInternal(vtkRenderer* renderer, vtkActor* actor,
     this->ProgressOffset += this->ProgressScaleFactor;
     }
 
-  if (typeflags & vtkPainter::STRIPS)
+  if ((typeflags & vtkPainter::STRIPS) && numStrips>0 )
     {
     //cout << this << "Strips" << endl;
     this->ProgressScaleFactor = static_cast<double>(numStrips)/total_cells;
