@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkFreeTypeUtilities.cxx,v $
+  Module:    vtkFreeTypeUtilities.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -39,7 +39,6 @@
 #define VTK_FTFC_DEBUG_CD 0
 
 //----------------------------------------------------------------------------
-vtkCxxRevisionMacro(vtkFreeTypeUtilities, "$Revision: 1.29 $");
 vtkInstantiatorNewMacro(vtkFreeTypeUtilities);
 
 //----------------------------------------------------------------------------
@@ -427,14 +426,14 @@ vtkFreeTypeUtilitiesFaceRequester(FTC_FaceID face_id,
          << ", I: " << tprop->GetItalic()
          << ", O: " << tprop->GetOrientation() << ")" << endl;
 #endif    
-    if (tprop->GetOrientation() != 0.0)
+    if ( tprop->GetOrientation() != 0.0 )
       {
       // FreeType documentation says that the transform should not be set
       // but we cache faces also by transform, so that there is a unique
       // (face, orientation) cache entry
 
       FT_Matrix matrix;
-      float angle = vtkMath::DegreesToRadians() * tprop->GetOrientation(); 
+      float angle = vtkMath::RadiansFromDegrees( tprop->GetOrientation() ); 
       matrix.xx = (FT_Fixed)( cos(angle) * 0x10000L);
       matrix.xy = (FT_Fixed)(-sin(angle) * 0x10000L);
       matrix.yx = (FT_Fixed)( sin(angle) * 0x10000L);
@@ -942,7 +941,7 @@ int vtkFreeTypeUtilities::GetBoundingBox(vtkTextProperty *tprop,
                         gindex, 
                         &glyph, 
                         vtkFreeTypeUtilities::GLYPH_REQUEST_BITMAP) ||
-        glyph->format != FT_GLYPH_FORMAT_BITMAP)
+        glyph->format != ft_glyph_format_bitmap)
       {
       continue;
       }
@@ -976,7 +975,7 @@ int vtkFreeTypeUtilities::GetBoundingBox(vtkTextProperty *tprop,
       if (face_has_kerning && previous_gindex && gindex)
         {
         FT_Get_Kerning(
-          face, previous_gindex, gindex, FT_KERNING_DEFAULT, &kerning_delta);
+          face, previous_gindex, gindex, ft_kerning_default, &kerning_delta);
         pen_x += kerning_delta.x >> 6;
         pen_y += kerning_delta.y >> 6;
         }
@@ -1182,7 +1181,7 @@ int vtkFreeTypeUtilities::PopulateImageData(vtkTextProperty *tprop,
                         gindex, 
                         &glyph, 
                         vtkFreeTypeUtilities::GLYPH_REQUEST_BITMAP) ||
-        glyph->format != FT_GLYPH_FORMAT_BITMAP)
+        glyph->format != ft_glyph_format_bitmap)
       {
       continue;
       }
@@ -1192,7 +1191,7 @@ int vtkFreeTypeUtilities::PopulateImageData(vtkTextProperty *tprop,
     bitmap_glyph = reinterpret_cast<FT_BitmapGlyph>(glyph);
     bitmap = &bitmap_glyph->bitmap;
 
-    if (bitmap->pixel_mode != FT_PIXEL_MODE_GRAY)
+    if (bitmap->pixel_mode != ft_pixel_mode_grays)
       {
       continue;
       }
@@ -1225,7 +1224,7 @@ int vtkFreeTypeUtilities::PopulateImageData(vtkTextProperty *tprop,
       if (face_has_kerning && previous_gindex && gindex)
         {
         FT_Get_Kerning(
-          face, previous_gindex, gindex, FT_KERNING_DEFAULT, &kerning_delta);
+          face, previous_gindex, gindex, ft_kerning_default, &kerning_delta);
         pen_x += kerning_delta.x >> 6;
         pen_y += kerning_delta.y >> 6;
         }
