@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPOVExporter.cxx,v $
+  Module:    vtkPOVExporter.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,7 +15,7 @@
 /*=========================================================================
 
   Program:   VTK/ParaView Los Alamos National Laboratory Modules (PVLANL)
-  Module:    $RCSfile: vtkPOVExporter.cxx,v $
+  Module:    vtkPOVExporter.cxx
 
 Copyright (c) 2007, Los Alamos National Security, LLC
 
@@ -83,7 +83,6 @@ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "vtkObjectFactory.h"
 
-vtkCxxRevisionMacro(vtkPOVExporter, "$Revision: 1.6 $");
 vtkStandardNewMacro(vtkPOVExporter);
 
 //Can't use printf("%d", a_vtkIdType) because vtkIdType is not always int.
@@ -144,6 +143,10 @@ vtkPOVExporter::vtkPOVExporter()
 
 vtkPOVExporter::~vtkPOVExporter()
 {
+  if(this->FileName!=0)
+    {
+    delete[] this->FileName;
+    }
   delete this->Internals;
 }
 
