@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkProperty.h,v $
+  Module:    vtkProperty.h
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,7 +29,6 @@
 #define __vtkProperty_h
 
 #include "vtkObject.h"
-#include "vtkTexture.h" // used to include texture unit enum.
 
 // shading models
 #define VTK_FLAT    0
@@ -54,7 +53,7 @@ class vtkPropertyInternals;
 class VTK_RENDERING_EXPORT vtkProperty : public vtkObject
 {
 public:
-  vtkTypeRevisionMacro(vtkProperty,vtkObject);
+  vtkTypeMacro(vtkProperty,vtkObject);
   void PrintSelf(ostream& os, vtkIndent indent);
 
   // Description:
@@ -91,6 +90,12 @@ public:
   virtual void PostRender(vtkActor*, vtkRenderer*);
   //ETX
 
+  // Description:
+  // Set/Get lighting flag for an object. Initial value is true.
+  vtkGetMacro(Lighting,bool);
+  vtkSetMacro(Lighting,bool);
+  vtkBooleanMacro(Lighting,bool);
+  
   // Description:
   // Set the shading interpolation method for an object.
   vtkSetClampMacro(Interpolation,int,VTK_FLAT,VTK_PHONG);
@@ -137,7 +142,7 @@ public:
 
   // Description:
   // Set/Get the specular power.
-  vtkSetClampMacro(SpecularPower,double,0.0,100.0);
+  vtkSetClampMacro(SpecularPower,double,0.0,128.0);
   vtkGetMacro(SpecularPower,double);
 
   // Description:
@@ -253,16 +258,16 @@ public:
   // this will return null.
   vtkGetObjectMacro(ShaderProgram, vtkShaderProgram);
 
-  // Description
-  // Provide values to initialize shader variables. This is a conduit to initialize
-  // shader variables that change over time, useful for animation, gui widget inputs,
-  // etc.
-  // name - hardware name of the uniform variable
-  // numVars - number of variables being set
-  // x - values
-  virtual void AddShaderVariable(const char* name, int numVars, int* x);
-  virtual void AddShaderVariable(const char* name, int numVars, float* x);
-  virtual void AddShaderVariable(const char* name, int numVars, double* x);
+  // Description:
+  // Provide values to initialize shader variables.
+  // Useful to initialize shader variables that change over time
+  // (animation, GUI widgets inputs, etc. )
+  // - \p name - hardware name of the uniform variable
+  // - \p numVars - number of variables being set
+  // - \p x - values
+  virtual void AddShaderVariable(const char *name, int numVars, int *x);
+  virtual void AddShaderVariable(const char *name, int numVars, float *x);
+  virtual void AddShaderVariable(const char *name, int numVars, double *x);
 
   // Description:
   // Methods to provide to add shader variables from tcl.
@@ -339,6 +344,7 @@ public:
   // must be assigned unique names.
   void SetTexture(int unit, vtkTexture* texture);
   vtkTexture* GetTexture(int unit);
+  void RemoveTexture(int unit);
 
 
   // Description:
@@ -360,6 +366,23 @@ public:
   // resources to release.
   virtual void ReleaseGraphicsResources(vtkWindow *win);
 
+  // Description:
+  // Used to specify which texture unit a texture will use.
+  // Only relevant when multitexturing.
+//BTX
+  enum VTKTextureUnit
+  {
+    VTK_TEXTURE_UNIT_0 = 0,
+    VTK_TEXTURE_UNIT_1,
+    VTK_TEXTURE_UNIT_2,
+    VTK_TEXTURE_UNIT_3,
+    VTK_TEXTURE_UNIT_4,
+    VTK_TEXTURE_UNIT_5,
+    VTK_TEXTURE_UNIT_6,
+    VTK_TEXTURE_UNIT_7
+  };
+//ETX
+
 protected:
   vtkProperty();
   ~vtkProperty();
@@ -391,6 +414,7 @@ protected:
   int   EdgeVisibility;
   int   BackfaceCulling;
   int   FrontfaceCulling;
+  bool Lighting;
 
   int Shading;
 
@@ -402,8 +426,12 @@ protected:
 
   vtkXMLMaterial* Material; // TODO: I wonder if this reference needs to be maintained.
 
+  // Description:
+  // Read this->Material from new style shaders.
+  // Default implementation is empty.
+  virtual void ReadFrameworkMaterial();
+  
 //BTX
-private:
   // These friends are provided only for the time being
   // till we device a graceful way of loading texturing for GLSL.
   friend class vtkGLSLShaderProgram;
@@ -412,8 +440,10 @@ private:
   // Don't use these methods. They will be removed. They are provided only
   // for the time-being.
   vtkTexture* GetTextureAtIndex(int index);
-  int GetTextureIndex(const char* name);
+  int GetTextureUnitAtIndex(int index);
+  int GetTextureUnit(const char* name);
 //ETX
+
 private:
   vtkProperty(const vtkProperty&);  // Not implemented.
   void operator=(const vtkProperty&);  // Not implemented.
