@@ -1,10 +1,10 @@
 /*=========================================================================
 
   Program:   Insight Segmentation & Registration Toolkit
-  Module:    $RCSfile: metaCommand.cxx,v $
+  Module:    metaCommand.cxx
   Language:  C++
-  Date:      $Date: 2008-04-25 13:31:38 $
-  Version:   $Revision: 1.23 $
+  Date:      $Date$
+  Version:   $Revision$
 
   Copyright (c) 2002 Insight Consortium. All rights reserved.
   See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.
@@ -18,6 +18,7 @@
 #pragma warning(disable:4702)
 #endif
 
+#include "metaUtils.h"
 #include "metaCommand.h"
 
 #include <stdio.h>
@@ -132,7 +133,8 @@ SetOption(METAIO_STL::string name,
               << " You should use the SetOptionLongTag(optionName,longTagName)"
               << " if you want to use a longer tag. The longtag will be"
               << " refered as --LongTag and the short tag as -ShortTag."
-              << " Replace -" << shortTag << " by --" << shortTag 
+              << " Replace -" << shortTag.c_str() 
+              << " by --" << shortTag.c_str() 
               << METAIO_STREAM::endl;
     }
 
@@ -178,7 +180,8 @@ SetOption(METAIO_STL::string name,
               << " You should use the SetOptionLongTag(optionName,longTagName)"
               << " if you want to use a longer tag. The longtag will be "
               << " refered as --LongTag and the short tag as -ShortTag "
-              << " Replace -" << shortTag << " by --" << shortTag 
+              << " Replace -" << shortTag.c_str() 
+              << " by --" << shortTag.c_str() 
               << METAIO_STREAM::endl;
     }
 
@@ -358,6 +361,35 @@ SetOptionRange(METAIO_STL::string optionName,
   return false;
 }
 
+/** Set the range of an option */
+bool MetaCommand::
+SetOptionEnumerations(METAIO_STL::string optionName,
+                      METAIO_STL::string name,
+                      METAIO_STL::string optionEnums)
+                     
+{
+  OptionVector::iterator it = m_OptionVector.begin();
+  while(it != m_OptionVector.end())
+    {
+    if((*it).name == optionName)
+      {
+      METAIO_STL::vector<Field> & fields = (*it).fields;
+      METAIO_STL::vector<Field>::iterator itField = fields.begin();
+      while(itField != fields.end())
+        {
+        if((*itField).name == name)
+          {
+          (*itField).rangeMin = optionEnums;
+          return true;
+          }
+        itField++;
+        }
+      }
+    it++;
+    }
+  return false;
+}
+
 
 /** Return the value of the option as a boolean */
 bool MetaCommand::
@@ -698,7 +730,22 @@ ListOptions()
                           << METAIO_STREAM::endl;
       METAIO_STREAM::cout << "      Value: " << (*itField).value.c_str() 
                           << METAIO_STREAM::endl;
-      
+      if( (*itField).type == ENUM )
+        {
+        METAIO_STREAM::cout << "      Enum list: " 
+                            << (*itField).rangeMin.c_str()
+                            << METAIO_STREAM::endl;
+        }
+      else
+        {
+        METAIO_STREAM::cout << "      RangeMin: " 
+                            << (*itField).rangeMin.c_str()
+                            << METAIO_STREAM::endl;
+        METAIO_STREAM::cout << "      RangeMax: " 
+                            << (*itField).rangeMax.c_str()
+                            << METAIO_STREAM::endl;
+        }
+        
       if((*itField).externaldata)
         {
         METAIO_STREAM::cout << "      External Data: true" 
@@ -718,6 +765,16 @@ ListOptions()
         {
         METAIO_STREAM::cout << "      Required: false" << METAIO_STREAM::endl;
         }
+
+      if((*itField).userDefined)
+        {
+        METAIO_STREAM::cout << "      User Defined: true" << METAIO_STREAM::endl;
+        }
+      else
+        {
+        METAIO_STREAM::cout << "      User Defined: false" << METAIO_STREAM::endl;
+        }
+
       itField++;
       }
     METAIO_STREAM::cout << METAIO_STREAM::endl;
@@ -839,12 +896,17 @@ void MetaCommand::WriteXMLOptionToCout(METAIO_STL::string optionName,
     {
     optionType = "integer";
     }
+  else if((*itField).type == MetaCommand::ENUM)
+    {
+    optionType = "string-enumeration";
+    }
   else
     {
     optionType = this->TypeToString((*itField).type).c_str();
     }
 
-  METAIO_STREAM::cout << "<" << optionType << ">" << METAIO_STREAM::endl;
+  METAIO_STREAM::cout << "<" << optionType.c_str() 
+                      << ">" << METAIO_STREAM::endl;
  
 
   METAIO_STREAM::cout << "<name>" << (*it).name.c_str() << "</name>" 
@@ -890,9 +952,24 @@ void MetaCommand::WriteXMLOptionToCout(METAIO_STL::string optionName,
     {
     METAIO_STREAM::cout << "<channel>output</channel>" << METAIO_STREAM::endl;
     } 
+
+  if((*itField).type == MetaCommand::ENUM)
+    {
+    METAIO_STL::vector< METAIO_STL::string > enumVector;
+    MET_StringToVector< METAIO_STL::string>( (*itField).rangeMin, enumVector );
+    METAIO_STL::vector< METAIO_STL::string >::iterator itenum;
+    itenum = enumVector.begin();
+    while(itenum != enumVector.end() )
+      {
+      METAIO_STREAM::cout << "<element>" << (*itenum).c_str() << "</element>" 
+                          << METAIO_STREAM::endl;
+      ++itenum;
+      }
+    }
       
   // Write out the closing tag 
-  METAIO_STREAM::cout << "</" << optionType << ">" << METAIO_STREAM::endl;
+  METAIO_STREAM::cout << "</" << optionType.c_str()
+                      << ">" << METAIO_STREAM::endl;
 }
 
 /** List the current options in Slicer's xml format (www.slicer.org) */
@@ -920,16 +997,26 @@ void MetaCommand::ListOptionsSlicerXML()
   ParameterGroupVector::const_iterator itGroup = m_ParameterGroup.begin();
   while(itGroup != m_ParameterGroup.end())
     {
-    METAIO_STREAM::cout << " <parameters>" <<  METAIO_STREAM::endl;
-    METAIO_STREAM::cout << "  <label>" << (*itGroup).name <<  "</label>" <<  METAIO_STREAM::endl;
-    
-    if((*itGroup).description.size() == 0)
+    if( (*itGroup).advanced == true )
       {
-      METAIO_STREAM::cout << "  <description>" << (*itGroup).name << "</description>" <<  METAIO_STREAM::endl;
+      METAIO_STREAM::cout << " <parameters advanced=\"true\">" <<  METAIO_STREAM::endl;
       }
     else
       {
-      METAIO_STREAM::cout << "  <description>" << (*itGroup).description << "</description>" <<  METAIO_STREAM::endl;
+      METAIO_STREAM::cout << " <parameters>" <<  METAIO_STREAM::endl;
+      }
+    METAIO_STREAM::cout << "  <label>" << (*itGroup).name.c_str() 
+                        <<  "</label>" <<  METAIO_STREAM::endl;
+    
+    if((*itGroup).description.size() == 0)
+      {
+      METAIO_STREAM::cout << "  <description>" << (*itGroup).name.c_str()
+                          << "</description>" <<  METAIO_STREAM::endl;
+      }
+    else
+      {
+      METAIO_STREAM::cout << "  <description>" << (*itGroup).description.c_str() 
+                          << "</description>" <<  METAIO_STREAM::endl;
       }
 
     METAIO_STL::vector<METAIO_STL::string>::const_iterator itOption = (*itGroup).options.begin();
@@ -2088,6 +2175,8 @@ METAIO_STL::string MetaCommand::TypeToString(TypeEnumType type)
       return "image";
     case FILE:
       return "file";
+    case ENUM:
+      return "enum";
     default:
       return "not defined";
     }
@@ -2118,6 +2207,22 @@ MetaCommand::TypeEnumType MetaCommand::StringToType(const char* type)
     {
     return FLAG;
     }
+  else if(!strcmp(type,"bool"))
+    {
+    return BOOL;
+    }
+  else if(!strcmp(type,"image"))
+    {
+    return IMAGE;
+    }
+  else if(!strcmp(type,"enum"))
+    {
+    return ENUM;
+    }
+  else if(!strcmp(type,"file"))
+    {
+    return FILE;
+    }
 
   return INT; // by default
 
@@ -2164,7 +2269,8 @@ bool MetaCommand::SetOptionLabel(METAIO_STL::string optionName,
  *  If the group doesn't exist it is automatically created. */
 bool MetaCommand::SetParameterGroup(METAIO_STL::string optionName,
                                     METAIO_STL::string groupName,
-                                    METAIO_STL::string groupDescription
+                                    METAIO_STL::string groupDescription,
+                                    bool advanced
                                     )
 {
   // Check if the group exists
@@ -2195,7 +2301,8 @@ bool MetaCommand::SetParameterGroup(METAIO_STL::string optionName,
 
   if(!optionExists)
     {
-    METAIO_STREAM::cout << "The option " << optionName << " doesn't exist" << METAIO_STREAM::endl;
+    METAIO_STREAM::cout << "The option " << optionName.c_str() 
+                        << " doesn't exist" << METAIO_STREAM::endl;
     return false;
     }
    
@@ -2204,6 +2311,7 @@ bool MetaCommand::SetParameterGroup(METAIO_STL::string optionName,
     ParameterGroup pgroup;
     pgroup.name = groupName;
     pgroup.description = groupDescription;
+    pgroup.advanced = advanced;
     pgroup.options.push_back(optionName);
     m_ParameterGroup.push_back(pgroup);
     }
@@ -2278,7 +2386,7 @@ bool MetaCommand::LoadArgumentsFromXML(const char* filename,
       }
     cur = cur->next;
     }
-
+  xmlFreeDoc(doc);
 #else 
   METAIO_STREAM::cout << "LoadArguments(" << filename << ") requires libxml2" << METAIO_STREAM::endl; 
   if(createMissingArguments)
