@@ -1,7 +1,7 @@
 /*=========================================================================
 
 Program:   Visualization Toolkit
-Module:    $RCSfile: vtkOpenGLHAVSVolumeMapper.cxx,v $
+Module:    vtkOpenGLHAVSVolumeMapper.cxx
 
 Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
 All rights reserved.
@@ -32,6 +32,7 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkUnstructuredGrid.h"
 #include "vtkUnstructuredGridPartialPreIntegration.h"
 #include "vtkVolumeProperty.h"
+#include "vtkOpenGLRenderWindow.h"
 
 #include "vtkHAVSVolumeMapper_kbufferVP.h"
 #include "vtkHAVSVolumeMapper_k2BeginFP.h"
@@ -41,7 +42,6 @@ PURPOSE.  See the above copyright notice for more information.
 #include "vtkHAVSVolumeMapper_k6FP.h"
 #include "vtkHAVSVolumeMapper_k6EndFP.h"
 
-vtkCxxRevisionMacro(vtkOpenGLHAVSVolumeMapper, "$Revision: 1.13 $");
 vtkStandardNewMacro(vtkOpenGLHAVSVolumeMapper);
 
 //----------------------------------------------------------------------------
@@ -54,6 +54,7 @@ vtkOpenGLHAVSVolumeMapper::vtkOpenGLHAVSVolumeMapper()
   this->PsiTableTexture            = 0;
   this->FramebufferObjectSize      = 0;
   this->OrderedTriangles           = 0;
+  this->RenderWindow=0;
 }
 
 //----------------------------------------------------------------------------
@@ -117,17 +118,16 @@ void vtkOpenGLHAVSVolumeMapper::Initialize(vtkRenderer *ren,
                                            vtkVolume *vol)
 {
   // Check for the required extensions only.
-  if (!this->SupportedByHardware())
+  if (!this->SupportedByHardware(ren))
     {
     this->InitializationError = vtkHAVSVolumeMapper::UNSUPPORTED_EXTENSIONS;
     return;
     }
 
-  vtkOpenGLExtensionManager * extensions = vtkOpenGLExtensionManager::New();
-  
-  // Set render window to current render window
-  extensions->SetRenderWindow(ren->GetRenderWindow());
-  
+  vtkOpenGLExtensionManager *extensions=
+    static_cast<vtkOpenGLRenderWindow *>(ren->GetRenderWindow())
+    ->GetExtensionManager();
+
   // Load required extensions
   
   // supports_GL_1_3=1 as checked by this->SupportedByHardware()
@@ -187,8 +187,6 @@ void vtkOpenGLHAVSVolumeMapper::Initialize(vtkRenderer *ren,
       }
     }
   
-  extensions->Delete();
-  
   if (!supports_vertex_buffer_object)
     {
     this->SetGPUDataStructures(false);
@@ -524,10 +522,12 @@ void vtkOpenGLHAVSVolumeMapper::InitializeFramebufferObject()
 
 //----------------------------------------------------------------------------
 void vtkOpenGLHAVSVolumeMapper::Render(vtkRenderer *ren, 
-                                                       vtkVolume *vol)
+                                       vtkVolume *vol)
 {
   ren->GetRenderWindow()->MakeCurrent();
   
+  this->RenderWindow=ren->GetRenderWindow();
+
   if (!this->Initialized)
     {
     this->InitializationError = 
@@ -1090,45 +1090,45 @@ void vtkOpenGLHAVSVolumeMapper::DrawBlend(int screenWidth, int screenHeight,
 //----------------------------------------------------------------------------
 void vtkOpenGLHAVSVolumeMapper::PrintSelf(ostream& os, vtkIndent indent)
 {
-  vtkOpenGLExtensionManager * extensions = vtkOpenGLExtensionManager::New();
-  // set render window to current render window
-  extensions->SetRenderWindow(NULL);
-  if ( this->Initialized )
+  if(this->RenderWindow!=0)
     {
-    os << indent << "Supports GL_VERSION_1_3 (edge_clamp (1.2) and"
-       << " multitexture (1.3) minimal version required by"
-       << " GL_ARB_draw_buffers): "
-       << extensions->ExtensionSupported( "GL_VERSION_1_3" );
-    
-    os << indent << "Supports GL_VERSION_2_0 (GL_ARB_draw_buffers is a core"
-       << "feature): "
-       << extensions->ExtensionSupported( "GL_VERSION_2_0" );
-    
-    os << indent << "Supports GL_ARB_draw_buffers: "
-       << extensions->ExtensionSupported( "GL_ARB_draw_buffers" );
-    
-    os << indent << "Supports GL_EXT_framebuffer_object: " 
-       << extensions->ExtensionSupported( "GL_EXT_framebuffer_object" )
-       << endl;
-    os << indent << "Supports GL_ARB_vertex_program: "
-       << extensions->ExtensionSupported( "GL_ARB_vertex_program" ) << endl;
-    os << indent << "Supports GL_ARB_fragment_program: "
-       << extensions->ExtensionSupported( "GL_ARB_fragment_program" ) << endl;
-    os << indent << "Supports GL_ARB_texture_float" 
-       << extensions->ExtensionSupported( "GL_ARB_texture_float" ) << endl;
-    os << indent << "Supports GL_ATI_texture_float: " 
-       << extensions->ExtensionSupported( "GL_ATI_texture_float" ) << endl;
-    
-    
-    os << indent << "Supports GL_VERSION_1_5 (for optional core feature VBO): "
-       << extensions->ExtensionSupported( "GL_VERSION_1_5" ) <<endl;
-    
-    os << indent << "Supports (optional) GL_ARB_vertex_buffer_object: "
-       << extensions->ExtensionSupported( "GL_ARB_vertex_buffer_object" )
-       <<endl;
-    
+    vtkOpenGLExtensionManager *extensions=
+      static_cast<vtkOpenGLRenderWindow *>(this->RenderWindow.GetPointer())
+      ->GetExtensionManager();
+    if ( this->Initialized )
+      {
+      os << indent << "Supports GL_VERSION_1_3 (edge_clamp (1.2) and"
+         << " multitexture (1.3) minimal version required by"
+         << " GL_ARB_draw_buffers): "
+         << extensions->ExtensionSupported( "GL_VERSION_1_3" );
+
+      os << indent << "Supports GL_VERSION_2_0 (GL_ARB_draw_buffers is a core"
+         << "feature): "
+         << extensions->ExtensionSupported( "GL_VERSION_2_0" );
+
+      os << indent << "Supports GL_ARB_draw_buffers: "
+         << extensions->ExtensionSupported( "GL_ARB_draw_buffers" );
+
+      os << indent << "Supports GL_EXT_framebuffer_object: " 
+         << extensions->ExtensionSupported( "GL_EXT_framebuffer_object" )
+         << endl;
+      os << indent << "Supports GL_ARB_vertex_program: "
+         << extensions->ExtensionSupported( "GL_ARB_vertex_program" ) << endl;
+      os << indent << "Supports GL_ARB_fragment_program: "
+         << extensions->ExtensionSupported( "GL_ARB_fragment_program" ) << endl;
+      os << indent << "Supports GL_ARB_texture_float"
+         << extensions->ExtensionSupported( "GL_ARB_texture_float" ) << endl;
+      os << indent << "Supports GL_ATI_texture_float: "
+         << extensions->ExtensionSupported( "GL_ATI_texture_float" ) << endl;
+
+      os << indent << "Supports GL_VERSION_1_5 (for optional core feature VBO): "
+         << extensions->ExtensionSupported( "GL_VERSION_1_5" ) <<endl;
+
+      os << indent << "Supports (optional) GL_ARB_vertex_buffer_object: "
+         << extensions->ExtensionSupported( "GL_ARB_vertex_buffer_object" )
+         <<endl;
+      }
     }
-  extensions->Delete();
   
   os << indent << "Framebuffer Object Size: " 
      << this->FramebufferObjectSize << endl;
@@ -1139,10 +1139,12 @@ void vtkOpenGLHAVSVolumeMapper::PrintSelf(ostream& os, vtkIndent indent)
 //----------------------------------------------------------------------------
 // Check the OpenGL extension manager for GPU features necessary for the
 // HAVS algorithm. 
-bool vtkOpenGLHAVSVolumeMapper::SupportedByHardware()
+bool vtkOpenGLHAVSVolumeMapper::SupportedByHardware(vtkRenderer *r)
 {
-  vtkOpenGLExtensionManager * extensions = vtkOpenGLExtensionManager::New();
-  
+  vtkOpenGLExtensionManager *extensions=
+    static_cast<vtkOpenGLRenderWindow *>(r->GetRenderWindow())
+    ->GetExtensionManager();
+
   // Temporarily filter out the Macs, as this mapper makes the ATI driver crash
   // (RogueResearch2 on VTK, ATI Radeon X1600 OpenGL Engine 2.0 ATI-1.4.56) and
   // makes the Nvidia driver render some corrupted image (kamino on ParaView3
@@ -1186,8 +1188,6 @@ bool vtkOpenGLHAVSVolumeMapper::SupportedByHardware()
     extensions->ExtensionSupported( "GL_ARB_texture_float" );
   int supports_GL_ATI_texture_float = 
     extensions->ExtensionSupported( "GL_ATI_texture_float" );
-
-  extensions->Delete();
   
   return !iAmAMac && supports_GL_1_3 && supports_draw_buffers &&
     supports_GL_ARB_fragment_program && supports_GL_ARB_vertex_program &&
