@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkHandleWidget.cxx,v $
+  Module:    vtkHandleWidget.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -26,7 +26,6 @@
 #include "vtkWidgetEvent.h"
 
 
-vtkCxxRevisionMacro(vtkHandleWidget, "$Revision: 1.8 $");
 vtkStandardNewMacro(vtkHandleWidget);
 
 //----------------------------------------------------------------------------------
@@ -274,5 +273,6 @@ void vtkHandleWidget::PrintSelf(ostream& os, vtkIndent indent)
 
   os << indent << "Enable Axis Constraint: " 
      << (this->EnableAxisConstraint ? "On\n" : "Off\n");
-  
+
+  os << indent << "WidgetState: " << this->WidgetState << endl;  
 }
