@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkPointHandleRepresentation3D.cxx,v $
+  Module:    vtkPointHandleRepresentation3D.cxx
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -29,7 +29,6 @@
 #include "vtkFocalPlanePointPlacer.h"
 #include "vtkCamera.h"
 
-vtkCxxRevisionMacro(vtkPointHandleRepresentation3D, "$Revision: 1.16 $");
 vtkStandardNewMacro(vtkPointHandleRepresentation3D);
 
 vtkCxxSetObjectMacro(vtkPointHandleRepresentation3D,Property,vtkProperty);
@@ -665,6 +664,15 @@ void vtkPointHandleRepresentation3D::ReleaseGraphicsResources(vtkWindow *win)
 int vtkPointHandleRepresentation3D::RenderOpaqueGeometry(vtkViewport *viewport)
 {
   this->BuildRepresentation();
+
+  // Sanity check
+  double worldPos[3];
+  this->GetWorldPosition( worldPos );
+  if (worldPos[0] == VTK_DOUBLE_MAX)
+    {
+    return 0;
+    }
+
   return this->Actor->RenderOpaqueGeometry(viewport);
 }
 
@@ -673,6 +681,15 @@ int vtkPointHandleRepresentation3D::RenderTranslucentPolygonalGeometry(
   vtkViewport *viewport)
 {
   this->BuildRepresentation();
+
+  // Sanity check
+  double worldPos[3];
+  this->GetWorldPosition( worldPos );
+  if (worldPos[0] == VTK_DOUBLE_MAX)
+    {
+    return 0;
+    }
+  
   return this->Actor->RenderTranslucentPolygonalGeometry(viewport);
 }
 //-----------------------------------------------------------------------------
