@@ -1,7 +1,7 @@
 /*=========================================================================
 
   Program:   Visualization Toolkit
-  Module:    $RCSfile: vtkWrapJava.c,v $
+  Module:    vtkWrapJava.c
 
   Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
   All rights reserved.
@@ -15,6 +15,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include "vtkParse.h"
 
 int numberOfWrappedFunctions = 0;
@@ -162,7 +163,7 @@ void return_result(FILE *fp)
     case 0x303: fprintf(fp,"jstring "); break;
     case 0x109:
     case 0x309:  
-      fprintf(fp,"jobject "); break;
+      fprintf(fp,"jlong "); break;
       
     case 0x301: case 0x307: case 0x313:
     case 0x304: case 0x305: case 0x306: case 0x30A: case 0x30B: case 0x30C:
@@ -247,10 +248,6 @@ void output_temp(FILE *fp, int i, int aType, char *Id, int aCount)
     }
 
   fprintf(fp,";\n");
-  if ((i == MAX_ARGS) && ((aType % 0x1000 == 0x309)||(aType % 0x1000 == 0x109)))
-    {
-    fprintf(fp,"  jobject tempH;\n");
-    }
 }
 
 void get_args(FILE *fp, int i)
@@ -262,7 +259,9 @@ void get_args(FILE *fp, int i)
     {
     fprintf(fp,"  env->GetJavaVM(&(temp%i->vm));\n",i);
     fprintf(fp,"  temp%i->uobj = env->NewGlobalRef(id0);\n",i);
-    fprintf(fp,"  temp%i->mid = env->GetMethodID(env->GetObjectClass(id0),vtkJavaUTFToChar(env,id1),\"()V\");\n",i);
+    fprintf(fp,"  char *temp%i_str;\n",i);
+    fprintf(fp,"  temp%i_str = vtkJavaUTFToChar(env,id1);\n",i);
+    fprintf(fp,"  temp%i->mid = env->GetMethodID(env->GetObjectClass(id0),temp%i_str,\"()V\");\n",i,i);
     return;
     }
 
@@ -286,7 +285,7 @@ void get_args(FILE *fp, int i)
       break;
     case 0x109:
     case 0x309:
-      fprintf(fp,"  temp%i = (%s *)(vtkJavaGetPointerFromObject(env,id%i,(char *) \"%s\"));\n",i,currentFunction->ArgClasses[i],i,currentFunction->ArgClasses[i]);
+      fprintf(fp,"  temp%i = (%s *)(vtkJavaGetPointerFromObject(env,id%i));\n",i,currentFunction->ArgClasses[i],i);
       break;
     case 0x301:
     case 0x307:
@@ -323,6 +322,7 @@ void copy_and_release_args(FILE *fp, int i)
   /* handle VAR FUNCTIONS */
   if (currentFunction->ArgTypes[i] == 0x5000)
     {
+    fprintf(fp,"  if (temp%i_str) delete[] temp%i_str;\n",i,i);
     return;
     }
 
@@ -343,6 +343,9 @@ void copy_and_release_args(FILE *fp, int i)
         }
       fprintf(fp,"  env->ReleaseDoubleArrayElements(id%i,(jdouble *)tempArray%i,0);\n",i,i);      
       break;
+    case 0x303:
+      fprintf(fp,"  if (temp%i) delete[] temp%i;\n",i,i);
+      break;
     case 0x304:
     case 0x306:
     case 0x30A:
@@ -380,19 +383,7 @@ void do_return(FILE *fp)
     case 0x109:
     case 0x309:  
       {
-      fprintf(fp,"  if (temp%i == NULL) { return NULL; }\n", MAX_ARGS);
-      fprintf(fp,"  tempH = vtkJavaGetObjectFromPointer((void *)temp%i);\n", MAX_ARGS);
-      fprintf(fp,"  if (!tempH)\n    {\n");
-      fprintf(fp,"    tempH = vtkJavaCreateNewJavaStubForObject(env, (vtkObject *)temp%i);\n", MAX_ARGS);
-      fprintf(fp,"    if (!tempH)\n      {\n");
-      fprintf(fp,"      // clear the exception first\n");
-      fprintf(fp,"      env->ExceptionClear();\n");
-      fprintf(fp,"      // no java stub for this class exists? Use function return type\n");
-      fprintf(fp,"      tempH = vtkJavaCreateNewJavaStub(env, \"vtk/%s\", (void *)temp%i);\n",
-                  currentFunction->ReturnClass, MAX_ARGS);
-      fprintf(fp,"      }\n");
-      fprintf(fp,"    }\n");      
-      fprintf(fp,"  return tempH;\n");
+      fprintf(fp,"  return (jlong)(size_t)temp%i;", MAX_ARGS);
       break;
       }
       
@@ -603,8 +594,8 @@ void HandleDataReader(FILE *fp, FileInfo *data)
             data->ClassName,currentFunction->Name, numberOfWrappedFunctions);
     fprintf(fp,"{\n");
     fprintf(fp,"  %s *op;\n",data->ClassName);
-    fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj,(char *) \"%s\");\n",
-            data->ClassName, data->ClassName);
+    fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n",
+            data->ClassName);
     fprintf(fp,"  jboolean isCopy;\n");
     fprintf(fp,"  jbyte *data = env->GetByteArrayElements(id0,&isCopy);\n");
     fprintf(fp,"  op->SetBinaryInputString((const char *)data,id1);\n");
@@ -614,10 +605,10 @@ void HandleDataReader(FILE *fp, FileInfo *data)
 
 void HandleDataArray(FILE *fp, FileInfo *data)
 {
-  char* type = 0;
-  char* jtype = 0;
-  char* fromtype = 0;
-  char* jfromtype = 0;
+  const char *type = 0;
+  const char *jtype = 0;
+  const char *fromtype = 0;
+  const char *jfromtype = 0;
 
   if (!strcmp("vtkCharArray",data->ClassName) )
     {
@@ -703,8 +694,8 @@ void HandleDataArray(FILE *fp, FileInfo *data)
   fprintf(fp,"  %s  *temp20;\n", type);
   fprintf(fp,"  vtkIdType size;\n");
   fprintf(fp,"\n");
-  fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj,(char *) \"%s\");\n", 
-    data->ClassName, data->ClassName);
+  fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n", 
+    data->ClassName);
   fprintf(fp,"  temp20 = static_cast<%s*>(op->GetVoidPointer(0));\n", type);
   fprintf(fp,"  size = op->GetMaxId()+1;\n");
   fprintf(fp,"  return vtkJavaMakeJArrayOf%sFrom%s(env,temp20,size);\n", fromtype, fromtype);
@@ -718,8 +709,8 @@ void HandleDataArray(FILE *fp, FileInfo *data)
   fprintf(fp,"  int length;\n");
   fprintf(fp,"  tempArray0 = (%s *)(env->Get%sArrayElements(id0,NULL));\n", type, jfromtype);
   fprintf(fp,"  length = env->GetArrayLength(id0);\n");
-  fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj,(char *) \"%s\");\n", 
-    data->ClassName, data->ClassName);
+  fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n", 
+    data->ClassName);
   fprintf(fp,"  op->SetNumberOfTuples(length/op->GetNumberOfComponents());\n");
   fprintf(fp,"  memcpy(op->GetVoidPointer(0), tempArray0, length*sizeof(%s));\n", type);
   fprintf(fp,"  env->Release%sArrayElements(id0,(j%s *)tempArray0,0);\n", jfromtype, jtype);
@@ -731,6 +722,9 @@ void outputFunction(FILE *fp, FileInfo *data)
 {
   int i;
   int args_ok = 1;
+  char *jniFunction = 0;
+  char *begPtr = 0;
+  char *endPtr = 0;
   CurrentData = data;
 
   /* some functions will not get wrapped no matter what else */
@@ -849,6 +843,26 @@ void outputFunction(FILE *fp, FileInfo *data)
     if (!DoneOne())
     {
       fprintf(fp,"\n");
+
+      /* Underscores are escaped in method names, see
+           http://java.sun.com/javase/6/docs/technotes/guides/jni/spec/design.html#wp133
+         VTK class names contain no underscore and do not need to be escaped.  */
+      jniFunction = currentFunction->Name;
+      begPtr = currentFunction->Name;
+      endPtr = strchr(begPtr, '_');
+      if(endPtr)
+        {
+        jniFunction = (char *)malloc(2*strlen(currentFunction->Name) + 1);
+        jniFunction[0] = '\0';
+        while (endPtr)
+          {
+          strncat(jniFunction, begPtr, endPtr - begPtr + 1);
+          strcat(jniFunction, "1");
+          begPtr = endPtr + 1;
+          endPtr = strchr(begPtr, '_');
+          }
+        strcat(jniFunction, begPtr);
+        }
       
       if(currentFunction->IsLegacy)
         {
@@ -857,7 +871,7 @@ void outputFunction(FILE *fp, FileInfo *data)
       fprintf(fp,"extern \"C\" JNIEXPORT ");
       return_result(fp);
       fprintf(fp," JNICALL Java_vtk_%s_%s_1%i(JNIEnv *env, jobject obj",
-              data->ClassName,currentFunction->Name, numberOfWrappedFunctions);
+              data->ClassName, jniFunction, numberOfWrappedFunctions);
       
       for (i = 0; i < currentFunction->NumberOfArguments; i++)
           {
@@ -884,8 +898,8 @@ void outputFunction(FILE *fp, FileInfo *data)
         get_args(fp, i);
         }
       
-      fprintf(fp,"\n  op = (%s *)vtkJavaGetPointerFromObject(env,obj,(char *) \"%s\");\n",
-              data->ClassName,data->ClassName);
+      fprintf(fp,"\n  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n",
+              data->ClassName);
       
       
       switch (currentFunction->ReturnType % 0x1000)
@@ -922,7 +936,7 @@ void outputFunction(FILE *fp, FileInfo *data)
       if (currentFunction->NumberOfArguments == 1 && currentFunction->ArgTypes[0] == 0x5000)
         {
         fprintf(fp,"  op->%sArgDelete(vtkJavaVoidFuncArgDelete);\n",
-                currentFunction->Name);
+                jniFunction);
         }
       
       /* now copy and release any arrays */
@@ -939,6 +953,10 @@ void outputFunction(FILE *fp, FileInfo *data)
       
       wrappedFunctions[numberOfWrappedFunctions] = currentFunction;
       numberOfWrappedFunctions++;
+      if (jniFunction != currentFunction->Name)
+        {
+        free(jniFunction);
+        }
     } /* isDone() */
   } /* isAbstract */
 }
@@ -993,34 +1011,40 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
 
   if ((!data->NumberOfSuperClasses)&&(data->HasDelete))
     {
+    fprintf(fp,"\nextern \"C\" JNIEXPORT void JNICALL Java_vtk_%s_VTKDeleteReference(JNIEnv *,jclass,jlong id)\n",
+            data->ClassName);
+    fprintf(fp,"{\n  %s *op;\n",data->ClassName);
+    fprintf(fp,"  op = reinterpret_cast<%s*>(id);\n",
+            data->ClassName);
+    fprintf(fp,"  op->Delete();\n");
+    fprintf(fp,"}\n");
+
     fprintf(fp,"\nextern \"C\" JNIEXPORT void JNICALL Java_vtk_%s_VTKDelete(JNIEnv *env,jobject obj)\n",
             data->ClassName);
     fprintf(fp,"{\n  %s *op;\n",data->ClassName);
-    fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj,(char *) \"%s\");\n",
-            data->ClassName,data->ClassName);
-    fprintf(fp,"  vtkJavaDeleteObject(env,obj);\n");
+    fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n",
+            data->ClassName);
     fprintf(fp,"  op->Delete();\n");
     fprintf(fp,"}\n");
+    
+    fprintf(fp,"\nextern \"C\" JNIEXPORT void JNICALL Java_vtk_%s_VTKRegister(JNIEnv *env,jobject obj)\n",
+            data->ClassName);
+    fprintf(fp,"{\n  %s *op;\n",data->ClassName);
+    fprintf(fp,"  op = (%s *)vtkJavaGetPointerFromObject(env,obj);\n",
+            data->ClassName);
+    fprintf(fp,"  op->Register(op);\n");
+    fprintf(fp,"}\n");
     }
   if (data->IsConcrete)
     {
-    fprintf(fp,"\nextern \"C\" JNIEXPORT void JNICALL Java_vtk_%s_VTKInit(JNIEnv *env, jobject obj)",
+    fprintf(fp,"\nextern \"C\" JNIEXPORT jlong JNICALL Java_vtk_%s_VTKInit(JNIEnv *, jobject)",
             data->ClassName);
     fprintf(fp,"\n{");
     fprintf(fp,"\n  %s *aNewOne = %s::New();",data->ClassName, data->ClassName);
-    fprintf(fp,"\n  int jid= vtkJavaRegisterNewObject(env,obj,(void *)aNewOne);");
-    fprintf(fp,"\n  vtkJavaRegisterCastFunction(env,obj,jid,(void *)%s_Typecast);", data->ClassName);
+    fprintf(fp,"\n  return (jlong)(size_t)(void*)aNewOne;");
     fprintf(fp,"\n}\n");  
     } 
 
-  fprintf(fp,"\nextern \"C\" JNIEXPORT void JNICALL Java_vtk_%s_VTKCastInit(JNIEnv *env, jobject obj)",
-                data->ClassName);
-  fprintf(fp,"\n{");
-  fprintf(fp,"\n  int jid= vtkJavaGetId(env,obj);");
-  fprintf(fp,"\n  vtkJavaRegisterCastFunction(env,obj,jid,(void *)%s_Typecast);", 
-            data->ClassName);
-  fprintf(fp,"\n}\n");
-
   /* for vtkRenderWindow we want to add a special method to support
    * native AWT rendering
    *
@@ -1038,7 +1062,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
     fprintf(fp,"\nextern \"C\" JNIEXPORT jstring JNICALL Java_vtk_vtkObject_Print(JNIEnv *env,jobject obj)\n");
     fprintf(fp,"{\n  vtkObject *op;\n");
     fprintf(fp,"  jstring tmp;\n\n");
-    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj,(char *) \"vtkObject\");\n");
+    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj);\n");
     
     fprintf(fp,"  vtksys_ios::ostringstream vtkmsg_with_warning_C4701;\n");
     fprintf(fp,"  op->Print(vtkmsg_with_warning_C4701);\n");
@@ -1052,7 +1076,7 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
     fprintf(fp,"\nextern \"C\" JNIEXPORT jstring JNICALL Java_vtk_vtkObject_PrintRevisions(JNIEnv *env,jobject obj)\n");
     fprintf(fp,"{\n  vtkObject *op;\n");
     fprintf(fp,"  jstring tmp;\n\n");
-    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj,(char *) \"vtkObject\");\n");
+    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj);\n");
     
     fprintf(fp,"  vtksys_ios::ostringstream vtkmsg_with_warning_C4701;\n");
     fprintf(fp,"  op->PrintRevisions(vtkmsg_with_warning_C4701);\n");
@@ -1068,12 +1092,16 @@ void vtkParseOutput(FILE *fp, FileInfo *data)
     fprintf(fp,"  vtkJavaCommand *cbc = vtkJavaCommand::New();\n");
     fprintf(fp,"  cbc->AssignJavaVM(env);\n");
     fprintf(fp,"  cbc->SetGlobalRef(env->NewGlobalRef(id1));\n");
-    fprintf(fp,"  cbc->SetMethodID(env->GetMethodID(env->GetObjectClass(id1),vtkJavaUTFToChar(env,id2),\"()V\"));\n");
+    fprintf(fp,"  char    *temp2;\n");
+    fprintf(fp,"  temp2 = vtkJavaUTFToChar(env,id2);\n");
+    fprintf(fp,"  cbc->SetMethodID(env->GetMethodID(env->GetObjectClass(id1),temp2,\"()V\"));\n");
     fprintf(fp,"  char    *temp0;\n");
     fprintf(fp,"  temp0 = vtkJavaUTFToChar(env,id0);\n");
-    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj,(char *) \"vtkObject\");\n");
+    fprintf(fp,"  op = (vtkObject *)vtkJavaGetPointerFromObject(env,obj);\n");
     fprintf(fp,"  unsigned long     temp20;\n");
     fprintf(fp,"  temp20 = op->AddObserver(temp0,cbc);\n");
+    fprintf(fp,"  if (temp0) delete[] temp0;\n");
+    fprintf(fp,"  if (temp2) delete[] temp2;\n");
     fprintf(fp,"  cbc->Delete();\n");
     fprintf(fp,"  return temp20;\n}\n");
    }
